@@ -1,24 +1,25 @@
-/*
- *  File:       arena.cc
- *  Summary:    Functions related to the monster arena (stage and watch fights).
- */
+/**
+ * @file
+ * @brief Functions related to the monster arena (stage and watch fights).
+**/
 
 #include "AppHdr.h"
 
-#include "externs.h"
-#include "options.h"
 #include "arena.h"
-#include "artefact.h"
+
 #include "cio.h"
 #include "colour.h"
 #include "command.h"
+#include "coord.h"
 #include "dungeon.h"
 #include "env.h"
+#include "externs.h"
 #include "initfile.h"
 #include "items.h"
 #include "itemname.h" // for make_name()
 #include "l_defs.h"
 #include "libutil.h"
+#include "los.h"
 #include "macro.h"
 #include "maps.h"
 #include "message.h"
@@ -28,15 +29,19 @@
 #include "mon-util.h"
 #include "mon-place.h"
 #include "mgen_data.h"
-#include "coord.h"
 #include "mon-stuff.h"
-#include "spl-mis.h"
+#include "ng-init.h"
+#include "options.h"
+#include "spl-miscast.h"
 #include "spl-util.h"
 #include "state.h"
+#ifdef USE_TILE
+ #include "tileview.h"
+#endif
 #include "view.h"
 #include "viewgeom.h"
 
-#define DEBUG_DIAGNOSTICS 1
+#define DEBUG_DIAGNOSTICS
 
 extern void world_reacts();
 
@@ -78,6 +83,8 @@ namespace arena
         }
     };
 
+    std::string teams;
+
     int total_trials = 0;
 
     bool contest_canceled = false;
@@ -110,11 +117,11 @@ namespace arena
     std::vector<int> uniques_list;
     std::vector<int> a_spawners;
     std::vector<int> b_spawners;
-    char             to_respawn[MAX_MONSTERS];
+    int8_t           to_respawn[MAX_MONSTERS];
 
     int item_drop_times[MAX_ITEMS];
 
-    bool banned_glyphs[256];
+    bool banned_glyphs[128];
 
     std::string arena_type = "";
     faction faction_a(true);
@@ -128,7 +135,7 @@ namespace arena
     int message_pos = 0;
     level_id place(BRANCH_MAIN_DUNGEON, 20);
 
-    void adjust_spells(monsters* mons, bool no_summons, bool no_animate)
+    void adjust_spells(monster* mons, bool no_summons, bool no_animate)
     {
         monster_spells &spells(mons->spells);
         for (int i = 0; i < NUM_MONSTER_SPELL_SLOTS; ++i)
@@ -156,7 +163,7 @@ namespace arena
         if (!Options.arena_list_eq || file == NULL)
             return;
 
-        const monsters* mon = &menv[imon];
+        const monster* mon = &menv[imon];
 
         std::vector<int> items;
 
@@ -196,8 +203,12 @@ namespace arena
                 const int imon = dgn_place_monster(spec, you.absdepth0,
                                                    loc, false, true, false);
                 if (imon == -1)
-                    end(1, false, "Failed to create monster at (%d,%d) grd: %s",
-                        loc.x, loc.y, dungeon_feature_name(grd(loc)));
+                {
+                    game_ended_with_error(
+                        make_stringf(
+                            "Failed to create monster at (%d,%d) grd: %s",
+                            loc.x, loc.y, dungeon_feature_name(grd(loc))));
+                }
                 list_eq(imon);
                 to_respawn[imon] = i;
             }
@@ -209,11 +220,11 @@ namespace arena
         if (number >= 0)
             text = make_stringf("(%d) %s", number, text.c_str());
 
-        if (text.length() > sz)
-            text = text.substr(0, sz);
+        unsigned len = strwidth(text);
+        if (len > sz)
+            text = chop_string(text, len = sz);
 
-        int padding = (sz - text.length()) / 2 + text.length();
-        cprintf("%*s", padding, text.c_str());
+        cprintf("%s%s", std::string((sz - len) / 2, ' ').c_str(), text.c_str());
     }
 
     void setup_level()
@@ -268,15 +279,16 @@ namespace arena
             env.floor_colour = LIGHTGREY;
 
 #ifdef USE_TILE
-        TileNewLevel(true);
+        tile_new_level(true);
 #endif
+        los_changed();
         env.markers.activate_all();
     }
 
     std::string find_monster_spec()
     {
-        if (!SysEnv.arena_teams.empty())
-            return (SysEnv.arena_teams);
+        if (!teams.empty())
+            return (teams);
         else
             return ("random v random");
     }
@@ -368,7 +380,8 @@ namespace arena
 
         const std::string glyphs = strip_tag_prefix(spec, "ban_glyphs:");
         for (unsigned int i = 0; i < glyphs.size(); i++)
-            banned_glyphs[static_cast<int>(glyphs[i])] = true;
+            if (!(glyphs[i] & !127))
+                banned_glyphs[static_cast<int>(glyphs[i])] = true;
 
         std::vector<std::string> factions = split_string(" v ", spec);
 
@@ -477,7 +490,6 @@ namespace arena
 
         you.position.y = -1;
         coord_def yplace(dgn_find_feature_marker(DNGN_ESCAPE_HATCH_UP));
-        you.set_arena_los(yplace);
         crawl_view.set_player_at(yplace);
 
         you.mutation[MUT_ACUTE_VISION] = 3;
@@ -486,6 +498,9 @@ namespace arena
 
         you.hp = you.hp_max = 99;
 
+        for (int i = 0; i < NUM_STATS; ++i)
+            you.base_stats[i] = 20;
+
         Options.show_gold_turns = false;
 
         show_fight_banner();
@@ -510,10 +525,10 @@ namespace arena
         setup_others();
     }
 
-    // Temporarily unset crawl_state.arena to force a --more-- to happen.
+    // Temporarily reset crawl_state.type to force a --more-- to happen.
     void more()
     {
-        unwind_bool state(crawl_state.arena, false);
+        unwind_var<game_type> type(crawl_state.type, GAME_TYPE_NORMAL);
 
         ::more();
     }
@@ -706,7 +721,7 @@ namespace arena
 
     void handle_keypress(int ch)
     {
-        if (ch == ESCAPE || tolower(ch) == 'q' || ch == CONTROL('G'))
+        if (key_is_escape(ch) || tolower(ch) == 'q')
         {
             contest_canceled = true;
             mpr("Canceled contest at user request");
@@ -732,8 +747,8 @@ namespace arena
 
         cursor_control coff(true);
 
-        unwind_bool  ar     (crawl_state.arena,           false);
-        unwind_bool  ar_susp(crawl_state.arena_suspended, true);
+        unwind_var<game_type> type(crawl_state.type, GAME_TYPE_NORMAL);
+        unwind_bool ar_susp(crawl_state.arena_suspended, true);
         coord_def yplace(dgn_find_feature_marker(DNGN_ESCAPE_HATCH_UP));
         unwind_var<coord_def> pos(you.position);
         you.position = yplace;
@@ -764,7 +779,7 @@ namespace arena
                 // from ending attempt to displace whatever is in
                 // our position.
                 int       midx  = mgrd(pos);
-                monsters* other = &menv[midx];
+                monster* other = &menv[midx];
 
                 if (to_respawn[midx] == -1)
                 {
@@ -812,7 +827,7 @@ namespace arena
 
     void do_fight()
     {
-        viewwindow(false);
+        viewwindow();
         mesclr(true);
         {
             cursor_control coff(false);
@@ -820,9 +835,9 @@ namespace arena
             {
                 if (kbhit())
                 {
-                    const int ch = getch();
+                    const int ch = getchm();
                     handle_keypress(ch);
-                    ASSERT(crawl_state.arena && !crawl_state.arena_suspended);
+                    ASSERT(crawl_state.game_is_arena() && !crawl_state.arena_suspended);
                     if (contest_canceled)
                         return;
                 }
@@ -835,7 +850,7 @@ namespace arena
                 if ((turns++ % 100) == 0)
                     count_foes();
 
-                viewwindow(false);
+                viewwindow();
                 you.time_taken = 10;
                 // Make sure we don't starve.
                 you.hunger = 10999;
@@ -850,7 +865,7 @@ namespace arena
                 dump_messages();
                 ASSERT(you.pet_target == MHITNOT);
             }
-            viewwindow(false);
+            viewwindow();
         }
 
         mesclr();
@@ -927,8 +942,12 @@ namespace arena
         dump_messages();
     }
 
-    void global_setup()
+    void global_setup(const std::string& arena_teams)
     {
+        // [ds] Turning off view_lock crashes arena.
+        Options.view_lock_x = Options.view_lock_y = true;
+
+        teams = arena_teams;
         // Set various options from the arena spec's tags
         try
         {
@@ -937,7 +956,7 @@ namespace arena
         catch (const std::string &error)
         {
             write_error(error);
-            end(0, false, "%s", error.c_str());
+            game_ended_with_error(error);
         }
 
         if (file != NULL)
@@ -1012,7 +1031,7 @@ namespace arena
             catch (const std::string &error)
             {
                 write_error(error);
-                end(0, false, "%s", error.c_str());
+                game_ended_with_error(error);
             }
             do_fight();
 
@@ -1072,8 +1091,9 @@ monster_type arena_pick_random_monster(const level_id &place, int power,
         return (type);
     }
 
-    end(1, false, "No random monsters for place '%s'",
-        arena::place.describe().c_str());
+    game_ended_with_error(
+        make_stringf("No random monsters for place '%s'",
+                     arena::place.describe().c_str()));
     return (NUM_MONSTERS);
 }
 
@@ -1083,7 +1103,7 @@ bool arena_veto_random_monster(monster_type type)
         return (true);
     if (!arena::allow_zero_xp && mons_class_flag(type, M_NO_EXP_GAIN))
         return (true);
-    if (arena::banned_glyphs[mons_char(type)])
+    if (!(mons_char(type) & !127) && arena::banned_glyphs[mons_char(type)])
         return (true);
 
     return (false);
@@ -1110,19 +1130,20 @@ bool arena_veto_place_monster(const mgen_data &mg, bool first_band_member,
 
     }
     return (!arena::allow_bands && !first_band_member
-            || arena::banned_glyphs[mons_char(mg.cls)]);
+            || !(mons_char(mg.cls) & !127)
+               && arena::banned_glyphs[mons_char(mg.cls)]);
 }
 
 // XXX: Still having some trouble with book-keeping if a slime creature
 // is placed via splitting.
-void arena_placed_monster(monsters *monster)
+void arena_placed_monster(monster* mons)
 {
-    if (monster->attitude == ATT_FRIENDLY)
+    if (mons->attitude == ATT_FRIENDLY)
     {
         arena::faction_a.active_members++;
         arena::faction_b.won = false;
     }
-    else if (monster->attitude == ATT_HOSTILE)
+    else if (mons->attitude == ATT_HOSTILE)
     {
         arena::faction_b.active_members++;
         arena::faction_a.won = false;
@@ -1130,29 +1151,29 @@ void arena_placed_monster(monsters *monster)
     else
     {
         mprf(MSGCH_ERROR, "Placed neutral (%d) monster %s",
-             static_cast<int>(monster->attitude),
-             monster->name(DESC_PLAIN, true).c_str());
+             static_cast<int>(mons->attitude),
+             mons->name(DESC_PLAIN, true).c_str());
     }
 
     if (!arena::allow_summons || !arena::allow_animate)
     {
-        arena::adjust_spells(monster, !arena::allow_summons,
+        arena::adjust_spells(mons, !arena::allow_summons,
                              !arena::allow_animate);
     }
 
-    if (monster->type == MONS_TEST_SPAWNER)
+    if (mons->type == MONS_TEST_SPAWNER)
     {
-        if (monster->attitude == ATT_FRIENDLY)
-            arena::a_spawners.push_back(monster->mindex());
-        else if (monster->attitude == ATT_HOSTILE)
-            arena::b_spawners.push_back(monster->mindex());
+        if (mons->attitude == ATT_FRIENDLY)
+            arena::a_spawners.push_back(mons->mindex());
+        else if (mons->attitude == ATT_HOSTILE)
+            arena::b_spawners.push_back(mons->mindex());
     }
 
-    const bool summoned = monster->is_summoned();
+    const bool summoned = mons->is_summoned();
 
 #ifdef DEBUG_DIAGNOSTICS
     mprf("%s %s!",
-         monster->full_name(DESC_CAP_A, true).c_str(),
+         mons->full_name(DESC_CAP_A, true).c_str(),
          arena::is_respawning                ? "respawns" :
          (summoned && ! arena::real_summons) ? "is summoned"
                                              : "enters the arena");
@@ -1160,7 +1181,7 @@ void arena_placed_monster(monsters *monster)
 
     for (int i = 0; i < NUM_MONSTER_SLOTS; i++)
     {
-        short it = monster->inv[i];
+        short it = mons->inv[i];
         if (it != NON_ITEM)
         {
             item_def &item(mitm[it]);
@@ -1178,33 +1199,33 @@ void arena_placed_monster(monsters *monster)
         }
     }
 
-    if (arena::name_monsters && !monster->is_named())
-        monster->mname = make_name(random_int(), false);
+    if (arena::name_monsters && !mons->is_named())
+        mons->mname = make_name(random_int(), false);
 
     if (summoned)
     {
         // Real summons drop corpses and items.
         if (arena::real_summons)
         {
-            monster->del_ench(ENCH_ABJ, true, false);
+            mons->del_ench(ENCH_ABJ, true, false);
             for (int i = 0; i < NUM_MONSTER_SLOTS; i++)
             {
-                short it = monster->inv[i];
+                short it = mons->inv[i];
                 if (it != NON_ITEM)
                     mitm[it].flags &= ~ISFLAG_SUMMONED;
             }
         }
 
         if (arena::move_summons)
-            monster_teleport(monster, true, true);
+            monster_teleport(mons, true, true);
 
         if (!arena::allow_chain_summons)
-            arena::adjust_spells(monster, true, false);
+            arena::adjust_spells(mons, true, false);
     }
 }
 
 // Take care of respawning slime creatures merging and then splitting.
-void arena_split_monster(monsters *split_from, monsters *split_to)
+void arena_split_monster(monster* split_from, monster* split_to)
 {
     if (!arena::respawn)
         return;
@@ -1218,12 +1239,12 @@ void arena_split_monster(monsters *split_from, monsters *split_to)
     arena::to_respawn[split_to->mindex()] = member_idx;
 }
 
-void arena_monster_died(monsters *monster, killer_type killer,
+void arena_monster_died(monster* mons, killer_type killer,
                         int killer_index, bool silent, int corpse)
 {
-    if (monster->attitude == ATT_FRIENDLY)
+    if (mons->attitude == ATT_FRIENDLY)
         arena::faction_a.active_members--;
-    else if (monster->attitude == ATT_HOSTILE)
+    else if (mons->attitude == ATT_HOSTILE)
         arena::faction_b.active_members--;
 
     if (arena::faction_a.active_members > 0
@@ -1240,55 +1261,55 @@ void arena_monster_died(monsters *monster, killer_type killer,
     else if (arena::faction_a.active_members <= 0
              && arena::faction_b.active_members <= 0)
     {
-        if (monster->flags & MF_HARD_RESET && !MON_KILL(killer))
-            end(1, false, "Last arena monster was dismissed.");
+        if (mons->flags & MF_HARD_RESET && !MON_KILL(killer))
+            game_ended_with_error("Last arena monster was dismissed.");
         // If all monsters are dead, and the last one to die is a giant
         // spore or ball lightning, then that monster's faction is the
         // winner, since self-destruction is their purpose.  But if a
         // trap causes the spore to explode, and that kills everything,
         // it's a tie, since it counts as the trap killing everyone.
-        else if (mons_self_destructs(monster) && MON_KILL(killer))
+        else if (mons_self_destructs(mons) && MON_KILL(killer))
         {
-            if (monster->attitude == ATT_FRIENDLY)
+            if (mons->attitude == ATT_FRIENDLY)
                 arena::faction_a.won = true;
-            else if (monster->attitude == ATT_HOSTILE)
+            else if (mons->attitude == ATT_HOSTILE)
                 arena::faction_b.won = true;
         }
     }
 
     // Only respawn those monsers which were initally placed in the
     // arena.
-    const int midx = monster->mindex();
+    const int midx = mons->mindex();
     if (arena::respawn && arena::to_respawn[midx] != -1
         // Don't respawn when a slime 'dies' from merging with another
         // slime.
-        && !(monster->type == MONS_SLIME_CREATURE && silent
+        && !(mons->type == MONS_SLIME_CREATURE && silent
              && killer == KILL_MISC
              && killer_index == NON_MONSTER))
     {
         arena::faction *fac = NULL;
-        if (monster->attitude == ATT_FRIENDLY)
+        if (mons->attitude == ATT_FRIENDLY)
             fac = &arena::faction_a;
-        else if (monster->attitude == ATT_HOSTILE)
+        else if (mons->attitude == ATT_HOSTILE)
             fac = &arena::faction_b;
 
         if (fac)
         {
             int member_idx = arena::to_respawn[midx];
             fac->respawn_list.push_back(member_idx);
-            fac->respawn_pos.push_back(monster->pos());
+            fac->respawn_pos.push_back(mons->pos());
 
             // Un-merge slime when it respawns, but only if it's
             // specifically a slime, and not a random monster which
             // happens to be a slime.
-            if (monster->type == MONS_SLIME_CREATURE
+            if (mons->type == MONS_SLIME_CREATURE
                 && (fac->members.get_monster(member_idx).mid
                     == MONS_SLIME_CREATURE))
             {
-                for (unsigned int i = 1; i < monster->number; i++)
+                for (unsigned int i = 1; i < mons->number; i++)
                 {
                     fac->respawn_list.push_back(member_idx);
-                    fac->respawn_pos.push_back(monster->pos());
+                    fac->respawn_pos.push_back(mons->pos());
                 }
             }
 
@@ -1300,12 +1321,12 @@ void arena_monster_died(monsters *monster, killer_type killer,
         arena::item_drop_times[corpse] = arena::turns;
 
     // Won't be dropping any items.
-    if (monster->flags & MF_HARD_RESET)
+    if (mons->flags & MF_HARD_RESET)
         return;
 
     for (int i = 0; i < NUM_MONSTER_SLOTS; i++)
     {
-        int idx = monster->inv[i];
+        int idx = mons->inv[i];
         if (idx == NON_ITEM)
             continue;
 
@@ -1428,8 +1449,17 @@ int arena_cull_items()
 
 /////////////////////////////////////////////////////////////////////////////
 
-void run_arena()
+static void _init_arena()
 {
+    run_map_global_preludes();
+    run_map_local_preludes();
+    initialise_item_descriptions();
+}
+
+NORETURN void run_arena(const std::string& teams)
+{
+    _init_arena();
+
     ASSERT(!crawl_state.arena_suspended);
 
 #ifdef WIZARD
@@ -1437,7 +1467,8 @@ void run_arena()
     unwind_bool wiz(you.wizard, true);
 #endif
 
-    arena::global_setup();
+    arena::global_setup(teams);
     arena::simulate();
     arena::global_shutdown();
+    game_ended();
 }
