@@ -1,8 +1,7 @@
-/*
- *  File:       artefact.cc
- *  Summary:    Random and unrandom artefact functions.
- *  Written by: Linley Henzell
- */
+/**
+ * @file
+ * @brief Random and unrandom artefact functions.
+**/
 
 #include "AppHdr.h"
 
@@ -78,7 +77,6 @@ static bool _god_fits_artefact(const god_type which_god, const item_def &item,
         break;
 
     case GOD_SIF_MUNA:
-    case GOD_KIKUBAAQUDGHA:
     case GOD_VEHUMET:
         // The magic gods: no weapons, no preventing spellcasting.
         if (item.base_type == OBJ_WEAPONS)
@@ -113,11 +111,8 @@ static bool _god_fits_artefact(const god_type which_god, const item_def &item,
 
     if (type_bad && !name_check_only)
     {
-        ASSERT(!"God attempting to gift invalid type of item.");
-        mprf(MSGCH_ERROR, "%s attempting to gift invalid type of item.",
-             god_name(which_god).c_str());
-        // Prevent infinite loop in make_item_randart().
-        return (true);
+        die("%s attempting to gift invalid type of item.",
+            god_name(which_god).c_str());
     }
 
     if (type_bad)
@@ -179,8 +174,11 @@ static bool _god_fits_artefact(const god_type which_god, const item_def &item,
             return (false);
         break;
 
-    case GOD_SIF_MUNA:
     case GOD_KIKUBAAQUDGHA:
+        // Necromancy god.
+        if (item.base_type == OBJ_WEAPONS && brand != SPWPN_PAIN)
+            return (false);
+    case GOD_SIF_MUNA:
     case GOD_VEHUMET:
         // The magic gods: no preventing spellcasting.
         if (artefact_wpn_property(item, ARTP_PREVENT_SPELLCASTING))
@@ -200,9 +198,7 @@ static bool _god_fits_artefact(const god_type which_god, const item_def &item,
         // Fedhas forbids necromancy involving corpses, only reaping
         // really applies.
         if (brand == SPWPN_REAPING)
-        {
             return (false);
-        }
         break;
 
     case GOD_CHEIBRIADOS:
@@ -227,8 +223,7 @@ static bool _god_fits_artefact(const god_type which_god, const item_def &item,
     return (true);
 }
 
-static std::string _replace_name_parts(const std::string name_in,
-                                       const item_def& item)
+std::string replace_name_parts(const std::string name_in, const item_def& item)
 {
     std::string name = name_in;
 
@@ -261,7 +256,7 @@ static std::string _replace_name_parts(const std::string name_in,
     name = replace_all(name, "@player_name@", you.your_name);
 
     name = replace_all(name, "@player_species@",
-                 species_name(static_cast<species_type>(you.species), 1));
+                 species_name(static_cast<species_type>(you.species), true));
 
     if (name.find("@branch_name@", 0) != std::string::npos)
     {
@@ -295,8 +290,8 @@ static std::string _replace_name_parts(const std::string name_in,
         {
             const branch_type branch =
                      static_cast<branch_type>(random2(BRANCH_TARTARUS));
-            place = place_name( get_packed_place(branch, 1, LEVEL_DUNGEON),
-                                true, false );
+            place = place_name(get_packed_place(branch, 1, LEVEL_DUNGEON),
+                                true, false);
         }
         if (!place.empty())
             name = replace_all(name, "@branch_name@", place);
@@ -336,13 +331,13 @@ static std::string _replace_name_parts(const std::string name_in,
 // Functions defined in art-func.h are referenced in art-data.h
 #include "art-func.h"
 
-static unrandart_entry unranddata[UNRAND_LAST] = {
+static unrandart_entry unranddata[] = {
 #include "art-data.h"
 };
 
-static unrandart_entry *_seekunrandart( const item_def &item );
+static unrandart_entry *_seekunrandart(const item_def &item);
 
-bool is_known_artefact( const item_def &item )
+bool is_known_artefact(const item_def &item)
 {
     if (!item_type_known(item))
         return (false);
@@ -350,24 +345,24 @@ bool is_known_artefact( const item_def &item )
     return (is_artefact(item));
 }
 
-bool is_artefact( const item_def &item )
+bool is_artefact(const item_def &item)
 {
     return (item.flags & ISFLAG_ARTEFACT_MASK);
 }
 
 // returns true is item is a pure randart
-bool is_random_artefact( const item_def &item )
+bool is_random_artefact(const item_def &item)
 {
     return (item.flags & ISFLAG_RANDART);
 }
 
 // returns true if item in an unrandart
-bool is_unrandom_artefact( const item_def &item )
+bool is_unrandom_artefact(const item_def &item)
 {
     return (item.flags & ISFLAG_UNRANDART);
 }
 
-bool is_special_unrandom_artefact( const item_def &item )
+bool is_special_unrandom_artefact(const item_def &item)
 {
     return (item.flags & ISFLAG_UNRANDART
             && (_seekunrandart(item)->flags & UNRAND_FLAG_SPECIAL));
@@ -381,31 +376,31 @@ unique_item_status_type get_unique_item_status(const item_def& item)
     return (UNIQ_NOT_EXISTS);
 }
 
-unique_item_status_type get_unique_item_status( int art )
+unique_item_status_type get_unique_item_status(int art)
 {
     ASSERT(art > UNRAND_START && art < UNRAND_LAST);
     return (you.unique_items[art - UNRAND_START]);
 }
 
 void set_unique_item_status(const item_def& item,
-                            unique_item_status_type status )
+                            unique_item_status_type status)
 {
     if (item.flags & ISFLAG_UNRANDART)
         set_unique_item_status(item.special, status);
 }
 
-void set_unique_item_status( int art, unique_item_status_type status )
+void set_unique_item_status(int art, unique_item_status_type status)
 {
     ASSERT(art > UNRAND_START && art < UNRAND_LAST);
     you.unique_items[art - UNRAND_START] = status;
 }
 
-static long _calc_seed( const item_def &item )
+static uint32_t _calc_seed(const item_def &item)
 {
     return (item.special & RANDART_SEED_MASK);
 }
 
-void artefact_desc_properties( const item_def &item,
+void artefact_desc_properties(const item_def &item,
                               artefact_properties_t &proprt,
                               artefact_known_props_t &known,
                               bool force_fake_props)
@@ -414,9 +409,9 @@ void artefact_desc_properties( const item_def &item,
     if (item.base_type == OBJ_BOOKS)
         return;
 
-    artefact_wpn_properties( item, proprt, known);
+    artefact_wpn_properties(item, proprt, known);
 
-    if (!force_fake_props && item_ident( item, ISFLAG_KNOW_PROPERTIES ))
+    if (!force_fake_props && item_ident(item, ISFLAG_KNOW_PROPERTIES))
         return;
 
     // Only jewellery need fake randart properties.
@@ -459,8 +454,8 @@ void artefact_desc_properties( const item_def &item,
     {
         proprt[fake_rap] += fake_plus;
 
-        if (item_ident( item, ISFLAG_KNOW_PROPERTIES )
-            || item_ident( item, ISFLAG_KNOW_TYPE ))
+        if (item_ident(item, ISFLAG_KNOW_PROPERTIES)
+            || item_ident(item, ISFLAG_KNOW_TYPE))
         {
             known[fake_rap] = true;
         }
@@ -559,8 +554,8 @@ void artefact_desc_properties( const item_def &item,
     if (fake_rap2 != ARTP_NUM_PROPERTIES && fake_plus2 != 0)
         proprt[fake_rap2] += fake_plus2;
 
-    if (item_ident( item, ISFLAG_KNOW_PROPERTIES )
-        || item_ident( item, ISFLAG_KNOW_TYPE ))
+    if (item_ident(item, ISFLAG_KNOW_PROPERTIES)
+        || item_ident(item, ISFLAG_KNOW_TYPE))
     {
         if (fake_rap != ARTP_NUM_PROPERTIES && proprt[fake_rap] != 0)
             known[fake_rap] = true;
@@ -570,18 +565,18 @@ void artefact_desc_properties( const item_def &item,
     }
 }
 
-inline static void _randart_propset( artefact_properties_t &p,
+inline static void _randart_propset(artefact_properties_t &p,
                                      artefact_prop_type pt,
                                      int value,
-                                     bool neg )
+                                     bool neg)
 {
     // This shouldn't be called with 0, else no property gets added after all.
     ASSERT(value != 0);
     p[pt] = (neg? -value : value);
 }
 
-static int _randart_add_one_property( const item_def &item,
-                                      artefact_properties_t &proprt )
+static int _randart_add_one_property(const item_def &item,
+                                      artefact_properties_t &proprt)
 {
     // This function assumes that at least some properties are still possible.
     const object_class_type cl = item.base_type;
@@ -589,8 +584,7 @@ static int _randart_add_one_property( const item_def &item,
 
     const bool negench = one_chance_in(4);
 
-    const artefact_prop_type artprops[] = { ARTP_AC, ARTP_EVASION,
-                                            ARTP_STRENGTH, ARTP_INTELLIGENCE,
+    const artefact_prop_type artprops[] = { ARTP_STRENGTH, ARTP_INTELLIGENCE,
                                             ARTP_DEXTERITY };
 
     do
@@ -602,26 +596,6 @@ static int _randart_add_one_property( const item_def &item,
 
         switch (prop)
         {
-        case ARTP_AC:
-            if (cl == OBJ_ARMOUR
-                || cl == OBJ_JEWELLERY && ty == RING_PROTECTION)
-            {
-                continue;
-            }
-            _randart_propset(proprt, ARTP_AC,
-                             1 + random2(3) + random2(3) + random2(3),
-                             negench);
-            break;
-
-        case ARTP_EVASION:
-            if (cl == OBJ_JEWELLERY && ty == RING_EVASION)
-                continue;
-
-            _randart_propset(proprt, ARTP_EVASION,
-                             1 + random2(3) + random2(3) + random2(3),
-                             negench);
-            break;
-
         case ARTP_STRENGTH:
             if (cl == OBJ_JEWELLERY && ty == RING_STRENGTH)
                 continue;
@@ -656,9 +630,9 @@ static int _randart_add_one_property( const item_def &item,
 }
 
 // An artefact will pass this check if it has any non-stat properties, and
-// also if it has enough stat (AC, EV, Str, Dex, Int, Acc, Dam) properties.
+// also if it has enough stat (Str, Dex, Int, Acc, Dam) properties.
 // Returns how many (more) stat properties we need to add.
-static int _need_bonus_stat_props( const artefact_properties_t &proprt )
+static int _need_bonus_stat_props(const artefact_properties_t &proprt)
 {
     int num_stats   = 0;
     int num_acc_dam = 0;
@@ -704,7 +678,7 @@ void static _get_randart_properties(const item_def &item,
     const int atype = item.sub_type;
     int power_level = 0;
 
-    const long seed = _calc_seed(item);
+    const uint32_t seed = _calc_seed(item);
     rng_save_excursion exc;
     seed_rng(seed);
 
@@ -722,7 +696,7 @@ void static _get_randart_properties(const item_def &item,
 
     if (aclass == OBJ_WEAPONS) // Only weapons get brands, of course.
     {
-        proprt[ARTP_BRAND] = SPWPN_FLAMING + random2(15);        // brand
+        proprt[ARTP_BRAND] = SPWPN_FLAMING + random2(16);        // brand
 
         if (one_chance_in(6))
             proprt[ARTP_BRAND] = SPWPN_FLAMING + random2(2);
@@ -812,7 +786,7 @@ void static _get_randart_properties(const item_def &item,
         else
             power_level++;
 
-        if (!is_weapon_brand_ok(atype, proprt[ARTP_BRAND]))
+        if (!is_weapon_brand_ok(atype, proprt[ARTP_BRAND], true))
         {
             proprt[ARTP_BRAND] = SPWPN_NORMAL;
             power_level--;
@@ -821,33 +795,7 @@ void static _get_randart_properties(const item_def &item,
 
     if (!one_chance_in(5))
     {
-        // AC mod - not for armours or rings of protection.
-        if (one_chance_in(4 + power_level)
-            && aclass != OBJ_ARMOUR
-            && (aclass != OBJ_JEWELLERY || atype != RING_PROTECTION))
-        {
-            proprt[ARTP_AC] = 1 + random2(3) + random2(3) + random2(3);
-            power_level++;
-            if (one_chance_in(4))
-            {
-                proprt[ARTP_AC] -= 1 + random2(3) + random2(3) + random2(3);
-                power_level--;
-            }
-        }
-
-        // EV mod - not for rings of evasion.
-        if (one_chance_in(4 + power_level)
-            && (aclass != OBJ_JEWELLERY || atype != RING_EVASION))
-        {
-            proprt[ARTP_EVASION] = 1 + random2(3) + random2(3) + random2(3);
-            power_level++;
-            if (one_chance_in(4))
-            {
-                proprt[ARTP_EVASION] -= 1 + random2(3) + random2(3)
-                    + random2(3);
-                power_level--;
-            }
-        }
+        // TODO: compensate for the removal of +AC/+EV
 
         // Str mod - not for rings of strength.
         if (one_chance_in(4 + power_level)
@@ -994,6 +942,7 @@ void static _get_randart_properties(const item_def &item,
     if (!done_powers
         && one_chance_in(4 + power_level)
         && (aclass != OBJ_JEWELLERY || atype != RING_LIFE_PROTECTION)
+        && (aclass != OBJ_ARMOUR || atype != ARM_PEARL_DRAGON_ARMOUR)
         && proprt[ARTP_BRAND] != SPWPN_DRAINING
         && proprt[ARTP_BRAND] != SPWPN_VAMPIRICISM
         && proprt[ARTP_BRAND] != SPWPN_PAIN
@@ -1248,13 +1197,13 @@ static bool _init_artefact_book(item_def &book)
 
 static bool _init_artefact_properties(item_def &item)
 {
-    ASSERT( is_artefact( item ) );
+    ASSERT(is_artefact(item));
 
     CrawlVector &rap = item.props[ARTEFACT_PROPS_KEY].get_vector();
     for (vec_size i = 0; i < ART_PROPERTIES; i++)
         rap[i] = static_cast<short>(0);
 
-    if (is_unrandom_artefact( item ))
+    if (is_unrandom_artefact(item))
     {
         const unrandart_entry *unrand = _seekunrandart(item);
 
@@ -1283,21 +1232,22 @@ static bool _init_artefact_properties(item_def &item)
     return (true);
 }
 
-void artefact_wpn_properties( const item_def &item,
+void artefact_wpn_properties(const item_def &item,
                              artefact_properties_t  &proprt,
                              artefact_known_props_t &known)
 {
-    ASSERT( is_artefact( item ) );
-    ASSERT( item.props.exists( KNOWN_PROPS_KEY ) );
+    ASSERT(is_artefact(item));
+    if (!item.props.exists(KNOWN_PROPS_KEY))
+        return;
 
     const CrawlStoreValue &_val = item.props[KNOWN_PROPS_KEY];
-    ASSERT( _val.get_type() == SV_VEC );
+    ASSERT(_val.get_type() == SV_VEC);
     const CrawlVector &known_vec = _val.get_vector();
-    ASSERT( known_vec.get_type()     == SV_BOOL );
-    ASSERT( known_vec.size()         == ART_PROPERTIES);
-    ASSERT( known_vec.get_max_size() == ART_PROPERTIES);
+    ASSERT(known_vec.get_type()     == SV_BOOL);
+    ASSERT(known_vec.size()         == ART_PROPERTIES);
+    ASSERT(known_vec.get_max_size() == ART_PROPERTIES);
 
-    if (item_ident( item, ISFLAG_KNOW_PROPERTIES ))
+    if (item_ident(item, ISFLAG_KNOW_PROPERTIES))
     {
         for (vec_size i = 0; i < ART_PROPERTIES; i++)
             known[i] = static_cast<bool>(true);
@@ -1308,18 +1258,18 @@ void artefact_wpn_properties( const item_def &item,
             known[i] = known_vec[i];
     }
 
-    if (item.props.exists( ARTEFACT_PROPS_KEY ))
+    if (item.props.exists(ARTEFACT_PROPS_KEY))
     {
         const CrawlVector &rap_vec =
             item.props[ARTEFACT_PROPS_KEY].get_vector();
-        ASSERT( rap_vec.get_type()     == SV_SHORT );
-        ASSERT( rap_vec.size()         == ART_PROPERTIES);
-        ASSERT( rap_vec.get_max_size() == ART_PROPERTIES);
+        ASSERT(rap_vec.get_type()     == SV_SHORT);
+        ASSERT(rap_vec.size()         == ART_PROPERTIES);
+        ASSERT(rap_vec.get_max_size() == ART_PROPERTIES);
 
         for (vec_size i = 0; i < ART_PROPERTIES; i++)
             proprt[i] = rap_vec[i].get_short();
     }
-    else if (is_unrandom_artefact( item ))
+    else if (is_unrandom_artefact(item))
     {
         const unrandart_entry *unrand = _seekunrandart(item);
 
@@ -1333,49 +1283,49 @@ void artefact_wpn_properties( const item_def &item,
 }
 
 
-void artefact_wpn_properties( const item_def &item,
-                              artefact_properties_t &proprt )
+void artefact_wpn_properties(const item_def &item,
+                              artefact_properties_t &proprt)
 {
     artefact_known_props_t known;
 
     artefact_wpn_properties(item, proprt, known);
 }
 
-int artefact_wpn_property( const item_def &item, artefact_prop_type prop,
-                           bool &_known )
+int artefact_wpn_property(const item_def &item, artefact_prop_type prop,
+                           bool &_known)
 {
     artefact_properties_t  proprt;
     artefact_known_props_t known;
 
-    artefact_wpn_properties( item, proprt, known );
+    artefact_wpn_properties(item, proprt, known);
 
     _known = known[prop];
 
-    return ( proprt[prop] );
+    return (proprt[prop]);
 }
 
-int artefact_wpn_property( const item_def &item, artefact_prop_type prop )
+int artefact_wpn_property(const item_def &item, artefact_prop_type prop)
 {
     bool known;
 
-    return artefact_wpn_property( item, prop, known );
+    return artefact_wpn_property(item, prop, known);
 }
 
-int artefact_known_wpn_property( const item_def &item,
-                                 artefact_prop_type prop )
+int artefact_known_wpn_property(const item_def &item,
+                                 artefact_prop_type prop)
 {
     artefact_properties_t  proprt;
     artefact_known_props_t known;
 
-    artefact_wpn_properties( item, proprt, known );
+    artefact_wpn_properties(item, proprt, known);
 
     if (known[prop])
-        return ( proprt[prop] );
+        return (proprt[prop]);
     else
         return (0);
 }
 
-int _artefact_num_props( const artefact_properties_t &proprt )
+static int _artefact_num_props(const artefact_properties_t &proprt)
 {
     int num = 0;
 
@@ -1387,30 +1337,23 @@ int _artefact_num_props( const artefact_properties_t &proprt )
     return num;
 }
 
-void artefact_wpn_learn_prop( item_def &item, artefact_prop_type prop )
+void artefact_wpn_learn_prop(item_def &item, artefact_prop_type prop)
 {
-    ASSERT( is_artefact( item ) );
-    ASSERT( item.props.exists( KNOWN_PROPS_KEY ) );
+    ASSERT(is_artefact(item));
+    ASSERT(item.props.exists(KNOWN_PROPS_KEY));
     CrawlStoreValue &_val = item.props[KNOWN_PROPS_KEY];
-    ASSERT( _val.get_type() == SV_VEC );
+    ASSERT(_val.get_type() == SV_VEC);
     CrawlVector &known_vec = _val.get_vector();
-    ASSERT( known_vec.get_type()     == SV_BOOL );
-    ASSERT( known_vec.size()         == ART_PROPERTIES);
-    ASSERT( known_vec.get_max_size() == ART_PROPERTIES);
+    ASSERT(known_vec.get_type()     == SV_BOOL);
+    ASSERT(known_vec.size()         == ART_PROPERTIES);
+    ASSERT(known_vec.get_max_size() == ART_PROPERTIES);
 
-    if (item_ident( item, ISFLAG_KNOW_PROPERTIES ))
+    if (item_ident(item, ISFLAG_KNOW_PROPERTIES))
         return;
 
     known_vec[prop] = static_cast<bool>(true);
     if (Options.autoinscribe_artefacts)
-        add_autoinscription( item, artefact_auto_inscription(item));
-}
-
-bool artefact_wpn_known_prop( const item_def &item, artefact_prop_type prop )
-{
-    bool known;
-    artefact_wpn_property( item, prop, known );
-    return known;
+        add_autoinscription(item, artefact_auto_inscription(item));
 }
 
 static std::string _get_artefact_type(const item_def &item,
@@ -1425,7 +1368,7 @@ static std::string _get_artefact_type(const item_def &item,
     case OBJ_ARMOUR:
         return "armour";
     case OBJ_JEWELLERY:
-        // Only in appearance distinguish between amulets and rings.
+        // Distinguish between amulets and rings only in appearance.
         if (!appear)
             return "jewellery";
 
@@ -1438,7 +1381,7 @@ static std::string _get_artefact_type(const item_def &item,
     }
 }
 
-static bool _pick_db_name( const item_def &item )
+static bool _pick_db_name(const item_def &item)
 {
     switch (item.base_type)
     {
@@ -1452,18 +1395,18 @@ static bool _pick_db_name( const item_def &item )
     }
 }
 
-std::string artefact_name_lookup(const item_def &item,
+static std::string _artefact_name_lookup(const item_def &item,
                                  const std::string &lookup)
 {
     const std::string name = getRandNameString(lookup);
-    return (!name.empty()? _replace_name_parts(name, item) : name);
+    return (!name.empty()? replace_name_parts(name, item) : name);
 }
 
-bool artefact_name_lookup(std::string &result,
+static bool _artefact_name_lookup(std::string &result,
                           const item_def &item,
                           const std::string &lookup)
 {
-    result = artefact_name_lookup(item, lookup);
+    result = _artefact_name_lookup(item, lookup);
     return (!result.empty());
 }
 
@@ -1476,13 +1419,13 @@ std::string artefact_name(const item_def &item, bool appearance)
            || item.base_type == OBJ_JEWELLERY
            || item.base_type == OBJ_BOOKS);
 
-    if (is_unrandom_artefact( item ))
+    if (is_unrandom_artefact(item))
     {
-        const unrandart_entry *unrand = _seekunrandart( item );
+        const unrandart_entry *unrand = _seekunrandart(item);
         return (item_type_known(item) ? unrand->name : unrand->unid_name);
     }
 
-    const long seed = _calc_seed( item );
+    const uint32_t seed = _calc_seed(item);
 
     std::string lookup;
     std::string result;
@@ -1511,7 +1454,7 @@ std::string artefact_name(const item_def &item, bool appearance)
     lookup += _get_artefact_type(item, appearance);
 
     rng_save_excursion rng_state;
-    seed_rng( seed );
+    seed_rng(seed);
 
     if (appearance)
     {
@@ -1537,19 +1480,19 @@ std::string artefact_name(const item_def &item, bool appearance)
         std::string name;
         do
         {
-            (artefact_name_lookup(name, item, lookup)
+            (_artefact_name_lookup(name, item, lookup)
 
              // If nothing found, try god name alone.
              || (god_gift
-                 && artefact_name_lookup(name, item,
-                                         god_name(
+                 && _artefact_name_lookup(name, item,
+                                          god_name(
                                              static_cast<god_type>(item_orig),
                                              false)))
 
              // If still nothing found, try base type alone.
-             || artefact_name_lookup(name, item, _get_artefact_type(item)));
+             || _artefact_name_lookup(name, item, _get_artefact_type(item)));
         }
-        while (--tries > 0 && name.length() > 25);
+        while (--tries > 0 && strwidth(name) > 25);
 
         if (name.empty()) // still nothing found?
             result += "of Bugginess";
@@ -1578,9 +1521,9 @@ std::string artefact_name(const item_def &item, bool appearance)
     return result;
 }
 
-std::string get_artefact_name( const item_def &item, bool force_known )
+std::string get_artefact_name(const item_def &item, bool force_known)
 {
-    ASSERT( is_artefact( item ) );
+    ASSERT(is_artefact(item));
 
     if (item_type_known(item) || force_known)
     {
@@ -1595,20 +1538,13 @@ std::string get_artefact_name( const item_def &item, bool force_known )
     return artefact_name(item, false);
 }
 
-void set_artefact_name( item_def &item, const std::string &name )
+void set_artefact_name(item_def &item, const std::string &name)
 {
-    ASSERT( is_artefact( item ));
-    ASSERT( !name.empty() );
+    ASSERT(is_artefact(item));
+    ASSERT(!name.empty());
     item.props[ARTEFACT_NAME_KEY].get_string() = name;
 }
 
-void set_artefact_appearance( item_def &item, const std::string &appear )
-{
-    ASSERT( is_artefact( item ));
-    ASSERT( !appear.empty() );
-    item.props[ARTEFACT_APPEAR_KEY].get_string() = appear;
-}
-
 int find_unrandart_index(const item_def& artefact)
 {
     return (artefact.special);
@@ -1624,13 +1560,12 @@ unrandart_entry* get_unrand_entry(int unrand_index)
         return &unranddata[unrand_index];
 }
 
-static unrandart_entry *_seekunrandart( const item_def &item )
+static unrandart_entry *_seekunrandart(const item_def &item)
 {
     return get_unrand_entry(item.special);
 }
 
-int find_okay_unrandart(unsigned char aclass, unsigned char atype,
-                        unrand_special_type specialness, bool in_abyss)
+int find_okay_unrandart(uint8_t aclass, uint8_t atype, bool in_abyss)
 {
     int ret = -1;
 
@@ -1656,19 +1591,23 @@ int find_okay_unrandart(unsigned char aclass, unsigned char atype,
 
         // Never randomly generated until lost in the abyss.
         if ((!in_abyss || status != UNIQ_LOST_IN_ABYSS)
-            && index >= UNRAND_CEREBOV && index <= UNRAND_ASMODEUS)
+            && entry->flags & UNRAND_FLAG_NOGEN)
         {
             continue;
         }
 
         if (entry->base_type != aclass
-            || (atype != OBJ_RANDOM && entry->sub_type != atype))
-        {
-            continue;
-        }
-
-        if (specialness != UNRANDSPEC_EITHER
-            && specialness != get_unrand_specialness(index))
+            || atype != OBJ_RANDOM && entry->sub_type != atype
+               // Acquirement.
+               && (aclass != OBJ_WEAPONS
+                   || weapon_skill(entry->base_type, atype) !=
+                      weapon_skill(entry->base_type, entry->sub_type)
+                   || hands_reqd(entry->base_type,
+                                 atype,
+                                 you.body_size()) !=
+                      hands_reqd(entry->base_type,
+                                 entry->sub_type,
+                                 you.body_size())))
         {
             continue;
         }
@@ -1682,28 +1621,12 @@ int find_okay_unrandart(unsigned char aclass, unsigned char atype,
     return (ret);
 }
 
-unrand_special_type get_unrand_specialness(int unrand_index)
-{
-    if (unrand_index < UNRAND_START || unrand_index > UNRAND_LAST)
-        return (UNRANDSPEC_NORMAL);
-
-    if (unranddata[unrand_index].flags & UNRAND_FLAG_SPECIAL)
-        return (UNRANDSPEC_SPECIAL);
-
-    return (UNRANDSPEC_NORMAL);
-}
-
-unrand_special_type get_unrand_specialness(const item_def &item)
-{
-    ASSERT(is_unrandom_artefact(item));
-    return get_unrand_specialness(item.special);
-}
-
-int get_unrandart_num( const char *name )
+int get_unrandart_num(const char *name)
 {
     std::string quoted = "\"";
     quoted += name;
     quoted += "\"";
+    lowercase(quoted);
 
     for (unsigned int i = 0; i < ARRAYSZ(unranddata); ++i)
     {
@@ -1717,8 +1640,8 @@ int get_unrandart_num( const char *name )
     return SPWPN_NORMAL;
 }
 
-static bool _randart_is_redundant( const item_def &item,
-                                   artefact_properties_t &proprt )
+static bool _randart_is_redundant(const item_def &item,
+                                   artefact_properties_t &proprt)
 {
     if (item.base_type != OBJ_JEWELLERY)
         return (false);
@@ -1827,12 +1750,13 @@ static bool _randart_is_redundant( const item_def &item,
     return (false);
 }
 
-static bool _randart_is_conflicting( const item_def &item,
-                                     artefact_properties_t &proprt )
+static bool _randart_is_conflicting(const item_def &item,
+                                     artefact_properties_t &proprt)
 {
     if (item.base_type == OBJ_WEAPONS
         && get_weapon_brand(item) == SPWPN_HOLY_WRATH
-        && proprt[ARTP_CURSED] != 0)
+        && (is_demonic(item)
+            || proprt[ARTP_CURSED] != 0))
     {
         return (true);
     }
@@ -1841,10 +1765,10 @@ static bool _randart_is_conflicting( const item_def &item,
         return (false);
 
     if (item.sub_type == AMU_STASIS
-       && (proprt[ARTP_BLINK] != 0
-           || proprt[ARTP_CAUSE_TELEPORTATION] != 0
-           || proprt[ARTP_ANGRY] != 0
-           || proprt[ARTP_BERSERK] != 0))
+        && (proprt[ARTP_BLINK] != 0
+            || proprt[ARTP_CAUSE_TELEPORTATION] != 0
+            || proprt[ARTP_ANGRY] != 0
+            || proprt[ARTP_BERSERK] != 0))
     {
         return (true);
     }
@@ -1887,32 +1811,32 @@ static bool _randart_is_conflicting( const item_def &item,
     return (false);
 }
 
-bool randart_is_bad( const item_def &item, artefact_properties_t &proprt )
+bool randart_is_bad(const item_def &item, artefact_properties_t &proprt)
 {
     if (item.base_type == OBJ_BOOKS)
         return (false);
 
-    if (_artefact_num_props( proprt ) == 0)
+    if (_artefact_num_props(proprt) == 0)
         return (true);
 
     // Weapons must have a brand and at least one other property.
     if (item.base_type == OBJ_WEAPONS
         && (proprt[ARTP_BRAND] == SPWPN_NORMAL
-            || _artefact_num_props( proprt ) < 2))
+            || _artefact_num_props(proprt) < 2))
     {
         return (true);
     }
 
-    return (_randart_is_redundant( item, proprt )
-            || _randart_is_conflicting( item, proprt ));
+    return (_randart_is_redundant(item, proprt)
+            || _randart_is_conflicting(item, proprt));
 }
 
-bool randart_is_bad( const item_def &item )
+bool randart_is_bad(const item_def &item)
 {
     artefact_properties_t proprt;
-    artefact_wpn_properties( item, proprt );
+    artefact_wpn_properties(item, proprt);
 
-    return randart_is_bad( item, proprt);
+    return randart_is_bad(item, proprt);
 }
 
 static void _artefact_setup_prop_vectors(item_def &item)
@@ -1928,7 +1852,7 @@ static void _artefact_setup_prop_vectors(item_def &item)
     for (vec_size i = 0; i < ART_PROPERTIES; i++)
         rap[i].get_short() = 0;
 
-    if (!item.props.exists( KNOWN_PROPS_KEY ))
+    if (!item.props.exists(KNOWN_PROPS_KEY))
     {
         props[KNOWN_PROPS_KEY].new_vector(SV_BOOL).resize(ART_PROPERTIES);
         CrawlVector &known = item.props[KNOWN_PROPS_KEY].get_vector();
@@ -1938,14 +1862,9 @@ static void _artefact_setup_prop_vectors(item_def &item)
     }
 }
 
-void artefact_set_name(item_def &item, const std::string &name)
-{
-    item.props[ARTEFACT_NAME_KEY].get_string() = name;
-}
-
 // If force_mundane is true, normally mundane items are forced to
 // nevertheless become artefacts.
-bool make_item_randart( item_def &item, bool force_mundane )
+bool make_item_randart(item_def &item, bool force_mundane)
 {
     if (item.base_type != OBJ_WEAPONS
         && item.base_type != OBJ_ARMOUR
@@ -2005,10 +1924,10 @@ bool make_item_randart( item_def &item, bool force_mundane )
     if (item.props.exists(ARTEFACT_NAME_KEY))
         ASSERT(item.props[ARTEFACT_NAME_KEY].get_type() == SV_STR);
     else
-        artefact_set_name(item, artefact_name(item, false));
+        set_artefact_name(item, artefact_name(item, false));
 
     // get artefact appearance
-    if (item.props.exists( ARTEFACT_APPEAR_KEY ))
+    if (item.props.exists(ARTEFACT_APPEAR_KEY))
         ASSERT(item.props[ARTEFACT_APPEAR_KEY].get_type() == SV_STR);
     else
         item.props[ARTEFACT_APPEAR_KEY].get_string() =
@@ -2017,7 +1936,44 @@ bool make_item_randart( item_def &item, bool force_mundane )
     return (true);
 }
 
-bool make_item_unrandart( item_def &item, int unrand_index )
+static void _make_faerie_armour(item_def &item)
+{
+    item_def doodad;
+    for (int i=0; i<100; i++)
+    {
+        doodad.clear();
+        doodad.base_type = item.base_type;
+        doodad.sub_type = item.sub_type;
+        if (!make_item_randart(doodad))
+        {
+            i--; // Forbidden props are not absolute, artefactness is.
+            continue;
+        }
+
+        // These make little sense for a casting mon.
+        if (artefact_wpn_property(doodad, ARTP_BERSERK)
+            || artefact_wpn_property(doodad, ARTP_ANGRY)
+            || artefact_wpn_property(doodad, ARTP_PREVENT_SPELLCASTING)
+            || artefact_wpn_property(doodad, ARTP_CAUSE_TELEPORTATION)
+            || artefact_wpn_property(doodad, ARTP_PREVENT_TELEPORTATION)
+            || artefact_wpn_property(doodad, ARTP_MUTAGENIC)
+            || artefact_wpn_property(doodad, ARTP_PONDEROUS))
+        {
+            continue;
+        }
+    }
+    ASSERT(is_artefact(doodad));
+    ASSERT(doodad.sub_type == item.sub_type);
+
+    doodad.props[ARTEFACT_NAME_KEY].get_string()
+        = item.props[ARTEFACT_NAME_KEY].get_string();
+    doodad.props[ARTEFACT_APPEAR_KEY].get_string()
+        = item.props[ARTEFACT_APPEAR_KEY].get_string();
+    item.props = doodad.props;
+    item.plus = 2 + random2(5);
+}
+
+bool make_item_unrandart(item_def &item, int unrand_index)
 {
     ASSERT(unrand_index > UNRAND_START);
     ASSERT(unrand_index < (UNRAND_START + NO_UNRANDARTS));
@@ -2036,14 +1992,14 @@ bool make_item_unrandart( item_def &item, int unrand_index )
     _init_artefact_properties(item);
 
     if (unrand->prpty[ARTP_CURSED] != 0)
-        do_curse_item( item );
+        do_curse_item(item);
 
     // get true artefact name
-    ASSERT(!item.props.exists( ARTEFACT_NAME_KEY ));
+    ASSERT(!item.props.exists(ARTEFACT_NAME_KEY));
     item.props[ARTEFACT_NAME_KEY].get_string() = unrand->name;
 
     // get artefact appearance
-    ASSERT(!item.props.exists( ARTEFACT_APPEAR_KEY ));
+    ASSERT(!item.props.exists(ARTEFACT_APPEAR_KEY));
     item.props[ARTEFACT_APPEAR_KEY].get_string() = unrand->unid_name;
 
     set_unique_item_status(unrand_index, UNIQ_EXISTS);
@@ -2053,15 +2009,17 @@ bool make_item_unrandart( item_def &item, int unrand_index )
         item.plus  = random_range(-4, 16);
         item.plus2 = random_range(-4, 16);
     }
+    else if (unrand_index == UNRAND_FAERIE)
+        _make_faerie_armour(item);
 
     return (true);
 }
 
-const char *unrandart_descrip( int which_descrip, const item_def &item )
+const char *unrandart_descrip(int which_descrip, const item_def &item)
 {
     // Eventually it would be great to have randomly generated descriptions
     // for randarts.
-    const unrandart_entry *unrand = _seekunrandart( item );
+    const unrandart_entry *unrand = _seekunrandart(item);
 
     return ((which_descrip == 0) ? unrand->desc :
             (which_descrip == 1) ? unrand->desc_id :
@@ -2069,31 +2027,17 @@ const char *unrandart_descrip( int which_descrip, const item_def &item )
                                  : "Unknown.");
 }
 
-void artefact_clear_properties(item_def &item)
-{
-    ASSERT( is_artefact( item ) );
-    ASSERT( item.props.exists( ARTEFACT_PROPS_KEY ) );
-
-    CrawlVector &rap_vec = item.props[ARTEFACT_PROPS_KEY].get_vector();
-    ASSERT( rap_vec.get_type()     == SV_SHORT );
-    ASSERT( rap_vec.size()         == ART_PROPERTIES);
-    ASSERT( rap_vec.get_max_size() == ART_PROPERTIES);
-
-    for (vec_size i = 0; i < ART_PROPERTIES; i++)
-        rap_vec[i].get_short() = 0;
-}
-
 // Set all non-zero properties in proprt on the randart supplied.
-void artefact_merge_properties(item_def &item,
-                               const artefact_properties_t &proprt)
+static void _artefact_merge_properties(item_def &item,
+                                       const artefact_properties_t &proprt)
 {
-    ASSERT( is_artefact( item ) );
-    ASSERT( item.props.exists( ARTEFACT_PROPS_KEY ) );
+    ASSERT(is_artefact(item));
+    ASSERT(item.props.exists(ARTEFACT_PROPS_KEY));
 
     CrawlVector &rap_vec = item.props[ARTEFACT_PROPS_KEY].get_vector();
-    ASSERT( rap_vec.get_type()     == SV_SHORT );
-    ASSERT( rap_vec.size()         == ART_PROPERTIES);
-    ASSERT( rap_vec.get_max_size() == ART_PROPERTIES);
+    ASSERT(rap_vec.get_type()     == SV_SHORT);
+    ASSERT(rap_vec.size()         == ART_PROPERTIES);
+    ASSERT(rap_vec.get_max_size() == ART_PROPERTIES);
 
     for (vec_size i = 0; i < ART_PROPERTIES; i++)
     {
@@ -2103,24 +2047,17 @@ void artefact_merge_properties(item_def &item,
     }
 }
 
-void artefact_set_properties(item_def &item,
-                             const artefact_properties_t &proprt)
-{
-    artefact_clear_properties(item);
-    artefact_merge_properties(item, proprt);
-}
-
-void artefact_set_property( item_def          &item,
+void artefact_set_property(item_def          &item,
                             artefact_prop_type prop,
-                            int                val )
+                            int                val)
 {
-    ASSERT( is_artefact( item ) );
-    ASSERT( item.props.exists( ARTEFACT_PROPS_KEY ) );
+    ASSERT(is_artefact(item));
+    ASSERT(item.props.exists(ARTEFACT_PROPS_KEY));
 
     CrawlVector &rap_vec = item.props[ARTEFACT_PROPS_KEY].get_vector();
-    ASSERT( rap_vec.get_type()     == SV_SHORT );
-    ASSERT( rap_vec.size()         == ART_PROPERTIES);
-    ASSERT( rap_vec.get_max_size() == ART_PROPERTIES);
+    ASSERT(rap_vec.get_type()     == SV_SHORT);
+    ASSERT(rap_vec.size()         == ART_PROPERTIES);
+    ASSERT(rap_vec.get_max_size() == ART_PROPERTIES);
 
     rap_vec[prop].get_short() = val;
 }
@@ -2134,16 +2071,16 @@ void cheibriados_make_item_ponderous(item_def &item)
         item.flags |= ISFLAG_RANDART;
         _artefact_setup_prop_vectors(item);
         const std::string suffix =
-            artefact_name_lookup(
+            _artefact_name_lookup(
                 item,
                 god_name(GOD_CHEIBRIADOS) + " ponderous");
-        artefact_set_name(item, item_base_name(item) + " " + suffix);
+        set_artefact_name(item, item_base_name(item) + " " + suffix);
     }
     artefact_properties_t props;
     props.init(0);
     props[ARTP_PONDEROUS] = true;
     props[ARTP_BRAND] = brand;
-    artefact_merge_properties(item, props);
+    _artefact_merge_properties(item, props);
 
     if (Options.autoinscribe_artefacts)
         add_autoinscription(item, artefact_auto_inscription(item));
