@@ -1,16 +1,15 @@
-/*
- *  File:         bitary.h
- *  Summary:      Bit array data type.
- *  Created by:   Robert Vollmert
+/**
+ * @file
+ * @brief Bit array data type.
  *
  * Just contains the operations required by los.cc
  * for the moment.
- */
+**/
 
 #ifndef BITARY_H
 #define BITARY_H
 
-struct bit_array
+class bit_array
 {
 public:
     bit_array(unsigned long size = 0);
