@@ -1,8 +1,7 @@
-/*
- *  File:       chardump.cc
- *  Summary:    Dumps character info out to the morgue file.
- *  Written by: Linley Henzell
- */
+/**
+ * @file
+ * @brief Dumps character info out to the morgue file.
+**/
 
 #include "AppHdr.h"
 
@@ -35,7 +34,6 @@
 #include "itemname.h"
 #include "items.h"
 #include "kills.h"
-#include "macro.h"
 #include "message.h"
 #include "menu.h"
 #include "mutation.h"
@@ -51,10 +49,12 @@
 #include "spl-cast.h"
 #include "spl-util.h"
 #include "stash.h"
+#include "state.h"
 #include "stuff.h"
 #include "env.h"
 #include "transform.h"
 #include "travel.h"
+#include "unicode.h"
 #include "view.h"
 #include "viewchar.h"
 #include "xom.h"
@@ -90,7 +90,8 @@ static void _sdump_separator(dump_params &);
 #ifdef CLUA_BINDINGS
 static void _sdump_lua(dump_params &);
 #endif
-static bool write_dump(const std::string &fname, dump_params &);
+static bool _write_dump(const std::string &fname, dump_params &,
+                        bool print_dump_path = false);
 
 struct dump_section_handler
 {
@@ -180,12 +181,18 @@ bool dump_char(const std::string &fname, bool show_prices, bool full_id,
         dump_section(par);
     }
 
-    return write_dump(fname, par);
+    return _write_dump(fname, par, se == NULL);
 }
 
 static void _sdump_header(dump_params &par)
 {
-    par.text += " " CRAWL " version " + Version::Long();
+    std::string type = crawl_state.game_type_name();
+    if (type.empty())
+        type = CRAWL;
+    else
+        type += " DCSS";
+
+    par.text += " " + type + " version " + Version::Long();
     par.text += " character file.\n\n";
 }
 
@@ -226,11 +233,11 @@ static void _sdump_hunger(dump_params &par)
 static void _sdump_transform(dump_params &par)
 {
     std::string &text(par.text);
-    if (you.attribute[ATTR_TRANSFORMATION])
+    if (you.form)
     {
         std::string verb = par.se? "were" : "are";
 
-        switch (you.attribute[ATTR_TRANSFORMATION])
+        switch (you.form)
         {
         case TRAN_SPIDER:
             text += "You " + verb + " in spider-form.";
@@ -242,7 +249,7 @@ static void _sdump_transform(dump_params &par)
             text += "bat-form.";
             break;
         case TRAN_BLADE_HANDS:
-            text += "Your hands " + verb + " blades.";
+            text += "Your " + blade_parts() + " " + verb + " blades.";
             break;
         case TRAN_STATUE:
             text += "You " + verb + " a stone statue.";
@@ -259,6 +266,8 @@ static void _sdump_transform(dump_params &par)
         case TRAN_PIG:
             text += "You " + verb + " a filthy swine.";
             break;
+        case TRAN_NONE:
+            break;
         }
 
         text += "\n\n";
@@ -284,28 +293,28 @@ static void _sdump_visits(dump_params &par)
     for (unsigned int i = 0; i < branches_visited.size(); i++)
         branches_total += branches_visited[i];
 
-    text += make_stringf("You %svisited %ld branch",
+    text += make_stringf("You %svisited %d branch",
                          have.c_str(), branches_visited.size());
     if (branches_visited.size() != 1)
         text += "es";
-    text += make_stringf(" of the dungeon, and %s %ld of its levels.\n",
+    text += make_stringf(" of the dungeon, and %s %d of its levels.\n",
                          seen.c_str(), branches_total.levels_seen);
 
     PlaceInfo place_info = you.get_place_info(LEVEL_PANDEMONIUM);
     if (place_info.num_visits > 0)
     {
-        text += make_stringf("You %svisited Pandemonium %ld time",
+        text += make_stringf("You %svisited Pandemonium %d time",
                              have.c_str(), place_info.num_visits);
         if (place_info.num_visits > 1)
             text += "s";
-        text += make_stringf(", and %s %ld of its levels.\n",
+        text += make_stringf(", and %s %d of its levels.\n",
                              seen.c_str(), place_info.levels_seen);
     }
 
     place_info = you.get_place_info(LEVEL_ABYSS);
     if (place_info.num_visits > 0)
     {
-        text += make_stringf("You %svisited the Abyss %ld time",
+        text += make_stringf("You %svisited the Abyss %d time",
                              have.c_str(), place_info.num_visits);
         if (place_info.num_visits > 1)
             text += "s";
@@ -315,7 +324,7 @@ static void _sdump_visits(dump_params &par)
     place_info = you.get_place_info(LEVEL_LABYRINTH);
     if (place_info.num_visits > 0)
     {
-        text += make_stringf("You %svisited %ld Labyrinth",
+        text += make_stringf("You %svisited %d Labyrinth",
                              have.c_str(), place_info.num_visits);
         if (place_info.num_visits > 1)
             text += "s";
@@ -362,18 +371,18 @@ static void _sdump_visits(dump_params &par)
 
         if (num_zigs > 0)
         {
-            text += make_stringf("You %svisited %ld Ziggurat",
+            text += make_stringf("You %svisited %d Ziggurat",
                                  have.c_str(), num_zigs);
             if (num_zigs > 1)
                 text += "s";
-            text += make_stringf(", and %s %ld of %s levels.\n",
+            text += make_stringf(", and %s %d of %s levels.\n",
                                  seen.c_str(), zig_levels,
                                  num_zigs > 1 ? "their" : "its");
         }
 
         if (!misc_portals.empty())
         {
-            text += make_stringf("You %svisited %ld portal chamber",
+            text += make_stringf("You %svisited %d portal chamber",
                                  have.c_str(), misc_portals.size());
             if (misc_portals.size() > 1)
                 text += "s";
@@ -548,84 +557,30 @@ static void _sdump_lua(dump_params &par)
  //
  // munge_description
  //
- // Convert dollar signs to EOL and word wrap to 80 characters.
- // (for some obscure reason get_item_description uses dollar
- // signs instead of EOL).
- //  - It uses $ signs because they're easier to manipulate than the EOL
- //  macro, which is of uncertain length (well, that and I didn't know how
- //  to do it any better at the time) (LH)
+ // word wrap to 80 characters.
+ // XXX: should be replaced by some other linewrapping function
+ //      now EOL munging is gone
  //---------------------------------------------------------------
-std::string munge_description(const std::string & inStr)
+std::string munge_description(std::string inStr)
 {
     std::string outStr;
-    std::string eol = "\n";
 
     outStr.reserve(inStr.length() + 32);
 
     const int kIndent = 3;
-    int lineLen = kIndent;
 
-    unsigned int i = 0;
+    if (inStr.empty()) // always at least an empty line
+        return "\n";
 
-    outStr += std::string(kIndent, ' ');
-
-    while (i < inStr.length())
+    while (!inStr.empty())
     {
-        const char ch = inStr[i];
-
-        if (ch == '$')
-        {
-            outStr += eol;
-
-            outStr += std::string(kIndent, ' ');
-            lineLen = kIndent;
-
-            while (inStr[++i] == '$')
-                ;
-        }
-        else if (isspace(ch))
-        {
-            if (lineLen >= 79)
-            {
-                outStr += eol;
-                outStr += std::string(kIndent, ' ');
-                lineLen = kIndent;
-
-            }
-            else if (lineLen > 0)
-            {
-                outStr += ch;
-                ++lineLen;
-            }
-            ++i;
-        }
-        else
-        {
-            std::string word;
-
-            while (i < inStr.length()
-                   && lineLen + word.length() < 79
-                   && !isspace(inStr[i]) && inStr[i] != '$')
-            {
-                word += inStr[i++];
-            }
-
-            if (lineLen + word.length() >= 79)
-            {
-                outStr += eol;
-                outStr += std::string(kIndent, ' ');
-                lineLen = kIndent;
-            }
-
-            outStr += word;
-            lineLen += word.length();
-        }
+        outStr += std::string(kIndent, ' ')
+                + wordwrap_line(inStr, 79 - kIndent)
+                + "\n";
     }
 
-    outStr += eol;
-
     return (outStr);
-}                               // end munge_description()
+}
 
 static void _sdump_messages(dump_params &par)
 {
@@ -646,12 +601,12 @@ static void _sdump_screenshot(dump_params &par)
 static void _sdump_notes(dump_params &par)
 {
     std::string &text(par.text);
-    if ( note_list.empty() )
+    if (note_list.empty())
         return;
 
     text += "\nNotes\nTurn   | Place   | Note\n";
     text += "--------------------------------------------------------------\n";
-    for ( unsigned i = 0; i < note_list.size(); ++i )
+    for (unsigned i = 0; i < note_list.size(); ++i)
     {
         text += note_list[i].describe();
         text += "\n";
@@ -733,14 +688,14 @@ static bool _dump_item_origin(const item_def &item, int value)
         return (true);
     }
     if (fs(IODS_EGO_ARMOUR) && item.base_type == OBJ_ARMOUR
-        && item_type_known( item ))
+        && item_type_known(item))
     {
-        const int spec_ench = get_armour_ego_type( item );
+        const int spec_ench = get_armour_ego_type(item);
         return (spec_ench != SPARM_NORMAL);
     }
 
     if (fs(IODS_EGO_WEAPON) && item.base_type == OBJ_WEAPONS
-        && item_type_known( item ))
+        && item_type_known(item))
     {
         return (get_weapon_brand(item) != SPWPN_NORMAL);
     }
@@ -773,7 +728,7 @@ static bool _dump_item_origin(const item_def &item, int value)
     if (refpr == -1)
         return (false);
     if (value == -1)
-        value = item_value( item, false );
+        value = item_value(item, false);
     return (value >= refpr);
 #undef fs
 }
@@ -799,7 +754,7 @@ static void _sdump_inventory(dump_params &par)
 
     for (i = 0; i < ENDOFPACK; i++)
     {
-        if (you.inv[i].is_valid())
+        if (you.inv[i].defined())
         {
             // adds up number of each class in invent.
             inv_class2[you.inv[i].base_type]++;
@@ -837,13 +792,13 @@ static void _sdump_inventory(dump_params &par)
                 case OBJ_CORPSES:    text += "Carrion";         break;
 
                 default:
-                    DEBUGSTR("Bad item class");
+                    die("Bad item class");
                 }
                 text += "\n";
 
                 for (j = 0; j < ENDOFPACK; j++)
                 {
-                    if (you.inv[j].is_valid() && you.inv[j].base_type == i)
+                    if (you.inv[j].defined() && you.inv[j].base_type == i)
                     {
                         text += " ";
                         text += you.inv[j].name(DESC_INVENTORY_EQUIP);
@@ -855,8 +810,8 @@ static void _sdump_inventory(dump_params &par)
                         {
                             text += " (";
 
-                            itoa( ival = item_value( you.inv[j], true ),
-                                  tmp_quant, 10 );
+                            itoa(ival = item_value(you.inv[j], true),
+                                  tmp_quant, 10);
 
                             text += tmp_quant;
                             text += " gold)";
@@ -868,13 +823,13 @@ static void _sdump_inventory(dump_params &par)
                             text += "\n" "   (" + origin_desc(you.inv[j]) + ")";
                         }
 
-                        if (is_dumpable_artefact( you.inv[j], false )
+                        if (is_dumpable_artefact(you.inv[j], false)
                             || Options.dump_book_spells
                                && you.inv[j].base_type == OBJ_BOOKS)
                         {
-                            text2 = get_item_description( you.inv[j],
+                            text2 = get_item_description(you.inv[j],
                                                           false,
-                                                          true );
+                                                          true);
 
                             text += munge_description(text2);
                         }
@@ -905,7 +860,7 @@ static void _sdump_skills(dump_params &par)
     else
         text += " You have ";
 
-    itoa( you.exp_available, tmp_quant, 10 );
+    itoa(you.exp_available, tmp_quant, 10);
     text += tmp_quant;
     text += " experience left.";
 
@@ -957,7 +912,8 @@ static void _sdump_spells(dump_params &par)
         SPTYP_EARTH,
         SPTYP_AIR,
         SPTYP_CONJURATION,
-        SPTYP_ENCHANTMENT,
+        SPTYP_HEXES,
+        SPTYP_CHARMS,
         SPTYP_DIVINATION,
         SPTYP_TRANSLOCATION,
         SPTYP_SUMMONING,
@@ -984,7 +940,7 @@ static void _sdump_spells(dump_params &par)
             text += "You had ";
         else
             text += "You have ";
-        itoa( spell_levels, tmp_quant, 10 );
+        itoa(spell_levels, tmp_quant, 10);
         text += tmp_quant;
         text += " spell levels left.";
     }
@@ -1003,12 +959,12 @@ static void _sdump_spells(dump_params &par)
 
         text += "You " + verb + " the following spells:\n\n";
 
-        text += " Your Spells              Type           Power          Success   Level" "\n";
+        text += " Your Spells              Type           Power        Success   Level  Hunger" "\n";
 
         for (int j = 0; j < 52; j++)
         {
-            const char letter = index_to_letter( j );
-            const spell_type spell  = get_spell_by_letter( letter );
+            const char letter = index_to_letter(j);
+            const spell_type spell  = get_spell_by_letter(letter);
 
             if (spell != SPELL_NO_SPELL)
             {
@@ -1016,19 +972,16 @@ static void _sdump_spells(dump_params &par)
 
                 spell_line += letter;
                 spell_line += " - ";
-                spell_line += spell_title( spell );
+                spell_line += spell_title(spell);
 
-                if ( spell_line.length() > 24 )
-                    spell_line = spell_line.substr(0, 24);
-
-                for (int i = spell_line.length(); i < 26; i++)
-                    spell_line += ' ';
+                spell_line = chop_string(spell_line, 24);
+                spell_line += "  ";
 
                 bool already = false;
 
                 for (int i = 0; spell_type_index[i] != 0; i++)
                 {
-                    if (spell_typematch( spell, spell_type_index[i] ))
+                    if (spell_typematch(spell, spell_type_index[i]))
                     {
                         spell_line += spell_type_shortname(spell_type_index[i],
                                                            already);
@@ -1036,21 +989,22 @@ static void _sdump_spells(dump_params &par)
                     }
                 }
 
-                for (int i = spell_line.length(); i < 41; ++i )
-                    spell_line += ' ';
+                spell_line = chop_string(spell_line, 41);
 
                 spell_line += spell_power_string(spell);
 
-                for (int i = spell_line.length(); i < 56; ++i )
-                    spell_line += ' ';
+                spell_line = chop_string(spell_line, 54);
 
                 spell_line += failure_rate_to_string(spell_fail(spell));
 
-                for (int i = spell_line.length(); i < 68; i++)
-                    spell_line += ' ';
+                spell_line = chop_string(spell_line, 66);
 
-                itoa(spell_difficulty(spell), tmp_quant, 10 );
+                itoa(spell_difficulty(spell), tmp_quant, 10);
                 spell_line += tmp_quant;
+
+                spell_line = chop_string(spell_line, 71);
+
+                spell_line += spell_hunger_string(spell);
                 spell_line += "\n";
 
                 text += spell_line;
@@ -1069,7 +1023,6 @@ static void _sdump_kills(dump_params &par)
 static std::string _sdump_kills_place_info(PlaceInfo place_info,
                                           std::string name = "")
 {
-    PlaceInfo   gi = you.global_info;
     std::string out;
 
     if (name.empty())
@@ -1155,7 +1108,7 @@ static void _sdump_kills_by_place(dump_params &par)
         result += _sdump_kills_place_info(pi);
     }
 
-    if (result.length() > 0)
+    if (!result.empty())
         text += header + result + footer + "\n";
 }
 
@@ -1173,7 +1126,7 @@ static void _sdump_hiscore(dump_params &par)
     if (!par.se)
         return;
 
-    std::string hiscore = hiscores_format_single_long( *(par.se), true );
+    std::string hiscore = hiscores_format_single_long(*(par.se), true);
     trim_string(hiscore);
     par.text += hiscore;
     par.text += "\n\n";
@@ -1193,7 +1146,7 @@ static void _sdump_vault_list(dump_params &par)
 #ifdef WIZARD
         || you.wizard
 #endif
-       )
+     )
     {
         par.text += "Vault maps used:\n\n";
         par.text += dump_vault_maps();
@@ -1230,7 +1183,7 @@ const char *hunger_level(void)
                                  : (vamp ? "almost alive" : "completely stuffed"));
 }
 
-static std::string morgue_directory()
+std::string morgue_directory()
 {
     std::string dir = (!Options.morgue_dir.empty() ? Options.morgue_dir :
                        !SysEnv.crawl_dir.empty()   ? SysEnv.crawl_dir
@@ -1244,13 +1197,6 @@ static std::string morgue_directory()
 
 void dump_map(FILE *fp, bool debug, bool dist)
 {
-    // Duplicate the screenshot() trick.
-    FixedVector<unsigned, NUM_DCHAR_TYPES> char_table_bk;
-    char_table_bk = Options.char_table;
-
-    init_char_table(CSET_ASCII);
-    init_show_table();
-
     if (debug)
     {
         // Write the whole map out without checking for mappedness. Handy
@@ -1261,7 +1207,7 @@ void dump_map(FILE *fp, bool debug, bool dist)
             {
                 if (you.pos() == coord_def(x, y))
                     fputc('@', fp);
-                else if (grd[x][y] == DNGN_FLOOR_SPECIAL)
+                else if (testbits(env.pgrid[x][y], FPROP_HIGHLIGHT))
                     fputc('?', fp);
                 else if (dist && grd[x][y] == DNGN_FLOOR
                          && travel_point_distance[x][y] > 0
@@ -1270,7 +1216,10 @@ void dump_map(FILE *fp, bool debug, bool dist)
                     fputc('0' + travel_point_distance[x][y], fp);
                 }
                 else
-                    fputc(get_feature_def(grd[x][y]).symbol, fp);
+                {
+                    fputs(OUTS(stringize_glyph(
+                               get_feature_def(grd[x][y]).symbol)), fp);
+                }
             }
             fputc('\n', fp);
         }
@@ -1292,15 +1241,14 @@ void dump_map(FILE *fp, bool debug, bool dist)
         for (int y = min_y; y <= max_y; ++y)
         {
             for (int x = min_x; x <= max_x; ++x)
-                fputc( env.map_knowledge[x][y].glyph(), fp );
+            {
+                fputs(OUTS(stringize_glyph(
+                           get_cell_glyph(coord_def(x, y)).ch)), fp);
+            }
 
             fputc('\n', fp);
         }
     }
-
-    // Restore char and feature tables
-    Options.char_table = char_table_bk;
-    init_show_table();
 }
 
 void dump_map(const char* fname, bool debug, bool dist)
@@ -1314,7 +1262,8 @@ void dump_map(const char* fname, bool debug, bool dist)
     fclose(fp);
 }
 
-static bool write_dump(const std::string &fname, dump_params &par)
+static bool _write_dump(const std::string &fname, dump_params &par,
+                        bool print_dump_path)
 {
     bool succeeded = false;
 
@@ -1335,15 +1284,21 @@ static bool write_dump(const std::string &fname, dump_params &par)
     file_name += ".txt";
     FILE *handle = fopen_replace(file_name.c_str());
 
-#if DEBUG_DIAGNOSTICS
+#ifdef DEBUG_DIAGNOSTICS
     mprf(MSGCH_DIAGNOSTICS, "File name: %s", file_name.c_str());
 #endif
 
     if (handle != NULL)
     {
-        fputs(par.text.c_str(), handle);
+        fputs(OUTS(par.text), handle);
         fclose(handle);
         succeeded = true;
+        if (print_dump_path)
+#ifdef DGAMELAUNCH
+            mprf("Char dumped successfully.");
+#else
+            mprf("Char dumped to '%s'.", file_name.c_str());
+#endif
     }
     else
         mprf(MSGCH_ERROR, "Error opening file '%s'", file_name.c_str());
@@ -1356,6 +1311,7 @@ void display_notes()
     formatted_scroller scr;
     scr.set_flags(MF_START_AT_END);
     scr.set_tag("notes");
+    scr.set_highlighter(new MenuHighlighter);
     scr.set_title(new MenuEntry("Turn   | Place   | Note"));
     for (unsigned int i = 0; i < note_list.size(); ++i)
     {
@@ -1398,6 +1354,7 @@ void whereis_record(const char *status)
 
     if (FILE *handle = fopen_replace(file_name.c_str()))
     {
+        // no need to bother with supporting ancient charsets for DGL
         fprintf(handle, "%s:status=%s\n",
                 xlog_status_line().c_str(),
                 status? status : "");
@@ -1405,3 +1362,142 @@ void whereis_record(const char *status)
     }
 }
 #endif
+
+
+///////////////////////////////////////////////////////////////////////////////
+// Turn timestamps
+//
+// For DGL installs, write a timestamp at regular intervals into a file in
+// the morgue directory. The timestamp file is named
+// "timestamp-<player>-<starttime>.ts". All timestamps are standard Unix
+// time_t, but currently only the low 4 bytes are saved even on systems
+// with 64-bit time_t.
+//
+// Timestamp files are append only, and Crawl will check and handle cases
+// where a previous Crawl process crashed at a higher turn count on the same
+// game.
+//
+// Having timestamps associated with the game allows for much easier seeking
+// within Crawl ttyrecs by external tools such as FooTV.
+
+#ifdef DGL_TURN_TIMESTAMPS
+
+#include "syscalls.h"
+#include <sys/stat.h>
+
+// File-format version for timestamp files. Crawl will never append to a
+const uint32_t DGL_TIMESTAMP_VERSION = 1;
+const int VERSION_SIZE = sizeof(DGL_TIMESTAMP_VERSION);
+const int TIMESTAMP_SIZE = sizeof(uint32_t);
+
+// Returns the name of the timestamp file based on the morgue_dir,
+// character name and the game start time.
+std::string dgl_timestamp_filename()
+{
+    const std::string filename =
+        ("timestamp-" + you.your_name + "-" + make_file_time(you.birth_time));
+    return morgue_directory() + strip_filename_unsafe_chars(filename) + ".ts";
+}
+
+// Returns true if the given file exists and is not a timestamp file
+// of a known version.
+bool dgl_unknown_timestamp_file(const std::string &filename)
+{
+    if (FILE *inh = fopen_u(filename.c_str(), "rb"))
+    {
+        reader r(inh);
+        const uint32_t file_version = unmarshallInt(r);
+        fclose(inh);
+        return (file_version != DGL_TIMESTAMP_VERSION);
+    }
+    return (false);
+}
+
+// Returns a filehandle to use to write turn timestamps, NULL if
+// timestamps should not be written.
+FILE *dgl_timestamp_filehandle()
+{
+    static FILE *timestamp_file;
+    static bool opened_file = false;
+    if (!opened_file)
+    {
+        opened_file = true;
+
+        const std::string filename = dgl_timestamp_filename();
+        // First check if there's already a timestamp file. If it exists
+        // but has a different version, we cannot safely modify it, so bail.
+        if (!dgl_unknown_timestamp_file(filename))
+            timestamp_file = fopen_u(filename.c_str(), "ab");
+    }
+    return timestamp_file;
+}
+
+// Records a timestamp in the .ts file at the given offset. If no timestamp
+// file exists, a new file will be created.
+void dgl_record_timestamp(unsigned long file_offset, time_t time)
+{
+    static bool timestamp_first_write = true;
+    if (FILE *ftimestamp = dgl_timestamp_filehandle())
+    {
+        writer w(dgl_timestamp_filename(), ftimestamp, true);
+        if (timestamp_first_write)
+        {
+            unsigned long ts_size = file_size(ftimestamp);
+            if (!ts_size)
+            {
+                marshallInt(w, DGL_TIMESTAMP_VERSION);
+                ts_size += sizeof(DGL_TIMESTAMP_VERSION);
+            }
+
+            // It's possible that the file we want to write is already
+            // larger than the offset we expect if the game previously
+            // crashed. When the game crashes, turn count is
+            // effectively rewound to the point of the last save. In
+            // such cases, we should not add timestamps until we reach
+            // the correct turn count again.
+            if (ts_size && ts_size > file_offset)
+                return;
+
+            if (file_offset > ts_size)
+            {
+                const int backlog =
+                    (file_offset - ts_size) / TIMESTAMP_SIZE;
+                for (int i = 0; i < backlog; ++i)
+                    marshallInt(w, 0);
+            }
+
+            timestamp_first_write = false;
+        }
+        fseek(ftimestamp, 0, SEEK_END);
+        // [ds] FIXME: Eventually switch to 8 byte timestamps.
+        marshallInt(w, static_cast<uint32_t>(time));
+        fflush(ftimestamp);
+    }
+}
+
+// Record timestamps every so many turns:
+const int TIMESTAMP_TURN_INTERVAL = 100;
+// Stop recording timestamps after this turncount.
+const long TIMESTAMP_TURN_MAX = 500000L;
+void dgl_record_timestamp(long turn)
+{
+    if (turn && turn < TIMESTAMP_TURN_MAX && !(turn % TIMESTAMP_TURN_INTERVAL))
+    {
+        const time_t now = time(NULL);
+        const unsigned long offset =
+            (VERSION_SIZE +
+             (turn / TIMESTAMP_TURN_INTERVAL - 1) * TIMESTAMP_SIZE);
+        dgl_record_timestamp(offset, now);
+    }
+}
+
+#endif
+
+// Records a timestamp for the current player turn if appropriate.
+void record_turn_timestamp()
+{
+#ifdef DGL_TURN_TIMESTAMPS
+    if (crawl_state.need_save)
+        dgl_record_timestamp(you.num_turns);
+#endif
+}
