@@ -1,8 +1,3 @@
-/*
- *  File:       clua.cc
- *  Created by: dshaligram on Wed Aug 2 12:54:15 2006 UTC
- */
-
 #include "AppHdr.h"
 
 #include "clua.h"
@@ -15,6 +10,8 @@
 #include "libutil.h"
 #include "state.h"
 #include "stuff.h"
+#include "syscalls.h"
+#include "unicode.h"
 
 #include <algorithm>
 
@@ -84,7 +81,7 @@ void CLua::getglobal(const char *name)
 std::string CLua::setuniqregistry()
 {
     char name[100];
-    snprintf(name, sizeof name, "__cru%lu", uniqindex++);
+    snprintf(name, sizeof name, "__cru%u", uniqindex++);
     lua_pushstring(state(), name);
     lua_insert(state(), -2);
     lua_settable(state(), LUA_REGISTRYINDEX);
@@ -111,15 +108,19 @@ void CLua::getregistry(const char *name)
     _getregistry(state(), name);
 }
 
-void CLua::save(const char *file)
+void CLua::gc()
+{
+    lua_gc(state(), LUA_GCCOLLECT, 0);
+}
+
+void CLua::save(writer &outf)
 {
     if (!_state)
         return;
 
-    CLuaSave clsave = { file, NULL };
-    callfn("c_save", "u", &clsave);
-    if (clsave.handle)
-        fclose(clsave.handle);
+    std::string res;
+    callfn("c_save", ">s", &res);
+    outf.write(res.c_str(), res.size());
 }
 
 int CLua::file_write(lua_State *ls)
@@ -129,7 +130,7 @@ int CLua::file_write(lua_State *ls)
         luaL_argerror(ls, 1, "Expected filehandle at arg 1");
         return (0);
     }
-    CLuaSave *sf = static_cast<CLuaSave *>( lua_touserdata(ls, 1) );
+    CLuaSave *sf = static_cast<CLuaSave *>(lua_touserdata(ls, 1));
     if (!sf)
         return (0);
 
@@ -146,7 +147,7 @@ int CLua::file_write(lua_State *ls)
 FILE *CLua::CLuaSave::get_file()
 {
     if (!handle)
-        handle = fopen(filename, "w");
+        handle = fopen_u(filename, "w");
 
     return (handle);
 }
@@ -244,12 +245,21 @@ int CLua::loadfile(lua_State *ls, const char *filename, bool trusted,
                        make_stringf("Can't find \"%s\"", filename).c_str());
         return (-1);
     }
-    return (luaL_loadfile(ls, file.c_str()));
+
+    FileLineInput f(file.c_str());
+    std::string script;
+    while (!f.eof())
+        script += f.get_line() + "\n";
+
+    // prefixing with @ stops lua from adding [string "%s"]
+    return luaL_loadbuffer(ls, &script[0], script.length(),
+                           ("@" + file).c_str());
 }
 
-int CLua::execfile(const char *filename, bool trusted, bool die_on_fail)
+int CLua::execfile(const char *filename, bool trusted, bool die_on_fail,
+                   bool force)
 {
-    if (sourced_files.find(filename) != sourced_files.end())
+    if (!force && sourced_files.find(filename) != sourced_files.end())
         return 0;
 
     sourced_files.insert(filename);
@@ -280,7 +290,7 @@ bool CLua::runhook(const char *hook, const char *params, ...)
     if (!lua_istable(ls, -1))
     {
         lua_pop(ls, 1);
-        CL_RESETSTACK_RETURN( ls, stack_top, false );
+        CL_RESETSTACK_RETURN(ls, stack_top, false);
     }
     for (int i = 1; ; ++i)
     {
@@ -300,7 +310,7 @@ bool CLua::runhook(const char *hook, const char *params, ...)
 
         lua_settop(ls, currtop);
     }
-    CL_RESETSTACK_RETURN( ls, stack_top, true );
+    CL_RESETSTACK_RETURN(ls, stack_top, true);
 }
 
 void CLua::fnreturns(const char *format, ...)
@@ -404,6 +414,7 @@ int CLua::push_args(lua_State *ls, const char *format, va_list args,
             lua_pushnumber(ls, va_arg(args, int));
             break;
         case 'L':
+            die("ambiguous long in Lua push_args");
             lua_pushnumber(ls, va_arg(args, long));
             break;
         case 'b':
@@ -416,7 +427,7 @@ int CLua::push_args(lua_State *ls, const char *format, va_list args,
             clua_push_map(ls, va_arg(args, map_def *));
             break;
         case 'M':
-            push_monster(ls, va_arg(args, monsters *));
+            push_monster(ls, va_arg(args, monster*));
             break;
         case 'I':
             lua_push_moninf(ls, va_arg(args, monster_info *));
@@ -728,7 +739,7 @@ void CLua::print_stack()
     int              i = 0;
     lua_State       *L = state();
 
-    fprintf(stderr, EOL);
+    fprintf(stderr, "\n");
     while (lua_getstack(L, i++, &dbg) == 1)
     {
         lua_getinfo(L, "lnuS", &dbg);
@@ -739,11 +750,11 @@ void CLua::print_stack()
         else
             file++;
 
-        fprintf(stderr, "%s, function %s, line %d" EOL, file,
+        fprintf(stderr, "%s, function %s, line %d\n", file,
                 dbg.name, dbg.currentline);
     }
 
-    fprintf(stderr, EOL);
+    fprintf(stderr, "\n");
 }
 
 ////////////////////////////////////////////////////////////////////////
@@ -772,7 +783,7 @@ static lua_pat_op pat_ops[] =
     { "||", " or ",  true,  true },
 };
 
-unsigned long lua_text_pattern::lfndx = 0L;
+unsigned int lua_text_pattern::lfndx = 0;
 
 bool lua_text_pattern::is_lua_pattern(const std::string &s)
 {
@@ -809,7 +820,7 @@ bool lua_text_pattern::valid() const
     return translated? isvalid : translate();
 }
 
-bool lua_text_pattern::matches( const std::string &s ) const
+bool lua_text_pattern::matches(const std::string &s) const
 {
     if (isvalid && !translated)
         translate();
@@ -823,7 +834,7 @@ bool lua_text_pattern::matches( const std::string &s ) const
 void lua_text_pattern::pre_pattern(std::string &pat, std::string &fn) const
 {
     // Trim trailing spaces
-    pat.erase( pat.find_last_not_of(" \t\n\r") + 1 );
+    pat.erase(pat.find_last_not_of(" \t\n\r") + 1);
 
     fn += " pmatch([[";
     fn += pat;
@@ -834,7 +845,7 @@ void lua_text_pattern::pre_pattern(std::string &pat, std::string &fn) const
 
 void lua_text_pattern::post_pattern(std::string &pat, std::string &fn) const
 {
-    pat.erase( 0, pat.find_first_not_of(" \t\n\r") );
+    pat.erase(0, pat.find_first_not_of(" \t\n\r"));
 
     fn += " pmatch([[";
     fn += pat;
@@ -845,7 +856,7 @@ void lua_text_pattern::post_pattern(std::string &pat, std::string &fn) const
 
 std::string lua_text_pattern::new_fn_name()
 {
-    return (make_stringf("__ch_stash_search_%lu", lfndx++));
+    return (make_stringf("__ch_stash_search_%u", lfndx++));
 }
 
 bool lua_text_pattern::translate() const
@@ -881,7 +892,7 @@ bool lua_text_pattern::translate() const
                 currop = &lop;
                 luafn += lop.luatok;
 
-                i += strlen( lop.token ) - 1;
+                i += strlen(lop.token) - 1;
 
                 break;
             }
@@ -900,7 +911,7 @@ bool lua_text_pattern::translate() const
 
     const_cast<lua_text_pattern *>(this)->translated = true;
 
-    int err = clua.execstring( luafn.c_str(), "stash-search" );
+    int err = clua.execstring(luafn.c_str(), "stash-search");
     if (err)
     {
         lua_text_pattern *self = const_cast<lua_text_pattern *>(this);
@@ -940,7 +951,7 @@ static int _clua_panic(lua_State *ls)
 
 static void *_clua_allocator(void *ud, void *ptr, size_t osize, size_t nsize)
 {
-    CLua *cl = static_cast<CLua *>( ud );
+    CLua *cl = static_cast<CLua *>(ud);
     cl->memory_used += nsize - osize;
 
     if (nsize > osize && cl->memory_used >= CLUA_MAX_MEMORY_USE * 1024
