@@ -1,8 +1,7 @@
-/*
- *  File:       command.h
- *  Summary:    Misc commands.
- *  Written by: Linley Henzell
- */
+/**
+ * @file
+ * @brief Misc commands.
+**/
 
 
 #ifndef COMMAND_H
@@ -18,12 +17,14 @@ void list_jewellery();
 void swap_inv_slots(int slot1, int slot2, bool verbose);
 
 void show_levelmap_help();
+void show_pickup_menu_help();
 void show_targeting_help();
 void show_interlevel_travel_branch_help();
 void show_interlevel_travel_depth_help();
 void show_stash_search_help();
 void show_butchering_help();
-void list_commands(int hotkey = 0, bool do_redraw_screen = false);
+void list_commands(int hotkey = 0, bool do_redraw_screen = false,
+                   std::string highlight_string = "");
 #ifdef WIZARD
 int list_wizard_commands(bool do_redraw_screen = false);
 #endif
