@@ -19,6 +19,10 @@ inline bool map_bounds(const coord_def &p)
     return map_bounds(p.x, p.y);
 }
 
+// Checks that a given point is within the map, excluding 'margin' squares at
+// the edge of the map.
+bool map_bounds_with_margin(coord_def p, int margin);
+
 // Determines if the coordinate is within bounds of an LOS array.
 inline bool show_bounds(const coord_def &p)
 {
@@ -26,11 +30,9 @@ inline bool show_bounds(const coord_def &p)
             && p.y >= 0 && p.y < ENV_SHOW_DIAMETER);
 }
 
-int grid_distance( const coord_def& p1, const coord_def& p2 );
-int grid_distance( int x, int y, int x2, int y2 );
-int distance( const coord_def& p1, const coord_def& p2 );
-int distance( int x, int y, int x2, int y2);
-bool adjacent( const coord_def& p1, const coord_def& p2 );
+int grid_distance(const coord_def& p1, const coord_def& p2);
+int distance(const coord_def& p1, const coord_def& p2);
+bool adjacent(const coord_def& p1, const coord_def& p2);
 
 // Conversion between different coordinate systems.
 // XXX: collect all of these here?
