@@ -1,9 +1,7 @@
-/*
- *  database.cc
- *
- *  Created by Peter Berger on 4/15/07.
- *  Copyright 2007 __MyCompanyName__. All rights reserved.
- */
+/**
+ * @file
+ * database.cc
+**/
 
 #include "AppHdr.h"
 
@@ -12,7 +10,6 @@
 #include <fcntl.h>
 #include <stdlib.h>
 #include <cstdlib>
-#include <fstream>
 #ifndef TARGET_COMPILER_VC
 #include <unistd.h>
 #endif
@@ -23,6 +20,12 @@
 #include "libutil.h"
 #include "random.h"
 #include "stuff.h"
+#include "syscalls.h"
+#include "unicode.h"
+
+static std::string _query_database(DBM *db, std::string key,
+                                   bool canonicalise_key, bool run_lua);
+static void _add_entry(DBM *db, const std::string &k, std::string &v);
 
 // TextDB handles dependency checking the db vs text files, creating the
 // db, loading, and destroying the DB.
@@ -46,18 +49,20 @@ class TextDB
     void _regenerate_db();
 
  private:
+    bool open_db();
     const char* const _db_name;            // relative to savedir
     std::vector<std::string> _input_files; // relative to datafile dirs
     DBM* _db;
+    time_t timestamp;
 };
 
 // Convenience functions for (read-only) access to generic
 // berkeley DB databases.
-static void _store_text_db(const std::string &in, const std::string &out);
+static void _store_text_db(const std::string &in, DBM *db);
 
 static TextDB AllDBs[] =
 {
-    TextDB( "db/descriptions",
+    TextDB("descriptions",
             "descript/features.txt",
             "descript/items.txt",
             "descript/unident.txt",
@@ -68,9 +73,17 @@ static TextDB AllDBs[] =
             "descript/skills.txt",
             "descript/ability.txt",
             "descript/cards.txt",
+#ifdef USE_TILE
+            "descript/commands.txt",
+#endif
             NULL),
 
-    TextDB( "db/randart",
+    TextDB("gamestart",
+            "descript/species.txt",
+            "descript/backgrounds.txt",
+            NULL),
+
+    TextDB("randart",
             "database/randname.txt",
             "database/rand_wpn.txt", // mostly weapons
             "database/rand_arm.txt", // mostly armour
@@ -80,44 +93,51 @@ static TextDB AllDBs[] =
             "database/monname.txt",  // orcish names for Beogh to choose from
             NULL),
 
-    TextDB( "db/speak",
+    TextDB("speak",
             "database/monspeak.txt", // monster speech
             "database/monspell.txt", // monster spellcasting speech
+            "database/monflee.txt",  // monster fleeing speech
             "database/wpnnoise.txt", // noisy weapon speech
             "database/insult.txt",   // imp/demon taunts
             "database/godspeak.txt", // god speech
             NULL),
 
-    TextDB( "db/shout",
+    TextDB("shout",
             "database/shout.txt",
             "database/insult.txt",   // imp/demon taunts, again
             NULL),
 
-    TextDB( "db/misc",
+    TextDB("misc",
             "database/miscname.txt", // names for miscellaneous things
             NULL),
 
-    TextDB( "db/quotes",
+    TextDB("quotes",
             "database/quotes.txt",   // quotes for items and monsters
             NULL),
 
-    TextDB( "db/help",               // database for outsourced help texts
+    TextDB("help",               // database for outsourced help texts
             "database/help.txt",
             NULL),
 
-    TextDB( "db/FAQ",                // database for Frequently Asked Questions
+    TextDB("FAQ",                // database for Frequently Asked Questions
             "database/FAQ.txt",
             NULL),
 };
 
 static TextDB& DescriptionDB = AllDBs[0];
-static TextDB& RandartDB     = AllDBs[1];
-static TextDB& SpeakDB       = AllDBs[2];
-static TextDB& ShoutDB       = AllDBs[3];
-static TextDB& MiscDB        = AllDBs[4];
-static TextDB& QuotesDB      = AllDBs[5];
-static TextDB& HelpDB        = AllDBs[6];
-static TextDB& FAQDB         = AllDBs[7];
+static TextDB& GameStartDB   = AllDBs[1];
+static TextDB& RandartDB     = AllDBs[2];
+static TextDB& SpeakDB       = AllDBs[3];
+static TextDB& ShoutDB       = AllDBs[4];
+static TextDB& MiscDB        = AllDBs[5];
+static TextDB& QuotesDB      = AllDBs[6];
+static TextDB& HelpDB        = AllDBs[7];
+static TextDB& FAQDB         = AllDBs[8];
+
+static std::string _db_cache_path(const std::string &db)
+{
+    return savedir_versioned_path("db/" + db);
+}
 
 // ----------------------------------------------------------------------
 // TextDB
@@ -125,7 +145,7 @@ static TextDB& FAQDB         = AllDBs[7];
 
 TextDB::TextDB(const char* db_name, ...)
     : _db_name(db_name),
-      _db(NULL)
+      _db(NULL), timestamp(-1)
 {
     va_list args;
     va_start(args, db_name);
@@ -136,22 +156,44 @@ TextDB::TextDB(const char* db_name, ...)
         if (input_file == 0)
             break;
 
-        ASSERT( strstr(input_file, ".txt") != 0 );      // probably forgot the terminating 0
+        // probably forgot the terminating 0
+        ASSERT(strstr(input_file, ".txt") != 0);
         _input_files.push_back(input_file);
     }
     va_end(args);
 }
 
+bool TextDB::open_db()
+{
+    if (_db)
+        return true;
+
+    const std::string full_db_path = _db_cache_path(_db_name);
+    _db = dbm_open(full_db_path.c_str(), O_RDONLY, 0660);
+    if (!_db)
+        return false;
+
+    std::string ts = _query_database(_db, "TIMESTAMP", false, false);
+    if (ts.empty())
+        return false;
+    char *err;
+    timestamp = strtol(ts.c_str(), &err, 10);
+    if (*err)
+        return false;
+
+    return true;
+}
+
 void TextDB::init()
 {
-    if (_needs_update())
-        _regenerate_db();
+    open_db();
 
-    const std::string full_db_path = get_savedir_path(_db_name);
-    _db = dbm_open(full_db_path.c_str(), O_RDONLY, 0660);
+    if (!_needs_update())
+        return;
+    _regenerate_db();
 
-    if (_db == NULL)
-        end(1, true, "Failed to open DB: %s", full_db_path.c_str());
+    if (!open_db())
+        end(1, true, "Failed to open DB: %s", _db_cache_path(_db_name).c_str());
 }
 
 void TextDB::shutdown()
@@ -165,11 +207,10 @@ void TextDB::shutdown()
 
 bool TextDB::_needs_update() const
 {
-    std::string full_db_path = get_savedir_path(std::string(_db_name) + ".db");
     for (unsigned int i = 0; i < _input_files.size(); i++)
     {
         std::string full_input_path = datafile_path(_input_files[i], true);
-        if (is_newer(full_input_path, full_db_path))
+        if (file_modtime(full_input_path) > timestamp)
             return (true);
     }
     return (false);
@@ -177,25 +218,37 @@ bool TextDB::_needs_update() const
 
 void TextDB::_regenerate_db()
 {
-    std::string db_path = get_savedir_path(_db_name);
+    shutdown();
+#ifdef DEBUG_DIAGNOSTICS
+    printf("Regenerating db: %s\n", _db_name);
+#endif
+
+    std::string db_path = _db_cache_path(_db_name);
     std::string full_db_path = db_path + ".db";
 
     {
         std::string output_dir = get_parent_directory(db_path);
-        if (!check_dir("DB directory", output_dir))
+        if (!check_mkdir("DB directory", &output_dir))
             end(1);
     }
 
     file_lock lock(db_path + ".lk", "wb");
-    unlink( full_db_path.c_str() );
+#ifndef DGL_REWRITE_PROTECT_DB_FILES
+    unlink_u(full_db_path.c_str());
+#endif
 
+    std::string now = make_stringf("%ld", time(0));
+    if (!(_db = dbm_open(db_path.c_str(), O_RDWR | O_CREAT, 0660)))
+        end(1, true, "Unable to open DB: %s", db_path.c_str());
     for (unsigned int i = 0; i < _input_files.size(); i++)
     {
         std::string full_input_path = datafile_path(_input_files[i], true);
-        _store_text_db(full_input_path, db_path);
+        _store_text_db(full_input_path, _db);
     }
+    _add_entry(_db, "TIMESTAMP", now);
 
-    DO_CHMOD_PRIVATE(full_db_path.c_str());
+    dbm_close(_db);
+    _db = 0;
 }
 
 // ----------------------------------------------------------------------
@@ -345,22 +398,20 @@ static void _add_entry(DBM *db, const std::string &k, std::string &v)
         end(1, true, "Error storing %s", k.c_str());
 }
 
-static void _parse_text_db(std::ifstream &inf, DBM *db)
+static void _parse_text_db(LineInput &inf, DBM *db)
 {
-    char buf[1000];
-
     std::string key;
     std::string value;
 
     bool in_entry = false;
     while (!inf.eof())
     {
-        inf.getline(buf, sizeof buf);
+        std::string line = inf.get_line();
 
-        if (*buf == '#')
+        if (!line.empty() && line[0] == '#')
             continue;
 
-        if (!strncmp(buf, "%%%%", 4))
+        if (!line.compare(0, 4, "%%%%"))
         {
             if (!key.empty())
                 _add_entry(db, key, value);
@@ -375,13 +426,12 @@ static void _parse_text_db(std::ifstream &inf, DBM *db)
 
         if (key.empty())
         {
-            key = buf;
+            key = line;
             trim_string(key);
             lowercase(key);
         }
         else
         {
-            std::string line = buf;
             trim_string_right(line);
             value += line + "\n";
         }
@@ -391,21 +441,13 @@ static void _parse_text_db(std::ifstream &inf, DBM *db)
         _add_entry(db, key, value);
 }
 
-static void _store_text_db(const std::string &in, const std::string &out)
+static void _store_text_db(const std::string &in, DBM *db)
 {
-    std::ifstream inf(in.c_str());
-    if (!inf)
+    UTF8FileLineInput inf(in.c_str());
+    if (inf.error())
         end(1, true, "Unable to open input file: %s", in.c_str());
 
-    if (DBM *db = dbm_open(out.c_str(), O_RDWR | O_CREAT, 0660))
-    {
-        _parse_text_db(inf, db);
-        dbm_close(db);
-    }
-    else
-        end(1, true, "Unable to open DB: %s", out.c_str());
-
-    inf.close();
+    _parse_text_db(inf, db);
 }
 
 static std::string _chooseStrByWeight(std::string entry, int fixed_weight = -1)
@@ -452,7 +494,7 @@ static std::string _chooseStrByWeight(std::string entry, int fixed_weight = -1)
     }
 
     if (parts.size() == 0)
-        return("BUG, EMPTY ENTRY");
+        return "BUG, EMPTY ENTRY";
 
     int choice = 0;
     if (fixed_weight != -1)
@@ -462,9 +504,9 @@ static std::string _chooseStrByWeight(std::string entry, int fixed_weight = -1)
 
     for (int i = 0, size = parts.size(); i < size; i++)
         if (choice < weights[i])
-            return(parts[i]);
+            return parts[i];
 
-    return("BUG, NO STRING CHOSEN");
+    return "BUG, NO STRING CHOSEN";
 }
 
 #define MAX_RECURSION_DEPTH 10
@@ -506,22 +548,30 @@ static void _call_recursive_replacement(std::string &str, DBM *database,
                                         int &num_replacements,
                                         int recursion_depth = 0);
 
-std::string getWeightedSpeechString(const std::string &key,
-                                    const std::string &suffix,
-                                    const int weight)
+static std::string _query_weighted_randomised(DBM *database,
+                                              const std::string &key,
+                                              const std::string &suffix = "",
+                                              const int weight = -1)
 {
-    if (!SpeakDB)
+    if (!database)
         return ("");
 
-    std::string result = _getWeightedString(SpeakDB, key, suffix, weight);
+    std::string result = _getWeightedString(database, key, suffix, weight);
     if (result.empty())
         return "";
 
     int num_replacements = 0;
-    _call_recursive_replacement(result, SpeakDB, suffix, num_replacements);
+    _call_recursive_replacement(result, database, suffix, num_replacements);
     return (result);
 }
 
+std::string getWeightedSpeechString(const std::string &key,
+                                    const std::string &suffix,
+                                    const int weight)
+{
+    return _query_weighted_randomised(SpeakDB, key, suffix, weight);
+}
+
 static std::string _getRandomisedStr(DBM *database, const std::string &key,
                                      const std::string &suffix,
                                      int &num_replacements,
@@ -634,6 +684,11 @@ std::string getLongDescription(const std::string &key)
     return _query_database(DescriptionDB.get(), key, true, true);
 }
 
+std::string getWeightedRandomisedDescription(const std::string &key)
+{
+    return _query_weighted_randomised(DescriptionDB, key);
+}
+
 std::vector<std::string> getLongDescKeysByRegex(const std::string &regex,
                                                 db_find_filter filter)
 {
@@ -658,6 +713,17 @@ std::vector<std::string> getLongDescBodiesByRegex(const std::string &regex,
     return database_find_bodies(DescriptionDB.get(), regex, true, filter);
 }
 
+/////////////////////////////////////////////////////////////////////////////
+// GameStart DB specific functions.
+std::string getGameStartDescription(const std::string &key)
+{
+    if (!GameStartDB.get())
+        return ("");
+
+    return _query_database(GameStartDB.get(), key, true, true);
+}
+
+
 /////////////////////////////////////////////////////////////////////////////
 // Shout DB specific functions.
 std::string getShoutString(const std::string &monst,
