@@ -1,8 +1,7 @@
-/*
- *  File:       dbg-asrt.cc
- *  Summary:    Assertions and crashing.
- *  Written by: Linley Henzell and Jesse Jones
- */
+/**
+ * @file
+ * @brief Assertions and crashing.
+**/
 
 #include "AppHdr.h"
 
@@ -22,6 +21,7 @@
 #include "dungeon.h"
 #include "env.h"
 #include "initfile.h"
+#include "itemname.h"
 #include "jobs.h"
 #include "libutil.h"
 #include "mapmark.h"
@@ -34,20 +34,55 @@
 #include "spl-util.h"
 #include "state.h"
 #include "travel.h"
+#include "hiscores.h"
+#include "zotdef.h"
 
-#ifdef DEBUG
-static std::string _assert_msg;
+#if defined(USE_TILE) && (defined(TARGET_OS_WINDOWS) || defined(TARGET_COMPILER_MINGW))
+#define NOCOMM            /* Comm driver APIs and definitions */
+#define NOLOGERROR        /* LogError() and related definitions */
+#define NOPROFILER        /* Profiler APIs */
+#define NOLFILEIO         /* _l* file I/O routines */
+#define NOOPENFILE        /* OpenFile and related definitions */
+#define NORESOURCE        /* Resource management */
+#define NOATOM            /* Atom management */
+#define NOLANGUAGE        /* Character test routines */
+#define NOLSTRING         /* lstr* string management routines */
+#define NODBCS            /* Double-byte character set routines */
+#define NOKEYBOARDINFO    /* Keyboard driver routines */
+#define NOCOLOR           /* COLOR_* color values */
+#define NODRAWTEXT        /* DrawText() and related definitions */
+#define NOSCALABLEFONT    /* Truetype scalable font support */
+#define NOMETAFILE        /* Metafile support */
+#define NOSYSTEMPARAMSINFO /* SystemParametersInfo() and SPI_* definitions */
+#define NODEFERWINDOWPOS  /* DeferWindowPos and related definitions */
+#define NOKEYSTATES       /* MK_* message key state flags */
+#define NOWH              /* SetWindowsHook and related WH_* definitions */
+#define NOCLIPBOARD       /* Clipboard APIs and definitions */
+#define NOICONS           /* IDI_* icon IDs */
+#define NOMDI             /* MDI support */
+#define NOCTLMGR          /* Control management and controls */
+#define NOHELP            /* Help support */
+#define WIN32_LEAN_AND_MEAN /* No cryptography etc */
+#define NONLS             /* All NLS defines and routines */
+#define NOSERVICE         /* All Service Controller routines, SERVICE_ equates, etc. */
+#define NOKANJI           /* Kanji support stuff. */
+#define NOMCX             /* Modem Configuration Extensions */
+#include <windows.h>
+#undef max
+#include <SDL/SDL_syswm.h>
 #endif
 
+static std::string _assert_msg;
+
 static void _dump_compilation_info(FILE* file)
 {
     std::string comp_info = compilation_info();
     if (!comp_info.empty())
     {
-        fprintf(file, "Compilation info:" EOL);
-        fprintf(file, "<<<<<<<<<<<" EOL);
+        fprintf(file, "Compilation info:\n");
+        fprintf(file, "<<<<<<<<<<<\n");
         fprintf(file, "%s", comp_info.c_str());
-        fprintf(file, ">>>>>>>>>>>" EOL EOL);
+        fprintf(file, ">>>>>>>>>>>\n\n");
     }
 }
 
@@ -55,10 +90,10 @@ static void _dump_level_info(FILE* file)
 {
     CrawlHashTable &props = env.properties;
 
-    fprintf(file, "Place info:" EOL);
+    fprintf(file, "Place info:\n");
 
     fprintf(file, "absdepth0 = %d, branch = %d, level_type = %d, "
-                  "type_name = %s" EOL EOL,
+                  "type_name = %s\n\n",
             you.absdepth0, (int) you.where_are_you, (int) you.level_type,
             you.level_type_name.c_str());
 
@@ -70,9 +105,9 @@ static void _dump_level_info(FILE* file)
     else
         orig_place = props[LEVEL_ID_KEY].get_string();
 
-    fprintf(file, "Level id: %s" EOL, place.c_str());
+    fprintf(file, "Level id: %s\n", place.c_str());
     if (place != orig_place)
-        fprintf(file, "Level id when level was generated: %s" EOL,
+        fprintf(file, "Level id when level was generated: %s\n",
                 orig_place.c_str());
 
     debug_dump_levgen();
@@ -82,56 +117,39 @@ static void _dump_player(FILE *file)
 {
     // Only dump player info during arena mode if the player is likely
     // the cause of the crash.
-    if ((crawl_state.arena || crawl_state.arena_suspended)
+    if ((crawl_state.game_is_arena() || crawl_state.arena_suspended)
         && !in_bounds(you.pos()) && you.hp > 0 && you.hp_max > 0
-        && you.strength > 0 && you.intel > 0 && you.dex > 0)
+        && you.strength() > 0 && you.intel() > 0 && you.dex() > 0)
     {
         // Arena mode can change behavior of the rest of the code and/or lead
         // to asserts.
-        crawl_state.arena          = false;
+        crawl_state.type            = GAME_TYPE_NORMAL;
         crawl_state.arena_suspended = false;
         return;
     }
 
     // Arena mode can change behavior of the rest of the code and/or lead
     // to asserts.
-    crawl_state.arena          = false;
     crawl_state.arena_suspended = false;
 
-    fprintf(file, "Player:" EOL);
-    fprintf(file, "{{{{{{{{{{{" EOL);
+    fprintf(file, "Player:\n");
+    fprintf(file, "{{{{{{{{{{{\n");
 
-    bool name_overrun = true;
-    for (int i = 0; i < 30; ++i)
-    {
-        if (you.class_name[i] == '\0')
-        {
-            name_overrun = false;
-            break;
-        }
-    }
+    fprintf(file, "Name:       [%s]\n", you.your_name.c_str());
+    fprintf(file, "Species:    %s\n", species_name(you.species).c_str());
+    fprintf(file, "Job:        %s\n\n", get_job_name(you.char_class));
+    fprintf(file, "class_name: %s\n\n", you.class_name.c_str());
 
-    if (name_overrun)
-    {
-        fprintf(file, "job_name runs past end of buffer." EOL);
-        you.class_name[29] = '\0';
-    }
-
-    fprintf(file, "Name:       [%s]" EOL, you.your_name.c_str());
-    fprintf(file, "Species:    %s" EOL, species_name(you.species, 27).c_str());
-    fprintf(file, "Job:        %s" EOL EOL, get_job_name(you.char_class));
-    fprintf(file, "class_name: %s" EOL EOL, you.class_name);
-
-    fprintf(file, "HP: %d/%d; base: %d/%d" EOL, you.hp, you.hp_max,
+    fprintf(file, "HP: %d/%d; base: %d/%d\n", you.hp, you.hp_max,
             you.base_hp, you.base_hp2);
-    fprintf(file, "MP: %d/%d; base: %d/%d" EOL,
+    fprintf(file, "MP: %d/%d; base: %d/%d\n",
             you.magic_points, you.max_magic_points,
             you.base_magic_points, you.base_magic_points2);
-    fprintf(file, "Stats: %d (%d) %d (%d) %d (%d)" EOL,
-            you.strength, you.max_strength, you.intel, you.max_intel,
-            you.dex, you.max_dex);
+    fprintf(file, "Stats: %d (%d) %d (%d) %d (%d)\n",
+            you.strength(), you.max_strength(), you.intel(), you.max_intel(),
+            you.dex(), you.max_dex());
     fprintf(file, "Position: %s, god:%s (%d), turn_is_over: %d, "
-                  "banished: %d" EOL,
+                  "banished: %d\n",
             debug_coord_str(you.pos()).c_str(),
             god_name(you.religion).c_str(), (int) you.religion,
             (int) you.turn_is_over, (int) you.banished);
@@ -139,25 +157,25 @@ static void _dump_player(FILE *file)
     if (in_bounds(you.pos()))
     {
         const dungeon_feature_type feat = grd(you.pos());
-        fprintf(file, "Standing on/in/over feature: %s" EOL,
+        fprintf(file, "Standing on/in/over feature: %s\n",
                 raw_feature_description(feat, NUM_TRAPS, true).c_str());
     }
-    fprintf(file, EOL);
+    fprintf(file, "\n");
 
     if (you.running.runmode != RMODE_NOT_RUNNING)
     {
-        fprintf(file, "Runrest:" EOL);
-        fprintf(file, "    mode: %d" EOL, you.running.runmode);
-        fprintf(file, "      mp: %d" EOL, you.running.mp);
-        fprintf(file, "      hp: %d" EOL, you.running.hp);
-        fprintf(file, "     pos: %s" EOL EOL,
+        fprintf(file, "Runrest:\n");
+        fprintf(file, "    mode: %d\n", you.running.runmode);
+        fprintf(file, "      mp: %d\n", you.running.mp);
+        fprintf(file, "      hp: %d\n", you.running.hp);
+        fprintf(file, "     pos: %s\n\n",
                 debug_coord_str(you.running.pos).c_str());
     }
 
     if (you.delay_queue.size() > 0)
     {
-        fprintf(file, "Delayed (%lu):" EOL,
-                (unsigned long) you.delay_queue.size());
+        fprintf(file, "Delayed (%u):\n",
+                (unsigned int)you.delay_queue.size());
         for (unsigned int i = 0; i < you.delay_queue.size(); ++i)
         {
             const delay_queue_item &item = you.delay_queue[i];
@@ -168,16 +186,16 @@ static void _dump_player(FILE *file)
             {
                 fprintf(file, " <invalid>");
             }
-            fprintf(file, EOL);
-            fprintf(file, "    duration: %d" EOL, item.duration);
-            fprintf(file, "    parm1:    %d" EOL, item.parm1);
-            fprintf(file, "    parm2:    %d" EOL, item.parm2);
-            fprintf(file, "    started:  %d" EOL EOL, (int) item.started);
+            fprintf(file, "\n");
+            fprintf(file, "    duration: %d\n", item.duration);
+            fprintf(file, "    parm1:    %d\n", item.parm1);
+            fprintf(file, "    parm2:    %d\n", item.parm2);
+            fprintf(file, "    started:  %d\n\n", (int) item.started);
         }
-        fprintf(file, EOL);
+        fprintf(file, "\n");
     }
 
-    fprintf(file, "Spell bugs:" EOL);
+    fprintf(file, "Spell bugs:\n");
     for (size_t i = 0; i < you.spells.size(); ++i)
     {
         const spell_type spell = you.spells[i];
@@ -187,7 +205,7 @@ static void _dump_player(FILE *file)
 
         if (!is_valid_spell(spell))
         {
-            fprintf(file, "    spell slot #%d: invalid spell #%d" EOL,
+            fprintf(file, "    spell slot #%d: invalid spell #%d\n",
                     (int)i, (int)spell);
             continue;
         }
@@ -195,53 +213,53 @@ static void _dump_player(FILE *file)
         const unsigned int flags = get_spell_flags(spell);
 
         if (flags & SPFLAG_MONSTER)
-            fprintf(file, "    spell slot #%d: monster only spell %s" EOL,
+            fprintf(file, "    spell slot #%d: monster only spell %s\n",
                     (int)i, spell_title(spell));
         else if (flags & SPFLAG_TESTING)
-            fprintf(file, "    spell slot #%d: testing spell %s" EOL,
+            fprintf(file, "    spell slot #%d: testing spell %s\n",
                     (int)i, spell_title(spell));
         else if (count_bits(get_spell_disciplines(spell)) == 0)
-            fprintf(file, "    spell slot #%d: school-less spell %s" EOL,
+            fprintf(file, "    spell slot #%d: school-less spell %s\n",
                     (int)i, spell_title(spell));
     }
-    fprintf(file, EOL);
+    fprintf(file, "\n");
 
-    fprintf(file, "Durations:" EOL);
+    fprintf(file, "Durations:\n");
     for (int i = 0; i < NUM_DURATIONS; ++i)
         if (you.duration[i] != 0)
-            fprintf(file, "    #%d: %d" EOL, i, you.duration[i]);
+            fprintf(file, "    #%d: %d\n", i, you.duration[i]);
 
-    fprintf(file, EOL);
+    fprintf(file, "\n");
 
-    fprintf(file, "Attributes:" EOL);
+    fprintf(file, "Attributes:\n");
     for (int i = 0; i < NUM_ATTRIBUTES; ++i)
         if (you.attribute[i] != 0)
-            fprintf(file, "    #%d: %lu" EOL, i, you.attribute[i]);
+            fprintf(file, "    #%d: %d\n", i, you.attribute[i]);
 
-    fprintf(file, EOL);
+    fprintf(file, "\n");
 
-    fprintf(file, "Mutations:" EOL);
+    fprintf(file, "Mutations:\n");
     for (int i = 0; i < NUM_MUTATIONS; ++i)
         if (you.mutation[i] > 0)
-            fprintf(file, "    #%d: %d" EOL, i, you.mutation[i]);
+            fprintf(file, "    #%d: %d\n", i, you.mutation[i]);
 
-    fprintf(file, EOL);
+    fprintf(file, "\n");
 
-    fprintf(file, "Demon mutations:" EOL);
+    fprintf(file, "Demon mutations:\n");
     for (int i = 0; i < NUM_MUTATIONS; ++i)
-        if (you.demon_pow[i] > 0)
-            fprintf(file, "    #%d: %d" EOL, i, you.demon_pow[i]);
+        if (you.innate_mutations[i] > 0)
+            fprintf(file, "    #%d: %d\n", i, you.innate_mutations[i]);
 
-    fprintf(file, EOL);
+    fprintf(file, "\n");
 
-    fprintf(file, "Inventory bugs:" EOL);
+    fprintf(file, "Inventory bugs:\n");
     for (int i = 0; i < ENDOFPACK; ++i)
     {
         item_def &item(you.inv[i]);
 
         if (item.base_type == OBJ_UNASSIGNED && item.quantity != 0)
         {
-            fprintf(file, "    slot #%d: unassigned item has quant %d" EOL,
+            fprintf(file, "    slot #%d: unassigned item has quant %d\n",
                     i, item.quantity);
             continue;
         }
@@ -251,41 +269,41 @@ static void _dump_player(FILE *file)
             item.quantity = 1;
 
             fprintf(file, "    slot #%d: otherwise valid item '%s' has "
-                          "invalid quantity %d" EOL,
+                          "invalid quantity %d\n",
                     i, item.name(DESC_PLAIN, false, true).c_str(),
                     orig_quant);
             item.quantity = orig_quant;
             continue;
         }
-        else if (!item.is_valid())
+        else if (!item.defined())
             continue;
 
         const std::string name = item.name(DESC_PLAIN, false, true);
 
         if (item.link != i)
         {
-            fprintf(file, "    slot #%d: item '%s' has invalid link %d" EOL,
+            fprintf(file, "    slot #%d: item '%s' has invalid link %d\n",
                     i, name.c_str(), item.link);
         }
 
         if (item.slot < 0 || item.slot > 127)
         {
-            fprintf(file, "    slot #%d: item '%s' has invalid slot %d" EOL,
+            fprintf(file, "    slot #%d: item '%s' has invalid slot %d\n",
                     i, name.c_str(), item.slot);
         }
 
         if (!item.pos.equals(-1, -1))
         {
-            fprintf(file, "    slot #%d: item '%s' has invalid pos %s" EOL,
+            fprintf(file, "    slot #%d: item '%s' has invalid pos %s\n",
                     i, name.c_str(), debug_coord_str(item.pos).c_str());
         }
     }
-    fprintf(file, EOL);
+    fprintf(file, "\n");
 
-    fprintf(file, "Equipment:" EOL);
+    fprintf(file, "Equipment:\n");
     for (int i = 0; i < NUM_EQUIP; ++i)
     {
-        char eq = you.equip[i];
+        int8_t eq = you.equip[i];
 
         if (eq == -1)
             continue;
@@ -293,13 +311,29 @@ static void _dump_player(FILE *file)
         fprintf(file, "    eq slot #%d, inv slot #%d", i, (int) eq);
         if (eq < 0 || eq >= ENDOFPACK)
         {
-            fprintf(file, " <invalid>" EOL);
+            fprintf(file, " <invalid>\n");
             continue;
         }
-        fprintf(file, ": %s" EOL,
-                you.inv[eq].name(DESC_PLAIN, false, true).c_str());
+        const bool unknown = !item_type_known(you.inv[eq]);
+        const bool melded  = you.melded[i];
+        std::string suffix = "";
+        if (unknown || melded)
+        {
+            suffix = " (";
+            if (unknown)
+            {
+                suffix += "unknown";
+                if (melded)
+                    suffix += ", ";
+            }
+            if (melded)
+                suffix += "melded";
+            suffix += ")";
+        }
+        fprintf(file, ": %s%s\n",
+                you.inv[eq].name(DESC_PLAIN, false, true).c_str(), suffix.c_str());
     }
-    fprintf(file, EOL);
+    fprintf(file, "\n");
 
     if (in_bounds(you.pos()) && monster_at(you.pos()))
     {
@@ -307,17 +341,17 @@ static void _dump_player(FILE *file)
         const unsigned short midx = mgrd(you.pos());
 
         if (invalid_monster_index(midx))
-            fprintf(file, "invalid monster index %d" EOL, (int) midx);
+            fprintf(file, "invalid monster index %d\n", (int) midx);
         else
         {
-            const monsters *mon = &menv[midx];
-            fprintf(file, "%s:" EOL, debug_mon_str(mon).c_str());
+            const monster* mon = &menv[midx];
+            fprintf(file, "%s:\n", debug_mon_str(mon).c_str());
             debug_dump_mon(mon, true);
         }
-        fprintf(file, EOL);
+        fprintf(file, "\n");
     }
 
-    fprintf(file, "}}}}}}}}}}}" EOL EOL);
+    fprintf(file, "}}}}}}}}}}}\n\n");
 }
 
 static void _debug_marker_scan()
@@ -465,32 +499,52 @@ static void _debug_dump_lua_persist(FILE* file)
 
 static void _dump_ver_stuff(FILE* file)
 {
-    fprintf(file, "Version: %s %s" EOL, CRAWL, Version::Long().c_str());
+    fprintf(file, "Version: %s %s\n", CRAWL, Version::Long().c_str());
 #if defined(UNIX)
     fprintf(file, "Platform: unix");
 #   if defined(TARGET_OS_MACOSX)
     fprintf(file, " (OS X)");
 #   endif
-    fprintf(file, EOL);
+    fprintf(file, "\n");
 #elif defined(TARGET_OS_WINDOWS)
-    fprintf(file, "Platform: Windows" EOL);
+    fprintf(file, "Platform: Windows\n");
 #elif defined(TARGET_OS_DOS)
-    fprintf(file, "Platform: DOS" EOL);
+    fprintf(file, "Platform: DOS\n");
 #endif // UNIX
 
-#if TARGET_CPU_BITS == 64
-    fprintf(file, "Bits: 64" EOL);
-#else
-    fprintf(file, "Bits: 32" EOL);
-#endif
+    fprintf(file, "Bits: %d\n", (int)sizeof(void*)*8);
+    fprintf(file, "Game mode: %s\n",
+            gametype_to_str(crawl_state.type).c_str());
 
 #ifdef USE_TILE
-    fprintf(file, "Tiles: yes" EOL EOL);
+    fprintf(file, "Tiles: yes\n\n");
 #else
-    fprintf(file, "Tiles: no" EOL EOL);
+    fprintf(file, "Tiles: no\n\n");
 #endif
 }
 
+static void _dump_command_line(FILE *file)
+{
+    fprintf(file, "Command line:");
+    for (int i = 0, size = crawl_state.command_line_arguments.size();
+         i < size; ++i)
+    {
+        fprintf(file, " %s", crawl_state.command_line_arguments[i].c_str());
+    }
+    if (crawl_state.command_line_arguments.empty())
+        fprintf(file, " (unknown)");
+    fprintf(file, "\n\n");
+}
+
+// Dump any game options that could affect stability.
+static void _dump_options(FILE *file)
+{
+    fprintf(file, "RC options:\n");
+    fprintf(file, "restart_after_game = %s\n",
+            Options.restart_after_game? "true" : "false");
+    fprintf(file, "\n\n");
+}
+
 // Defined in stuff.cc.  Not a part of crawl_state, since that's a
 // global C++ instance which is free'd by exit() hooks when exit()
 // is called, and we don't want to reference free'd memory.
@@ -505,7 +559,7 @@ void do_crash_dump()
         // free'd and invalid, plus their content likely wouldn't help
         // tracking it down anyway.  Thus, just do the bare bones
         // info to stderr and quit.
-        fprintf(stderr, "Crashed while calling exit()!!!!" EOL);
+        fprintf(stderr, "Crashed while calling exit()!!!!\n");
 
         _dump_ver_stuff(stderr);
 
@@ -524,16 +578,18 @@ void do_crash_dump()
 
     char name[180];
 
-    sprintf(name, "%scrash-%s-%s.txt", dir.c_str(),
-            you.your_name.c_str(), make_file_time(time(NULL)).c_str());
+    // Want same time for file name and crash milestone.
+    const time_t t = time(NULL);
+    snprintf(name, sizeof(name), "%scrash-%s-%s.txt", dir.c_str(),
+            you.your_name.c_str(), make_file_time(t).c_str());
 
-    fprintf(stderr, EOL "Writing crash info to %s" EOL, name);
+    fprintf(stderr, "\nWriting crash info to %s\n", name);
     errno = 0;
     FILE* file = crawl_state.test ? stderr : freopen(name, "w+", stderr);
 
     if (file == NULL || errno != 0)
     {
-        fprintf(stdout, EOL "Unable to open file '%s' for writing: %s" EOL,
+        fprintf(stdout, "\nUnable to open file '%s' for writing: %s\n",
                 name, strerror(errno));
         file = stdout;
     }
@@ -544,20 +600,22 @@ void do_crash_dump()
 
     set_msg_dump_file(file);
 
-#ifdef DEBUG
     if (!_assert_msg.empty())
-        fprintf(file, "%s" EOL EOL, _assert_msg.c_str());
-#endif
+        fprintf(file, "%s\n\n", _assert_msg.c_str());
 
     _dump_ver_stuff(file);
 
+    _dump_command_line(file);
+
+    _dump_options(file);
+
     // First get the immediate cause of the crash and the stack trace,
     // since that's most important and later attempts to get more information
     // might themselves cause crashes.
     dump_crash_info(file);
     write_stack_trace(file, 0);
 
-    fprintf(file, EOL);
+    fprintf(file, "\n");
 
     // Next information on how the binary was compiled
     _dump_compilation_info(file);
@@ -568,20 +626,20 @@ void do_crash_dump()
 
     // Dumping information on marker inconsistancy is unlikely to crash,
     // as is dumping the descriptions of non-Lua markers.
-    fprintf(file, "Markers:" EOL);
-    fprintf(file, "<<<<<<<<<<<<<<<<<<<<<<" EOL);
+    fprintf(file, "Markers:\n");
+    fprintf(file, "<<<<<<<<<<<<<<<<<<<<<<\n");
     _debug_marker_scan();
     _debug_dump_markers();
-    fprintf(file, ">>>>>>>>>>>>>>>>>>>>>>" EOL);
+    fprintf(file, ">>>>>>>>>>>>>>>>>>>>>>\n");
 
     // Dumping current messages is unlikely to crash.
     if (file != stdout)
     {
-        fprintf(file, EOL "Messages:" EOL);
-        fprintf(file, "<<<<<<<<<<<<<<<<<<<<<<" EOL);
+        fprintf(file, "\nMessages:\n");
+        fprintf(file, "<<<<<<<<<<<<<<<<<<<<<<\n");
         std::string messages = get_last_messages(NUM_STORED_MESSAGES);
         fprintf(file, "%s", messages.c_str());
-        fprintf(file, ">>>>>>>>>>>>>>>>>>>>>>" EOL);
+        fprintf(file, ">>>>>>>>>>>>>>>>>>>>>>\n");
     }
 
     // Dumping the player state and crawl state is next least likely to cause
@@ -589,37 +647,42 @@ void do_crash_dump()
     crawl_state.dump();
     _dump_player(file);
 
+    if (crawl_state.game_is_zotdef())
+        fprintf(file, "ZotDef wave data: %s\n", zotdef_debug_wave_desc().c_str());
+
     // Next item and monster scans.  Any messages will be sent straight to
     // the file because of set_msg_dump_file()
-#if DEBUG_ITEM_SCAN
+#ifdef DEBUG_ITEM_SCAN
     debug_item_scan();
 #endif
-#if DEBUG_MONS_SCAN
+#ifdef DEBUG_MONS_SCAN
     debug_mons_scan();
 #endif
 
     // If anything has screwed up the Lua runtime stacks then trying to
     // print those stacks will likely crash, so do this after the others.
-    fprintf(file, "clua stack:" EOL);
+    fprintf(file, "clua stack:\n");
     clua.print_stack();
 
-    fprintf(file, "dlua stack:" EOL);
+    fprintf(file, "dlua stack:\n");
     dlua.print_stack();
 
     // Lastly try to dump the Lua persistent data and the contents of the Lua
     // markers, since actually running Lua code has the greatest chance of
     // crashing.
-    fprintf(file, "Lua persistent data:" EOL);
-    fprintf(file, "<<<<<<<<<<<<<<<<<<<<<<" EOL);
+    fprintf(file, "Lua persistent data:\n");
+    fprintf(file, "<<<<<<<<<<<<<<<<<<<<<<\n");
     _debug_dump_lua_persist(file);
-    fprintf(file, ">>>>>>>>>>>>>>>>>>>>>>" EOL EOL);
-    fprintf(file, "Lua marker contents:" EOL);
-    fprintf(file, "<<<<<<<<<<<<<<<<<<<<<<" EOL);
+    fprintf(file, ">>>>>>>>>>>>>>>>>>>>>>\n\n");
+    fprintf(file, "Lua marker contents:\n");
+    fprintf(file, "<<<<<<<<<<<<<<<<<<<<<<\n");
     _debug_dump_lua_markers(file);
-    fprintf(file, ">>>>>>>>>>>>>>>>>>>>>>" EOL);
+    fprintf(file, ">>>>>>>>>>>>>>>>>>>>>>\n");
 
     set_msg_dump_file(NULL);
 
+    mark_milestone("crash", _assert_msg, false, t);
+
     if (file != stderr)
         fclose(file);
 }
@@ -629,17 +692,34 @@ void do_crash_dump()
 
 // Assertions and such
 
-#ifdef DEBUG
 //---------------------------------------------------------------
 // BreakStrToDebugger
 //---------------------------------------------------------------
-static void _BreakStrToDebugger(const char *mesg)
+static NORETURN void _BreakStrToDebugger(const char *mesg, bool assert)
 {
-#if defined(TARGET_OS_MACOSX) || defined(TARGET_COMPILER_MINGW)
+#if defined(USE_TILE) && (defined(TARGET_COMPILER_MINGW) || defined(TARGET_OS_WINDOWS))
+    SDL_SysWMinfo SysInfo;
+    SDL_VERSION(&SysInfo.version);
+    if (SDL_GetWMInfo(&SysInfo) > 0)
+    {
+        MessageBoxW(SysInfo.window, utf8_to_16(mesg).c_str(),
+                   assert ? L"Assertion failed!" : L"Error",
+                   MB_OK|MB_ICONERROR);
+    }
+    // Print the message to STDERR in addition to the above message box,
+    // so it's in the message history if we call Crawl from a shell.
+    fprintf(stderr, "%s", mesg);
+
+    int* p = NULL;
+    *p = 0;
+    abort();
+
+#elif defined(TARGET_OS_MACOSX) || defined(TARGET_COMPILER_MINGW)
     fprintf(stderr, "%s", mesg);
 // raise(SIGINT);               // this is what DebugStr() does on OS X according to Tech Note 2030
     int* p = NULL;              // but this gives us a stack crawl...
     *p = 0;
+    abort();                    // just to be sure
 
 #else
     fprintf(stderr, "%s\n", mesg);
@@ -647,44 +727,65 @@ static void _BreakStrToDebugger(const char *mesg)
 #endif
 }
 
+#ifdef ASSERTS
 //---------------------------------------------------------------
 //
 // AssertFailed
 //
 //---------------------------------------------------------------
-void AssertFailed(const char *expr, const char *file, int line)
+NORETURN void AssertFailed(const char *expr, const char *file, int line, bool save_game)
 {
     char mesg[512];
 
+    if (save_game)
+        crawl_state.game_wants_emergency_save = true;
+
     const char *fileName = file + strlen(file); // strip off path
 
     while (fileName > file && fileName[-1] != '\\')
         --fileName;
 
-    sprintf(mesg, "ASSERT(%s) in '%s' at line %d failed.", expr, fileName,
-            line);
+    snprintf(mesg, sizeof(mesg), "ASSERT(%s) in '%s' at line %d failed.",
+             expr, fileName, line);
 
     _assert_msg = mesg;
 
-    _BreakStrToDebugger(mesg);
+    _BreakStrToDebugger(mesg, true);
 }
+#endif
 
-//---------------------------------------------------------------
-//
-// DEBUGSTR
-//
-//---------------------------------------------------------------
-void DEBUGSTR(const char *format, ...)
+#undef die
+NORETURN void die(const char *file, int line, const char *format, ...)
 {
-    char mesg[2048];
+    char tmp[2048], mesg[2048];
 
     va_list args;
 
     va_start(args, format);
-    vsprintf(mesg, format, args);
+    vsnprintf(tmp, sizeof(tmp), format, args);
     va_end(args);
 
-    _BreakStrToDebugger(mesg);
+    snprintf(mesg, sizeof(mesg), "ERROR in '%s' at line %d: %s",
+             file, line, tmp);
+
+    _assert_msg = mesg;
+
+    _BreakStrToDebugger(mesg, false);
 }
 
-#endif
+NORETURN void die_noline(const char *format, ...)
+{
+    char tmp[2048], mesg[2048];
+
+    va_list args;
+
+    va_start(args, format);
+    vsnprintf(tmp, sizeof(tmp), format, args);
+    va_end(args);
+
+    snprintf(mesg, sizeof(mesg), "ERROR: %s", tmp);
+
+    _assert_msg = mesg;
+
+    _BreakStrToDebugger(mesg, false);
+}
