@@ -1,8 +1,7 @@
-/*
- *  File:       dgn-overview.h
- *  Summary:    "Overview" functionality
- *  Written by: Linley Henzell
- */
+/**
+ * @file
+ * @brief "Overview" functionality
+**/
 
 
 #ifndef OVERMAP_H
@@ -11,6 +10,8 @@
 #include "stash.h"
 #include <vector>
 
+void overview_clear();
+
 void seen_notable_thing(dungeon_feature_type which_thing, const coord_def& pos);
 bool move_notable_thing(const coord_def& orig, const coord_def& dest);
 bool overview_knows_portal(dungeon_feature_type portal);
@@ -31,6 +32,10 @@ void clear_level_exclusion_annotation(level_id li = level_id::current());
 std::string get_level_annotation(level_id li = level_id::current(),
                                  bool skip_excl = false);
 
+std::string get_coloured_level_annotation(int col,
+                                 level_id li = level_id::current(),
+                                 bool skip_excl = false);
+
 bool level_annotation_has(std::string str,
                           level_id li = level_id::current());
 
