@@ -9,9 +9,9 @@
 #include "dgn-shoals.h"
 #include "dgn-height.h"
 #include "env.h"
-#include "flood_find.h"
 #include "fprop.h"
 #include "items.h"
+#include "itemprop.h"
 #include "libutil.h"
 #include "mapmark.h"
 #include "maps.h"
@@ -22,6 +22,7 @@
 #include "mon-util.h"
 #include "random.h"
 #include "terrain.h"
+#include "traps.h"
 #include "view.h"
 
 #include <algorithm>
@@ -65,19 +66,19 @@ enum shoals_height_thresholds
     SHT_STONE = 400,
     SHT_ROCK  = 135,
     SHT_FLOOR = 0,
-    SHT_SHALLOW_WATER = -14
+    SHT_SHALLOW_WATER = -30,
 };
 
 enum tide_direction
 {
     TIDE_RISING,
-    TIDE_FALLING
+    TIDE_FALLING,
 };
 
 static tide_direction _shoals_tide_direction;
-static monsters *tide_caller = NULL;
+static monster* tide_caller = NULL;
 static coord_def tide_caller_pos;
-static long tide_called_turns = 0L;
+static int tide_called_turns = 0;
 static int tide_called_peak = 0;
 static int shoals_plant_quota = 0;
 
@@ -100,6 +101,8 @@ static int _shoals_feature_height(dungeon_feature_type feat)
 {
     switch (feat)
     {
+    case DNGN_STONE_WALL:
+        return SHT_STONE;
     case DNGN_FLOOR:
         return SHT_FLOOR;
     case DNGN_SHALLOW_WATER:
@@ -107,6 +110,20 @@ static int _shoals_feature_height(dungeon_feature_type feat)
     case DNGN_DEEP_WATER:
         return SHT_SHALLOW_WATER - 1;
     default:
+        return feat_is_solid(feat)? SHT_ROCK : SHT_FLOOR;
+    }
+}
+
+static int _shoals_feature_sequence_number(dungeon_feature_type feat)
+{
+    switch (feat)
+    {
+    case DNGN_SHALLOW_WATER:
+        return 1;
+    case DNGN_DEEP_WATER:
+        return 2;
+    default:
+    case DNGN_FLOOR:
         return 0;
     }
 }
@@ -147,11 +164,11 @@ static dgn_island_plan _shoals_island_plan()
 
     plan.n_island_centre_delta_points = int_range(50, 60);
     plan.island_centre_radius_range = int_range(3, 10);
-    plan.island_centre_point_height_increment = int_range(40, 60);
+    plan.island_centre_point_height_increment = int_range(80, 110);
 
     plan.n_island_aux_delta_points = int_range(25, 45);
     plan.island_aux_radius_range = int_range(2, 7);
-    plan.island_aux_point_height_increment = int_range(25, 35);
+    plan.island_aux_point_height_increment = int_range(50, 65);
 
     return (plan);
 }
@@ -215,7 +232,9 @@ static std::vector<coord_def> _shoals_water_depth_change_points()
         coord_def c(*ri);
         if (grd(c) == DNGN_DEEP_WATER
             && dgn_has_adjacent_feat(c, DNGN_SHALLOW_WATER))
+        {
             points.push_back(c);
+        }
     }
     return points;
 }
@@ -274,9 +293,6 @@ static void _shoals_furniture(int margin)
 {
     if (at_branch_bottom())
     {
-        unwind_var<dungeon_feature_set> vault_exc(dgn_Vault_Excavatable_Feats);
-        dgn_Vault_Excavatable_Feats.insert(DNGN_STONE_WALL);
-
         const coord_def p = _pick_shoals_island();
         const char *SHOAL_RUNE_HUT = "shoal_rune_hut";
         const map_def *vault = random_map_for_tag(SHOAL_RUNE_HUT);
@@ -293,15 +309,15 @@ static void _shoals_furniture(int margin)
                 vault = random_map_for_tag("shoal_hut");
             while (!vault && --tries > 0);
             if (vault)
-                dgn_place_map(vault, false, false, _pick_shoals_island(), 0);
+                dgn_place_map(vault, false, false, _pick_shoals_island());
         }
 
         // Fixup pass to connect vaults.
-        for (int i = 0, size = Level_Vaults.size(); i < size; ++i)
+        for (int i = 0, size = env.level_vaults.size(); i < size; ++i)
         {
-            vault_placement &vp(Level_Vaults[i]);
+            vault_placement &vp(*env.level_vaults[i]);
             if (vp.map.has_tag(SHOAL_RUNE_HUT))
-                dgn_dig_vault_loose(vp);
+                vp.connect();
         }
     }
 
@@ -351,7 +367,7 @@ static int _shoals_contiguous_feature_flood(
             {
                 const coord_def adj(*ai);
                 if (in_bounds(adj) && !rmap(adj) && grd(adj) == feat
-                    && unforbidden(adj, MMT_VAULT))
+                    && !map_masked(adj, MMT_VAULT))
                 {
                     rmap(adj) = nregion;
                     visit.push_back(adj);
@@ -439,7 +455,7 @@ _shoals_point_feat_cluster(dungeon_feature_type feat,
     {
         coord_def c(*ri);
         if (!region_map(c) && grd(c) == feat
-            && unforbidden(c, MMT_VAULT))
+            && !map_masked(c, MMT_VAULT))
         {
             const int featcount =
                 _shoals_contiguous_feature_flood(region_map,
@@ -493,7 +509,8 @@ static void _shoals_make_plant_near(coord_def c, int radius,
         const coord_def plant_place(
             dgn_random_point_from(c, random2(1 + radius), _shoals_margin));
         if (!plant_place.origin()
-            && !monster_at(plant_place))
+            && !monster_at(plant_place)
+            && !map_masked(plant_place, MMT_VAULT))
         {
             const dungeon_feature_type feat(grd(plant_place));
             if (_shoals_plantworthy_feat(feat)
@@ -603,7 +620,7 @@ static std::vector<coord_def> _shoals_windshadows(grid_bool &windy)
     // To avoid plants cropping up inside vaults, mark everything inside
     // vaults as "windy".
     for (rectangle_iterator ri(1); ri; ++ri)
-        if (!unforbidden(*ri, MMT_VAULT))
+        if (map_masked(*ri, MMT_VAULT))
             windy(*ri) = true;
 
     // Now we know the places in the wind shadow:
@@ -614,7 +631,9 @@ static std::vector<coord_def> _shoals_windshadows(grid_bool &windy)
         if (!windy(p) && grd(p) == DNGN_FLOOR
             && (dgn_has_adjacent_feat(p, DNGN_STONE_WALL)
                 || dgn_has_adjacent_feat(p, DNGN_ROCK_WALL)))
+        {
             wind_shadows.push_back(p);
+        }
     }
     return wind_shadows;
 }
@@ -632,7 +651,7 @@ static void _shoals_generate_wind_sheltered_plants(
     _shoals_plant_supercluster(spot, DNGN_FLOOR, &windy);
 }
 
-static void _shoals_generate_flora()
+void dgn_shoals_generate_flora()
 {
     // Water clusters are groups of plants clustered near the water.
     // Wind clusters are groups of plants clustered in wind shadow --
@@ -670,11 +689,12 @@ static void _shoals_generate_flora()
 
 void dgn_build_shoals_level(int level_number)
 {
-    dgn_Build_Method += make_stringf(" shoals+ [%d]", level_number);
-    dgn_Layout_Type   = "shoals";
+    env.level_build_method += make_stringf(" shoals+ [%d]", level_number);
+    env.level_layout_types.insert("shoals");
 
     const int shoals_depth = level_id::current().depth - 1;
-    dgn_replace_area(0, 0, GXM-1, GYM-1, DNGN_ROCK_WALL, DNGN_OPEN_SEA);
+    if (you.level_type != LEVEL_LABYRINTH && you.level_type != LEVEL_PORTAL_VAULT)
+        dgn_replace_area(0, 0, GXM-1, GYM-1, DNGN_ROCK_WALL, DNGN_OPEN_SEA);
     _shoals_init_heights();
     _shoals_init_islands(shoals_depth);
     _shoals_cliffs();
@@ -684,9 +704,6 @@ void dgn_build_shoals_level(int level_number)
     _shoals_deepen_edges();
     _shoals_smooth_water();
     _shoals_furniture(_shoals_margin);
-
-    // This has to happen after placing shoal rune vault!
-    _shoals_generate_flora();
 }
 
 // Search the map for vaults and set the terrain heights for features
@@ -699,7 +716,7 @@ void shoals_postprocess_level()
     for (rectangle_iterator ri(1); ri; ++ri)
     {
         const coord_def c(*ri);
-        if (!(dgn_Map_Mask(c) & MMT_VAULT))
+        if (!(env.level_map_mask(c) & MMT_VAULT))
             continue;
 
         // Don't mess with tide immune squares at all.
@@ -707,7 +724,7 @@ void shoals_postprocess_level()
             continue;
 
         const dungeon_feature_type feat(grd(c));
-        if (!_shoals_tide_susceptible_feat(feat))
+        if (!_shoals_tide_susceptible_feat(feat) && !feat_is_solid(feat))
             continue;
 
         const dungeon_feature_type expected_feat(_shoals_feature_at(c));
@@ -719,7 +736,7 @@ void shoals_postprocess_level()
 
     // Apply tide now, since the tide is likely to be nonzero unless
     // this is Shoals:1
-    shoals_apply_tides(0, true);
+    shoals_apply_tides(0, true, false);
 }
 
 static void _shoals_run_tide(int &tide, int &acc)
@@ -768,6 +785,26 @@ static coord_def _shoals_escape_place_from(coord_def bad_place,
     return chosen;
 }
 
+static void _clear_net_trapping_status(coord_def c)
+{
+    actor *victim = actor_at(c);
+    if (!victim)
+        return;
+
+    if (victim->atype() == ACT_MONSTER)
+    {
+        monster* mvictim = victim->as_monster();
+        if (you.can_see(mvictim))
+            mprf("The net is swept off %s.", mvictim->name(DESC_NOCAP_THE).c_str());
+        mons_clear_trapping_net(mvictim);
+    }
+    else
+    {
+        mpr("The tide washes the net away!");
+        clear_trapping_net();
+    }
+}
+
 static bool _shoals_tide_sweep_items_clear(coord_def c)
 {
     int link = igrd(c);
@@ -780,13 +817,20 @@ static bool _shoals_tide_sweep_items_clear(coord_def c)
         // item clear here, let dungeon_terrain_changed teleport the item
         // to the nearest safe square.
         item_def &item(*si);
+
         // Let the tide break up stacks
-        if (!is_rune(item) && coinflip())
+        if (!item_is_rune(item) && coinflip())
+            continue;
+
+        if (item_is_stationary(item) && !one_chance_in(5))
             continue;
 
         const coord_def target(_shoals_escape_place_from(c, NULL, &item));
         if (!target.origin())
         {
+            if (item_is_stationary(item))
+                _clear_net_trapping_status(c);
+
             int id = si.link();
             move_item_to_grid(&id, target);
         }
@@ -798,12 +842,12 @@ static bool _shoals_tide_sweep_items_clear(coord_def c)
 static bool _shoals_tide_sweep_actors_clear(coord_def c)
 {
     actor *victim = actor_at(c);
-    if (!victim || victim->airborne() || victim->swimming())
+    if (!victim || !victim->ground_level() || victim->swimming())
         return true;
 
     if (victim->atype() == ACT_MONSTER)
     {
-        monsters *mvictim = dynamic_cast<monsters *>(victim);
+        const monster* mvictim = victim->as_monster();
         // Plants and statues cannot be moved away; the tide cannot
         // drown them.
         if (mons_class_is_stationary(mvictim->type))
@@ -819,7 +863,14 @@ static bool _shoals_tide_sweep_actors_clear(coord_def c)
     if (evacuation_point.origin())
         return false;
 
-    victim->move_to_pos(evacuation_point);
+    bool clear_net = false;
+    if (victim->caught())
+    {
+        int net = get_trapping_net(c);
+        if (net != NON_ITEM)
+            clear_net = !move_item_to_grid(&net, evacuation_point);
+    }
+    victim->move_to_pos(evacuation_point, clear_net);
     return true;
 }
 
@@ -836,20 +887,25 @@ static void _shoals_tide_wash_blood_away_at(coord_def c)
     env.pgrid(c) &= ~FPROP_BLOODY;
 }
 
-static void _shoals_apply_tide_feature_at(coord_def c,
-                                          dungeon_feature_type feat)
+static dungeon_feature_type _shoals_apply_tide_feature_at(
+    coord_def c,
+    dungeon_feature_type feat)
 {
     if (feat == DNGN_DEEP_WATER && !_shoals_tide_sweep_clear(c))
         feat = DNGN_SHALLOW_WATER;
 
     const dungeon_feature_type current_feat = grd(c);
+
+    // Return DNGN_UNSEEN if the feature isn't changed.
     if (feat == current_feat)
-        return;
+        return (DNGN_UNSEEN);
 
     if (Generating_Level)
         grd(c) = feat;
     else
         dungeon_terrain_changed(c, feat, true, false, true);
+
+    return (feat);
 }
 
 // Determines if the tide is rising or falling based on before and
@@ -865,7 +921,7 @@ static tide_direction _shoals_feature_tide_height_change(
     return height_delta < 0? TIDE_RISING : TIDE_FALLING;
 }
 
-static void _shoals_apply_tide_at(coord_def c, int tide)
+static void _shoals_apply_tide_at(coord_def c, int tide, bool incremental_tide)
 {
     if (is_tide_immune(c))
         return;
@@ -878,6 +934,17 @@ static void _shoals_apply_tide_at(coord_def c, int tide)
         newfeat = DNGN_FLOOR;
     const dungeon_feature_type oldfeat = grd(c);
 
+    // If the tide is affecting squares incrementally, never go
+    // straight from floor -> deep water or vice versa, always force
+    // an intermediate shallow water step.
+    if (incremental_tide)
+    {
+        const int oldfeat_seq = _shoals_feature_sequence_number(oldfeat);
+        const int newfeat_seq = _shoals_feature_sequence_number(newfeat);
+        if (std::abs(oldfeat_seq - newfeat_seq) >= 2)
+            newfeat = DNGN_SHALLOW_WATER;
+    }
+
     if (oldfeat == newfeat
         || (_shoals_feature_tide_height_change(oldfeat, newfeat) !=
             _shoals_tide_direction))
@@ -885,7 +952,18 @@ static void _shoals_apply_tide_at(coord_def c, int tide)
         return;
     }
 
-    _shoals_apply_tide_feature_at(c, newfeat);
+    const dungeon_feature_type final_feature =
+        _shoals_apply_tide_feature_at(c, newfeat);
+
+    if (incremental_tide
+        && final_feature == DNGN_DEEP_WATER
+        && c == you.pos()
+        && !you.ground_level()
+        && !you.permanent_levitation()
+        && !you.permanent_flight())
+    {
+        mprf(MSGCH_WARN, "The tide rushes in under you.");
+    }
 }
 
 static int _shoals_tide_at(coord_def pos, int base_tide)
@@ -893,6 +971,7 @@ static int _shoals_tide_at(coord_def pos, int base_tide)
     if (!tide_caller)
         return base_tide;
 
+    // try to avoid the costly sqrt() call
     const int rl_distance = grid_distance(pos, tide_caller_pos);
     if (rl_distance > TIDE_CALL_RADIUS)
         return base_tide;
@@ -910,7 +989,7 @@ static std::vector<coord_def> _shoals_extra_tide_seeds()
     return find_marker_positions_by_prop("tide_seed");
 }
 
-static void _shoals_apply_tide(int tide)
+static void _shoals_apply_tide(int tide, bool incremental_tide)
 {
     std::vector<coord_def> pages[2];
     int current_page = 0;
@@ -941,7 +1020,8 @@ static void _shoals_apply_tide(int tide)
             const bool was_wet(_shoals_tide_passable_feat(herefeat));
             seen_points(c) = true;
             if (_shoals_tide_susceptible_feat(herefeat))
-                _shoals_apply_tide_at(c, _shoals_tide_at(c, tide));
+                _shoals_apply_tide_at(c, _shoals_tide_at(c, tide),
+                                      incremental_tide);
 
             const bool is_wet(feat_is_water(grd(c)));
 
@@ -988,17 +1068,17 @@ static void _shoals_init_tide()
     CrawlHashTable &props = you.props;
     if (!props.exists(PROPS_SHOALS_TIDE_KEY))
     {
-        props[PROPS_SHOALS_TIDE_KEY] = short(0);
-        props[PROPS_SHOALS_TIDE_VEL] = short(PEAK_TIDE_VELOCITY);
-        props[PROPS_SHOALS_TIDE_UPDATE_TIME] = 0L;
+        props[PROPS_SHOALS_TIDE_KEY].get_short() = 0;
+        props[PROPS_SHOALS_TIDE_VEL].get_short() = PEAK_TIDE_VELOCITY;
+        props[PROPS_SHOALS_TIDE_UPDATE_TIME].get_int() = 0;
     }
     if (!env.properties.exists(PROPS_SHOALS_TIDE_KEY))
     {
-        env.properties[PROPS_SHOALS_TIDE_KEY] = short(0);
+        env.properties[PROPS_SHOALS_TIDE_KEY].get_short() = 0;
     }
 }
 
-static monsters *_shoals_find_tide_caller()
+static monster* _shoals_find_tide_caller()
 {
     for (monster_iterator mi; mi; ++mi)
         if (mi->has_ench(ENCH_TIDE))
@@ -1006,7 +1086,7 @@ static monsters *_shoals_find_tide_caller()
     return NULL;
 }
 
-void shoals_apply_tides(long turns_elapsed, bool force)
+void shoals_apply_tides(int turns_elapsed, bool force, bool incremental_tide)
 {
     if (!player_in_branch(BRANCH_SHOALS)
         || (!turns_elapsed && !force)
@@ -1022,9 +1102,9 @@ void shoals_apply_tides(long turns_elapsed, bool force)
     // has been updating the tide.
     if (turns_elapsed > 1)
     {
-        const long last_updated_time =
-            props[PROPS_SHOALS_TIDE_UPDATE_TIME].get_long();
-        const long turn_delta = (you.elapsed_time - last_updated_time) / 10;
+        const int last_updated_time =
+            props[PROPS_SHOALS_TIDE_UPDATE_TIME].get_int();
+        const int turn_delta = (you.elapsed_time - last_updated_time) / 10;
         turns_elapsed = std::min(turns_elapsed, turn_delta);
     }
 
@@ -1033,11 +1113,11 @@ void shoals_apply_tides(long turns_elapsed, bool force)
     if (turns_elapsed > TIDE_UNIT * 2)
         turns_elapsed = turns_elapsed % TIDE_UNIT + TIDE_UNIT;
 
-    unwind_var<monsters*> tide_caller_unwind(tide_caller,
+    unwind_var<monster* > tide_caller_unwind(tide_caller,
                                              _shoals_find_tide_caller());
     if (tide_caller)
     {
-        tide_called_turns = tide_caller->props[TIDE_CALL_TURN].get_long();
+        tide_called_turns = tide_caller->props[TIDE_CALL_TURN].get_int();
         tide_called_turns = you.num_turns - tide_called_turns;
         if (tide_called_turns < 1L)
             tide_called_turns = 1L;
@@ -1052,10 +1132,10 @@ void shoals_apply_tides(long turns_elapsed, bool force)
     while (turns_elapsed-- > 0)
         _shoals_run_tide(tide, acc);
 
-    props[PROPS_SHOALS_TIDE_KEY] = short(tide);
-    props[PROPS_SHOALS_TIDE_VEL] = short(acc);
-    props[PROPS_SHOALS_TIDE_UPDATE_TIME] = you.elapsed_time;
-    env.properties[PROPS_SHOALS_TIDE_KEY] = short(tide);
+    props[PROPS_SHOALS_TIDE_KEY].get_short() = tide;
+    props[PROPS_SHOALS_TIDE_VEL].get_short() = acc;
+    props[PROPS_SHOALS_TIDE_UPDATE_TIME].get_int() = you.elapsed_time;
+    env.properties[PROPS_SHOALS_TIDE_KEY].get_short() = tide;
 
     if (force
         || tide_caller
@@ -1063,22 +1143,22 @@ void shoals_apply_tides(long turns_elapsed, bool force)
     {
         _shoals_tide_direction =
             tide > old_tide ? TIDE_RISING : TIDE_FALLING;
-        _shoals_apply_tide(tide / TIDE_MULTIPLIER);
+        _shoals_apply_tide(tide / TIDE_MULTIPLIER, incremental_tide);
     }
 }
 
-void shoals_release_tide(monsters *mons)
+void shoals_release_tide(monster* mons)
 {
     if (player_in_branch(BRANCH_SHOALS))
     {
         if (player_can_hear(mons->pos()))
         {
             mprf(MSGCH_SOUND, "The tide is released from %s call.",
-                 mons->name(DESC_NOCAP_YOUR, true).c_str());
+                 apostrophise(mons->name(DESC_NOCAP_YOUR, true)).c_str());
             if (you.see_cell(mons->pos()))
                 flash_view_delay(ETC_WATER, 150);
         }
-        shoals_apply_tides(0, true);
+        shoals_apply_tides(0, true, true);
     }
 }
 
@@ -1102,7 +1182,7 @@ static void _shoals_force_tide(CrawlHashTable &props, int increment)
     tide = std::min(HIGH_TIDE, std::max(LOW_TIDE, tide));
     props[PROPS_SHOALS_TIDE_KEY] = short(tide);
     _shoals_tide_direction = increment > 0 ? TIDE_RISING : TIDE_FALLING;
-    _shoals_apply_tide(tide / TIDE_MULTIPLIER);
+    _shoals_apply_tide(tide / TIDE_MULTIPLIER, false);
 }
 
 void wizard_mod_tide()
@@ -1124,7 +1204,7 @@ void wizard_mod_tide()
         const int res =
             cancelable_get_line(buf, sizeof buf, NULL, _tidemod_keyfilter);
         mesclr(true);
-        if (res == ESCAPE)
+        if (key_is_escape(res))
             break;
         if (!res)
         {
@@ -1135,7 +1215,7 @@ void wizard_mod_tide()
         if (res == '+' || res == '-')
         {
             _shoals_force_tide(you.props, res == '+'? 2 : -2);
-            viewwindow(false, true);
+            viewwindow();
         }
     }
 }
