@@ -1,13 +1,14 @@
-/*
- * File:      exclude.cc
- * Summary:   Code related to travel exclusions.
- */
+/**
+ * @file
+ * @brief Code related to travel exclusions.
+**/
 
 #include "AppHdr.h"
 
 #include "exclude.h"
 
 #include <algorithm>
+#include <sstream>
 
 #include "cloud.h"
 #include "coord.h"
@@ -22,10 +23,10 @@
 #include "tags.h"
 #include "terrain.h"
 #include "travel.h"
-#include "tutorial.h"
+#include "hints.h"
 #include "view.h"
 
-static bool _mon_needs_auto_exclude(const monsters *mon, bool sleepy = false)
+static bool _mon_needs_auto_exclude(const monster* mon, bool sleepy = false)
 {
     if (mons_is_stationary(mon))
     {
@@ -41,7 +42,7 @@ static bool _mon_needs_auto_exclude(const monsters *mon, bool sleepy = false)
 }
 
 // Check whether a given monster is listed in the auto_exclude option.
-bool need_auto_exclude(const monsters *mon, bool sleepy)
+bool need_auto_exclude(const monster* mon, bool sleepy)
 {
     // This only works if the name is lowercased.
     std::string name = mon->name(DESC_BASENAME,
@@ -52,7 +53,8 @@ bool need_auto_exclude(const monsters *mon, bool sleepy)
     for (unsigned i = 0; i < Options.auto_exclude.size(); ++i)
         if (Options.auto_exclude[i].matches(name)
             && _mon_needs_auto_exclude(mon, sleepy)
-            && mon->attitude == ATT_HOSTILE)
+            && (mon->attitude == ATT_HOSTILE
+                || mon->type == MONS_HYPERACTIVE_BALLISTOMYCETE))
         {
             return (true);
         }
@@ -60,13 +62,27 @@ bool need_auto_exclude(const monsters *mon, bool sleepy)
     return (false);
 }
 
+// Nightstalker reduces LOS, so reducing the maximum exclusion radius
+// only makes sense. This is only possible because it's a permanent
+// mutation; the lantern of Shadows should not have this effect.
+int _get_full_exclusion_radius()
+{
+    return (LOS_RADIUS - player_mutation_level(MUT_NIGHTSTALKER));
+}
+
 // If the monster is in the auto_exclude list, automatically set an
 // exclusion.
-void set_auto_exclude(const monsters *mon)
+void set_auto_exclude(const monster* mon)
 {
+    if (!player_in_mappable_area())
+        return;
+
     if (need_auto_exclude(mon) && !is_exclude_root(mon->pos()))
     {
-        set_exclude(mon->pos(), LOS_RADIUS, true);
+        int rad = _get_full_exclusion_radius();
+        if (mon->type == MONS_HYPERACTIVE_BALLISTOMYCETE)
+            rad = 2;
+        set_exclude(mon->pos(), rad, true);
         // FIXME: If this happens for several monsters in the same turn
         //        (as is possible for some vaults), this could be really
         //        annoying. (jpeg)
@@ -75,50 +91,48 @@ void set_auto_exclude(const monsters *mon)
              mon->name(DESC_NOCAP_THE).c_str());
 
 #ifdef USE_TILE
-        viewwindow(false);
+        viewwindow();
 #endif
-        learned_something_new(TUT_AUTO_EXCLUSION, mon->pos());
+        learned_something_new(HINT_AUTO_EXCLUSION, mon->pos());
     }
 }
 
 // Clear auto exclusion if the monster is killed or wakes up with the
 // player in sight. If sleepy is true, stationary monsters are ignored.
-void remove_auto_exclude(const monsters *mon, bool sleepy)
+void remove_auto_exclude(const monster* mon, bool sleepy)
 {
     if (need_auto_exclude(mon, sleepy))
     {
         del_exclude(mon->pos());
 #ifdef USE_TILE
-        viewwindow(false);
+        viewwindow();
 #endif
     }
 }
 
-opacity_type _feat_opacity(dungeon_feature_type feat)
+static opacity_type _feat_opacity(dungeon_feature_type feat)
 {
     return (feat_is_opaque(feat) ? OPC_OPAQUE : OPC_CLEAR);
 }
 
 // A cell is considered clear unless the player knows it's
 // opaque.
-struct opacity_excl : public opacity_func
+class opacity_excl : public opacity_func
 {
+public:
     CLONE(opacity_excl)
 
     opacity_type operator()(const coord_def& p) const
     {
-        if (!is_terrain_seen(p))
+        map_cell& cell = env.map_knowledge(p);
+        if (!cell.seen())
             return OPC_CLEAR;
-        else if (!is_terrain_changed(p))
+        else if (!cell.changed())
             return _feat_opacity(env.grid(p));
-        else if (env.map_knowledge(p).object.cls == SH_FEATURE)
-            return _feat_opacity(env.map_knowledge(p).object.feat);
+        else if (cell.feat() != DNGN_UNSEEN)
+            return _feat_opacity(cell.feat());
         else
-        {
-            // If you have seen monsters, items or clouds there,
-            // it must have been passable.
             return OPC_CLEAR;
-        }
     }
 };
 static opacity_excl opc_excl;
@@ -236,14 +250,14 @@ void exclude_set::add_exclude_points(travel_exclude& ex)
             exclude_points.insert(*ri);
 }
 
-void exclude_set::update_excluded_points()
+void exclude_set::update_excluded_points(bool recompute_los)
 {
     for (iterator it = exclude_roots.begin(); it != exclude_roots.end(); ++it)
     {
         travel_exclude &ex = it->second;
         if (!ex.uptodate)
         {
-            recompute_excluded_points();
+            recompute_excluded_points(recompute_los);
             return;
         }
     }
@@ -278,7 +292,7 @@ travel_exclude* exclude_set::get_exclude_root(const coord_def &p)
     if (it != exclude_roots.end())
         return (&(it->second));
 
-    return  (false);
+    return (NULL);
 }
 
 size_t exclude_set::size() const
@@ -313,13 +327,13 @@ exclude_set::iterator exclude_set::end()
 
 /////////////////////////////////////////////////////////////////////////
 
-void _mark_excludes_non_updated(const coord_def &p)
+static void _mark_excludes_non_updated(const coord_def &p)
 {
     for (exclude_set::iterator it = curr_excludes.begin();
          it != curr_excludes.end(); ++it)
     {
         travel_exclude &ex = it->second;
-        ex.uptodate = ex.uptodate && ex.in_bounds(p);
+        ex.uptodate = ex.uptodate && !ex.in_bounds(p);
     }
 }
 
@@ -342,7 +356,7 @@ void update_exclusion_los(std::vector<coord_def> changed)
     for (unsigned int i = 0; i < changed.size(); ++i)
         _mark_excludes_non_updated(changed[i]);
 
-    curr_excludes.update_excluded_points();
+    curr_excludes.update_excluded_points(true);
 }
 
 bool is_excluded(const coord_def &p, const exclude_set &exc)
@@ -355,6 +369,25 @@ bool is_exclude_root(const coord_def &p)
     return (curr_excludes.get_exclude_root(p));
 }
 
+int get_exclusion_radius(const coord_def &p)
+{
+    if (travel_exclude *exc = curr_excludes.get_exclude_root(p))
+    {
+        if (!exc->radius)
+            return 1;
+        else
+            return exc->radius;
+    }
+    return 0;
+}
+
+std::string get_exclusion_desc(const coord_def &p)
+{
+    if (travel_exclude *exc = curr_excludes.get_exclude_root(p))
+        return exc->desc;
+    return "";
+}
+
 #ifdef USE_TILE
 // update Gmap for squares surrounding exclude centre
 static void _tile_exclude_gmap_update(const coord_def &p)
@@ -362,23 +395,17 @@ static void _tile_exclude_gmap_update(const coord_def &p)
     for (int x = -8; x <= 8; x++)
         for (int y = -8; y <= 8; y++)
         {
-            int px = p.x+x, py = p.y+y;
-            if (in_bounds(coord_def(px,py)))
-            {
-                tiles.update_minimap(px, py);
-            }
+            const coord_def pc(p.x + x, p.y + y);
+            if (in_bounds(pc))
+                tiles.update_minimap(pc);
         }
 }
 #endif
 
 static void _exclude_update()
 {
-    if (can_travel_interlevel())
-    {
-        LevelInfo &li = travel_cache.get_level_info(level_id::current());
-        li.update();
-    }
     set_level_exclusion_annotation(curr_excludes.get_exclusion_desc());
+    travel_cache.update_excludes();
 }
 
 static void _exclude_update(const coord_def &p)
@@ -411,31 +438,55 @@ void clear_excludes()
         return;
 
 #ifdef USE_TILE
-    exclude_set::iterator it;
-    //for (int i = curr_excludes.size()-1; i >= 0; i--)
-    for (it = curr_excludes.begin(); it != curr_excludes.end(); ++it)
-        _tile_exclude_gmap_update(it->second.pos);
+    // Tiles needs to update the minimap for each exclusion that is removed,
+    // but the exclusions need to be removed first.  Therefore, make a copy
+    // of the current set of exclusions and iterate through the copy below.
+    exclude_set excludes = curr_excludes;
 #endif
 
     curr_excludes.clear();
     clear_level_exclusion_annotation();
 
+#ifdef USE_TILE
+    for (exclude_set::iterator it = excludes.begin(); it != excludes.end(); ++it)
+        _tile_exclude_gmap_update(it->second.pos);
+#endif
+
     _exclude_update();
 }
 
+static void _exclude_gate(const coord_def &p, bool del = false)
+{
+    std::set<coord_def> all_doors;
+    find_connected_identical(p, grd(p), all_doors);
+    for (std::set<coord_def>::const_iterator dc = all_doors.begin();
+         dc != all_doors.end(); ++dc)
+    {
+        if (del)
+            del_exclude(*dc);
+        else
+            set_exclude(*dc, 0);
+    }
+}
+
 // Cycles the radius of an exclusion, including "off" state;
 // may start at 0 < radius < LOS_RADIUS, but won't cycle there.
 void cycle_exclude_radius(const coord_def &p)
 {
     if (travel_exclude *exc = curr_excludes.get_exclude_root(p))
     {
-        if (exc->radius == LOS_RADIUS)
+        if (feat_is_door(grd(p)))
+        {
+            _exclude_gate(p, exc->radius == 0);
+            return;
+        }
+        if (exc->radius > 0)
             set_exclude(p, 0);
         else
             del_exclude(p);
     }
     else
-        set_exclude(p, LOS_RADIUS);
+        set_exclude(p, _get_full_exclusion_radius());
 }
 
 // Remove a possible exclude.
@@ -449,9 +500,7 @@ void del_exclude(const coord_def &p)
 void set_exclude(const coord_def &p, int radius, bool autoexcl, bool vaultexcl,
                  bool defer_updates)
 {
-    // Sanity checks; excludes can be set in Pan and regular dungeon
-    // levels only.
-    if (!player_in_mappable_area())
+    if (!player_in_mappable_area()) // currently only Abyss
         return;
 
     if (!in_bounds(p))
@@ -461,10 +510,9 @@ void set_exclude(const coord_def &p, int radius, bool autoexcl, bool vaultexcl,
     {
         if (exc->desc.empty() && defer_updates)
         {
-            int cl = env.cgrid(p);
-
+            const int cl = env.cgrid(p);
             if (env.cgrid(p) != EMPTY_CLOUD)
-                exc->desc = cloud_name(cl) + " cloud";
+                exc->desc = cloud_name_at_index(cl) + " cloud";
         }
         else if (exc->radius == radius)
             return;
@@ -480,17 +528,17 @@ void set_exclude(const coord_def &p, int radius, bool autoexcl, bool vaultexcl,
         {
             // Don't list a monster in the exclusion annotation if the
             // exclusion was triggered by e.g. the flamethrowers' lua check.
-            const show_type& obj = get_map_knowledge_obj(p);
-            if (obj.cls == SH_MONSTER)
+            const map_cell& cell = env.map_knowledge(p);
+            if (cell.monster() != MONS_NO_MONSTER)
             {
-                desc = mons_type_name(obj.mons, DESC_PLAIN);
-                if (is_map_knowledge_detected_mons(p))
+                desc = mons_type_name(cell.monster(), DESC_PLAIN);
+                if (cell.detected_monster())
                     desc += " (detected)";
             }
             else
             {
                 // Maybe it's a door or staircase?
-                const dungeon_feature_type feat = env.map_knowledge(p).feat();
+                const dungeon_feature_type feat = cell.feat();
                 if (feat_is_door(feat))
                     desc = "door";
                 else
@@ -505,10 +553,9 @@ void set_exclude(const coord_def &p, int radius, bool autoexcl, bool vaultexcl,
         }
         else
         {
-            int cl = env.cgrid(p);
-
+            const int cl = env.cgrid(p);
             if (env.cgrid(p) != EMPTY_CLOUD)
-                desc = cloud_name(cl) + " cloud";
+                desc = cloud_name_at_index(cl) + " cloud";
         }
 
         curr_excludes.add_exclude(p, radius, autoexcl, desc, vaultexcl);
@@ -527,8 +574,10 @@ void maybe_remove_autoexclusion(const coord_def &p)
         if (!exc->autoex)
             return;
 
-        const monsters *m = monster_at(p);
-        if (!m || !you.can_see(m) || m->attitude != ATT_HOSTILE
+        const monster* m = monster_at(p);
+        if (!m || !you.can_see(m)
+            || m->attitude != ATT_HOSTILE
+                && m->type != MONS_HYPERACTIVE_BALLISTOMYCETE
             || strcmp(mons_type_name(m->type, DESC_PLAIN).c_str(),
                       exc->desc.c_str()) != 0)
         {
@@ -546,6 +595,11 @@ std::string exclude_set::get_exclusion_desc()
          it != exclude_roots.end(); ++it)
     {
         travel_exclude &ex = it->second;
+
+        // Don't count cloud exclusions.
+        if (strstr(ex.desc.c_str(), "cloud"))
+            continue;
+
         if (ex.desc != "")
             desc.push_back(ex.desc);
         else
@@ -634,7 +688,7 @@ void marshallExcludes(writer& outf, const exclude_set& excludes)
     }
 }
 
-void unmarshallExcludes(reader& inf, char minorVersion, exclude_set &excludes)
+void unmarshallExcludes(reader& inf, int minorVersion, exclude_set &excludes)
 {
     excludes.clear();
     int nexcludes = unmarshallShort(inf);
@@ -642,8 +696,7 @@ void unmarshallExcludes(reader& inf, char minorVersion, exclude_set &excludes)
     {
         for (int i = 0; i < nexcludes; ++i)
         {
-            coord_def c;
-            unmarshallCoord(inf, c);
+            const coord_def c      = unmarshallCoord(inf);
             const int radius       = unmarshallShort(inf);
             const bool autoexcl    = unmarshallBoolean(inf);
             const std::string desc = unmarshallString(inf);
