@@ -37,9 +37,9 @@ void apply_feature_overrides()
         feature_def                 &feat   = Features[fov.object];
 
         if (ofeat.symbol)
-            feat.symbol = ofeat.symbol;
+            feat.symbol = get_glyph_override(ofeat.symbol);
         if (ofeat.magic_symbol)
-            feat.magic_symbol = ofeat.magic_symbol;
+            feat.magic_symbol = get_glyph_override(ofeat.magic_symbol);
         if (ofeat.colour)
             feat.colour = ofeat.colour;
         if (ofeat.map_colour)
@@ -53,11 +53,12 @@ void apply_feature_overrides()
     }
 }
 
-void _init_feat(feature_def &f, dungeon_feature_type feat)
+static void _init_feat(feature_def &f, dungeon_feature_type feat)
 {
     switch(feat)
     {
         case DNGN_UNSEEN:
+        case DNGN_EXPLORE_HORIZON:
         default:
             break;
 
@@ -76,6 +77,13 @@ void _init_feat(feature_def &f, dungeon_feature_type feat)
             f.minimap      = MF_WALL;
             break;
 
+        case DNGN_SLIMY_WALL:
+            f.dchar        = DCHAR_WALL;
+            f.colour       = LIGHTGREEN;
+            f.magic_symbol = Options.char_table[ DCHAR_WALL_MAGIC ];
+            f.minimap      = MF_WALL;
+            break;
+
         case DNGN_CLEAR_ROCK_WALL:
         case DNGN_CLEAR_STONE_WALL:
         case DNGN_CLEAR_PERMAROCK_WALL:
@@ -85,19 +93,29 @@ void _init_feat(feature_def &f, dungeon_feature_type feat)
             f.minimap      = MF_WALL;
             break;
 
-        case DNGN_TREES:
-            f.dchar        = DCHAR_TREES;
+        case DNGN_GRATE:
+            f.dchar        = DCHAR_GRATE;
             f.magic_symbol = Options.char_table[ DCHAR_WALL_MAGIC ];
-            f.colour       = BLACK; // overridden later
+            f.colour       = LIGHTBLUE;
+            f.minimap      = MF_WALL;
+            break;
+
+        case DNGN_TREE:
+            f.dchar        = DCHAR_TREE;
+            f.magic_symbol = Options.char_table[ DCHAR_WALL_MAGIC ];
+            f.colour       = ETC_TREE;
+            f.minimap      = MF_WALL;
+            break;
+
+        case DNGN_SWAMP_TREE:
+            f.dchar        = DCHAR_TREE;
+            f.magic_symbol = Options.char_table[ DCHAR_WALL_MAGIC ];
+            f.colour       = ETC_SWAMP_TREE;
             f.minimap      = MF_WALL;
             break;
 
         case DNGN_OPEN_SEA:
-#ifdef USE_TILE
-            f.dchar        = DCHAR_WAVY;
-#else
             f.dchar        = DCHAR_WALL;
-#endif
             f.colour       = BLUE;
             f.minimap      = MF_WATER;
             break;
@@ -181,13 +199,6 @@ void _init_feat(feature_def &f, dungeon_feature_type feat)
             f.minimap      = MF_FLOOR;
             break;
 
-        case DNGN_FLOOR_SPECIAL:
-            f.dchar        = DCHAR_FLOOR;
-            f.colour       = YELLOW;
-            f.magic_symbol = Options.char_table[ DCHAR_FLOOR_MAGIC ];
-            f.minimap      = MF_FLOOR;
-            break;
-
         case DNGN_EXIT_HELL:
             f.dchar       = DCHAR_ARCH;
             f.colour      = LIGHTRED;
@@ -270,6 +281,15 @@ void _init_feat(feature_def &f, dungeon_feature_type feat)
             f.minimap     = MF_STAIR_BRANCH;
             break;
 
+        case DNGN_TEMP_PORTAL:
+            f.dchar       = DCHAR_ARCH;
+            f.colour      = ETC_SHIMMER_BLUE;
+            f.map_colour  = LIGHTGREY;
+            f.colour      = ETC_SHIMMER_BLUE;
+            f.minimap     = MF_STAIR_UP;
+            break;
+
+
         case DNGN_ESCAPE_HATCH_DOWN:
             f.dchar      = DCHAR_STAIRS_DOWN;
             f.colour     = BROWN;
@@ -390,6 +410,7 @@ void _init_feat(feature_def &f, dungeon_feature_type feat)
             f.minimap     = MF_STAIR_BRANCH;
             break;
 
+        case DNGN_ENTER_DWARVEN_HALL:
         case DNGN_ENTER_ORCISH_MINES:
         case DNGN_ENTER_HIVE:
         case DNGN_ENTER_LAIR:
@@ -420,6 +441,7 @@ void _init_feat(feature_def &f, dungeon_feature_type feat)
             f.minimap     = MF_STAIR_BRANCH;
             break;
 
+        case DNGN_RETURN_FROM_DWARVEN_HALL:
         case DNGN_RETURN_FROM_ORCISH_MINES:
         case DNGN_RETURN_FROM_HIVE:
         case DNGN_RETURN_FROM_LAIR:
@@ -601,6 +623,15 @@ void _init_feat(feature_def &f, dungeon_feature_type feat)
             f.minimap     = MF_FEATURE;
             break;
 
+        case DNGN_ALTAR_ASHENZARI:
+            f.colour      = LIGHTRED;
+            f.dchar       = DCHAR_ALTAR;
+            f.flags      |= FFT_NOTABLE;
+            f.map_colour  = DARKGREY;
+            f.seen_colour = LIGHTRED;
+            f.minimap     = MF_FEATURE;
+            break;
+
         case DNGN_FOUNTAIN_BLUE:
             f.colour  = BLUE;
             f.dchar   = DCHAR_FOUNTAIN;
@@ -636,7 +667,7 @@ void _init_feat(feature_def &f, dungeon_feature_type feat)
     }
 }
 
-void _init_item(feature_def &f, show_item_type item)
+static void _init_item(feature_def &f, show_item_type item)
 {
     f.minimap = MF_ITEM;
     switch (item)
