@@ -1,7 +1,7 @@
-/*
- * File:      envmap.cc
- * Summary:   Functions dealing with env.map_knowledge.
- */
+/**
+ * @file
+ * @brief Functions dealing with env.map_knowledge.
+**/
 
 #include "AppHdr.h"
 
@@ -15,8 +15,11 @@ bool is_sanctuary(const coord_def& p)
 {
     if (!map_bounds(p))
         return (false);
-    return (testbits(env.pgrid(p), FPROP_SANCTUARY_1)
-            || testbits(env.pgrid(p), FPROP_SANCTUARY_2));
+    const bool sanct = (testbits(env.pgrid(p), FPROP_SANCTUARY_1)
+                        || testbits(env.pgrid(p), FPROP_SANCTUARY_2));
+    if (sanct)
+        ASSERT(in_bounds(env.sanctuary_pos));
+    return (sanct);
 }
 
 bool is_bloodcovered(const coord_def& p)
@@ -31,13 +34,27 @@ bool is_tide_immune(const coord_def &p)
 
 bool is_moldy(const coord_def & p)
 {
-    return (env.pgrid(p) & FPROP_MOLD);
+    return (env.pgrid(p) & FPROP_MOLD
+            || env.pgrid(p) & FPROP_GLOW_MOLD);
+}
+
+bool glowing_mold(const coord_def & p)
+{
+    return (env.pgrid(p) & FPROP_GLOW_MOLD);
+}
+
+void remove_mold(const coord_def & p)
+{
+    env.pgrid(p) &= ~FPROP_MOLD;
+    env.pgrid(p) &= ~FPROP_GLOW_MOLD;
 }
 
 feature_property_type str_to_fprop(const std::string &str)
 {
     if (str == "bloody")
         return (FPROP_BLOODY);
+    if (str == "highlight")
+        return (FPROP_HIGHLIGHT);
     if (str == "mold")
         return (FPROP_MOLD);
     if (str == "no_cloud_gen")
@@ -52,6 +69,15 @@ feature_property_type str_to_fprop(const std::string &str)
         return (FPROP_NO_TIDE);
     if (str == "no_submerge")
         return (FPROP_NO_SUBMERGE);
+    if (str == "no_jiyva")
+        return (FPROP_NO_JIYVA);
 
     return (FPROP_NONE);
 }
+
+#ifdef USE_TILE
+char blood_rotation(const coord_def & p)
+{
+    return (env.pgrid(p) & FPROP_BLOOD_EAST) >> 16;
+}
+#endif
