@@ -1,10 +1,7 @@
-/*
- * File:    ghost.cc
- * Summary: Player ghost and random Pandemonium demon handling.
- *
- * Created for Dungeon Crawl Reference by dshaligram on
- * Thu Mar 15 20:10:20 2007 UTC.
- */
+/**
+ * @file
+ * @brief Player ghost and random Pandemonium demon handling.
+**/
 
 #include "AppHdr.h"
 
@@ -21,9 +18,9 @@
 #include "ng-input.h"
 #include "random.h"
 #include "skills2.h"
+#include "spl-cast.h"
 #include "mon-util.h"
 #include "mon-transit.h"
-#include "place.h"
 #include "player.h"
 #include "religion.h"
 
@@ -69,6 +66,7 @@ static spell_type search_order_conj[] = {
     SPELL_SANDBLAST,
     SPELL_MAGIC_DART,
     SPELL_HIBERNATION,
+    SPELL_FLAME_TONGUE,
     SPELL_CORONA,
     SPELL_NO_SPELL,                        // end search
 };
@@ -76,24 +74,29 @@ static spell_type search_order_conj[] = {
 // Order for looking for summonings and self-enchants for the 3rd spell
 // slot.
 static spell_type search_order_third[] = {
-// 0
     SPELL_SYMBOL_OF_TORMENT,
     SPELL_SUMMON_GREATER_DEMON,
     SPELL_SUMMON_HORRIBLE_THINGS,
+    SPELL_SUMMON_DRAGON,
+    SPELL_TUKIMAS_BALL,
     SPELL_HAUNT,
+    SPELL_SUMMON_HYDRA,
     SPELL_SUMMON_DEMON,
     SPELL_DEMONIC_HORDE,
     SPELL_HASTE,
+    SPELL_SILENCE,
+    SPELL_SUMMON_BUTTERFLIES,
+    SPELL_SUMMON_ELEMENTAL,
     SPELL_SUMMON_SWARM,
     SPELL_SUMMON_UGLY_THING,
     SPELL_SWIFTNESS,
-// 10
     SPELL_SUMMON_ICE_BEAST,
     SPELL_ANIMATE_DEAD,
     SPELL_INVISIBILITY,
     SPELL_SUMMON_SCORPIONS,
     SPELL_CALL_IMP,
     SPELL_SUMMON_SMALL_MAMMALS,
+    SPELL_MALIGN_GATEWAY,
     SPELL_CONTROLLED_BLINK,
     SPELL_BLINK,
     SPELL_NO_SPELL,                        // end search
@@ -103,7 +106,6 @@ static spell_type search_order_third[] = {
 // this fails, go through conjurations.  Note: Dig must be in misc2
 // (5th) position to work.
 static spell_type search_order_misc[] = {
-// 0
     SPELL_AGONY,
     SPELL_BANISHMENT,
     SPELL_FREEZING_CLOUD,
@@ -114,7 +116,7 @@ static spell_type search_order_misc[] = {
     SPELL_SLOW,
     SPELL_POLYMORPH_OTHER,
     SPELL_TELEPORT_OTHER,
-// 10
+    SPELL_EVAPORATE, // replaced with Mephitic Cloud, though at lower priority
     SPELL_DIG,
     SPELL_CORONA,
     SPELL_NO_SPELL,                        // end search
@@ -162,13 +164,17 @@ void ghost_demon::init_random_demon()
     ev = 5 + random2(20);
     ac = 5 + random2(20);
 
+    // Is demon a spellcaster?
+    // Non-spellcasters get some boosts to their melee and speed instead.
+    spellcaster = !one_chance_in(10);
+
     see_invis = !one_chance_in(10);
 
     if (!one_chance_in(3))
         resists.fire = random_range(1, 2);
     else
     {
-        resists.fire = 0; // res_fire
+        resists.fire = 0;
 
         if (one_chance_in(10))
             resists.fire = -1;
@@ -179,6 +185,7 @@ void ghost_demon::init_random_demon()
     else
     {
         resists.cold = 0;
+
         if (one_chance_in(10))
             resists.cold = -1;
     }
@@ -194,7 +201,7 @@ void ghost_demon::init_random_demon()
     // special attack type (uses weapon brand code):
     brand = SPWPN_NORMAL;
 
-    if (!one_chance_in(3))
+    if (!one_chance_in(3) || !spellcaster)
     {
         do
         {
@@ -202,20 +209,16 @@ void ghost_demon::init_random_demon()
             // some brands inappropriate (e.g. holy wrath)
         }
         while (brand == SPWPN_HOLY_WRATH
-               || (brand == SPWPN_ORC_SLAYING
-                   && you.mons_species() != MONS_ORC)
-               || (brand == SPWPN_DRAGON_SLAYING
-                   && you.mons_species() != MONS_DRACONIAN)
+               || brand == SPWPN_ORC_SLAYING
+               || brand == SPWPN_DRAGON_SLAYING
                || brand == SPWPN_PROTECTION
+               || brand == SPWPN_EVASION
                || brand == SPWPN_FLAME
-               || brand == SPWPN_FROST);
+               || brand == SPWPN_FROST
+               || brand == SPWPN_RETURNING
+               || brand == SPWPN_REACHING);
     }
 
-    // Is demon a spellcaster?
-    // Upped from one_chance_in(3)... spellcasters are more interesting
-    // and I expect named demons to typically have a trick or two. - bwr
-    spellcaster = !one_chance_in(10);
-
     // Does demon fly?
     fly = (one_chance_in(3) ? FL_NONE :
            one_chance_in(5) ? FL_LEVITATE
@@ -244,7 +247,11 @@ void ghost_demon::init_random_demon()
             spells[1] = RANDOM_ELEMENT(search_order_conj);
 
         if (!one_chance_in(4))
+        {
             spells[2] = RANDOM_ELEMENT(search_order_third);
+            if (spells[2] == SPELL_TUKIMAS_BALL)
+                spells[2] = SPELL_NO_SPELL;
+        }
 
         if (coinflip())
         {
@@ -291,6 +298,10 @@ void ghost_demon::init_random_demon()
             spells[2] = SPELL_SMITING;
         if (one_chance_in(25))
             spells[2] = SPELL_HELLFIRE_BURST;
+        if (one_chance_in(22))
+            spells[2] = SPELL_SUMMON_HYDRA;
+        if (one_chance_in(20))
+            spells[2] = SPELL_SUMMON_DRAGON;
         if (one_chance_in(12))
             spells[2] = SPELL_SUMMON_GREATER_DEMON;
         if (one_chance_in(12))
@@ -302,6 +313,8 @@ void ghost_demon::init_random_demon()
             spells[3] = SPELL_SUMMON_GREATER_DEMON;
         if (one_chance_in(20))
             spells[3] = SPELL_SUMMON_DEMON;
+        if (one_chance_in(20))
+            spells[3] = SPELL_MALIGN_GATEWAY;
 
         // At least they can summon demons.
         if (spells[3] == SPELL_NO_SPELL)
@@ -316,10 +329,12 @@ void ghost_demon::init_random_demon()
 // real speed, not a movement cost, so higher is better.
 static int _player_ghost_base_movement_speed()
 {
-    int speed = (you.species == SP_NAGA ? 8 : 10);
+    int speed = 10;
 
     if (player_mutation_level(MUT_FAST))
         speed += player_mutation_level(MUT_FAST) + 1;
+    if (player_mutation_level(MUT_SLOW))
+        speed -= player_mutation_level(MUT_SLOW) + 1;
 
     if (player_equip_ego_type(EQ_BOOTS, SPARM_RUNNING))
         speed += 2;
@@ -356,6 +371,8 @@ void ghost_demon::init_player_ghost()
 
     if (you.weapon())
     {
+        // This includes ranged weapons, but they're treated as melee.
+
         const item_def& weapon = *you.weapon();
         if (weapon.base_type == OBJ_WEAPONS || weapon.base_type == OBJ_STAVES)
         {
@@ -372,13 +389,17 @@ void ghost_demon::init_player_ghost()
                 // the weapon.
                 if (brand == SPWPN_HOLY_WRATH)
                     brand = SPWPN_NORMAL;
+
+                // Don't copy ranged-only brands from launchers (reaping etc.).
+                if (brand > MAX_PAN_LORD_BRANDS)
+                    brand = SPWPN_NORMAL;
             }
         }
     }
     else
     {
         // Unarmed combat.
-        if (you.species == SP_TROLL)
+        if (you.innate_mutations[MUT_CLAWS])
             damage += you.experience_level;
 
         damage += you.skills[SK_UNARMED_COMBAT];
@@ -387,7 +408,7 @@ void ghost_demon::init_player_ghost()
     damage *= 30 + you.skills[SK_FIGHTING];
     damage /= 30;
 
-    damage += you.strength / 4;
+    damage += you.strength() / 4;
 
     if (damage > MAX_GHOST_DAMAGE)
         damage = MAX_GHOST_DAMAGE;
@@ -399,7 +420,7 @@ void ghost_demon::init_player_ghost()
     if (!is_good_god(you.religion))
         religion = you.religion;
 
-    best_skill = ::best_skill(SK_FIGHTING, (NUM_SKILLS - 1), 99);
+    best_skill = ::best_skill(SK_FIRST_SKILL, SK_LAST_SKILL);
     best_skill_level = you.skills[best_skill];
     xl = you.experience_level;
 
@@ -410,10 +431,10 @@ void ghost_demon::init_player_ghost()
     add_spells();
 }
 
-static unsigned char _ugly_thing_assign_colour(unsigned char force_colour,
-                                               unsigned char force_not_colour)
+static uint8_t _ugly_thing_assign_colour(uint8_t force_colour,
+                                         uint8_t force_not_colour)
 {
-    unsigned char colour;
+    uint8_t colour;
 
     if (force_colour != BLACK)
         colour = force_colour;
@@ -427,11 +448,34 @@ static unsigned char _ugly_thing_assign_colour(unsigned char force_colour,
     return (colour);
 }
 
-static mon_attack_flavour _ugly_thing_colour_to_flavour(unsigned char u_colour)
+static mon_attack_flavour _very_ugly_thing_flavour_upgrade(mon_attack_flavour u_att_flav)
+{
+    switch (u_att_flav)
+    {
+    case AF_FIRE:
+        u_att_flav = AF_NAPALM;
+        break;
+
+    case AF_POISON_NASTY:
+        u_att_flav = AF_POISON_MEDIUM;
+        break;
+
+    case AF_DISEASE:
+        u_att_flav = AF_ROT;
+        break;
+
+    default:
+        break;
+    }
+
+    return (u_att_flav);
+}
+
+mon_attack_flavour ugly_thing_colour_to_flavour(uint8_t u_colour)
 {
     mon_attack_flavour u_att_flav = AF_PLAIN;
 
-    switch (u_colour)
+    switch (make_low_colour(u_colour))
     {
     case RED:
         u_att_flav = AF_FIRE;
@@ -461,34 +505,14 @@ static mon_attack_flavour _ugly_thing_colour_to_flavour(unsigned char u_colour)
         break;
     }
 
-    return (u_att_flav);
-}
-
-static mon_attack_flavour _very_ugly_thing_flavour_upgrade(mon_attack_flavour u_att_flav)
-{
-    switch (u_att_flav)
-    {
-    case AF_FIRE:
-        u_att_flav = AF_NAPALM;
-        break;
-
-    case AF_POISON_NASTY:
-        u_att_flav = AF_POISON_MEDIUM;
-        break;
-
-    case AF_DISEASE:
-        u_att_flav = AF_ROT;
-        break;
-
-    default:
-        break;
-    }
+    if (is_high_colour(u_colour))
+        u_att_flav = _very_ugly_thing_flavour_upgrade(u_att_flav);
 
     return (u_att_flav);
 }
 
 void ghost_demon::init_ugly_thing(bool very_ugly, bool only_mutate,
-                                  unsigned char force_colour)
+                                  uint8_t force_colour)
 {
     // Movement speed: 11, the same as in mon-data.h.
     speed = 11;
@@ -516,7 +540,7 @@ void ghost_demon::init_ugly_thing(bool very_ugly, bool only_mutate,
     const mon_attack_type att_types[] =
     {
         AT_BITE, AT_STING, AT_ENGULF, AT_CLAW, AT_PECK, AT_HEADBUTT, AT_PUNCH,
-        AT_KICK, AT_TENTACLE_SLAP, AT_TAIL_SLAP, AT_GORE, AT_CONSTRICT
+        AT_KICK, AT_TENTACLE_SLAP, AT_TAIL_SLAP, AT_GORE, AT_TRUNK_SLAP
     };
 
     att_type = RANDOM_ELEMENT(att_types);
@@ -529,7 +553,7 @@ void ghost_demon::init_ugly_thing(bool very_ugly, bool only_mutate,
                                                    : BLACK);
 
     // Pick a compatible attack flavour for this colour.
-    att_flav = _ugly_thing_colour_to_flavour(colour);
+    att_flav = ugly_thing_colour_to_flavour(colour);
 
     // Pick a compatible resistance for this attack flavour.
     ugly_thing_add_resistance(false, att_flav);
@@ -565,9 +589,9 @@ void ghost_demon::ugly_thing_to_very_ugly_thing()
     ugly_thing_add_resistance(true, att_flav);
 }
 
-void ghost_demon::ugly_thing_add_resistance(bool very_ugly,
-                                            mon_attack_flavour u_att_flav)
+mon_resist_def ugly_thing_resists(bool very_ugly, mon_attack_flavour u_att_flav)
 {
+    mon_resist_def resists;
     resists.elec = 0;
     resists.poison = 0;
     resists.fire = 0;
@@ -609,6 +633,13 @@ void ghost_demon::ugly_thing_add_resistance(bool very_ugly,
     default:
         break;
     }
+    return resists;
+}
+
+void ghost_demon::ugly_thing_add_resistance(bool very_ugly,
+                                            mon_attack_flavour u_att_flav)
+{
+    resists = ::ugly_thing_resists(very_ugly, u_att_flav);
 }
 
 void ghost_demon::init_dancing_weapon(const item_def& weapon, int power)
@@ -620,11 +651,6 @@ void ghost_demon::init_dancing_weapon(const item_def& weapon, int power)
     if (power > 150)
         power = 150;
 
-    resists.poison = 1;
-    resists.fire = 1;
-    resists.cold = 1;
-    resists.elec = 1;
-
     colour = weapon.colour;
     fly = FL_LEVITATE;
 
@@ -682,6 +708,11 @@ void ghost_demon::init_dancing_weapon(const item_def& weapon, int power)
     // Sabre:             speed 11, 3+7  damage, 2 AC,  5 HP, 12 EV
 }
 
+static bool _know_spell(spell_type spell)
+{
+    return (you.has_spell(spell) && spell_fail(spell) < 50);
+}
+
 static spell_type search_first_list(int ignore_spell)
 {
     for (unsigned i = 0;
@@ -693,7 +724,7 @@ static spell_type search_first_list(int ignore_spell)
         if (search_order_conj[i] == ignore_spell)
             continue;
 
-        if (you.has_spell(search_order_conj[i]))
+        if (_know_spell(search_order_conj[i]))
             return (search_order_conj[i]);
     }
 
@@ -711,7 +742,7 @@ static spell_type search_second_list(int ignore_spell)
         if (search_order_third[i] == ignore_spell)
             continue;
 
-        if (you.has_spell(search_order_third[i]))
+        if (_know_spell(search_order_third[i]))
             return (search_order_third[i]);
     }
 
@@ -729,7 +760,7 @@ static spell_type search_third_list(int ignore_spell)
         if (search_order_misc[i] == ignore_spell)
             continue;
 
-        if (you.has_spell(search_order_misc[i]))
+        if (_know_spell(search_order_misc[i]))
             return (search_order_misc[i]);
     }
 
@@ -756,21 +787,31 @@ void ghost_demon::add_spells()
     if (spells[4] == SPELL_NO_SPELL)
         spells[4] = search_first_list(spells[3]);
 
-    if (you.has_spell(SPELL_DIG))
+    if (_know_spell(SPELL_DIG))
         spells[4] = SPELL_DIG;
 
     // Look for Blink or Teleport Self for the emergency slot.
-    if (you.has_spell(SPELL_CONTROLLED_BLINK)
-        || you.has_spell(SPELL_BLINK))
+    if (_know_spell(SPELL_CONTROLLED_BLINK)
+        || _know_spell(SPELL_BLINK))
     {
         spells[5] = SPELL_CONTROLLED_BLINK;
     }
 
-    if (you.has_spell(SPELL_TELEPORT_SELF))
+    if (_know_spell(SPELL_TELEPORT_SELF))
         spells[5] = SPELL_TELEPORT_SELF;
 
     for (int i = 0; i < NUM_MONSTER_SPELL_SLOTS; ++i)
         spells[i] = translate_spell(spells[i]);
+
+    spellcaster = has_spells();
+}
+
+bool ghost_demon::has_spells() const
+{
+    for (int i = 0; i < NUM_MONSTER_SPELL_SLOTS; ++i)
+        if (spells[i] != SPELL_NO_SPELL)
+            return true;
+    return false;
 }
 
 // When passed the number for a player spell, returns the equivalent
@@ -783,14 +824,14 @@ spell_type ghost_demon::translate_spell(spell_type spel) const
         return (SPELL_BLINK);        // approximate
     case SPELL_DEMONIC_HORDE:
         return (SPELL_CALL_IMP);
-    case SPELL_AGONY:
-    case SPELL_SYMBOL_OF_TORMENT:
-        // Too powerful to give ghosts Torment for Agony?  Nah.
-        return (SPELL_SYMBOL_OF_TORMENT);
     case SPELL_DELAYED_FIREBALL:
         return (SPELL_FIREBALL);
     case SPELL_PETRIFY:
         return (SPELL_PARALYSE);
+    case SPELL_EVAPORATE:
+        return (SPELL_MEPHITIC_CLOUD);
+    case SPELL_STICKY_FLAME:
+        return (SPELL_STICKY_FLAME_RANGE);
     default:
         break;
     }
@@ -802,6 +843,10 @@ std::vector<ghost_demon> ghost_demon::find_ghosts()
 {
     std::vector<ghost_demon> gs;
 
+    // No ghosts in the Temple.
+    if (player_in_branch(BRANCH_ECUMENICAL_TEMPLE))
+        return (gs);
+
     if (!you.is_undead)
     {
         ghost_demon player;
@@ -832,7 +877,7 @@ void ghost_demon::find_transiting_ghosts(
         {
             if (i->mons.type == MONS_PLAYER_GHOST)
             {
-                const monsters &m = i->mons;
+                const monster& m = i->mons;
                 if (m.ghost.get())
                 {
                     announce_ghost(*m.ghost);
@@ -846,12 +891,12 @@ void ghost_demon::find_transiting_ghosts(
 
 void ghost_demon::announce_ghost(const ghost_demon &g)
 {
-#if DEBUG_BONES | DEBUG_DIAGNOSTICS
+#if defined(DEBUG_BONES) || defined(DEBUG_DIAGNOSTICS)
     mprf(MSGCH_DIAGNOSTICS, "Saving ghost: %s", g.name.c_str());
 #endif
 }
 
-void ghost_demon::find_extra_ghosts( std::vector<ghost_demon> &gs, int n )
+void ghost_demon::find_extra_ghosts(std::vector<ghost_demon> &gs, int n)
 {
     for (monster_iterator mi; mi && n > 0; ++mi)
     {
@@ -871,40 +916,21 @@ void ghost_demon::find_extra_ghosts( std::vector<ghost_demon> &gs, int n )
 // Returns the number of extra ghosts allowed on the level.
 int ghost_demon::n_extra_ghosts()
 {
-    const int lev = you.absdepth0 + 1;
-    const int subdepth = subdungeon_depth(you.where_are_you, you.absdepth0);
-
-    if (you.level_type == LEVEL_PANDEMONIUM
-        || you.level_type == LEVEL_ABYSS
-        || (you.level_type == LEVEL_DUNGEON
-            && (you.where_are_you == BRANCH_CRYPT
-                || you.where_are_you == BRANCH_TOMB
-                || you.where_are_you == BRANCH_HALL_OF_ZOT
-                || player_in_hell()))
-        || lev > 22)
+    if (you.level_type != LEVEL_ABYSS
+        && you.level_type != LEVEL_PANDEMONIUM)
     {
-        return (MAX_GHOSTS - 1);
+        const int subdepth  = level_id::current().depth;
+        // Single ghosts-only: D:1-8, Lair:1, Orc:1, and non-dungeon
+        // areas at this depth, such as portal vaults.
+        if (subdepth < 9 && you.where_are_you == BRANCH_MAIN_DUNGEON
+            || subdepth < 2 && you.where_are_you == BRANCH_LAIR
+            || subdepth < 2 && you.where_are_you == BRANCH_ORCISH_MINES)
+        {
+            return (0);
+        }
     }
 
-    if (you.where_are_you == BRANCH_ECUMENICAL_TEMPLE)
-        return (0);
-
-    // No multiple ghosts until level 9 of the main dungeon.
-    if (lev < 9 && you.where_are_you == BRANCH_MAIN_DUNGEON
-        || subdepth < 2 && you.where_are_you == BRANCH_LAIR
-        || subdepth < 2 && you.where_are_you == BRANCH_ORCISH_MINES)
-    {
-        return (0);
-    }
-
-    if (you.where_are_you == BRANCH_LAIR
-        || you.where_are_you == BRANCH_ORCISH_MINES
-        || you.where_are_you == BRANCH_MAIN_DUNGEON && lev < 15)
-    {
-        return (1);
-    }
-
-    return (1 + x_chance_in_y(lev, 20) + x_chance_in_y(lev, 40));
+    return (MAX_GHOSTS - 1);
 }
 
 // Sanity checks for some ghost values.
@@ -959,7 +985,9 @@ bool debug_check_ghosts()
         // Name validation.
         if (!validate_player_name(ghost.name, false))
             return (false);
-        if (ghost.name.length() > (kNameLen - 1) || ghost.name.length() == 0)
+        // Many combining characters can come per every letter, but if there's
+        // that much, it's probably a maliciously forged ghost of some kind.
+        if (ghost.name.length() > kNameLen * 10 || ghost.name.empty())
             return (false);
         if (ghost.name != trimmed_string(ghost.name))
             return (false);
@@ -971,3 +999,169 @@ bool debug_check_ghosts()
     }
     return (true);
 }
+
+int ghost_level_to_rank(const int xl)
+{
+    if (xl <  4) return 0;
+    if (xl <  7) return 1;
+    if (xl < 11) return 2;
+    if (xl < 16) return 3;
+    if (xl < 22) return 4;
+    if (xl < 26) return 5;
+    if (xl < 27) return 6;
+    return 7;
+}
+
+///////////////////////////////////////////////////////////////////////////////
+// Laboratory rats!
+
+std::string adjective_for_labrat_colour (uint8_t l_colour)
+{
+    switch (l_colour)
+    {
+    case CYAN:           return ("armoured");
+    case YELLOW:         return ("beastly");
+    case RED:            return ("fiery");
+    case LIGHTCYAN:      return ("gaseous");
+    case LIGHTRED:       return ("parasitic");
+    case LIGHTBLUE:      return ("airborne");
+    case LIGHTMAGENTA:   return ("mutated");
+    case MAGENTA:        return ("shifting");
+    case GREEN:          return ("venomous");
+    case LIGHTGRAY:      return ("");
+    default:
+        die("invalid labrat adjective");
+        break;
+    }
+
+    return ("");
+}
+
+#ifdef USE_TILE
+int tile_offset_for_labrat_colour (uint8_t l_colour)
+{
+    switch (l_colour)
+    {
+    case LIGHTGRAY:      return (0);
+    case CYAN:           return (1);
+    case YELLOW:         return (2);
+    case RED:            return (3);
+    case LIGHTCYAN:      return (4);
+    case LIGHTRED:       return (5);
+    case LIGHTBLUE:      return (6);
+    case LIGHTMAGENTA:   return (7);
+    case MAGENTA:        return (8);
+    case GREEN:          return (9);
+    default:
+        return (0);
+    }
+}
+#endif
+
+uint8_t colour_for_labrat_adjective (std::string adjective)
+{
+    if (adjective == "armoured")    return CYAN;
+    if (adjective == "beastly")     return YELLOW;
+    if (adjective == "fiery")       return RED;
+    if (adjective == "gaseous")     return LIGHTCYAN;
+    if (adjective == "parasitic")   return LIGHTRED;
+    if (adjective == "airborne")    return LIGHTBLUE;
+    if (adjective == "mutated")     return LIGHTMAGENTA;
+    if (adjective == "shifting")    return MAGENTA;
+    if (adjective == "venomous")    return GREEN;
+    if (adjective == "plain")       return LIGHTGRAY;
+
+    return BLACK;
+}
+
+static const uint8_t labrat_colour_values[] = {
+    CYAN, YELLOW, RED, LIGHTCYAN, LIGHTRED, LIGHTBLUE, LIGHTMAGENTA, MAGENTA, GREEN
+};
+
+uint8_t _labrat_random_colour()
+{
+    return (RANDOM_ELEMENT(labrat_colour_values));
+}
+
+void ghost_demon::init_labrat (uint8_t force_colour)
+{
+    // Base init for "plain" laboratory rats. Kept in line with mon-data.h.
+    xl = 5;
+    max_hp = hit_points(xl, 3, 5);
+    ac = 5;
+    ev = 5;
+    speed = 12;
+    colour = force_colour;
+    damage = 9 + random2(3);
+
+    if (colour == BLACK)
+        colour = _labrat_random_colour();
+
+    spells.init(SPELL_NO_SPELL);
+
+    resists.elec = 0;
+    resists.poison = 0;
+    resists.fire = 0;
+    resists.sticky_flame = false;
+    resists.cold = 0;
+    resists.acid = 0;
+    resists.rotting = false;
+
+    switch (colour)
+    {
+    case CYAN: // armoured
+        ac = 20;
+        speed = 9;
+        break;
+    case YELLOW: // beastly
+        xl = 4 + random2(4);
+        ac = 2 + random2(5);
+        ev = 7 + random2(5);
+        speed = 10 + random2(8);
+        break;
+    case RED: // fiery
+        att_flav = AF_FIRE;
+        spells[0] = SPELL_FIRE_BREATH;
+        spellcaster = true;
+        resists.fire = 3;
+        resists.sticky_flame = true;
+        break;
+    case LIGHTCYAN: // gaseous
+        spells[0] = SPELL_MEPHITIC_CLOUD;
+        spellcaster = true;
+        resists.poison = 1; // otherwise it'll confuse itself
+        break;
+    case LIGHTRED: // leeching
+        att_flav = AF_VAMPIRIC;
+        break;
+    case LIGHTBLUE: // floating
+        fly = FL_LEVITATE;
+        spells[0] = SPELL_SHOCK;
+        spellcaster = true;
+        resists.elec = 1;
+        speed = 15;
+        ev = 15;
+        break;
+    case LIGHTMAGENTA: // mutated
+    {
+        att_flav = AF_MUTATE;
+        const mon_attack_type possibles[] = { AT_CLAW, AT_PECK,
+                AT_TENTACLE_SLAP, AT_TRUNK_SLAP, AT_SNAP, AT_SPLASH };
+        att_type = RANDOM_ELEMENT(possibles);
+        break;
+    }
+    case MAGENTA:
+        spells[0] = spells[1] = spells[2] = spells[3] =
+            spells[4] = spells[5] = SPELL_BLINK;
+        spellcaster = true;
+        break;
+    case GREEN:
+        att_flav = AF_POISON;
+        resists.poison = 3;
+        break;
+    case LIGHTGRAY:
+        break;
+    default:
+        break;
+    }
+}
