@@ -3,6 +3,7 @@
 #include "godconduct.h"
 
 #include "fight.h"
+#include "godpassive.h"
 #include "godwrath.h"
 #include "monster.h"
 #include "mon-util.h"
@@ -16,18 +17,18 @@
 // god_conduct_trigger
 
 god_conduct_trigger::god_conduct_trigger(
-    conduct_type c, int pg, bool kn, const monsters *vict)
+    conduct_type c, int pg, bool kn, const monster* vict)
   : conduct(c), pgain(pg), known(kn), enabled(true), victim(NULL)
 {
     if (vict)
     {
-        victim.reset(new monsters);
+        victim.reset(new monster);
         *(victim.get()) = *vict;
     }
 }
 
 void god_conduct_trigger::set(conduct_type c, int pg, bool kn,
-                              const monsters *vict)
+                              const monster* vict)
 {
     conduct = c;
     pgain = pg;
@@ -35,7 +36,7 @@ void god_conduct_trigger::set(conduct_type c, int pg, bool kn,
     victim.reset(NULL);
     if (vict)
     {
-        victim.reset(new monsters);
+        victim.reset(new monster);
         *victim.get() = *vict;
     }
 }
@@ -49,15 +50,16 @@ god_conduct_trigger::~god_conduct_trigger()
 // This function is the merger of done_good() and naughty().
 // Returns true if god was interested (good or bad) in conduct.
 bool did_god_conduct(conduct_type thing_done, int level, bool known,
-                     const monsters *victim)
+                     const monster* victim)
 {
-    ASSERT(!crawl_state.arena);
+    ASSERT(!crawl_state.game_is_arena());
 
     bool retval = false;
 
     if (you.religion != GOD_NO_GOD && you.religion != GOD_XOM)
     {
         int piety_change = 0;
+        int piety_denom = 1;
         int penance = 0;
 
         god_acting gdact;
@@ -122,12 +124,13 @@ bool did_god_conduct(conduct_type thing_done, int level, bool known,
         case DID_NECROMANCY:
         case DID_UNHOLY:
         case DID_ATTACK_HOLY:
+        case DID_VIOLATE_HOLY_CORPSE:
             switch (you.religion)
             {
             case GOD_ZIN:
             case GOD_SHINING_ONE:
             case GOD_ELYVILON:
-                if (!known && thing_done != DID_ATTACK_HOLY)
+                if (!known && thing_done != DID_ATTACK_HOLY && thing_done != DID_VIOLATE_HOLY_CORPSE)
                 {
                     simple_god_message(" forgives your inadvertent unholy act, "
                                        "just this once.");
@@ -190,7 +193,7 @@ bool did_god_conduct(conduct_type thing_done, int level, bool known,
             break;
 
        case DID_KILL_SLIME:
-            if (you.religion == GOD_JIYVA)
+            if (you.religion == GOD_JIYVA && !victim->is_shapeshifter())
             {
                 retval = true;
                 piety_change = -level;
@@ -199,7 +202,7 @@ bool did_god_conduct(conduct_type thing_done, int level, bool known,
             break;
 
        case DID_KILL_PLANT:
-       case DID_ALLY_KILLED_PLANT:
+       case DID_PLANT_KILLED_BY_SERVANT:
            // Piety loss but no penance for killing a plant.
            if (you.religion == GOD_FEDHAS)
            {
@@ -234,7 +237,8 @@ bool did_god_conduct(conduct_type thing_done, int level, bool known,
                 break;
 
             case GOD_JIYVA:
-                if (victim && mons_is_slime(victim))
+                if (victim && mons_is_slime(victim)
+                    && !victim->is_shapeshifter())
                 {
                     piety_change = -(level/2 + 3);
                     penance = level/2 + 3;
@@ -265,13 +269,13 @@ bool did_god_conduct(conduct_type thing_done, int level, bool known,
             break;
 
         case DID_FRIEND_DIED:
+        case DID_SOULED_FRIEND_DIED:
             switch (you.religion)
             {
-
             case GOD_FEDHAS:
-                // Ballistomycetes dying is penalized separately.
+                // Ballistomycetes dying is penalised separately.
                 if (victim && fedhas_protects(victim)
-                    && victim->type != MONS_BALLISTOMYCETE)
+                    && victim->mons_species() != MONS_BALLISTOMYCETE)
                 {
                     // level is (1 + monsterHD/2) for this conduct,
                     // trying a fixed cost since plant HD aren't that
@@ -297,6 +301,10 @@ bool did_god_conduct(conduct_type thing_done, int level, bool known,
                 // Converted allies (marked as TSOites) can be martyrs.
                 if (victim && victim->god == GOD_SHINING_ONE)
                     break;
+
+                // Zin only cares about the deaths of those with souls.
+                if (thing_done == DID_FRIEND_DIED)
+                    break;
                 // fall through
 
             case GOD_OKAWARU:
@@ -314,10 +322,10 @@ bool did_god_conduct(conduct_type thing_done, int level, bool known,
             {
             case GOD_ELYVILON:
                 // Killing is only disapproved of during prayer.
-                if (you.duration[DUR_PRAYER])
+                if (you.duration[DUR_LIFESAVING])
                 {
                     simple_god_message(" does not appreciate your shedding "
-                                       "blood during prayer!");
+                                       "blood when asking for salvation!");
                     retval = true;
                     piety_change = -level;
                     penance = level * 2;
@@ -337,8 +345,10 @@ bool did_god_conduct(conduct_type thing_done, int level, bool known,
 
                 simple_god_message(" accepts your kill.");
                 retval = true;
-                if (random2(level + 18 - you.experience_level / 2) > 5)
-                    piety_change = 1;
+                piety_denom = level + 18 - you.experience_level / 2;
+                piety_change = piety_denom - 6;
+                piety_denom = std::max(piety_denom, 1);
+                piety_change = std::max(piety_change, 0);
                 break;
 
             default:
@@ -361,11 +371,11 @@ bool did_god_conduct(conduct_type thing_done, int level, bool known,
                 simple_god_message(" accepts your kill.");
                 retval = true;
                 // Holy gods are easier to please this way.
-                if (random2(level + 18 - (is_good_god(you.religion) ? 0 :
-                                          you.experience_level / 2)) > 4)
-                {
-                    piety_change = 1;
-                }
+                piety_denom = level + 18 - (is_good_god(you.religion) ? 0 :
+                                          you.experience_level / 2);
+                piety_change = piety_denom - 5;
+                piety_denom = std::max(piety_denom, 1);
+                piety_change = std::max(piety_change, 0);
                 break;
 
             default:
@@ -388,11 +398,11 @@ bool did_god_conduct(conduct_type thing_done, int level, bool known,
                 simple_god_message(" accepts your kill.");
                 retval = true;
                 // Holy gods are easier to please this way.
-                if (random2(level + 18 - (is_good_god(you.religion) ? 0 :
-                                          you.experience_level / 2)) > 3)
-                {
-                    piety_change = 1;
-                }
+                piety_denom = level + 18 - (is_good_god(you.religion) ? 0 :
+                                          you.experience_level / 2);
+                piety_change = piety_denom - 4;
+                piety_denom = std::max(piety_denom, 1);
+                piety_change = std::max(piety_change, 0);
                 break;
 
             default:
@@ -407,8 +417,8 @@ bool did_god_conduct(conduct_type thing_done, int level, bool known,
             {
                 simple_god_message(" accepts your kill.");
                 retval = true;
-                if (random2(level + 18) > 3)
-                    piety_change = 1;
+                piety_denom = level + 18;
+                piety_change = piety_denom - 4;
             }
             break;
 
@@ -419,8 +429,8 @@ bool did_god_conduct(conduct_type thing_done, int level, bool known,
             {
                 simple_god_message(" accepts your kill.");
                 retval = true;
-                if (random2(level + 18) > 3)
-                    piety_change = 1;
+                piety_denom = level + 18;
+                piety_change = piety_denom - 4;
             }
             break;
 
@@ -431,8 +441,8 @@ bool did_god_conduct(conduct_type thing_done, int level, bool known,
                 simple_god_message(" appreciates your killing of a heretic "
                                    "priest.");
                 retval = true;
-                if (random2(level + 10) > 5)
-                    piety_change = 1;
+                piety_denom = level + 10;
+                piety_change = piety_denom - 6;
             }
             break;
 
@@ -443,8 +453,8 @@ bool did_god_conduct(conduct_type thing_done, int level, bool known,
                 simple_god_message(" appreciates your killing of a magic "
                                    "user.");
                 retval = true;
-                if (random2(level + 10) > 5)
-                    piety_change = 1;
+                piety_denom = level + 10;
+                piety_change = piety_denom - 6;
             }
             break;
 
@@ -453,21 +463,34 @@ bool did_god_conduct(conduct_type thing_done, int level, bool known,
                 && !god_hates_attacking_friend(you.religion, victim))
             {
                 retval = true;
-                if (random2(level + 18 - you.experience_level / 2) > 5)
+                piety_denom = level + 18 - you.experience_level / 2;
+                piety_change = piety_denom - 6;
+                piety_denom = std::max(piety_denom, 1);
+                piety_change = std::max(piety_change, 0);
+
+                const int speed_delta =
+                    cheibriados_monster_player_speed_delta(victim);
+                dprf("Che DID_KILL_FAST: %s speed delta: %d",
+                     victim->name(DESC_PLAIN, true).c_str(),
+                     speed_delta);
+                if (speed_delta > 0 && x_chance_in_y(speed_delta, 12))
                 {
-                    piety_change = 1;
-                    const int speed_delta =
-                        cheibriados_monster_player_speed_delta(victim);
-                    dprf("Che DID_KILL_FAST: %s speed delta: %d",
-                         victim->name(DESC_PLAIN, true).c_str(),
-                         speed_delta);
-                    if (speed_delta > 0 && x_chance_in_y(speed_delta, 12))
-                        ++piety_change;
+                    piety_change *= 2;
+                    simple_god_message(" thoroughly appreciates the change of pace.");
                 }
-                simple_god_message(
-                    piety_change == 2
-                    ? " thoroughly appreciates the change of pace."
-                    : " appreciates the change of pace.");
+                else
+                    simple_god_message(" appreciates the change of pace.");
+            }
+            break;
+
+        case DID_KILL_ARTIFICIAL:
+            if (you.religion == GOD_YREDELEMNUL
+                && !god_hates_attacking_friend(you.religion, victim))
+            {
+                simple_god_message(" accepts your kill.");
+                retval = true;
+                piety_denom = level + 18;
+                piety_change = piety_denom - 3;
             }
             break;
 
@@ -497,21 +520,21 @@ bool did_god_conduct(conduct_type thing_done, int level, bool known,
             case GOD_MAKHLEB:
             case GOD_BEOGH:
             case GOD_LUGONU:
+            case GOD_OKAWARU:
                 if (god_hates_attacking_friend(you.religion, victim))
                     break;
 
                 simple_god_message(" accepts your kill.");
                 retval = true;
-                if (random2(level + 18) > 2)
-                    piety_change = 1;
+                piety_denom = level + 18;
+                piety_change = piety_denom - 3;
 
                 if (you.religion == GOD_YREDELEMNUL)
                 {
                     simple_god_message(" appreciates your killing of a holy "
                                        "being.");
                     retval = true;
-                    if (random2(level + 10) > 5)
-                        piety_change = 1;
+                    piety_change *= 2;
                 }
                 break;
 
@@ -533,8 +556,8 @@ bool did_god_conduct(conduct_type thing_done, int level, bool known,
 
                 simple_god_message(" accepts your slave's kill.");
                 retval = true;
-                if (random2(level + 18) > 2)
-                    piety_change = 1;
+                piety_denom = level + 18;
+                piety_change = piety_denom - 3;
                 break;
 
             default:
@@ -568,8 +591,8 @@ bool did_god_conduct(conduct_type thing_done, int level, bool known,
 
                 simple_god_message(" accepts your collateral kill.");
                 retval = true;
-                if (random2(level + 18) > 2)
-                    piety_change = 1;
+                piety_denom = level + 18;
+                piety_change = piety_denom - 3;
                 break;
 
             default:
@@ -588,8 +611,10 @@ bool did_god_conduct(conduct_type thing_done, int level, bool known,
             case GOD_LUGONU:
                 simple_god_message(" accepts your slave's kill.");
                 retval = true;
-                if (random2(level + 10 - you.experience_level/3) > 5)
-                    piety_change = 1;
+                piety_denom = level + 10 - you.experience_level/3;
+                piety_change = piety_denom - 6;
+                piety_denom = std::max(piety_denom, 1);
+                piety_change = std::max(piety_change, 0);
                 break;
             default:
                 break;
@@ -606,8 +631,10 @@ bool did_god_conduct(conduct_type thing_done, int level, bool known,
             case GOD_LUGONU:
                 simple_god_message(" accepts your collateral kill.");
                 retval = true;
-                if (random2(level + 10 - you.experience_level/3) > 5)
-                    piety_change = 1;
+                piety_denom = level + 10 - you.experience_level/3;
+                piety_change = piety_denom - 6;
+                piety_denom = std::max(piety_denom, 1);
+                piety_change = std::max(piety_change, 0);
                 break;
             default:
                 break;
@@ -623,8 +650,10 @@ bool did_god_conduct(conduct_type thing_done, int level, bool known,
             case GOD_LUGONU:
                 simple_god_message(" accepts your slave's kill.");
                 retval = true;
-                if (random2(level + 10 - you.experience_level/3) > 5)
-                    piety_change = 1;
+                piety_denom = level + 10 - you.experience_level/3;
+                piety_change = piety_denom - 6;
+                piety_denom = std::max(piety_denom, 1);
+                piety_change = std::max(piety_change, 0);
                 break;
             default:
                 break;
@@ -641,11 +670,11 @@ bool did_god_conduct(conduct_type thing_done, int level, bool known,
             case GOD_LUGONU:
                 simple_god_message(" accepts your collateral kill.");
                 retval = true;
-                if (random2(level + 10 - (is_good_god(you.religion) ? 0 :
-                                          you.experience_level/3)) > 5)
-                {
-                    piety_change = 1;
-                }
+                piety_denom = level + 10 - (is_good_god(you.religion) ? 0 :
+                                            you.experience_level/3);
+                piety_change = piety_denom - 6;
+                piety_denom = std::max(piety_denom, 1);
+                piety_change = std::max(piety_change, 0);
                 break;
             default:
                 break;
@@ -655,12 +684,15 @@ bool did_god_conduct(conduct_type thing_done, int level, bool known,
         case DID_DEMON_KILLED_BY_UNDEAD_SLAVE:
             switch (you.religion)
             {
+            case GOD_KIKUBAAQUDGHA:
             case GOD_MAKHLEB:
             case GOD_BEOGH:
                 simple_god_message(" accepts your slave's kill.");
                 retval = true;
-                if (random2(level + 10 - you.experience_level/3) > 5)
-                    piety_change = 1;
+                piety_denom = level + 10 - you.experience_level/3;
+                piety_change = piety_denom - 6;
+                piety_denom = std::max(piety_denom, 1);
+                piety_change = std::max(piety_change, 0);
                 break;
             default:
                 break;
@@ -676,11 +708,11 @@ bool did_god_conduct(conduct_type thing_done, int level, bool known,
             case GOD_BEOGH:
                 simple_god_message(" accepts your collateral kill.");
                 retval = true;
-                if (random2(level + 10 - (is_good_god(you.religion) ? 0 :
-                                          you.experience_level/3)) > 5)
-                {
-                    piety_change = 1;
-                }
+                piety_denom = level + 10 - (is_good_god(you.religion) ? 0 :
+                                            you.experience_level/3);
+                piety_change = piety_denom - 6;
+                piety_denom = std::max(piety_denom, 1);
+                piety_change = std::max(piety_change, 0);
                 break;
             default:
                 break;
@@ -693,12 +725,38 @@ bool did_god_conduct(conduct_type thing_done, int level, bool known,
             {
                 simple_god_message(" accepts your collateral kill.");
                 retval = true;
-
-                if (random2(level + 10) > 5)
-                    piety_change = 1;
+                piety_denom = level + 10;
+                piety_change = piety_denom - 6;
             }
             break;
 
+        case DID_UNCLEAN_KILLED_BY_SERVANT:
+        case DID_CHAOTIC_KILLED_BY_SERVANT:
+            if (you.religion == GOD_ZIN)
+            {
+                simple_god_message(" accepts your collateral kill.");
+                retval = true;
+
+                piety_denom = level + 10;
+                piety_change = piety_denom - 6;
+            }
+            break;
+
+        case DID_ARTIFICIAL_KILLED_BY_UNDEAD_SLAVE:
+            if (you.religion == GOD_YREDELEMNUL)
+            {
+                simple_god_message(" accepts your slave's kill.");
+                retval = true;
+                piety_denom = level + 18;
+                piety_change = piety_denom - 3;
+            }
+            break;
+
+        // Currently used only when confused undead kill artificial
+        // beings, which Yredelemnul doesn't care about.
+        case DID_ARTIFICIAL_KILLED_BY_SERVANT:
+            break;
+
         case DID_SPELL_MEMORISE:
             if (you.religion == GOD_TROG)
             {
@@ -733,8 +791,8 @@ bool did_god_conduct(conduct_type thing_done, int level, bool known,
                 // kills tend to do both at the same time (unlike
                 // melee).  This means high level spells probably work
                 // pretty much like they used to (use spell, get piety).
-                piety_change = div_rand_round(level + 10, 80);
-                dprf("Spell practise, level: %d, dpiety: %d", level, piety_change);
+                piety_change = level + 10;
+                piety_denom = 80;
                 retval = true;
             }
             break;
@@ -764,7 +822,7 @@ bool did_god_conduct(conduct_type thing_done, int level, bool known,
                     && you.attribute[ATTR_CARD_COUNTDOWN])
                 {
                     you.attribute[ATTR_CARD_COUNTDOWN]--;
-#if DEBUG_DIAGNOSTICS || DEBUG_CARDS || DEBUG_GIFTS
+#if defined(DEBUG_DIAGNOSTICS) || defined(DEBUG_CARDS) || defined(DEBUG_GIFTS)
                     mprf(MSGCH_DIAGNOSTICS, "Countdown down to %d",
                          you.attribute[ATTR_CARD_COUNTDOWN]);
 #endif
@@ -786,11 +844,17 @@ bool did_god_conduct(conduct_type thing_done, int level, bool known,
                 if (thing_done == DID_CAUSE_GLOWING)
                 {
                     static long last_glowing_lecture = -1L;
-                    if (last_glowing_lecture != you.num_turns)
+                    if (!level)
+                    {
+                        simple_god_message(" is not enthusiastic about the "
+                                           "mutagenic glow surrounding you.");
+                    }
+                    else if (last_glowing_lecture != you.num_turns)
                     {
-                        simple_god_message(" does not appreciate the mutagenic "
-                                           "glow surrounding you!");
                         last_glowing_lecture = you.num_turns;
+                        // Increase contamination within yellow glow.
+                        simple_god_message(" does not appreciate the extra "
+                                           "mutagenic glow surrounding you!");
                     }
                 }
 
@@ -841,6 +905,14 @@ bool did_god_conduct(conduct_type thing_done, int level, bool known,
             }
             break;
 
+        case DID_ATTACK_IN_SANCTUARY:
+            if (you.religion == GOD_ZIN)
+            {
+                piety_change = -level;
+                retval = true;
+            }
+            break;
+
         case DID_DESECRATE_ORCISH_REMAINS:
             if (you.religion == GOD_BEOGH)
             {
@@ -874,45 +946,50 @@ bool did_god_conduct(conduct_type thing_done, int level, bool known,
             }
             break;
 
-        case DID_GLUTTONY:
-            if (you.religion == GOD_CHEIBRIADOS)
+        case DID_DESTROY_SPELLBOOK:
+            if (you.religion == GOD_SIF_MUNA)
             {
-                if (x_chance_in_y(level, 2000))
-                {
-                    // Honeycomb or greater guarantees piety gain.
-                    // Message in here to avoid grape message spam.
-                    simple_god_message(" encourages your appreciation of food.");
-                    piety_change = 1;
-                }
+                piety_change = -level;
+                penance = level * (known ? 2 : 1);
+                retval = true;
+            }
+            break;
+
+        case DID_EXPLORATION:
+            if (you.religion == GOD_ASHENZARI)
+            {
+                // levels: x1, x2, x3, x4
+                piety_change = 1 + ash_bondage_level();
+                piety_change *= 8; // base gain per dungeon level
+                piety_denom = level;
                 retval = true;
             }
             break;
 
         case DID_NOTHING:
-        case DID_STABBING:                          // unused
-        case DID_STIMULANTS:                        // unused
-        case DID_EAT_MEAT:                          // unused
-        case DID_CREATE_LIFE:                       // unused
-        case DID_SPELL_NONUTILITY:                  // unused
-        case DID_DEDICATED_BUTCHERY:                // unused
         case NUM_CONDUCTS:
             break;
         }
 
-        if (piety_change > 0)
-            gain_piety(piety_change);
-        else
-            dock_piety(-piety_change, penance);
+#ifdef DEBUG_DIAGNOSTICS
+        int old_piety = you.piety;
+#endif
 
-#if DEBUG_DIAGNOSTICS
-        if (retval)
+        if (piety_change > 0)
+            gain_piety(piety_change, piety_denom);
+        else
+            dock_piety(div_rand_round(-piety_change, piety_denom), penance);
+
+#ifdef DEBUG_DIAGNOSTICS
+        // don't announce exploration piety unless you actually got a boost
+        if (retval && (thing_done != DID_EXPLORATION || old_piety != you.piety))
         {
             static const char *conducts[] =
             {
                 "",
                 "Necromancy", "Holy", "Unholy", "Attack Holy", "Attack Neutral",
-                "Attack Friend", "Friend Died", "Stab", "Unchivalric Attack",
-                "Poison", "Field Sacrifice", "Kill Living", "Kill Undead",
+                "Attack Friend", "Friend Died", "Unchivalric Attack",
+                "Poison", "Kill Living", "Kill Undead",
                 "Kill Demon", "Kill Natural Unholy", "Kill Natural Evil",
                 "Kill Unclean", "Kill Chaotic", "Kill Wizard", "Kill Priest",
                 "Kill Holy", "Kill Fast", "Undead Slave Kill Living",
@@ -921,19 +998,25 @@ bool did_god_conduct(conduct_type thing_done, int level, bool known,
                 "Servant Kill Demon", "Servant Kill Natural Unholy",
                 "Servant Kill Natural Evil", "Undead Slave Kill Holy",
                 "Servant Kill Holy", "Spell Memorise", "Spell Cast",
-                "Spell Practise", "Spell Nonutility", "Cards", "Stimulants",
-                "Drink Blood", "Cannibalism", "Eat Meat", "Eat Souled Being",
+                "Spell Practise", "Cards",
+                "Drink Blood", "Cannibalism","Eat Souled Being",
                 "Deliberate Mutation", "Cause Glowing", "Use Unclean",
                 "Use Chaos", "Desecrate Orcish Remains", "Destroy Orcish Idol",
-                "Create Life", "Kill Slime", "Kill Plant", "Ally Kill Plant",
-                "Was Hasty", "Gluttony", "Corpse Violation"
+                "Kill Slime", "Kill Plant", "Servant Kill Plant",
+                "Was Hasty", "Corpse Violation",
+                "Souled Friend Died", "Servant Kill Unclean",
+                "Servant Kill Chaotic", "Attack In Sanctuary",
+                "Kill Artificial", "Undead Slave Kill Artificial",
+                "Servant Kill Artificial", "Destroy Spellbook",
+                "Exploration", "Desecrated Holy Remains",
             };
 
             COMPILE_CHECK(ARRAYSZ(conducts) == NUM_CONDUCTS, c1);
             mprf(MSGCH_DIAGNOSTICS,
-                 "conduct: %s; piety: %d (%+d); penance: %d (%+d)",
+                 "conduct: %s; piety: %d (%+d/%d); penance: %d (%+d)",
                  conducts[thing_done],
-                 you.piety, piety_change, you.penance[you.religion], penance);
+                 you.piety, piety_change, piety_denom,
+                 you.penance[you.religion], penance);
 
         }
 #endif
@@ -955,9 +1038,9 @@ bool did_god_conduct(conduct_type thing_done, int level, bool known,
 // since a Beogh worshipper zapping an orc with lightning might cause it to
 // become a follower on the first hit, and the second hit would be
 // against a friendly orc.
-static FixedVector<bool, NUM_MONSTERS> _first_attack_conduct;
-static FixedVector<bool, NUM_MONSTERS> _first_attack_was_unchivalric;
-static FixedVector<bool, NUM_MONSTERS> _first_attack_was_friendly;
+static FixedVector<bool, MAX_MONSTERS> _first_attack_conduct;
+static FixedVector<bool, MAX_MONSTERS> _first_attack_was_unchivalric;
+static FixedVector<bool, MAX_MONSTERS> _first_attack_was_friendly;
 
 void god_conduct_turn_start()
 {
@@ -966,25 +1049,15 @@ void god_conduct_turn_start()
     _first_attack_was_friendly.init(false);
 }
 
-#define NEW_GIFT_FLAGS (MF_JUST_SUMMONED | MF_GOD_GIFT)
-
-void set_attack_conducts(god_conduct_trigger conduct[3], const monsters *mon,
+void set_attack_conducts(god_conduct_trigger conduct[3], const monster* mon,
                          bool known)
 {
     const unsigned int midx = mon->mindex();
 
     if (mon->friendly())
     {
-        if ((mon->flags & NEW_GIFT_FLAGS) == NEW_GIFT_FLAGS
-            && mon->god != GOD_XOM)
-        {
-            mprf(MSGCH_ERROR, "Newly created friendly god gift '%s' was hurt "
-                 "by you, shouldn't be possible; please file a bug report.",
-                 mon->name(DESC_PLAIN, true).c_str());
-            _first_attack_was_friendly[midx] = true;
-        }
-        else if (_first_attack_conduct[midx]
-                 || _first_attack_was_friendly[midx])
+        if (_first_attack_conduct[midx]
+            || _first_attack_was_friendly[midx])
         {
             conduct[0].set(DID_ATTACK_FRIEND, 5, known, mon);
             _first_attack_was_friendly[midx] = true;
