@@ -1,7 +1,7 @@
-/*
- *  File:       goditem.cc
- *  Summary:    Gods' attitude towards items.
- */
+/**
+ * @file
+ * @brief Gods' attitude towards items.
+**/
 
 #include "AppHdr.h"
 
@@ -126,8 +126,8 @@ bool is_potentially_evil_item(const item_def& item)
     return (false);
 }
 
-// This is a subset of is_evil_item
-bool is_corpse_violating_item(const item_def & item)
+// This is a subset of is_evil_item().
+bool is_corpse_violating_item(const item_def& item)
 {
     bool retval = false;
 
@@ -145,8 +145,8 @@ bool is_corpse_violating_item(const item_def & item)
     {
         const int item_brand = get_weapon_brand(item);
         retval = (item_brand == SPWPN_REAPING);
-    }
         break;
+    }
     case OBJ_MISSILES:
     {
         const int item_brand = get_ammo_brand(item);
@@ -190,11 +190,11 @@ bool is_evil_item(const item_def& item)
         }
         break;
     case OBJ_MISSILES:
-    {
+        {
         const int item_brand = get_ammo_brand(item);
         retval = (item_brand == SPMSL_REAPING);
+        }
         break;
-    }
     case OBJ_WANDS:
         retval = (item.sub_type == WAND_DRAINING);
         break;
@@ -236,6 +236,7 @@ bool is_unclean_item(const item_def& item)
     {
     case OBJ_BOOKS:
         retval = is_unclean_spellbook(item);
+        break;
     case OBJ_STAVES:
         retval = is_unclean_rod(item);
         break;
@@ -291,12 +292,44 @@ bool is_chaotic_item(const item_def& item)
         break;
     }
 
-    if (is_artefact(item) && artefact_wpn_property(item, ARTP_MUTAGENIC))
+    if (is_artefact(item) && item.base_type != OBJ_BOOKS
+        && artefact_wpn_property(item, ARTP_MUTAGENIC))
+    {
         retval = true;
+    }
 
     return (retval);
 }
 
+bool is_potentially_hasty_item(const item_def& item)
+{
+    switch (item.base_type)
+    {
+    case OBJ_WEAPONS:
+        {
+        const int item_brand = get_weapon_brand(item);
+        if (item_brand == SPWPN_CHAOS)
+            return (true);
+        }
+        break;
+    case OBJ_MISSILES:
+        {
+        const int item_brand = get_ammo_brand(item);
+        if (item_brand == SPMSL_CHAOS || item_brand == SPMSL_RAGE)
+            return (true);
+        }
+        break;
+    case OBJ_WANDS:
+        if (item.sub_type == WAND_RANDOM_EFFECTS)
+            return (true);
+        break;
+    default:
+        break;
+    }
+
+    return (false);
+}
+
 bool is_hasty_item(const item_def& item)
 {
     bool retval = false;
@@ -336,7 +369,7 @@ bool is_hasty_item(const item_def& item)
         break;
     }
 
-    if (is_artefact(item)
+    if (is_artefact(item) && item.base_type != OBJ_BOOKS
         && (artefact_wpn_property(item, ARTP_ANGRY)
             || artefact_wpn_property(item, ARTP_BERSERK)))
     {
@@ -346,6 +379,35 @@ bool is_hasty_item(const item_def& item)
     return (retval);
 }
 
+bool is_poisoned_item(const item_def& item)
+{
+    switch (item.base_type)
+    {
+    case OBJ_WEAPONS:
+        {
+        const int item_brand = get_weapon_brand(item);
+        if (item_brand == SPWPN_VENOM)
+            return (true);
+        }
+        break;
+    case OBJ_MISSILES:
+        {
+        const int item_brand = get_ammo_brand(item);
+        if (item_brand == SPMSL_POISONED || item_brand == SPMSL_CURARE)
+            return (true);
+        }
+        break;
+    case OBJ_STAVES:
+        if (item.sub_type == STAFF_POISON)
+            return (true);
+        break;
+    default:
+        break;
+    }
+
+    return (false);
+}
+
 bool is_holy_discipline(int discipline)
 {
     return (discipline & SPTYP_HOLY);
@@ -374,6 +436,15 @@ bool is_unholy_spell(spell_type spell, god_type god)
     return (flags & SPFLAG_UNHOLY);
 }
 
+bool is_corpse_violating_spell(spell_type spell, god_type god)
+{
+    UNUSED(god);
+
+    unsigned int flags = get_spell_flags(spell);
+
+    return (flags & SPFLAG_CORPSE_VIOLATING);
+}
+
 bool is_evil_spell(spell_type spell, god_type god)
 {
     UNUSED(god);
@@ -410,15 +481,6 @@ bool is_hasty_spell(spell_type spell, god_type god)
     return (flags & SPFLAG_HASTY);
 }
 
-bool is_corpse_violating_spell(spell_type spell, god_type god)
-{
-    UNUSED(god);
-
-    unsigned int flags = get_spell_flags(spell);
-
-    return (flags & SPFLAG_CORPSE_VIOLATING);
-}
-
 // The default suitable() function for is_spellbook_type().
 bool is_any_spell(spell_type spell, god_type god)
 {
@@ -433,9 +495,7 @@ bool is_spellbook_type(const item_def& item, bool book_or_rod,
                        bool (*suitable)(spell_type spell, god_type god),
                        god_type god)
 {
-    const bool is_spellbook = (item.base_type == OBJ_BOOKS
-                                  && item.sub_type != BOOK_MANUAL
-                                  && item.sub_type != BOOK_DESTRUCTION);
+    const bool is_spellbook = item_is_spellbook(item);
     const bool is_rod = item_is_rod(item);
 
     if (!is_spellbook && !is_rod)
@@ -581,38 +641,9 @@ conduct_type god_hates_item_handling(const item_def &item)
         break;
 
     case GOD_SHINING_ONE:
-    {
-        if (!item_type_known(item))
-            return (DID_NOTHING);
-
-        switch (item.base_type)
-        {
-        case OBJ_WEAPONS:
-        {
-            const int item_brand = get_weapon_brand(item);
-            if (item_brand == SPWPN_VENOM)
-                return (DID_POISON);
-            break;
-        }
-
-        case OBJ_MISSILES:
-        {
-            const int item_brand = get_ammo_brand(item);
-            if (item_brand == SPMSL_POISONED || item_brand == SPMSL_CURARE)
-                return (DID_POISON);
-            break;
-        }
-
-        case OBJ_STAVES:
-            if (item.sub_type == STAFF_POISON)
-                return (DID_POISON);
-            break;
-
-        default:
-            break;
-        }
+        if (item_type_known(item) && is_poisoned_item(item))
+            return (DID_POISON);
         break;
-    }
 
     case GOD_YREDELEMNUL:
         if (item_type_known(item) && is_holy_item(item))
@@ -620,12 +651,8 @@ conduct_type god_hates_item_handling(const item_def &item)
         break;
 
     case GOD_TROG:
-        if (item.base_type == OBJ_BOOKS
-            && item.sub_type != BOOK_MANUAL
-            && item.sub_type != BOOK_DESTRUCTION)
-        {
+        if (item_is_spellbook(item))
             return (DID_SPELL_MEMORISE);
-        }
         break;
 
     case GOD_FEDHAS:
@@ -634,8 +661,11 @@ conduct_type god_hates_item_handling(const item_def &item)
         break;
 
     case GOD_CHEIBRIADOS:
-        if (item_type_known(item) && is_hasty_item(item))
+        if (item_type_known(item)
+            && (is_potentially_hasty_item(item) || is_hasty_item(item)))
+        {
             return (DID_HASTY);
+        }
         break;
 
     default:
