@@ -7,11 +7,14 @@
 #include "areas.h"
 #include "artefact.h"
 #include "coordit.h"
+#include "database.h"
 #include "effects.h"
 #include "env.h"
 #include "food.h"
 #include "fprop.h"
 #include "godabil.h"
+#include "goditem.h"
+#include "godpassive.h"
 #include "invent.h"
 #include "itemprop.h"
 #include "items.h"
@@ -31,6 +34,9 @@
 #include "terrain.h"
 #include "view.h"
 
+static bool _offer_items();
+static void _zin_donate_gold();
+
 static bool _confirm_pray_sacrifice(god_type god)
 {
     if (Options.stash_tracking == STM_EXPLICIT && is_stash(you.pos()))
@@ -42,7 +48,7 @@ static bool _confirm_pray_sacrifice(god_type god)
     for (stack_iterator si(you.pos(), true); si; ++si)
     {
         if (god_likes_item(god, *si)
-            && has_warning_inscription(*si, OPER_PRAY))
+            && needs_handle_warning(*si, OPER_PRAY))
         {
             std::string prompt = "Really sacrifice stack with ";
             prompt += si->name(DESC_NOCAP_A);
@@ -78,14 +84,6 @@ std::string god_prayer_reaction()
 
 bool god_accepts_prayer(god_type god)
 {
-    harm_protection_type hpt = god_protects_from_harm(god, false);
-
-    if (hpt == HPT_PRAYING || hpt == HPT_PRAYING_PLUS_ANYTIME
-        || hpt == HPT_RELIABLE_PRAYING_PLUS_ANYTIME)
-    {
-        return (true);
-    }
-
     if (god_likes_fresh_corpses(god))
         return (true);
 
@@ -94,17 +92,13 @@ bool god_accepts_prayer(god_type god)
     case GOD_ZIN:
         return (zin_sustenance(false));
 
-    case GOD_KIKUBAAQUDGHA:
-        return (you.piety >= piety_breakpoint(4));
-
-    case GOD_YREDELEMNUL:
-        return (yred_injury_mirror(false));
-
     case GOD_JIYVA:
-        return (jiyva_grant_jelly(false));
+        return (jiyva_can_paralyse_jellies());
 
+    case GOD_ELYVILON:
     case GOD_BEOGH:
     case GOD_NEMELEX_XOBEH:
+    case GOD_ASHENZARI:
         return (true);
 
     default:
@@ -118,10 +112,15 @@ static bool _bless_weapon(god_type god, brand_type brand, int colour)
 {
     item_def& wpn = *you.weapon();
 
-    if (is_artefact(wpn) || (is_range_weapon(wpn) && brand != SPWPN_HOLY_WRATH))
+    if (wpn.base_type != OBJ_WEAPONS
+        || (is_range_weapon(wpn) && brand != SPWPN_HOLY_WRATH)
+        || is_artefact(wpn))
+    {
         return (false);
+    }
 
-    std::string prompt = "Do you wish to have your weapon ";
+    std::string prompt = "Do you wish to have " + wpn.name(DESC_NOCAP_YOUR)
+                       + " ";
     if (brand == SPWPN_PAIN)
         prompt += "bloodied with pain";
     else if (brand == SPWPN_DISTORTION)
@@ -153,7 +152,7 @@ static bool _bless_weapon(god_type god, brand_type brand, int colour)
         enchant_weapon(ENCHANT_TO_DAM, true, wpn);
 
     if (is_cursed)
-        do_uncurse_item(wpn);
+        do_uncurse_item(wpn, false);
 
     if (god == GOD_SHINING_ONE)
     {
@@ -171,11 +170,14 @@ static bool _bless_weapon(god_type god, brand_type brand, int colour)
     {
         convert2bad(wpn);
 
+        wpn.flags &= ~ISFLAG_BLESSED_WEAPON;
+
         burden_change();
     }
 
     you.wield_change = true;
-    you.num_gifts[god]++;
+    you.num_current_gifts[god]++;
+    you.num_total_gifts[god]++;
     std::string desc  = old_name + " ";
             desc += (god == GOD_SHINING_ONE   ? "blessed by the Shining One" :
                      god == GOD_LUGONU        ? "corrupted by Lugonu" :
@@ -204,6 +206,8 @@ static bool _bless_weapon(god_type god, brand_type brand, int colour)
 
     if (god == GOD_KIKUBAAQUDGHA)
     {
+        you.gift_timeout = 0; // no protection during pain branding weapon
+
         torment(TORMENT_KIKUBAAQUDGHA, you.pos());
 
         // Bloodify surrounding squares (75% chance).
@@ -220,22 +224,24 @@ static bool _bless_weapon(god_type god, brand_type brand, int colour)
     return (true);
 }
 
+// Prayer at your god's altar.
 static bool _altar_prayer()
 {
     // Different message from when first joining a religion.
     mpr("You prostrate yourself in front of the altar and pray.");
 
-    if (you.religion == GOD_XOM)
-        return (false);
-
     god_acting gdact;
 
     bool did_bless = false;
 
+    // donate gold to gain piety distributed over time
+    if (you.religion == GOD_ZIN)
+        _zin_donate_gold();
+
     // TSO blesses weapons with holy wrath, and long blades and demon
     // whips specially.
     if (you.religion == GOD_SHINING_ONE
-        && !you.num_gifts[GOD_SHINING_ONE]
+        && !you.num_total_gifts[GOD_SHINING_ONE]
         && !player_under_penance()
         && you.piety > 160)
     {
@@ -252,7 +258,7 @@ static bool _altar_prayer()
 
     // Lugonu blesses weapons with distortion.
     if (you.religion == GOD_LUGONU
-        && !you.num_gifts[GOD_LUGONU]
+        && !you.num_total_gifts[GOD_LUGONU]
         && !player_under_penance()
         && you.piety > 160)
     {
@@ -264,7 +270,7 @@ static bool _altar_prayer()
 
     // Kikubaaqudgha blesses weapons with pain, or gives you a Necronomicon.
     if (you.religion == GOD_KIKUBAAQUDGHA
-        && !you.num_gifts[GOD_KIKUBAAQUDGHA]
+        && !you.num_total_gifts[GOD_KIKUBAAQUDGHA]
         && !player_under_penance()
         && you.piety > 160)
     {
@@ -305,7 +311,8 @@ static bool _altar_prayer()
                 simple_god_message(" grants you a gift!");
                 more();
 
-                you.num_gifts[you.religion]++;
+                you.num_current_gifts[you.religion]++;
+                you.num_total_gifts[you.religion]++;
                 did_bless = true;
                 take_note(Note(NOTE_GOD_GIFT, you.religion));
                 mitm[thing_created].inscription = "god gift";
@@ -316,8 +323,6 @@ static bool _altar_prayer()
         return (did_bless);
     }
 
-    offer_items();
-
     return (did_bless);
 }
 
@@ -329,10 +334,8 @@ void pray()
         return;
     }
 
-    // almost all prayers take time
-    you.turn_is_over = true;
-
-    const bool was_praying = !!you.duration[DUR_PRAYER];
+    // only successful prayer takes time
+    you.turn_is_over = false;
 
     bool something_happened = false;
     const god_type altar_god = feat_altar_god(grd(you.pos()));
@@ -340,7 +343,6 @@ void pray()
     {
         if (you.flight_mode() == FL_LEVITATE)
         {
-            you.turn_is_over = false;
             mpr("You are floating high above the altar.");
             return;
         }
@@ -348,17 +350,19 @@ void pray()
         if (you.religion != GOD_NO_GOD && altar_god == you.religion)
         {
             something_happened = _altar_prayer();
+            // at least "prostrating" took time
+            you.turn_is_over = true;
         }
         else if (altar_god != GOD_NO_GOD)
         {
             if (you.species == SP_DEMIGOD)
             {
-                you.turn_is_over = false;
                 mpr("Sorry, a being of your status cannot worship here.");
                 return;
             }
 
             god_pitch(feat_altar_god(grd(you.pos())));
+            you.turn_is_over = true;
             return;
         }
     }
@@ -370,84 +374,51 @@ void pray()
              you.is_undead ? "un" : "");
 
         // Zen meditation is timeless.
-        you.turn_is_over = false;
-        return;
-    }
-    else if (!god_accepts_prayer(you.religion))
-    {
-        if (!something_happened)
-        {
-            simple_god_message(" ignores your prayer.");
-            you.turn_is_over = false;
-        }
-        return;
-    }
-
-    // Beoghites and Nemelexites can abort now instead of offering
-    // something they don't want to lose.
-    if (altar_god == GOD_NO_GOD
-        && (you.religion == GOD_BEOGH ||  you.religion == GOD_NEMELEX_XOBEH)
-        && !_confirm_pray_sacrifice(you.religion))
-    {
-        you.turn_is_over = false;
         return;
     }
 
     mprf(MSGCH_PRAY, "You %s prayer to %s.",
-         was_praying ? "renew your" : "offer a",
+         you.duration[DUR_JELLY_PRAYER] ? "renew your" : "offer a",
          god_name(you.religion).c_str());
 
-    you.duration[DUR_PRAYER] = 9 + (random2(you.piety) / 20)
-                                 + (random2(you.piety) / 20);
+    switch(you.religion)
+    {
+    case GOD_ZIN:
+        //jmf: this "good" god will feed you (a la Nethack)
+        if (do_zin_sustenance())
+            something_happened = true;
+        break;
 
-    if (player_under_penance())
+    case GOD_JIYVA:
+        you.duration[DUR_JELLY_PRAYER] = 200;
+
+        if (jiyva_can_paralyse_jellies())
+            jiyva_paralyse_jellies();
+        something_happened = true;
+        break;
+
+    case GOD_FEDHAS:
+        if (fedhas_fungal_bloom())
+            something_happened = true;
+        break;
+
+    default:
+        ;
+    }
+
+    // All sacrifices affect items you're standing on.
+    something_happened |= _offer_items();
+
+    if (you.religion == GOD_XOM)
+        mpr(getSpeakString("Xom prayer"), MSGCH_GOD);
+    else if (player_under_penance())
         simple_god_message(" demands penance!");
     else
-    {
         mpr(god_prayer_reaction().c_str(), MSGCH_PRAY, you.religion);
 
-        if (you.piety > 130)
-            you.duration[DUR_PRAYER] *= 3;
-        else if (you.piety > 70)
-            you.duration[DUR_PRAYER] *= 2;
-    }
-
-    // Assume for now that gods who like fresh corpses and/or butchery
-    // don't use prayer for anything else.
-    if (you.religion == GOD_ZIN
-        || you.religion == GOD_KIKUBAAQUDGHA
-        || you.religion == GOD_BEOGH
-        || you.religion == GOD_NEMELEX_XOBEH
-        || you.religion == GOD_JIYVA
-        || god_likes_fresh_corpses(you.religion))
-    {
-        you.duration[DUR_PRAYER] = 1;
-    }
-    else if (you.religion == GOD_ELYVILON
-        || you.religion == GOD_YREDELEMNUL)
-    {
-        you.duration[DUR_PRAYER] = 20;
-    }
-
-    // Gods who like fresh corpses, Kikuites, Beoghites and Nemelexites
-    // offer the items they're standing on.
-    if (altar_god == GOD_NO_GOD
-        && (god_likes_fresh_corpses(you.religion)
-            || you.religion == GOD_BEOGH || you.religion == GOD_NEMELEX_XOBEH))
-    {
-        offer_items();
-    }
-
-    if (!was_praying)
-        do_god_gift(true);
-
-    dprf("piety: %d (-%d)", you.piety, you.piety_hysteresis );
-}
-
-void end_prayer(void)
-{
-    mpr("Your prayer is over.", MSGCH_PRAY, you.religion);
-    you.duration[DUR_PRAYER] = 0;
+    if (something_happened)
+        you.turn_is_over = true;
+    dprf("piety: %d (-%d)", you.piety, you.piety_hysteresis);
 }
 
 static int _gold_to_donation(int gold)
@@ -455,7 +426,7 @@ static int _gold_to_donation(int gold)
     return static_cast<int>((gold * log((float)gold)) / MAX_PIETY);
 }
 
-void _zin_donate_gold()
+static void _zin_donate_gold()
 {
     if (you.gold == 0)
     {
@@ -523,9 +494,9 @@ static int _leading_sacrifice_group()
     int weights[5];
     get_pure_deck_weights(weights);
     int best_i = -1, maxweight = -1;
-    for ( int i = 0; i < 5; ++i )
+    for (int i = 0; i < 5; ++i)
     {
-        if ( best_i == -1 || weights[i] > maxweight )
+        if (best_i == -1 || weights[i] > maxweight)
         {
             maxweight = weights[i];
             best_i = i;
@@ -536,7 +507,7 @@ static int _leading_sacrifice_group()
 
 static void _give_sac_group_feedback(int which)
 {
-    ASSERT( which >= 0 && which < 5 );
+    ASSERT(which >= 0 && which < 5);
     const char* names[] = {
         "Escape", "Destruction", "Dungeons", "Summoning", "Wonder"
     };
@@ -544,9 +515,56 @@ static void _give_sac_group_feedback(int which)
          names[which]);
 }
 
+static void _ashenzari_sac_scroll(const item_def& item)
+{
+    int scr = (you.species == SP_CAT) ? SCR_CURSE_JEWELLERY :
+              random_choose(SCR_CURSE_WEAPON, SCR_CURSE_ARMOUR,
+                            SCR_CURSE_JEWELLERY, -1);
+    int it = items(0, OBJ_SCROLLS, scr, true, 0, MAKE_ITEM_NO_RACE,
+                   0, 0, GOD_ASHENZARI);
+    if (it == NON_ITEM)
+    {
+        mpr("You feel the world is against you.");
+        return;
+    }
+
+    mitm[it].quantity = 1;
+    if (!move_item_to_grid(&it, you.pos(), true))
+        destroy_item(it, true); // can't happen
+}
+
+// Is the destroyed weapon valuable enough to gain piety by doing so?
+// Unholy and evil weapons are handled specially.
+static bool _destroyed_valuable_weapon(int value, int type)
+{
+    // Artefacts, including most randarts.
+    if (random2(value) >= random2(250))
+        return (true);
+
+    // Medium valuable items are more likely to net piety at low piety,
+    // more so for missiles, since they're worth less as single items.
+    if (random2(value) >= random2((type == OBJ_MISSILES) ? 10 : 100)
+        && one_chance_in(1 + you.piety / 50))
+    {
+        return (true);
+    }
+
+    // If not for the above, missiles shouldn't yield piety.
+    if (type == OBJ_MISSILES)
+        return (false);
+
+    // Weapons, on the other hand, are always acceptable to boost low
+    // piety.
+    if (you.piety < piety_breakpoint(0) || player_under_penance())
+        return (true);
+
+    return (false);
+}
+
 // God effects of sacrificing one item from a stack (e.g., a weapon, one
 // out of 20 arrows, etc.).  Does not modify the actual item in any way.
-static piety_gain_t _sacrifice_one_item_noncount(const item_def& item)
+static piety_gain_t _sacrifice_one_item_noncount(const item_def& item,
+       int *js, bool first)
 {
     piety_gain_t relative_piety_gain = PIETY_NONE;
 
@@ -554,20 +572,48 @@ static piety_gain_t _sacrifice_one_item_noncount(const item_def& item)
     //      item-accepting gods and corpse-accepting gods.
     if (god_likes_fresh_corpses(you.religion))
     {
+        gain_piety(13, 19);
+
+        // The feedback is not accurate any longer on purpose; it only reveals
+        // the rate you get piety at.
         if (x_chance_in_y(13, 19))
-        {
-            gain_piety(1);
             relative_piety_gain = PIETY_SOME;
-        }
     }
     else
     {
         switch (you.religion)
         {
-        case GOD_SHINING_ONE:
-            gain_piety(1);
-            relative_piety_gain = PIETY_SOME;
+        case GOD_ELYVILON:
+        {
+            const int value = item_value(item) / item.quantity;
+            const bool valuable_weapon =
+                _destroyed_valuable_weapon(value, item.base_type);
+            const bool unholy_weapon = is_unholy_item(item);
+            const bool evil_weapon = is_evil_item(item);
+
+            if (valuable_weapon || unholy_weapon || evil_weapon)
+            {
+                if (unholy_weapon || evil_weapon)
+                {
+                    const char *desc_weapon = evil_weapon ? "evil" : "unholy";
+
+                    // Print this in addition to the above!
+                    if (first)
+                    {
+                        simple_god_message(make_stringf(
+                                 " welcomes the destruction of %s %s weapon%s.",
+                                 item.quantity == 1 ? "this" : "these",
+                                 desc_weapon,
+                                 item.quantity == 1 ? ""     : "s").c_str(),
+                                 GOD_ELYVILON);
+                    }
+                }
+
+                gain_piety(1);
+                relative_piety_gain = PIETY_SOME;
+            }
             break;
+        }
 
         case GOD_BEOGH:
         {
@@ -587,11 +633,9 @@ static piety_gain_t _sacrifice_one_item_noncount(const item_def& item)
             else if (item.sub_type == CORPSE_SKELETON)
                 chance -= 2;
 
+            gain_piety(chance, 20);
             if (x_chance_in_y(chance, 20))
-            {
-                gain_piety(1);
                 relative_piety_gain = PIETY_SOME;
-            }
             break;
         }
 
@@ -603,27 +647,34 @@ static piety_gain_t _sacrifice_one_item_noncount(const item_def& item)
             if (you.attribute[ATTR_CARD_COUNTDOWN] && x_chance_in_y(value, 800))
             {
                 you.attribute[ATTR_CARD_COUNTDOWN]--;
-#if DEBUG_DIAGNOSTICS || DEBUG_CARDS || DEBUG_SACRIFICE
+#if defined(DEBUG_DIAGNOSTICS) || defined(DEBUG_CARDS) || defined(DEBUG_SACRIFICE)
                 mprf(MSGCH_DIAGNOSTICS, "Countdown down to %d",
                      you.attribute[ATTR_CARD_COUNTDOWN]);
 #endif
             }
             // Nemelex piety gain is fairly fast... at least when you
             // have low piety.
-            if (item.base_type == OBJ_CORPSES && one_chance_in(2 + you.piety/50)
-                || x_chance_in_y(value/2 + 1, 30 + you.piety/2))
+            int piety_change, piety_denom;
+            if (item.base_type == OBJ_CORPSES)
             {
-                if (is_artefact(item))
-                {
-                    gain_piety(2);
-                    relative_piety_gain = PIETY_LOTS;
-                }
-                else
-                {
-                    gain_piety(1);
-                    relative_piety_gain = PIETY_SOME;
-                }
+                piety_change = 1;
+                piety_denom = 2 + you.piety/50;
             }
+            else
+            {
+                piety_change = value/2 + 1;
+                if (is_artefact(item))
+                    piety_change *= 2;
+                piety_denom = 30 + you.piety/2;
+            }
+
+            gain_piety(piety_change, piety_denom);
+
+            // Preserving the old behaviour of giving the big message for
+            // artifacts and artifacts only.
+            relative_piety_gain = x_chance_in_y(piety_change, piety_denom) ?
+                                    is_artefact(item) ?
+                                      PIETY_LOTS : PIETY_SOME : PIETY_NONE;
 
             if (item.base_type == OBJ_FOOD && item.sub_type == FOOD_CHUNK
                 || is_blood_potion(item))
@@ -634,7 +685,7 @@ static piety_gain_t _sacrifice_one_item_noncount(const item_def& item)
             }
             else if (item.base_type == OBJ_CORPSES)
             {
-#if DEBUG_GIFTS || DEBUG_CARDS || DEBUG_SACRIFICE
+#if defined(DEBUG_GIFTS) || defined(DEBUG_CARDS) || defined(DEBUG_SACRIFICE)
                 mprf(MSGCH_DIAGNOSTICS, "Corpse mass is %d",
                      item_mass(item));
 #endif
@@ -651,14 +702,17 @@ static piety_gain_t _sacrifice_one_item_noncount(const item_def& item)
             const int value = item_value(item) / item.quantity;
             // compress into range 0..250
             const int stepped = stepdown_value(value, 50, 50, 200, 250);
-            const int gain = div_rand_round(stepped, 50);
-            if (gain > 0)
-            {
-                gain_piety(gain);
-                relative_piety_gain = (gain > 1) ? PIETY_LOTS : PIETY_SOME;
-            }
+            gain_piety(stepped, 50);
+            relative_piety_gain = (piety_gain_t)std::min(2,
+                                    div_rand_round(stepped, 50));
+            jiyva_slurp_bonus(div_rand_round(stepped, 50), js);
+            break;
         }
 
+        case GOD_ASHENZARI:
+            _ashenzari_sac_scroll(item);
+            break;
+
         default:
             break;
         }
@@ -667,12 +721,12 @@ static piety_gain_t _sacrifice_one_item_noncount(const item_def& item)
     return (relative_piety_gain);
 }
 
-piety_gain_t sacrifice_item_stack(const item_def& item)
+piety_gain_t sacrifice_item_stack(const item_def& item, int *js)
 {
     piety_gain_t relative_gain = PIETY_NONE;
     for (int j = 0; j < item.quantity; ++j)
     {
-        const piety_gain_t gain = _sacrifice_one_item_noncount(item);
+        const piety_gain_t gain = _sacrifice_one_item_noncount(item, js, !j);
 
         // Update piety gain if necessary.
         if (gain != PIETY_NONE)
@@ -686,33 +740,56 @@ piety_gain_t sacrifice_item_stack(const item_def& item)
     return (relative_gain);
 }
 
-void offer_items()
+static bool _check_nemelex_sacrificing_item_type(const item_def& item)
 {
-    if (you.religion == GOD_NO_GOD)
-        return;
+    switch (item.base_type)
+    {
+    case OBJ_ARMOUR:
+        return (you.nemelex_sacrificing[NEM_GIFT_ESCAPE]);
+
+    case OBJ_WEAPONS:
+    case OBJ_STAVES:
+    case OBJ_MISSILES:
+        return (you.nemelex_sacrificing[NEM_GIFT_DESTRUCTION]);
+
+    case OBJ_CORPSES:
+        return (you.nemelex_sacrificing[NEM_GIFT_SUMMONING]);
+
+    case OBJ_POTIONS:
+        if (is_blood_potion(item))
+            return (you.nemelex_sacrificing[NEM_GIFT_SUMMONING]);
+        return (you.nemelex_sacrificing[NEM_GIFT_WONDERS]);
+
+    case OBJ_FOOD:
+        if (item.sub_type == FOOD_CHUNK)
+            return (you.nemelex_sacrificing[NEM_GIFT_SUMMONING]);
+    // else fall through
+    case OBJ_WANDS:
+    case OBJ_SCROLLS:
+        return (you.nemelex_sacrificing[NEM_GIFT_WONDERS]);
+
+    case OBJ_JEWELLERY:
+    case OBJ_BOOKS:
+    case OBJ_MISCELLANY:
+        return (you.nemelex_sacrificing[NEM_GIFT_DUNGEONS]);
+
+    default:
+        return false;
+    }
+}
+
+static bool _offer_items()
+{
+    if (you.religion == GOD_NO_GOD || !god_likes_items(you.religion))
+        return false;
+
+    if (!_confirm_pray_sacrifice(you.religion))
+        return false;
 
     int i = you.visible_igrd(you.pos());
 
-    if (!god_likes_items(you.religion) && i != NON_ITEM)
-    {
-        simple_god_message(" doesn't care about such mundane gifts.",
-                           you.religion);
-        return;
-    }
-
     god_acting gdact;
 
-    // donate gold to gain piety distributed over time
-    if (you.religion == GOD_ZIN)
-    {
-        _zin_donate_gold();
-
-        return; // doesn't accept anything else for sacrifice
-    }
-
-    if (i == NON_ITEM) // nothing to sacrifice
-        return;
-
     int num_sacced = 0;
     int num_disliked = 0;
     item_def *disliked_item = 0;
@@ -736,6 +813,14 @@ void offer_items()
             continue;
         }
 
+        // Skip items you don't want to sacrifice right now.
+        if (you.religion == GOD_NEMELEX_XOBEH
+            && !_check_nemelex_sacrificing_item_type(item))
+        {
+            i = next;
+            continue;
+        }
+
         // Ignore {!D} inscribed items.
         if (!check_warning_inscriptions(item, OPER_DESTROY))
         {
@@ -758,7 +843,7 @@ void offer_items()
         }
 
 
-#if DEBUG_DIAGNOSTICS || DEBUG_SACRIFICE
+#if defined(DEBUG_DIAGNOSTICS) || defined(DEBUG_SACRIFICE)
         mprf(MSGCH_DIAGNOSTICS, "Sacrifice item value: %d",
              item_value(item));
 #endif
@@ -777,26 +862,19 @@ void offer_items()
         if (old_leading != new_leading || one_chance_in(50))
             _give_sac_group_feedback(new_leading);
 
-#if DEBUG_GIFTS || DEBUG_CARDS || DEBUG_SACRIFICE
+#if defined(DEBUG_GIFTS) || defined(DEBUG_CARDS) || defined(DEBUG_SACRIFICE)
         _show_pure_deck_chances();
 #endif
     }
 
-    if (num_sacced > 0 && you.religion == GOD_KIKUBAAQUDGHA)
-    {
-        simple_god_message(" torments the living!");
-        torment(TORMENT_KIKUBAAQUDGHA, you.pos());
-        lose_piety(random_range(8, 12));
-    }
-
     // Explanatory messages if nothing the god likes is sacrificed.
     else if (num_sacced == 0 && num_disliked > 0)
     {
         ASSERT(disliked_item);
 
-        if (disliked_item->base_type == OBJ_ORBS)
+        if (item_is_orb(*disliked_item))
             simple_god_message(" wants the Orb's power used on the surface!");
-        else if (is_rune(*disliked_item))
+        else if (item_is_rune(*disliked_item))
             simple_god_message(" wants the runes to be proudly displayed.");
         else if (disliked_item->base_type == OBJ_MISCELLANY
                  && disliked_item->sub_type == MISC_HORN_OF_GERYON)
@@ -805,11 +883,18 @@ void offer_items()
         // sacrifices.
         else if (god_likes_fresh_corpses(you.religion))
             simple_god_message(" only cares about fresh corpses!");
-        else if (you.religion == GOD_SHINING_ONE)
-            simple_god_message(" only cares about unholy and evil items!");
         else if (you.religion == GOD_BEOGH)
             simple_god_message(" only cares about orcish remains!");
         else if (you.religion == GOD_NEMELEX_XOBEH)
-            simple_god_message(" expects you to use your decks, not offer them!");
+            if (disliked_item->base_type == OBJ_GOLD)
+                simple_god_message(" does not care about gold!");
+            else
+                simple_god_message(" expects you to use your decks, not offer them!");
+        else if (you.religion == GOD_ASHENZARI)
+            simple_god_message(" can corrupt only scrolls of remove curse.");
     }
+    if (num_sacced == 0 && you.religion == GOD_ELYVILON)
+        mpr("There are no weapons here to destroy!");
+
+    return (num_sacced > 0);
 }
