@@ -1,7 +1,7 @@
-/*
- *  File:       godwrath.cc
- *  Summary:    Divine retribution.
- */
+/**
+ * @file
+ * @brief Divine retribution.
+**/
 
 #include "AppHdr.h"
 
@@ -19,7 +19,6 @@
 #include "food.h"
 #include "ghost.h"
 #include "godabil.h"
-#include "it_use2.h"
 #include "libutil.h"
 #include "message.h"
 #include "misc.h"
@@ -32,12 +31,15 @@
 #include "mon-stuff.h"
 #include "mutation.h"
 #include "ouch.h"
+#include "player-stats.h"
+#include "potion.h"
 #include "religion.h"
-#include "spells1.h"
-#include "spells2.h"
-#include "spells3.h"
-#include "spells4.h"
-#include "spl-mis.h"
+#include "spl-clouds.h"
+#include "spl-goditem.h"
+#include "spl-miscast.h"
+#include "spl-selfench.h"
+#include "spl-summoning.h"
+#include "spl-transloc.h"
 #include "stash.h"
 #include "state.h"
 #include "transform.h"
@@ -46,6 +48,8 @@
 
 #include <sstream>
 
+#define random_mons(...) static_cast<monster_type>(random_choose(__VA_ARGS__))
+
 static void _god_smites_you(god_type god, const char *message = NULL,
                             kill_method_type death_type = NUM_KILLBY);
 static bool _beogh_idol_revenge();
@@ -68,9 +72,13 @@ static bool _yred_random_zombified_hostile()
     else
         z_type = skel ? MONS_SKELETON_SMALL : MONS_ZOMBIE_SMALL;
 
-    return (create_monster(mgen_data::hostile_at(z_type,
-                    "the anger of Yredelemnul", true,
-                    0, 0, you.pos(), 0, GOD_YREDELEMNUL, z_base)) != -1);
+    mgen_data temp = mgen_data::hostile_at(z_type, "the anger of Yredelemnul",
+                                           true, 0, 0, you.pos(), 0,
+                                           GOD_YREDELEMNUL, z_base);
+
+    temp.extra_flags |= (MF_NO_REWARD | MF_HARD_RESET);
+
+    return (create_monster(temp, false) != -1);
 }
 
 static bool _okawaru_random_servant()
@@ -90,10 +98,13 @@ static bool _okawaru_random_servant()
                 (temp_rand < 95) ? MONS_HILL_GIANT         //  5%
                                  : MONS_TITAN);            //  5%
 
-    return (create_monster(
-                mgen_data::hostile_at(mon_type, "the fury of Okawaru",
-                                      true, 6, MON_SUMM_WRATH, you.pos(), 0,
-                                      GOD_OKAWARU)) != -1);
+    mgen_data temp = mgen_data::hostile_at(mon_type, "the fury of Okawaru",
+                                           true, 0, 0, you.pos(), 0,
+                                           GOD_OKAWARU);
+
+    temp.extra_flags |= (MF_NO_REWARD | MF_HARD_RESET);
+
+    return (create_monster(temp, false) != -1);
 }
 
 static bool _tso_retribution()
@@ -233,7 +244,7 @@ static void _ely_dull_inventory_weapons()
 
     for (int i = 0; i < ENDOFPACK; ++i)
     {
-        if (!you.inv[i].is_valid())
+        if (!you.inv[i].defined())
             continue;
 
         if (you.inv[i].base_type == OBJ_WEAPONS
@@ -329,29 +340,77 @@ static bool _cheibriados_retribution()
 {
     // time god/slowness theme
     const god_type god = GOD_CHEIBRIADOS;
-    simple_god_message(" bends time around you.", god);
-    switch (random2(5))
+
+    // Chei retribution might only make sense in combat.
+    // We can crib some Xom code for this. {bh}
+    int tension = get_tension(GOD_CHEIBRIADOS);
+    int wrath_value = random2(tension);
+
+    bool glammer = false;
+
+    // Determine the level of wrath
+    int wrath_type = 0;
+    if (wrath_value < 2)       { wrath_type = 0; }
+    else if (wrath_value < 4)  { wrath_type = 1; }
+    else if (wrath_value < 8)  { wrath_type = 2; }
+    else if (wrath_value < 16) { wrath_type = 3; }
+    else                       { wrath_type = 4; }
+
+    // Strip away extra speed
+    if (one_chance_in(5 - wrath_type))
+        dec_haste_player(10000);
+
+    // Chance to be overwhelmed by the divine experience
+    if (one_chance_in(5 - wrath_type))
+        glammer = true;
+
+    switch(wrath_type)
     {
-    case 0:
-    case 1:
-    case 2:
+    // Very high tension wrath
+    case 4:
+        simple_god_message(" adjusts the clock.", god);
+        MiscastEffect(&you, -god, SPTYP_RANDOM, 8, 90,
+                      "the meddling of Cheibriados");
+        if (one_chance_in(wrath_type - 1))
+            break;
+    // High tension wrath
     case 3:
         mpr("You lose track of time.");
-        you.put_to_sleep(NULL, 50);
-        break;
-
-    case 4:
+        you.put_to_sleep(NULL, 30 + random2(20));
+        dec_penance(god, 1);
+        if (one_chance_in(wrath_type - 2))
+            break;
+    // Medium tension
+    case 2:
         if (you.duration[DUR_SLOW] < 180 * BASELINE_DELAY)
         {
-            dec_penance(god, 1);
             mpr("You feel the world leave you behind!", MSGCH_WARN);
             you.set_duration(DUR_EXHAUSTED, 200);
             slow_player(100);
         }
+
+        if (one_chance_in(wrath_type - 2))
+            break;
+    // Low tension
+    case 1:
+        mpr("Time shudders.");
+        cheibriados_time_step(2+random2(4));
+        if (one_chance_in(3))
+            break;
+    // No tension wrath.
+    case 0:
+        if (curse_an_item(true, false))
+            simple_god_message(" makes up for lost time.", god);
+        else
+            glammer = true;
+
+    default:
         break;
     }
 
-    return (true);
+    if (wrath_type > 2)
+        dec_penance(god, 1 + random2(wrath_type));
+    return (glammer);
 }
 
 static bool _makhleb_retribution()
@@ -361,12 +420,15 @@ static bool _makhleb_retribution()
 
     if (random2(you.experience_level) > 7 && !one_chance_in(5))
     {
-        bool success = (create_monster(
-                           mgen_data::hostile_at(
-                               static_cast<monster_type>(
-                                   MONS_EXECUTIONER + random2(5)),
-                               "the fury of Makhleb",
-                               true, 0, 0, you.pos(), 0, god)) != -1);
+        mgen_data temp =
+            mgen_data::hostile_at(random_mons(MONS_EXECUTIONER, MONS_GREEN_DEATH,
+                                  MONS_BLUE_DEATH, MONS_BALRUG, MONS_CACODEMON, -1),
+                                  "the fury of Makhleb",
+                                  true, 0, 0, you.pos(), 0, god);
+
+        temp.extra_flags |= (MF_NO_REWARD | MF_HARD_RESET);
+
+        bool success = create_monster(temp, false) != -1;
 
         simple_god_message(success ? " sends a greater servant after you!"
                                    : "'s greater servant is unavoidably "
@@ -379,12 +441,16 @@ static bool _makhleb_retribution()
 
         for (; how_many > 0; --how_many)
         {
-            if (create_monster(
-                    mgen_data::hostile_at(
-                        static_cast<monster_type>(
-                            MONS_NEQOXEC + random2(5)),
-                        "the fury of Makhleb",
-                        true, 0, 0, you.pos(), 0, god)) != -1)
+            mgen_data temp =
+                mgen_data::hostile_at(random_mons(MONS_HELLWING, MONS_NEQOXEC,
+                                      MONS_ORANGE_DEMON, MONS_SMOKE_DEMON,
+                                      MONS_YNOXINUL, -1),
+                                      "the fury of Makhleb",
+                                      true, 0, 0, you.pos(), 0, god);
+
+            temp.extra_flags |= (MF_NO_REWARD | MF_HARD_RESET);
+
+            if (create_monster(temp, false) != -1)
             {
                 count++;
             }
@@ -409,10 +475,10 @@ static bool _kikubaaqudgha_retribution()
     if (random2(you.experience_level) > 4)
     {
         // Either zombies, or corpse rot + skeletons.
-        receive_corpses(you.experience_level * 4, you.pos());
+        kiku_receive_corpses(you.experience_level * 4, you.pos());
 
         if (coinflip())
-            corpse_rot();
+            corpse_rot(&you);
     }
 
     if (coinflip())
@@ -476,7 +542,7 @@ static bool _yredelemnul_retribution()
                         count++;
                 }
                 else
-                    count += yred_random_servants(100, true);
+                    count += yred_random_servants(0, true);
             }
 
             simple_god_message(count > 1 ? " sends servants to punish you." :
@@ -520,7 +586,7 @@ static bool _trog_retribution()
 
                 points -= cost;
 
-                if (summon_berserker(cost * 20, god, 0, true))
+                if (summon_berserker(cost * 20, 0))
                     count++;
             }
         }
@@ -546,7 +612,7 @@ static bool _trog_retribution()
 
         case 1:
         case 2:
-            lose_stat(STAT_STRENGTH, 1 + random2(you.strength / 5), true,
+            lose_stat(STAT_STR, 1 + random2(you.strength() / 5), true,
                       "divine retribution from Trog");
             break;
 
@@ -554,7 +620,7 @@ static bool _trog_retribution()
             if (!you.duration[DUR_PARALYSIS])
             {
                 dec_penance(god, 3);
-                mpr( "You suddenly pass out!", MSGCH_WARN );
+                mpr("You suddenly pass out!", MSGCH_WARN);
                 you.duration[DUR_PARALYSIS] = 2 + random2(6);
             }
             break;
@@ -564,7 +630,7 @@ static bool _trog_retribution()
             if (you.duration[DUR_SLOW] < 180 * BASELINE_DELAY)
             {
                 dec_penance(god, 1);
-                mpr( "You suddenly feel exhausted!", MSGCH_WARN );
+                mpr("You suddenly feel exhausted!", MSGCH_WARN);
                 you.set_duration(DUR_EXHAUSTED, 200);
                 slow_player(100);
             }
@@ -657,8 +723,7 @@ static bool _beogh_retribution()
 
                 item_colour(wpn);
 
-                if (coinflip())
-                    menv[mon].flags |= MF_HARD_RESET;
+                menv[mon].flags |= (MF_NO_REWARD | MF_HARD_RESET);
 
                 ghost_demon newstats;
                 newstats.init_dancing_weapon(wpn,
@@ -705,10 +770,13 @@ static bool _beogh_retribution()
         else
             punisher = MONS_ORC;
 
-        int mons = create_monster(
-                       mgen_data::hostile_at(punisher,
-                           "the wrath of Beogh",
-                           true, 0, 0, you.pos(), MG_PERMIT_BANDS, god));
+        mgen_data temp = mgen_data::hostile_at(punisher, "the wrath of Beogh",
+                                               true, 0, 0, you.pos(),
+                                               MG_PERMIT_BANDS, god);
+
+        temp.extra_flags |= (MF_NO_REWARD | MF_HARD_RESET);
+
+        int mons = create_monster(temp, false);
 
         // sometimes name band leader
         if (mons != -1 && one_chance_in(3))
@@ -752,7 +820,7 @@ static bool _sif_muna_retribution()
     {
     case 0:
     case 1:
-        lose_stat(STAT_INTELLIGENCE, 1 + random2(you.intel / 5), true,
+        lose_stat(STAT_INT, 1 + random2(you.intel() / 5), true,
                   "divine retribution from Sif Muna");
         break;
 
@@ -769,6 +837,10 @@ static bool _sif_muna_retribution()
         break;
 
     case 7:
+        if (!forget_spell())
+            mpr("You get a splitting headache.");
+                break;
+
     case 8:
         if (you.magic_points > 0)
         {
@@ -782,7 +854,7 @@ static bool _sif_muna_retribution()
         // a duration of one round, thus potentially exposing
         // the player to real danger.
         antimagic();
-        mpr("You sense a dampening of magic.", MSGCH_WARN);
+        mpr("Your magical effects suddenly unravel.", MSGCH_WARN);
         break;
     }
 
@@ -815,12 +887,15 @@ static bool _lugonu_retribution()
 
     if (random2(you.experience_level) > 7 && !one_chance_in(5))
     {
-        bool success = (create_monster(
-                           mgen_data::hostile_at(
-                               static_cast<monster_type>(
-                                   MONS_GREEN_DEATH + random2(3)),
-                               "the touch of Lugonu",
-                               true, 0, 0, you.pos(), 0, god)) != -1);
+        mgen_data temp =
+            mgen_data::hostile_at(random_mons(MONS_GREEN_DEATH,
+                                  MONS_BLUE_DEATH, MONS_BALRUG, -1),
+                                  "the touch of Lugonu",
+                                  true, 0, 0, you.pos(), 0, god);
+
+        temp.extra_flags |= (MF_NO_REWARD | MF_HARD_RESET);
+
+        bool success = create_monster(temp, false) != -1;
 
         simple_god_message(success ? " sends a demon after you!"
                                    : "'s demon is unavoidably detained.", god);
@@ -832,12 +907,16 @@ static bool _lugonu_retribution()
 
         for (; how_many > 0; --how_many)
         {
-            if (create_monster(
-                   mgen_data::hostile_at(
-                       static_cast<monster_type>(
-                           MONS_NEQOXEC + random2(5)),
-                       "the touch of Lugonu",
-                       true, 0, 0, you.pos(), 0, god)) != -1)
+            mgen_data temp =
+                mgen_data::hostile_at(random_mons(MONS_HELLWING, MONS_NEQOXEC,
+                                      MONS_ORANGE_DEMON, MONS_SMOKE_DEMON,
+                                      MONS_YNOXINUL, -1),
+                                      "the touch of Lugonu",
+                                      true, 0, 0, you.pos(), 0, god);
+
+            temp.extra_flags |= (MF_NO_REWARD | MF_HARD_RESET);
+
+            if (create_monster(temp, false) != -1)
             {
                 success = true;
             }
@@ -890,7 +969,7 @@ static bool _jiyva_retribution()
     {
         int tries = 0;
         bool found_one = false;
-        monsters *mon;
+        monster* mon;
 
         while (tries < 10)
         {
@@ -960,10 +1039,14 @@ static bool _jiyva_retribution()
         {
             const monster_type slime = RANDOM_ELEMENT(slimes);
 
-            if (create_monster(
-                    mgen_data::hostile_at(static_cast<monster_type>(slime),
-                        "the vengeance of Jiyva",
-                        true, 0, 0, you.pos(), 0, god)) != -1)
+            mgen_data temp =
+                mgen_data::hostile_at(static_cast<monster_type>(slime),
+                                      "the vengeance of Jiyva",
+                                      true, 0, 0, you.pos(), 0, god);
+
+            temp.extra_flags |= (MF_NO_REWARD | MF_HARD_RESET);
+
+            if (create_monster(temp, false) != -1)
             {
                 success = true;
             }
@@ -989,11 +1072,12 @@ static bool _fedhas_retribution()
     case 0:
         // Try and spawn some hostile giant spores, if none are created
         // fall through to the elemental miscast effects.
-        if (corpse_spores(BEH_HOSTILE))
+        if (fedhas_corpse_spores(BEH_HOSTILE, false))
         {
             simple_god_message(" produces spores.", GOD_FEDHAS);
             break;
         }
+
     case 1:
     {
         // Elemental miscast effects.
@@ -1019,7 +1103,11 @@ static bool _fedhas_retribution()
                       random2avg(88, 3), "the enmity of Fedhas Madash");
         break;
     }
+
     case 2:
+    {
+        bool success = false;
+
         // We are going to spawn some oklobs but first we need to find
         // out a little about the situation.
         std::vector<std::vector<coord_def> > radius_points;
@@ -1040,14 +1128,17 @@ static bool _fedhas_retribution()
             }
         }
 
-        mgen_data temp(MONS_OKLOB_PLANT,
-                       BEH_HOSTILE, 0, 0, 0,
-                       coord_def(),
-                       MHITNOT,
-                       MG_FORCE_PLACE,
-                       GOD_FEDHAS);
+        mgen_data temp =
+            mgen_data::hostile_at(MONS_OKLOB_PLANT,
+                                  "the enmity of Fedhas Madash",
+                                  false,
+                                  0,
+                                  0,
+                                  coord_def(-1, -1),
+                                  MG_FORCE_PLACE,
+                                  GOD_FEDHAS);
 
-        temp.non_actor_summoner = "the enmity of Fedhas Madash";
+       temp.extra_flags |= (MF_NO_REWARD | MF_HARD_RESET);
 
         // If we have a lot of space to work with we can do something
         // flashy.
@@ -1063,6 +1154,9 @@ static bool _fedhas_retribution()
                        1, radius_points[0].size(),
                        seen_count);
 
+            if (seen_count > 0)
+                success = true;
+
             temp.cls = MONS_OKLOB_PLANT;
 
             place_ring(radius_points[max_idx],
@@ -1070,6 +1164,9 @@ static bool _fedhas_retribution()
                        temp,
                        random_range(3, 8), 1,
                        seen_count);
+
+            if (seen_count > 0)
+                success = true;
         }
         // Otherwise we do something with the nearest neighbors
         // (assuming the player isn't already surrounded).
@@ -1089,17 +1186,25 @@ static bool _fedhas_retribution()
                 temp.cls = coinflip() ?
                            MONS_WANDERING_MUSHROOM : MONS_OKLOB_PLANT;
 
-                create_monster(temp,false);
+                if (create_monster(temp, false) != -1)
+                    success = true;
             }
+        }
 
+        if (success)
+        {
             god_speaks(god, "Plants grow around you in an ominous manner.");
             return (false);
         }
+
+        break;
     }
+    }
+
     return (true);
 }
 
-bool divine_retribution(god_type god, bool no_bonus)
+bool divine_retribution(god_type god, bool no_bonus, bool force)
 {
     ASSERT(god != GOD_NO_GOD);
 
@@ -1109,8 +1214,8 @@ bool divine_retribution(god_type god, bool no_bonus)
     // Good gods don't use divine retribution on their followers, and
     // gods don't use divine retribution on followers of gods they don't
     // hate.
-    if ((god == you.religion && is_good_god(god))
-        || (god != you.religion && !god_hates_your_god(god)))
+    if (!force && ((god == you.religion && is_good_god(god))
+        || (god != you.religion && !god_hates_your_god(god))))
     {
         return (false);
     }
@@ -1118,7 +1223,6 @@ bool divine_retribution(god_type god, bool no_bonus)
     god_acting gdact(god, true);
 
     bool do_more    = true;
-    bool did_retrib = true;
     switch (god)
     {
     // One in ten chance that Xom might do something good...
@@ -1137,36 +1241,39 @@ bool divine_retribution(god_type god, bool no_bonus)
     case GOD_SIF_MUNA:      do_more = _sif_muna_retribution(); break;
     case GOD_ELYVILON:      do_more = _elyvilon_retribution(); break;
     case GOD_JIYVA:         do_more = _jiyva_retribution(); break;
-    case GOD_FEDHAS:         do_more = _fedhas_retribution(); break;
+    case GOD_FEDHAS:        do_more = _fedhas_retribution(); break;
     case GOD_CHEIBRIADOS:   do_more = _cheibriados_retribution(); break;
 
+    case GOD_ASHENZARI:
+        // No reduction with time.
+        return (false);
+
     default:
-#if DEBUG_DIAGNOSTICS || DEBUG_RELIGION
+#if defined(DEBUG_DIAGNOSTICS) || defined(DEBUG_RELIGION)
         mprf(MSGCH_DIAGNOSTICS, "No retribution defined for %s.",
              god_name(god).c_str());
 #endif
         do_more    = false;
-        did_retrib = false;
-        break;
+        return (false);
     }
 
     if (no_bonus)
-        return (did_retrib);
+        return (true);
 
     // Sometimes divine experiences are overwhelming...
     if (do_more && one_chance_in(5) && you.experience_level < random2(37))
     {
         if (coinflip())
         {
-            mpr( "The divine experience confuses you!", MSGCH_WARN);
+            mpr("The divine experience confuses you!", MSGCH_WARN);
             confuse_player(3 + random2(10));
         }
         else
         {
             if (you.duration[DUR_SLOW] < 180 * BASELINE_DELAY)
             {
-                mpr( "The divine experience leaves you feeling exhausted!",
-                     MSGCH_WARN );
+                mpr("The divine experience leaves you feeling exhausted!",
+                     MSGCH_WARN);
 
                 slow_player(random2(20));
             }
@@ -1177,7 +1284,7 @@ bool divine_retribution(god_type god, bool no_bonus)
     // point...the punishment might have reduced penance further.
     dec_penance(god, 1 + random2(3));
 
-    return (did_retrib);
+    return (true);
 }
 
 bool do_god_revenge(conduct_type thing_done)
@@ -1334,7 +1441,7 @@ static void _god_smites_you(god_type god, const char *message,
         if (death_type != KILLED_BY_BEOGH_SMITING
             && death_type != KILLED_BY_TSO_SMITING)
         {
-            aux = "smote by " + god_name(god);
+            aux = "smitten by " + god_name(god);
         }
 
         // If there's a message, display it before smiting.
@@ -1351,3 +1458,21 @@ static void _god_smites_you(god_type god, const char *message,
         dec_penance(god, 1);
     }
 }
+
+int ash_reduce_xp(int amount)
+{
+    if (!you.penance[GOD_ASHENZARI] || !you.exp_docked_total)
+        return 0;
+
+    int lost = std::min(amount / 2, you.exp_docked);
+    you.exp_docked -= lost;
+    dprf("Ashenzari: exp stolen: %d, %d/%d to go", lost, you.exp_docked,
+         you.exp_docked_total);
+
+    int new_pen = (((int64_t)you.exp_docked * 50) + you.exp_docked_total - 1)
+                / you.exp_docked_total;
+    if (new_pen < you.penance[GOD_ASHENZARI])
+        dec_penance(GOD_ASHENZARI, you.penance[GOD_ASHENZARI] - new_pen);
+
+    return lost;
+}
