@@ -1,8 +1,7 @@
-/*
- *  File:       it_use3.cc
- *  Summary:    Functions for using some of the wackier inventory items.
- *  Written by: Linley Henzell
- */
+/**
+ * @file
+ * @brief Functions for using some of the wackier inventory items.
+**/
 
 #include "AppHdr.h"
 
@@ -23,6 +22,7 @@
 #include "directn.h"
 #include "effects.h"
 #include "env.h"
+#include "exercise.h"
 #include "fight.h"
 #include "food.h"
 #include "invent.h"
@@ -38,16 +38,18 @@
 #include "coord.h"
 #include "misc.h"
 #include "player.h"
+#include "player-stats.h"
 #include "religion.h"
 #include "godconduct.h"
 #include "skills.h"
 #include "skills2.h"
-#include "spells1.h"
-#include "spells2.h"
 #include "spl-book.h"
 #include "spl-cast.h"
+#include "spl-clouds.h"
+#include "spl-summoning.h"
 #include "state.h"
 #include "stuff.h"
+#include "terrain.h"
 #include "areas.h"
 #include "view.h"
 #include "shout.h"
@@ -150,6 +152,7 @@ void noisy_equipment()
 
 void shadow_lantern_effect()
 {
+    // Currently only mummies and lich form get more shadows.
     if (x_chance_in_y(player_spec_death() + 1, 8))
     {
         create_monster(mgen_data(MONS_SHADOW, BEH_FRIENDLY, &you, 2, 0,
@@ -160,7 +163,7 @@ void shadow_lantern_effect()
         // This should only get called when we are wielding a lantern of
         // shadows.
         ASSERT(lantern && lantern->base_type == OBJ_MISCELLANY
-                && lantern->sub_type == MISC_LANTERN_OF_SHADOWS);
+               && lantern->sub_type == MISC_LANTERN_OF_SHADOWS);
 
         bool known = fully_identified(*lantern);
         did_god_conduct(DID_NECROMANCY, 1, known);
@@ -206,6 +209,8 @@ static bool _reaching_weapon_attack(const item_def& wpn)
     args.mode = TARG_HOSTILE;
     args.range = 2;
     args.top_prompt = "Attack whom?";
+    targetter_reach hitfunc(&you, REACH_TWO);
+    args.hitfunc = &hitfunc;
 
     direction(beam, args);
 
@@ -221,7 +226,10 @@ static bool _reaching_weapon_attack(const item_def& wpn)
     const coord_def delta = beam.target - you.pos();
     const int x_distance  = abs(delta.x);
     const int y_distance  = abs(delta.y);
-    monsters* mons = monster_at(beam.target);
+    monster* mons = monster_at(beam.target);
+    // don't allow targeting of submerged trapdoor spiders
+    if (mons && mons->submerged() && feat_is_floor(grd(beam.target)))
+        mons = NULL;
 
     const int x_middle = std::max(beam.target.x, you.pos().x)
                             - (x_distance / 2);
@@ -263,7 +271,7 @@ static bool _reaching_weapon_attack(const item_def& wpn)
         if ((beam.target.x == you.pos().x || beam.target.y == you.pos().y)
             && monster_at(middle))
         {
-            const int skill = weapon_skill( wpn.base_type, wpn.sub_type );
+            const int skill = weapon_skill(wpn.base_type, wpn.sub_type);
 
             if (x_chance_in_y(5 + (3 * skill), 40))
             {
@@ -285,7 +293,7 @@ static bool _reaching_weapon_attack(const item_def& wpn)
         if (success)
         {
             // Monster might have died or gone away.
-            if (monsters* m = monster_at(beam.target))
+            if (monster* m = monster_at(beam.target))
                 if (mons_is_mimic(m->type))
                     mimic_alert(m);
         }
@@ -301,10 +309,21 @@ static bool _evoke_horn_of_geryon(item_def &item)
     // Note: This assumes that the Vestibule has not been changed.
     bool rc = false;
 
-    if (player_in_branch( BRANCH_VESTIBULE_OF_HELL ))
+    if (silenced(you.pos()))
+    {
+        mpr("You can't produce a sound!");
+        return false;
+    }
+    else if (player_in_branch(BRANCH_VESTIBULE_OF_HELL))
     {
         mpr("You produce a weird and mournful sound.");
 
+        if (you.char_direction == GDT_ASCENDING)
+        {
+            mpr("But nothing happens...");
+            return false;
+        }
+
         for (int count_x = 0; count_x < GXM; count_x++)
             for (int count_y = 0; count_y < GYM; count_y++)
             {
@@ -356,22 +375,22 @@ static bool _evoke_horn_of_geryon(item_def &item)
 
 static bool _efreet_flask(int slot)
 {
-    bool friendly = x_chance_in_y(10 + you.skills[SK_EVOCATIONS] / 3, 20);
+    bool friendly = x_chance_in_y(10 + you.skill(SK_EVOCATIONS) / 3, 20);
 
     mpr("You open the flask...");
 
-    const int monster =
+    const int mons =
         create_monster(
             mgen_data(MONS_EFREET,
                       friendly ? BEH_FRIENDLY : BEH_HOSTILE,
                       &you, 0, 0, you.pos(),
                       MHITYOU, MG_FORCE_BEH));
 
-    if (monster != -1)
+    if (mons != -1)
     {
         mpr("...and a huge efreet comes out.");
 
-        if (player_angers_monster(&menv[monster]))
+        if (player_angers_monster(&menv[mons]))
             friendly = false;
 
         if (silenced(you.pos()))
@@ -393,64 +412,107 @@ static bool _efreet_flask(int slot)
     return (true);
 }
 
+static bool _is_crystal_ball(const item_def &item)
+{
+    return (item.base_type == OBJ_MISCELLANY
+            && (item.sub_type == MISC_CRYSTAL_BALL_OF_ENERGY
+                || item.sub_type == MISC_CRYSTAL_BALL_OF_SEEING));
+}
+
+static bool _check_crystal_ball(int subtype, bool known)
+{
+    if (you.intel() <= 1)
+    {
+        mpr("You lack the intelligence to focus on the shapes in the ball.");
+        return (false);
+    }
+
+    if (you.confused())
+    {
+        mpr("You are unable to concentrate on the shapes in the ball.");
+        return (false);
+    }
+
+    if (subtype == MISC_CRYSTAL_BALL_OF_ENERGY
+        && known
+        && you.magic_points == you.max_magic_points)
+    {
+        mpr("With no energy to recover, the crystal ball of energy is "
+            "presently useless to you.");
+        return (false);
+    }
+
+    int min_evo = 2;
+    if (known && subtype == MISC_CRYSTAL_BALL_OF_SEEING)
+        min_evo = 3;
+    if (you.skills[SK_EVOCATIONS] < min_evo)
+    {
+        mpr("You lack the skill to use this item.");
+        return false;
+    }
+
+    return (true);
+}
+
 static bool _ball_of_seeing(void)
 {
     bool ret = false;
 
     mpr("You gaze into the crystal ball.");
 
-    int use = (!you.confused() ? random2(you.skills[SK_EVOCATIONS] * 6)
-                               : random2(5));
+    int use = random2(you.skill(SK_EVOCATIONS) * 6);
 
     if (use < 2)
-    {
-        lose_stat( STAT_INTELLIGENCE, 1, false, "using a ball of seeing");
-    }
+        lose_stat(STAT_INT, 1, false, "using a ball of seeing");
     else if (use < 5 && enough_mp(1, true))
     {
-        mpr("You feel power drain from you!");
+        mpr("You feel your power drain away!");
         set_mp(0, false);
+        // if you're out of mana, the switch chain falls through to confusion
     }
     else if (use < 10 || you.level_type == LEVEL_LABYRINTH)
     {
         if (you.level_type == LEVEL_LABYRINTH)
             mpr("You see a maze of twisty little passages, all alike.");
-        confuse_player( 10 + random2(10), false );
+        confuse_player(10 + random2(10));
     }
     else if (use < 15 || coinflip())
-    {
         mpr("You see nothing.");
-    }
-    else if (magic_mapping( 15, 50 + random2( you.skills[SK_EVOCATIONS]), true))
+    else if (magic_mapping(6 + you.skill(SK_EVOCATIONS),
+                           50 + random2(you.skill(SK_EVOCATIONS)), true))
     {
         mpr("You see a map of your surroundings!");
         ret = true;
     }
     else
-    {
         mpr("You see nothing.");
-    }
 
     return (ret);
 }
 
-static bool _disc_of_storms(void)
+bool disc_of_storms(bool drac_breath)
 {
-    const int fail_rate = (30 - you.skills[SK_EVOCATIONS]);
+    const int fail_rate = (30 - you.skill(SK_EVOCATIONS));
     bool rc = false;
+    int power;
 
-    if (player_res_electricity() || x_chance_in_y(fail_rate, 100))
+    if ((player_res_electricity() || x_chance_in_y(fail_rate, 100))
+         && !drac_breath)
+    {
         canned_msg(MSG_NOTHING_HAPPENS);
-    else if (x_chance_in_y(fail_rate, 100))
+    }
+    else if (x_chance_in_y(fail_rate, 100) && !drac_breath)
         mpr("The disc glows for a moment, then fades.");
-    else if (x_chance_in_y(fail_rate, 100))
+    else if (x_chance_in_y(fail_rate, 100) && !drac_breath)
         mpr("Little bolts of electricity crackle over the disc.");
     else
     {
-        mpr("The disc erupts in an explosion of electricity!");
+        if (!drac_breath)
+            mpr("The disc erupts in an explosion of electricity!");
         rc = true;
 
-        const int disc_count = roll_dice(2, 1 + you.skills[SK_EVOCATIONS] / 7);
+        const int disc_count = (drac_breath) ? roll_dice(2, 1 + you.experience_level / 7) :
+                                roll_dice(2, 1 + you.skill(SK_EVOCATIONS) / 7);
 
         for (int i = 0; i < disc_count; ++i)
         {
@@ -460,33 +522,36 @@ static bool _disc_of_storms(void)
 
             const zap_type which_zap = RANDOM_ELEMENT(types);
 
-            beam.range = you.skills[SK_EVOCATIONS]/3 + 5; // 5--14
+            beam.range = (drac_breath) ? you.experience_level / 3 + 5 :
+                                         you.skill(SK_EVOCATIONS)/3 + 5; // 5--14
             beam.source = you.pos();
             beam.target = you.pos() + coord_def(random2(13)-6, random2(13)-6);
-
+            power = (drac_breath) ? 25 + you.experience_level : 30 + you.skill(SK_EVOCATIONS) * 2;
             // Non-controlleable, so no player tracer.
-            zapping(which_zap, 30 + you.skills[SK_EVOCATIONS] * 2, beam);
+            zapping(which_zap, power, beam);
 
         }
 
-        for (radius_iterator ri(you.pos(), LOS_RADIUS, false); ri; ++ri)
+        if (!drac_breath)
         {
-            if (grd(*ri) < DNGN_MAXWALL)
-                continue;
+            for (radius_iterator ri(you.pos(), LOS_RADIUS, false); ri; ++ri)
+            {
+                if (grd(*ri) < DNGN_MAXWALL)
+                    continue;
 
-            if (one_chance_in(60 - you.skills[SK_EVOCATIONS]))
-                place_cloud(CLOUD_RAIN, *ri,
-                            random2(you.skills[SK_EVOCATIONS]), KC_YOU);
+                if (one_chance_in(60 - you.skill(SK_EVOCATIONS)))
+                    place_cloud(CLOUD_RAIN, *ri,
+                                random2(you.skill(SK_EVOCATIONS)), &you);
+            }
         }
     }
-
     return (rc);
 }
 
 void tome_of_power(int slot)
 {
-    int powc = 5 + you.skills[SK_EVOCATIONS]
-                 + roll_dice( 5, you.skills[SK_EVOCATIONS] );
+    int powc = 5 + you.skill(SK_EVOCATIONS)
+                 + roll_dice(5, you.skill(SK_EVOCATIONS));
 
     msg::stream << "The book opens to a page covered in "
                 << weird_writing() << '.' << std::endl;
@@ -508,25 +573,24 @@ void tome_of_power(int slot)
     }
 
     mpr("You find yourself reciting the magical words!");
-    exercise( SK_EVOCATIONS, 1 );
+    practise(EX_WILL_READ_TOME);
 
     if (x_chance_in_y(7, 50))
     {
         mpr("A cloud of weird smoke pours from the book's pages!");
-        big_cloud( random_smoke_type(), KC_YOU,
-                   you.pos(), 20, 10 + random2(8) );
+        big_cloud(random_smoke_type(), &you, you.pos(), 20, 10 + random2(8));
         xom_is_stimulated(16);
     }
     else if (x_chance_in_y(2, 43))
     {
         mpr("A cloud of choking fumes pours from the book's pages!");
-        big_cloud(CLOUD_POISON, KC_YOU, you.pos(), 20, 7 + random2(5));
+        big_cloud(CLOUD_POISON, &you, you.pos(), 20, 7 + random2(5));
         xom_is_stimulated(64);
     }
     else if (x_chance_in_y(2, 41))
     {
         mpr("A cloud of freezing gas pours from the book's pages!");
-        big_cloud(CLOUD_COLD, KC_YOU, you.pos(), 20, 8 + random2(5));
+        big_cloud(CLOUD_COLD, &you, you.pos(), 20, 8 + random2(5));
         xom_is_stimulated(64);
     }
     else if (x_chance_in_y(3, 39))
@@ -555,9 +619,9 @@ void tome_of_power(int slot)
     }
     else
     {
-        viewwindow(false);
+        viewwindow();
 
-        int temp_rand = random2(23) + random2(you.skills[SK_EVOCATIONS] / 3);
+        int temp_rand = random2(23) + random2(you.skill(SK_EVOCATIONS) / 3);
 
         if (temp_rand > 25)
             temp_rand = 25;
@@ -571,7 +635,7 @@ void tome_of_power(int slot)
              (temp_rand >  9) ? SPELL_VENOM_BOLT :
              (temp_rand >  8) ? SPELL_BOLT_OF_DRAINING :
              (temp_rand >  7) ? SPELL_BOLT_OF_INACCURACY :
-             (temp_rand >  6) ? SPELL_STICKY_FLAME :
+             (temp_rand >  6) ? SPELL_STICKY_FLAME_RANGE :
              (temp_rand >  5) ? SPELL_TELEPORT_SELF :
              (temp_rand >  4) ? SPELL_CIGOTUVIS_DEGENERATION :
              (temp_rand >  3) ? SPELL_POLYMORPH_OTHER :
@@ -580,7 +644,7 @@ void tome_of_power(int slot)
              (temp_rand >  0) ? SPELL_THROW_FROST
                               : SPELL_MAGIC_DART);
 
-        your_spells( spell_casted, powc, false );
+        your_spells(spell_casted, powc, false);
     }
 }
 
@@ -592,17 +656,17 @@ void skill_manual(int slot)
     item_def& manual(you.inv[slot]);
     const bool known = item_type_known(manual);
     if (!known)
-        set_ident_flags( manual, ISFLAG_KNOW_TYPE );
-    const int skill = manual.plus;
+        set_ident_flags(manual, ISFLAG_KNOW_TYPE);
+    const skill_type skill = static_cast<skill_type>(manual.plus);
 
     mprf("You read about %s.", skill_name(skill));
 
-    exercise(skill, 500);
+    practise(EX_READ_MANUAL, skill);
 
     if (--manual.plus2 <= 0)
     {
         mpr("The manual crumbles into dust.");
-        dec_inv_item_quantity( slot, 1 );
+        dec_inv_item_quantity(slot, 1);
     }
     else
         mpr("The manual looks somewhat more worn.");
@@ -616,38 +680,31 @@ static bool _box_of_beasts(item_def &box)
 
     mpr("You open the lid...");
 
-    if (x_chance_in_y(60 + you.skills[SK_EVOCATIONS], 100))
+    if (x_chance_in_y(60 + you.skill(SK_EVOCATIONS), 100))
     {
-        monster_type beasty = MONS_NO_MONSTER;
+        const monster_type beasts[] = {
+            MONS_MEGABAT,   MONS_HOUND,     MONS_JACKAL,
+            MONS_RAT,       MONS_ICE_BEAST, MONS_SNAKE,
+            MONS_YAK,       MONS_BUTTERFLY, MONS_WATER_MOCCASIN,
+            MONS_CROCODILE, MONS_HELL_HOUND
+        };
 
-        // If you worship a good god, don't summon an evil beast (in
+        monster_type mon = MONS_NO_MONSTER;
+
+        // If you worship a good god, don't summon an unholy beast (in
         // this case, the hell hound).
         do
-        {
-            int temp_rand = random2(11);
+            mon = RANDOM_ELEMENT(beasts);
+        while (player_will_anger_monster(mon));
 
-            beasty = ((temp_rand == 0) ? MONS_GIANT_BAT :
-                      (temp_rand == 1) ? MONS_HOUND :
-                      (temp_rand == 2) ? MONS_JACKAL :
-                      (temp_rand == 3) ? MONS_RAT :
-                      (temp_rand == 4) ? MONS_ICE_BEAST :
-                      (temp_rand == 5) ? MONS_SNAKE :
-                      (temp_rand == 6) ? MONS_YAK :
-                      (temp_rand == 7) ? MONS_BUTTERFLY :
-                      (temp_rand == 8) ? MONS_WATER_MOCCASIN :
-                      (temp_rand == 9) ? MONS_CROCODILE
-                                       : MONS_HELL_HOUND);
-        }
-        while (player_will_anger_monster(beasty));
-
-        beh_type beha = BEH_FRIENDLY;
-
-        if (one_chance_in(you.skills[SK_EVOCATIONS] + 5))
-            beha = BEH_HOSTILE;
+        const bool friendly = (!one_chance_in(you.skill(SK_EVOCATIONS) + 5));
 
         if (create_monster(
-                mgen_data(beasty, beha, &you, 2 + random2(4), 0,
-                          you.pos(), MHITYOU)) != -1)
+                mgen_data(mon,
+                          friendly ? BEH_FRIENDLY : BEH_HOSTILE, &you,
+                          2 + random2(4), 0,
+                          you.pos(),
+                          MHITYOU)) != -1)
         {
             success = true;
 
@@ -675,37 +732,40 @@ static bool _ball_of_energy(void)
 
     mpr("You gaze into the crystal ball.");
 
-    int use = ((!you.confused()) ? random2(you.skills[SK_EVOCATIONS] * 6)
-                                 : random2(6));
+    int use = random2(you.skill(SK_EVOCATIONS) * 6);
 
-    if (use < 2 || you.max_magic_points == 0)
+    if (use < 2)
     {
-        lose_stat(STAT_INTELLIGENCE, 1, false, "using a ball of energy");
+        const int loss = roll_dice(1, 2 * you.max_intel() / 3);
+        lose_stat(STAT_INT, loss, false, "using a ball of energy");
     }
-    else if (use < 4 && enough_mp(1, true)
-             || you.magic_points == you.max_magic_points)
+    else if (use < 4 && enough_mp(1, true))
     {
-        mpr( "You feel your power drain away!" );
-        set_mp( 0, false );
+        mpr("You feel your power drain away!");
+        set_mp(0, false);
     }
     else if (use < 6)
     {
-        confuse_player( 10 + random2(10), false );
+        confuse_player(10 + random2(10));
+    }
+    else if (use < 8)
+    {
+        you.paralyse(NULL, 2 + random2(2));
     }
     else
     {
         int proportional = (you.magic_points * 100) / you.max_magic_points;
 
-        if (random2avg(77 - you.skills[SK_EVOCATIONS] * 2, 4) > proportional
+        if (random2avg(77 - you.skill(SK_EVOCATIONS) * 2, 4) > proportional
             || one_chance_in(25))
         {
-            mpr( "You feel your power drain away!" );
-            set_mp( 0, false );
+            mpr("You feel your power drain away!");
+            set_mp(0, false);
         }
         else
         {
-            mpr( "You are suffused with power!" );
-            inc_mp( 6 + roll_dice( 2, you.skills[SK_EVOCATIONS] ), false );
+            mpr("You are suffused with power!");
+            inc_mp(6 + roll_dice(2, you.skill(SK_EVOCATIONS)), false);
 
             ret = true;
         }
@@ -714,18 +774,6 @@ static bool _ball_of_energy(void)
     return (ret);
 }
 
-static bool _ball_of_fixation(void)
-{
-    mpr("You gaze into the crystal ball.");
-    mpr("You are mesmerised by a rainbow of scintillating colours!");
-
-    const int duration = random_range(15, 40);
-    you.set_duration(DUR_PARALYSIS, duration);
-    you.set_duration(DUR_SLOW,      duration);
-
-    return (true);
-}
-
 bool evoke_item(int slot)
 {
     if (you.berserk())
@@ -734,7 +782,7 @@ bool evoke_item(int slot)
         return (false);
     }
 
-    if (player_in_bat_form())
+    if (!player_can_handle_equipment())
     {
         canned_msg(MSG_PRESENT_FORM);
         return (false);
@@ -742,10 +790,10 @@ bool evoke_item(int slot)
 
     if (slot == -1)
     {
-        slot = prompt_invent_item( "Evoke which item? (* to show all)",
+        slot = prompt_invent_item("Evoke which item? (* to show all)",
                                    MT_INVLIST,
                                    OSEL_EVOKABLE, true, true, true, 0, -1,
-                                   NULL, OPER_EVOKE );
+                                   NULL, OPER_EVOKE);
 
         if (prompt_failed(slot))
             return (false);
@@ -755,7 +803,7 @@ bool evoke_item(int slot)
 
     ASSERT(slot >= 0);
 
-#if DEBUG // Used only by an assert
+#ifdef ASSERTS // Used only by an assert
     const bool wielded = (you.equip[EQ_WEAPON] == slot);
 #endif /* DEBUG */
 
@@ -804,9 +852,9 @@ bool evoke_item(int slot)
     case OBJ_STAVES:
         ASSERT(wielded);
 
-        if (item_is_rod( item ))
+        if (item_is_rod(item))
         {
-            pract = staff_spell( slot );
+            pract = staff_spell(slot);
             // [ds] Early exit, no turns are lost.
             if (pract == -1)
                 return (false);
@@ -815,11 +863,17 @@ bool evoke_item(int slot)
         }
         else if (item.sub_type == STAFF_CHANNELING)
         {
-            if (you.magic_points < you.max_magic_points
-                && x_chance_in_y(you.skills[SK_EVOCATIONS] + 11, 40))
+            if (item_type_known(item)
+                && !you.is_undead && you.hunger_state == HS_STARVING)
+            {
+                canned_msg(MSG_TOO_HUNGRY);
+                return (false);
+            }
+            else if (you.magic_points < you.max_magic_points
+                     && x_chance_in_y(you.skill(SK_EVOCATIONS) + 11, 40))
             {
                 mpr("You channel some magical energy.");
-                inc_mp( 1 + random2(3), false );
+                inc_mp(1 + random2(3), false);
                 make_hungry(50, false, true);
                 pract = 1;
                 did_work = true;
@@ -843,6 +897,13 @@ bool evoke_item(int slot)
             break;
         }
 
+        if (_is_crystal_ball(item)
+            && !_check_crystal_ball(item.sub_type, item_type_known(item)))
+        {
+            unevokable = true;
+            break;
+        }
+
         switch (item.sub_type)
         {
         case MISC_BOTTLED_EFREET:
@@ -856,33 +917,33 @@ bool evoke_item(int slot)
             break;
 
         case MISC_AIR_ELEMENTAL_FAN:
-            if (you.skills[SK_EVOCATIONS] <= random2(30))
+            if (you.skill(SK_EVOCATIONS) <= random2(30))
                 canned_msg(MSG_NOTHING_HAPPENS);
             else
             {
-                cast_summon_elemental(100, GOD_NO_GOD, MONS_AIR_ELEMENTAL, 4);
+                cast_summon_elemental(100, GOD_NO_GOD, MONS_AIR_ELEMENTAL, 4, 3);
                 pract = (one_chance_in(5) ? 1 : 0);
                 ident = true;
             }
             break;
 
         case MISC_LAMP_OF_FIRE:
-            if (you.skills[SK_EVOCATIONS] <= random2(30))
+            if (you.skill(SK_EVOCATIONS) <= random2(30))
                 canned_msg(MSG_NOTHING_HAPPENS);
             else
             {
-                cast_summon_elemental(100, GOD_NO_GOD, MONS_FIRE_ELEMENTAL, 4);
+                cast_summon_elemental(100, GOD_NO_GOD, MONS_FIRE_ELEMENTAL, 4, 3);
                 pract = (one_chance_in(5) ? 1 : 0);
                 ident = true;
             }
             break;
 
         case MISC_STONE_OF_EARTH_ELEMENTALS:
-            if (you.skills[SK_EVOCATIONS] <= random2(30))
+            if (you.skill(SK_EVOCATIONS) <= random2(30))
                 canned_msg(MSG_NOTHING_HAPPENS);
             else
             {
-                cast_summon_elemental(100, GOD_NO_GOD, MONS_EARTH_ELEMENTAL, 4);
+                cast_summon_elemental(100, GOD_NO_GOD, MONS_EARTH_ELEMENTAL, 4, 5);
                 pract = (one_chance_in(5) ? 1 : 0);
                 ident = true;
             }
@@ -903,16 +964,25 @@ bool evoke_item(int slot)
                 pract = 1, ident = true;
             break;
 
+#if TAG_MAJOR_VERSION == 32
         case MISC_CRYSTAL_BALL_OF_FIXATION:
-            if (_ball_of_fixation())
-                pract = 1, ident = true;
+            mpr("Nothing happens.");
+            pract = 0, ident = true;
             break;
+#endif
 
         case MISC_DISC_OF_STORMS:
-            if (_disc_of_storms())
+            if (disc_of_storms())
                 pract = (coinflip() ? 2 : 1), ident = true;
             break;
 
+        case MISC_QUAD_DAMAGE:
+            mpr("QUAD DAMAGE!");
+            you.duration[DUR_QUAD_DAMAGE] = 30 * BASELINE_DELAY;
+            ASSERT(in_inventory(item));
+            dec_inv_item_quantity(item.link, 1);
+            break;
+
         default:
             did_work = false;
             unevokable = true;
@@ -928,12 +998,12 @@ bool evoke_item(int slot)
     if (!did_work)
         canned_msg(MSG_NOTHING_HAPPENS);
     else if (pract > 0)
-        exercise( SK_EVOCATIONS, pract );
+        practise(EX_DID_EVOKE_ITEM, pract);
 
     if (ident && !item_type_known(item))
     {
-        set_ident_type( item.base_type, item.sub_type, ID_KNOWN_TYPE );
-        set_ident_flags( item, ISFLAG_KNOW_TYPE );
+        set_ident_type(item.base_type, item.sub_type, ID_KNOWN_TYPE);
+        set_ident_flags(item, ISFLAG_KNOW_TYPE);
 
         mprf("You are wielding %s.",
              item.name(DESC_NOCAP_A).c_str());
