@@ -1,7 +1,7 @@
-/*
- * File:     l_crawl.cc
- * Summary:  General game bindings.
- */
+/**
+ * @file
+ * @brief General game bindings.
+**/
 
 #include "AppHdr.h"
 
@@ -27,6 +27,7 @@
 #include "random.h"
 #include "religion.h"
 #include "stuff.h"
+#include "tutorial.h"
 #include "view.h"
 
 #ifdef UNIX
@@ -91,10 +92,18 @@ static int crawl_formatted_mpr(lua_State *ls)
     return (0);
 }
 
+// Print to stderr for debugging hooks.
+LUAFN(crawl_stderr)
+{
+    const char *text = luaL_checkstring(ls, 1);
+    fprintf(stderr, "%s\n", text);
+    return (0);
+}
+
 LUAWRAP(crawl_delay, delay(luaL_checkint(ls, 1)))
 LUAWRAP(crawl_more, more())
 LUAWRAP(crawl_flush_prev_message, flush_prev_message())
-LUAWRAP(crawl_mesclr, mesclr())
+LUAWRAP(crawl_mesclr, mesclr(lua_isboolean(ls, 1) ? lua_toboolean(ls, 1) : false))
 LUAWRAP(crawl_redraw_screen, redraw_screen())
 
 static int crawl_set_more_autoclear(lua_State *ls)
@@ -104,21 +113,11 @@ static int crawl_set_more_autoclear(lua_State *ls)
         luaL_argerror(ls, 1, "needs a boolean argument");
         return (0);
     }
-    set_more_autoclear( lua_toboolean(ls, 1) );
+    set_more_autoclear(lua_toboolean(ls, 1));
 
     return (0);
 }
 
-static int crawl_input_line(lua_State *ls)
-{
-    // This is arbitrary, but anybody entering so many characters is psychotic.
-    char linebuf[500];
-
-    get_input_line(linebuf, sizeof linebuf);
-    lua_pushstring(ls, linebuf);
-    return (1);
-}
-
 static int crawl_c_input_line(lua_State *ls)
 {
     char linebuf[500];
@@ -131,7 +130,7 @@ static int crawl_c_input_line(lua_State *ls)
     return (1);
 }
 
-LUARET1(crawl_getch, number, getch())
+LUARET1(crawl_getch, number, getchm())
 LUARET1(crawl_kbhit, number, kbhit())
 LUAWRAP(crawl_flush_input, flush_input_buffer(FLUSH_LUA))
 
@@ -183,8 +182,8 @@ static void crawl_sendkeys_proc(lua_State *ls, int argi)
         if (!keys)
             return;
 
-        for ( ; *keys; ++keys)
-            macro_buf_add(*keys);
+        for (; *keys; ++keys)
+            macro_sendkeys_end_add_expanded(*keys);
     }
     else if (lua_istable(ls, argi))
     {
@@ -203,8 +202,7 @@ static void crawl_sendkeys_proc(lua_State *ls, int argi)
     }
     else if (lua_isnumber(ls, argi))
     {
-        int key = luaL_checkint(ls, argi);
-        macro_buf_add(key);
+        macro_sendkeys_end_add_expanded(luaL_checkint(ls, argi));
     }
 }
 
@@ -237,9 +235,17 @@ static int crawl_process_command(lua_State *ls)
 
 static int crawl_process_keys(lua_State *ls)
 {
-    if (you_are_delayed() || you.turn_is_over)
+    const delay_type current_delay = current_delay_action();
+    if (current_delay && current_delay != DELAY_MACRO)
     {
-        luaL_error(ls, "Cannot currently process new keys");
+        luaL_error(ls, "Cannot currently process new keys (%s delay active)",
+                   delay_name(current_delay));
+        return (0);
+    }
+
+    if (you.turn_is_over)
+    {
+        luaL_error(ls, "Cannot currently process new keys (turn is over)");
         return (0);
     }
 
@@ -261,7 +267,7 @@ static int crawl_process_keys(lua_State *ls)
 
     flush_input_buffer(FLUSH_BEFORE_COMMAND);
     for (int i = 1, len = strlen(keys); i < len; i++)
-        macro_buf_add(keys[i]);
+        macro_sendkeys_end_add_expanded(keys[i]);
 
     process_command(cmd);
 
@@ -411,10 +417,10 @@ static int crawl_message_filter(lua_State *ls)
 
     int num = lua_isnumber(ls, 2)? luaL_checkint(ls, 2) : -1;
     message_filter **mf =
-            clua_new_userdata< message_filter* >( ls, MESSF_METATABLE );
+            clua_new_userdata< message_filter* >(ls, MESSF_METATABLE);
     if (mf)
     {
-        *mf = new message_filter( num, pattern );
+        *mf = new message_filter(num, pattern);
         return (1);
     }
     return (0);
@@ -495,33 +501,21 @@ static int crawl_article_a(lua_State *ls)
     return (1);
 }
 
-static int crawl_travel_stopper(lua_State *ls)
-{
-    const char *s = luaL_checkstring(ls, 1);
-
-    if (!s)
-        return (0);
-
-    std::string stopper = s;
-
-    Options.travel_stop_message.push_back(message_filter(stopper));
-
-    return (0);
-}
-
 LUARET1(crawl_game_started, boolean, crawl_state.need_save)
-LUARET1(crawl_random2, number, random2( luaL_checkint(ls, 1) ))
-LUARET1(crawl_one_chance_in, boolean, one_chance_in( luaL_checkint(ls, 1) ))
+LUARET1(crawl_random2, number, random2(luaL_checkint(ls, 1)))
+LUARET1(crawl_one_chance_in, boolean, one_chance_in(luaL_checkint(ls, 1)))
 LUARET1(crawl_random2avg, number,
-        random2avg( luaL_checkint(ls, 1), luaL_checkint(ls, 2) ))
+        random2avg(luaL_checkint(ls, 1), luaL_checkint(ls, 2)))
 LUARET1(crawl_random_range, number,
-        random_range( luaL_checkint(ls, 1), luaL_checkint(ls, 2),
-                      lua_isnumber(ls, 3)? luaL_checkint(ls, 3) : 1 ))
+        random_range(luaL_checkint(ls, 1), luaL_checkint(ls, 2),
+                      lua_isnumber(ls, 3)? luaL_checkint(ls, 3) : 1))
 LUARET1(crawl_coinflip, boolean, coinflip())
 LUARET1(crawl_roll_dice, number,
         lua_gettop(ls) == 1
-        ? roll_dice( 1, luaL_checkint(ls, 1) )
-        : roll_dice( luaL_checkint(ls, 1), luaL_checkint(ls, 2) ))
+        ? roll_dice(1, luaL_checkint(ls, 1))
+        : roll_dice(luaL_checkint(ls, 1), luaL_checkint(ls, 2)))
+LUARET1(crawl_x_chance_in_y, boolean, x_chance_in_y(luaL_checkint(ls, 1),
+                                                    luaL_checkint(ls, 2)))
 
 static int crawl_is_tiles(lua_State *ls)
 {
@@ -534,6 +528,29 @@ static int crawl_is_tiles(lua_State *ls)
     return (1);
 }
 
+static int crawl_get_command (lua_State *ls)
+{
+    if (lua_gettop(ls) == 0)
+    {
+        lua_pushnil(ls);
+        return (1);
+    }
+
+    const command_type cmd = name_to_command(luaL_checkstring(ls, 1));
+
+    std::string cmd_name = command_to_string(cmd, true);
+    if (strcmp(cmd_name.c_str(), "<") == 0)
+        cmd_name = "<<";
+
+    lua_pushstring(ls, cmd_name.c_str());
+    return (1);
+}
+
+LUAWRAP(crawl_endgame, screen_end_game(luaL_checkstring(ls, 1)))
+LUAWRAP(crawl_tutorial_hunger, set_tutorial_hunger(luaL_checkint(ls, 1)))
+LUAWRAP(crawl_tutorial_skill, set_tutorial_skill(luaL_checkstring(ls, 1), luaL_checkint(ls, 2)))
+LUAWRAP(crawl_tutorial_hint, tutorial_init_hint(luaL_checkstring(ls, 1)))
+
 static int crawl_random_element(lua_State *ls)
 {
     const int table_idx = 1;
@@ -623,6 +640,7 @@ static const struct luaL_reg crawl_clib[] =
 {
     { "mpr",            crawl_mpr },
     { "formatted_mpr",  crawl_formatted_mpr },
+    { "stderr",  crawl_stderr },
     { "more",           crawl_more },
     { "more_autoclear", crawl_set_more_autoclear },
     { "flush_prev_message", crawl_flush_prev_message },
@@ -633,10 +651,10 @@ static const struct luaL_reg crawl_clib[] =
     { "random2avg"   ,  crawl_random2avg },
     { "coinflip",       crawl_coinflip },
     { "roll_dice",      crawl_roll_dice },
+    { "x_chance_in_y",  crawl_x_chance_in_y },
     { "random_range",   crawl_random_range },
     { "random_element", crawl_random_element },
     { "redraw_screen",  crawl_redraw_screen },
-    { "input_line",     crawl_input_line },
     { "c_input_line",   crawl_c_input_line},
     { "getch",          crawl_getch },
     { "yesno",          crawl_yesno },
@@ -654,7 +672,6 @@ static const struct luaL_reg crawl_clib[] =
     { "msgch_num",      crawl_msgch_num },
     { "msgch_name",     crawl_msgch_name },
     { "take_note",      crawl_take_note },
-    { "add_travel_stopper", crawl_travel_stopper },
 
     { "regex",          crawl_regex },
     { "message_filter", crawl_message_filter },
@@ -665,6 +682,12 @@ static const struct luaL_reg crawl_clib[] =
     { "game_started",   crawl_game_started },
     { "is_tiles",       crawl_is_tiles },
     { "err_trace",      crawl_err_trace },
+    { "get_command",    crawl_get_command },
+    { "endgame",        crawl_endgame },
+
+    { "tutorial_hunger", crawl_tutorial_hunger },
+    { "tutorial_skill",  crawl_tutorial_skill },
+    { "tutorial_hint",   crawl_tutorial_hint },
 
     { NULL, NULL },
 };
@@ -690,16 +713,15 @@ LUAFN(_crawl_args)
 
 LUAFN(_crawl_milestone)
 {
-#ifdef DGL_MILESTONES
     mark_milestone(luaL_checkstring(ls, 1),
-                   luaL_checkstring(ls, 2));
-#endif
+                   luaL_checkstring(ls, 2),
+                   lua_toboolean(ls, 3));
     return (0);
 }
 
 LUAFN(_crawl_redraw_view)
 {
-    viewwindow(false);
+    viewwindow();
     return (0);
 }
 
@@ -709,9 +731,7 @@ LUAFN(_crawl_redraw_stats)
     you.redraw_quiver       = true;
     you.redraw_hit_points   = true;
     you.redraw_magic_points = true;
-    you.redraw_strength     = true;
-    you.redraw_intelligence = true;
-    you.redraw_dexterity    = true;
+    you.redraw_stats.init(true);
     you.redraw_experience   = true;
     you.redraw_armour_class = true;
     you.redraw_evasion      = true;
@@ -737,7 +757,7 @@ LUAFN(_crawl_millis)
 }
 #endif
 
-std::string _crawl_make_name(lua_State *ls)
+static std::string _crawl_make_name(lua_State *ls)
 {
     // A quick wrapper around itemname:make_name. Seed is random_int().
     // Possible parameters: all caps, max length, char start. By default
