@@ -1,7 +1,7 @@
-/*
- * File:     l_items.cc
- * Summary:  functions for managed Lua item manipulation
- */
+/**
+ * @file
+ * @brief functions for managed Lua item manipulation
+**/
 
 #include "AppHdr.h"
 
@@ -22,7 +22,7 @@
 #include "output.h"
 #include "player.h"
 #include "skills2.h"
-#include "spells2.h"
+#include "spl-summoning.h"
 #include "stuff.h"
 
 /////////////////////////////////////////////////////////////////////
@@ -31,7 +31,7 @@
 struct item_wrapper
 {
     item_def *item;
-    long turn;
+    int turn;
 
     bool valid() const { return turn == you.num_turns; }
 };
@@ -57,7 +57,7 @@ void lua_push_floor_items(lua_State *ls, int link)
 {
     lua_newtable(ls);
     int index = 0;
-    for ( ; link != NON_ITEM; link = mitm[link].link)
+    for (; link != NON_ITEM; link = mitm[link].link)
     {
         clua_push_item(ls, &mitm[link]);
         lua_rawseti(ls, -2, ++index);
@@ -72,7 +72,7 @@ void lua_push_inv_items(lua_State *ls = NULL)
     int index = 0;
     for (unsigned slot = 0; slot < ENDOFPACK; ++slot)
     {
-        if (you.inv[slot].is_valid())
+        if (you.inv[slot].defined())
         {
             clua_push_item(ls, &you.inv[slot]);
             lua_rawseti(ls, -2, ++index);
@@ -105,7 +105,7 @@ static int l_item_do_wield(lua_State *ls)
     UDATA_ITEM(item);
 
     int slot = -1;
-    if (item && item->is_valid() && in_inventory(*item))
+    if (item && item->defined() && in_inventory(*item))
         slot = item->link;
     bool res = wield_weapon(true, slot);
     lua_pushboolean(ls, res);
@@ -172,7 +172,7 @@ static int l_item_do_remove(lua_State *ls)
 
     bool result = false;
     if (eq == EQ_WEAPON)
-        result = wield_weapon(true, -1);
+        result = wield_weapon(true, SLOT_BARE_HANDS);
     else if (eq == EQ_LEFT_RING || eq == EQ_RIGHT_RING || eq == EQ_AMULET)
         result = remove_ring(item->link);
     else
@@ -371,7 +371,7 @@ static std::string _item_name(lua_State *ls, item_def* item)
         ndesc = description_type_by_name(lua_tostring(ls, 1));
     else if (lua_isnumber(ls, 1))
         ndesc = static_cast<description_level_type>(luaL_checkint(ls, 1));
-    bool terse = lua_toboolean(ls, 2);
+    const bool terse = lua_toboolean(ls, 2);
     return (item->name(ndesc, terse));
 }
 
@@ -380,10 +380,7 @@ static int l_item_do_name(lua_State *ls)
     UDATA_ITEM(item);
 
     if (item)
-    {
-        std::string name = _item_name(ls, item);
-        lua_pushstring(ls, name.c_str());
-    }
+        lua_pushstring(ls, _item_name(ls, item).c_str());
     else
         lua_pushnil(ls);
     return (1);
@@ -439,7 +436,7 @@ IDEF(ininventory)
 
 IDEF(equip_type)
 {
-    if (!item || !item->is_valid())
+    if (!item || !item->defined())
         return (0);
 
     equipment_type eq = EQ_NONE;
@@ -466,10 +463,10 @@ IDEF(equip_type)
 
 IDEF(weap_skill)
 {
-    if (!item || !item->is_valid())
+    if (!item || !item->defined())
         return (0);
 
-    int skill = range_skill(*item);
+    skill_type skill = range_skill(*item);
     if (skill == SK_THROWING)
         skill = weapon_skill(*item);
     if (skill == SK_FIGHTING)
@@ -480,9 +477,19 @@ IDEF(weap_skill)
     return (2);
 }
 
+IDEF(is_ranged)
+{
+    if (!item || !item->defined())
+        return (0);
+
+    lua_pushboolean(ls, is_range_weapon(*item));
+
+    return (1);
+}
+
 IDEF(dropped)
 {
-    if (!item || !item->is_valid())
+    if (!item || !item->defined())
         return (0);
 
     lua_pushboolean(ls, item->flags & ISFLAG_DROPPED);
@@ -492,7 +499,7 @@ IDEF(dropped)
 
 IDEF(can_cut_meat)
 {
-    if (!item || !item->is_valid())
+    if (!item || !item->defined())
         return (0);
 
     lua_pushboolean(ls, can_cut_meat(*item));
@@ -502,7 +509,7 @@ IDEF(can_cut_meat)
 
 IDEF(artefact)
 {
-    if (!item || !item->is_valid())
+    if (!item || !item->defined())
         return (0);
 
     lua_pushboolean(ls, is_artefact(*item));
@@ -512,31 +519,16 @@ IDEF(artefact)
 
 IDEF(branded)
 {
-    if (!item || !item->is_valid() || !item_type_known(*item))
+    if (!item || !item->defined() || !item_type_known(*item))
         return (0);
 
-    bool branded = false;
-    switch (item->base_type)
-    {
-    case OBJ_WEAPONS:
-        branded = get_weapon_brand(*item) != SPWPN_NORMAL;
-        break;
-    case OBJ_ARMOUR:
-        branded = get_armour_ego_type(*item) != SPARM_NORMAL;
-        break;
-    case OBJ_MISSILES:
-        branded = get_ammo_brand(*item) != SPMSL_NORMAL;
-        break;
-    default:
-        break;
-    }
-    lua_pushboolean(ls, branded);
+    lua_pushboolean(ls, item_is_branded(*item));
     return (1);
 }
 
 IDEF(snakable)
 {
-    if (!item || !item->is_valid())
+    if (!item || !item->defined())
         return (0);
 
     lua_pushboolean(ls, item_is_snakable(*item));
@@ -551,7 +543,7 @@ static int l_item_do_pluses (lua_State *ls)
 
     UDATA_ITEM(item);
 
-    if (!item || !item->is_valid() || !item_ident(*item, ISFLAG_KNOW_PLUSES))
+    if (!item || !item->defined() || !item_ident(*item, ISFLAG_KNOW_PLUSES))
     {
         lua_pushboolean(ls, false);
         return (1);
@@ -572,7 +564,7 @@ static int l_item_do_destroy (lua_State *ls)
 
     UDATA_ITEM(item);
 
-    if (!item || !item->is_valid())
+    if (!item || !item->defined())
     {
         lua_pushboolean(ls, false);
         return (0);
@@ -593,7 +585,7 @@ static int l_item_do_dec_quantity (lua_State *ls)
 
     UDATA_ITEM(item);
 
-    if (!item || !item->is_valid())
+    if (!item || !item->defined())
     {
         lua_pushboolean(ls, false);
         return (1);
@@ -621,7 +613,7 @@ static int l_item_do_inc_quantity (lua_State *ls)
 
     UDATA_ITEM(item);
 
-    if (!item || !item->is_valid())
+    if (!item || !item->defined())
     {
         lua_pushboolean(ls, false);
         return (1);
@@ -640,9 +632,9 @@ static int l_item_do_inc_quantity (lua_State *ls)
 
 IDEFN(inc_quantity, do_inc_quantity)
 
-unsigned long str_to_item_status_flags (std::string flag)
+iflags_t str_to_item_status_flags (std::string flag)
 {
-    unsigned long flags = 0;
+    iflags_t flags = 0;
     if (flag.find("curse") != std::string::npos)
         flags &= ISFLAG_KNOW_CURSE;
     // type is dealt with using item_type_known.
@@ -664,7 +656,7 @@ static int l_item_do_identified (lua_State *ls)
 
     UDATA_ITEM(item);
 
-    if (!item || !item->is_valid())
+    if (!item || !item->defined())
     {
         lua_pushnil(ls);
         return (1);
@@ -679,7 +671,7 @@ static int l_item_do_identified (lua_State *ls)
         else
         {
             const bool check_type = strip_tag(flags, "type");
-            const unsigned long item_flags = str_to_item_status_flags(flags);
+            iflags_t item_flags = str_to_item_status_flags(flags);
             known_status = ((item_flags || check_type)
                             && (!item_flags || item_ident(*item, item_flags))
                             && (!check_type || item_type_known(*item)));
@@ -715,7 +707,11 @@ IDEF(sub_type)
 
 IDEF(ego_type)
 {
-    ASSERT_DLUA;
+    if (CLua::get_vm(ls).managed_vm && !item_ident(*item, ISFLAG_KNOW_TYPE))
+    {
+        lua_pushstring(ls, "unknown");
+        return (1);
+    }
 
     lua_pushstring(ls, ego_type_string(*item).c_str());
     return (1);
@@ -733,6 +729,16 @@ IDEF(artefact_name)
     return (1);
 }
 
+IDEF(is_cursed)
+{
+    ASSERT_DLUA;
+
+    bool cursed = item->cursed();
+
+    lua_pushboolean(ls, cursed);
+    return (1);
+}
+
 // Library functions below
 static int l_item_inventory(lua_State *ls)
 {
@@ -766,7 +772,7 @@ static int l_item_swap_slots(lua_State *ls)
     bool verbose = lua_toboolean(ls, 3);
     if (slot1 < 0 || slot1 >= ENDOFPACK
         || slot2 < 0 || slot2 >= ENDOFPACK
-        || slot1 == slot2 || !you.inv[slot1].is_valid())
+        || slot1 == slot2 || !you.inv[slot1].defined())
     {
         return (0);
     }
@@ -891,7 +897,7 @@ static int l_item_equipped_at(lua_State *ls)
 static int l_item_inslot(lua_State *ls)
 {
     int index = luaL_checkint(ls, 1);
-    if (index >= 0 && index < 52 && you.inv[index].is_valid())
+    if (index >= 0 && index < 52 && you.inv[index].defined())
         clua_push_item(ls, &you.inv[index]);
     else
         lua_pushnil(ls);
@@ -926,6 +932,7 @@ static ItemAccessor item_attrs[] =
     { "equipped",          l_item_equipped },
     { "equip_type",        l_item_equip_type },
     { "weap_skill",        l_item_weap_skill },
+    { "is_ranged",         l_item_is_ranged },
     { "dropped",           l_item_dropped },
     { "can_cut_meat",      l_item_can_cut_meat },
     { "pluses",            l_item_pluses },
@@ -937,6 +944,7 @@ static ItemAccessor item_attrs[] =
     { "sub_type",          l_item_sub_type },
     { "ego_type",          l_item_ego_type },
     { "artefact_name",     l_item_artefact_name },
+    { "is_cursed",         l_item_is_cursed },
 };
 
 static int item_get(lua_State *ls)
