@@ -1,8 +1,8 @@
-/*
- *  File:       libunix.cc
- *  Summary:    Functions for unix and curses support
- *              Needed by makefile.unix.
- */
+/**
+ * @file
+ * @brief Functions for unix and curses support
+ *             Needed by makefile.unix.
+**/
 
 /* Some replacement routines missing in gcc
    Some of these are inspired by/stolen from the Linux-conio package
@@ -35,6 +35,7 @@
 #include "files.h"
 #include "state.h"
 #include "stuff.h"
+#include "unicode.h"
 #include "view.h"
 #include "viewgeom.h"
 
@@ -43,12 +44,9 @@
 #include <sys/resource.h>
 #endif
 
-#ifdef UNICODE_GLYPHS
 #include <wchar.h>
 #include <locale.h>
 #include <langinfo.h>
-#endif
-
 #include <termios.h>
 
 static struct termios def_term;
@@ -84,11 +82,10 @@ static int   Current_Colour = COLOR_PAIR(BG_COL * 8 + FG_COL);
 
 static int curs_fg_attr(int col);
 static int curs_bg_attr(int col);
-static int waddstr_with_altcharset(WINDOW* w, const char* s);
 
 static bool cursor_is_enabled = true;
 
-static unsigned int convert_to_curses_attr( int chattr )
+static unsigned int convert_to_curses_attr(int chattr)
 {
     switch (chattr & CHATTR_ATTRMASK)
     {
@@ -102,13 +99,13 @@ static unsigned int convert_to_curses_attr( int chattr )
     }
 }
 
-static inline short macro_colour( short col )
+static inline short macro_colour(short col)
 {
     return (Options.colour[ col ]);
 }
 
 // Translate DOS colors to curses.
-static short translate_colour( short col )
+static short translate_colour(short col)
 {
     switch (col)
     {
@@ -149,41 +146,20 @@ static short translate_colour( short col )
     }
 }
 
-static void setup_colour_pairs( void )
+static void setup_colour_pairs(void)
 {
     short i, j;
 
     for (i = 0; i < 8; i++)
         for (j = 0; j < 8; j++)
         {
-            if (( i > 0 ) || ( j > 0 ))
+            if ((i > 0) || (j > 0))
                 init_pair(i * 8 + j, j, i);
         }
 
     init_pair(63, COLOR_BLACK, Options.background_colour);
 }
 
-#ifdef UNICODE_GLYPHS
-static std::string unix_glyph2string(unsigned gly)
-{
-    char buf[50];  // Overkill, I know.
-    wchar_t wcbuf[2];
-    wcbuf[0] = gly;
-    wcbuf[1] = 0;
-    if (wcstombs(buf, wcbuf, sizeof buf) != (size_t) -1)
-        return (buf);
-
-    return std::string(1, gly);
-}
-
-static int unix_multibyte_strlen(const std::string &s)
-{
-    const char *cs = s.c_str();
-    size_t len = mbsrtowcs(NULL, &cs, 0, NULL);
-    return (len == (size_t) -1? s.length() : len);
-}
-#endif
-
 static void unix_handle_terminal_resize();
 
 static void termio_init()
@@ -204,17 +180,6 @@ static void termio_init()
 
     tcsetattr(0, TCSAFLUSH, &game_term);
 
-    crawl_state.unicode_ok = false;
-#ifdef UNICODE_GLYPHS
-    if (setlocale(LC_ALL, UNICODE_LOCALE)
-        && !strcmp(nl_langinfo(CODESET), "UTF-8"))
-    {
-        crawl_state.unicode_ok       = true;
-        crawl_state.glyph2strfn      = unix_glyph2string;
-        crawl_state.multibyte_strlen = unix_multibyte_strlen;
-    }
-#endif
-
     crawl_state.terminal_resize_handler = unix_handle_terminal_resize;
 }
 
@@ -264,34 +229,50 @@ static int proc_mouse_event(int c, const MEVENT *me)
 }
 #endif
 
-static int raw_m_getch()
+static int pending = 0;
+
+int getchk()
 {
-    const int c = getch();
-    switch (c)
+    if (pending)
     {
-#ifdef NCURSES_MOUSE_VERSION
-    case KEY_MOUSE:
-    {
-        MEVENT me;
-        getmouse(&me);
-        return (proc_mouse_event(c, &me));
+        int c = pending;
+        pending = 0;
+        return c;
     }
-#endif
-    default:
+
+    wint_t c;
+    switch (get_wch(&c))
+    {
+    case ERR:
         // getch() returns -1 on EOF, convert that into an Escape. Evil hack,
         // but the alternative is to explicitly check for -1 everywhere where
         // we might otherwise spin in a tight keyboard input loop.
-        return (c == -1? ESCAPE : c);
+        return ESCAPE;
+    case OK:
+        // a normal (printable) key
+        return c;
     }
+
+    return -c;
 }
 
 int m_getch()
 {
     int c;
     do
-        c = raw_m_getch();
-    while ((c == CK_MOUSE_MOVE || c == CK_MOUSE_CLICK)
-           && !crawl_state.mouse_enabled);
+    {
+        c = getchk();
+
+#ifdef NCURSES_MOUSE_VERSION
+        if (c == -KEY_MOUSE)
+        {
+            MEVENT me;
+            getmouse(&me);
+            c = proc_mouse_event(c, &me);
+        }
+#endif
+    } while ((c == CK_MOUSE_MOVE || c == CK_MOUSE_CLICK)
+             && !crawl_state.mouse_enabled);
 
     return (c);
 }
@@ -303,17 +284,17 @@ int getch_ck()
     {
     // [dshaligram] MacOS ncurses returns 127 for backspace.
     case 127:
-    case KEY_BACKSPACE: return CK_BKSP;
-    case KEY_DC:    return CK_DELETE;
-    case KEY_HOME:  return CK_HOME;
-    case KEY_PPAGE: return CK_PGUP;
-    case KEY_END:   return CK_END;
-    case KEY_NPAGE: return CK_PGDN;
-    case KEY_UP:    return CK_UP;
-    case KEY_DOWN:  return CK_DOWN;
-    case KEY_LEFT:  return CK_LEFT;
-    case KEY_RIGHT: return CK_RIGHT;
-    default:        return c;
+    case -KEY_BACKSPACE: return CK_BKSP;
+    case -KEY_DC:    return CK_DELETE;
+    case -KEY_HOME:  return CK_HOME;
+    case -KEY_PPAGE: return CK_PGUP;
+    case -KEY_END:   return CK_END;
+    case -KEY_NPAGE: return CK_PGDN;
+    case -KEY_UP:    return CK_UP;
+    case -KEY_DOWN:  return CK_DOWN;
+    case -KEY_LEFT:  return CK_LEFT;
+    case -KEY_RIGHT: return CK_RIGHT;
+    default:         return c;
     }
 }
 
@@ -321,8 +302,11 @@ int getch_ck()
 
 static void handle_sigwinch(int)
 {
+    crawl_state.last_winch = time(0);
     if (crawl_state.waiting_for_command)
         handle_terminal_resize();
+    else
+        crawl_state.terminal_resized = true;
 }
 
 #ifdef SIGHUP_SAVE
@@ -341,7 +325,7 @@ static void handle_hangup(int)
     // hangup signal.
     alarm(HANGUP_KILL_DELAY_SECONDS);
 
-    interrupt_activity( AI_FORCE_INTERRUPT );
+    interrupt_activity(AI_FORCE_INTERRUPT);
 
 #ifdef USE_TILE
     // XXX: Will a tiles build ever need to handle the HUP signal?
@@ -351,7 +335,7 @@ static void handle_hangup(int)
     // on key-presses to immediately return, including any call that was
     // still blocking in the main thread when the HUP signal was caught.
     // This should guarantee that the main thread will un-stall and
-    // will eventually return to _input() in acr.cc, which will then
+    // will eventually return to _input() in main.cc, which will then
     // call sighup_save_and_exit().
     //
     // The point to all this is that if a user is playing a game on a
@@ -400,7 +384,7 @@ static void unix_handle_terminal_resize()
     unixcurses_startup();
 }
 
-static void unixcurses_defkeys( void )
+static void unixcurses_defkeys(void)
 {
 #ifdef NCURSES_VERSION
     // keypad 0-9 (only if the "application mode" was successfully initialised)
@@ -437,30 +421,31 @@ static void unixcurses_defkeys( void )
 
 int unixcurses_get_vi_key(int keyin)
 {
-    switch (keyin)
+    switch (-keyin)
     {
-    // 1001..1009: passed without change
-    case 1031: return 1007;
-    case 1034: return 1001;
-    case 1040: return 1005;
+    // -1001..-1009: passed without change
+    case 1031: return -1007;
+    case 1034: return -1001;
+    case 1040: return -1005;
 
-    case KEY_HOME:   return 1007;
-    case KEY_END:    return 1001;
-    case KEY_DOWN:   return 1002;
-    case KEY_UP:     return 1008;
-    case KEY_LEFT:   return 1004;
-    case KEY_RIGHT:  return 1006;
-    case KEY_NPAGE:  return 1003;
-    case KEY_PPAGE:  return 1009;
-    case KEY_A1:     return 1007;
-    case KEY_A3:     return 1009;
-    case KEY_B2:     return 1005;
-    case KEY_C1:     return 1001;
-    case KEY_C3:     return 1003;
+    case KEY_HOME:   return -1007;
+    case KEY_END:    return -1001;
+    case KEY_DOWN:   return -1002;
+    case KEY_UP:     return -1008;
+    case KEY_LEFT:   return -1004;
+    case KEY_RIGHT:  return -1006;
+    case KEY_NPAGE:  return -1003;
+    case KEY_PPAGE:  return -1009;
+    case KEY_A1:     return -1007;
+    case KEY_A3:     return -1009;
+    case KEY_B2:     return -1005;
+    case KEY_C1:     return -1001;
+    case KEY_C3:     return -1003;
     case KEY_SHOME:  return 'Y';
     case KEY_SEND:   return 'B';
     case KEY_SLEFT:  return 'H';
     case KEY_SRIGHT: return 'L';
+    case KEY_BTAB:   return CK_SHIFT_TAB;
     }
     return keyin;
 }
@@ -470,7 +455,7 @@ int unixcurses_get_vi_key(int keyin)
 #define KPADAPP "\033[?1051l\033[?1052l\033[?1060l\033[?1061h"
 #define KPADCUR "\033[?1051l\033[?1052l\033[?1060l\033[?1061l"
 
-void unixcurses_startup( void )
+void unixcurses_startup(void)
 {
     termio_init();
 
@@ -503,7 +488,8 @@ void unixcurses_startup( void )
 
 #ifdef DGL_ENABLE_CORE_DUMP
     rlimit lim;
-    if (!getrlimit(RLIMIT_CORE, &lim)) {
+    if (!getrlimit(RLIMIT_CORE, &lim))
+    {
         lim.rlim_cur = RLIM_INFINITY;
         setrlimit(RLIMIT_CORE, &lim);
     }
@@ -518,9 +504,13 @@ void unixcurses_startup( void )
 #ifdef CURSES_USE_KEYPAD
     keypad(stdscr, TRUE);
 
-#if CURSES_SET_ESCDELAY
+#ifdef CURSES_SET_ESCDELAY
+#ifdef NCURSES_REENTRANT
+    set_escdelay(CURSES_SET_ESCDELAY);
+#else
     ESCDELAY = CURSES_SET_ESCDELAY;
 #endif
+#endif
 #endif
 
     meta(stdscr, TRUE);
@@ -602,131 +592,51 @@ extern "C" char *itoa(int value, char *strptr, int radix)
     return strptr;
 }
 
-int cprintf(const char *format,...)
+void cprintf(const char *format, ...)
 {
-    int i;
     char buffer[2048];          // One full screen if no control seq...
 
     va_list argp;
 
     va_start(argp, format);
-    vsprintf(buffer, format, argp);
+    vsnprintf(buffer, sizeof(buffer), format, argp);
     va_end(argp);
-    i = waddstr_with_altcharset(stdscr, buffer);
-    return (i);
-}
 
-// Wrapper around curses waddstr(); handles switching to the alt charset
-// when necessary, and performing the funny DEC translation.
-// Returns a curses success value.
-static int waddstr_with_altcharset(WINDOW* w, const char* str)
-{
-    int ret = OK;
-    if (Options.char_set == CSET_ASCII || Options.char_set == CSET_UNICODE)
+    ucs_t c;
+    char *bp = buffer;
+    while(int s = utf8towc(&c, bp))
     {
-        // In ascii, we don't expect any high-bit chars.
-        // In Unicode, str is UTF-8 and we shouldn't touch anything.
-        ret = waddstr(w, str);
+        bp += s;
+        putwch(c);
     }
-    else
-    {
-        // Otherwise, high bit indicates alternate charset.
-        // DEC line-drawing chars don't have the high bit, so
-        // we map them into 0xE0 and above.
-        const bool bDEC = (Options.char_set == CSET_DEC);
-        const char* begin = str;
-
-        while (true)
-        {
-            const char* end = begin;
-            // Output a range of normal characters (the common case)
-            while (*end && (unsigned char)*end <= 127) ++end;
-            if (end-begin) ret = waddnstr(w, begin, end-begin);
-
-            // Then a single high-bit character
-            chtype c = (unsigned char)*end;
-            if (c == 0)
-                break;
-            else if (bDEC && c >= 0xE0)
-                ret = waddch(w, (c & 0x7F) | A_ALTCHARSET);
-            else // if (*end > 127)
-                ret = waddch(w, c | A_ALTCHARSET);
-            begin = end+1;
-        }
-    }
-    return ret;
 }
 
-int putch(unsigned char chr)
+int putwch(ucs_t chr)
 {
-    if (chr == 0)
-        chr = ' ';
-
-    return (addch(chr));
-}
-
-int putwch(unsigned chr)
-{
-#ifdef UNICODE_GLYPHS
-    if (chr <= 127 || Options.char_set != CSET_UNICODE)
-        return (putch(chr));
-
     wchar_t c = chr;
+    if (!c)
+        c = ' ';
+    // TODO: recognize unsupported characters and try to transliterate
     return (addnwstr(&c, 1));
-#else
-    return (putch(static_cast<unsigned char>(chr)));
-#endif
 }
 
-char getche()
+void put_colour_ch(int colour, ucs_t ch)
 {
-    char chr;
-
-    chr = getch();
-    addch(chr);
-    return (chr);
+    textattr(colour);
+    putwch(ch);
 }
 
-
-int window(int x1, int y1, int x2, int y2)
+void puttext(int x1, int y1, const crawl_view_buffer &vbuf)
 {
-    x1 = y1 = x2 = y2 = 0;      /* Do something to them.. makes gcc happy :) */
-    return OK;
-}
-
-void put_colour_ch(int colour, unsigned ch)
-{
-    if (Options.char_set == CSET_ASCII || Options.char_set == CSET_UNICODE)
+    const screen_cell_t *cell = vbuf;
+    const coord_def size = vbuf.size();
+    for (int y = 0; y < size.y; ++y)
     {
-        // Unicode can't or-in curses attributes; but it also doesn't have
-        // to fool around with altcharset
-        textattr(colour);
-        putwch(ch);
-    }
-    else
-    {
-        const unsigned oldch = ch;
-        if (oldch & 0x80) ch |= A_ALTCHARSET;
-        // Shift the DEC line drawing set
-        if (oldch >= 0xE0 && Options.char_set == CSET_DEC)
-            ch ^= 0x80;
-        textattr(colour);
-
-        // putch truncates ch to a byte, so don't use it.
-        if ((ch & 0x7f) == 0) ch |= ' ';
-        addch(ch);
-    }
-}
-
-void puttext(int x1, int y1, int x2, int y2, const screen_buffer_t *buf)
-{
-    for (int y = y1; y <= y2; ++y)
-    {
-        cgotoxy(x1, y);
-        for (int x = x1; x <= x2; ++x)
+        cgotoxy(x1, y1 + y);
+        for (int x = 0; x < size.x; ++x)
         {
-            put_colour_ch( buf[1], buf[0] );
-            buf += 2;
+            put_colour_ch(cell->colour, cell->glyph);
+            cell++;
         }
     }
     update_screen();
@@ -743,15 +653,15 @@ void update_screen(void)
 
 void clear_to_end_of_line(void)
 {
-    textcolor( LIGHTGREY );
-    textbackground( BLACK );
+    textcolor(LIGHTGREY);
+    textbackground(BLACK);
     clrtoeol();
 }
 
 void clear_to_end_of_screen(void)
 {
-    textcolor( LIGHTGREY );
-    textbackground( BLACK );
+    textcolor(LIGHTGREY);
+    textbackground(BLACK);
     clrtobot();
 }
 
@@ -765,23 +675,12 @@ int get_number_of_cols(void)
     return (COLS);
 }
 
-void get_input_line_from_curses( char *const buff, int len )
-{
-    echo();
-    wgetnstr( stdscr, buff, len );
-    noecho();
-
-    // Don't return a partial string if a HUP signal interrupted things.
-    if (crawl_state.seen_hups)
-        buff[0] = '\0';
-}
-
 int clrscr()
 {
     int retval;
 
-    textcolor( LIGHTGREY );
-    textbackground( BLACK );
+    textcolor(LIGHTGREY);
+    textbackground(BLACK);
     retval = clear();
 #ifdef DGAMELAUNCH
     printf("%s", DGL_CLEAR_SCREEN);
@@ -825,23 +724,22 @@ static int curs_fg_attr(int col)
     short fg, bg;
 
     FG_COL = col & 0x00ff;
-    fg = translate_colour( macro_colour( FG_COL ) );
-    bg = translate_colour( BG_COL == BLACK ? Options.background_colour
-                                           : BG_COL );
+    fg = translate_colour(macro_colour(FG_COL));
+    bg = translate_colour(BG_COL == BLACK ? Options.background_colour
+                                           : BG_COL);
 
     // calculate which curses flags we need...
     unsigned int flags = 0;
 
-#ifdef USE_COLOUR_OPTS
     unsigned brand = get_brand(col);
     if (brand != CHATTR_NORMAL)
     {
-        flags |= convert_to_curses_attr( brand );
+        flags |= convert_to_curses_attr(brand);
 
         if ((brand & CHATTR_ATTRMASK) == CHATTR_HILITE)
         {
             bg = translate_colour(
-                    macro_colour( (brand & CHATTR_COLMASK) >> 8 ));
+                    macro_colour((brand & CHATTR_COLMASK) >> 8));
 
             if (fg == bg)
                 fg = COLOR_BLACK;
@@ -856,7 +754,6 @@ static int curs_fg_attr(int col)
             fg = COLOR_WHITE;
         }
     }
-#endif
 
     // curses typically uses A_BOLD to give bright foreground colour,
     // but various termcaps may disagree
@@ -875,12 +772,12 @@ static int curs_fg_attr(int col)
     // figure out which colour pair we want
     const int pair = (fg == 0 && bg == 0) ? 63 : (bg * 8 + fg);
 
-    return ( COLOR_PAIR(pair) | flags );
+    return (COLOR_PAIR(pair) | flags);
 }
 
 void textcolor(int col)
 {
-    (void)attrset( Current_Colour = curs_fg_attr(col) );
+    (void)attrset(Current_Colour = curs_fg_attr(col));
 }
 
 static int curs_bg_attr(int col)
@@ -888,17 +785,16 @@ static int curs_bg_attr(int col)
     short fg, bg;
 
     BG_COL = col & 0x00ff;
-    fg = translate_colour( macro_colour( FG_COL ) );
-    bg = translate_colour( BG_COL == BLACK ? Options.background_colour
-                                           : BG_COL );
+    fg = translate_colour(macro_colour(FG_COL));
+    bg = translate_colour(BG_COL == BLACK ? Options.background_colour
+                                           : BG_COL);
 
     unsigned int flags = 0;
 
-#ifdef USE_COLOUR_OPTS
     unsigned brand = get_brand(col);
     if (brand != CHATTR_NORMAL)
     {
-        flags |= convert_to_curses_attr( brand );
+        flags |= convert_to_curses_attr(brand);
 
         if ((brand & CHATTR_ATTRMASK) == CHATTR_HILITE)
         {
@@ -916,7 +812,6 @@ static int curs_bg_attr(int col)
             fg = COLOR_WHITE;
         }
     }
-#endif
 
     // curses typically uses A_BOLD to give bright foreground colour,
     // but various termcaps may disagree
@@ -935,12 +830,12 @@ static int curs_bg_attr(int col)
     // figure out which colour pair we want
     const int pair = (fg == 0 && bg == 0) ? 63 : (bg * 8 + fg);
 
-    return ( COLOR_PAIR(pair) | flags );
+    return (COLOR_PAIR(pair) | flags);
 }
 
 void textbackground(int col)
 {
-    (void)attrset( Current_Colour = curs_bg_attr(col) );
+    (void)attrset(Current_Colour = curs_bg_attr(col));
 }
 
 
@@ -949,7 +844,6 @@ int gotoxy_sys(int x, int y)
     return (move(y - 1, x - 1));
 }
 
-#ifdef UNICODE_GLYPHS
 typedef cchar_t char_info;
 inline bool operator == (const cchar_t &a, const cchar_t &b)
 {
@@ -958,7 +852,8 @@ inline bool operator == (const cchar_t &a, const cchar_t &b)
 inline char_info character_at(int y, int x)
 {
     cchar_t c;
-    mvin_wch(y, x, &c);
+    // (void) is to hush an incorrect clang warning.
+    (void)mvin_wch(y, x, &c);
     return (c);
 }
 inline bool valid_char(const cchar_t &c)
@@ -985,37 +880,8 @@ static void flip_colour(cchar_t &ch)
     }
 
     const int newpair = (fg * 8 + bg);
-    ch.attr = COLOR_PAIR(newpair) | (ch.attr & A_ALTCHARSET);
+    ch.attr = COLOR_PAIR(newpair);
 }
-#else // ! UNICODE_GLYPHS
-typedef unsigned long char_info;
-#define character_at(y,x)    mvinch(y,x)
-#define valid_char(x) (x)
-#define write_char_at(y,x,c) mvaddch(y, x, c)
-
-#define char_info_character(c) ((c) & A_CHARTEXT)
-#define char_info_colour(c)    ((c) & A_COLOR)
-#define char_info_attributes(c) ((c) & A_ATTRIBUTES)
-
-static void flip_colour(char_info &ch)
-{
-    const unsigned colour = char_info_colour(ch);
-    const int pair        = PAIR_NUMBER(colour);
-
-    int fg     = pair & 7;
-    int bg     = (pair >> 3) & 7;
-
-    if (pair == 63)
-    {
-        fg    = COLOR_WHITE;
-        bg    = COLOR_BLACK;
-    }
-
-    const int newpair = (fg * 8 + bg);
-    ch = (char_info_character(ch) | COLOR_PAIR(newpair) |
-          (char_info_attributes(ch) & A_ALTCHARSET));
-}
-#endif
 
 static char_info oldch, oldmangledch;
 static int faked_x = -1, faked_y;
@@ -1036,55 +902,53 @@ void fakecursorxy(int x, int y)
     faked_x = x - 1;
     faked_y = y - 1;
     flip_colour(c);
-    write_char_at( y - 1, x - 1, oldmangledch = c);
+    write_char_at(y - 1, x - 1, oldmangledch = c);
     move(y - 1, x - 1);
 }
 
 int wherex()
 {
-    int x, y;
-
-    getyx(stdscr, y, x);
-    return (x + 1);
+    return getcurx(stdscr) + 1;
 }
 
 
 int wherey()
 {
-    int x, y;
-
-    getyx(stdscr, y, x);
-    return (y + 1);
+    return getcury(stdscr) + 1;
 }
 
-void delay( unsigned long time )
+void delay(unsigned long time)
 {
     refresh();
     if (time)
-        usleep( time * 1000 );
+        usleep(time * 1000);
 }
 
-
-/*
-   Note: kbhit now in macro.cc
- */
-
 /* This is Juho Snellman's modified kbhit, to work with macros */
-int kbhit()
+bool kbhit()
 {
+    if (pending)
+        return true;
+
+    wint_t c;
     int i;
 
     nodelay(stdscr, TRUE);
     timeout(0);  // apparently some need this to guarantee non-blocking -- bwr
-    i = wgetch(stdscr);
+    i = get_wch(&c);
     nodelay(stdscr, FALSE);
 
-    if (i == -1)
-        i = 0;
-    else
-        ungetch(i);
-
-    return (i);
+    switch(i)
+    {
+    case OK:
+        pending = c;
+        return true;
+    case KEY_CODE_YES:
+        pending = -c;
+        return true;
+    default:
+        return false;
+    }
 }
 
 extern "C" {
