@@ -7,7 +7,7 @@
 #include <stdarg.h>
 #include <stdio.h>
 
-typedef unsigned short screen_buffer_t;
+class crawl_view_buffer;
 
 void init_libw32c(void);
 void deinit_libw32c(void);
@@ -23,26 +23,22 @@ void clear_to_end_of_line();
 void gotoxy_sys(int x, int y);
 void textcolor(int c);
 void textattr(int c);
-void cprintf( const char *format, ... );
+void cprintf(const char *format, ...);
 // void cprintf(const char *s);
 void set_string_input(bool value);
 bool set_buffering(bool value);
 int get_console_string(char *buf, int maxlen);
 void print_timings(void);
 
-void window(int x, int y, int lx, int ly);
 int wherex(void);
 int wherey(void);
-void putch(char c);
-void putwch(unsigned c);
-int getch(void);
+void putwch(wchar_t c);
+int getchk(void);
 int getch_ck(void);
-int key_to_command(int);
-int getche(void);
-int kbhit(void);
+bool kbhit(void);
 void delay(int ms);
 void textbackground(int c);
-void puttext(int x1, int y1, int x2, int y2, const screen_buffer_t *);
+void puttext(int x, int y, const crawl_view_buffer &vbuf);
 void update_screen();
 
 void enable_smart_cursor(bool cursor);
