@@ -1,6 +1,6 @@
-/*
- *  File:        los.cc
- *  Summary:     Line-of-sight algorithm.
+/**
+ * @file
+ * @brief Line-of-sight algorithm.
  *
  *
  *
@@ -38,7 +38,7 @@
  * The code provides functions for filling LOS information
  * around a given center efficiently, and for querying rays
  * between two given cells.
- */
+**/
 
 #include "AppHdr.h"
 
@@ -47,6 +47,7 @@
 #include <cmath>
 #include <algorithm>
 
+#include "areas.h"
 #include "bitary.h"
 #include "coord.h"
 #include "coord-circle.h"
@@ -54,6 +55,7 @@
 #include "debug.h"
 #include "externs.h"
 #include "geom2d.h"
+#include "losglobal.h"
 #include "losparam.h"
 #include "player.h"
 #include "ray.h"
@@ -124,10 +126,14 @@ void clear_rays_on_exit()
 // Pre-squared LOS radius.
 int _los_radius_sq = LOS_RADIUS_SQ;
 
+static void _handle_los_change();
+
 void set_los_radius(int r)
 {
     ASSERT(r <= LOS_MAX_RADIUS);
     _los_radius_sq = r * r + 1;
+    invalidate_los();
+    _handle_los_change();
 }
 
 // XXX: just for monster_los
@@ -188,7 +194,7 @@ struct los_ray : public ray_def
 
     coord_def operator[](unsigned int i)
     {
-        ASSERT(0 <= i && i < length);
+        ASSERT(i < length);
         return ray_coords[start+i];
     }
 };
@@ -234,7 +240,7 @@ struct cellray
     // XXX: Currently ray/cellray[0] is the first point outside the origin.
     coord_def operator[](unsigned int i)
     {
-        ASSERT(0 <= i && i <= end);
+        ASSERT(i <= end);
         return ray_coords[ray.start+i];
     }
 
@@ -250,7 +256,7 @@ struct cellray
 // This determines which ray is considered better by find_ray,
 // used with list::sort.
 // Returns true if a is strictly better than b, false else.
-bool _is_better(const cellray& a, const cellray& b)
+static bool _is_better(const cellray& a, const cellray& b)
 {
     // Only compare cellrays with equal target.
     ASSERT(a.target() == b.target());
@@ -261,19 +267,19 @@ bool _is_better(const cellray& a, const cellray& b)
     else if (a.imbalance > b.imbalance)
         return (false);
     else
-        return (a.first_diag);
+        return (a.first_diag && !b.first_diag);
 }
 
 enum compare_type
 {
     C_SUBRAY,
     C_SUPERRAY,
-    C_NEITHER
+    C_NEITHER,
 };
 
 // Check whether one of the passed cellrays is a subray of the
 // other in terms of footprint.
-compare_type _compare_cellrays(const cellray& a, const cellray& b)
+static compare_type _compare_cellrays(const cellray& a, const cellray& b)
 {
     if (a.target() != b.target())
         return C_NEITHER;
@@ -336,7 +342,7 @@ static std::vector<int> _find_minimal_cellrays()
 
             bool erased = false;
             for (min_it = min.begin();
-                 min_it != min.end() && !dup; )
+                 min_it != min.end() && !dup;)
             {
                 switch(_compare_cellrays(*min_it, c))
                 {
@@ -444,12 +450,12 @@ static void _create_blockrays()
     smoke_rays = new bit_array(n_min_rays);
 
 #ifdef DEBUG_DIAGNOSTICS
-    mprf( MSGCH_DIAGNOSTICS, "Cellrays: %d Fullrays: %u Minimal cellrays: %u",
-          n_cellrays, fullrays.size(), n_min_rays );
+    mprf(MSGCH_DIAGNOSTICS, "Cellrays: %d Fullrays: %u Minimal cellrays: %u",
+          n_cellrays, fullrays.size(), n_min_rays);
 #endif
 }
 
-static int _gcd( int x, int y )
+static int _gcd(int x, int y)
 {
     int tmp;
     while (y != 0)
@@ -462,14 +468,14 @@ static int _gcd( int x, int y )
     return x;
 }
 
-bool complexity_lt( const std::pair<int,int>& lhs,
-                    const std::pair<int,int>& rhs )
+bool complexity_lt(const std::pair<int,int>& lhs,
+                    const std::pair<int,int>& rhs)
 {
     return lhs.first * lhs.second < rhs.first * rhs.second;
 }
 
 // Cast all rays
-void raycast()
+static void raycast()
 {
     static bool done_raycast = false;
     if (done_raycast)
@@ -506,7 +512,7 @@ void raycast()
         const int xangle = xyangles[i].first;
         const int yangle = xyangles[i].second;
 
-        for (int intercept = 1; intercept < LOS_INTERCEPT_MULT*yangle; ++intercept )
+        for (int intercept = 1; intercept < LOS_INTERCEPT_MULT*yangle; ++intercept)
         {
             double xstart = ((double)intercept) / (LOS_INTERCEPT_MULT*yangle);
             double ystart = 0.5;
@@ -529,7 +535,7 @@ static int _imbalance(ray_def ray, const coord_def& target)
     {
         coord_def old = ray.pos();
         if (!ray.advance())
-            ASSERT(false);
+            die("can't advance ray");
         switch((ray.pos() - old).abs())
         {
         case 1:
@@ -543,8 +549,7 @@ static int _imbalance(ray_def ray, const coord_def& target)
                 imb = diags;
             break;
         default:
-            ASSERT(false);
-            break;
+            die("ray imbalance out of range");
         }
     }
     return imb;
@@ -560,7 +565,7 @@ void cellray::calc_params()
 // Find ray in positive quadrant.
 // opc has been translated for this quadrant.
 // XXX: Allow finding ray of minimum opacity.
-bool _find_ray_se(const coord_def& target, ray_def& ray,
+static bool _find_ray_se(const coord_def& target, ray_def& ray,
                   const opacity_func& opc, const circle_def& bds,
                   bool cycle)
 {
@@ -585,7 +590,7 @@ bool _find_ray_se(const coord_def& target, ray_def& ray,
 #endif
 
     unsigned int start = cycle ? ray.cycle_idx + 1 : 0;
-    ASSERT(0 <= start && start <= min.size());
+    ASSERT(start <= min.size());
 
     int blocked = OPC_OPAQUE;
     for (unsigned int i = start;
@@ -684,7 +689,7 @@ dungeon_feature_type ray_blocker(const coord_def& source,
     ray_def ray;
     if (!find_ray(source, target, ray, opc_default))
     {
-        ASSERT (false);
+        ASSERT (you.xray_vision);
         return (NUM_FEATURES);
     }
 
@@ -808,7 +813,7 @@ bool cell_see_cell(const coord_def& p1, const coord_def& p2)
 // Smoke will now only block LOS after two cells of smoke. This is
 // done by updating with a second array.
 
-void _losight_quadrant(los_grid& sh, const los_param& dat, int sx, int sy)
+static void _losight_quadrant(los_grid& sh, const los_param& dat, int sx, int sy)
 {
     const unsigned int num_cellrays = cellray_ends.size();
 
@@ -898,3 +903,57 @@ void losight(los_grid& sh, const coord_def& center,
 {
     losight(sh, los_param_funcs(center, opc, bounds));
 }
+
+/////////////////////////////////////
+// A start at tracking LOS changes.
+
+// Something that affects LOS (with default parameters)
+// has changed somewhere.
+static void _handle_los_change()
+{
+    invalidate_agrid();
+}
+
+static bool _mons_block_sight(const monster* mons)
+{
+    return (mons->type == MONS_BUSH || mons->type == MONS_DOOR_MIMIC);
+}
+
+void los_actor_moved(const actor* act, const coord_def& oldpos)
+{
+    if (act->atype() == ACT_MONSTER && _mons_block_sight(act->as_monster()))
+    {
+        invalidate_los_around(oldpos);
+        invalidate_los_around(act->pos());
+        _handle_los_change();
+    }
+}
+
+void los_monster_died(const monster* mon)
+{
+    if (_mons_block_sight(mon))
+    {
+        invalidate_los_around(mon->pos());
+        _handle_los_change();
+    }
+}
+
+// Might want to pass new/old terrain.
+void los_terrain_changed(const coord_def& p)
+{
+    invalidate_los_around(p);
+    _handle_los_change();
+}
+
+// Might want to pass new/old cloud type.
+void los_cloud_changed(const coord_def& p)
+{
+    invalidate_los_around(p);
+    _handle_los_change();
+}
+
+void los_changed()
+{
+    invalidate_los();
+    _handle_los_change();
+}
