@@ -1,7 +1,10 @@
-/*
+/**
+ * @defgroup mapdef Map Parsing
+ * @brief Parse .des files into map objects.
+ * @section DESCRIPTION
+ *
  * mapdef.h:
  * Header for map structures used by the level compiler.
- * Created by: dshaligram on Wed Nov 22 08:41:20 2006 UTC
  *
  * NOTE: When we refer to map, this could be a full map, filling an entire
  * level or a minivault that occupies just a portion of the level.
@@ -26,6 +29,21 @@
 // Invalid heightmap height.
 static const int INVALID_HEIGHT = -31999;
 
+// Exception thrown when a map cannot be loaded from its .dsc file
+// because the .dsc file has changed under it.
+class map_load_exception : public std::exception
+{
+public:
+    map_load_exception(const std::string &_mapname) : mapname(_mapname) { }
+    ~map_load_exception() throw () { }
+    const char *what() const throw()
+    {
+        return mapname.c_str();
+    }
+private:
+    std::string mapname;
+};
+
 // [dshaligram] Maps can be mirrored; for every orientation, there must be
 // a suitable mirror.
 enum map_section_type                  // see maps.cc and dungeon.cc {dlb}
@@ -158,7 +176,7 @@ public:
     map_colour_list colours;
 };
 
-typedef std::pair<unsigned long, int> map_weighted_fprop;
+typedef std::pair<feature_property_type, int> map_weighted_fprop;
 class map_fprop_list : public std::vector<map_weighted_fprop>
 {
 public:
@@ -180,12 +198,12 @@ public:
     {
     }
 
-    unsigned long get_property();
+    feature_property_type get_property();
 
 public:
     std::string key;
     bool fix;
-    unsigned long fixed_prop;
+    feature_property_type fixed_prop;
     map_fprop_list fprops;
 };
 
@@ -207,7 +225,7 @@ public:
 };
 
 #ifdef USE_TILE
-typedef std::pair<int, int> map_weighted_tile;
+typedef std::pair<std::string, int> map_weighted_tile;
 class map_tile_list : public std::vector<map_weighted_tile>
 {
 public:
@@ -219,11 +237,11 @@ class tile_spec
 public:
     tile_spec(const std::string &_key, bool _fix, bool _rand, bool _last, bool _floor, bool _feat, const map_tile_list &_tiles)
         : key(_key), fix(_fix), chose_fixed(false), no_random(_rand), last_tile(_last), floor(_floor), feat(_feat),
-          fixed_tile(0), tiles(_tiles)
+          fixed_tile(""), tiles(_tiles)
     {
     }
 
-    int get_tile();
+    std::string get_tile();
 
 public:
     std::string key;
@@ -233,7 +251,7 @@ public:
     bool last_tile;
     bool floor;
     bool feat;
-    int fixed_tile;
+    std::string fixed_tile;
     map_tile_list tiles;
 };
 #endif
@@ -324,6 +342,9 @@ public:
     std::string add_fheight(const std::string &arg);
     void clear_markers();
 
+    void write_maplines(writer &) const;
+    void read_maplines(reader&);
+
 #ifdef USE_TILE
     std::string add_floortile(const std::string &s);
     std::string add_rocktile(const std::string &s);
@@ -477,14 +498,14 @@ private:
     struct overlay_def
     {
         overlay_def() :
-            colour(0), rocktile(0), floortile(0), tile(0),
+            colour(0), rocktile(""), floortile(""), tile(""),
             no_random(false), last_tile(false), property(0), height(INVALID_HEIGHT),
             keyspec_idx(0)
         {}
         int colour;
-        int rocktile;
-        int floortile;
-        int tile;
+        std::string rocktile;
+        std::string floortile;
+        std::string tile;
         bool no_random;
         bool last_tile;
         int property;
@@ -500,7 +521,7 @@ private:
 
     enum
     {
-        SUBVAULT_GLYPH = 1
+        SUBVAULT_GLYPH = 1,
     };
 
     int map_width;
@@ -515,11 +536,14 @@ enum item_spec_type
     ISPEC_DAMAGED = -4,
     ISPEC_BAD     = -5,
     ISPEC_RANDART = -6,
-    ISPEC_ACQUIREMENT = -9
+    ISPEC_MUNDANE = -7,
+    ISPEC_ACQUIREMENT = -9,
 };
 
-struct item_spec
+class mons_spec;
+class item_spec
 {
+public:
     int genweight;
 
     object_class_type base_type;
@@ -529,6 +553,7 @@ struct item_spec
     int allow_uniques;
     int level;
     int race;
+    int item_special;
     int qty;
     int acquirement_source;
     level_id place;
@@ -538,10 +563,25 @@ struct item_spec
 
     item_spec() : genweight(10), base_type(OBJ_RANDOM), sub_type(OBJ_RANDOM),
         plus(-1), plus2(-1), ego(0), allow_uniques(1), level(-1),
-        race(MAKE_ITEM_RANDOM_RACE), qty(0), acquirement_source(0),
-        place(), props()
+        race(MAKE_ITEM_RANDOM_RACE), item_special(0), qty(0),
+        acquirement_source(0), place(), props(),
+        _corpse_monster_spec(NULL)
     {
     }
+
+    item_spec(const item_spec &other);
+    item_spec &operator = (const item_spec &other);
+    ~item_spec();
+
+    bool corpselike() const;
+    const mons_spec &corpse_monster_spec() const;
+    void set_corpse_monster_spec(const mons_spec &spec);
+
+private:
+    mons_spec *_corpse_monster_spec;
+
+private:
+    void release_corpse_monster_spec();
 };
 typedef std::vector<item_spec> item_spec_list;
 
@@ -553,6 +593,8 @@ public:
     void clear();
 
     item_spec get_item(int index);
+    item_spec random_item ();
+    item_spec random_item_weighted ();
     size_t size() const { return items.size(); }
     bool empty() const { return items.empty(); }
 
@@ -576,11 +618,15 @@ private:
 private:
     item_spec item_by_specifier(const std::string &spec);
     item_spec_slot parse_item_spec(std::string spec);
+    void build_deck_spec(std::string s, item_spec* spec);
     item_spec parse_single_spec(std::string s);
     int parse_acquirement_source(const std::string &source);
     void parse_raw_name(std::string name, item_spec &spec);
     void parse_random_by_class(std::string c, item_spec &spec);
     item_spec pick_item(item_spec_slot &slot);
+    item_spec parse_corpse_spec(item_spec &result, std::string s);
+    bool monster_corpse_is_valid(monster_type *, const std::string &name,
+                                 bool corpse, bool skeleton, bool chunk);
 
 private:
     std::vector<item_spec_slot> items;
@@ -590,30 +636,34 @@ private:
 class mons_spec
 {
  public:
-    int  mid;
+    int mid;
     level_id place;
     monster_type monbase;     // Base monster for zombies and dracs.
     mon_attitude_type attitude;
-    int  number;              // Head count for hydras, etc.
-    int  quantity;            // Number of monsters (usually 1).
-    int  genweight, mlevel;
+    int number;               // Head count for hydras, etc.
+    int quantity;             // Number of monsters (usually 1).
+    int genweight, mlevel;
     bool fix_mons;
     bool generate_awake;
     bool patrolling;
     bool band;
-    int  colour;
-    int  hd;
-    int  hp;
-    int  abjuration_duration;
-    int  summon_type;
+    int colour;
+
+    god_type god;
+    bool god_gift;
+
+    int hd;
+    int hp;
+    int abjuration_duration;
+    int summon_type;
 
     item_list items;
     std::string monname;
     std::string non_actor_summoner;
 
     bool explicit_spells;
-    monster_spells spells;
-    unsigned long extra_monster_flags;
+    std::vector<monster_spells> spells;
+    uint64_t extra_monster_flags;
 
     CrawlHashTable props;
 
@@ -625,9 +675,10 @@ class mons_spec
         : mid(id), place(), monbase(base), attitude(ATT_HOSTILE), number(num),
           quantity(1), genweight(gw), mlevel(ml), fix_mons(_fixmons),
           generate_awake(awaken), patrolling(false), band(false),
-          colour(BLACK), hd(0), hp(0), abjuration_duration(0), summon_type(0),
-          items(), monname(""), non_actor_summoner(""), explicit_spells(false),
-          spells(), extra_monster_flags(0L), props()
+          colour(BLACK), god(GOD_NO_GOD), god_gift(false), hd(0), hp(0),
+          abjuration_duration(0), summon_type(0), items(), monname(""),
+          non_actor_summoner(""), explicit_spells(false), spells(),
+          extra_monster_flags(0), props()
     {
     }
 };
@@ -680,7 +731,7 @@ private:
     mons_spec get_zombified_monster(const std::string &name,
                                     monster_type zomb) const;
     mons_spec_slot parse_mons_spec(std::string spec);
-    void parse_mons_spells(mons_spec &slot, const std::string &spells);
+    void parse_mons_spells(mons_spec &slot, std::vector<std::string> &spells);
     mons_spec pick_monster(mons_spec_slot &slot);
     int fix_demon(int id) const;
     bool check_mimic(const std::string &s, int *mid, bool *fix) const;
@@ -690,19 +741,90 @@ private:
     std::string error;
 };
 
+/**
+ * @class shop_spec
+ * @ingroup mapdef
+ * @brief Specify how to create a shop.
+ *
+ * This specification struct is used when converting a vault-specified shop from
+ * a string into something the builder can use to actually create and place a
+ * shop.
+**/
+struct shop_spec
+{
+    shop_type sh_type;  /**< One of the shop_type enum values. */
+
+    std::string name;   /**< If provided, this is apostrophised and used as the
+                          *  shop keeper's name, ie, Plog as a name becomes
+                          *  Plog's. */
+
+    std::string type;   /**< If provided, this is used as the shop type name,
+                          *  ie, Hide, Antique, Wand, etc. */
+
+    std::string suffix; /**< If provided, this is used as the shop suffix,
+                          *  ie, Shop, Boutique, Parlour, etc. */
+
+    int greed;          /**< If provided, this value is used for price
+                          *  calculation. The higher the greed, the more
+                          *  inflation applied to prices. */
+
+    int num_items;      /**< Cap the number of items in a shop at this. */
+
+    item_list items;    /**< If provided, and `use_all` is false, items will be
+                          *  selected at random from the list and used to
+                          *  populate the shop. If `use_all` is true, the items
+                          *  contained will be placed precisely and in order. If
+                          *  the number of items contained is less than
+                          *  specified, random items according to the shop's
+                          *  type will be used to fill in the rest of the stock.
+                          *  */
+
+    bool use_all;       /**< True if all items in `items` should be used. */
+
+    shop_spec (shop_type sh, std::string n="", std::string t="",
+               std::string s="", int g=-1, int ni=-1, bool u=false)
+        : sh_type(sh), name(n), type(t), suffix(s),
+          greed(g), num_items(ni), items(), use_all(u) { }
+};
+
+/**
+ * @class trap_spec
+ * @ingroup mapdef
+ * @brief Specify how to create a trap.
+ *
+ * This specification struct is used when converting a vault-specified trap
+ * string into something that the builder can use to place a trap.
+**/
+struct trap_spec
+{
+    trap_type tr_type; /*> One of the trap_type enum values. */
+    trap_spec (trap_type tr)
+        : tr_type(static_cast<trap_type>(tr)) { }
+};
+
+
+/**
+ * @class feature_spec
+ * @ingroup mapdef
+ * @brief Specify how to create a feature.
+ *
+ * This specification struct is used firstly when a feature is specified in
+ * vault code (any feature), and secondly, if that feature is either a trap or a
+ * shop, as a container for a std::auto_ptr to that shop_spec or trap_spec.
+**/
 struct feature_spec
 {
-    int genweight;
-    int feat;
-    int shop;
-    int trap;
-    int glyph;
+    int genweight;                 /**> The weight of this specific feature. */
+    int feat;                      /**> The specific feature being placed. */
+    std::auto_ptr<shop_spec> shop; /**> A pointer to a shop_spec. */
+    std::auto_ptr<trap_spec> trap; /**> A pointer to a trap_spec. */
+    int glyph;                     /**> What glyph to use instead. */
 
-    feature_spec(int f, int wt = 10)
-        : genweight(wt), feat(f), shop(-1),
-          trap(-1), glyph(-1)
-    { }
-    feature_spec() : genweight(0), feat(0), shop(-1), trap(-1), glyph(-1) { }
+    feature_spec();
+    feature_spec(int f, int wt = 10);
+    feature_spec(const feature_spec& other);
+    feature_spec& operator = (const feature_spec& other);
+    void init_with (const feature_spec& other);
 };
 
 typedef std::vector<feature_spec> feature_spec_list;
@@ -772,17 +894,20 @@ private:
     feature_spec parse_trap(std::string s, int weight);
 };
 
-typedef std::vector<level_range> depth_ranges;
-
 class map_def;
-struct dlua_set_map
+class dlua_set_map
 {
+public:
     dlua_set_map(map_def *map);
     ~dlua_set_map();
+private:
+    std::auto_ptr<lua_datum> old_map;
 };
 
 class map_def;
-dungeon_feature_type map_feature_at(map_def *map, const coord_def &c, int rawfeat);
+dungeon_feature_type map_feature_at(map_def *map,
+                                    const coord_def &c,
+                                    int rawfeat);
 
 struct map_file_place
 {
@@ -801,6 +926,137 @@ struct map_file_place
     }
 };
 
+const int DEFAULT_CHANCE_PRIORITY = 100;
+struct map_chance
+{
+    int chance_priority;
+    int chance;
+    map_chance() : chance_priority(-1), chance(-1) { }
+    map_chance(int _priority, int _chance)
+        : chance_priority(_priority), chance(_chance) { }
+    map_chance(int _chance)
+        : chance_priority(DEFAULT_CHANCE_PRIORITY), chance(_chance) { }
+    bool valid() const { return chance_priority >= 0 && chance >= 0; }
+    bool dummy_chance() const { return chance_priority == 0 && chance >= 0; }
+    std::string describe() const;
+    // Returns true if the vault makes the random CHANCE_ROLL.
+    bool roll() const;
+    void write(writer &) const;
+    void read(reader &);
+};
+
+// For the bison parser's token union:
+struct map_chance_pair
+{
+   int priority;
+   int chance;
+};
+
+typedef std::vector<level_range> depth_ranges_v;
+class depth_ranges
+{
+private:
+    depth_ranges_v depths;
+public:
+    static depth_ranges parse_depth_ranges(
+        const std::string &depth_ranges_string);
+    void read(reader &);
+    void write(writer &) const;
+    void clear() { depths.clear(); }
+    bool empty() const { return depths.empty(); }
+    bool is_usable_in(const level_id &lid) const;
+    void add_depth(const level_range &range) { depths.push_back(range); }
+    void add_depths(const depth_ranges &other_ranges);
+    std::string describe() const;
+};
+
+template <typename X>
+struct depth_range_X
+{
+    depth_ranges depths;
+    X depth_thing;
+    depth_range_X() : depths(), depth_thing() { }
+    depth_range_X(const std::string &depth_range_string, const X &thing)
+        : depths(depth_ranges::parse_depth_ranges(depth_range_string)),
+          depth_thing(thing)
+    {
+    }
+    bool is_usable_in(const level_id &lid) const
+    {
+        return depths.is_usable_in(lid);
+    }
+    template <typename reader_fn_type>
+    static depth_range_X read(reader &inf, reader_fn_type reader_fn)
+    {
+        depth_range_X range_x;
+        range_x.depths.read(inf);
+        range_x.depth_thing = reader_fn(inf);
+        return range_x;
+    }
+    template <typename writer_fn_type>
+    void write(writer &outf, writer_fn_type writer_fn) const
+    {
+        depths.write(outf);
+        writer_fn(outf, depth_thing);
+    }
+};
+
+template <typename X>
+class depth_ranges_X
+{
+private:
+    typedef std::vector<depth_range_X<X> > depth_range_X_v;
+
+    X default_thing;
+    depth_range_X_v depth_range_Xs;
+public:
+    depth_ranges_X() : default_thing(), depth_range_Xs() { }
+    depth_ranges_X(const X &_default_thing)
+        : default_thing(_default_thing), depth_range_Xs() { }
+    void clear(const X &_default_X = X())
+    {
+        depth_range_Xs.clear();
+        set_default(_default_X);
+    }
+    void set_default(const X &_default_X)
+    {
+        default_thing = _default_X;
+    }
+    X get_default() const { return default_thing; }
+    void add_range(const std::string &depth_range_string,
+                   const X &thing)
+    {
+        depth_range_Xs.push_back(depth_range_X<X>(depth_range_string, thing));
+    }
+    X depth_value(const level_id &lid) const
+    {
+        typename depth_range_X_v::const_iterator i = depth_range_Xs.begin();
+        for ( ; i != depth_range_Xs.end(); ++i)
+            if (i->is_usable_in(lid))
+                return i->depth_thing;
+        return default_thing;
+    }
+    template <typename reader_fn_type>
+    static depth_ranges_X read(reader &inf, reader_fn_type reader_fn)
+    {
+        depth_ranges_X ranges;
+        ranges.clear(reader_fn(inf));
+        const int count = unmarshallShort(inf);
+        for (int i = 0; i < count; ++i)
+            ranges.depth_range_Xs.push_back(
+                depth_range_X<X>::read(inf, reader_fn));
+        return ranges;
+    }
+    template <typename writer_fn_type>
+    void write(writer &outf, writer_fn_type writer_fn) const
+    {
+        writer_fn(outf, default_thing);
+        marshallShort(outf, depth_range_Xs.size());
+        for (int i = 0, size = depth_range_Xs.size(); i < size; ++i)
+            depth_range_Xs[i].write(outf, writer_fn);
+    }
+};
+
 /////////////////////////////////////////////////////////////////////////////
 // map_def: map definitions for maps loaded from .des files.
 //
@@ -830,15 +1086,20 @@ class map_def
 {
 public:
     std::string     name;
+    // Description for the map that can be shown to players.
+    std::string     description;
     std::string     tags;
     level_id        place;
 
     depth_ranges     depths;
     map_section_type orient;
 
-    int              chance_priority;
-    int              chance;
-    int              weight;
+    typedef depth_ranges_X<map_chance> range_chance_t;
+    typedef depth_ranges_X<int> range_weight_t;
+
+    range_chance_t   _chance;
+    range_weight_t   _weight;
+
     int              weight_depth_mult;
     int              weight_depth_div;
 
@@ -864,14 +1125,14 @@ public:
 
     map_def         *original;
 
-    unsigned char   rock_colour, floor_colour;
-    int             rock_tile, floor_tile;
+    uint8_t         rock_colour, floor_colour;
+    std::string     rock_tile, floor_tile;
 
     dungeon_feature_type border_fill_type;
 private:
     // This map has been loaded from an index, and not fully realised.
     bool            index_only;
-    long            cache_offset;
+    mutable long    cache_offset;
     std::string     file;
 
     typedef Matrix<bool> subvault_mask;
@@ -882,12 +1143,21 @@ private:
 
 public:
     map_def();
+
+    std::string desc_or_name() const;
+
+    std::string describe() const;
     void init();
     void reinit();
 
     void load();
+    void strip();
+
+    int weight(const level_id &lid) const;
+    map_chance chance(const level_id &lid) const;
 
     bool in_map(const coord_def &p) const;
+    bool map_already_used() const;
 
     coord_def size() const { return coord_def(map.width(), map.height()); }
 
@@ -896,13 +1166,20 @@ public:
     coord_def find_first_glyph(const std::string &glyphs) const;
 
     void write_index(writer&) const;
-    void write_full(writer&);
+    void write_full(writer&) const;
+    void write_maplines(writer &) const;
 
     void read_index(reader&);
-    void read_full(reader&);
+    void read_full(reader&, bool check_cache_version);
+    void read_maplines(reader&);
 
     void set_file(const std::string &s);
     std::string run_lua(bool skip_main);
+    bool run_hook(const std::string &hook_name, bool die_on_lua_error = false);
+    bool run_postplace_hook(bool die_on_lua_error = false);
+    void copy_hooks_from(const map_def &other_map,
+                         const std::string &hook_name);
+
 
     // Returns true if the validation passed.
     bool test_lua_validate(bool croak = false);
@@ -913,7 +1190,7 @@ public:
     // Executes post-generation lua code.
     bool run_lua_epilogue(bool croak = false);
 
-    std::string validate_map_def();
+    std::string validate_map_def(const depth_ranges &);
     std::string validate_temple_map();
     // Returns true if this map is in the middle of validation.
     bool is_validating() const { return (validating_map_flag); }
@@ -935,8 +1212,7 @@ public:
 
     bool has_depth() const;
     void add_depth(const level_range &depth);
-    void add_depths(depth_ranges::const_iterator s,
-                    depth_ranges::const_iterator e);
+    void add_depths(const depth_ranges &depth);
 
     bool can_dock(map_section_type) const;
     coord_def dock_pos(map_section_type) const;
@@ -948,9 +1224,20 @@ public:
     std::vector<coord_def> anchor_points() const;
 
     bool is_minivault() const;
+    bool is_overwritable_layout() const;
     bool has_tag(const std::string &tag) const;
     bool has_tag_prefix(const std::string &tag) const;
     bool has_tag_suffix(const std::string &suffix) const;
+
+    template <typename TagIterator>
+    bool has_any_tag(TagIterator begin, TagIterator end) const
+    {
+        for ( ; begin != end; ++begin)
+            if (has_tag(*begin))
+                return true;
+        return false;
+    }
+
     std::vector<std::string> get_tags() const;
 
     std::vector<std::string> get_shuffle_strings() const;
@@ -994,16 +1281,18 @@ public:
     };
 
 private:
-
     void write_depth_ranges(writer&) const;
     void read_depth_ranges(reader&);
     bool test_lua_boolchunk(dlua_chunk &, bool def = false, bool croak = false);
     std::string rewrite_chunk_errors(const std::string &s) const;
     std::string apply_subvault(string_spec &);
+    std::string validate_map_placeable();
 };
 
 const int CHANCE_ROLL = 10000;
 
+void map_register_flag(const std::string &flag);
+
 std::string escape_string(std::string in, const std::string &toesc,
                           const std::string &escapewith);
 
@@ -1015,4 +1304,5 @@ std::string mapdef_split_key_item(const std::string &s,
 
 const char *map_section_name(int msect);
 
+int store_tilename_get_index(const std::string tilename);
 #endif
