@@ -1,8 +1,7 @@
-/*
- *  File:       mapmark.cc
- *  Summary:    Level markers (annotations).
- *  Created by: dshaligram on Sat Jul 21 12:17:29 2007 UTC
- */
+/**
+ * @file
+ * @brief Level markers (annotations).
+**/
 
 #include "AppHdr.h"
 
@@ -31,6 +30,9 @@ map_marker::marker_reader map_marker::readers[NUM_MAP_MARKER_TYPES] =
     &map_lua_marker::read,
     &map_corruption_marker::read,
     &map_wiz_props_marker::read,
+    &map_tomb_marker::read,
+    &map_malign_gateway_marker::read,
+    &map_phoenix_marker::read
 };
 
 map_marker::marker_parser map_marker::parsers[NUM_MAP_MARKER_TYPES] =
@@ -38,6 +40,7 @@ map_marker::marker_parser map_marker::parsers[NUM_MAP_MARKER_TYPES] =
     &map_feature_marker::parse,
     &map_lua_marker::parse,
     NULL,
+    NULL,
     NULL
 };
 
@@ -64,7 +67,7 @@ void map_marker::read(reader &inf)
 {
     // Don't read type! The type has to be read by someone who knows how
     // to look up the unmarshall function.
-    unmarshallCoord(inf, pos);
+    pos = unmarshallCoord(inf);
 }
 
 std::string map_marker::property(const std::string &pname) const
@@ -74,9 +77,9 @@ std::string map_marker::property(const std::string &pname) const
 
 map_marker *map_marker::read_marker(reader &inf)
 {
-    const map_marker_type type =
+    const map_marker_type mtype =
         static_cast<map_marker_type>(unmarshallShort(inf));
-    return readers[type]? (*readers[type])(inf, type) : NULL;
+    return readers[mtype]? (*readers[mtype])(inf, mtype) : NULL;
 }
 
 map_marker *map_marker::parse_marker(
@@ -251,18 +254,18 @@ void map_lua_marker::write(writer &outf) const
 
     // Call dlua_marker_function(table, 'read')
     lua_pushstring(dlua, "read");
-    if (!dlua.callfn("dlua_marker_function", 2, 1))
+    if (!dlua.callfn("dlua_marker_reader_name", 2, 1))
         end(1, false, "lua_marker: write error: %s", dlua.error.c_str());
 
-    // Right, what's on top should be a function. Save it.
-    dlua_chunk reader(dlua);
-    if (!reader.error.empty())
-        end(1, false, "lua_marker: couldn't save read function: %s",
-            reader.error.c_str());
+    // Right, what's on top should be a table name. Save it.
+    if (!lua_isstring(dlua, -1))
+        end(1, false, "Expected marker class name (string) to save, got %s",
+            lua_typename(dlua, lua_type(dlua, -1)));
 
-    marshallString(outf, reader.compiled_chunk());
+    const std::string marker_class(lua_tostring(dlua, -1));
+    marshallString(outf, marker_class);
 
-    // Okay, saved the reader. Now ask the writer to do its thing.
+    // Okay, saved the marker's class. Now ask the writer to do its thing.
 
     // Call: dlua_marker_method(table, fname, marker)
     get_table();
@@ -281,13 +284,8 @@ void map_lua_marker::read(reader &inf)
     if (!(initialised = unmarshallByte(inf)))
         return;
 
-    lua_stack_cleaner cln(dlua);
-    // Read the Lua chunk we saved.
-    const std::string compiled = unmarshallString(inf, LUA_CHUNK_MAX_SIZE);
-
-    dlua_chunk chunk = dlua_chunk::precompiled(compiled);
-    if (chunk.load(dlua))
-        end(1, false, "lua_marker::read error: %s", chunk.error.c_str());
+    const std::string marker_class = unmarshallString(inf);
+    lua_pushstring(dlua, marker_class.c_str());
     dlua_push_userdata(dlua, this, MAPMARK_METATABLE);
     lua_pushlightuserdata(dlua, &inf);
     if (!dlua.callfn("dlua_marker_read", 3, 1))
@@ -441,7 +439,7 @@ map_marker *map_lua_marker::parse(
 
 map_corruption_marker::map_corruption_marker(const coord_def &p,
                                              int dur)
-    : map_marker(MAT_CORRUPTION_NEXUS, p), duration(dur), radius(0)
+    : map_marker(MAT_CORRUPTION_NEXUS, p), duration(dur)
 {
 }
 
@@ -449,14 +447,12 @@ void map_corruption_marker::write(writer &out) const
 {
     map_marker::write(out);
     marshallShort(out, duration);
-    marshallShort(out, radius);
 }
 
 void map_corruption_marker::read(reader &in)
 {
     map_marker::read(in);
     duration = unmarshallShort(in);
-    radius   = unmarshallShort(in);
 }
 
 map_marker *map_corruption_marker::read(reader &in, map_marker_type)
@@ -469,7 +465,6 @@ map_marker *map_corruption_marker::read(reader &in, map_marker_type)
 map_marker *map_corruption_marker::clone() const
 {
     map_corruption_marker *mark = new map_corruption_marker(pos, duration);
-    mark->radius = radius;
     return (mark);
 }
 
@@ -565,13 +560,164 @@ std::string map_wiz_props_marker::debug_describe() const
 }
 
 //////////////////////////////////////////////////////////////////////////
-// Map markers in env.
+// map_tomb_marker
 
-map_markers::map_markers() : markers()
+map_tomb_marker::map_tomb_marker(const coord_def &p,
+                                 int dur, int src, int targ)
+    : map_marker(MAT_TOMB, p), duration(dur), source(src), target(targ)
 {
 }
 
-map_markers::map_markers(const map_markers &c) : markers()
+void map_tomb_marker::write(writer &out) const
+{
+    map_marker::write(out);
+    marshallShort(out, duration);
+    marshallShort(out, source);
+    marshallShort(out, target);
+}
+
+void map_tomb_marker::read(reader &in)
+{
+    map_marker::read(in);
+    duration = unmarshallShort(in);
+    source   = unmarshallShort(in);
+    target   = unmarshallShort(in);
+}
+
+map_marker *map_tomb_marker::read(reader &in, map_marker_type)
+{
+    map_tomb_marker *mc = new map_tomb_marker();
+    mc->read(in);
+    return (mc);
+}
+
+map_marker *map_tomb_marker::clone() const
+{
+    map_tomb_marker *mark = new map_tomb_marker(pos, duration,
+                                                source, target);
+    return (mark);
+}
+
+std::string map_tomb_marker::debug_describe() const
+{
+    return make_stringf("Tomb (%d, %d, %d)", duration,
+                                             source, target);
+}
+
+//////////////////////////////////////////////////////////////////////////
+// map_malign_gateway_marker
+
+map_malign_gateway_marker::map_malign_gateway_marker(const coord_def &p,
+                                 int dur, bool ip, std::string sum, beh_type b,
+                                 god_type gd, int pow)
+    : map_marker(MAT_MALIGN, p), duration(dur), is_player(ip), monster_summoned(false),
+      summoner_string(sum), behaviour(b), god(gd), power(pow)
+{
+}
+
+void map_malign_gateway_marker::write(writer &out) const
+{
+    map_marker::write(out);
+    marshallShort(out, duration);
+    marshallBoolean(out, is_player);
+    marshallBoolean(out, monster_summoned);
+    marshallString(out, summoner_string);
+    marshallUByte(out, behaviour);
+    marshallUByte(out, god);
+    marshallShort(out, power);
+}
+
+void map_malign_gateway_marker::read(reader &in)
+{
+    map_marker::read(in);
+    duration  = unmarshallShort(in);
+    is_player = unmarshallBoolean(in);
+
+    monster_summoned = unmarshallBoolean(in);
+    summoner_string = unmarshallString(in);
+    behaviour = static_cast<beh_type>(unmarshallUByte(in));
+
+    god       = static_cast<god_type>(unmarshallByte(in));
+    power     = unmarshallShort(in);
+}
+
+map_marker *map_malign_gateway_marker::read(reader &in, map_marker_type)
+{
+    map_malign_gateway_marker *mc = new map_malign_gateway_marker();
+    mc->read(in);
+    return (mc);
+}
+
+map_marker *map_malign_gateway_marker::clone() const
+{
+    map_malign_gateway_marker *mark = new map_malign_gateway_marker(pos, duration, is_player, summoner_string, behaviour, god, power);
+    return (mark);
+}
+
+std::string map_malign_gateway_marker::debug_describe() const
+{
+    return make_stringf("Malign gateway (%d, %s)", duration, is_player ? "player" : "monster");
+}
+
+//////////////////////////////////////////////////////////////////////////
+// map_phoenix_marker
+
+map_phoenix_marker::map_phoenix_marker(const coord_def& p,
+                    int tst, int tso, beh_type bh, god_type gd, coord_def cp
+                    )
+    : map_marker(MAT_PHOENIX, p), turn_start(tst), turn_stop(tso),
+            behaviour(bh), god(gd), corpse_pos(cp)
+{
+}
+
+void map_phoenix_marker::write(writer &out) const
+{
+    map_marker::write(out);
+    marshallShort(out, turn_start);
+    marshallShort(out, turn_stop);
+    marshallUByte(out, behaviour);
+    marshallUByte(out, god);
+    marshallCoord(out, corpse_pos);
+}
+
+void map_phoenix_marker::read(reader &in)
+{
+    map_marker::read(in);
+
+    turn_start = unmarshallShort(in);
+    turn_stop = unmarshallShort(in);
+    behaviour = static_cast<beh_type>(unmarshallUByte(in));
+    god       = static_cast<god_type>(unmarshallByte(in));
+    corpse_pos = unmarshallCoord(in);
+}
+
+map_marker *map_phoenix_marker::read(reader &in, map_marker_type)
+{
+    map_phoenix_marker *mc = new map_phoenix_marker();
+    mc->read(in);
+    return (mc);
+}
+
+map_marker *map_phoenix_marker::clone() const
+{
+    map_phoenix_marker *mark = new map_phoenix_marker(pos, turn_start, turn_stop, behaviour, god, corpse_pos);
+    return (mark);
+}
+
+std::string map_phoenix_marker::debug_describe() const
+{
+    return make_stringf("Phoenix marker (%d, %d)", turn_start, turn_stop);
+}
+
+//////////////////////////////////////////////////////////////////////////
+// Map markers in env.
+
+map_markers::map_markers() : markers(), have_inactive_markers(false)
+{
+}
+
+map_markers::map_markers(const map_markers &c)
+  : markers(), have_inactive_markers(false)
 {
     init_from(c);
 }
@@ -596,29 +742,58 @@ void map_markers::init_from(const map_markers &c)
     for (dgn_marker_map::const_iterator i = c.markers.begin();
          i != c.markers.end(); ++i)
     {
-        add( i->second->clone() );
+        add(i->second->clone());
     }
+    have_inactive_markers = c.have_inactive_markers;
+}
+
+void map_markers::clear_need_activate()
+{
+    have_inactive_markers = false;
 }
 
 void map_markers::activate_all(bool verbose)
 {
-    std::vector<map_marker*> to_remove;
-
     for (dgn_marker_map::iterator i = markers.begin();
-         i != markers.end(); ++i)
+         i != markers.end();)
     {
-        i->second->activate(verbose);
-        if (i->second->property("post_activate_remove") != "")
-            to_remove.push_back(i->second);
+        map_marker *marker = i->second;
+        ++i;
+        marker->activate(verbose);
     }
 
-    for (unsigned int i = 0; i < to_remove.size(); i++)
-        remove(to_remove[i]);
+    for (dgn_marker_map::iterator i = markers.begin();
+         i != markers.end();)
+    {
+        map_marker *marker = i->second;
+        ++i;
+        if (!marker->property("post_activate_remove").empty())
+            remove(marker);
+    }
+
+    have_inactive_markers = false;
+}
+
+void map_markers::activate_markers_at(coord_def p)
+{
+    const std::vector<map_marker *> activatees = get_markers_at(p);
+    for (int i = 0, size = activatees.size(); i < size; ++i)
+        activatees[i]->activate();
+
+    const std::vector<map_marker *> active_markers = get_markers_at(p);
+    for (int i = 0, size = active_markers.size(); i < size; ++i)
+    {
+        const std::string prop =
+            active_markers[i]->property("post_activate_remove");
+        if (!prop.empty())
+            remove(active_markers[i]);
+    }
 }
 
 void map_markers::add(map_marker *marker)
 {
     markers.insert(dgn_pos_marker(marker->pos, marker));
+    have_inactive_markers = true;
 }
 
 void map_markers::unlink_marker(const map_marker *marker)
@@ -635,10 +810,17 @@ void map_markers::unlink_marker(const map_marker *marker)
     }
 }
 
+void map_markers::check_empty()
+{
+    if (markers.empty())
+        have_inactive_markers = false;
+}
+
 void map_markers::remove(map_marker *marker)
 {
     unlink_marker(marker);
     delete marker;
+    check_empty();
 }
 
 void map_markers::remove_markers_at(const coord_def &c,
@@ -646,7 +828,7 @@ void map_markers::remove_markers_at(const coord_def &c,
 {
     std::pair<dgn_marker_map::iterator, dgn_marker_map::iterator>
         els = markers.equal_range(c);
-    for (dgn_marker_map::iterator i = els.first; i != els.second; )
+    for (dgn_marker_map::iterator i = els.first; i != els.second;)
     {
         dgn_marker_map::iterator todel = i++;
         if (type == MAT_ANY || todel->second->get_type() == type)
@@ -655,6 +837,7 @@ void map_markers::remove_markers_at(const coord_def &c,
             markers.erase(todel);
         }
     }
+    check_empty();
 }
 
 map_marker *map_markers::find(const coord_def &c, map_marker_type type)
@@ -680,11 +863,12 @@ map_marker *map_markers::find(map_marker_type type)
 
 void map_markers::move(const coord_def &from, const coord_def &to)
 {
+    unwind_bool inactive(have_inactive_markers);
     std::pair<dgn_marker_map::iterator, dgn_marker_map::iterator>
         els = markers.equal_range(from);
 
     std::list<map_marker*> tmarkers;
-    for (dgn_marker_map::iterator i = els.first; i != els.second; )
+    for (dgn_marker_map::iterator i = els.first; i != els.second;)
     {
         dgn_marker_map::iterator curr = i++;
         tmarkers.push_back(curr->second);
@@ -701,6 +885,7 @@ void map_markers::move(const coord_def &from, const coord_def &to)
 
 void map_markers::move_marker(map_marker *marker, const coord_def &to)
 {
+    unwind_bool inactive(have_inactive_markers);
     unlink_marker(marker);
     marker->pos = to;
     add(marker);
@@ -766,12 +951,13 @@ void map_markers::clear()
          i != markers.end(); ++i)
         delete i->second;
     markers.clear();
+    check_empty();
 }
 
 static const long MARKERS_COOKY = 0x17742C32;
 void map_markers::write(writer &outf) const
 {
-    marshallLong(outf, MARKERS_COOKY);
+    marshallInt(outf, MARKERS_COOKY);
 
     std::vector<unsigned char> buf;
 
@@ -784,22 +970,20 @@ void map_markers::write(writer &outf) const
         i->second->write(tmp_outf);
 
         // Write the marker data, prefixed by a size
-        marshallLong(outf, buf.size());
-        for ( std::vector<unsigned char>::const_iterator bi = buf.begin();
-              bi != buf.end(); ++bi )
+        marshallInt(outf, buf.size());
+        for (std::vector<unsigned char>::const_iterator bi = buf.begin();
+              bi != buf.end(); ++bi)
         {
             outf.writeByte(*bi);
         }
     }
 }
 
-void map_markers::read(reader &inf, int minorVersion)
+void map_markers::read(reader &inf)
 {
-    UNUSED(minorVersion);
-
     clear();
 
-    const long cooky = unmarshallLong(inf);
+    const int cooky = unmarshallInt(inf);
     ASSERT(cooky == MARKERS_COOKY);
     UNUSED(cooky);
 
@@ -807,7 +991,7 @@ void map_markers::read(reader &inf, int minorVersion)
     for (int i = 0; i < nmarkers; ++i)
     {
         // used by tools
-        unmarshallLong(inf);
+        unmarshallInt(inf);
         if (map_marker *mark = map_marker::read_marker(inf))
         {
             add(mark);
@@ -893,3 +1077,24 @@ std::vector<map_marker*> find_markers_by_prop(
     }
     return (markers);
 }
+
+///////////////////////////////////////////////////////////////////
+
+// Safely remove all markers and dungeon listeners at the given square.
+void remove_markers_and_listeners_at(coord_def p)
+{
+    // Look for Lua markers on this square that are listening for
+    // non-positional events, (such as bazaar portals listening for
+    // turncount changes) and detach them manually from the dungeon
+    // event dispatcher.
+    const std::vector<map_marker *> markers = env.markers.get_markers_at(p);
+    for (int i = 0, size = markers.size(); i < size; ++i)
+    {
+        if (markers[i]->get_type() == MAT_LUA_MARKER)
+            dungeon_events.remove_listener(
+                dynamic_cast<map_lua_marker*>(markers[i]));
+    }
+
+    env.markers.remove_markers_at(p);
+    dungeon_events.clear_listeners_at(p);
+}
