@@ -1,8 +1,7 @@
-/*
- *  File:       maps.cc
- *  Summary:    Functions used to create vaults.
- *  Written by: Linley Henzell
- */
+/**
+ * @file
+ * @brief Functions used to create vaults.
+**/
 
 #include "AppHdr.h"
 
@@ -22,7 +21,6 @@
 #include "env.h"
 #include "enum.h"
 #include "files.h"
-#include "flood_find.h"
 #ifdef USE_TILE
 #include "initfile.h"
 #endif
@@ -34,16 +32,18 @@
 #include "coord.h"
 #include "random.h"
 #include "state.h"
+#include "syscalls.h"
 #include "tags.h"
 #include "terrain.h"
+#include "tutorial.h"
 
-static int write_vault(map_def &mdef,
-                       vault_placement &,
-                       bool check_place);
-static int apply_vault_definition(
-                        map_def &def,
-                        vault_placement &,
-                        bool check_place);
+static map_section_type write_vault(map_def &mdef,
+                                    vault_placement &,
+                                    bool check_place);
+static map_section_type apply_vault_definition(
+    map_def &def,
+    vault_placement &,
+    bool check_place);
 
 static bool resolve_map(map_def &def);
 
@@ -58,7 +58,6 @@ point_vector map_anchor_points;
 //////////////////////////////////////////////////////////////////////////
 // New style vault definitions
 
-typedef std::vector<map_def> map_vector;
 static map_vector vdefs;
 
 // Parameter array that vault code can use.
@@ -85,13 +84,10 @@ dgn_map_parameters::dgn_map_parameters(const string_vector &parameters)
 
 // Make sure that vault_n, where n is a number, is a vault which can be put
 // anywhere, while other vault names are for specific level ranges, etc.
-int vault_main( vault_placement &place, const map_def *vault,
-                bool check_place)
+map_section_type vault_main(vault_placement &place, const map_def *vault,
+                            bool check_place)
 {
 #ifdef DEBUG_DIAGNOSTICS
-    mprf(MSGCH_DIAGNOSTICS, "Generating level: %s (%d,%d)",
-         vault->name.c_str(), place.pos.x, place.pos.y);
-
     if (crawl_state.map_stat_gen)
         mapgen_report_map_try(*vault);
 #endif
@@ -100,12 +96,12 @@ int vault_main( vault_placement &place, const map_def *vault,
     // level, except for branch entry vaults where dungeon.cc just
     // rejects the vault and places a vanilla entry.
 
-    return (write_vault( const_cast<map_def&>(*vault), place, check_place ));
+    return (write_vault(const_cast<map_def&>(*vault), place, check_place));
 }
 
-static int write_vault(map_def &mdef,
-                       vault_placement &place,
-                       bool check_place)
+static map_section_type write_vault(map_def &mdef,
+                                    vault_placement &place,
+                                    bool check_place)
 {
     mdef.load();
 
@@ -134,8 +130,20 @@ static int write_vault(map_def &mdef,
     return (MAP_NONE);
 }
 
+void dgn_flush_map_environments()
+{
+    // Clean up cached environments.
+    dlua.callfn("dgn_flush_map_environments", 0, 0);
+}
+
+void dgn_flush_map_environment_for(const std::string &mapname)
+{
+    dlua.callfn("dgn_flush_map_environment_for", "s", mapname.c_str());
+}
+
 static bool resolve_map_lua(map_def &map)
 {
+    dgn_flush_map_environment_for(map.name);
     map.reinit();
     std::string err = map.run_lua(true);
     if (!err.empty())
@@ -309,9 +317,21 @@ void fit_region_into_map_bounds(coord_def &pos, const coord_def &size,
 }
 
 // Used for placement of vaults.
-static bool _may_overwrite_feature(const dungeon_feature_type grid,
+static bool _may_overwrite_feature(const coord_def p,
                                    bool water_ok, bool wall_ok = true)
 {
+    // If there's a mask specifying where vaults can be placed, don't
+    // allow stepping outside it.
+    if (Vault_Placement_Mask && !(*Vault_Placement_Mask)(p))
+        return (false);
+
+    // If in the abyss, the placement mask is the only check necessary
+    // for terrain.
+    if (Vault_Placement_Mask && player_in_level_area(LEVEL_ABYSS))
+        return (true);
+
+    const dungeon_feature_type grid = grd(p);
+
     // Deep water grids may be overwritten if water_ok == true.
     if (grid == DNGN_DEEP_WATER)
         return (water_ok);
@@ -327,7 +347,7 @@ static bool _may_overwrite_feature(const dungeon_feature_type grid,
         return (false);
     }
 
-    if (feat_is_wall(grid) || grid == DNGN_TREES)
+    if (feat_is_wall(grid) || feat_is_tree(grid))
         return (wall_ok);
 
     // Otherwise, feel free to clobber this feature.
@@ -343,8 +363,8 @@ bool map_safe_vault_place(const map_def &map,
 
     const bool water_ok =
         map.has_tag("water_ok") || player_in_branch(BRANCH_SWAMP);
-    const std::vector<std::string> &lines = map.map.get_lines();
 
+    const std::vector<std::string> &lines = map.map.get_lines();
     for (rectangle_iterator ri(c, c + size - 1); ri; ++ri)
     {
         const coord_def cp(*ri);
@@ -353,21 +373,20 @@ bool map_safe_vault_place(const map_def &map,
         if (lines[dp.y][dp.x] == ' ')
             continue;
 
-        // Also check adjacent squares for collisions, because being next
-        // to another vault may block off one of this vault's exits.
-        for (int y = -1; y <= 1; ++y)
-            for (int x = -1; x <= 1; ++x)
+        if (!map.has_tag("can_overwrite"))
+        {
+            // Also check adjacent squares for collisions, because being next
+            // to another vault may block off one of this vault's exits.
+            for (adjacent_iterator ai(cp); ai; ++ai)
             {
-                const coord_def vp(x + cp.x, y + cp.y);
-                if (map_bounds(vp) && (dgn_Map_Mask(vp) & MMT_VAULT))
+                if (map_bounds(*ai) && (env.level_map_mask(*ai) & MMT_VAULT))
                     return (false);
             }
-
-        const dungeon_feature_type dfeat = grd(cp);
+        }
 
         // Don't overwrite features other than floor, rock wall, doors,
         // nor water, if !water_ok.
-        if (!_may_overwrite_feature(dfeat, water_ok))
+        if (!_may_overwrite_feature(cp, water_ok))
             return (false);
 
         // Don't overwrite monsters or items, either!
@@ -395,7 +414,7 @@ static bool _connected_minivault_place(const coord_def &c,
         if (lines[ci.y - c.y][ci.x - c.x] == ' ')
             continue;
 
-        if (_may_overwrite_feature(grd(ci), water_ok, false))
+        if (_may_overwrite_feature(ci, water_ok, false))
             return (true);
     }
 
@@ -413,8 +432,8 @@ static coord_def _find_minivault_place(
     // Find a target area which can be safely overwritten.
     for (int tries = 0; tries < 600; ++tries)
     {
-        coord_def v1(random_range( margin, GXM - margin - place.size.x ),
-                     random_range( margin, GYM - margin - place.size.y ));
+        coord_def v1(random_range(margin, GXM - margin - place.size.x),
+                     random_range(margin, GYM - margin - place.size.y));
 
         if (check_place && !map_place_valid(place.map, v1, place.size))
         {
@@ -480,17 +499,17 @@ static bool apply_vault_grid(map_def &def,
     // Floating maps can go anywhere, ask the map_def to suggest a place.
     if (orient == MAP_FLOAT)
     {
-        const bool minivault = def.has_tag("minivault");
+        const bool minivault = def.is_minivault();
         if (map_bounds(place.pos))
         {
             start = place.pos - size / 2;
-            fit_region_into_map_bounds(start, size, minivault ? 2 : 0);
+            fit_region_into_map_bounds(start, size, minivault ? MAPGEN_BORDER : 0);
         }
         else if (minivault)
         {
             start = _find_minivault_place(place, check_place);
             if (map_bounds(start))
-                fit_region_into_map_bounds(start, size, 2);
+                fit_region_into_map_bounds(start, size, MAPGEN_BORDER);
         }
         else
             start = def.float_place();
@@ -513,50 +532,42 @@ static bool apply_vault_grid(map_def &def,
     return (true);
 }
 
-static int apply_vault_definition(
-        map_def &def,
-        vault_placement &place,
-        bool check_place)
+static map_section_type apply_vault_definition(
+    map_def &def,
+    vault_placement &place,
+    bool check_place)
 {
     if (!apply_vault_grid(def, place, check_place))
         return (MAP_NONE);
 
-    int orient = def.orient;
-    if (orient == MAP_NONE)
-        orient = MAP_NORTH;
-
-    return (orient);
+    const map_section_type orient = def.orient;
+    return (orient == MAP_NONE? MAP_NORTH : orient);
 }
 
 ///////////////////////////////////////////////////////////////////////////
 // Map lookups
 
-template <typename I>
-static bool map_has_no_tags(const map_def &map, I begin, I end)
-{
-    for ( ; begin != end; ++begin)
-        if (map.has_tag(*begin))
-            return (false);
-
-    return (true);
-}
-
-static bool vault_unforbidden(const map_def &map)
-{
-    return (you.uniq_map_names.find(map.name) == you.uniq_map_names.end()
-            && Level_Unique_Maps.find(map.name) == Level_Unique_Maps.end()
-            && map_has_no_tags(map, you.uniq_map_tags.begin(),
-                               you.uniq_map_tags.end())
-            && map_has_no_tags(map, Level_Unique_Tags.begin(),
-                               Level_Unique_Tags.end()));
-}
-
 static bool map_matches_layout_type(const map_def &map)
 {
-    if (dgn_Layout_Type.empty() || !map.has_tag_prefix("layout_"))
-        return (true);
+    if (env.level_layout_types.empty() || !map.has_tag_prefix("layout_"))
+        return true;
 
-    return map.has_tag("layout_" + dgn_Layout_Type);
+    for (string_set::const_iterator i = env.level_layout_types.begin();
+         i != env.level_layout_types.end(); ++i)
+    {
+        if (map.has_tag("layout_" + *i))
+            return true;
+    }
+
+    return false;
+}
+
+static bool _map_matches_species(const map_def &map)
+{
+    if (you.species < 0 || you.species >= NUM_SPECIES)
+        return true;
+    return !map.has_tag("no_species_"
+           + lowercase_string(get_species_abbrev(you.species)));
 }
 
 const map_def *find_map_by_name(const std::string &name)
@@ -568,6 +579,14 @@ const map_def *find_map_by_name(const std::string &name)
     return (NULL);
 }
 
+// Discards Lua code loaded by all maps to reduce memory use. If any stripped
+// map is reused, its data will be reloaded from the .dsc
+void strip_all_maps()
+{
+    for (unsigned i = 0, size = vdefs.size(); i < size; ++i)
+        vdefs[i].strip();
+}
+
 std::vector<std::string> find_map_matches(const std::string &name)
 {
     std::vector<std::string> matches;
@@ -578,72 +597,73 @@ std::vector<std::string> find_map_matches(const std::string &name)
     return (matches);
 }
 
-std::vector<map_def> find_maps_for_tag (const std::string tag, bool check_depth,
-                                          bool check_used)
+mapref_vector find_maps_for_tag(const std::string tag,
+                                bool check_depth,
+                                bool check_used)
 {
-    std::vector<map_def> maps;
+    mapref_vector maps;
     level_id place = level_id::current();
 
     for (unsigned i = 0, size = vdefs.size(); i < size; ++i)
     {
-        map_def mapdef = vdefs[i];
+        const map_def &mapdef = vdefs[i];
         if (mapdef.has_tag(tag)
             && !mapdef.has_tag("dummy")
             && (!check_depth || !mapdef.has_depth()
-                 || mapdef.is_usable_in(place))
-            && (!check_used || vault_unforbidden(mapdef)))
+                || mapdef.is_usable_in(place))
+            && (!check_used || !mapdef.map_already_used()))
         {
-                maps.push_back(mapdef);
+            maps.push_back(&mapdef);
         }
     }
-
     return (maps);
 }
 
-int weight_map_vector (std::vector<map_def> maps)
+struct map_selector
 {
-    int weights = 0;
-
-    for (std::vector<map_def>::iterator mi = maps.begin(); mi != maps.end(); ++mi)
-        weights += mi->weight;
-
-    return (weights);
-}
-
-struct map_selector {
 private:
     enum select_type
     {
         PLACE,
         DEPTH,
-        TAG
+        DEPTH_AND_CHANCE,
+        TAG,
     };
 
 public:
     bool accept(const map_def &md) const;
-    void announce(int vault) const;
+    void announce(const map_def *map) const;
 
     bool valid() const
     {
         return (sel == TAG || place.is_valid());
     }
 
-    static map_selector by_place(const level_id &place, bool mini)
+    static map_selector by_place(const level_id &_place, bool _mini)
     {
-        return map_selector(map_selector::PLACE, place, "", mini, false);
+        return map_selector(map_selector::PLACE, _place, "", _mini, false);
     }
 
-    static map_selector by_depth(const level_id &place, bool mini)
+    static map_selector by_depth(const level_id &_place, bool _mini)
     {
-        return map_selector(map_selector::DEPTH, place, "", mini, true);
+        return map_selector(map_selector::DEPTH, _place, "", _mini, true);
     }
 
-    static map_selector by_tag(const std::string &tag,
-                               bool check_depth,
-                               const level_id &place = level_id::current())
+    static map_selector by_depth_chance(const level_id &_place)
     {
-        return map_selector(map_selector::TAG, place, tag,
-                            false, check_depth);
+        return map_selector(map_selector::DEPTH_AND_CHANCE, _place, "", false,
+                            true);
+    }
+
+    static map_selector by_tag(const std::string &_tag,
+                               bool _check_depth,
+                               bool _check_chance,
+                               const level_id &_place = level_id::current())
+    {
+        map_selector msel = map_selector(map_selector::TAG, _place, _tag,
+                                         false, _check_depth);
+        msel.ignore_chance = !_check_chance;
+        return (msel);
     }
 
 private:
@@ -653,10 +673,15 @@ private:
         : ignore_chance(false), preserve_dummy(false),
           sel(_typ), place(_pl), tag(_tag),
           mini(_mini), check_depth(_check_depth),
-          check_layout(sel == DEPTH && place == level_id::current())
+          check_layout((sel == DEPTH || sel == DEPTH_AND_CHANCE)
+                    && place == level_id::current())
     {
+        if (_typ == PLACE)
+            ignore_chance = true;
     }
 
+    bool depth_selectable(const map_def &) const;
+
 public:
     bool ignore_chance;
     bool preserve_dummy;
@@ -668,69 +693,98 @@ public:
     const bool check_layout;
 };
 
+bool map_selector::depth_selectable(const map_def &mapdef) const
+{
+    return (!mapdef.place.is_valid()
+            && mapdef.is_usable_in(place)
+            // Some tagged levels cannot be selected as random
+            // maps in a specific depth:
+            && !mapdef.has_tag_suffix("entry")
+            && !mapdef.has_tag("unrand")
+            && !mapdef.has_tag("place_unique")
+            && !mapdef.has_tag("tutorial")
+            && (!mapdef.has_tag_prefix("temple_")
+                || mapdef.has_tag_prefix("uniq_altar_"))
+            && _map_matches_species(mapdef)
+            && (!check_layout || map_matches_layout_type(mapdef)));
+}
+
 bool map_selector::accept(const map_def &mapdef) const
 {
     switch (sel)
     {
     case PLACE:
+        if (mapdef.has_tag_prefix("tutorial")
+            && (!crawl_state.game_is_tutorial()
+                || !mapdef.has_tag(get_tutorial_map())))
+        {
+            return (false);
+        }
         return (mapdef.is_minivault() == mini
                 && mapdef.place == place
-                && !mapdef.has_tag("layout")
-                && !mapdef.has_tag("place_unique")
-                && (!mapdef.has_tag_prefix("temple_")
-                    || mapdef.has_tag_prefix("uniq_altar_"))
                 && map_matches_layout_type(mapdef)
-                && vault_unforbidden(mapdef));
+                && !mapdef.map_already_used());
+
     case DEPTH:
+    {
+        const map_chance chance(mapdef.chance(place));
         return (mapdef.is_minivault() == mini
-                && !mapdef.place.is_valid()
-                && mapdef.is_usable_in(place)
-                // Some tagged levels cannot be selected by depth. This is
-                // the only thing preventing Pandemonium demon vaults from
-                // showing up in the main dungeon.
-                && !mapdef.has_tag_suffix("entry")
-                && !mapdef.has_tag("pan")
-                && !mapdef.has_tag("unrand")
-                && !mapdef.has_tag("bazaar")
-                && !mapdef.has_tag("layout")
-                && !mapdef.has_tag("place_unique")
-                && (!mapdef.has_tag_prefix("temple_")
-                    || mapdef.has_tag_prefix("uniq_altar_"))
-                && (!check_layout || map_matches_layout_type(mapdef))
-                && vault_unforbidden(mapdef));
+                && (!chance.valid() || chance.dummy_chance())
+                && depth_selectable(mapdef)
+                && !mapdef.map_already_used());
+    }
+
+    case DEPTH_AND_CHANCE:
+    {
+        const map_chance chance(mapdef.chance(place));
+        // Only vaults with valid chance
+        return (chance.valid()
+                && !chance.dummy_chance()
+                && depth_selectable(mapdef)
+                && !mapdef.map_already_used());
+    }
+
     case TAG:
         return (mapdef.has_tag(tag)
-                && (!check_depth || !mapdef.has_depth()
+                && (!check_depth
+                    || !mapdef.has_depth()
                     || mapdef.is_usable_in(place))
+                && _map_matches_species(mapdef)
                 && map_matches_layout_type(mapdef)
-                && vault_unforbidden(mapdef));
+                && !mapdef.map_already_used());
+
     default:
         return (false);
     }
 }
 
-void map_selector::announce(int vault) const
+void map_selector::announce(const map_def *vault) const
 {
 #ifdef DEBUG_DIAGNOSTICS
-    if (vault != -1)
+    if (vault)
     {
-        const char *format =
-            sel == PLACE? "[PLACE] Found map %s for %s" :
-            sel == DEPTH? "[DEPTH] Found random map %s for %s" :
-                          "[TAG] Found map %s tagged '%s'";
+        if (sel == DEPTH_AND_CHANCE)
+        {
+            mprf(MSGCH_DIAGNOSTICS,
+                 "[CHANCE+DEPTH] Found map %s for %s (%s)",
+                 vault->name.c_str(), place.describe().c_str(),
+                 vault->chance(place).describe().c_str());
+        }
+        else
+        {
+            const char *format =
+                sel == PLACE? "[PLACE] Found map %s for %s" :
+                sel == DEPTH? "[DEPTH] Found random map %s for %s" :
+                "[TAG] Found map %s tagged '%s'";
 
-        mprf(MSGCH_DIAGNOSTICS, format,
-             vdefs[vault].name.c_str(),
-             sel == TAG? tag.c_str() : place.describe().c_str());
+            mprf(MSGCH_DIAGNOSTICS, format,
+                 vault->name.c_str(),
+                 sel == TAG? tag.c_str() : place.describe().c_str());
+        }
     }
 #endif
 }
 
-static bool _compare_vault_chance_priority(int a, int b)
-{
-    return (vdefs[b].chance_priority < vdefs[a].chance_priority);
-}
-
 static std::string _vault_chance_tag(const map_def &map)
 {
     if (map.has_tag_prefix("chance_"))
@@ -762,17 +816,96 @@ static vault_indices _eligible_maps_for_selector(const map_selector &sel)
 }
 
 static const map_def *_random_map_by_selector(const map_selector &sel);
-static const map_def *_random_map_in_list(const map_selector &sel,
-                                          const vault_indices &filtered)
+
+static bool _vault_chance_new(const map_def &map,
+                              const level_id &place,
+                              std::set<std::string> &chance_tags)
 {
-    int mapindex = -1;
-    int rollsize = 0;
+    if (map.chance(place).valid())
+    {
+        // There may be several alternatives for a portal
+        // vault that want to be governed by one common
+        // CHANCE. In this case each vault will use a
+        // CHANCE, and a common chance_xxx tag. Pick the
+        // first such vault for the chance roll. Note that
+        // at this point we ignore chance_priority.
+        const std::string tag = _vault_chance_tag(map);
+        if (chance_tags.find(tag) == chance_tags.end())
+        {
+            if (!tag.empty())
+                chance_tags.insert(tag);
+            return (true);
+        }
+    }
+    return (false);
+}
 
-    // First build a list of vaults that could be used:
-    vault_indices eligible;
+class vault_chance_roll_iterator
+{
+public:
+    vault_chance_roll_iterator(const mapref_vector &_maps)
+        : maps(_maps), place(level_id::current()),
+          current(_maps.begin()), end(_maps.end())
+    {
+        find_valid();
+    }
 
+    operator bool () const { return current != end; }
+    const map_def *operator * () const { return *current; }
+    const map_def *operator -> () const { return *current; }
+
+    vault_chance_roll_iterator &operator ++ ()
+    {
+        ++current;
+        find_valid();
+        return (*this);
+    }
+
+    vault_chance_roll_iterator operator ++ (int)
+    {
+        vault_chance_roll_iterator copy(*this);
+        operator ++ ();
+        return (copy);
+    }
+
+private:
+    void find_valid()
+    {
+        while (current != end && !(*current)->chance(place).roll())
+            ++current;
+    }
+
+private:
+    const std::vector<const map_def *> &maps;
+    level_id place;
+    mapref_vector::const_iterator current;
+    mapref_vector::const_iterator end;
+};
+
+static const map_def *_resolve_chance_vault(const map_selector &sel,
+                                            const map_def *map)
+{
+    const std::string chance_tag = _vault_chance_tag(*map);
+    // If this map has a chance_ tag, convert the search into
+    // a lookup for that tag.
+    if (!chance_tag.empty())
+    {
+        map_selector msel = map_selector::by_tag(chance_tag,
+                                                 sel.check_depth,
+                                                 false,
+                                                 sel.place);
+        return _random_map_by_selector(msel);
+    }
+    return (map);
+}
+
+static mapref_vector
+_random_chance_maps_in_list(const map_selector &sel,
+                            const vault_indices &filtered)
+{
     // Vaults that are eligible and have >0 chance.
-    vault_indices chance;
+    mapref_vector chance;
+    mapref_vector chosen_chances;
 
     typedef std::set<std::string> tag_set;
     tag_set chance_tags;
@@ -780,67 +913,68 @@ static const map_def *_random_map_in_list(const map_selector &sel,
     for (unsigned f = 0, size = filtered.size(); f < size; ++f)
     {
         const int i = filtered[f];
-        if (!sel.ignore_chance && vdefs[i].chance > 0)
+        if (!sel.ignore_chance
+            && _vault_chance_new(vdefs[i], sel.place, chance_tags))
         {
-            // There may be several alternatives for a portal
-            // vault that want to be governed by one common
-            // CHANCE. In this case each vault will use a
-            // CHANCE, and a common chance_xxx tag. Pick the
-            // first such vault for the chance roll. Note that
-            // at this point we ignore chance_priority.
-            const std::string tag = _vault_chance_tag(vdefs[i]);
-            if (chance_tags.find(tag) == chance_tags.end())
-            {
-                if (!tag.empty())
-                    chance_tags.insert(tag);
-                chance.push_back(i);
-            }
+            chance.push_back(&vdefs[i]);
+        }
+    }
+
+    for (vault_chance_roll_iterator vc(chance); vc; ++vc)
+        if (const map_def *chosen = _resolve_chance_vault(sel, *vc))
+        {
+            chosen_chances.push_back(chosen);
+            sel.announce(chosen);
+        }
+
+    return (chosen_chances);
+}
+
+static const map_def *
+_random_map_in_list(const map_selector &sel,
+                    const vault_indices &filtered)
+{
+    const map_def *chosen_map = NULL;
+    int rollsize = 0;
+
+    // First build a list of vaults that could be used:
+    mapref_vector eligible;
+
+    // Vaults that are eligible and have >0 chance.
+    mapref_vector chance;
+
+    typedef std::set<std::string> tag_set;
+    tag_set chance_tags;
+
+    for (unsigned f = 0, size = filtered.size(); f < size; ++f)
+    {
+        const int i = filtered[f];
+        if (!sel.ignore_chance && vdefs[i].chance(sel.place).valid())
+        {
+            if (_vault_chance_new(vdefs[i], sel.place, chance_tags))
+                chance.push_back(&vdefs[i]);
         }
         else
         {
-            eligible.push_back(i);
+            eligible.push_back(&vdefs[i]);
         }
     }
 
-    // Sort chances by priority, high priority first.
-    std::sort(chance.begin(), chance.end(), _compare_vault_chance_priority);
-
-    // Check CHANCEs.
-    for (vault_indices::const_iterator i = chance.begin();
-         i != chance.end(); ++i)
-    {
-        const map_def &map(vdefs[*i]);
-        if (random2(CHANCE_ROLL) < map.chance)
+    for (vault_chance_roll_iterator vc(chance); vc; ++vc)
+        if (const map_def *chosen = _resolve_chance_vault(sel, *vc))
         {
-            const std::string chance_tag = _vault_chance_tag(map);
-            // If this map has a chance_ tag, convert the search into
-            // a lookup for that tag.
-            if (!chance_tag.empty())
-            {
-                map_selector msel = map_selector::by_tag(chance_tag,
-                                                         sel.check_depth,
-                                                         sel.place);
-                msel.ignore_chance = true;
-                return _random_map_by_selector(msel);
-            }
-
-            mapindex = *i;
+            chosen_map = chosen;
             break;
         }
-    }
 
-    if (mapindex == -1)
+    if (!chosen_map)
     {
-        int absdepth = 0;
-        if (sel.place.level_type == LEVEL_DUNGEON && sel.place.is_valid())
-            absdepth = sel.place.absdepth();
-
-        for (vault_indices::const_iterator i = eligible.begin();
+        const level_id &here(level_id::current());
+        for (mapref_vector::const_iterator i = eligible.begin();
              i != eligible.end(); ++i)
         {
-            const map_def &map(vdefs[*i]);
-            const int weight = map.weight
-                + absdepth * map.weight_depth_mult / map.weight_depth_div;
+            const map_def &map(**i);
+            const int weight = map.weight(here);
 
             if (weight <= 0)
                 continue;
@@ -848,19 +982,18 @@ static const map_def *_random_map_in_list(const map_selector &sel,
             rollsize += weight;
 
             if (rollsize && x_chance_in_y(weight, rollsize))
-                mapindex = *i;
+                chosen_map = &map;
         }
     }
 
-    if (!sel.preserve_dummy && mapindex != -1
-        && vdefs[mapindex].has_tag("dummy"))
+    if (!sel.preserve_dummy && chosen_map
+        && chosen_map->has_tag("dummy"))
     {
-        mapindex = -1;
+        chosen_map = NULL;
     }
 
-    sel.announce(mapindex);
-
-    return (mapindex == -1? NULL : &vdefs[mapindex]);
+    sel.announce(chosen_map);
+    return (chosen_map);
 }
 
 static const map_def *_random_map_by_selector(const map_selector &sel)
@@ -881,11 +1014,19 @@ const map_def *random_map_in_depth(const level_id &place, bool want_minivault)
         map_selector::by_depth(place, want_minivault));
 }
 
+mapref_vector random_chance_maps_in_depth(const level_id &place)
+{
+    map_selector sel = map_selector::by_depth_chance(place);
+    const vault_indices eligible = _eligible_maps_for_selector(sel);
+    return _random_chance_maps_in_list(sel, eligible);
+}
+
 const map_def *random_map_for_tag(const std::string &tag,
-                                  bool check_depth)
+                                  bool check_depth,
+                                  bool check_chance)
 {
     return _random_map_by_selector(
-        map_selector::by_tag(tag, check_depth));
+        map_selector::by_tag(tag, check_depth, check_chance));
 }
 
 int map_count()
@@ -897,7 +1038,7 @@ int map_count_for_tag(const std::string &tag,
                       bool check_depth)
 {
     return _eligible_maps_for_selector(
-        map_selector::by_tag(tag, check_depth)).size();
+        map_selector::by_tag(tag, check_depth, false)).size();
 }
 
 /////////////////////////////////////////////////////////////////////////////
@@ -934,13 +1075,18 @@ void reset_map_parser()
 
 static bool checked_des_index_dir = false;
 
+static std::string _des_cache_dir(const std::string &relpath = "")
+{
+    return catpath(savedir_versioned_path("des"), relpath);
+}
+
 static void check_des_index_dir()
 {
     if (checked_des_index_dir)
         return;
 
-    std::string desdir = get_savedir_path("des");
-    if (!check_dir("Data file cache", desdir, true))
+    std::string desdir = _des_cache_dir();
+    if (!check_mkdir("Data file cache", &desdir, true))
         end(1, true, "Can't create data file cache: %s", desdir.c_str());
 
     checked_des_index_dir = true;
@@ -951,16 +1097,16 @@ std::string get_descache_path(const std::string &file,
 {
     const std::string basename =
         change_file_extension(get_base_filename(file), ext);
-    return get_savedir_path("des/" + basename);
+    return _des_cache_dir(basename);
 }
 
 static bool verify_file_version(const std::string &file)
 {
-    FILE *fp = fopen(file.c_str(), "rb");
+    FILE *fp = fopen_u(file.c_str(), "rb");
     if (!fp)
         return (false);
     reader inf(fp);
-    const long ver = unmarshallLong(inf);
+    const long ver = unmarshallInt(inf);
     fclose(fp);
 
     return (ver == MAP_CACHE_VERSION);
@@ -980,31 +1126,33 @@ static bool load_map_index(const std::string &base)
 {
     // If there's a global prelude, load that first.
     {
-        FILE *fp = fopen((base + ".lux").c_str(), "rb");
+        FILE *fp = fopen_u((base + ".lux").c_str(), "rb");
         if (fp)
         {
-            reader inf(fp);
+            reader inf(fp, TAG_MINOR_VERSION);
             lc_global_prelude.read(inf);
             fclose(fp);
 
-            global_preludes.push_back( lc_global_prelude );
+            global_preludes.push_back(lc_global_prelude);
         }
     }
 
-    FILE* fp = fopen((base + ".idx").c_str(), "rb");
+    FILE* fp = fopen_u((base + ".idx").c_str(), "rb");
     if (!fp)
         end(1, true, "Unable to read %s", (base + ".idx").c_str());
-    reader inf(fp);
 
+    reader inf(fp, TAG_MINOR_VERSION);
     // Discard version (it's been checked by verify_map_index).
-    (void) unmarshallLong(inf);
+    (void) unmarshallInt(inf);
     const int nmaps = unmarshallShort(inf);
     const int nexist = vdefs.size();
-    vdefs.resize( nexist + nmaps, map_def() );
+    vdefs.resize(nexist + nmaps, map_def());
     for (int i = 0; i < nmaps; ++i)
     {
         map_def &vdef(vdefs[nexist + i]);
         vdef.read_index(inf);
+        vdef.description = unmarshallString(inf);
+
         vdef.set_file(base);
         lc_loaded_maps[vdef.name] = vdef.place_loaded_from;
         vdef.place_loaded_from.clear();
@@ -1050,12 +1198,12 @@ static void write_map_prelude(const std::string &filebase)
     const std::string luafile = filebase + ".lux";
     if (lc_global_prelude.empty())
     {
-        unlink(luafile.c_str());
+        unlink_u(luafile.c_str());
         return;
     }
 
-    FILE *fp = fopen(luafile.c_str(), "wb");
-    writer outf(fp);
+    FILE *fp = fopen_u(luafile.c_str(), "wb");
+    writer outf(luafile, fp);
     lc_global_prelude.write(outf);
     fclose(fp);
 }
@@ -1063,12 +1211,12 @@ static void write_map_prelude(const std::string &filebase)
 static void write_map_full(const std::string &filebase, size_t vs, size_t ve)
 {
     const std::string cfile = filebase + ".dsc";
-    FILE *fp = fopen(cfile.c_str(), "wb");
+    FILE *fp = fopen_u(cfile.c_str(), "wb");
     if (!fp)
         end(1, true, "Unable to open %s for writing", cfile.c_str());
 
-    writer outf(fp);
-    marshallLong(outf, MAP_CACHE_VERSION);
+    writer outf(cfile, fp);
+    marshallInt(outf, MAP_CACHE_VERSION);
     for (size_t i = vs; i < ve; ++i)
         vdefs[i].write_full(outf);
     fclose(fp);
@@ -1077,15 +1225,20 @@ static void write_map_full(const std::string &filebase, size_t vs, size_t ve)
 static void write_map_index(const std::string &filebase, size_t vs, size_t ve)
 {
     const std::string cfile = filebase + ".idx";
-    FILE *fp = fopen(cfile.c_str(), "wb");
+    FILE *fp = fopen_u(cfile.c_str(), "wb");
     if (!fp)
         end(1, true, "Unable to open %s for writing", cfile.c_str());
 
-    writer outf(fp);
-    marshallLong(outf, MAP_CACHE_VERSION);
+    writer outf(cfile, fp);
+    marshallInt(outf, MAP_CACHE_VERSION);
     marshallShort(outf, ve > vs? ve - vs : 0);
     for (size_t i = vs; i < ve; ++i)
+    {
         vdefs[i].write_index(outf);
+        marshallString(outf, vdefs[i].description);
+        vdefs[i].place_loaded_from.clear();
+        vdefs[i].strip();
+    }
     fclose(fp);
 }
 
@@ -1113,7 +1266,7 @@ static void parse_maps(const std::string &s)
     if (load_map_cache(s))
         return;
 
-    FILE *dat = fopen(s.c_str(), "r");
+    FILE *dat = fopen_u(s.c_str(), "r");
     if (!dat)
         end(1, true, "Failed to open %s for reading", s.c_str());
 
@@ -1123,40 +1276,63 @@ static void parse_maps(const std::string &s)
     extern FILE *yyin;
     yyin = dat;
 
-    size_t file_start = vdefs.size();
+    const size_t file_start = vdefs.size();
     yyparse();
     fclose(dat);
 
-    global_preludes.push_back( lc_global_prelude );
+    global_preludes.push_back(lc_global_prelude);
     write_map_cache(s, file_start, vdefs.size());
 }
 
 void read_map(const std::string &file)
 {
-    parse_maps( lc_desfile = datafile_path(file) );
+    parse_maps(lc_desfile = datafile_path(file));
+    dgn_flush_map_environments();
+    // Force GC to prevent heap from swelling unnecessarily.
+    dlua.gc();
 }
 
 void read_maps()
 {
-    if (dlua.execfile("clua/loadmaps.lua", true, true))
+    if (dlua.execfile("clua/loadmaps.lua", true, true, true))
         end(1, false, "Lua error: %s", dlua.error.c_str());
 
-    // Clean up cached environments.
-    dlua.callfn("dgn_flush_map_environments", 0, 0);
     lc_loaded_maps.clear();
-
     sanity_check_maps();
 }
 
-void add_parsed_map( const map_def &md )
+// If a .dsc file has been changed under the running Crawl, discard
+// all map knowledge and reload maps. This will not affect maps that
+// have already been used, but it might trigger exciting happenings if
+// the new maps fail sanity checks or remove maps that the game
+// expects to be present.
+void reread_maps()
+{
+    dprf("reread_maps:: discarding %u existing maps",
+         vdefs.size());
+
+    // BOOM!
+    vdefs.clear();
+    map_files_read.clear();
+    read_maps();
+}
+
+void dump_map(const map_def &map)
+{
+    if (crawl_state.dump_maps)
+        fprintf(stderr, "\n----------------------------------------\n%s\n",
+                map.describe().c_str());
+}
+
+void add_parsed_map(const map_def &md)
 {
     map_def map = md;
 
     map.fixup();
-    vdefs.push_back( map );
+    vdefs.push_back(map);
 }
 
-void run_map_preludes()
+void run_map_global_preludes()
 {
     for (int i = 0, size = global_preludes.size(); i < size; ++i)
     {
@@ -1167,6 +1343,10 @@ void run_map_preludes()
                 mprf(MSGCH_ERROR, "Lua error: %s", chunk.orig_error().c_str());
         }
     }
+}
+
+void run_map_local_preludes()
+{
     for (int i = 0, size = vdefs.size(); i < size; ++i)
     {
         if (!vdefs[i].prelude.empty())
