@@ -1,14 +1,7 @@
-/*
- *  File:       message.cc
- *  Summary:    Functions used to print messages.
- *
- * Todo:
- *   - change uses of cancelable_get_line to msgwin_get_line
- *   - Handle resizing properly, in particular initial resize.
- *
- * Maybe:
- *   - Redraw message window at same places that cause refresh?
- */
+/**
+ * @file
+ * @brief Functions used to print messages.
+**/
 
 #include "AppHdr.h"
 
@@ -33,11 +26,11 @@
 #include "stuff.h"
 #include "areas.h"
 #include "tags.h"
+#include "tagstring.h"
 #include "travel.h"
-#include "tutorial.h"
+#include "hints.h"
 #include "view.h"
 #include "shout.h"
-#include "viewchar.h"
 #include "viewgeom.h"
 
 #include <sstream>
@@ -84,7 +77,7 @@ struct message_item
           turn(you.num_turns)
     {
          // Don't join long messages.
-         join = jn && pure_text().length() < 40;
+         join = jn && strwidth(pure_text()) < 40;
     }
 
     // Constructor for restored messages.
@@ -151,8 +144,8 @@ struct message_item
                     seplen++;
                 }
                 sep += " </lightgrey>";
-                if (pure_text().length() + seplen + other.pure_text().length()
-                    > msgwin_line_length())
+                if (strwidth(pure_text()) + seplen + strwidth(other.pure_text())
+                    > (int)msgwin_line_length())
                 {
                     return false;
                 }
@@ -195,6 +188,13 @@ class circ_vec
 public:
     circ_vec() : end(0) {}
 
+    void clear()
+    {
+        end = 0;
+        for (int i = 0; i < SIZE; ++i)
+            data[i] = T();
+    }
+
     int size() const
     {
         return SIZE;
@@ -240,7 +240,7 @@ enum prefix_type
     P_NEW_CMD, // new command, but no new turn
     P_NEW_TURN,
     P_FULL_MORE,   // single-character more prompt (full window)
-    P_OTHER_MORE   // the other type of --more-- prompt
+    P_OTHER_MORE,  // the other type of --more-- prompt
 };
 
 // Could also go with coloured glyphs.
@@ -553,7 +553,7 @@ public:
         {
             cgotoxy(use_first_col() ? 2 : 1, last_row, GOTO_MSG);
             textcolor(channel_to_colour(MSGCH_PROMPT));
-            if (Tutorial.tutorial_left)
+            if (crawl_state.game_is_hints())
             {
                 std::string more_str = "--more-- Press Space ";
 #ifdef USE_TILE
@@ -578,6 +578,11 @@ void display_message_window()
     msgwin.show();
 }
 
+void clear_message_window()
+{
+    msgwin = message_window();
+}
+
 void scroll_message_window(int n)
 {
     msgwin.scroll(n);
@@ -668,6 +673,14 @@ public:
     {
         return msgs;
     }
+
+    void clear()
+    {
+        msgs.clear();
+        prev_msg = message_item();
+        last_of_turn = false;
+        temp = 0;
+    }
 };
 
 // Circular buffer for keeping past messages.
@@ -695,15 +708,16 @@ msg_colour_type msg_colour(int col)
     return static_cast<msg_colour_type>(col);
 }
 
-int colour_msg(msg_colour_type col)
+static int colour_msg(msg_colour_type col)
 {
-    return static_cast<int>(col);
+    if (col == MSGCOL_MUTED)
+        return (DARKGREY);
+    else
+        return static_cast<int>(col);
 }
 
-#ifdef USE_COLOUR_MESSAGES
-
 // Returns a colour or MSGCOL_MUTED.
-msg_colour_type channel_to_colour(msg_channel_type channel, int param)
+static msg_colour_type channel_to_msgcol(msg_channel_type channel, int param)
 {
     if (you.asleep())
         return (MSGCOL_DARKGREY);
@@ -771,10 +785,6 @@ msg_colour_type channel_to_colour(msg_channel_type channel, int param)
             ret = MSGCOL_LIGHTRED;
             break;
 
-        case MSGCH_TUTORIAL:
-            ret = MSGCOL_MAGENTA;
-            break;
-
         case MSGCH_MONSTER_SPELL:
         case MSGCH_MONSTER_ENCHANT:
         case MSGCH_FRIEND_SPELL:
@@ -782,6 +792,8 @@ msg_colour_type channel_to_colour(msg_channel_type channel, int param)
             ret = MSGCOL_LIGHTMAGENTA;
             break;
 
+        case MSGCH_TUTORIAL:
+        case MSGCH_ORB:
         case MSGCH_BANISHMENT:
             ret = MSGCOL_MAGENTA;
             break;
@@ -841,30 +853,28 @@ msg_colour_type channel_to_colour(msg_channel_type channel, int param)
     return (ret);
 }
 
-#else // don't use colour messages
-
-msg_colour_type channel_to_colour(msg_channel_type channel, int param)
+int channel_to_colour(msg_channel_type channel, int param)
 {
-    return (MSGCOL_LIGHTGREY);
+    return colour_msg(channel_to_msgcol(channel, param));
 }
 
-#endif
-
 static void do_message_print(msg_channel_type channel, int param,
                              const char *format, va_list argp)
 {
     va_list ap;
     va_copy(ap, argp);
     char buff[200];
-    size_t len = vsnprintf( buff, sizeof( buff ), format, argp );
-    if (len < sizeof( buff )) {
-        mpr( buff, channel, param );
+    size_t len = vsnprintf(buff, sizeof(buff), format, argp);
+    if (len < sizeof(buff))
+    {
+        mpr(buff, channel, param);
     }
-    else {
-        char *heapbuf = (char*)malloc( len + 1 );
-        vsnprintf( heapbuf, len + 1, format, ap );
-        mpr( heapbuf, channel, param );
-        free( heapbuf );
+    else
+    {
+        char *heapbuf = (char*)malloc(len + 1);
+        vsnprintf(heapbuf, len + 1, format, ap);
+        mpr(heapbuf, channel, param);
+        free(heapbuf);
     }
     va_end(ap);
 }
@@ -895,12 +905,12 @@ void mprf(const char *format, ...)
 }
 
 #ifdef DEBUG_DIAGNOSTICS
-void dprf( const char *format, ... )
+void dprf(const char *format, ...)
 {
     va_list  argp;
-    va_start( argp, format );
-    do_message_print( MSGCH_DIAGNOSTICS, 0, format, argp );
-    va_end( argp );
+    va_start(argp, format);
+    do_message_print(MSGCH_DIAGNOSTICS, 0, format, argp);
+    va_end(argp);
 }
 #endif
 
@@ -944,8 +954,9 @@ static void debug_channel_arena(msg_channel_type channel)
     case MSGCH_MULTITURN_ACTION:
     case MSGCH_EXAMINE:
     case MSGCH_EXAMINE_FILTER:
+    case MSGCH_ORB:
     case MSGCH_TUTORIAL:
-        DEBUGSTR("Invalid channel '%s' in arena mode",
+        die("Invalid channel '%s' in arena mode",
                  channel_to_str(channel).c_str());
         break;
     default:
@@ -980,7 +991,7 @@ void mpr(std::string text, msg_channel_type channel, int param, bool nojoin)
     if (crawl_state.game_crashed)
         return;
 
-    if (crawl_state.arena)
+    if (crawl_state.game_is_arena())
         debug_channel_arena(channel);
 
     if (!crawl_state.io_inited)
@@ -1010,6 +1021,14 @@ void mpr(std::string text, msg_channel_type channel, int param, bool nojoin)
     bool domore = check_more(text, channel);
     bool join = !domore && !nojoin && check_join(text, channel);
 
+    if (you.duration[DUR_QUAD_DAMAGE])
+    {
+        // No sound, so we simulate the reverb with all caps.
+        formatted_string fs = formatted_string::parse_string(text);
+        fs.all_caps();
+        text = fs.to_colour_string();
+    }
+
     std::string col = colour_to_str(colour_msg(colour));
     text = "<" + col + ">" + text + "</" + col + ">"; // XXX
     message_item msg = message_item(text, channel, param, join);
@@ -1032,9 +1051,7 @@ static std::string show_prompt(std::string prompt)
 
     // FIXME: duplicating mpr code.
     msg_colour_type colour = prepare_message(prompt, MSGCH_PROMPT, 0);
-    std::string col = colour_to_str(colour_msg(colour));
-    std::string text = "<" + col + ">" + prompt + "</" + col + ">"; // XXX
-    return text;
+    return colour_string(prompt, colour_msg(colour));
 }
 
 static std::string _prompt;
@@ -1048,6 +1065,7 @@ void msgwin_reply(std::string reply)
 {
     msgwin_clear_temporary();
     msgwin_set_temporary(false);
+    reply = replace_all(reply, "<", "<<");
     mpr(_prompt + "<lightgrey>" + reply + "</lightgrey>", MSGCH_PROMPT);
     msgwin.got_input();
 }
@@ -1060,9 +1078,11 @@ void msgwin_got_input()
 int msgwin_get_line(std::string prompt, char *buf, int len,
                     input_history *mh, int (*keyproc)(int& c))
 {
-    msgwin_prompt(prompt);
+    if (prompt != "")
+        msgwin_prompt(prompt);
+
     int ret = cancelable_get_line(buf, len, mh, keyproc);
-    msgwin_reply(ret == 0 ? buf : "");
+    msgwin_reply(buf);
     return ret;
 }
 
@@ -1137,33 +1157,18 @@ static void mpr_check_patterns(const std::string& message,
         }
     }
 
-    if (channel != MSGCH_DIAGNOSTICS && channel != MSGCH_EQUIPMENT
-        && channel != MSGCH_TALK && channel != MSGCH_TALK_VISUAL
-        && channel != MSGCH_FRIEND_SPELL && channel != MSGCH_FRIEND_ENCHANT
-        && channel != MSGCH_FRIEND_ACTION && channel != MSGCH_SOUND)
-    {
-        interrupt_activity(AI_MESSAGE,
-                           channel_to_str(channel) + ":" + message);
-    }
+    if (channel != MSGCH_DIAGNOSTICS && channel != MSGCH_EQUIPMENT)
+        interrupt_activity(AI_MESSAGE, channel_to_str(channel) + ":" + message);
 
     // Any sound has a chance of waking the PC if the PC is asleep.
     if (channel == MSGCH_SOUND)
         you.check_awaken(5);
 
-    // Check messages for all forms of running now.
-    if (you.running)
-        for (unsigned i = 0; i < Options.travel_stop_message.size(); ++i)
-            if (Options.travel_stop_message[i].is_filtered( channel, message ))
-            {
-                stop_running();
-                break;
-            }
-
     if (!Options.sound_mappings.empty())
         for (unsigned i = 0; i < Options.sound_mappings.size(); i++)
         {
             // Maybe we should allow message channel matching as for
-            // travel_stop_message?
+            // force_more_message?
             if (Options.sound_mappings[i].pattern.matches(message))
             {
                 play_sound(Options.sound_mappings[i].soundfile.c_str());
@@ -1200,7 +1205,7 @@ static msg_colour_type prepare_message(const std::string& imsg,
         return MSGCOL_MUTED;
     }
 
-    msg_colour_type colour = channel_to_colour(channel, param);
+    msg_colour_type colour = channel_to_msgcol(channel, param);
 
     if (colour != MSGCOL_MUTED)
         mpr_check_patterns(imsg, channel, param);
@@ -1228,6 +1233,8 @@ void flush_prev_message()
 
 void mesclr(bool force)
 {
+    if (!crawl_state.io_inited)
+        return;
     // Unflushed message will be lost with clear_messages,
     // so they shouldn't really exist, but some of the delay
     // code appears to do this intentionally.
@@ -1257,12 +1264,12 @@ static void readkey_more(bool user_forced)
     int keypress;
     mouse_control mc(MOUSE_MODE_MORE);
     do
-        keypress = c_getch();
+        keypress = getch_ck();
     while (keypress != ' ' && keypress != '\r' && keypress != '\n'
-           && keypress != ESCAPE && keypress != -1
+           && !key_is_escape(keypress)
            && (user_forced || keypress != CK_MOUSE_CLICK));
 
-    if (keypress == ESCAPE)
+    if (key_is_escape(keypress))
         set_more_autoclear(true);
 }
 
@@ -1281,7 +1288,7 @@ static bool _pre_more()
         return true;
 #endif
 
-    if (crawl_state.arena)
+    if (crawl_state.game_is_arena())
     {
         delay(Options.arena_delay);
         return true;
@@ -1291,7 +1298,7 @@ static bool _pre_more()
         return true;
 
 #ifdef WIZARD
-    if(luaterp_running())
+    if (luaterp_running())
         return true;
 #endif
 
@@ -1303,6 +1310,8 @@ static bool _pre_more()
 
 void more(bool user_forced)
 {
+    if (!crawl_state.io_inited)
+        return;
     flush_prev_message();
     msgwin.more(false, user_forced);
     mesclr();
@@ -1315,27 +1324,35 @@ static bool is_channel_dumpworthy(msg_channel_type channel)
             && channel != MSGCH_TUTORIAL);
 }
 
+void clear_message_store()
+{
+    messages.clear();
+}
+
 std::string get_last_messages(int mcount)
 {
     flush_prev_message();
 
     std::string text;
     // XXX: should use some message_history iterator here
+    const store_t& msgs = messages.get_store();
+    // XXX: loop wraps around otherwise. This could be done better.
+    mcount = std::min(mcount, NUM_STORED_MESSAGES);
     for (int i = -1; mcount > 0; --i)
     {
-        const message_item msg = messages.get_store()[i];
+        const message_item msg = msgs[i];
         if (!msg)
             break;
         if (is_channel_dumpworthy(msg.channel))
         {
-            text = msg.pure_text() + EOL + text;
+            text = msg.pure_text() + "\n" + text;
             mcount--;
         }
     }
 
     // An extra line of clearance.
     if (!text.empty())
-        text += EOL;
+        text += "\n";
     return text;
 }
 
@@ -1344,34 +1361,31 @@ std::string get_last_messages(int mcount)
 void save_messages(writer& outf)
 {
     store_t msgs = messages.get_store();
-    marshallLong(outf, msgs.size());
+    marshallInt(outf, msgs.size());
     for (int i = 0; i < msgs.size(); ++i)
     {
         marshallString4(outf, msgs[i].text);
-        marshallLong(outf, msgs[i].channel);
-        marshallLong(outf, msgs[i].param);
-        marshallLong(outf, msgs[i].repeats);
-        marshallLong(outf, msgs[i].turn);
+        marshallInt(outf, msgs[i].channel);
+        marshallInt(outf, msgs[i].param);
+        marshallInt(outf, msgs[i].repeats);
+        marshallInt(outf, msgs[i].turn);
     }
 }
 
 void load_messages(reader& inf)
 {
-    if (inf.getMinorVersion() < TAG_MINOR_MSGWIN)
-        return;
-
     unwind_var<bool> save_more(crawl_state.show_more_prompt, false);
 
-    int num = unmarshallLong(inf);
+    int num = unmarshallInt(inf);
     for (int i = 0; i < num; ++i)
     {
         std::string text;
         unmarshallString4(inf, text);
 
-        msg_channel_type channel = (msg_channel_type) unmarshallLong(inf);
-        int           param      = unmarshallLong(inf);
-        int           repeats    = unmarshallLong(inf);
-        int           turn       = unmarshallLong(inf);
+        msg_channel_type channel = (msg_channel_type) unmarshallInt(inf);
+        int           param      = unmarshallInt(inf);
+        int           repeats    = unmarshallInt(inf);
+        int           turn       = unmarshallInt(inf);
 
         message_item msg(message_item(text, channel, param, repeats, turn));
         if (msg)
@@ -1384,7 +1398,11 @@ void load_messages(reader& inf)
 
 void replay_messages(void)
 {
-    formatted_scroller hist(MF_START_AT_END, "");
+    formatted_scroller hist(MF_START_AT_END | MF_ALWAYS_SHOW_MORE, "");
+    hist.set_more(formatted_string::parse_string(
+                        "<cyan>[up/<< : Page up.    down/Space/> : Page down."
+                        "                           Esc exits.]</cyan>"));
+
     const store_t msgs = messages.get_store();
     for (int i = 0; i < msgs.size(); ++i)
         if (channel_message_history(msgs[i].channel))
@@ -1407,6 +1425,7 @@ void replay_messages(void)
                 hist.add_item_formatted_string(line);
             }
         }
+
     hist.show();
 }
 
