@@ -7,6 +7,7 @@
 #include "env.h"
 #include "fprop.h"
 #include "mon-behv.h"
+#include "mon-iter.h"
 #include "mon-pathfind.h"
 #include "mon-place.h"
 #include "mon-stuff.h"
@@ -14,6 +15,7 @@
 #include "monster.h"
 #include "player.h"
 #include "random.h"
+#include "state.h"
 #include "stuff.h"
 #include "terrain.h"
 #include "traps.h"
@@ -24,7 +26,7 @@
 // find a path, so we won't need to calculate again.
 // Should there be a direct path to the target for a monster thus marked, it
 // will still be able to come nearer (and the mark will then be cleared).
-static void _mark_neighbours_target_unreachable(monsters *mon)
+static void _mark_neighbours_target_unreachable(monster* mon)
 {
     // Highly intelligent monsters are perfectly capable of pathfinding
     // and don't need their neighbour's advice.
@@ -33,7 +35,7 @@ static void _mark_neighbours_target_unreachable(monsters *mon)
         return;
 
     const bool flies         = mons_flies(mon);
-    const bool amphibious    = mons_amphibious(mon);
+    const bool amphibious    = (mons_habitat(mon) == HT_AMPHIBIOUS);
     const habitat_type habit = mons_primary_habitat(mon);
 
     for (radius_iterator ri(mon->pos(), 2, true, false); ri; ++ri)
@@ -46,7 +48,7 @@ static void _mark_neighbours_target_unreachable(monsters *mon)
         if (!mon->see_cell(*ri))
             continue;
 
-        monsters* const m = monster_at(*ri);
+        monster* const m = monster_at(*ri);
         if (m == NULL)
             continue;
 
@@ -59,17 +61,24 @@ static void _mark_neighbours_target_unreachable(monsters *mon)
         if (mons_primary_habitat(m) != habit)
             continue;
 
+        // Wall clinging monsters use different pathfinding.
+        if (mon->can_cling_to_walls() != m->can_cling_to_walls())
+            continue;
+
         // A flying monster has an advantage over a non-flying one.
         // Same for a swimming one.
-        if (!flies && mons_flies(m) || !amphibious && mons_amphibious(m))
+        if (!flies && mons_flies(m)
+            || !amphibious && mons_habitat(m) == HT_AMPHIBIOUS)
+        {
             continue;
+        }
 
         if (m->travel_target == MTRAV_NONE)
             m->travel_target = MTRAV_UNREACHABLE;
     }
 }
 
-static void _set_no_path_found(monsters *mon)
+static void _set_no_path_found(monster* mon)
 {
 #ifdef DEBUG_PATHFIND
     mpr("No path found!");
@@ -80,7 +89,7 @@ static void _set_no_path_found(monsters *mon)
     _mark_neighbours_target_unreachable(mon);
 }
 
-static bool _target_is_unreachable(monsters *mon)
+static bool _target_is_unreachable(monster* mon)
 {
     return (mon->travel_target == MTRAV_UNREACHABLE
             || mon->travel_target == MTRAV_KNOWN_UNREACHABLE);
@@ -92,7 +101,7 @@ static bool _target_is_unreachable(monsters *mon)
 // Check whether there's an unobstructed path to the player (in sight!),
 // either by using an existing travel_path or calculating a new one.
 // Returns true if no further handling necessary, else false.
-bool try_pathfind(monsters *mon, const dungeon_feature_type can_move)
+bool try_pathfind(monster* mon, const dungeon_feature_type can_move)
 {
     // Just because we can *see* the player, that doesn't mean
     // we can actually get there.
@@ -101,7 +110,8 @@ bool try_pathfind(monsters *mon, const dungeon_feature_type can_move)
     // next turn, and even extend that flag to neighbouring
     // monsters of similar movement restrictions.
 
-    bool need_pathfind = !can_go_straight(mon->pos(), you.pos(), can_move);
+    bool need_pathfind = !can_go_straight(mon, mon->pos(), PLAYER_POS,
+                                          can_move);
 
     // Smart monsters that can fire through obstacles won't use
     // pathfinding.
@@ -115,11 +125,11 @@ bool try_pathfind(monsters *mon, const dungeon_feature_type can_move)
     // Also don't use pathfinding if the monster can shoot
     // across the blocking terrain, and is smart enough to
     // realise that.
-    if (need_pathfind
+    if ((!crawl_state.game_is_zotdef()) && need_pathfind
         && mons_intel(mon) >= I_NORMAL && !mon->friendly()
         && (mons_has_ranged_spell(mon, true)
             || mons_has_ranged_attack(mon))
-        && exists_ray(mon->pos(), you.pos(), opc_solid))
+        && exists_ray(mon->pos(), PLAYER_POS, opc_solid))
     {
         need_pathfind = false;
     }
@@ -140,8 +150,13 @@ bool try_pathfind(monsters *mon, const dungeon_feature_type can_move)
 
     // If the target is "unreachable" (the monster already tried,
     // and failed, to find a path), there's a chance of trying again.
-    if (_target_is_unreachable(mon) && !one_chance_in(12))
+    // The chance is higher for wall clinging monsters to help them avoid
+    // shallow water.
+    if (_target_is_unreachable(mon) && !one_chance_in(12)
+        && !(mon->can_cling_to_walls() && one_chance_in(4)))
+    {
         return (false);
+    }
 
 #ifdef DEBUG_PATHFIND
     mprf("%s: Player out of reach! What now?",
@@ -154,13 +169,13 @@ bool try_pathfind(monsters *mon, const dungeon_feature_type can_move)
         const coord_def targ = mon->travel_path[len - 1];
 
         // Current target still valid?
-        if (can_go_straight(targ, you.pos(), can_move))
+        if (can_go_straight(mon, targ, PLAYER_POS, can_move))
         {
             // Did we reach the target?
             if (mon->pos() == mon->travel_path[0])
             {
                 // Get next waypoint.
-                mon->travel_path.erase( mon->travel_path.begin() );
+                mon->travel_path.erase(mon->travel_path.begin());
 
                 if (!mon->travel_path.empty())
                 {
@@ -168,7 +183,7 @@ bool try_pathfind(monsters *mon, const dungeon_feature_type can_move)
                     return (true);
                 }
             }
-            else if (can_go_straight(mon->pos(), mon->travel_path[0],
+            else if (can_go_straight(mon, mon->pos(), mon->travel_path[0],
                                      can_move))
             {
                 mon->target = mon->travel_path[0];
@@ -178,13 +193,15 @@ bool try_pathfind(monsters *mon, const dungeon_feature_type can_move)
     }
 
     // Use pathfinding to find a (new) path to the player.
-    const int dist = grid_distance(mon->pos(), you.pos());
+    const int dist = grid_distance(mon->pos(), PLAYER_POS);
 
 #ifdef DEBUG_PATHFIND
     mprf("Need to calculate a path... (dist = %d)", dist);
 #endif
-    const int range = mons_tracking_range(mon);
-    if (range > 0 && dist > range)
+    // All monsters can find the Orb in Zotdef
+    const int range = (crawl_state.game_is_zotdef() || mon->friendly() ? 1000 : mons_tracking_range(mon));
+
+    if (range > 0 && dist > dist_range(range))
     {
         mon->travel_target = MTRAV_UNREACHABLE;
 #ifdef DEBUG_PATHFIND
@@ -196,13 +213,13 @@ bool try_pathfind(monsters *mon, const dungeon_feature_type can_move)
 
 #ifdef DEBUG_PATHFIND
     mprf("Need a path for %s from (%d, %d) to (%d, %d), max. dist = %d",
-         mon->name(DESC_PLAIN).c_str(), mon->pos(), you.pos(), range);
+         mon->name(DESC_PLAIN).c_str(), mon->pos(), PLAYER_POS, range);
 #endif
     monster_pathfind mp;
     if (range > 0)
         mp.set_range(range);
 
-    if (mp.init_pathfind(mon, you.pos()))
+    if (mp.init_pathfind(mon, PLAYER_POS))
     {
         mon->travel_path = mp.calc_waypoints();
         if (!mon->travel_path.empty())
@@ -237,7 +254,7 @@ static bool _is_level_exit(const coord_def& pos)
 }
 
 // Returns true if a monster left the level.
-bool pacified_leave_level(monsters *mon, std::vector<level_exit> e,
+bool pacified_leave_level(monster* mon, std::vector<level_exit> e,
                           int e_index)
 {
     // If a pacified monster is leaving the level, and has reached an
@@ -246,7 +263,7 @@ bool pacified_leave_level(monsters *mon, std::vector<level_exit> e,
     // player, make it leave the level.
     if (_is_level_exit(mon->pos())
         || (e_index != -1 && mon->pos() == e[e_index].target)
-        || grid_distance(mon->pos(), you.pos()) >= LOS_RADIUS * 4)
+        || distance(mon->pos(), you.pos()) >= dist_range(LOS_RADIUS * 4))
     {
         make_mons_leave_level(mon);
         return (true);
@@ -271,7 +288,7 @@ static int _count_water_neighbours(coord_def p)
 
 // Pick the nearest water grid that is surrounded by the most
 // water squares within LoS.
-bool find_siren_water_target(monsters *mon)
+bool find_siren_water_target(monster* mon)
 {
     ASSERT(mon->type == MONS_SIREN);
 
@@ -357,8 +374,9 @@ bool find_siren_water_target(monsters *mon)
     monster_pathfind mp;
 #ifdef WIZARD
     // Remove old highlighted areas to make place for the new ones.
-    for (rectangle_iterator ri(1); ri; ++ri)
-        env.pgrid(*ri) &= ~(FPROP_HIGHLIGHT);
+    if (you.wizard)
+        for (rectangle_iterator ri(1); ri; ++ri)
+            env.pgrid(*ri) &= ~(FPROP_HIGHLIGHT);
 #endif
 
     if (mp.init_pathfind(mon, best_target))
@@ -368,8 +386,9 @@ bool find_siren_water_target(monsters *mon)
         if (!mon->travel_path.empty())
         {
 #ifdef WIZARD
-            for (unsigned int i = 0; i < mon->travel_path.size(); i++)
-                env.pgrid(mon->travel_path[i]) |= FPROP_HIGHLIGHT;
+            if (you.wizard)
+                for (unsigned int i = 0; i < mon->travel_path.size(); i++)
+                    env.pgrid(mon->travel_path[i]) |= FPROP_HIGHLIGHT;
 #endif
 #ifdef DEBUG_PATHFIND
             mprf("Found a path to (%d, %d) with %d surrounding water squares",
@@ -385,7 +404,7 @@ bool find_siren_water_target(monsters *mon)
     return (false);
 }
 
-bool find_wall_target(monsters *mon)
+bool find_wall_target(monster* mon)
 {
     ASSERT(mons_wall_shielded(mon));
 
@@ -476,7 +495,7 @@ bool find_wall_target(monsters *mon)
 }
 
 // Returns true if further handling neeeded.
-static bool _handle_monster_travelling(monsters *mon,
+static bool _handle_monster_travelling(monster* mon,
                                        const dungeon_feature_type can_move)
 {
 #ifdef DEBUG_PATHFIND
@@ -490,7 +509,7 @@ static bool _handle_monster_travelling(monsters *mon,
 #ifdef DEBUG_PATHFIND
         mpr("Arrived at first waypoint.");
 #endif
-        mon->travel_path.erase( mon->travel_path.begin() );
+        mon->travel_path.erase(mon->travel_path.begin());
         if (mon->travel_path.empty())
         {
 #ifdef DEBUG_PATHFIND
@@ -510,7 +529,7 @@ static bool _handle_monster_travelling(monsters *mon,
     }
 
     // Can we still see our next waypoint?
-    if (!can_go_straight(mon->pos(), mon->travel_path[0], can_move))
+    if (!can_go_straight(mon, mon->pos(), mon->travel_path[0], can_move))
     {
 #ifdef DEBUG_PATHFIND
         mpr("Can't see waypoint grid.");
@@ -528,7 +547,7 @@ static bool _handle_monster_travelling(monsters *mon,
         const int size = mon->travel_path.size();
         for (int i = size - 1; i >= 0; --i)
         {
-            if (can_go_straight(mon->pos(), mon->travel_path[i], can_move))
+            if (can_go_straight(mon, mon->pos(), mon->travel_path[i], can_move))
             {
                 mon->target = mon->travel_path[i];
                 erase = i;
@@ -545,7 +564,7 @@ static bool _handle_monster_travelling(monsters *mon,
             // Erase all waypoints that came earlier:
             // we don't need them anymore.
             while (0 < erase--)
-                mon->travel_path.erase( mon->travel_path.begin() );
+                mon->travel_path.erase(mon->travel_path.begin());
         }
         else
         {
@@ -587,7 +606,7 @@ static bool _handle_monster_travelling(monsters *mon,
     return (false);
 }
 
-static bool _choose_random_patrol_target_grid(monsters *mon)
+static bool _choose_random_patrol_target_grid(monster* mon)
 {
     const int intel = mons_intel(mon);
 
@@ -600,12 +619,11 @@ static bool _choose_random_patrol_target_grid(monsters *mon)
         return (true);
 
     // If there's no chance we'll find the patrol point, quit right away.
-    if (grid_distance(mon->pos(), mon->patrol_point) > 2 * LOS_RADIUS)
+    if (distance(mon->pos(), mon->patrol_point) > dist_range(2 * LOS_RADIUS))
         return (false);
 
     // Can the monster see the patrol point from its current position?
-    const bool patrol_seen = mon->mon_see_cell(mon->patrol_point,
-                                 habitat2grid(mons_primary_habitat(mon)));
+    const bool patrol_seen = mon->see_cell(mon->patrol_point);
 
     if (intel == I_PLANT && !patrol_seen)
     {
@@ -639,7 +657,7 @@ static bool _choose_random_patrol_target_grid(monsters *mon)
         if (*ri == mon->pos())
             continue;
 
-        if (!mon->can_pass_through_feat(grd(*ri)))
+        if (!in_bounds(*ri) || !mon->can_pass_through_feat(grd(*ri)))
             continue;
 
         // Don't bother moving to squares (currently) occupied by a
@@ -703,7 +721,7 @@ static bool _choose_random_patrol_target_grid(monsters *mon)
 
     return (count_grids);
 }// Returns true if further handling neeeded.
-static bool _handle_monster_patrolling(monsters *mon)
+static bool _handle_monster_patrolling(monster* mon)
 {
     if (!_choose_random_patrol_target_grid(mon))
     {
@@ -783,7 +801,7 @@ static bool _handle_monster_patrolling(monsters *mon)
     return (false);
 }
 
-void set_random_target(monsters* mon)
+void set_random_target(monster* mon)
 {
     mon->target = random_in_bounds(); // If we don't find anything better.
     for (int tries = 0; tries < 150; ++tries)
@@ -801,19 +819,195 @@ void set_random_target(monsters* mon)
     }
 }
 
-void check_wander_target(monsters *mon, bool isPacified,
+// Try to find a band leader for the given monster
+static monster * _active_band_leader(monster * mon)
+{
+    // Not a band member
+    if (!mon->props.exists("band_leader"))
+        return (NULL);
+
+    // Try to find our fearless leader.
+    unsigned leader_mid = mon->props["band_leader"].get_int();
+
+    return (monster_by_mid(leader_mid));
+}
+
+// Return true if a target still needs to be set. If returns false, mon->target
+// was set.
+static bool _band_wander_target(monster * mon, dungeon_feature_type can_move)
+{
+    int dist_thresh = LOS_RADIUS + HERD_COMFORT_RANGE;
+    monster * band_leader = _active_band_leader(mon);
+    if (band_leader == NULL)
+        return (true);
+
+    int leader_dist = grid_distance(mon->pos(), band_leader->pos());
+    if (leader_dist > dist_thresh)
+    {
+        monster_pathfind mp;
+        mp.set_range(1000);
+
+        if (mp.init_pathfind(mon, band_leader->pos()))
+        {
+            mon->travel_path = mp.calc_waypoints();
+            if (!mon->travel_path.empty())
+            {
+                // Okay then, we found a path.  Let's use it!
+                mon->target = mon->travel_path[0];
+                mon->travel_target = MTRAV_PATROL;
+                return (false);
+            }
+            else
+                return (true);
+        }
+
+        return (true);
+    }
+
+    std::vector<coord_def> positions;
+
+    for (radius_iterator r_it(mon->get_los_no_trans(), mon); r_it; ++r_it)
+    {
+        if (!in_bounds(*r_it))
+            continue;
+
+        int dist = grid_distance(*r_it, band_leader->pos());
+        if (dist < HERD_COMFORT_RANGE)
+        {
+            positions.push_back(*r_it);
+        }
+    }
+
+    if (positions.empty())
+        return (true);
+
+    mon->target = positions[random2(positions.size())];
+
+    ASSERT(in_bounds(mon->target));
+    return (false);
+}
+
+// Returns true if a movement target still needs to be set
+static bool _herd_wander_target(monster * mon, dungeon_feature_type can_move)
+{
+    std::vector<monster_iterator> friends;
+    std::map<int, std::vector<coord_def> > distance_positions;
+
+    int dist_thresh = LOS_RADIUS + HERD_COMFORT_RANGE;
+
+    for (monster_iterator mit; mit; ++mit)
+    {
+        if (mit->mindex() == mon->mindex()
+            || mons_genus(mit->type) != mons_genus(mon->type)
+            || grid_distance(mit->pos(), mon->pos()) > dist_thresh)
+        {
+            continue;
+        }
+
+        friends.push_back(mit);
+    }
+
+    if (friends.empty())
+        return (true);
+
+    for (radius_iterator r_it(mon->get_los_no_trans(), true) ; r_it; ++r_it)
+    {
+        if (!in_bounds(*r_it))
+            continue;
+
+        int count = 0;
+        for (unsigned i = 0; i < friends.size(); i++)
+        {
+            if (grid_distance(friends[i]->pos(), *r_it) < HERD_COMFORT_RANGE
+                && friends[i]->see_cell_no_trans(*r_it))
+            {
+                count++;
+            }
+        }
+        if (count > 0)
+            distance_positions[count].push_back(*r_it);
+    }
+    std::map<int, std::vector<coord_def> >::reverse_iterator back =
+        distance_positions.rbegin();
+
+    if (back == distance_positions.rend())
+        return (true);
+
+    mon->target = back->second[random2(back->second.size())];
+    return (false);
+}
+
+static bool _herd_ok(monster * mon)
+{
+    bool in_bounds = false;
+    bool intermediate_range = false;
+    int intermediate_thresh = LOS_RADIUS + HERD_COMFORT_RANGE;
+
+    for (monster_iterator mit(mon); mit; ++mit)
+    {
+        if (mit->mindex() == mon->mindex())
+            continue;
+
+        if (mons_genus(mit->type) == mons_genus(mon->type) )
+        {
+            int g_dist = grid_distance(mit->pos(), mon->pos());
+            if (g_dist < HERD_COMFORT_RANGE
+                && mon->see_cell_no_trans(mit->pos()))
+            {
+                in_bounds = true;
+                break;
+            }
+            else if (g_dist < intermediate_thresh)
+            {
+                intermediate_range = true;
+            }
+        }
+    }
+
+    return (in_bounds || !intermediate_range);
+}
+
+// Return true if we don't have to do anything to keep within an ok distance
+// of our band leader. (If no leader exists we don't have to do anything).
+static bool _band_ok(monster * mon)
+{
+    // Don't have to worry about being close to the leader if no leader can be
+    // found.
+    monster * leader = _active_band_leader(mon);
+
+    if (!leader)
+        return (true);
+
+    int g_dist = grid_distance(leader->pos(), mon->pos());
+
+    // If in range, or sufficiently out of range we can just wander around for
+    // a while longer.
+    if (g_dist < HERD_COMFORT_RANGE && mon->see_cell_no_trans(leader->pos())
+        || g_dist >= (LOS_RADIUS + HERD_COMFORT_RANGE))
+    {
+        return (true);
+    }
+
+    return (false);
+}
+
+
+void check_wander_target(monster* mon, bool isPacified,
                          dungeon_feature_type can_move)
 {
     // default wander behaviour
     if (mon->pos() == mon->target
-        || mons_is_batty(mon) || !isPacified && one_chance_in(20))
+        || mons_is_batty(mon) || !isPacified && one_chance_in(20)
+        || herd_monster(mon) && !_herd_ok(mon)
+        || !_band_ok(mon))
     {
         bool need_target = true;
 
         if (!can_move)
         {
-            can_move = (mons_amphibious(mon) ? DNGN_DEEP_WATER
-                                             : DNGN_SHALLOW_WATER);
+            can_move =
+                (mons_habitat(mon) == HT_AMPHIBIOUS) ? DNGN_DEEP_WATER
+                                                     : DNGN_SHALLOW_WATER;
         }
 
         if (mon->is_travelling())
@@ -824,6 +1018,15 @@ void check_wander_target(monsters *mon, bool isPacified,
         if (need_target && mon->is_patrolling())
             need_target = _handle_monster_patrolling(mon);
 
+        if (need_target && herd_monster(mon))
+            need_target = _herd_wander_target(mon, can_move);
+
+        if (need_target
+            && _active_band_leader(mon) != NULL)
+        {
+            need_target = _band_wander_target(mon, can_move);
+        }
+
         // XXX: This is really dumb wander behaviour... instead of
         // changing the goal square every turn, better would be to
         // have the monster store a direction and have the monster
@@ -850,7 +1053,7 @@ static void _find_all_level_exits(std::vector<level_exit> &e)
     }
 }
 
-int mons_find_nearest_level_exit(const monsters *mon,
+int mons_find_nearest_level_exit(const monster* mon,
                                  std::vector<level_exit> &e,
                                  bool reset)
 {
@@ -865,7 +1068,7 @@ int mons_find_nearest_level_exit(const monsters *mon,
         if (e[i].unreachable)
             continue;
 
-        int dist = grid_distance(mon->pos(), e[i].target);
+        int dist = distance(mon->pos(), e[i].target);
 
         if (old_dist == -1 || old_dist >= dist)
         {
@@ -885,15 +1088,17 @@ int mons_find_nearest_level_exit(const monsters *mon,
     return (retval);
 }
 
-void set_random_slime_target(monsters* mon)
+void set_random_slime_target(monster* mon)
 {
     // Strictly neutral slimes will go for the nearest item.
     const coord_def pos = mon->pos();
     int mindist = LOS_MAX_RADIUS_SQ + 1;
-    for (radius_iterator ri(&mon->get_los()); ri; ++ri)
+    for (radius_iterator ri(mon->get_los()); ri; ++ri)
     {
         // XXX: an iterator that spirals out would be nice.
-        if (distance(pos, *ri) >= mindist)
+        if (!in_bounds(*ri) || distance(pos, *ri) >= mindist)
+            continue;
+        if (testbits(env.pgrid(*ri), FPROP_NO_JIYVA))
             continue;
         for (stack_iterator si(*ri); si; ++si)
         {
