@@ -1,8 +1,7 @@
-/*
- *  File:       mon-pick.cc
- *  Summary:    Functions used to help determine which monsters should appear.
- *  Written by: Linley Henzell
- */
+/**
+ * @file
+ * @brief Functions used to help determine which monsters should appear.
+**/
 
 #include "AppHdr.h"
 
@@ -10,7 +9,6 @@
 
 #include "externs.h"
 #include "branch.h"
-#include "misc.h"
 #include "mon-util.h"
 #include "place.h"
 
@@ -40,135 +38,6 @@ int mons_level(int mcls, const level_id &place)
     return monster_level;
 }
 
-typedef int (*mons_level_function)(int);
-
-struct global_level_info
-{
-    mons_level_function level_func;
-    branch_type         branch;
-    int                 avg_depth;
-};
-
-static int _mons_misc_level(int mcls)
-{
-    switch (mons_char(mcls))
-    {
-    case '&':
-        return 35;
-
-    case '1':
-        return 30;
-
-    case '2':
-        return 25;
-
-    case '3':
-        return 20;
-
-    case '4':
-        return 15;
-
-    case '5':
-        return 10;
-    }
-
-    if (mons_is_unique(mcls))
-        return (mons_type_hit_dice(mcls) * 14 / 10 + 1);
-
-    switch (mcls)
-    {
-    case MONS_HUMAN:
-    case MONS_ELF:
-    case MONS_DRACONIAN:
-        return 1;
-
-    case MONS_BIG_FISH:
-    case MONS_GIANT_GOLDFISH:
-    case MONS_JELLYFISH:
-        return 8;
-
-    case MONS_ANT_LARVA:
-        return 10;
-
-    case MONS_ELECTRIC_EEL:
-    case MONS_LAVA_FISH:
-    case MONS_LAVA_SNAKE:
-    case MONS_LAVA_WORM:
-    case MONS_SALAMANDER:
-    case MONS_HOG:
-        return 14;
-
-    case MONS_FIRE_VORTEX:
-        return 18;
-
-    case MONS_MINOTAUR:
-    case MONS_BALL_LIGHTNING:
-    case MONS_ORANGE_STATUE:
-    case MONS_SILVER_STATUE:
-    case MONS_ICE_STATUE:
-    case MONS_SPATIAL_VORTEX:
-    case MONS_MOLTEN_GARGOYLE:
-    case MONS_WATER_ELEMENTAL:
-        return 20;
-
-    case MONS_METAL_GARGOYLE:
-    case MONS_VAULT_GUARD:
-        return 24;
-
-    case MONS_QUEEN_ANT:
-        return 25;
-
-    case MONS_ANGEL:
-        return 27;
-
-    case MONS_DAEVA:
-        return 28;
-    }
-
-    return 0;
-}
-
-static global_level_info g_lev_infos[] = {
-    {mons_standard_level, BRANCH_MAIN_DUNGEON,  1},
-    {_mons_misc_level,    BRANCH_MAIN_DUNGEON,  1},
-    {mons_mineorc_level,  BRANCH_ORCISH_MINES,  8},
-    {mons_lair_level,     BRANCH_LAIR,         10},
-    {mons_hallelf_level,  BRANCH_ELVEN_HALLS,  11},
-    {mons_swamp_level,    BRANCH_SWAMP,        14},
-    {mons_shoals_level,   BRANCH_SHOALS,       14},
-    {mons_pitsnake_level, BRANCH_SNAKE_PIT,    15},
-    {mons_pitslime_level, BRANCH_SLIME_PITS,   16},
-    {mons_crypt_level,    BRANCH_CRYPT,        19},
-    {mons_tomb_level,     BRANCH_TOMB,         21},
-    {mons_hallzot_level,  BRANCH_HALL_OF_ZOT,  27},
-    {mons_dis_level,      BRANCH_DIS,          29},
-    {mons_gehenna_level,  BRANCH_GEHENNA,      29},
-    {mons_cocytus_level,  BRANCH_COCYTUS,      29},
-    {mons_tartarus_level, BRANCH_TARTARUS,     29},
-
-    {NULL,                NUM_BRANCHES,         0}
-};
-
-int mons_global_level(int mcls)
-{
-    for (int i = 0; g_lev_infos[i].level_func != NULL; i++)
-    {
-        int level     = (*g_lev_infos[i].level_func)(mcls);
-        int rel_level = level - absdungeon_depth(g_lev_infos[i].branch, 1);
-
-        if (g_lev_infos[i].branch == BRANCH_HALL_OF_ZOT)
-            rel_level++;
-
-        if (level >= 1 && level < 99 && rel_level != 0)
-        {
-            level = rel_level + g_lev_infos[i].avg_depth - 1;
-            return (level);
-        }
-    }
-
-    return (0);
-}
-
 // NOTE: Higher values returned means the monster is "more common".
 // A return value of zero means the monster will never appear. {dlb}
 int mons_rarity(int mcls, const level_id &place)
@@ -196,6 +65,7 @@ bool mons_abyss(int mcls)
     case MONS_BALRUG:
     case MONS_BLUE_DEATH:
     case MONS_BLUE_DEVIL:
+    case MONS_BONE_DRAGON:
     case MONS_BRAIN_WORM:
     case MONS_CACODEMON:
     case MONS_CHAOS_SPAWN:
@@ -206,9 +76,11 @@ bool mons_abyss(int mcls)
     case MONS_DEMONIC_CRAWLER:
     case MONS_EARTH_ELEMENTAL:
     case MONS_EFREET:
+    case MONS_EIDOLON:
     case MONS_EXECUTIONER:
     case MONS_EYE_OF_DEVASTATION:
     case MONS_EYE_OF_DRAINING:
+    case MONS_FIRE_BAT:
     case MONS_FIRE_ELEMENTAL:
     case MONS_FLAMING_CORPSE:
     case MONS_FLAYED_GHOST:
@@ -224,8 +96,10 @@ bool mons_abyss(int mcls)
     case MONS_GREEN_DEATH:
     case MONS_GUARDIAN_SERPENT:
     case MONS_HAIRY_DEVIL:
+    case MONS_HELLEPHANT:
     case MONS_HELLION:
     case MONS_HELLWING:
+    case MONS_HELL_HOG:
     case MONS_HELL_HOUND:
     case MONS_HELL_KNIGHT:
     case MONS_HUNGRY_GHOST:
@@ -235,11 +109,11 @@ bool mons_abyss(int mcls)
     case MONS_INSUBSTANTIAL_WISP:
     case MONS_IRON_DEVIL:
     case MONS_IRON_GOLEM:
+    case MONS_IRON_IMP:
     case MONS_JELLY:
     case MONS_LEMURE:
     case MONS_LICH:
     case MONS_LOROCYPROCA:
-    case MONS_MANES:
     case MONS_MIDGE:
     case MONS_MUMMY:
     case MONS_NAGA_MAGE:
@@ -248,6 +122,7 @@ bool mons_abyss(int mcls)
     case MONS_NECROPHAGE:
     case MONS_NEQOXEC:
     case MONS_ORANGE_DEMON:
+    case MONS_PHANTASMAL_WARRIOR:
     case MONS_PHANTOM:
     case MONS_PIT_FIEND:
     case MONS_RAKSHASA:
@@ -258,14 +133,14 @@ bool mons_abyss(int mcls)
     case MONS_SHADOW_DEMON:
     case MONS_SHADOW_IMP:
     case MONS_SHINING_EYE:
+    case MONS_SILENT_SPECTRE:
     case MONS_SIXFIRHY:
-    case MONS_SKELETAL_DRAGON:
     case MONS_SKELETAL_WARRIOR:
     case MONS_SKELETON_LARGE:
     case MONS_SKELETON_SMALL:
+    case MONS_SKY_BEAST:
     case MONS_SMOKE_DEMON:
     case MONS_SOUL_EATER:
-    case MONS_SPECTRAL_WARRIOR:
     case MONS_SPINY_WORM:
     case MONS_STONE_GOLEM:
     case MONS_SUN_DEMON:
@@ -296,6 +171,7 @@ bool mons_abyss(int mcls)
     case MONS_PURPLE_DRACONIAN:
     case MONS_PALE_DRACONIAN:
     case MONS_GREEN_DRACONIAN:
+    case MONS_GREY_DRACONIAN:
     case MONS_DRACONIAN_CALLER:
     case MONS_DRACONIAN_MONK:
     case MONS_DRACONIAN_SCORCHER:
@@ -303,6 +179,13 @@ bool mons_abyss(int mcls)
     case MONS_DRACONIAN_ANNIHILATOR:
     case MONS_DRACONIAN_ZEALOT:
     case MONS_DRACONIAN_SHIFTER:
+    case MONS_DEEP_DWARF:
+    case MONS_DEEP_DWARF_SCION:
+    case MONS_DEEP_DWARF_ARTIFICER:
+    case MONS_DEEP_DWARF_NECROMANCER:
+    case MONS_DEEP_DWARF_BERSERKER:
+    case MONS_DEEP_DWARF_DEATH_KNIGHT:
+    case MONS_UNBORN_DEEP_DWARF:
         return (true);
     default:
         return (false);
@@ -317,8 +200,8 @@ int mons_rare_abyss(int mcls)
     case MONS_ABOMINATION_SMALL:
         return 99;
 
+    case MONS_IRON_IMP:
     case MONS_LEMURE:
-    case MONS_MANES:
     case MONS_MIDGE:
     case MONS_UFETUBUS:
     case MONS_WHITE_IMP:
@@ -340,7 +223,7 @@ int mons_rare_abyss(int mcls)
     case MONS_ZOMBIE_SMALL:
         return 35;
 
-    case MONS_SKELETAL_DRAGON:
+    case MONS_BONE_DRAGON:
     case MONS_SIXFIRHY:
         return 20;
 
@@ -361,6 +244,7 @@ int mons_rare_abyss(int mcls)
         return 15;
 
     case MONS_EYE_OF_DRAINING:
+    case MONS_FIRE_BAT:
     case MONS_FLAMING_CORPSE:
     case MONS_LICH:
         return 14;
@@ -370,6 +254,7 @@ int mons_rare_abyss(int mcls)
     case MONS_GOLDEN_EYE:
         return 12;
 
+    case MONS_HELL_HOG:
     case MONS_HELL_HOUND:
     case MONS_HUNGRY_GHOST:
     case MONS_SHADOW:
@@ -405,8 +290,10 @@ int mons_rare_abyss(int mcls)
     case MONS_VAPOUR:
     case MONS_SIMULACRUM_LARGE:
     case MONS_SIMULACRUM_SMALL:
+    case MONS_SKY_BEAST:
         return 9;
 
+    case MONS_EIDOLON:
     case MONS_FUNGUS:
     case MONS_GIANT_EYEBALL:
     case MONS_PHANTOM:
@@ -427,16 +314,24 @@ int mons_rare_abyss(int mcls)
     case MONS_IRON_GOLEM:
     case MONS_NAGA_MAGE:
     case MONS_NAGA_WARRIOR:
+    case MONS_PHANTASMAL_WARRIOR:
     case MONS_PIT_FIEND:
     case MONS_RED_DEVIL:
     case MONS_SHINING_EYE:
-    case MONS_SPECTRAL_WARRIOR:
     case MONS_SPINY_WORM:
     case MONS_STONE_GOLEM:
     case MONS_TENTACLED_MONSTROSITY:
     case MONS_WIZARD:
     case MONS_WOOD_GOLEM:
     case MONS_DEATH_DRAKE:
+    case MONS_SILENT_SPECTRE:
+    case MONS_DEEP_DWARF:
+    case MONS_DEEP_DWARF_SCION:
+    case MONS_DEEP_DWARF_ARTIFICER:
+    case MONS_DEEP_DWARF_NECROMANCER:
+    case MONS_DEEP_DWARF_BERSERKER:
+    case MONS_DEEP_DWARF_DEATH_KNIGHT:
+    case MONS_UNBORN_DEEP_DWARF:
         return 5;
 
     case MONS_AIR_ELEMENTAL:
@@ -458,6 +353,7 @@ int mons_rare_abyss(int mcls)
     case MONS_PURPLE_DRACONIAN:
     case MONS_PALE_DRACONIAN:
     case MONS_GREEN_DRACONIAN:
+    case MONS_GREY_DRACONIAN:
     case MONS_DRACONIAN_CALLER:
     case MONS_DRACONIAN_MONK:
     case MONS_DRACONIAN_SCORCHER:
@@ -465,6 +361,7 @@ int mons_rare_abyss(int mcls)
     case MONS_DRACONIAN_ANNIHILATOR:
     case MONS_DRACONIAN_ZEALOT:
     case MONS_DRACONIAN_SHIFTER:
+    case MONS_HELLEPHANT:
         return 3;
 
     case MONS_DANCING_WEAPON:
@@ -499,18 +396,18 @@ bool mons_pan(int mcls)
     case MONS_LICH:
     case MONS_MUMMY:
     case MONS_NECROPHAGE:
+    case MONS_PHANTASMAL_WARRIOR:
     case MONS_PHANTOM:
     case MONS_SHADOW:
     case MONS_SKELETON_LARGE:
     case MONS_SKELETON_SMALL:
-    case MONS_SPECTRAL_WARRIOR:
+    case MONS_SIMULACRUM_LARGE:
+    case MONS_SIMULACRUM_SMALL:
     case MONS_VAMPIRE:
     case MONS_WIGHT:
     case MONS_WRAITH:
     case MONS_ZOMBIE_LARGE:
     case MONS_ZOMBIE_SMALL:
-    case MONS_SIMULACRUM_LARGE:
-    case MONS_SIMULACRUM_SMALL:
     // "things"
     case MONS_ABOMINATION_LARGE:
     case MONS_ABOMINATION_SMALL:
@@ -532,7 +429,7 @@ bool mons_pan(int mcls)
     case MONS_STONE_GOLEM:
     case MONS_TOENAIL_GOLEM:
     case MONS_WOOD_GOLEM:
-    // dragon(s)
+    // dragons
     case MONS_MOTTLED_DRAGON:
     // elementals
     case MONS_AIR_ELEMENTAL:
@@ -547,11 +444,12 @@ bool mons_pan(int mcls)
     case MONS_BLUE_DEATH:
     case MONS_CACODEMON:
     case MONS_CHAOS_SPAWN:
+    case MONS_DEMONIC_CRAWLER:
     case MONS_EXECUTIONER:
     case MONS_GREEN_DEATH:
     case MONS_HELLWING:
+    case MONS_IRON_IMP:
     case MONS_LEMURE:
-    case MONS_MANES:
     case MONS_MIDGE:
     case MONS_NEQOXEC:
     case MONS_ORANGE_DEMON:
@@ -584,7 +482,7 @@ int mons_standard_level(int mcls)
     case MONS_SMALL_SNAKE:
         return 2;
 
-    case MONS_GIANT_BAT:
+    case MONS_MEGABAT:
     case MONS_KOBOLD:
     case MONS_RAT:
         return 4;
@@ -607,7 +505,7 @@ int mons_standard_level(int mcls)
         return 7;
 
     case MONS_FUNGUS:
-    case MONS_GIANT_ANT:
+    case MONS_WORKER_ANT:
     case MONS_GIANT_EYEBALL:
     case MONS_HOUND:
     case MONS_IGUANA:
@@ -615,6 +513,7 @@ int mons_standard_level(int mcls)
     case MONS_ORC_WIZARD:
     case MONS_PHANTOM:
     case MONS_SCORPION:
+    case MONS_SKELETON_SMALL:
         return 8;
 
     case MONS_WATER_MOCCASIN:
@@ -625,10 +524,11 @@ int mons_standard_level(int mcls)
     case MONS_NECROPHAGE:
     case MONS_QUASIT:
     case MONS_ZOMBIE_SMALL:
+    case MONS_SKY_BEAST:
         return 9;
 
     case MONS_DEEP_ELF_SOLDIER:
-    case MONS_GIANT_BEETLE:
+    case MONS_GOLIATH_BEETLE:
     case MONS_GIANT_FROG:
     case MONS_GIANT_SPORE:
     case MONS_MUMMY:
@@ -650,6 +550,12 @@ int mons_standard_level(int mcls)
     case MONS_MANTICORE:
     case MONS_PLANT:
     case MONS_WYVERN:
+    // item mimics
+    case MONS_ARMOUR_MIMIC:
+    case MONS_GOLD_MIMIC:
+    case MONS_POTION_MIMIC:
+    case MONS_SCROLL_MIMIC:
+    case MONS_WEAPON_MIMIC:
         return 12;
 
     case MONS_BIG_KOBOLD:
@@ -671,24 +577,23 @@ int mons_standard_level(int mcls)
     case MONS_TRAPDOOR_SPIDER:
         return 14;
 
-    case MONS_ARMOUR_MIMIC:
     case MONS_BRAIN_WORM:
     case MONS_CYCLOPS:
     case MONS_EFREET:
-    case MONS_ETTIN:
     case MONS_EYE_OF_DEVASTATION:
-    case MONS_GOLD_MIMIC:
     case MONS_HYDRA:
     case MONS_MOTTLED_DRAGON:
-    case MONS_POTION_MIMIC:
-    case MONS_SCROLL_MIMIC:
     case MONS_SKELETAL_WARRIOR:
-    case MONS_WEAPON_MIMIC:
+    // feature mimics
+    // case MONS_PORTAL_MIMIC:
+    case MONS_STAIR_MIMIC:
+    case MONS_SHOP_MIMIC:
+    case MONS_FOUNTAIN_MIMIC:
         return 15;
 
     case MONS_BLINK_FROG:
     case MONS_BUTTERFLY:
-    case MONS_GIANT_BLOWFLY:
+    case MONS_VAMPIRE_MOSQUITO:
     case MONS_GUARDIAN_SERPENT:
     case MONS_RAKSHASA:
     case MONS_SLIME_CREATURE:
@@ -714,6 +619,7 @@ int mons_standard_level(int mcls)
     case MONS_GARGOYLE:
     case MONS_GIANT_AMOEBA:
     case MONS_KOBOLD_DEMONOLOGIST:
+    case MONS_SKELETON_LARGE: // Higher than actual threat level so that they will still show up in The Vaults
         return 18;
 
     case MONS_GIANT_SLUG:
@@ -732,10 +638,9 @@ int mons_standard_level(int mcls)
     case MONS_EARTH_ELEMENTAL:
     case MONS_FIRE_ELEMENTAL:
     case MONS_GIANT_ORANGE_BRAIN:
-    case MONS_GIANT_SNAIL:
+    case MONS_AGATE_SNAIL:
     case MONS_GREAT_ORB_OF_EYES:
     case MONS_ICE_DRAGON:
-    case MONS_SKELETON_LARGE:
     case MONS_NAGA_MAGE:
     case MONS_NAGA_WARRIOR:
     case MONS_NECROMANCER:
@@ -743,10 +648,10 @@ int mons_standard_level(int mcls)
     case MONS_QUEEN_BEE:
     case MONS_RED_WASP:
     case MONS_SHADOW_WRAITH:
-    case MONS_SKELETON_SMALL:
     case MONS_SPINY_WORM:
     case MONS_VERY_UGLY_THING:
     case MONS_HARPY:
+    case MONS_FIRE_CRAB:
         return 20;
 
     case MONS_BOULDER_BEETLE:
@@ -781,10 +686,11 @@ int mons_standard_level(int mcls)
     case MONS_INSUBSTANTIAL_WISP:
     case MONS_LICH:
     case MONS_STONE_GIANT:
+    case MONS_ETTIN:
         return 25;
 
     case MONS_DEEP_ELF_CONJURER:
-    case MONS_SPECTRAL_WARRIOR:
+    case MONS_PHANTASMAL_WARRIOR:
     case MONS_STORM_DRAGON:
         return 26;
 
@@ -794,6 +700,7 @@ int mons_standard_level(int mcls)
         return 27;
 
     case MONS_ANCIENT_LICH:
+    case MONS_BONE_DRAGON:
     case MONS_DEEP_ELF_ANNIHILATOR:
     case MONS_DEEP_ELF_DEATH_MAGE:
     case MONS_DEEP_ELF_DEMONOLOGIST:
@@ -803,7 +710,6 @@ int mons_standard_level(int mcls)
     case MONS_IRON_DRAGON:
     case MONS_QUICKSILVER_DRAGON:
     case MONS_SHADOW_DRAGON:
-    case MONS_SKELETAL_DRAGON:
     case MONS_TITAN:
         return 30;
 
@@ -843,7 +749,7 @@ int mons_standard_rare(int mcls)
     case MONS_SALAMANDER:
         return 500;
 
-    case MONS_GIANT_BAT:
+    case MONS_MEGABAT:
     case MONS_GIANT_FROG:
     case MONS_GOBLIN:
     case MONS_HILL_GIANT:
@@ -861,7 +767,7 @@ int mons_standard_rare(int mcls)
         return 99;
 
     case MONS_CENTAUR_WARRIOR:
-    case MONS_GIANT_ANT:
+    case MONS_WORKER_ANT:
     case MONS_SNAKE:
         return 80;
 
@@ -898,8 +804,8 @@ int mons_standard_rare(int mcls)
         return 53;
 
     case MONS_BIG_KOBOLD:
-    case MONS_GIANT_BEETLE:
-    case MONS_GIANT_BLOWFLY:
+    case MONS_GOLIATH_BEETLE:
+    case MONS_VAMPIRE_MOSQUITO:
     case MONS_GIANT_COCKROACH:
     case MONS_GIANT_GECKO:
     case MONS_IGUANA:
@@ -907,6 +813,7 @@ int mons_standard_rare(int mcls)
     case MONS_HIPPOGRIFF:
     case MONS_HYDRA:
     case MONS_ICE_BEAST:
+    case MONS_SKY_BEAST:
     case MONS_KILLER_BEE:
     case MONS_ORC_WIZARD:
     case MONS_QUOKKA:
@@ -964,7 +871,6 @@ int mons_standard_rare(int mcls)
     case MONS_GIANT_SLUG:
         return 32;
 
-    case MONS_ARMOUR_MIMIC:
     case MONS_WATER_MOCCASIN:
     case MONS_DRAGON:
     case MONS_ETTIN:
@@ -972,19 +878,15 @@ int mons_standard_rare(int mcls)
     case MONS_CROCODILE:
     case MONS_GIANT_MITE:
     case MONS_GNOLL:
-    case MONS_GOLD_MIMIC:
     case MONS_KOMODO_DRAGON:
     case MONS_MUMMY:
     case MONS_NECROPHAGE:
-    case MONS_POTION_MIMIC:
-    case MONS_SCROLL_MIMIC:
     case MONS_QUASIT:
     case MONS_SKELETAL_WARRIOR:
     case MONS_SMALL_SNAKE:
     case MONS_SOUL_EATER:
     case MONS_SPINY_WORM:
     case MONS_VAMPIRE:
-    case MONS_WEAPON_MIMIC:
     case MONS_YELLOW_WASP:
         return 30;
 
@@ -1008,23 +910,35 @@ int mons_standard_rare(int mcls)
     case MONS_RED_WASP:
     case MONS_SIMULACRUM_SMALL:
     case MONS_SIMULACRUM_LARGE:
+    // item mimics
+    case MONS_ARMOUR_MIMIC:
+    case MONS_GOLD_MIMIC:
+    case MONS_POTION_MIMIC:
+    case MONS_SCROLL_MIMIC:
+    case MONS_WEAPON_MIMIC:
+    // feature mimics
+    // case MONS_PORTAL_MIMIC:
+    case MONS_STAIR_MIMIC:
+    case MONS_SHOP_MIMIC:
+    case MONS_FOUNTAIN_MIMIC:
         return 25;
 
     case MONS_BUTTERFLY:
     case MONS_FUNGUS:
-    case MONS_GIANT_SNAIL:
+    case MONS_AGATE_SNAIL:
     case MONS_ICE_DRAGON:
     case MONS_INSUBSTANTIAL_WISP:
+    case MONS_PHANTASMAL_WARRIOR:
     case MONS_RAKSHASA:
     case MONS_REDBACK:
     case MONS_SHADOW_DRAGON:
-    case MONS_SPECTRAL_WARRIOR:
     case MONS_SPHINX:
     case MONS_STEAM_DRAGON:
     case MONS_STORM_DRAGON:
     case MONS_VERY_UGLY_THING:
     case MONS_WIZARD:
     case MONS_HARPY:
+    case MONS_FIRE_CRAB:
         return 20;
 
     case MONS_BORING_BEETLE:
@@ -1066,6 +980,7 @@ int mons_standard_rare(int mcls)
     case MONS_GARGOYLE:
         return 6;
 
+    case MONS_BONE_DRAGON:
     case MONS_CRYSTAL_GOLEM:
     case MONS_DANCING_WEAPON:
     case MONS_DEEP_ELF_HIGH_PRIEST:
@@ -1074,7 +989,6 @@ int mons_standard_rare(int mcls)
     case MONS_IRON_DRAGON:
     case MONS_NAGA_MAGE:
     case MONS_NAGA_WARRIOR:
-    case MONS_SKELETAL_DRAGON:
     case MONS_QUICKSILVER_DRAGON:
     case MONS_VAPOUR:
         return 5;
@@ -1104,6 +1018,51 @@ int mons_standard_rare(int mcls)
     }
 }
 
+// The Dwarven Hall
+int mons_dwarf_level(int mcls)
+{
+    int mlev = absdungeon_depth(BRANCH_DWARVEN_HALL, 1);
+
+    if (!mons_dwarf_rare(mcls))
+        return mlev + 99;
+    // Depths are irrelevant for a depth-1 branch.
+    return mlev + 1;
+}
+
+int mons_dwarf_rare(int mcls)
+{
+    switch (mcls)
+    {
+    case MONS_DEEP_DWARF:
+        return 100;
+    case MONS_DEEP_DWARF_SCION:
+    case MONS_DEEP_DWARF_ARTIFICER:
+        return 70;
+    case MONS_DEEP_DWARF_NECROMANCER:
+    case MONS_DEEP_DWARF_BERSERKER:
+    case MONS_DEEP_DWARF_DEATH_KNIGHT:
+        return 60;
+    case MONS_UNBORN_DEEP_DWARF:
+        return 40;
+    case MONS_WRAITH:
+        return 30;
+    case MONS_PHANTASMAL_WARRIOR:
+    case MONS_EIDOLON:
+    case MONS_FIRE_GIANT:
+    case MONS_FROST_GIANT:
+    case MONS_STONE_GIANT:
+        return 5;
+    case MONS_DEEP_TROLL:
+    case MONS_IRON_TROLL:
+    case MONS_ROCK_TROLL:
+    case MONS_SHADOW_WRAITH:
+        return 2;
+
+    default:
+        return 0;
+    }
+}
+
 // The Orcish Mines
 int mons_mineorc_level(int mcls)
 {
@@ -1209,32 +1168,26 @@ int mons_hallelf_level(int mcls)
     case MONS_DEEP_ELF_FIGHTER:
     case MONS_ORC:
     case MONS_ORC_WARRIOR:
-        mlev++;
-        break;
-
     case MONS_ORC_WIZARD:
     case MONS_DEEP_ELF_MAGE:
     case MONS_DEEP_ELF_SUMMONER:
-        mlev += 2;
-        break;
-
     case MONS_FUNGUS:
     case MONS_DEEP_ELF_CONJURER:
     case MONS_SHAPESHIFTER:
     case MONS_ORC_KNIGHT:
-        mlev += 3;
+        mlev += 1;
         break;
 
     case MONS_ORC_SORCERER:
     case MONS_DEEP_ELF_PRIEST:
     case MONS_GLOWING_SHAPESHIFTER:
     case MONS_DEEP_ELF_KNIGHT:
-        mlev += 4;
+        mlev += 2;
         break;
 
     case MONS_ORC_PRIEST:
     case MONS_ORC_HIGH_PRIEST:
-        mlev += 5;
+        mlev += 3;
         break;
 
     case MONS_DEEP_ELF_HIGH_PRIEST:
@@ -1242,12 +1195,12 @@ int mons_hallelf_level(int mcls)
     case MONS_DEEP_ELF_ANNIHILATOR:
     case MONS_DEEP_ELF_SORCERER:
     case MONS_DEEP_ELF_DEATH_MAGE:
-        mlev += 7;
+        mlev += 5;
         break;
 
     case MONS_DEEP_ELF_BLADEMASTER:
     case MONS_DEEP_ELF_MASTER_ARCHER:
-        mlev += 10;
+        mlev += 8;
         break;
 
     default:
@@ -1327,7 +1280,7 @@ int mons_lair_level(int mcls)
     switch (mcls)
     {
     case MONS_GIANT_GECKO:
-    case MONS_GIANT_BAT:
+    case MONS_MEGABAT:
     case MONS_JACKAL:
     case MONS_GIANT_NEWT:
     case MONS_RAT:
@@ -1339,7 +1292,7 @@ int mons_lair_level(int mcls)
 
     case MONS_GIANT_FROG:
     case MONS_GILA_MONSTER:
-    case MONS_GREY_RAT:
+    case MONS_PORCUPINE:
     case MONS_HOUND:
     case MONS_BLACK_BEAR:
     case MONS_WORM:
@@ -1359,19 +1312,18 @@ int mons_lair_level(int mcls)
 
     case MONS_WATER_MOCCASIN:
     case MONS_BUTTERFLY:
-    case MONS_GIANT_BEETLE:
+    case MONS_GOLIATH_BEETLE:
     case MONS_GIANT_SLUG:
     case MONS_HIPPOGRIFF:
     case MONS_PLANT:
     case MONS_SPINY_FROG:
     case MONS_WAR_DOG:
     case MONS_YELLOW_WASP:
-    case MONS_BEAR:
         mlev += 3;
         break;
 
     case MONS_BLINK_FROG:
-    case MONS_GIANT_SNAIL:
+    case MONS_AGATE_SNAIL:
     case MONS_GIANT_SPORE:
     case MONS_KOMODO_DRAGON:
     case MONS_ORANGE_RAT:
@@ -1401,6 +1353,7 @@ int mons_lair_level(int mcls)
     case MONS_LINDWURM:
     case MONS_REDBACK:
     case MONS_WANDERING_MUSHROOM:
+    case MONS_ELEPHANT:
         mlev += 6;
         break;
 
@@ -1408,6 +1361,8 @@ int mons_lair_level(int mcls)
     case MONS_BOULDER_BEETLE:
     case MONS_DEATH_YAK:
     case MONS_SPINY_WORM:
+    case MONS_STAIR_MIMIC:
+    case MONS_FIRE_CRAB:
         mlev += 7;
         break;
 
@@ -1425,10 +1380,10 @@ int mons_lair_rare(int mcls)
     case MONS_RAT:
         return 200;
 
-    case MONS_GIANT_BAT:
+    case MONS_MEGABAT:
     case MONS_GIANT_TOAD:
     case MONS_GIANT_FROG:
-    case MONS_GREY_RAT:
+    case MONS_PORCUPINE:
     case MONS_QUOKKA:
         return 99;
 
@@ -1454,7 +1409,7 @@ int mons_lair_rare(int mcls)
     case MONS_HOUND:
         return 60;
 
-    case MONS_GIANT_SNAIL:
+    case MONS_AGATE_SNAIL:
         return 56;
 
     case MONS_GIANT_SLUG:
@@ -1476,6 +1431,7 @@ int mons_lair_rare(int mcls)
         return 45;
 
     case MONS_SHEEP:
+    case MONS_ELEPHANT:
     case MONS_FIRE_DRAKE:
         return 36;
 
@@ -1499,12 +1455,11 @@ int mons_lair_rare(int mcls)
         return 20;
 
     case MONS_BLACK_BEAR:
-    case MONS_BEAR:
     case MONS_GRIZZLY_BEAR:
     case MONS_POLAR_BEAR:
         return 15;
 
-    case MONS_GIANT_BEETLE:
+    case MONS_GOLIATH_BEETLE:
     case MONS_SCORPION:
     case MONS_OKLOB_PLANT:
     case MONS_STEAM_DRAGON:
@@ -1535,6 +1490,10 @@ int mons_lair_rare(int mcls)
     case MONS_GIANT_SPORE:
         return 2;
 
+    case MONS_STAIR_MIMIC:
+    case MONS_FIRE_CRAB:
+        return 1;
+
     default:
         return 0;
     }
@@ -1547,8 +1506,7 @@ int mons_swamp_level(int mcls)
 
     switch (mcls)
     {
-    case MONS_GIANT_BAT:
-    case MONS_GIANT_BLOWFLY:
+    case MONS_MEGABAT:
     case MONS_GIANT_FROG:
     case MONS_GIANT_AMOEBA:
     case MONS_GIANT_SLUG:
@@ -1570,9 +1528,10 @@ int mons_swamp_level(int mcls)
     case MONS_SNAKE:
     case MONS_BUTTERFLY:
     case MONS_CROCODILE:
-    case MONS_GIANT_MOSQUITO:
-    case MONS_GIANT_SNAIL:
+    case MONS_VAMPIRE_MOSQUITO:
+    case MONS_AGATE_SNAIL:
     case MONS_HYDRA:
+    case MONS_BOG_MUMMY:
         mlev += 2;
         break;
 
@@ -1585,6 +1544,7 @@ int mons_swamp_level(int mcls)
     case MONS_RED_WASP:
     case MONS_SPINY_FROG:
     case MONS_SWAMP_DRAGON:
+    // case MONS_PORTAL_MIMIC:
     case MONS_UGLY_THING:
         mlev += 3;
         break;
@@ -1608,7 +1568,7 @@ int mons_swamp_rare(int mcls)
 {
     switch (mcls)
     {
-    case MONS_GIANT_MOSQUITO:
+    case MONS_VAMPIRE_MOSQUITO:
         return 250;
 
     case MONS_PLANT:
@@ -1618,10 +1578,7 @@ int mons_swamp_rare(int mcls)
     case MONS_ALLIGATOR:
         return 150;
 
-    case MONS_GIANT_BLOWFLY:
-        return 100;
-
-    case MONS_GIANT_BAT:
+    case MONS_MEGABAT:
     case MONS_FUNGUS:
         return 99;
 
@@ -1636,6 +1593,7 @@ int mons_swamp_rare(int mcls)
 
     case MONS_RAT:
     case MONS_GIANT_LEECH:
+    case MONS_BOG_MUMMY:
         return 61;
 
     case MONS_SLIME_CREATURE:
@@ -1661,6 +1619,7 @@ int mons_swamp_rare(int mcls)
         return 25;
 
     case MONS_WORM:
+    // case MONS_PORTAL_MIMIC:
         return 20;
 
     case MONS_KOMODO_DRAGON:
@@ -1682,7 +1641,7 @@ int mons_swamp_rare(int mcls)
     case MONS_GIANT_GECKO:
     case MONS_GIANT_NEWT:
     case MONS_GIANT_SLUG:
-    case MONS_GIANT_SNAIL:
+    case MONS_AGATE_SNAIL:
     case MONS_TENTACLED_MONSTROSITY:
         return 10;
 
@@ -1698,7 +1657,7 @@ int mons_shoals_level(int mcls)
     switch (mcls)
     {
     case MONS_BUTTERFLY:
-    case MONS_GIANT_BAT:
+    case MONS_MEGABAT:
         break;
 
     case MONS_MERFOLK:
@@ -1727,6 +1686,7 @@ int mons_shoals_level(int mcls)
     case MONS_MERFOLK_JAVELINEER:
     case MONS_MERFOLK_IMPALER:
     case MONS_MERFOLK_AQUAMANCER:
+    // case MONS_PORTAL_MIMIC:
         mlev += 4;
         break;
 
@@ -1752,7 +1712,7 @@ int mons_shoals_rare(int mcls)
         return 40;
 
     case MONS_HIPPOGRIFF:
-    case MONS_GIANT_BAT:
+    case MONS_MEGABAT:
     case MONS_BUTTERFLY:
     case MONS_SHARK:
         return 35;
@@ -1775,6 +1735,7 @@ int mons_shoals_rare(int mcls)
     case MONS_OKLOB_PLANT:
     case MONS_ALLIGATOR_SNAPPING_TURTLE:
     case MONS_MERFOLK_JAVELINEER:
+    // case MONS_PORTAL_MIMIC:
         return 10;
 
     default:
@@ -1812,6 +1773,7 @@ int mons_pitsnake_level(int mcls)
         break;
 
     case MONS_GREATER_NAGA:
+    // case MONS_PORTAL_MIMIC:
         mlev += 5;
         break;
 
@@ -1847,6 +1809,7 @@ int mons_pitsnake_rare(int mcls)
     case MONS_GREATER_NAGA:
     case MONS_GUARDIAN_SERPENT:
     case MONS_SMALL_SNAKE:
+    // case MONS_PORTAL_MIMIC:
         return 15;
 
     default:
@@ -1854,6 +1817,96 @@ int mons_pitsnake_rare(int mcls)
     }
 }
 
+// The Spider Nest
+int mons_spidernest_level(int mcls)
+{
+    int mlev = absdungeon_depth(BRANCH_SPIDER_NEST, 1);
+
+    switch (mcls)
+    {
+    case MONS_GIANT_COCKROACH:
+    case MONS_GIANT_MITE:
+    case MONS_SCORPION:
+    case MONS_REDBACK:
+    case MONS_WOLF_SPIDER:
+    case MONS_TRAPDOOR_SPIDER:
+    case MONS_VAMPIRE_MOSQUITO:
+    case MONS_SPIDER:
+        mlev++;
+        break;
+
+    case MONS_YELLOW_WASP:
+    case MONS_RED_WASP:
+    case MONS_GOLIATH_BEETLE:
+        mlev += 2;
+        break;
+
+    case MONS_BORING_BEETLE:
+    case MONS_BOULDER_BEETLE:
+    case MONS_GIANT_CENTIPEDE:
+        mlev += 3;
+        break;
+
+    case MONS_EMPEROR_SCORPION:
+    case MONS_JUMPING_SPIDER:
+        mlev += 4;
+        break;
+
+    case MONS_TARANTELLA:
+    case MONS_GHOST_MOTH:
+    case MONS_DEMONIC_CRAWLER:
+        mlev += 5;
+        break;
+
+    default:
+        mlev += 99;
+    }
+
+    return (mlev);
+}
+
+int mons_spidernest_rare(int mcls)
+{
+    switch (mcls)
+    {
+    case MONS_GIANT_COCKROACH:
+    case MONS_GIANT_MITE:
+        return 60;
+
+    case MONS_SCORPION:
+    case MONS_REDBACK:
+    case MONS_WOLF_SPIDER:
+    case MONS_TRAPDOOR_SPIDER:
+        return 50;
+
+    case MONS_VAMPIRE_MOSQUITO:
+        return 40;
+
+    case MONS_YELLOW_WASP:
+    case MONS_RED_WASP:
+        return 20;
+
+    case MONS_GOLIATH_BEETLE:
+    case MONS_BORING_BEETLE:
+    case MONS_BOULDER_BEETLE:
+    case MONS_GIANT_CENTIPEDE:
+    case MONS_EMPEROR_SCORPION:
+    case MONS_SPIDER:
+        return 15;
+
+    case MONS_JUMPING_SPIDER:
+    case MONS_TARANTELLA:
+        return 10;
+
+    case MONS_GHOST_MOTH:
+    case MONS_DEMONIC_CRAWLER:
+        return 5;
+
+    default:
+        return 0;
+    }
+}
+
 // The Slime Pits
 int mons_pitslime_level(int mcls)
 {
@@ -2037,15 +2090,16 @@ int mons_crypt_level(int mcls)
     case MONS_ROTTING_HULK:
     case MONS_WRAITH:
     case MONS_FLYING_SKULL:
+    case MONS_SILENT_SPECTRE:
         mlev += 3;
         break;
 
+    case MONS_BONE_DRAGON:
     case MONS_FLAMING_CORPSE:
-    case MONS_SPECTRAL_WARRIOR:
+    case MONS_PHANTASMAL_WARRIOR:
     case MONS_SHADOW_WRAITH:
     case MONS_VAMPIRE_KNIGHT:
     case MONS_VAMPIRE_MAGE:
-    case MONS_SKELETAL_DRAGON:
     case MONS_ABOMINATION_SMALL:
     case MONS_MUMMY:
     case MONS_VAMPIRE:
@@ -2057,6 +2111,11 @@ int mons_crypt_level(int mcls)
     case MONS_ANCIENT_LICH:
     case MONS_LICH:
     case MONS_CURSE_SKULL:
+    // feature mimics
+    // case MONS_PORTAL_MIMIC:
+    case MONS_STAIR_MIMIC:
+    case MONS_SHOP_MIMIC:
+    case MONS_FOUNTAIN_MIMIC:
         mlev += 5;
         break;
 
@@ -2092,14 +2151,15 @@ int mons_crypt_rare(int mcls)
         return 33;
 
     case MONS_SHADOW:
+    case MONS_SILENT_SPECTRE:
         return 30;
 
     case MONS_NECROMANCER:
     case MONS_GHOUL:
         return 25;
 
+    case MONS_BONE_DRAGON:
     case MONS_MUMMY:
-    case MONS_SKELETAL_DRAGON:
         return 24;
 
     case MONS_VAMPIRE:
@@ -2113,7 +2173,7 @@ int mons_crypt_rare(int mcls)
     case MONS_ROTTING_HULK:
         return 17;
 
-    case MONS_SPECTRAL_WARRIOR:
+    case MONS_PHANTASMAL_WARRIOR:
         return 14;
 
     case MONS_FLYING_SKULL:
@@ -2131,6 +2191,11 @@ int mons_crypt_rare(int mcls)
         return 10;
 
     case MONS_ANCIENT_LICH:
+    // feature mimics
+    // case MONS_PORTAL_MIMIC:
+    case MONS_STAIR_MIMIC:
+    case MONS_SHOP_MIMIC:
+    case MONS_FOUNTAIN_MIMIC:
         return 8;
 
     case MONS_ABOMINATION_SMALL:
@@ -2235,6 +2300,64 @@ int mons_tomb_rare(int mcls)
     }
 }
 
+// The Enchanted Forest
+int mons_forest_level(int mcls)
+{
+    int mlev = absdungeon_depth(BRANCH_FOREST, 1);
+
+    switch (mcls)
+    {
+    case MONS_SPRIGGAN:
+    case MONS_SPRIGGAN_DRUID:
+    case MONS_GRIZZLY_BEAR:
+    case MONS_BLACK_BEAR:
+    case MONS_WOLF:
+        mlev++;
+        break;
+
+    case MONS_SPRIGGAN_RIDER:
+        mlev += 2;
+        break;
+
+    case MONS_SPRIGGAN_AIR_MAGE:
+        mlev += 3;
+
+    case MONS_SPRIGGAN_DEFENDER:
+        mlev += 5;
+        break;
+
+    default:
+        mlev += 99;
+    }
+
+    return (mlev);
+}
+
+int mons_forest_rare(int mcls)
+{
+    switch (mcls)
+    {
+    case MONS_SPRIGGAN:
+        return 99;
+
+    case MONS_SPRIGGAN_DRUID:
+    case MONS_SPRIGGAN_RIDER:
+    case MONS_GRIZZLY_BEAR:
+    case MONS_WOLF:
+        return 40;
+
+    case MONS_SPRIGGAN_AIR_MAGE:
+    case MONS_BLACK_BEAR:
+        return 20;
+
+    case MONS_SPRIGGAN_DEFENDER:
+        return 10;
+
+    default:
+        return 0;
+    }
+}
+
 // The Halls of Zot
 int mons_hallzot_level(int mcls)
 {
@@ -2246,12 +2369,13 @@ int mons_hallzot_level(int mcls)
     case MONS_GUARDIAN_MUMMY:
         mlev += 6;
         break;
+    case MONS_BONE_DRAGON:
     case MONS_KILLER_KLOWN:
     case MONS_SHADOW_DRAGON:
-    case MONS_SKELETAL_DRAGON:
     case MONS_STORM_DRAGON:
     case MONS_CURSE_TOE:
     case MONS_ORB_GUARDIAN:
+    case MONS_GHOST_MOTH:
         mlev += 5;
         break;
     case MONS_DEATH_COB:
@@ -2267,6 +2391,7 @@ int mons_hallzot_level(int mcls)
     case MONS_PURPLE_DRACONIAN:
     case MONS_PALE_DRACONIAN:
     case MONS_GREEN_DRACONIAN:
+    case MONS_GREY_DRACONIAN:
     case MONS_DRACONIAN_CALLER:
     case MONS_DRACONIAN_MONK:
     case MONS_DRACONIAN_SCORCHER:
@@ -2303,13 +2428,13 @@ int mons_hallzot_rare(int mcls)
         return 50;
     case MONS_GOLDEN_DRAGON:
         return 42;
+    case MONS_BONE_DRAGON:
     case MONS_DEATH_COB:
     case MONS_DRAGON:
     case MONS_ICE_DRAGON:
-    case MONS_SKELETAL_DRAGON:
         return 40;
     case MONS_SHADOW_DRAGON:
-    case MONS_DEATH_DRAKE:
+    case MONS_GHOST_MOTH:
         return 30;
     case MONS_GUARDIAN_MUMMY:
     case MONS_ELECTRIC_GOLEM:
@@ -2324,6 +2449,7 @@ int mons_hallzot_rare(int mcls)
     case MONS_PURPLE_DRACONIAN:
     case MONS_PALE_DRACONIAN:
     case MONS_GREEN_DRACONIAN:
+    case MONS_GREY_DRACONIAN:
         return 18;
 
     case MONS_DRACONIAN_CALLER:
@@ -2389,6 +2515,7 @@ int mons_dis_level(int mcls)
     case MONS_DEATH_DRAKE:
     case MONS_HAIRY_DEVIL:
     case MONS_IRON_DEVIL:
+    case MONS_IRON_IMP:
     case MONS_VAMPIRE:
     case MONS_WRAITH:
         mlev += 3;
@@ -2400,16 +2527,16 @@ int mons_dis_level(int mcls)
     case MONS_ICE_DEVIL:
     case MONS_ICE_DRAGON:
     case MONS_LICH:
+    case MONS_PHANTASMAL_WARRIOR:
     case MONS_REAPER:
     case MONS_SOUL_EATER:
-    case MONS_SPECTRAL_WARRIOR:
         mlev += 5;
         break;
 
     case MONS_ANCIENT_LICH:
+    case MONS_BONE_DRAGON:
     case MONS_FIEND:
     case MONS_IRON_DRAGON:
-    case MONS_SKELETAL_DRAGON:
         mlev += 6;
         break;
 
@@ -2426,6 +2553,7 @@ int mons_dis_rare(int mcls)
     {
     case MONS_IMP:
     case MONS_IRON_DEVIL:
+    case MONS_IRON_IMP:
     case MONS_ZOMBIE_LARGE:
     case MONS_ZOMBIE_SMALL:
         return 99;
@@ -2471,8 +2599,8 @@ int mons_dis_rare(int mcls)
     case MONS_SKELETON_SMALL:
         return 25;
 
+    case MONS_BONE_DRAGON:
     case MONS_HELL_HOG:
-    case MONS_SKELETAL_DRAGON:
         return 20;
 
     case MONS_VAMPIRE:
@@ -2490,9 +2618,9 @@ int mons_dis_rare(int mcls)
     case MONS_HELL_KNIGHT:
     case MONS_IRON_GOLEM:
     case MONS_LICH:
+    case MONS_PHANTASMAL_WARRIOR:
     case MONS_ROTTING_DEVIL:
     case MONS_SOUL_EATER:
-    case MONS_SPECTRAL_WARRIOR:
     case MONS_STONE_GOLEM:
         return 10;
 
@@ -2554,18 +2682,18 @@ int mons_gehenna_level(int mcls)
     case MONS_FLAYED_GHOST:
     case MONS_HELLION:
     case MONS_TORMENTOR:
+    case MONS_FIRE_CRAB:
         mlev += 5;
         break;
 
     case MONS_ANCIENT_LICH:
+    case MONS_BONE_DRAGON:
     case MONS_FIEND:
     case MONS_LICH:
+    case MONS_PHANTASMAL_WARRIOR:
     case MONS_PIT_FIEND:
     case MONS_REAPER:
-    case MONS_SERPENT_OF_HELL:
-    case MONS_SKELETAL_DRAGON:
     case MONS_SOUL_EATER:
-    case MONS_SPECTRAL_WARRIOR:
         mlev += 6;
         break;
 
@@ -2636,11 +2764,11 @@ int mons_gehenna_rare(int mcls)
         return 21;
 
     case MONS_HAIRY_DEVIL:
-    case MONS_SPECTRAL_WARRIOR:
+    case MONS_PHANTASMAL_WARRIOR:
         return 20;
 
+    case MONS_BONE_DRAGON:
     case MONS_CLAY_GOLEM:
-    case MONS_SKELETAL_DRAGON:
         return 10;
 
     case MONS_STONE_GOLEM:
@@ -2653,10 +2781,10 @@ int mons_gehenna_rare(int mcls)
     case MONS_FIEND:
     case MONS_IRON_GOLEM:
     case MONS_SOUL_EATER:
+    case MONS_FIRE_CRAB:
         return 5;
 
     case MONS_ANCIENT_LICH:
-    case MONS_SERPENT_OF_HELL:
         return 4;
 
     default:
@@ -2700,7 +2828,7 @@ int mons_cocytus_level(int mcls)
     case MONS_HAIRY_DEVIL:
     case MONS_HUNGRY_GHOST:
     case MONS_MUMMY:
-    case MONS_SPECTRAL_WARRIOR:
+    case MONS_PHANTASMAL_WARRIOR:
     case MONS_WRAITH:
         mlev += 4;
         break;
@@ -2708,13 +2836,14 @@ int mons_cocytus_level(int mcls)
     case MONS_ICE_DEVIL:
     case MONS_ICE_DRAGON:
     case MONS_TORMENTOR:
+    case MONS_WHITE_IMP:
         mlev += 5;
         break;
 
     case MONS_ANCIENT_LICH:
+    case MONS_BONE_DRAGON:
     case MONS_LICH:
     case MONS_REAPER:
-    case MONS_SKELETAL_DRAGON:
     case MONS_SOUL_EATER:
         mlev += 6;
         break;
@@ -2742,6 +2871,7 @@ int mons_cocytus_rare(int mcls)
 
     case MONS_BLUE_DEVIL:
     case MONS_ICE_DEVIL:
+    case MONS_WHITE_IMP:
         return 76;
 
     case MONS_FLYING_SKULL:
@@ -2781,14 +2911,14 @@ int mons_cocytus_rare(int mcls)
     case MONS_PHANTOM:
         return 25;
 
-    case MONS_SPECTRAL_WARRIOR:
+    case MONS_PHANTASMAL_WARRIOR:
         return 20;
 
     case MONS_SOUL_EATER:
         return 19;
 
+    case MONS_BONE_DRAGON:
     case MONS_LICH:
-    case MONS_SKELETAL_DRAGON:
     case MONS_SIMULACRUM_LARGE:
     case MONS_SIMULACRUM_SMALL:
         return 12;
@@ -2832,16 +2962,17 @@ int mons_tartarus_level(int mcls)
     case MONS_SHADOW:
     case MONS_SHADOW_DEMON:
     case MONS_WRAITH:
+    case MONS_SILENT_SPECTRE:
         mlev += 3;
         break;
 
     case MONS_BLUE_DEVIL:
+    case MONS_BONE_DRAGON:
     case MONS_FLAYED_GHOST:
     case MONS_HUNGRY_GHOST:
     case MONS_ICE_DEVIL:
     case MONS_MUMMY:
-    case MONS_SKELETAL_DRAGON:
-    case MONS_SPECTRAL_WARRIOR:
+    case MONS_PHANTASMAL_WARRIOR:
     case MONS_TORMENTOR:
     case MONS_SIMULACRUM_LARGE:
     case MONS_SIMULACRUM_SMALL:
@@ -2912,7 +3043,7 @@ int mons_tartarus_rare(int mcls)
     case MONS_SHADOW_DEMON:
         return 50;
 
-    case MONS_SPECTRAL_WARRIOR:
+    case MONS_PHANTASMAL_WARRIOR:
         return 45;
 
     case MONS_VAMPIRE:
@@ -2922,10 +3053,11 @@ int mons_tartarus_rare(int mcls)
     case MONS_TORMENTOR:
         return 42;
 
-    case MONS_SKELETAL_DRAGON:
+    case MONS_BONE_DRAGON:
         return 40;
 
     case MONS_SOUL_EATER:
+    case MONS_SILENT_SPECTRE:
         return 35;
 
     case MONS_ICE_DEVIL:
