@@ -52,6 +52,51 @@
       }
     },
 
+    {  MST_DEEP_DWARF_NECROMANCER,
+      {
+       SPELL_PAIN,
+       SPELL_DISPEL_UNDEAD,
+       SPELL_NO_SPELL,
+       SPELL_ANIMATE_DEAD,
+       SPELL_VAMPIRIC_DRAINING, // offense
+       SPELL_VAMPIRIC_DRAINING  // and defensive
+      }
+    },
+
+    {  MST_UNBORN_DEEP_DWARF,
+      {
+       SPELL_AGONY,
+       SPELL_DISPEL_UNDEAD,
+       SPELL_MIRROR_DAMAGE,
+       SPELL_ANIMATE_DEAD,
+       SPELL_HAUNT,
+       SPELL_MIRROR_DAMAGE
+      }
+    },
+
+
+    {  MST_BK_TROG,
+      {
+       SPELL_BROTHERS_IN_ARMS,
+       SPELL_NO_SPELL,
+       SPELL_TROGS_HAND,
+       SPELL_BERSERKER_RAGE,
+       SPELL_NO_SPELL,
+       SPELL_NO_SPELL
+      }
+    },
+
+    {  MST_BK_YREDELEMNUL,
+      {
+       SPELL_NO_SPELL,
+       SPELL_ANIMATE_DEAD,
+       SPELL_MIRROR_DAMAGE,
+       SPELL_SUMMON_UNDEAD, // cheaper to fake recall and gift with summon spell
+       SPELL_DRAIN_LIFE,
+       SPELL_MIRROR_DAMAGE
+      }
+    },
+
     {  MST_GUARDIAN_SERPENT,
       {
        SPELL_TELEPORT_OTHER,
@@ -176,7 +221,7 @@
     {  MST_GIANT_ORANGE_BRAIN,
       {
        SPELL_BRAIN_FEED,
-       SPELL_POLYMORPH_OTHER,
+       SPELL_CAUSE_FEAR,
        SPELL_SHADOW_CREATURES,
        SPELL_CONFUSE,
        SPELL_BLINK,
@@ -371,6 +416,28 @@
       }
     },
 
+    {  MST_FIEND,
+      {
+       SPELL_HELLFIRE,
+       SPELL_HELLFIRE,
+       SPELL_NO_SPELL,
+       SPELL_SYMBOL_OF_TORMENT,
+       SPELL_MELEE,
+       SPELL_NO_SPELL
+      }
+    },
+
+    {  MST_PIT_FIEND,
+      {
+       SPELL_HELLFIRE,
+       SPELL_HELLFIRE,
+       SPELL_MELEE,
+       SPELL_SYMBOL_OF_TORMENT,
+       SPELL_MELEE,
+       SPELL_NO_SPELL
+      }
+    },
+
     {  MST_ICE_FIEND,
       {
        SPELL_BOLT_OF_COLD,
@@ -461,7 +528,7 @@
 
     {  MST_SMOKE_DEMON,
       {
-       SPELL_STICKY_FLAME,
+       SPELL_STICKY_FLAME_RANGE,
        SPELL_STEAM_BALL,
        SPELL_NO_SPELL,
        SPELL_SMITING,
@@ -472,11 +539,11 @@
 
     {  MST_CACODEMON,
       {
-       SPELL_CALL_IMP,
-       SPELL_CALL_IMP,
-       SPELL_CALL_IMP,
+       SPELL_ENERGY_BOLT,
+       SPELL_SLOW,
+       SPELL_NO_SPELL,
        SPELL_POLYMORPH_OTHER,
-       SPELL_DIG,
+       SPELL_CONFUSE,
        SPELL_SUMMON_DEMON
       }
     },
@@ -487,7 +554,7 @@
        SPELL_POISONOUS_CLOUD,
        SPELL_NO_SPELL,
        SPELL_VENOM_BOLT,
-       SPELL_CALL_IMP,
+       SPELL_NO_SPELL,
        SPELL_BLINK
       }
     },
@@ -497,7 +564,7 @@
        SPELL_BOLT_OF_FIRE,
        SPELL_FIREBALL,
        SPELL_NO_SPELL,
-       SPELL_STICKY_FLAME,
+       SPELL_STICKY_FLAME_RANGE,
        SPELL_SMITING,
        SPELL_TELEPORT_SELF
       }
@@ -508,7 +575,7 @@
        SPELL_LIGHTNING_BOLT,
        SPELL_BOLT_OF_COLD,
        SPELL_NO_SPELL,
-       SPELL_CALL_IMP,
+       SPELL_NO_SPELL,
        SPELL_SHADOW_CREATURES,
        SPELL_TELEPORT_OTHER
       }
@@ -560,7 +627,7 @@
 
     {  MST_DEEP_ELF_CONJURER_II,
       {
-       SPELL_STICKY_FLAME,
+       SPELL_STICKY_FLAME_RANGE,
        SPELL_ISKENDERUNS_MYSTIC_BLAST,
        SPELL_INVISIBILITY,
        SPELL_STONE_ARROW,
@@ -813,22 +880,55 @@
 
     {  MST_SWAMP_DRAKE,
       {
-       SPELL_MEPHITIC_CLOUD,
-       SPELL_MEPHITIC_CLOUD,
+       SPELL_NOXIOUS_CLOUD,
+       SPELL_NOXIOUS_CLOUD,
        SPELL_NO_SPELL,
-       SPELL_MEPHITIC_CLOUD,
-       SPELL_MEPHITIC_CLOUD,
+       SPELL_NOXIOUS_CLOUD,
+       SPELL_NOXIOUS_CLOUD,
        SPELL_NO_SPELL
       }
     },
 
-    {  MST_SERPENT_OF_HELL,
+    {  MST_SERPENT_OF_HELL_GEHENNA,
       {
        SPELL_HELLFIRE,
-       SPELL_HELLFIRE,
+       SPELL_FIRE_BREATH,
        SPELL_NO_SPELL,
        SPELL_HELLFIRE,
-       SPELL_HELLFIRE,
+       SPELL_FIRE_BREATH,
+       SPELL_NO_SPELL
+      }
+    },
+
+    {  MST_SERPENT_OF_HELL_COCYTUS,
+      {
+       SPELL_FREEZING_CLOUD,
+       SPELL_COLD_BREATH,
+       SPELL_NO_SPELL,
+       SPELL_FREEZING_CLOUD,
+       SPELL_COLD_BREATH,
+       SPELL_NO_SPELL
+      }
+    },
+
+    {  MST_SERPENT_OF_HELL_DIS,
+      {
+       SPELL_METAL_SPLINTERS,
+       SPELL_QUICKSILVER_BOLT,
+       SPELL_NO_SPELL,
+       SPELL_METAL_SPLINTERS,
+       SPELL_QUICKSILVER_BOLT,
+       SPELL_NO_SPELL
+      }
+    },
+
+    {  MST_SERPENT_OF_HELL_TARTARUS,
+      {
+       SPELL_BOLT_OF_DRAINING,
+       SPELL_MIASMA,
+       SPELL_NO_SPELL,
+       SPELL_BOLT_OF_DRAINING,
+       SPELL_MIASMA,
        SPELL_NO_SPELL
       }
     },
@@ -888,7 +988,7 @@
       }
     },
 
-    {  MST_MYSTIC,
+    {  MST_NORRIS,
       {
        SPELL_BRAIN_FEED,
        SPELL_SMITING,
@@ -913,7 +1013,7 @@
     {  MST_DRAC_SCORCHER,
       {
        SPELL_BOLT_OF_FIRE,
-       SPELL_STICKY_FLAME,
+       SPELL_STICKY_FLAME_RANGE,
        SPELL_NO_SPELL,
        SPELL_FIREBALL,
        SPELL_HELLFIRE,
@@ -963,7 +1063,7 @@
        SPELL_NO_SPELL,
        SPELL_FREEZING_CLOUD,
        SPELL_SUMMON_ICE_BEAST,
-       SPELL_SUMMON_ICE_BEAST
+       SPELL_SUMMON_ICE_BEAST // in Zotdef forced off
       }
     },
 
@@ -1108,6 +1208,17 @@
       }
     },
 
+    {  MST_TERPSICHORE,
+      {
+       SPELL_TUKIMAS_BALL,
+       SPELL_SWIFTNESS,
+       SPELL_CORONA,
+       SPELL_BLINK,
+       SPELL_CANTRIP,
+       SPELL_CANTRIP
+      }
+    },
+
     {  MST_KIRKE,
       {
        SPELL_PORKALATOR,
@@ -1146,8 +1257,8 @@
        SPELL_NO_SPELL,
        SPELL_NO_SPELL,
        SPELL_INVISIBILITY,
+       SPELL_SWIFTNESS,
        SPELL_BLINK,
-       SPELL_NO_SPELL,
        SPELL_TELEPORT_SELF,
       }
     },
@@ -1168,9 +1279,9 @@
     {  MST_DISPATER,
       {
        SPELL_SUMMON_GREATER_DEMON,
+       SPELL_LEHUDIBS_CRYSTAL_SPEAR,
+       SPELL_NO_SPELL,
        SPELL_IRON_SHOT,
-       SPELL_SUMMON_DEMON,
-       SPELL_LIGHTNING_BOLT,
        SPELL_HELLFIRE,
        SPELL_SUMMON_GREATER_DEMON
       }
@@ -1180,9 +1291,9 @@
       {
        SPELL_BOLT_OF_FIRE,
        SPELL_HELLFIRE,
-       SPELL_SUMMON_DEMON,
-       SPELL_SUMMON_GREATER_DEMON,
-       SPELL_BOLT_OF_DRAINING,
+       SPELL_NO_SPELL,
+       SPELL_FIRE_SUMMON,
+       SPELL_NO_SPELL,
        SPELL_TELEPORT_SELF
       }
     },
@@ -1211,9 +1322,9 @@
 
     {  MST_MNOLEG,
       {
-       SPELL_SUMMON_GREATER_DEMON,
+       SPELL_SUMMON_EYEBALLS,
        SPELL_SMITING,
-       SPELL_INVISIBILITY,
+       SPELL_NO_SPELL,
        SPELL_SUMMON_HORRIBLE_THINGS,
        SPELL_SUMMON_HORRIBLE_THINGS,
        SPELL_TELEPORT_SELF
@@ -1290,7 +1401,7 @@
       {
        SPELL_VENOM_BOLT,
        SPELL_SLEEP,
-       SPELL_SLEEP,
+       SPELL_NO_SPELL,
        SPELL_POISON_ARROW,
        SPELL_VENOM_BOLT,
        SPELL_BLINK_OTHER
@@ -1308,6 +1419,17 @@
       }
     },
 
+    {  MST_IGNACIO,
+      {
+       SPELL_AGONY,
+       SPELL_PAIN,
+       SPELL_HASTE,
+       SPELL_NO_SPELL,
+       SPELL_NO_SPELL,
+       SPELL_HASTE
+      }
+    },
+
     {  MST_HAROLD,
       {
        SPELL_NO_SPELL,
@@ -1322,8 +1444,8 @@
     {  MST_MARA,
       {
        SPELL_MISLEAD,
-       SPELL_FAKE_MARA_SUMMON,
        SPELL_BOLT_OF_FIRE,
+       SPELL_FAKE_MARA_SUMMON,
        SPELL_SUMMON_ILLUSION,
        SPELL_PAIN,
        SPELL_TELEPORT_SELF
@@ -1333,9 +1455,9 @@
     {  MST_MARA_FAKE,
       {
        SPELL_MISLEAD,
-       SPELL_NO_SPELL,
        SPELL_BOLT_OF_FIRE,
        SPELL_NO_SPELL,
+       SPELL_NO_SPELL,
        SPELL_PAIN,
        SPELL_TELEPORT_SELF
       }
@@ -1363,6 +1485,215 @@
       }
     },
 
+    {  MST_BK_OKAWARU,
+      {
+       SPELL_NO_SPELL,
+       SPELL_NO_SPELL,
+       SPELL_MIGHT,
+       SPELL_HASTE,
+       SPELL_NO_SPELL,
+       SPELL_NO_SPELL
+      }
+    },
+
+    {  MST_SPRIGGAN_DRUID,
+      {
+       SPELL_SUNRAY,
+       SPELL_AWAKEN_FOREST,
+       SPELL_NO_SPELL,
+       SPELL_SUMMON_CANIFORMS,
+       SPELL_NO_SPELL,
+       SPELL_MINOR_HEALING
+      }
+    },
+
+    {  MST_SPRIGGAN_AIR_MAGE,
+      {
+       SPELL_AIRSTRIKE,
+       SPELL_LIGHTNING_BOLT,
+       SPELL_SWIFTNESS,
+       SPELL_SHOCK,
+       SPELL_NO_SPELL,
+       SPELL_HASTE
+      }
+    },
+
+    { MST_THE_ENCHANTRESS,
+      {
+       SPELL_PARALYSE,
+       SPELL_SLEEP,
+       SPELL_HASTE,
+       SPELL_BANISHMENT,
+       SPELL_INVISIBILITY,
+       SPELL_TELEPORT_SELF
+      }
+    },
+
+    {  MST_HELLEPHANT,
+      {
+       SPELL_FIRE_BREATH,
+       SPELL_NO_SPELL,
+       SPELL_BLINK,
+       SPELL_FIRE_BREATH,
+       SPELL_NO_SPELL,
+       SPELL_BLINK
+      }
+    },
+
+    {  MST_GRINDER,
+      {
+       SPELL_PAIN,
+       SPELL_PARALYSE,
+       SPELL_BLINK,
+       SPELL_PAIN,
+       SPELL_PARALYSE,
+       SPELL_BLINK
+      }
+    },
+
+    {  MST_IRON_ELEMENTAL,
+      {
+       SPELL_IRON_SHOT,
+       SPELL_SLOW,
+       SPELL_NO_SPELL,
+       SPELL_NO_SPELL,
+       SPELL_METAL_SPLINTERS,
+       SPELL_NO_SPELL
+      }
+    },
+
+    {  MST_IRON_GIANT,
+      {
+       SPELL_IRON_SHOT,
+       SPELL_NO_SPELL,
+       SPELL_NO_SPELL,
+       SPELL_NO_SPELL,
+       SPELL_IRON_ELEMENTALS,
+       SPELL_NO_SPELL
+      }
+    },
+
+    {  MST_MENNAS,
+      {
+       SPELL_NO_SPELL,
+       SPELL_CONFUSE,
+       SPELL_SILENCE,
+       SPELL_NO_SPELL,
+       SPELL_MINOR_HEALING,
+       SPELL_MINOR_HEALING
+      }
+    },
+
+    {  MST_NERGALLE,
+      {
+       SPELL_BOLT_OF_DRAINING,
+       SPELL_SUMMON_SPECTRAL_ORCS,
+       SPELL_NO_SPELL,
+       SPELL_DISPEL_UNDEAD,
+       SPELL_HASTE_OTHER,
+       SPELL_TELEPORT_SELF
+      }
+    },
+
+    {  MST_JUMPING_SPIDER,
+      {
+       SPELL_BLINK_CLOSE,
+       SPELL_BLINK_CLOSE,
+       SPELL_BLINK_CLOSE,
+       SPELL_BLINK_CLOSE,
+       SPELL_BLINK_CLOSE,
+       SPELL_BLINK_AWAY
+      }
+    },
+
+    {  MST_PEARL_DRAGON,
+      {
+       SPELL_HOLY_BREATH,
+       SPELL_HOLY_BREATH,
+       SPELL_NO_SPELL,
+       SPELL_HOLY_BREATH,
+       SPELL_HOLY_BREATH,
+       SPELL_NO_SPELL
+      }
+    },
+
+    {  MST_BLESSED_TOE,
+      {
+       SPELL_SUMMON_HOLIES,
+       SPELL_SUMMON_HOLIES,
+       SPELL_HEAL_OTHER,
+       SPELL_HOLY_WORD,
+       SPELL_SUMMON_HOLIES,
+       SPELL_NO_SPELL
+      }
+    },
+
+    {  MST_SPIRIT,
+      {
+       SPELL_BLINK_AWAY,
+       SPELL_BLINK_AWAY,
+       SPELL_BLINK_AWAY,
+       SPELL_BLINK_AWAY,
+       SPELL_BLINK_AWAY,
+       SPELL_BLINK_AWAY
+      }
+    },
+
+    {  MST_OPHAN,
+      {
+       SPELL_BOLT_OF_FIRE,
+       SPELL_HOLY_FLAMES,
+       SPELL_NO_SPELL,
+       SPELL_BOLT_OF_FIRE,
+       SPELL_HOLY_FLAMES,
+       SPELL_NO_SPELL
+      }
+    },
+
+    {  MST_BOG_MUMMY,
+      {
+       SPELL_THROW_FROST,
+       SPELL_SLOW,
+       SPELL_NO_SPELL,
+       SPELL_CORPSE_ROT,
+       SPELL_CORPSE_ROT,
+       SPELL_NO_SPELL
+      }
+    },
+
+    {  MST_JORY,
+      {
+       SPELL_LEHUDIBS_CRYSTAL_SPEAR,
+       SPELL_MESMERISE,
+       SPELL_BLINK,
+       SPELL_BRAIN_FEED,
+       SPELL_PAIN,
+       SPELL_VAMPIRIC_DRAINING,
+      }
+    },
+
+    {  MST_SHEDU,
+      {
+       SPELL_HEAL_OTHER,
+       SPELL_NO_SPELL,
+       SPELL_HEAL_OTHER,
+       SPELL_NO_SPELL,
+       SPELL_NO_SPELL,
+       SPELL_NO_SPELL,
+      }
+    },
+
+    {  MST_EIDOLON,
+      {
+       SPELL_PAIN,
+       SPELL_CAUSE_FEAR,
+       SPELL_NO_SPELL,
+       SPELL_CAUSE_FEAR,
+       SPELL_NO_SPELL,
+       SPELL_NO_SPELL,
+      }
+    },
+
     {  MST_TEST_SPAWNER,
       {
        SPELL_SHADOW_CREATURES,
