@@ -1,15 +1,14 @@
-/*
- * File:     ng-restr.cc
- * Summary:  Character choice restrictions.
+/**
+ * @file
+ * @brief Character choice restrictions.
  *
  * The functions in this file are "pure": They don't
  * access any global data.
- */
+**/
 
 #include "AppHdr.h"
 
 #include "ng-restr.h"
-#include "newgame.h"
 #include "species.h"
 #include "jobs.h"
 
@@ -22,14 +21,14 @@ char_choice_restriction job_allowed(species_type speci, job_type job)
         {
             case SP_DEEP_ELF:
             case SP_SLUDGE_ELF:
-            case SP_DEEP_DWARF:
             case SP_MERFOLK:
             case SP_HALFLING:
             case SP_KOBOLD:
             case SP_SPRIGGAN:
             case SP_NAGA:
             case SP_OGRE:
-            case SP_RED_DRACONIAN:
+            case SP_BASE_DRACONIAN:
+            case SP_DEMIGOD:
             case SP_MUMMY:
             case SP_VAMPIRE:
                 return (CC_RESTRICTED);
@@ -40,18 +39,19 @@ char_choice_restriction job_allowed(species_type speci, job_type job)
         case JOB_GLADIATOR:
             switch (speci)
         {
+            case SP_CAT:
+                return (CC_BANNED);
             case SP_DEEP_ELF:
-            case SP_DEEP_DWARF:
-            case SP_HILL_ORC:
+            case SP_SLUDGE_ELF:
             case SP_HALFLING:
             case SP_KOBOLD:
             case SP_SPRIGGAN:
             case SP_NAGA:
-            case SP_CENTAUR:
-            case SP_TROLL:
-            case SP_RED_DRACONIAN:
-            case SP_GHOUL:
+            case SP_OGRE:
+            case SP_BASE_DRACONIAN:
+            case SP_DEMIGOD:
             case SP_MUMMY:
+            case SP_VAMPIRE:
                 return (CC_RESTRICTED);
             default:
                 return (CC_UNRESTRICTED);
@@ -60,19 +60,19 @@ char_choice_restriction job_allowed(species_type speci, job_type job)
         case JOB_MONK:
             switch (speci)
         {
-            case SP_HUMAN:
+            case SP_CAT:
+                return (CC_BANNED);
             case SP_HIGH_ELF:
             case SP_DEEP_ELF:
             case SP_MOUNTAIN_DWARF:
             case SP_DEEP_DWARF:
-            case SP_HILL_ORC:
+            case SP_MERFOLK:
             case SP_HALFLING:
             case SP_KOBOLD:
             case SP_SPRIGGAN:
             case SP_NAGA:
-            case SP_RED_DRACONIAN:
+            case SP_OGRE:
             case SP_DEMIGOD:
-            case SP_DEMONSPAWN:
             case SP_MUMMY:
                 return (CC_RESTRICTED);
             default:
@@ -87,13 +87,7 @@ char_choice_restriction job_allowed(species_type speci, job_type job)
             case SP_HIGH_ELF:
             case SP_DEEP_ELF:
             case SP_SLUDGE_ELF:
-            case SP_MERFOLK:
-            case SP_HALFLING:
-            case SP_KOBOLD:
             case SP_SPRIGGAN:
-            case SP_NAGA:
-            case SP_CENTAUR:
-            case SP_RED_DRACONIAN:
             case SP_MUMMY:
             case SP_GHOUL:
             case SP_VAMPIRE:
@@ -102,71 +96,86 @@ char_choice_restriction job_allowed(species_type speci, job_type job)
                 return (CC_UNRESTRICTED);
         }
 
-        case JOB_PALADIN:
-            switch (speci)
-        {
-            case SP_DEMIGOD:
-            case SP_DEMONSPAWN:
-            case SP_MUMMY:
-            case SP_GHOUL:
-            case SP_VAMPIRE:
-                return (CC_BANNED);
-            case SP_DEEP_ELF:
-            case SP_SLUDGE_ELF:
-            case SP_HALFLING:
-            case SP_KOBOLD:
-            case SP_SPRIGGAN:
-            case SP_NAGA:
-            case SP_OGRE:
-            case SP_TROLL:
-            case SP_KENKU:
-                return (CC_RESTRICTED);
-            default:
-                return (CC_UNRESTRICTED);
-        }
-
-        case JOB_PRIEST:
-            switch (speci)
-        {
-            case SP_DEMIGOD:
-                return (CC_BANNED);
-            case SP_DEEP_ELF:
-            case SP_DEEP_DWARF:
-            case SP_MERFOLK:
-            case SP_HALFLING:
-            case SP_SPRIGGAN:
-            case SP_NAGA:
-            case SP_OGRE:
-            case SP_TROLL:
-            case SP_KENKU:
-            case SP_GHOUL:
-            case SP_VAMPIRE:
-                return (CC_RESTRICTED);
-            default:
-                return (CC_UNRESTRICTED);
-        }
-
         case JOB_CHAOS_KNIGHT:
             switch (speci)
         {
             case SP_DEMIGOD:
                 return (CC_BANNED);
-            case SP_DEEP_ELF:
-            case SP_SPRIGGAN:
-            case SP_NAGA:
-            case SP_KENKU:
-                return (CC_RESTRICTED);
             default:
-                return (CC_UNRESTRICTED);
+                return (CC_RESTRICTED);
         }
 
         case JOB_DEATH_KNIGHT:
             switch (speci)
         {
+            case SP_DEMIGOD:
+                return (CC_BANNED);
+            case SP_HIGH_ELF:
+            case SP_DEEP_ELF:
+            case SP_SLUDGE_ELF:
+            case SP_DEEP_DWARF:
+            case SP_HALFLING:
+            case SP_KOBOLD:
+            case SP_SPRIGGAN:
+            case SP_OGRE:
+            case SP_MUMMY:
+            case SP_GHOUL:
+            case SP_VAMPIRE:
+            case SP_CAT:
+                return (CC_RESTRICTED);
+            default:
+                return (CC_UNRESTRICTED);
+        }
+
+        case JOB_ABYSSAL_KNIGHT:
+            switch (speci)
+        {
+            case SP_DEMIGOD:
+                return (CC_BANNED);
+            case SP_HIGH_ELF:
+            case SP_DEEP_ELF:
+            case SP_SLUDGE_ELF:
+            case SP_MOUNTAIN_DWARF:
+            case SP_DEEP_DWARF:
+            case SP_HILL_ORC:
+            case SP_HALFLING:
+            case SP_KOBOLD:
+            case SP_OGRE:
+            case SP_TROLL:
+            case SP_MINOTAUR:
+            case SP_KENKU:
+            case SP_MUMMY:
+            case SP_GHOUL:
+            case SP_VAMPIRE:
+            case SP_CAT:
+                return (CC_RESTRICTED);
+            default:
+                return (CC_UNRESTRICTED);
+        }
+        case JOB_PRIEST:
+            switch (speci)
+        {
+            case SP_DEMIGOD:
+            case SP_DEMONSPAWN:
+            case SP_MUMMY:
+            case SP_GHOUL:
+            case SP_VAMPIRE:
+                return (CC_BANNED);
+            case SP_DEEP_ELF:
+            case SP_SLUDGE_ELF:
+            case SP_MOUNTAIN_DWARF:
+            case SP_DEEP_DWARF:
+            case SP_MERFOLK:
+            case SP_HALFLING:
+            case SP_KOBOLD:
             case SP_SPRIGGAN:
             case SP_NAGA:
+            case SP_CENTAUR:
+            case SP_OGRE:
+            case SP_TROLL:
             case SP_KENKU:
-            case SP_GHOUL:
+            case SP_BASE_DRACONIAN:
+            case SP_CAT:
                 return (CC_RESTRICTED);
             default:
                 return (CC_UNRESTRICTED);
@@ -181,10 +190,14 @@ char_choice_restriction job_allowed(species_type speci, job_type job)
             case SP_GHOUL:
             case SP_VAMPIRE:
                 return (CC_BANNED);
+            case SP_HIGH_ELF:
             case SP_DEEP_ELF:
             case SP_HALFLING:
             case SP_SPRIGGAN:
-            case SP_NAGA:
+            case SP_OGRE:
+            case SP_MINOTAUR:
+            case SP_KENKU:
+            case SP_CAT:
                 return (CC_RESTRICTED);
             default:
                 return (CC_UNRESTRICTED);
@@ -197,9 +210,8 @@ char_choice_restriction job_allowed(species_type speci, job_type job)
             case SP_SLUDGE_ELF:
             case SP_MOUNTAIN_DWARF:
             case SP_DEEP_DWARF:
-            case SP_HILL_ORC:
+            case SP_KOBOLD:
             case SP_SPRIGGAN:
-            case SP_NAGA:
             case SP_CENTAUR:
             case SP_OGRE:
             case SP_TROLL:
@@ -207,30 +219,27 @@ char_choice_restriction job_allowed(species_type speci, job_type job)
             case SP_KENKU:
             case SP_MUMMY:
             case SP_GHOUL:
-            case SP_VAMPIRE:
                 return (CC_RESTRICTED);
             default:
                 return (CC_UNRESTRICTED);
         }
 
-        case JOB_REAVER:
+        case JOB_WARPER:
             switch (speci)
         {
-            case SP_HUMAN:
+            case SP_DEEP_ELF:
             case SP_SLUDGE_ELF:
-            case SP_DEEP_DWARF:
+            case SP_MOUNTAIN_DWARF:
             case SP_HILL_ORC:
-            case SP_HALFLING:
+            case SP_MERFOLK:
             case SP_KOBOLD:
-            case SP_SPRIGGAN:
-            case SP_NAGA:
             case SP_CENTAUR:
             case SP_OGRE:
             case SP_TROLL:
             case SP_MINOTAUR:
             case SP_KENKU:
-            case SP_RED_DRACONIAN:
-            case SP_DEMONSPAWN:
+            case SP_BASE_DRACONIAN:
+            case SP_DEMIGOD:
             case SP_MUMMY:
             case SP_GHOUL:
             case SP_VAMPIRE:
@@ -248,12 +257,11 @@ char_choice_restriction job_allowed(species_type speci, job_type job)
             case SP_MERFOLK:
             case SP_HALFLING:
             case SP_KOBOLD:
-            case SP_SPRIGGAN:
             case SP_CENTAUR:
-            case SP_TROLL:
-            case SP_OGRE:
             case SP_MINOTAUR:
             case SP_GHOUL:
+            case SP_VAMPIRE:
+            case SP_CAT:
                 return (CC_RESTRICTED);
             default:
                 return (CC_UNRESTRICTED);
@@ -262,16 +270,20 @@ char_choice_restriction job_allowed(species_type speci, job_type job)
         case JOB_CONJURER:
             switch (speci)
         {
+            case SP_SLUDGE_ELF:
+            case SP_MOUNTAIN_DWARF:
             case SP_DEEP_DWARF:
-            case SP_HILL_ORC:
             case SP_MERFOLK:
             case SP_HALFLING:
+            case SP_KOBOLD:
             case SP_SPRIGGAN:
             case SP_CENTAUR:
+            case SP_OGRE:
             case SP_TROLL:
             case SP_MINOTAUR:
             case SP_GHOUL:
             case SP_VAMPIRE:
+            case SP_CAT:
                 return (CC_RESTRICTED);
             default:
                 return (CC_UNRESTRICTED);
@@ -280,20 +292,22 @@ char_choice_restriction job_allowed(species_type speci, job_type job)
         case JOB_ENCHANTER:
             switch (speci)
         {
+            case SP_HIGH_ELF:
             case SP_SLUDGE_ELF:
             case SP_MOUNTAIN_DWARF:
             case SP_DEEP_DWARF:
             case SP_HILL_ORC:
-            case SP_NAGA:
             case SP_CENTAUR:
             case SP_OGRE:
             case SP_TROLL:
             case SP_MINOTAUR:
             case SP_KENKU:
-            case SP_RED_DRACONIAN:
+            case SP_BASE_DRACONIAN:
+            case SP_DEMIGOD:
             case SP_DEMONSPAWN:
-            case SP_GHOUL:
             case SP_MUMMY:
+            case SP_GHOUL:
+            case SP_CAT:
                 return (CC_RESTRICTED);
             default:
                 return (CC_UNRESTRICTED);
@@ -302,7 +316,6 @@ char_choice_restriction job_allowed(species_type speci, job_type job)
         case JOB_SUMMONER:
             switch (speci)
         {
-            case SP_HUMAN:
             case SP_HIGH_ELF:
             case SP_MOUNTAIN_DWARF:
             case SP_DEEP_DWARF:
@@ -310,12 +323,11 @@ char_choice_restriction job_allowed(species_type speci, job_type job)
             case SP_MERFOLK:
             case SP_HALFLING:
             case SP_SPRIGGAN:
+            case SP_NAGA:
             case SP_CENTAUR:
+            case SP_OGRE:
             case SP_TROLL:
             case SP_MINOTAUR:
-            case SP_RED_DRACONIAN:
-            case SP_DEMIGOD:
-            case SP_DEMONSPAWN:
             case SP_GHOUL:
                 return (CC_RESTRICTED);
             default:
@@ -329,65 +341,10 @@ char_choice_restriction job_allowed(species_type speci, job_type job)
             case SP_MOUNTAIN_DWARF:
             case SP_MERFOLK:
             case SP_HALFLING:
-            case SP_SPRIGGAN:
-            case SP_CENTAUR:
-            case SP_TROLL:
-            case SP_MINOTAUR:
-            case SP_GHOUL:
-                return (CC_RESTRICTED);
-            default:
-                return (CC_UNRESTRICTED);
-        }
-
-        case JOB_WARPER:
-            switch (speci)
-        {
-            case SP_HUMAN:
-            case SP_DEEP_ELF:
-            case SP_SLUDGE_ELF:
-            case SP_MOUNTAIN_DWARF:
-            case SP_DEEP_DWARF:
-            case SP_HILL_ORC:
-            case SP_MERFOLK:
-            case SP_HALFLING:
-            case SP_KOBOLD:
             case SP_CENTAUR:
             case SP_OGRE:
             case SP_TROLL:
             case SP_MINOTAUR:
-            case SP_KENKU:
-            case SP_RED_DRACONIAN:
-            case SP_DEMONSPAWN:
-            case SP_MUMMY:
-            case SP_GHOUL:
-            case SP_VAMPIRE:
-                return (CC_RESTRICTED);
-            default:
-                return (CC_UNRESTRICTED);
-        }
-
-        case JOB_ARCANE_MARKSMAN:
-            switch (speci)
-        {
-            case SP_HUMAN:
-            case SP_DEEP_ELF:
-            case SP_SLUDGE_ELF:
-            case SP_MOUNTAIN_DWARF:
-            case SP_DEEP_DWARF:
-            case SP_HILL_ORC:
-            case SP_MERFOLK:
-            case SP_HALFLING:
-            case SP_KOBOLD:
-            case SP_CENTAUR:
-            case SP_MINOTAUR:
-            case SP_KENKU:
-            case SP_RED_DRACONIAN:
-            case SP_DEMONSPAWN:
-            case SP_MUMMY:
-            case SP_GHOUL:
-            case SP_OGRE:
-            case SP_TROLL:
-            case SP_VAMPIRE:
                 return (CC_RESTRICTED);
             default:
                 return (CC_UNRESTRICTED);
@@ -396,7 +353,6 @@ char_choice_restriction job_allowed(species_type speci, job_type job)
         case JOB_TRANSMUTER:
             switch (speci)
         {
-            case SP_HUMAN:
             case SP_HIGH_ELF:
             case SP_DEEP_ELF:
             case SP_MOUNTAIN_DWARF:
@@ -404,15 +360,12 @@ char_choice_restriction job_allowed(species_type speci, job_type job)
             case SP_HILL_ORC:
             case SP_HALFLING:
             case SP_KOBOLD:
-            case SP_SPRIGGAN:
-            case SP_CENTAUR:
+            case SP_OGRE:
             case SP_TROLL:
             case SP_MINOTAUR:
             case SP_KENKU:
-            case SP_DEMONSPAWN:
             case SP_MUMMY:
             case SP_GHOUL:
-            case SP_VAMPIRE:
                 return (CC_RESTRICTED);
             default:
                 return (CC_UNRESTRICTED);
@@ -426,10 +379,13 @@ char_choice_restriction job_allowed(species_type speci, job_type job)
             case SP_HALFLING:
             case SP_SPRIGGAN:
             case SP_CENTAUR:
+            case SP_OGRE:
             case SP_TROLL:
             case SP_MINOTAUR:
+            case SP_MUMMY:
             case SP_GHOUL:
             case SP_VAMPIRE:
+            case SP_CAT:
                 return (CC_RESTRICTED);
             default:
                 return (CC_UNRESTRICTED);
@@ -440,14 +396,13 @@ char_choice_restriction job_allowed(species_type speci, job_type job)
         {
             case SP_MOUNTAIN_DWARF:
             case SP_DEEP_DWARF:
-            case SP_HILL_ORC:
             case SP_HALFLING:
             case SP_SPRIGGAN:
             case SP_CENTAUR:
+            case SP_OGRE:
             case SP_TROLL:
             case SP_MINOTAUR:
-            case SP_GHOUL:
-            case SP_VAMPIRE:
+            case SP_KENKU:
                 return (CC_RESTRICTED);
             default:
                 return (CC_UNRESTRICTED);
@@ -461,11 +416,13 @@ char_choice_restriction job_allowed(species_type speci, job_type job)
             case SP_HILL_ORC:
             case SP_MERFOLK:
             case SP_CENTAUR:
+            case SP_OGRE:
             case SP_TROLL:
             case SP_MINOTAUR:
             case SP_MUMMY:
             case SP_GHOUL:
             case SP_VAMPIRE:
+            case SP_CAT:
                 return (CC_RESTRICTED);
             default:
                 return (CC_UNRESTRICTED);
@@ -475,19 +432,16 @@ char_choice_restriction job_allowed(species_type speci, job_type job)
             switch (speci)
         {
             case SP_HIGH_ELF:
-            case SP_DEEP_DWARF:
-            case SP_HILL_ORC:
             case SP_MERFOLK:
             case SP_HALFLING:
-            case SP_SPRIGGAN:
-            case SP_NAGA:
             case SP_CENTAUR:
+            case SP_OGRE:
             case SP_TROLL:
             case SP_MINOTAUR:
             case SP_KENKU:
-            case SP_RED_DRACONIAN:
-            case SP_GHOUL:
+            case SP_MUMMY:
             case SP_VAMPIRE:
+            case SP_CAT:
                 return (CC_RESTRICTED);
             default:
                 return (CC_UNRESTRICTED);
@@ -502,8 +456,10 @@ char_choice_restriction job_allowed(species_type speci, job_type job)
             case SP_HILL_ORC:
             case SP_HALFLING:
             case SP_CENTAUR:
+            case SP_OGRE:
             case SP_TROLL:
             case SP_MINOTAUR:
+            case SP_MUMMY:
             case SP_GHOUL:
             case SP_VAMPIRE:
                 return (CC_RESTRICTED);
@@ -515,28 +471,33 @@ char_choice_restriction job_allowed(species_type speci, job_type job)
             switch (speci)
         {
             case SP_HIGH_ELF:
+            case SP_DEEP_ELF:
             case SP_MOUNTAIN_DWARF:
             case SP_DEEP_DWARF:
             case SP_HILL_ORC:
+            case SP_MERFOLK:
+            case SP_HALFLING:
             case SP_CENTAUR:
             case SP_OGRE:
             case SP_TROLL:
             case SP_MINOTAUR:
-            case SP_RED_DRACONIAN:
+            case SP_KENKU:
+            case SP_BASE_DRACONIAN:
+            case SP_DEMIGOD:
+            case SP_DEMONSPAWN:
             case SP_MUMMY:
             case SP_GHOUL:
-            case SP_VAMPIRE:
+            case SP_CAT:
                 return (CC_RESTRICTED);
             default:
                 return (CC_UNRESTRICTED);
         }
 
-        case JOB_THIEF:
-            return (CC_RESTRICTED);
-
         case JOB_ASSASSIN:
             switch (speci)
         {
+            case SP_CAT:
+                return (CC_BANNED);
             case SP_HIGH_ELF:
             case SP_DEEP_ELF:
             case SP_MOUNTAIN_DWARF:
@@ -547,7 +508,11 @@ char_choice_restriction job_allowed(species_type speci, job_type job)
             case SP_OGRE:
             case SP_TROLL:
             case SP_MINOTAUR:
-            case SP_RED_DRACONIAN:
+            case SP_BASE_DRACONIAN:
+            case SP_DEMIGOD:
+            case SP_DEMONSPAWN:
+            case SP_MUMMY:
+            case SP_GHOUL:
                 return (CC_RESTRICTED);
             default:
                 return (CC_UNRESTRICTED);
@@ -556,10 +521,16 @@ char_choice_restriction job_allowed(species_type speci, job_type job)
         case JOB_HUNTER:
             switch (speci)
         {
-            case SP_DEEP_DWARF:
-            case SP_KOBOLD:
+            case SP_CAT:
+                return (CC_BANNED);
+            case SP_DEEP_ELF:
+            case SP_HILL_ORC:
+            case SP_MERFOLK:
+            case SP_SPRIGGAN:
             case SP_NAGA:
-            case SP_RED_DRACONIAN:
+            case SP_KENKU:
+            case SP_BASE_DRACONIAN:
+            case SP_DEMIGOD:
             case SP_MUMMY:
             case SP_GHOUL:
             case SP_VAMPIRE:
@@ -571,17 +542,30 @@ char_choice_restriction job_allowed(species_type speci, job_type job)
         case JOB_ARTIFICER:
             switch (speci)
         {
+            case SP_CAT:
+                return (CC_BANNED);
+            case SP_HIGH_ELF:
+            case SP_DEEP_ELF:
+            case SP_SLUDGE_ELF:
+            case SP_NAGA:
             case SP_CENTAUR:
+            case SP_OGRE:
             case SP_TROLL:
             case SP_MINOTAUR:
-            case SP_OGRE:
+            case SP_KENKU:
+            case SP_DEMIGOD:
             case SP_MUMMY:
             case SP_GHOUL:
+            case SP_VAMPIRE:
                 return (CC_RESTRICTED);
             default:
                 return (CC_UNRESTRICTED);
         }
 
+// XXX: Arcane Marksmen are temporarily disabled
+        case JOB_ARCANE_MARKSMAN:
+            return (CC_BANNED);
+
         case JOB_WANDERER:
             return (CC_RESTRICTED);
 
@@ -608,76 +592,34 @@ char_choice_restriction book_restriction(startup_book_type booktype,
     ASSERT(is_valid_species(ng.species));
     switch (booktype)
     {
-        case SBT_FIRE: // Fire
+        case SBT_FIRE: // Fire/Earth book
             switch (ng.species)
         {
-            case SP_HUMAN:
             case SP_HIGH_ELF:
-            case SP_DEEP_ELF:
-            case SP_SLUDGE_ELF:
-            case SP_MOUNTAIN_DWARF:
-            case SP_HILL_ORC:
-            case SP_KOBOLD:
-            case SP_NAGA:
-            case SP_OGRE:
-            case SP_KENKU:
-            case SP_DEMIGOD:
-            case SP_DEMONSPAWN:
-            case SP_MUMMY:
-                return (CC_UNRESTRICTED);
-
-            default:
-                if (species_genus(ng.species) == GENPC_DRACONIAN)
-                    return (CC_UNRESTRICTED);
-                return (CC_RESTRICTED);
-        }
-            break;
-
-        case SBT_COLD: // Ice
-            switch (ng.species)
-        {
-            case SP_HUMAN:
-            case SP_HIGH_ELF:
-            case SP_DEEP_ELF:
-            case SP_SLUDGE_ELF:
-            case SP_HILL_ORC:
             case SP_MERFOLK:
-            case SP_KOBOLD:
-            case SP_NAGA:
-            case SP_OGRE:
+            case SP_HALFLING:
+            case SP_VAMPIRE:
             case SP_KENKU:
-            case SP_DEMIGOD:
-            case SP_DEMONSPAWN:
-            case SP_MUMMY:
-                return (CC_UNRESTRICTED);
-
-            default:
-                if (species_genus(ng.species) == GENPC_DRACONIAN)
-                    return (CC_UNRESTRICTED);
                 return (CC_RESTRICTED);
+            default:
+                return (CC_UNRESTRICTED);
         }
             break;
 
-        case SBT_SUMM: // Summoning
+        case SBT_COLD: // Ice/Air book
             switch (ng.species)
         {
-            case SP_DEEP_ELF:
-            case SP_SLUDGE_ELF:
-            case SP_KOBOLD:
-            case SP_NAGA:
-            case SP_OGRE:
-            case SP_KENKU:
-            case SP_MUMMY:
-            case SP_VAMPIRE:
-                return (CC_UNRESTRICTED);
-
-            default:
+            case SP_MOUNTAIN_DWARF:
+            case SP_DEEP_DWARF:
+            case SP_HILL_ORC:
                 return (CC_RESTRICTED);
+            default:
+                return (CC_UNRESTRICTED);
         }
             break;
 
         default:
-            return (CC_RESTRICTED);
+            return (CC_UNRESTRICTED);
     }
 }
 
@@ -691,11 +633,20 @@ char_choice_restriction weapon_restriction(weapon_type wpn,
     switch (wpn)
     {
         case WPN_UNARMED:
-            if (species_has_claws(ng.species))
+            if (!species_has_claws(ng.species) && ng.job != JOB_MONK)
+                return (CC_BANNED);
+            switch (ng.species)
+            {
+            case SP_DEEP_ELF:
+            case SP_HIGH_ELF:
+                return (CC_RESTRICTED);
+            default:
                 return (CC_UNRESTRICTED);
-            return (CC_BANNED);
+            }
 
         case WPN_SHORT_SWORD:
+            if (ng.job == JOB_MONK)
+                return (CC_BANNED);
             switch (ng.species)
             {
             case SP_NAGA:
@@ -706,8 +657,8 @@ char_choice_restriction weapon_restriction(weapon_type wpn,
                 // else fall through
             case SP_HIGH_ELF:
             case SP_DEEP_ELF:
-                // Sludge elves have bad aptitudes with short swords (110) but are
-                // still better with them than any other starting weapon.
+                // Sludge elves have bad aptitudes with short swords (-1), but
+                // are still better with them than any other starting weapon.
             case SP_SLUDGE_ELF:
             case SP_HALFLING:
             case SP_KOBOLD:
@@ -720,12 +671,16 @@ char_choice_restriction weapon_restriction(weapon_type wpn,
 
             // Maces and hand axes usually share the same restrictions.
         case WPN_MACE:
+            if (ng.job == JOB_MONK)
+                return (CC_BANNED);
             if (ng.species == SP_TROLL)
                 return (CC_UNRESTRICTED);
             if (ng.species == SP_VAMPIRE)
                 return (CC_RESTRICTED);
             // else fall-through
         case WPN_HAND_AXE:
+            if (ng.job == JOB_MONK)
+                return (CC_BANNED);
             switch (ng.species)
             {
             case SP_HUMAN:
@@ -748,12 +703,15 @@ char_choice_restriction weapon_restriction(weapon_type wpn,
             }
 
         case WPN_SPEAR:
+            if (ng.job == JOB_MONK)
+                return (CC_BANNED);
             switch (ng.species)
             {
             case SP_HUMAN:
             case SP_HILL_ORC:
             case SP_MERFOLK:
             case SP_NAGA:
+            case SP_OGRE:
             case SP_CENTAUR:
             case SP_MINOTAUR:
             case SP_KENKU:
@@ -766,11 +724,36 @@ char_choice_restriction weapon_restriction(weapon_type wpn,
                 return (species_genus(ng.species) == GENPC_DRACONIAN ? CC_UNRESTRICTED
                         : CC_RESTRICTED);
             }
+        case WPN_FALCHION:
+            if (ng.job != JOB_FIGHTER && ng.job != JOB_GLADIATOR)
+                return (CC_BANNED);
+            switch (ng.species)
+            {
+            case SP_HUMAN:
+            case SP_HILL_ORC:
+            case SP_MERFOLK:
+            case SP_NAGA:
+            case SP_CENTAUR:
+            case SP_MINOTAUR:
+            case SP_HIGH_ELF:
+            case SP_SLUDGE_ELF:
+            case SP_MOUNTAIN_DWARF:
+            case SP_DEEP_DWARF:
+            case SP_KENKU:
+            case SP_DEMIGOD:
+            case SP_DEMONSPAWN:
+            case SP_MUMMY:
+            case SP_VAMPIRE:
+                return (CC_UNRESTRICTED);
+
+            default:
+                return (species_genus(ng.species) == GENPC_DRACONIAN ? CC_UNRESTRICTED
+                        : CC_RESTRICTED);
+            }
 
         case WPN_TRIDENT:
-            if (ng.species != SP_MERFOLK
-                && (ng.job != JOB_GLADIATOR
-                    || species_size(ng.species, PSIZE_BODY) < SIZE_MEDIUM))
+            if (ng.job != JOB_FIGHTER && ng.job != JOB_GLADIATOR
+                || species_size(ng.species, PSIZE_BODY) < SIZE_MEDIUM)
             {
                 return (CC_BANNED);
             }
@@ -780,7 +763,7 @@ char_choice_restriction weapon_restriction(weapon_type wpn,
             switch (ng.species)
             {
             case SP_MOUNTAIN_DWARF:
-            case SP_OGRE:
+            case SP_DEEP_DWARF:
             case SP_GHOUL:
             case SP_VAMPIRE:
                 return (CC_UNRESTRICTED);
@@ -791,8 +774,24 @@ char_choice_restriction weapon_restriction(weapon_type wpn,
             // Both are polearms, right?
             return (weapon_restriction(WPN_SPEAR, ng));
 
+        case WPN_QUARTERSTAFF:
+            if (ng.job != JOB_MONK)
+                return (CC_BANNED);
+            switch (ng.species)
+            {
+            case SP_HALFLING:
+            case SP_MERFOLK:
+            case SP_SPRIGGAN:
+            case SP_TROLL:
+            case SP_VAMPIRE:
+                return (CC_RESTRICTED);
+            default:
+                return (CC_UNRESTRICTED);
+            }
+
         case WPN_ANKUS:
-            if (species_genus(ng.species) == GENPC_OGRE)
+            if (species_genus(ng.species) == GENPC_OGREISH
+                && ng.job != JOB_MONK)
                 return (CC_UNRESTRICTED);
             // intentional fall-through
         default:
@@ -808,148 +807,24 @@ char_choice_restriction religion_restriction(god_type god,
     ASSERT(is_valid_species(ng.species));
     ASSERT(is_valid_job(ng.job));
 
-    // Sanity check.
     if (ng.species == SP_DEMIGOD)
         return (CC_BANNED);
+    if (ng.job != JOB_PRIEST)
+        return (CC_BANNED);
 
     switch (god)
     {
         case GOD_BEOGH:
-            if (ng.species == SP_HILL_ORC)
+            if (ng.job == JOB_PRIEST && ng.species == SP_HILL_ORC)
                 return (CC_UNRESTRICTED);
             return (CC_BANNED);
 
         case GOD_ZIN:
-            switch (ng.species)
-        {
-            case SP_DEMONSPAWN:
-            case SP_MUMMY:
-            case SP_GHOUL:
-            case SP_VAMPIRE:
-                return (CC_BANNED);
-            case SP_SLUDGE_ELF:
-            case SP_MOUNTAIN_DWARF:
-            case SP_SPRIGGAN:
-            case SP_CENTAUR:
-            case SP_MINOTAUR:
-            case SP_OGRE:
-                return (CC_UNRESTRICTED);
-            default:
+            if (ng.job == JOB_PRIEST)
                 return (CC_RESTRICTED);
-        }
-
-        case GOD_YREDELEMNUL:
-            switch (ng.species)
-        {
-            case SP_HILL_ORC:
-                // Restrict in favour of Beogh, else unrestricted.
-                if (ng.job == JOB_PRIEST)
-                    return (CC_RESTRICTED);
-                return (CC_UNRESTRICTED);
-
-            case SP_DEEP_ELF:
-            case SP_KENKU:
-                // Unrestrict these only for Priests as Zin is worse, but
-                // Necromancy (DK) the better choice.
-                if (ng.job == JOB_PRIEST)
-                    return (CC_UNRESTRICTED);
-                return (CC_RESTRICTED);
-
-            case SP_HUMAN:
-            case SP_HIGH_ELF:
-            case SP_SLUDGE_ELF:
-            case SP_MOUNTAIN_DWARF:
-            case SP_DEEP_DWARF:
-            case SP_MERFOLK:
-            case SP_HALFLING:
-            case SP_KOBOLD:
-            case SP_SPRIGGAN:
-            case SP_CENTAUR:
-            case SP_OGRE:
-            case SP_TROLL:
-            case SP_MINOTAUR:
-            case SP_DEMONSPAWN:
-            case SP_MUMMY:
-            case SP_GHOUL:
-            case SP_VAMPIRE:
-                return (CC_UNRESTRICTED);
-            default:
-                if (species_genus(ng.species) == GENPC_DRACONIAN)
-                    return (CC_UNRESTRICTED);
-                return (CC_RESTRICTED);
-        }
-
-        case GOD_XOM:
-            switch (ng.species)
-        {
-            case SP_MOUNTAIN_DWARF:
-            case SP_HILL_ORC:
-            case SP_MERFOLK:
-            case SP_CENTAUR:
-            case SP_OGRE:
-            case SP_TROLL:
-            case SP_MINOTAUR:
-            case SP_KENKU:
-            case SP_DEMONSPAWN:
-                return (CC_UNRESTRICTED);
-            default:
-                if (species_genus(ng.species) == GENPC_DRACONIAN)
-                    return (CC_UNRESTRICTED);
-                return (CC_RESTRICTED);
-        }
-
-        case GOD_MAKHLEB:
-            switch (ng.species)
-        {
-            case SP_HUMAN:
-            case SP_HIGH_ELF:
-            case SP_DEEP_ELF:
-            case SP_SLUDGE_ELF:
-            case SP_MOUNTAIN_DWARF:
-            case SP_DEEP_DWARF:
-            case SP_HILL_ORC:
-            case SP_MERFOLK:
-            case SP_HALFLING:
-            case SP_KOBOLD:
-            case SP_NAGA:
-            case SP_CENTAUR:
-            case SP_OGRE:
-            case SP_TROLL:
-            case SP_MINOTAUR:
-            case SP_DEMONSPAWN:
-            case SP_MUMMY:
-            case SP_GHOUL:
-            case SP_VAMPIRE:
-                return (CC_UNRESTRICTED);
-            default:
-                if (species_genus(ng.species) == GENPC_DRACONIAN)
-                    return (CC_UNRESTRICTED);
-                return (CC_RESTRICTED);
-        }
-
-        case GOD_LUGONU:
-            switch (ng.species)
-        {
-            case SP_HUMAN:
-            case SP_MOUNTAIN_DWARF:
-            case SP_HILL_ORC:
-            case SP_MERFOLK:
-            case SP_SPRIGGAN:
-            case SP_CENTAUR:
-            case SP_OGRE:
-            case SP_TROLL:
-            case SP_MINOTAUR:
-            case SP_DEMONSPAWN:
-            case SP_GHOUL:
-            case SP_VAMPIRE:
-                return (CC_UNRESTRICTED);
-            default:
-                if (species_genus(ng.species) == GENPC_DRACONIAN)
-                    return (CC_UNRESTRICTED);
-                return (CC_RESTRICTED);
-        }
+            return (CC_BANNED);
 
         default:
-            return (CC_RESTRICTED);
+            return (CC_BANNED);
     }
 }
