@@ -3,17 +3,17 @@
 
 #include "feature.h"
 #include "pattern.h"
+#include "newgame_def.h"
 
-class InitLineInput;
+class LineInput;
 struct game_options
 {
 public:
     game_options();
-    void reset_startup_options();
     void reset_options();
 
     void read_option_line(const std::string &s, bool runscripts = false);
-    void read_options(InitLineInput &, bool runscripts,
+    void read_options(LineInput &, bool runscripts,
                       bool clear_aliases = true);
 
     void include(const std::string &file, bool resolve, bool runscript);
@@ -38,15 +38,18 @@ public:
     // View options
     std::vector<feature_override> feature_overrides;
     std::vector<mon_display>      mon_glyph_overrides;
-    unsigned cset_override[NUM_CSET][NUM_DCHAR_TYPES];
+    ucs_t cset_override[NUM_DCHAR_TYPES];
 
     std::string save_dir;       // Directory where saves and bones go.
     std::string macro_dir;      // Directory containing macro.txt
     std::string morgue_dir;     // Directory where character dumps and morgue
                                 // dumps are saved. Overrides crawl_dir.
+    std::string shared_dir;     // Directory where the logfile, scores and bones
+                                // are stored.  On a multi-user system, this dir
+                                // should be accessible by different people.
     std::vector<std::string> additional_macro_files;
 
-    std::string player_name;
+    uint32_t    seed;   // Non-random games.
 
 #ifdef DGL_SIMPLE_MESSAGING
     bool        messaging;      // Check for messages.
@@ -91,9 +94,10 @@ public:
     int         default_friendly_pickup;
 
     bool        show_gold_turns; // Show gold and turns in HUD.
+    bool        show_game_turns; // Show game turns instead of player turns.
     bool        show_beam;       // Show targeting beam by default.
 
-    long        autopickups;     // items to autopickup
+    uint32_t    autopickups;     // items to autopickup
     bool        show_inventory_weights; // show weights in inventory listings
     bool        colour_map;      // add colour to the map
     bool        clean_map;       // remove unseen clouds/monsters
@@ -134,21 +138,8 @@ public:
     msg_colour_type channels[NUM_MESSAGE_CHANNELS];  // msg channel colouring
     bool        darken_beyond_range; // for whether targeting is out of range
 
-    bool        use_old_selection_order; // use old order of species/classes in
-                                         // selection screen
-    int         weapon;          // auto-choose weapon for character
-    int         book;            // auto-choose book for character
-    int         wand;            // auto-choose wand for character
-    int         chaos_knight;    // choice of god for Chaos Knights (Xom/Makleb)
-    int         death_knight;    // choice of god/necromancy for Death Knights
-    god_type    priest;          // choice of god for priests (Zin/Yred)
-    bool        random_pick;     // randomly generate character
-    bool        good_random;     // when chosing randomly only choose
-                                 // unrestricted combinations
     int         hp_warning;      // percentage hp for danger warning
     int         magic_point_warning;    // percentage mp for danger warning
-    char        race;            // preselected race
-    char        cls;             // preselected class
     bool        clear_messages;   // clear messages each turn
     bool        show_more;        // Show message-full more prompts.
     bool        small_more;       // Show one-char more prompts.
@@ -166,7 +157,7 @@ public:
     bool        flush_input[NUM_FLUSH_REASONS]; // when to flush input buff
 
     char_set_type  char_set;
-    FixedVector<unsigned, NUM_DCHAR_TYPES> char_table;
+    FixedVector<ucs_t, NUM_DCHAR_TYPES> char_table;
 
     int         num_colours;     // used for setting up curses colour table (8 or 16)
 
@@ -206,8 +197,6 @@ public:
     bool        arena_dump_msgs_all;
     bool        arena_list_eq;
 
-    // Messages that stop travel
-    std::vector<message_filter> travel_stop_message;
     std::vector<message_filter> force_more_message;
 
     int         stash_tracking; // How stashes are tracked
@@ -263,6 +252,8 @@ public:
     // Some experimental improvements to explore
     bool        explore_improved;
 
+    bool        travel_key_stop;   // Travel stops on keypress.
+
     std::vector<sound_mapping> sound_mappings;
     std::vector<colour_mapping> menu_colour_mappings;
     std::vector<message_colour_mapping> message_colour_mappings;
@@ -295,6 +286,8 @@ public:
 
     int         assign_item_slot;   // How free slots are assigned
 
+    bool        restart_after_game; // If true, Crawl will not close on game-end
+
     std::vector<text_pattern> drop_filter;
 
     FixedArray<bool, NUM_DELAYS, NUM_AINTERRUPTS> activity_interrupts;
@@ -308,6 +301,8 @@ public:
                                     // on the term's own cursor.
     bool        use_fake_player_cursor;
 
+    bool        show_player_species;
+
     int         level_map_cursor_step;  // The cursor increment in the level
                                         // map.
 
@@ -319,7 +314,7 @@ public:
 
 #ifdef WIZARD
     // Parameters for fight simulations.
-    long        fsim_rounds;
+    int         fsim_rounds;
     int         fsim_str, fsim_int, fsim_dex;
     int         fsim_xl;
     std::string fsim_mons;
@@ -328,7 +323,7 @@ public:
 
 #ifdef USE_TILE
     char        tile_show_items[20]; // show which item types in tile inventory
-    bool        tile_title_screen;   // display title screen?
+    bool        tile_skip_title;     // wait for a key at title screen?
     bool        tile_menu_icons;     // display icons in menus?
     // minimap colours
     char        tile_player_col;
@@ -375,31 +370,19 @@ public:
     int         tile_key_repeat_delay;
     int         tile_tooltip_ms;
     tag_pref    tile_tag_pref;
-    tile_display_type  tile_display;
 
     bool        tile_show_minihealthbar;
     bool        tile_show_minimagicbar;
     bool        tile_show_demon_tier;
-    bool        tile_better_transparency;
+    bool        tile_force_regenerate_levels;
+    std::vector<std::string> tile_layout_priority;
 #endif
 
     typedef std::map<std::string, std::string> opt_map;
     opt_map     named_options;          // All options not caught above are
                                         // recorded here.
 
-    ///////////////////////////////////////////////////////////////////////
-    // These options cannot be directly set by the user. Instead they're
-    // set indirectly to the choices the user made for the last character
-    // created. XXX: Isn't there a better place for these?
-    std::string prev_name;
-    char        prev_race;
-    char        prev_cls;
-    int         prev_ck, prev_dk;
-    god_type    prev_pr;
-    int         prev_weapon;
-    int         prev_book;
-    int         prev_wand;
-    bool        prev_randpick;
+    newgame_def game;      // Choices for new game.
 
 private:
     typedef std::map<std::string, std::string> string_map;
@@ -411,7 +394,6 @@ private:
 public:
     // Convenience accessors for the second-class options in named_options.
     int         o_int(const char *name, int def = 0) const;
-    long        o_long(const char *name, long def = 0L) const;
     bool        o_bool(const char *name, bool def = false) const;
     std::string o_str(const char *name, const char *def = NULL) const;
     int         o_colour(const char *name, int def = LIGHTGREY) const;
@@ -427,8 +409,7 @@ private:
     void clear_feature_overrides();
     void clear_cset_overrides();
     void add_cset_override(char_set_type set, const std::string &overrides);
-    void add_cset_override(char_set_type set, dungeon_char_type dc,
-                           unsigned symbol);
+    void add_cset_override(char_set_type set, dungeon_char_type dc, int symbol);
     void add_feature_override(const std::string &);
 
     void add_message_colour_mappings(const std::string &);
@@ -462,6 +443,11 @@ private:
     static const std::string interrupt_prefix;
 };
 
+ucs_t get_glyph_override(int c);
+
+#ifdef DEBUG_GLOBALS
+#define Options (*real_Options)
+#endif
 extern game_options  Options;
 
 #endif
