@@ -5,14 +5,12 @@
 // make sure equip/unequip effects are done the right amount of
 // times, and to make sure melded flags don't get out of sync.
 
-// TODO: move (un)equip_*_effect to player-equip.cc so they can't
-//       accidentally be called from somewhere else.
-
 // XXX: the msg flag isn't implemented in all cases.
 void equip_item(equipment_type slot, int item_slot, bool msg=true);
 bool unequip_item(equipment_type slot, bool msg=true);
 bool meld_slot(equipment_type slot, bool msg=true);
 bool unmeld_slot(equipment_type slot, bool msg=true);
 
-#endif
+bool unwield_item(bool showMsgs = true);
 
+#endif
