@@ -1,11 +1,11 @@
-/*
- *  File:       quiver.cc
- *  Summary:    Player quiver functionality
+/**
+ * @file
+ * @brief Player quiver functionality
  *
- *  - Only change last_used when actually using
- *  - Not changing Qv; nobody knows about internals
- *  - Track last_used of each type so each weapon can do the right thing
- */
+ * - Only change last_used when actually using
+ * - Not changing Qv; nobody knows about internals
+ * - Track last_used of each type so each weapon can do the right thing
+**/
 
 #include "AppHdr.h"
 
@@ -78,6 +78,13 @@ void player_quiver::get_desired_item(const item_def** item_out, int* slot_out) c
 // If no item can be found, return the reason why.
 int player_quiver::get_fire_item(std::string* no_item_reason) const
 {
+    // Felids have no use for the quiver.
+    if (you.species == SP_CAT)
+    {
+        if (no_item_reason != NULL)
+            *no_item_reason = "You can't grasp things well enough to throw them.";
+        return -1;
+    }
     int slot;
     const item_def* desired_item;
 
@@ -150,8 +157,32 @@ static bool _wielded_slot_no_quiver(int slot)
                 || you.skills[SK_THROWING] == 0));
 }
 
+void quiver_item(int slot)
+{
+    const item_def item = you.inv[slot];
+    ASSERT(item.defined());
+
+    ammo_t t = AMMO_THROW;
+    const item_def *weapon = you.weapon();
+    if (weapon && item.launched_by(*weapon))
+        t = _get_weapon_ammo_type(weapon);
+
+    you.m_quiver->set_quiver(you.inv[slot], t);
+    mprf("Quivering %s for %s.", you.inv[slot].name(DESC_INVENTORY).c_str(),
+         t == AMMO_THROW    ? "throwing" :
+         t == AMMO_BLOWGUN  ? "blowguns" :
+         t == AMMO_SLING    ? "slings" :
+         t == AMMO_BOW      ? "bows" :
+                              "crossbows");
+}
+
 void choose_item_for_quiver()
 {
+    if (you.species == SP_CAT)
+    {
+        mpr("You can't grasp things well enough to throw them.");
+        return;
+    }
     int slot = prompt_invent_item("Quiver which item? (- for none, * to show all)",
                                   MT_INVLIST,
                                   OSEL_THROWABLE, true, true, true, '-',
@@ -191,22 +222,7 @@ void choose_item_for_quiver()
             }
         }
     }
-
-    const item_def item = you.inv[slot];
-    ASSERT(item.is_valid());
-
-    ammo_t t = AMMO_THROW;
-    const item_def *weapon = you.weapon();
-    if (weapon && item.launched_by(*weapon))
-        t = _get_weapon_ammo_type(weapon);
-
-    you.m_quiver->set_quiver(you.inv[slot], t);
-    mprf("Quivering %s for %s.", you.inv[slot].name(DESC_INVENTORY).c_str(),
-         t == AMMO_THROW    ? "throwing" :
-         t == AMMO_BLOWGUN  ? "blowguns" :
-         t == AMMO_SLING    ? "slings" :
-         t == AMMO_BOW      ? "bows" :
-                              "crossbows");
+    quiver_item(slot);
 }
 
 // Notification that item was fired with 'f'.
@@ -306,6 +322,10 @@ void player_quiver::on_inv_quantity_changed(int slot, int amt)
 // If current quiver slot is empty, fill it with something useful.
 void player_quiver::_maybe_fill_empty_slot()
 {
+    // Felids have no use for the quiver.
+    if (you.species == SP_CAT)
+        return;
+
     const item_def* weapon = you.weapon();
     const ammo_t slot = _get_weapon_ammo_type(weapon);
 
@@ -316,7 +336,7 @@ void player_quiver::_maybe_fill_empty_slot()
 #endif
 
     bool unquiver_weapon = false;
-    if (m_last_used_of_type[slot].is_valid())
+    if (m_last_used_of_type[slot].defined())
     {
         // If we're wielding an item previously quivered, the quiver may need
         // to be cleared. Else, any already quivered item is valid and we
@@ -372,7 +392,7 @@ void player_quiver::get_fire_order(std::vector<int>& v) const
 // fire order is empty.
 //
 // launcher determines what items match the 'launcher' fire_order type.
-void player_quiver::_get_fire_order( std::vector<int>& order,
+void player_quiver::_get_fire_order(std::vector<int>& order,
                                      bool ignore_inscription_etc,
                                      const item_def* launcher) const
 {
@@ -385,7 +405,7 @@ void player_quiver::_get_fire_order( std::vector<int>& order,
         if (launcher && launcher->sub_type == WPN_BLOWGUN)
         {
             for (int i_inv = inv_start; i_inv < ENDOFPACK; i_inv++)
-                if (you.inv[i_inv].is_valid()
+                if (you.inv[i_inv].defined()
                     && you.inv[i_inv].launched_by(*launcher))
                 {
                     order.push_back(i_inv);
@@ -397,7 +417,7 @@ void player_quiver::_get_fire_order( std::vector<int>& order,
     for (int i_inv = inv_start; i_inv < ENDOFPACK; i_inv++)
     {
         const item_def& item = you.inv[i_inv];
-        if (!item.is_valid())
+        if (!item.defined())
             continue;
 
         // Don't quiver a wielded weapon unless it's a weapon of returning
@@ -428,7 +448,7 @@ void player_quiver::_get_fire_order( std::vector<int>& order,
             if (_item_matches(item, (fire_type) Options.fire_order[i_flags],
                               launcher))
             {
-                order.push_back( (i_flags<<16) | (i_inv & 0xffff) );
+                order.push_back((i_flags<<16) | (i_inv & 0xffff));
                 break;
             }
         }
@@ -450,8 +470,8 @@ void player_quiver::save(writer& outf) const
     marshallShort(outf, QUIVER_COOKIE);
 
     marshallItem(outf, m_last_weapon);
-    marshallLong(outf, m_last_used_type);
-    marshallLong(outf, ARRAYSZ(m_last_used_of_type));
+    marshallInt(outf, m_last_used_type);
+    marshallInt(outf, ARRAYSZ(m_last_used_of_type));
 
     for (unsigned int i = 0; i < ARRAYSZ(m_last_used_of_type); i++)
         marshallItem(outf, m_last_used_of_type[i]);
@@ -463,10 +483,10 @@ void player_quiver::load(reader& inf)
     ASSERT(cooky == QUIVER_COOKIE); (void)cooky;
 
     unmarshallItem(inf, m_last_weapon);
-    m_last_used_type = (ammo_t)unmarshallLong(inf);
+    m_last_used_type = (ammo_t)unmarshallInt(inf);
     ASSERT(m_last_used_type >= AMMO_THROW && m_last_used_type < NUM_AMMO);
 
-    const unsigned long count = unmarshallLong(inf);
+    const unsigned int count = unmarshallInt(inf);
     ASSERT(count <= ARRAYSZ(m_last_used_of_type));
 
     for (unsigned int i = 0; i < count; i++)
@@ -518,7 +538,7 @@ preserve_quiver_slots::~preserve_quiver_slots()
 static bool _item_matches(const item_def &item, fire_type types,
                           const item_def* launcher)
 {
-    ASSERT(item.is_valid());
+    ASSERT(item.defined());
 
     if (types & FIRE_INSCRIBED)
         if (item.inscription.find("+f", 0) != std::string::npos)
@@ -567,9 +587,12 @@ static bool _item_matches(const item_def &item, fire_type types,
 // or -1 if not in inv.
 static int _get_pack_slot(const item_def& item)
 {
-    if (!item.is_valid())
+    if (!item.defined())
         return -1;
 
+    if (in_inventory(item) && _items_similar(item, you.inv[item.link], false))
+        return (item.link);
+
     // First try to find the exact same item.
     for (int i = 0; i < ENDOFPACK; i++)
     {
