@@ -1,8 +1,7 @@
-/*
- *  File:       shopping.cc
- *  Summary:    Shop keeper functions.
- *  Written by: Linley Henzell
- */
+/**
+ * @file
+ * @brief Shop keeper functions.
+**/
 
 #include "AppHdr.h"
 
@@ -18,6 +17,7 @@
 #include "artefact.h"
 #include "cio.h"
 #include "describe.h"
+#include "decks.h"
 #include "dgn-overview.h"
 #include "files.h"
 #include "food.h"
@@ -25,7 +25,7 @@
 #include "items.h"
 #include "itemname.h"
 #include "itemprop.h"
-#include "kills.h"
+#include "libutil.h"
 #include "macro.h"
 #include "menu.h"
 #include "notes.h"
@@ -43,9 +43,9 @@ ShoppingList shopping_list;
 
 static bool _in_shop_now = false;
 
-static bool _purchase( int shop, int item_got, int cost, bool id);
+static bool _purchase(int shop, int item_got, int cost, bool id);
 
-static void _shop_print( const char *shoppy, int line )
+static void _shop_print(const char *shoppy, int line)
 {
     cgotoxy(1, line + 19, GOTO_CRT);
     cprintf("%s", shoppy);
@@ -113,7 +113,7 @@ static std::string _purchase_keys(const std::string &s)
         list += (last = s[i]);
     }
 
-    list += _hyphenated_suffix( s[s.length() - 1], last );
+    list += _hyphenated_suffix(s[s.length() - 1], last);
     list += "</w>";
     return (list);
 }
@@ -130,7 +130,7 @@ static void _list_shop_keys(const std::string &purchasable, bool viewing,
     std::string shop_list = "";
     if (!viewing && you.level_type == LEVEL_DUNGEON)
     {
-        shop_list = "[<w>@</w>] ";
+        shop_list = "[<w>$</w>] ";
         if (num_selected > 0)
             shop_list += "Selected -> shopping list";
         else if (num_in_list > 0)
@@ -173,7 +173,7 @@ static void _list_shop_keys(const std::string &purchasable, bool viewing,
             "/<w>R-Click</w>"
 #endif
             "] exit            [<w>!</w>] %s   %s",
-            (viewing ? "to select items " : "to examine items"),
+            (viewing ? "to buy items    " : "to examine items"),
             pkeys.c_str()));
 
     fs.cprintf("%*s", get_number_of_cols() - fs.length() - 1, "");
@@ -218,11 +218,11 @@ static int _shop_get_item_value(const item_def& item, int greed, bool id,
     return result;
 }
 
-static std::string _shop_print_stock( const std::vector<int>& stock,
+static std::string _shop_print_stock(const std::vector<int>& stock,
                                       const std::vector<bool>& selected,
                                       const std::vector<bool>& in_list,
                                       const shop_struct& shop,
-                                      int total_cost )
+                                      int total_cost)
 {
     ShopInfo &si  = StashTrack.get_shop(shop.pos);
     const bool id = shoptype_identifies_stock(shop.type);
@@ -260,7 +260,7 @@ static std::string _shop_print_stock( const std::vector<int>& stock,
             textcolor(YELLOW);
 
         if (in_list[i])
-            cprintf("%c @ ", c);
+            cprintf("%c $ ", c);
         else if (selected[i])
             cprintf("%c + ", c);
         else
@@ -277,8 +277,8 @@ static std::string _shop_print_stock( const std::vector<int>& stock,
         else
             textcolor(i % 2 ? LIGHTGREY : WHITE);
 
-        cprintf("%-56s%5d gold",
-                item.name(DESC_NOCAP_A, false, id).substr(0, 56).c_str(),
+        cprintf("%s%5d gold",
+                chop_string(item.name(DESC_NOCAP_A, false, id), 56).c_str(),
                 gp_value);
 
         si.add_item(item, gp_value);
@@ -319,7 +319,7 @@ static int _count_identical(const std::vector<int>& stock,
 //  For the ? key, the text should read:
 //  [!] switch to examination mode
 //  [!] switch to selection mode
-static bool _in_a_shop( int shopidx, int &num_in_list )
+static bool _in_a_shop(int shopidx, int &num_in_list)
 {
     const shop_struct& shop = env.shop[shopidx];
 
@@ -375,7 +375,7 @@ static bool _in_a_shop( int shopidx, int &num_in_list )
             selected.resize(stock.size(), false);
         }
 
-            num_in_list  = 0;
+        num_in_list  = 0;
         int num_selected = 0;
         for (unsigned int i = 0; i < stock.size(); i++)
         {
@@ -414,7 +414,7 @@ static bool _in_a_shop( int shopidx, int &num_in_list )
                                                       id_stock);
 
                 unsigned int num = shopping_list.cull_identical_items(item,
-                                                             (long) cost);
+                                                                      cost);
                 if (num > 0)
                 {
                     in_list[i] = true;
@@ -465,8 +465,8 @@ static bool _in_a_shop( int shopidx, int &num_in_list )
         if (first)
         {
             first = false;
-            snprintf( info, INFO_SIZE, "%s What would you like to do? ",
-                      hello.c_str() );
+            snprintf(info, INFO_SIZE, "%s What would you like to do? ",
+                      hello.c_str());
         }
         else
             snprintf(info, INFO_SIZE, "What would you like to do? ");
@@ -477,17 +477,11 @@ static bool _in_a_shop( int shopidx, int &num_in_list )
         textcolor(LIGHTGREY);
 
         mouse_control mc(MOUSE_MODE_MORE);
-        int key = getch();
+        int key = getchm();
 
         if (key == '\\')
-        {
-            if (!check_item_knowledge(true))
-            {
-                _shop_print("You don't recognise anything yet!", 1);
-                _shop_more();
-            }
-        }
-        else if (key == 'x' || key == ESCAPE || key == CK_MOUSE_CMD)
+            check_item_knowledge();
+        else if (key == 'x' || key_is_escape(key) || key == CK_MOUSE_CMD)
             break;
         else if (key == '\r' || key == CK_MOUSE_CLICK)
         {
@@ -529,10 +523,10 @@ static bool _in_a_shop( int shopidx, int &num_in_list )
                 continue;
             else
             {
-                snprintf(info, INFO_SIZE, "Purchase for %d gold? (y/n) ",
+                snprintf(info, INFO_SIZE, "Purchase for %d gold? (y/n)",
                          total_purchase);
 
-                if ( _shop_yesno(info, 'n') )
+                if (_shop_yesno(info, 'n'))
                 {
                     int num_items = 0, outside_items = 0, quant;
                     for (int i = to_buy.size() - 1; i >= 0; --i)
@@ -580,10 +574,10 @@ static bool _in_a_shop( int shopidx, int &num_in_list )
 
                     if (outside_items)
                     {
-                        mprf( "I'll put %s outside for you.",
+                        mprf("I'll put %s outside for you.",
                               num_items == 1             ? "it" :
                               num_items == outside_items ? "them"
-                                                         : "part of them" );
+                                                         : "part of them");
                     }
                     bought_something = true;
                 }
@@ -598,7 +592,7 @@ static bool _in_a_shop( int shopidx, int &num_in_list )
         }
         else if (key == '?' || key == '*')
             browse_inventory(false);
-        else if (key == '@')
+        else if (key == '$')
         {
             if (viewing || (num_selected == 0 && num_in_list == 0)
                 || you.level_type != LEVEL_DUNGEON)
@@ -649,7 +643,7 @@ static bool _in_a_shop( int shopidx, int &num_in_list )
                 }
             }
         }
-        else if (!isalpha(key))
+        else if (!isaalpha(key))
         {
             _shop_print("Huh?", 1);
             _shop_more();
@@ -657,7 +651,7 @@ static bool _in_a_shop( int shopidx, int &num_in_list )
         else
         {
             key = tolower(key) - 'a';
-            if (key >= static_cast<int>(stock.size()) )
+            if (key >= static_cast<int>(stock.size()))
             {
                 _shop_print("No such item.", 1);
                 _shop_more();
@@ -674,7 +668,7 @@ static bool _in_a_shop( int shopidx, int &num_in_list )
                 // type-ID and once you can access the item (by buying it)
                 // you have its full ID anyway. Worst case, it won't get
                 // noted when you buy it.
-                const unsigned long old_flags = item.flags;
+                const uint64_t old_flags = item.flags;
                 if (id_stock)
                 {
                     item.flags |= (ISFLAG_IDENT_MASK | ISFLAG_NOTED_ID
@@ -693,8 +687,8 @@ static bool _in_a_shop( int shopidx, int &num_in_list )
                 {
                     if (gp_value > you.gold)
                     {
-                        if ( _shop_yesno("Remove from shopping list? (y/N)",
-                                         'n') )
+                        if (_shop_yesno("Remove from shopping list? (y/N)",
+                                         'n'))
                         {
                             shopping_list.del_thing(item);
                             in_list[key]  = false;
@@ -704,8 +698,8 @@ static bool _in_a_shop( int shopidx, int &num_in_list )
                     }
                     else
                     {
-                        if ( _shop_yesno("Remove item from shopping list and "
-                                         "buy it? (Y/n)",  'y') )
+                        if (_shop_yesno("Remove item from shopping list and "
+                                         "buy it? (Y/n)",  'y'))
                         {
                             shopping_list.del_thing(item);
                             in_list[key] = false;
@@ -738,7 +732,7 @@ bool shoptype_identifies_stock(shop_type type)
             && type != SHOP_GENERAL_ANTIQUE);
 }
 
-static bool _purchase( int shop, int item_got, int cost, bool id )
+static bool _purchase(int shop, int item_got, int cost, bool id)
 {
     you.del_gold(cost);
 
@@ -758,12 +752,12 @@ static bool _purchase( int shop, int item_got, int cost, bool id )
 
     const int quant = item.quantity;
     // Note that item will be invalidated if num == item.quantity.
-    const int num = move_item_to_player( item_got, item.quantity, true );
+    const int num = move_item_to_player(item_got, item.quantity, true);
 
     // Shopkeepers will now place goods you can't carry outside the shop.
     if (num < quant)
     {
-        move_item_to_grid( &item_got, env.shop[shop].pos );
+        move_item_to_grid(&item_got, env.shop[shop].pos);
         return (false);
     }
     return (true);
@@ -775,13 +769,13 @@ static bool _purchase( int shop, int item_got, int cost, bool id )
 // be returning just over 30 right now.  Note that this isn't used
 // as a multiple, its used in the old ring way: 7 * ret is added to
 // the price of the artefact. -- bwr
-int artefact_value( const item_def &item )
+int artefact_value(const item_def &item)
 {
-    ASSERT( is_artefact( item ) );
+    ASSERT(is_artefact(item));
 
     int ret = 10;
     artefact_properties_t prop;
-    artefact_wpn_properties( item, prop );
+    artefact_wpn_properties(item, prop);
 
     // Brands are already accounted for via existing ego checks
 
@@ -867,14 +861,14 @@ int artefact_value( const item_def &item )
     return ((ret > 0) ? ret : 0);
 }
 
-unsigned int item_value( item_def item, bool ident )
+unsigned int item_value(item_def item, bool ident)
 {
     // Note that we pass item in by value, since we want a local
     // copy to mangle as necessary.
     item.flags = (ident) ? (item.flags | ISFLAG_IDENT_MASK) : (item.flags);
 
-    if (is_unrandom_artefact( item )
-        && item_ident( item, ISFLAG_KNOW_PROPERTIES ))
+    if (is_unrandom_artefact(item)
+        && item_ident(item, ISFLAG_KNOW_PROPERTIES))
     {
         const unrandart_entry *entry = get_unrand_entry(item.special);
         if (entry->value != 0)
@@ -1011,11 +1005,12 @@ unsigned int item_value( item_def item, bool ident )
         case WPN_DEMON_BLADE:
         case WPN_TRIPLE_SWORD:
         case WPN_BLESSED_KATANA:
-        case WPN_HOLY_EUDEMON_BLADE:
+        case WPN_EUDEMON_BLADE:
         case WPN_BLESSED_DOUBLE_SWORD:
         case WPN_BLESSED_GREAT_SWORD:
         case WPN_BLESSED_TRIPLE_SWORD:
-        case WPN_HOLY_SCOURGE:
+        case WPN_SACRED_SCOURGE:
+        case WPN_TRISHULA:
         case WPN_LAJATANG:
             valued += 200;
             break;
@@ -1253,10 +1248,12 @@ unsigned int item_value( item_def item, bool ident )
     case OBJ_ARMOUR:
         switch (item.sub_type)
         {
+        case ARM_PEARL_DRAGON_ARMOUR:
         case ARM_GOLD_DRAGON_ARMOUR:
             valued += 1600;
             break;
 
+        case ARM_PEARL_DRAGON_HIDE:
         case ARM_GOLD_DRAGON_HIDE:
             valued += 1400;
             break;
@@ -1371,7 +1368,7 @@ unsigned int item_value( item_def item, bool ident )
 
         if (item_type_known(item))
         {
-            const int sparm = get_armour_ego_type( item );
+            const int sparm = get_armour_ego_type(item);
             switch (sparm)
             {
             case SPARM_NORMAL:
@@ -1436,7 +1433,7 @@ unsigned int item_value( item_def item, bool ident )
             valued /= 10;
         }
 
-        if (item_ident( item, ISFLAG_KNOW_PLUSES ))
+        if (item_ident(item, ISFLAG_KNOW_PLUSES))
         {
             valued += 5;
             if (item.plus >= 0)
@@ -1455,10 +1452,10 @@ unsigned int item_value( item_def item, bool ident )
             }
         }
 
-        if (is_artefact( item ))
+        if (is_artefact(item))
         {
             if (item_type_known(item))
-                valued += (7 * artefact_value( item ));
+                valued += (7 * artefact_value(item));
             else
                 valued += 50;
         }
@@ -1475,7 +1472,7 @@ unsigned int item_value( item_def item, bool ident )
         break;
 
     case OBJ_WANDS:
-        if ( !item_type_known(item) )
+        if (!item_type_known(item))
             valued += 200;
         else
         {
@@ -1533,7 +1530,7 @@ unsigned int item_value( item_def item, bool ident )
                 break;
             }
 
-            if (item_ident( item, ISFLAG_KNOW_PLUSES ))
+            if (item_ident(item, ISFLAG_KNOW_PLUSES))
             {
                 if (item.plus == 0)
                     valued -= 50;
@@ -1544,7 +1541,7 @@ unsigned int item_value( item_def item, bool ident )
         break;
 
     case OBJ_POTIONS:
-        if ( !item_type_known(item) )
+        if (!item_type_known(item))
             valued += 9;
         else
         {
@@ -1553,46 +1550,57 @@ unsigned int item_value( item_def item, bool ident )
             case POT_EXPERIENCE:
                 valued += 500;
                 break;
+
             case POT_GAIN_DEXTERITY:
             case POT_GAIN_INTELLIGENCE:
             case POT_GAIN_STRENGTH:
                 valued += 350;
                 break;
+
             case POT_CURE_MUTATION:
                 valued += 150;
                 break;
+
             case POT_MAGIC:
             case POT_RESISTANCE:
                 valued += 70;
                 break;
+
             case POT_INVISIBILITY:
                 valued += 55;
                 break;
+
             case POT_MUTATION:
             case POT_RESTORE_ABILITIES:
                 valued += 50;
                 break;
+
             case POT_BERSERK_RAGE:
             case POT_HEAL_WOUNDS:
                 valued += 30;
                 break;
+
             case POT_MIGHT:
             case POT_AGILITY:
             case POT_BRILLIANCE:
             case POT_SPEED:
                 valued += 25;
                 break;
+
             case POT_HEALING:
             case POT_LEVITATION:
                 valued += 20;
                 break;
+
             case POT_BLOOD:
             case POT_PORRIDGE:
                 valued += 10;
                 break;
+
             case POT_BLOOD_COAGULATED:
                 valued += 5;
                 break;
+
             case POT_CONFUSION:
             case POT_DECAY:
             case POT_DEGENERATION:
@@ -1665,7 +1673,7 @@ unsigned int item_value( item_def item, bool ident )
         break;
 
     case OBJ_SCROLLS:
-        if ( !item_type_known(item) )
+        if (!item_type_known(item))
             valued += 10;
         else
         {
@@ -1674,54 +1682,68 @@ unsigned int item_value( item_def item, bool ident )
             case SCR_ACQUIREMENT:
                 valued += 520;
                 break;
+
             case SCR_ENCHANT_WEAPON_III:
             case SCR_VORPALISE_WEAPON:
                 valued += 200;
                 break;
+
             case SCR_SUMMONING:
                 valued += 95;
                 break;
+
             case SCR_TORMENT:
             case SCR_HOLY_WORD:
             case SCR_SILENCE:
             case SCR_VULNERABILITY:
                 valued += 75;
                 break;
+
             case SCR_ENCHANT_WEAPON_II:
                 valued += 55;
                 break;
+
             case SCR_RECHARGING:
+            case SCR_AMNESIA:
                 valued += 50;
                 break;
+
             case SCR_ENCHANT_ARMOUR:
             case SCR_ENCHANT_WEAPON_I:
                 valued += 48;
                 break;
+
             case SCR_FEAR:
                 valued += 45;
                 break;
+
             case SCR_MAGIC_MAPPING:
                 valued += 35;
                 break;
+
             case SCR_BLINKING:
             case SCR_REMOVE_CURSE:
             case SCR_TELEPORTATION:
                 valued += 30;
                 break;
+
             case SCR_DETECT_CURSE:
             case SCR_IDENTIFY:
                 valued += 20;
                 break;
+
             case SCR_FOG:
                 valued += 10;
                 break;
+
             case SCR_NOISE:
             case SCR_RANDOM_USELESSNESS:
                 valued += 2;
                 break;
+
             case SCR_CURSE_ARMOUR:
             case SCR_CURSE_WEAPON:
-            case SCR_PAPER:
+            case SCR_CURSE_JEWELLERY:
             case SCR_IMMOLATION:
                 valued++;
                 break;
@@ -1733,11 +1755,11 @@ unsigned int item_value( item_def item, bool ident )
         if (item_known_cursed(item))
             valued -= 10;
 
-        if ( !item_type_known(item) )
+        if (!item_type_known(item))
             valued += 50;
         else
         {
-            if (item_ident( item, ISFLAG_KNOW_PLUSES )
+            if (item_ident(item, ISFLAG_KNOW_PLUSES)
                 && (item.sub_type == RING_PROTECTION
                     || item.sub_type == RING_STRENGTH
                     || item.sub_type == RING_EVASION
@@ -1750,6 +1772,12 @@ unsigned int item_value( item_def item, bool ident )
 
                 if (item.sub_type == RING_SLAYING && item.plus2 > 0)
                     valued += 10 * item.plus2;
+
+                if (item.plus < 0)
+                    valued -= 50;
+
+                if (item.sub_type == RING_SLAYING && item.plus2 < 0)
+                    valued -= 50;
             }
 
             switch (item.sub_type)
@@ -1757,26 +1785,33 @@ unsigned int item_value( item_def item, bool ident )
             case RING_INVISIBILITY:
                 valued += 100;
                 break;
+
             case RING_REGENERATION:
                 valued += 75;
                 break;
+
             case RING_FIRE:
             case RING_ICE:
                 valued += 62;
                 break;
+
             case RING_LIFE_PROTECTION:
                 valued += 60;
                 break;
+
             case RING_TELEPORT_CONTROL:
                 valued += 42;
                 break;
+
             case RING_MAGICAL_POWER:
             case RING_PROTECTION_FROM_MAGIC:
                 valued += 40;
                 break;
+
             case RING_WIZARDRY:
                 valued += 35;
                 break;
+
             case RING_LEVITATION:
             case RING_POISON_RESISTANCE:
             case RING_PROTECTION_FROM_COLD:
@@ -1784,14 +1819,17 @@ unsigned int item_value( item_def item, bool ident )
             case RING_SLAYING:
                 valued += 30;
                 break;
+
             case RING_SUSTAIN_ABILITIES:
             case RING_SUSTENANCE:
             case RING_TELEPORTATION: // usually cursed
                 valued += 25;
                 break;
+
             case RING_SEE_INVISIBLE:
                 valued += 20;
                 break;
+
             case RING_DEXTERITY:
             case RING_EVASION:
             case RING_INTELLIGENCE:
@@ -1799,28 +1837,34 @@ unsigned int item_value( item_def item, bool ident )
             case RING_STRENGTH:
                 valued += 10;
                 break;
+
             case RING_HUNGER:
                 valued -= 50;
                 break;
+
             case AMU_THE_GOURMAND:
             case AMU_GUARDIAN_SPIRIT:
             case AMU_FAITH:
                 valued += 35;
                 break;
+
             case AMU_CLARITY:
             case AMU_RESIST_CORROSION:
             case AMU_RESIST_MUTATION:
             case AMU_WARDING:
                 valued += 30;
                 break;
+
             case AMU_CONSERVATION:
             case AMU_CONTROLLED_FLIGHT:
                 valued += 25;
                 break;
+
             case AMU_RAGE:
             case AMU_STASIS:
                 valued += 20;
                 break;
+
             case AMU_INACCURACY:
                 valued -= 50;
                 break;
@@ -1832,9 +1876,9 @@ unsigned int item_value( item_def item, bool ident )
                 // in this branch we're guaranteed to know
                 // the item type!
                 if (valued < 0)
-                    valued = artefact_value( item ) - 5;
+                    valued = artefact_value(item) - 5;
                 else
-                    valued += artefact_value( item );
+                    valued += artefact_value(item);
             }
 
             valued *= 7;
@@ -1849,24 +1893,31 @@ unsigned int item_value( item_def item, bool ident )
             case MISC_RUNE_OF_ZOT:  // upped from 1200 to encourage collecting
                 valued += 10000;
                 break;
+
             case MISC_HORN_OF_GERYON:
                 valued += 5000;
                 break;
+
             case MISC_DISC_OF_STORMS:
                 valued += 2000;
                 break;
+
             case MISC_CRYSTAL_BALL_OF_SEEING:
                 valued += 500;
                 break;
+
             case MISC_BOTTLED_EFREET:
                 valued += 400;
                 break;
-            case MISC_CRYSTAL_BALL_OF_FIXATION:
+
             case MISC_EMPTY_EBONY_CASKET:
                 valued += 20;
                 break;
             default:
-                valued += 500;
+                if (is_deck(item))
+                    valued += 200 + item.special * 150;
+                else
+                    valued += 500;
             }
         }
         else
@@ -1876,15 +1927,19 @@ unsigned int item_value( item_def item, bool ident )
             case MISC_RUNE_OF_ZOT:
                 valued += 5000;
                 break;
+
             case MISC_HORN_OF_GERYON:
                 valued += 1000;
                 break;
+
             case MISC_CRYSTAL_BALL_OF_SEEING:
                 valued += 450;
                 break;
+
             case MISC_BOTTLED_EFREET:
                 valued += 350;
                 break;
+
             default:
                 valued += 400;
             }
@@ -1893,6 +1948,7 @@ unsigned int item_value( item_def item, bool ident )
 
     case OBJ_BOOKS:
         valued = 150;
+
         if (item_type_known(item))
         {
             double rarity = 0;
@@ -1945,22 +2001,21 @@ unsigned int item_value( item_def item, bool ident )
             valued = 120;
         else if (item.sub_type == STAFF_SMITING
                 || item.sub_type == STAFF_STRIKING
-                || item.sub_type == STAFF_WARDING
-                || item.sub_type == STAFF_DISCOVERY)
+                || item.sub_type == STAFF_WARDING)
         {
             valued = 150;
         }
         else
             valued = 250;
 
-        if (item_is_rod( item ) && item_ident( item, ISFLAG_KNOW_PLUSES ))
+        if (item_is_rod(item) && item_ident(item, ISFLAG_KNOW_PLUSES))
             valued += 50 * (item.plus2 / ROD_CHARGE_MULT);
-
         break;
 
     case OBJ_ORBS:
         valued = 250000;
         break;
+
     default:
         break;
     }                           // end switch
@@ -1994,7 +2049,7 @@ void shop()
     }
 
     // Quick out, if no inventory
-    if ( _shop_get_stock(i).empty() )
+    if (_shop_get_stock(i).empty())
     {
         const shop_struct& shop = env.shop[i];
         mprf("%s appears to be closed.", shop_name(shop.pos).c_str());
@@ -2020,20 +2075,32 @@ void shop()
         mpr("You can access your shopping list by pressing '$'.");
 }
 
+void destroy_shop(shop_struct *shop)
+{
+    if (shop)
+    {
+        unnotice_feature(level_pos(level_id::current(), shop->pos));
+
+        shop->pos  = coord_def(0, 0);
+        shop->type = SHOP_UNASSIGNED;
+    }
+}
+
+void destroy_shop_at(coord_def p)
+{
+    destroy_shop(get_shop(p));
+}
+
 shop_struct *get_shop(const coord_def& where)
 {
     if (grd(where) != DNGN_ENTER_SHOP)
         return (NULL);
 
-    // Check all shops for one at the correct position.
-    for (int i = 0; i < MAX_SHOPS; i ++)
-    {
-        shop_struct& shop = env.shop[i];
-        // A little bit of paranoia.
-        if (shop.pos == where && shop.type != SHOP_UNASSIGNED)
-            return (&shop);
-    }
-    return (NULL);
+    unsigned short t = env.tgrid(where);
+    ASSERT(t != NON_ENTITY && t < MAX_SHOPS);
+    ASSERT(env.shop[t].pos == where && env.shop[t].type != SHOP_UNASSIGNED);
+
+    return (&env.shop[t]);
 }
 
 std::string shop_name(const coord_def& where, bool add_stop)
@@ -2044,13 +2111,70 @@ std::string shop_name(const coord_def& where, bool add_stop)
     return (name);
 }
 
+std::string shop_type_name (shop_type type)
+{
+    switch (type)
+    {
+        case SHOP_WEAPON_ANTIQUE:
+            return "Antique Weapon";
+        case SHOP_ARMOUR_ANTIQUE:
+            return "Antique Armour";
+        case SHOP_WEAPON:
+            return "Weapon";
+        case SHOP_ARMOUR:
+            return "Armour";
+        case SHOP_JEWELLERY:
+            return "Jewellery";
+        case SHOP_WAND:
+            return "Magical Wand";
+        case SHOP_BOOK:
+            return "Book";
+        case SHOP_FOOD:
+            return "Food";
+        case SHOP_SCROLL:
+            return "Magic Scroll";
+        case SHOP_GENERAL_ANTIQUE:
+            return "Assorted Antiques";
+        case SHOP_DISTILLERY:
+            return "Distillery";
+        case SHOP_GENERAL:
+            return "General Store";
+        default:
+            return ("Bug");
+    }
+}
+
+std::string shop_type_suffix (shop_type type, const coord_def &where)
+{
+    if (type == SHOP_GENERAL
+        || type == SHOP_GENERAL_ANTIQUE
+        || type == SHOP_DISTILLERY)
+    {
+        return ("");
+    }
+
+    const char* suffixnames[] = {"Shoppe", "Boutique", "Emporium", "Shop"};
+    const int temp = (where.x + where.y) % 4;
+
+    return std::string(suffixnames[temp]);
+}
+
 std::string shop_name(const coord_def& where)
 {
     const shop_struct *cshop = get_shop(where);
 
-    // paranoia
+    // paranoia and shop mimics
     if (grd(where) != DNGN_ENTER_SHOP)
+    {
+        if (monster_at(where))
+        {
+            monster* mmimic = monster_at(where);
+            if (mons_is_feat_mimic(mmimic->type) && mmimic->props.exists("shop_name"))
+                return mmimic->props["shop_name"].get_string();
+        }
+
         return ("");
+    }
 
     if (!cshop)
     {
@@ -2060,39 +2184,35 @@ std::string shop_name(const coord_def& where)
 
     const shop_type type = cshop->type;
 
-    unsigned long seed = static_cast<unsigned long>( cshop->keeper_name[0] )
-        | (static_cast<unsigned long>( cshop->keeper_name[1] ) << 8)
-        | (static_cast<unsigned long>( cshop->keeper_name[1] ) << 16);
+    std::string sh_name = "";
 
-    std::string sh_name = apostrophise(make_name(seed, false)) + " ";
-
-    if (type == SHOP_WEAPON_ANTIQUE || type == SHOP_ARMOUR_ANTIQUE)
-        sh_name += "Antique ";
-
-    sh_name +=
-        (type == SHOP_WEAPON
-         || type == SHOP_WEAPON_ANTIQUE) ? "Weapon" :
-        (type == SHOP_ARMOUR
-         || type == SHOP_ARMOUR_ANTIQUE) ? "Armour" :
-
-        (type == SHOP_JEWELLERY)         ? "Jewellery" :
-        (type == SHOP_WAND)              ? "Magical Wand" :
-        (type == SHOP_BOOK)              ? "Book" :
-        (type == SHOP_FOOD)              ? "Food" :
-        (type == SHOP_SCROLL)            ? "Magic Scroll" :
-        (type == SHOP_GENERAL_ANTIQUE)   ? "Assorted Antiques" :
-        (type == SHOP_DISTILLERY)        ? "Distillery" :
-        (type == SHOP_GENERAL)           ? "General Store"
-                                         : "Bug";
-
-    if (type != SHOP_GENERAL
-        && type != SHOP_GENERAL_ANTIQUE
-        && type != SHOP_DISTILLERY)
+    if (!cshop->shop_name.empty())
     {
-        const char* suffixnames[] = {"Shoppe", "Boutique", "Emporium", "Shop"};
-        const int temp = (where.x + where.y) % 4;
-        sh_name += ' ';
-        sh_name += suffixnames[temp];
+        sh_name += apostrophise(cshop->shop_name) + " ";
+    }
+    else
+    {
+        uint32_t seed = static_cast<uint32_t>(cshop->keeper_name[0])
+            | (static_cast<uint32_t>(cshop->keeper_name[1]) << 8)
+            | (static_cast<uint32_t>(cshop->keeper_name[1]) << 16);
+
+        sh_name += apostrophise(make_name(seed, false)) + " ";
+    }
+
+    if (!cshop->shop_type_name.empty())
+        sh_name += cshop->shop_type_name;
+    else
+        sh_name += shop_type_name(type);
+
+    if (!cshop->shop_suffix_name.empty())
+    {
+        sh_name += " " + cshop->shop_suffix_name;
+    }
+    else
+    {
+        std::string sh_suffix = shop_type_suffix(type, where);
+        if (!sh_suffix.empty())
+            sh_name += " " + sh_suffix;
     }
 
     return (sh_name);
@@ -2144,13 +2264,13 @@ ShoppingList::ShoppingList()
 
 #define SETUP_THING()                             \
     CrawlHashTable *thing = new CrawlHashTable();  \
-    (*thing)[SHOPPING_THING_COST_KEY] = (long) cost; \
+    (*thing)[SHOPPING_THING_COST_KEY] = cost; \
     (*thing)[SHOPPING_THING_POS_KEY]  = pos;
 
 bool ShoppingList::add_thing(const item_def &item, int cost,
                              const level_pos* _pos)
 {
-    ASSERT(item.is_valid());
+    ASSERT(item.defined());
     ASSERT(cost > 0);
 
     SETUP_POS();
@@ -2227,6 +2347,13 @@ bool ShoppingList::is_on_list(std::string desc,
     return (find_thing(desc, pos) != -1);
 }
 
+void ShoppingList::del_thing_at_index(int idx)
+{
+    ASSERT(idx >= 0 && idx < list->size());
+    list->erase(idx);
+    refresh();
+}
+
 bool ShoppingList::del_thing(const item_def &item,
                              const level_pos* _pos)
 {
@@ -2241,9 +2368,7 @@ bool ShoppingList::del_thing(const item_def &item,
         return (false);
     }
 
-    list->erase(idx);
-    refresh();
-
+    del_thing_at_index(idx);
     return (true);
 }
 
@@ -2260,9 +2385,7 @@ bool ShoppingList::del_thing(std::string desc, const level_pos* _pos)
         return (false);
     }
 
-    list->erase(idx);
-    refresh();
-
+    del_thing_at_index(idx);
     return (true);
 }
 
@@ -2274,19 +2397,19 @@ bool ShoppingList::del_thing(std::string desc, const level_pos* _pos)
 // TODO:
 //
 // * If you get a randart which lets you turn invisible, then remove
-//   any ordinary rings of invisiblity from the shopping list.
+//   any ordinary rings of invisibility from the shopping list.
 //
 // * If you collected enough spellbooks that all the spells in a
 //   shopping list book are covered, then auto-remove it.
 unsigned int ShoppingList::cull_identical_items(const item_def& item,
-                                                long cost)
+                                                int cost)
 {
     // Can't put items in Bazaar shops in the shopping list, so
-    // don't bother transfering shopping list items to Bazaar shops.
+    // don't bother transferring shopping list items to Bazaar shops.
     if (cost != -1 && you.level_type != LEVEL_DUNGEON)
         return (0);
 
-    switch(item.base_type)
+    switch (item.base_type)
     {
     case OBJ_JEWELLERY:
     case OBJ_BOOKS:
@@ -2303,8 +2426,14 @@ unsigned int ShoppingList::cull_identical_items(const item_def& item,
 
     // Ignore stat-modification rings which reduce a stat, since they're
     // worthless.
-    if (item.plus < 0 && item.base_type == OBJ_JEWELLERY)
-        return (0);
+    if (item.base_type == OBJ_JEWELLERY)
+    {
+        if (item.sub_type == RING_SLAYING && item.plus < 0 && item.plus2 < 0)
+            return (0);
+
+        if (item.plus < 0)
+            return (0);
+    }
 
     // Item is already on shopping-list.
     const bool on_list = find_thing(item, level_pos::current()) != -1;
@@ -2346,7 +2475,7 @@ unsigned int ShoppingList::cull_identical_items(const item_def& item,
         // one on the shopping list.
         if (cost != -1)
         {
-            long list_cost = thing_cost(thing);
+            int list_cost = thing_cost(thing);
 
             if (cost >= list_cost)
                 continue;
@@ -2412,7 +2541,7 @@ int ShoppingList::size() const
         return (0);
     }
 
-    return ( list->size() );
+    return (list->size());
 }
 
 bool ShoppingList::items_are_same(const item_def& item_a,
@@ -2455,6 +2584,7 @@ void ShoppingList::forget_pos(const level_pos &pos)
             i--;
         }
     }
+    refresh();
 }
 
 void ShoppingList::gold_changed(int old_amount, int new_amount)
@@ -2473,7 +2603,7 @@ void ShoppingList::gold_changed(int old_amount, int new_amount)
         for (unsigned int i = min_unbuyable_idx; i < list->size(); i++)
         {
             const CrawlHashTable &thing = (*list)[i];
-            const long           cost   = thing_cost(thing);
+            const int            cost   = thing_cost(thing);
 
             if (cost > new_amount)
             {
@@ -2524,11 +2654,11 @@ void ShoppingListMenu::draw_title()
 {
     if (title)
     {
-        const long total_cost = you.props[SHOPPING_LIST_COST_KEY];
+        const int total_cost = you.props[SHOPPING_LIST_COST_KEY];
 
         cgotoxy(1, 1);
         textcolor(title->colour);
-        cprintf("%d %s%s, total cost %ld gp",
+        cprintf("%d %s%s, total cost %d gp",
                 title->quantity, title->text.c_str(),
                 title->quantity > 1? "s" : "",
                 total_cost);
@@ -2549,7 +2679,7 @@ void ShoppingList::fill_out_menu(Menu& shopmenu)
     {
         CrawlHashTable &thing = (*list)[i];
         level_pos      pos    = thing_pos(thing);
-        long           cost   = thing_cost(thing);
+        int            cost   = thing_cost(thing);
 
         std::string etitle =
             make_stringf("[%s] %s (%d gp)", short_place_name(pos.id).c_str(),
@@ -2607,8 +2737,8 @@ void ShoppingList::display()
                                         you.gold);
     shopmenu.set_more(formatted_string::parse_string(more_str));
 
-    shopmenu.set_flags( MF_SINGLESELECT | MF_ALWAYS_SHOW_MORE
-                        | MF_ALLOW_FORMATTING );
+    shopmenu.set_flags(MF_SINGLESELECT | MF_ALWAYS_SHOW_MORE
+                        | MF_ALLOW_FORMATTING);
 
     fill_out_menu(shopmenu);
 
@@ -2628,7 +2758,7 @@ void ShoppingList::display()
 
         if (shopmenu.menu_action == Menu::ACT_EXECUTE)
         {
-            const long cost = thing_cost(*thing);
+            const int cost = thing_cost(*thing);
 
             if (cost > you.gold)
             {
@@ -2651,7 +2781,7 @@ void ShoppingList::display()
             if (is_item)
             {
                 const item_def &item = get_thing_item(*thing);
-                describe_item( const_cast<item_def&>(item) );
+                describe_item(const_cast<item_def&>(item));
             }
             else // not an item, so we only stored a description.
             {
@@ -2662,9 +2792,7 @@ void ShoppingList::display()
                          (int) thing_cost(*thing));
 
                 print_description(info);
-
-                if (getch() == 0)
-                    getch();
+                wait_for_keypress();
             }
         }
         else if (shopmenu.menu_action == Menu::ACT_MISC)
@@ -2685,7 +2813,7 @@ void ShoppingList::display()
                 continue;
             }
 
-            list->erase(index);
+            del_thing_at_index(index);
             if (list->size() == 0)
                 break;
 
@@ -2693,19 +2821,19 @@ void ShoppingList::display()
             fill_out_menu(shopmenu);
         }
         else
-            DEBUGSTR("Invalid menu action type");
+            die("Invalid menu action type");
     }
     redraw_screen();
 }
 
-bool _compare_shopping_things(const CrawlStoreValue& a,
-                              const CrawlStoreValue& b)
+static bool _compare_shopping_things(const CrawlStoreValue& a,
+                                     const CrawlStoreValue& b)
 {
     const CrawlHashTable& hash_a = a.get_table();
     const CrawlHashTable& hash_b = b.get_table();
 
-    const long a_cost = hash_a[SHOPPING_THING_COST_KEY];
-    const long b_cost = hash_b[SHOPPING_THING_COST_KEY];
+    const int a_cost = hash_a[SHOPPING_THING_COST_KEY];
+    const int b_cost = hash_b[SHOPPING_THING_COST_KEY];
 
     return (a_cost < b_cost);
 }
@@ -2723,13 +2851,13 @@ void ShoppingList::refresh()
     max_buyable_cost   = -1;
     max_buyable_idx    = -1;
 
-    long total_cost = 0;
+    int total_cost = 0;
 
     for (unsigned int i = 0; i < list->size(); i++)
     {
         const CrawlHashTable &thing = (*list)[i];
 
-        const long cost = thing_cost(thing);
+        const int cost = thing_cost(thing);
 
         if (cost <= you.gold)
         {
@@ -2743,7 +2871,7 @@ void ShoppingList::refresh()
         }
         total_cost += cost;
     }
-    you.props[SHOPPING_LIST_COST_KEY] = (long) total_cost;
+    you.props[SHOPPING_LIST_COST_KEY].get_int() = total_cost;
 }
 
 int ShoppingList::find_thing(const item_def &item,
@@ -2800,7 +2928,7 @@ const item_def& ShoppingList::get_thing_item(const CrawlHashTable& thing)
     ASSERT(thing.exists(SHOPPING_THING_ITEM_KEY));
 
     const item_def &item = thing[SHOPPING_THING_ITEM_KEY].get_item();
-    ASSERT(item.is_valid());
+    ASSERT(item.defined());
 
     return (item);
 }
@@ -2813,10 +2941,10 @@ std::string ShoppingList::get_thing_desc(const CrawlHashTable& thing)
     return (desc);
 }
 
-long ShoppingList::thing_cost(const CrawlHashTable& thing)
+int ShoppingList::thing_cost(const CrawlHashTable& thing)
 {
     ASSERT(thing.exists(SHOPPING_THING_COST_KEY));
-    return (thing[SHOPPING_THING_COST_KEY].get_long());
+    return (thing[SHOPPING_THING_COST_KEY].get_int());
 }
 
 level_pos ShoppingList::thing_pos(const CrawlHashTable& thing)
