@@ -63,28 +63,10 @@ struct show_info
     monster_type mons;
 };
 
-class monsters;
-class show_def
-{
-    FixedArray<show_type, ENV_SHOW_DIAMETER, ENV_SHOW_DIAMETER> grid;
-    FixedArray<show_type, ENV_SHOW_DIAMETER, ENV_SHOW_DIAMETER> backup;
+class monster;
 
-    void _update_feat_at(const coord_def &gp, const coord_def &ep);
-    void _update_item_at(const coord_def &gp, const coord_def &ep);
-    void _update_cloud(int cloudno);
-    void _update_monster(const monsters *monster);
-    void _set_backup(const coord_def &e);
-
-public:
-    show_type operator()(const coord_def &ep) const { return grid(ep); }
-    show_type get_backup(const coord_def &ep) const { return backup(ep); }
-
-    void init();
-    void update_at(const coord_def &gp, const coord_def &ep);
-};
-
-// Convert a show object as in env.show to one to be stored in
-// env.map_knowledge (re-setting feature colour mainly).
-show_type to_knowledge(show_type obj);
+void show_init(bool terrain_only = false);
+void show_update_at(const coord_def &gp, bool terrain_only = false);
+void show_update_emphasis();
 
 #endif
