@@ -1,13 +1,27 @@
-/*
- *  File:       spl-data.h
- *  Summary:    Spell definitions and descriptions. See spell_desc struct in
- *              spl-util.cc.
- *  Written by: Linley Henzell
- */
+/**
+ * @file
+ * @brief Spell definitions and descriptions. See spell_desc struct in
+ *             spl-util.cc.
+**/
 
 #ifndef SPLDATA_H
 #define SPLDATA_H
 
+/*
+struct spell_desc
+{
+    enum, spell name,
+    spell schools,
+    flags,
+    level,
+    power_cap,
+    min_range, max_range, (-1 if not applicable)
+    noise_mod,
+    target_prompt,
+    monster spell: needs tracer?,
+    monster spell: utility spell?
+}
+*/
 
 {
     SPELL_TELEPORT_SELF, "Teleport Self",
@@ -24,7 +38,7 @@
 
 {
     SPELL_CAUSE_FEAR, "Cause Fear",
-     SPTYP_ENCHANTMENT,
+     SPTYP_HEXES,
      SPFLAG_AREA,
      5,
      200,
@@ -62,7 +76,7 @@
 },
 
 {
-     SPELL_APPORTATION, "Apportation",
+    SPELL_APPORTATION, "Apportation",
      SPTYP_TRANSLOCATION,
      SPFLAG_TARG_OBJ | SPFLAG_NOT_SELF,
      1,
@@ -120,7 +134,7 @@
      4,
      200,
      LOS_RADIUS, LOS_RADIUS,
-     0,
+     -4, // Dig needs to be silent for stalker
      NULL,
      false,
      true
@@ -180,9 +194,9 @@
 
 {
     SPELL_POLYMORPH_OTHER, "Polymorph Other",
-     SPTYP_TRANSMUTATION,  // removed enchantment, wasn't needed - bwr
+     SPTYP_TRANSMUTATION | SPTYP_HEXES,
      SPFLAG_DIR_OR_TARGET | SPFLAG_NOT_SELF | SPFLAG_CHAOTIC,
-     5,
+     4,
      200,
      LOS_RADIUS, LOS_RADIUS,
      0,
@@ -193,9 +207,9 @@
 
 {
     SPELL_SLOW, "Slow",
-     SPTYP_ENCHANTMENT,
+     SPTYP_HEXES,
      SPFLAG_DIR_OR_TARGET,
-     3,
+     2,
      200,
      LOS_RADIUS, LOS_RADIUS,
      0,
@@ -206,7 +220,7 @@
 
 {
     SPELL_HASTE, "Haste",
-     SPTYP_ENCHANTMENT,
+     SPTYP_CHARMS,
      SPFLAG_DIR_OR_TARGET | SPFLAG_HELPFUL | SPFLAG_HASTY,
      6,  // lowered to 6 from 8, since it's easily available from various items
          // and Swiftness is level 2 (and gives a similar effect).  It's also
@@ -221,7 +235,7 @@
 
 {
     SPELL_PETRIFY, "Petrify",
-     SPTYP_ENCHANTMENT | SPTYP_EARTH,
+     SPTYP_TRANSMUTATION | SPTYP_EARTH,
      SPFLAG_DIR_OR_TARGET,
      4,
      200,
@@ -234,7 +248,7 @@
 
 {
     SPELL_CONFUSE, "Confuse",
-     SPTYP_ENCHANTMENT,
+     SPTYP_HEXES,
      SPFLAG_DIR_OR_TARGET,
      3,
      200,
@@ -247,12 +261,12 @@
 
 {
     SPELL_INVISIBILITY, "Invisibility",
-     SPTYP_ENCHANTMENT,
+     SPTYP_HEXES,
      SPFLAG_DIR_OR_TARGET | SPFLAG_HELPFUL,
      6,
      200,
      LOS_RADIUS, LOS_RADIUS,
-     0,
+     -6,
      NULL,
      false,
      true
@@ -302,7 +316,7 @@
     SPELL_FREEZING_CLOUD, "Freezing Cloud",
      SPTYP_CONJURATION | SPTYP_ICE | SPTYP_AIR,
      SPFLAG_GRID | SPFLAG_AREA,
-     7,
+     6,
      200,
      6, 6,
      0,
@@ -314,7 +328,7 @@
 {
     SPELL_MEPHITIC_CLOUD, "Mephitic Cloud",
      SPTYP_CONJURATION | SPTYP_POISON | SPTYP_AIR,
-     SPFLAG_DIR_OR_TARGET | SPFLAG_AREA,
+     SPFLAG_DIR_OR_TARGET | SPFLAG_AREA | SPFLAG_ALLOW_SELF,
      3,
      200,
      5, 5,
@@ -326,9 +340,9 @@
 
 {
     SPELL_RING_OF_FLAMES, "Ring of Flames",
-     SPTYP_ENCHANTMENT | SPTYP_FIRE,
+     SPTYP_CHARMS | SPTYP_FIRE,
      SPFLAG_AREA,
-     8,
+     7,
      200,
      -1, -1,
      0,
@@ -367,7 +381,7 @@
     SPELL_TELEPORT_OTHER, "Teleport Other",
      SPTYP_TRANSLOCATION,
      SPFLAG_DIR_OR_TARGET | SPFLAG_NOT_SELF | SPFLAG_ESCAPE,
-     4,
+     3,
      200,
      LOS_RADIUS, LOS_RADIUS,
      0,
@@ -378,7 +392,7 @@
 
 {
     SPELL_DEATHS_DOOR, "Death's Door",
-     SPTYP_ENCHANTMENT | SPTYP_NECROMANCY,
+     SPTYP_CHARMS | SPTYP_NECROMANCY,
      SPFLAG_NONE,
      8,
      200,
@@ -389,22 +403,9 @@
      false
 },
 
-{
-    SPELL_SELECTIVE_AMNESIA, "Selective Amnesia",
-     SPTYP_ENCHANTMENT,
-     SPFLAG_NONE,
-     4,
-     0,
-     -1, -1,
-     0,
-     NULL,
-     false,
-     true
-},
-
 {
     SPELL_MASS_CONFUSION, "Mass Confusion",
-     SPTYP_ENCHANTMENT,
+     SPTYP_HEXES,
      SPFLAG_AREA,
      6,
      200,
@@ -469,7 +470,7 @@
 
 {
     SPELL_LEVITATION, "Levitation",
-     SPTYP_ENCHANTMENT | SPTYP_AIR,
+     SPTYP_CHARMS | SPTYP_AIR,
      SPFLAG_NONE,
      2,
      150,
@@ -487,7 +488,7 @@
      6,
      200,
      6, 6,
-     0,
+     -3, //the beam is silent
      NULL,
      true,
      false
@@ -519,11 +520,24 @@
      false
 },
 
+{
+    SPELL_TORNADO, "Tornado",
+     SPTYP_AIR,
+     SPFLAG_AREA,
+     9,
+     200,
+     TORNADO_RADIUS, TORNADO_RADIUS,
+     0,
+     NULL,
+     false,
+     false
+},
+
 {
     SPELL_POISONOUS_CLOUD, "Poisonous Cloud",
      SPTYP_CONJURATION | SPTYP_POISON | SPTYP_AIR,
-     SPFLAG_GRID | SPFLAG_AREA,
-     6,
+     SPFLAG_GRID | SPFLAG_AREA | SPFLAG_ALLOW_SELF,
+     5,
      200,
      6, 6,
      0,
@@ -572,7 +586,7 @@
 },
 
 {
-     SPELL_BLINK, "Blink",
+    SPELL_BLINK, "Blink",
      SPTYP_TRANSLOCATION,
      SPFLAG_ESCAPE,
      2,
@@ -585,7 +599,7 @@
 },
 
 {
-     SPELL_BLINK_RANGE, "Blink Range", // XXX needs better name
+    SPELL_BLINK_RANGE, "Blink Range", // XXX needs better name
      SPTYP_TRANSLOCATION,
      SPFLAG_ESCAPE | SPFLAG_MONSTER,
      2,
@@ -598,7 +612,7 @@
 },
 
 {
-     SPELL_BLINK_AWAY, "Blink Away",
+    SPELL_BLINK_AWAY, "Blink Away",
      SPTYP_TRANSLOCATION,
      SPFLAG_ESCAPE | SPFLAG_MONSTER,
      2,
@@ -611,7 +625,7 @@
 },
 
 {
-     SPELL_BLINK_CLOSE, "Blink Close",
+    SPELL_BLINK_CLOSE, "Blink Close",
      SPTYP_TRANSLOCATION,
      SPFLAG_MONSTER,
      2,
@@ -641,8 +655,8 @@
 {
     SPELL_SUMMON_SWARM, "Summon Swarm",
      SPTYP_SUMMONING,
-     SPFLAG_UNCLEAN | SPFLAG_BATTLE,
-     6,
+     SPFLAG_BATTLE,
+     5,
      200,
      -1, -1,
      0,
@@ -664,9 +678,22 @@
      false
 },
 
+{
+    SPELL_MALIGN_GATEWAY, "Malign Gateway",
+     SPTYP_SUMMONING | SPTYP_TRANSLOCATION,
+     SPFLAG_UNHOLY | SPFLAG_BATTLE | SPFLAG_CHAOTIC,
+     7,
+     200,
+     -1, -1,
+     0,
+     NULL,
+     false,
+     false
+},
+
 {
     SPELL_ENSLAVEMENT, "Enslavement",
-     SPTYP_ENCHANTMENT,
+     SPTYP_HEXES,
      SPFLAG_DIR_OR_TARGET | SPFLAG_NOT_SELF,
      4,
      200,
@@ -678,7 +705,7 @@
 },
 
 {
-   SPELL_ANIMATE_DEAD, "Animate Dead",
+    SPELL_ANIMATE_DEAD, "Animate Dead",
      SPTYP_NECROMANCY,
      SPFLAG_AREA | SPFLAG_NEUTRAL | SPFLAG_CORPSE_VIOLATING,
      4,
@@ -703,9 +730,10 @@
      false
 },
 
+#if TAG_MAJOR_VERSION == 32
 {
     SPELL_EXTENSION, "Extension",
-     SPTYP_ENCHANTMENT,
+     SPTYP_CHARMS,
      SPFLAG_NONE,
      5,
      200,
@@ -715,12 +743,13 @@
      false,
      true
 },
+#endif
 
 {
     SPELL_CONTROL_UNDEAD, "Control Undead",
-     SPTYP_ENCHANTMENT | SPTYP_NECROMANCY,
+     SPTYP_HEXES | SPTYP_NECROMANCY,
      SPFLAG_NONE,
-     6,
+     5,
      200,
      -1, -1,
      0,
@@ -744,7 +773,7 @@
 {
     SPELL_VAMPIRIC_DRAINING, "Vampiric Draining",
      SPTYP_NECROMANCY,
-     SPFLAG_DIR | SPFLAG_NOT_SELF | SPFLAG_BATTLE,
+     SPFLAG_DIR_OR_TARGET | SPFLAG_NOT_SELF | SPFLAG_BATTLE,
      3,
      200,
      1, 1,
@@ -758,7 +787,7 @@
     SPELL_HAUNT, "Haunt",
      SPTYP_SUMMONING | SPTYP_NECROMANCY,
      SPFLAG_TARGET | SPFLAG_NOT_SELF,
-     6,
+     7,
      200,
      LOS_RADIUS, LOS_RADIUS,
      0,
@@ -784,7 +813,7 @@
     SPELL_BORGNJORS_REVIVIFICATION, "Borgnjor's Revivification",
      SPTYP_NECROMANCY,
      SPFLAG_NONE,
-     5,
+     7,
      200,
      -1, -1,
      0,
@@ -796,7 +825,7 @@
 {
     SPELL_FREEZE, "Freeze",
      SPTYP_ICE,
-     SPFLAG_DIR | SPFLAG_NOT_SELF | SPFLAG_BATTLE,
+     SPFLAG_DIR_OR_TARGET | SPFLAG_NOT_SELF | SPFLAG_BATTLE,
      1,
      25,
      1, 1,
@@ -838,7 +867,7 @@
      SPFLAG_DIR_OR_TARGET,
      4,
      100,
-     5, 5,
+     1, 1,
      0,
      NULL,
      true,
@@ -860,7 +889,7 @@
 
 {
     SPELL_OZOCUBUS_ARMOUR, "Ozocubu's Armour",
-     SPTYP_ENCHANTMENT | SPTYP_ICE,
+     SPTYP_CHARMS | SPTYP_ICE,
      SPFLAG_NONE,
      3,
      200,
@@ -875,7 +904,7 @@
     SPELL_CALL_IMP, "Call Imp",
      SPTYP_SUMMONING,
      SPFLAG_UNHOLY | SPFLAG_BATTLE,
-     3,
+     2,
      200,
      -1, -1,
      0,
@@ -886,7 +915,7 @@
 
 {
     SPELL_REPEL_MISSILES, "Repel Missiles",
-     SPTYP_ENCHANTMENT | SPTYP_AIR,
+     SPTYP_CHARMS | SPTYP_AIR,
      SPFLAG_NONE,
      2,
      200,
@@ -899,7 +928,7 @@
 
 {
     SPELL_BERSERKER_RAGE, "Berserker Rage",
-     SPTYP_ENCHANTMENT,
+     SPTYP_CHARMS,
      SPFLAG_HASTY,
      3,
      0,
@@ -964,7 +993,7 @@
 
 {
     SPELL_REGENERATION, "Regeneration",
-     SPTYP_ENCHANTMENT | SPTYP_NECROMANCY,
+     SPTYP_CHARMS | SPTYP_NECROMANCY,
      SPFLAG_NONE,
      3,
      200,
@@ -975,23 +1004,11 @@
      true
 },
 
-{
-    SPELL_BONE_SHARDS, "Bone Shards",
-     SPTYP_NECROMANCY,
-     SPFLAG_DIR_OR_TARGET | SPFLAG_CORPSE_VIOLATING,
-     3,
-     200,
-     6, 6,
-     0,
-     NULL,
-     true,
-     false
-},
-
+// Monster-only, players can use Lugonu's ability
 {
     SPELL_BANISHMENT, "Banishment",
      SPTYP_TRANSLOCATION,
-     SPFLAG_DIR_OR_TARGET | SPFLAG_UNHOLY | SPFLAG_CHAOTIC,
+     SPFLAG_DIR_OR_TARGET | SPFLAG_UNHOLY | SPFLAG_CHAOTIC | SPFLAG_MONSTER,
      4,
      200,
      LOS_RADIUS, LOS_RADIUS,
@@ -1005,7 +1022,7 @@
     SPELL_CIGOTUVIS_DEGENERATION, "Cigotuvi's Degeneration",
      SPTYP_TRANSMUTATION | SPTYP_NECROMANCY,
      SPFLAG_DIR_OR_TARGET | SPFLAG_NOT_SELF | SPFLAG_CHAOTIC
-     | SPFLAG_CORPSE_VIOLATING,
+         | SPFLAG_CORPSE_VIOLATING,
      5,
      200,
      LOS_RADIUS, LOS_RADIUS,
@@ -1043,7 +1060,7 @@
 
 {
     SPELL_TUKIMAS_DANCE, "Tukima's Dance",
-     SPTYP_ENCHANTMENT,
+     SPTYP_HEXES,
      SPFLAG_NONE,
      3,
      150,
@@ -1054,6 +1071,19 @@
      false
 },
 
+{
+    SPELL_TUKIMAS_BALL, "Tukima's Ball",
+     SPTYP_HEXES,
+     SPFLAG_NONE,
+     9,
+     200,
+     -1,-1,
+     0,
+     NULL,
+     false,
+     true
+},
+
 {
     SPELL_SUMMON_DEMON, "Summon Demon",
      SPTYP_SUMMONING,
@@ -1106,22 +1136,9 @@
      false
 },
 
-{
-    SPELL_TUKIMAS_VORPAL_BLADE, "Tukima's Vorpal Blade",
-     SPTYP_ENCHANTMENT,
-     SPFLAG_HELPFUL | SPFLAG_BATTLE,
-     2,
-     200,
-     -1, -1,
-     0,
-     NULL,
-     false,
-     true
-},
-
 {
     SPELL_FIRE_BRAND, "Fire Brand",
-     SPTYP_ENCHANTMENT | SPTYP_FIRE,
+     SPTYP_CHARMS | SPTYP_FIRE,
      SPFLAG_HELPFUL | SPFLAG_BATTLE,
      2,
      200,
@@ -1134,7 +1151,7 @@
 
 {
     SPELL_FREEZING_AURA, "Freezing Aura",
-     SPTYP_ENCHANTMENT | SPTYP_ICE,
+     SPTYP_CHARMS | SPTYP_ICE,
      SPFLAG_HELPFUL | SPFLAG_BATTLE,
      2,
      200,
@@ -1147,7 +1164,7 @@
 
 {
     SPELL_LETHAL_INFUSION, "Lethal Infusion",
-     SPTYP_ENCHANTMENT | SPTYP_NECROMANCY,
+     SPTYP_CHARMS | SPTYP_NECROMANCY,
      SPFLAG_HELPFUL | SPFLAG_BATTLE,
      2,
      200,
@@ -1184,9 +1201,10 @@
      false
 },
 
+#if TAG_MAJOR_VERSION == 32
 {
     SPELL_STONEMAIL, "Stonemail",
-     SPTYP_ENCHANTMENT | SPTYP_EARTH,
+     SPTYP_CHARMS | SPTYP_EARTH,
      SPFLAG_NONE,
      6,
      200,
@@ -1196,6 +1214,7 @@
      false,
      true
 },
+#endif
 
 {
     SPELL_SHOCK, "Shock",
@@ -1212,7 +1231,7 @@
 
 {
     SPELL_SWIFTNESS, "Swiftness",
-     SPTYP_ENCHANTMENT | SPTYP_AIR,
+     SPTYP_CHARMS | SPTYP_AIR,
      SPFLAG_HASTY,
      2,
      200,
@@ -1225,7 +1244,7 @@
 
 {
     SPELL_FLY, "Flight",
-     SPTYP_ENCHANTMENT | SPTYP_AIR,
+     SPTYP_CHARMS | SPTYP_AIR,
      SPFLAG_NONE,
      4,
      200,
@@ -1238,7 +1257,7 @@
 
 {
     SPELL_INSULATION, "Insulation",
-     SPTYP_ENCHANTMENT | SPTYP_AIR,
+     SPTYP_CHARMS | SPTYP_AIR,
      SPFLAG_NONE,
      4,
      200,
@@ -1277,7 +1296,7 @@
 
 {
     SPELL_CONTROL_TELEPORT, "Control Teleport",
-     SPTYP_ENCHANTMENT | SPTYP_TRANSLOCATION,
+     SPTYP_CHARMS | SPTYP_TRANSLOCATION,
      SPFLAG_HELPFUL,
      4,
      200,
@@ -1290,7 +1309,7 @@
 
 {
     SPELL_POISON_WEAPON, "Poison Weapon",
-     SPTYP_ENCHANTMENT | SPTYP_POISON,
+     SPTYP_CHARMS | SPTYP_POISON,
      SPFLAG_HELPFUL | SPFLAG_BATTLE,
      3,
      0,
@@ -1303,7 +1322,7 @@
 
 {
     SPELL_RESIST_POISON, "Resist Poison",
-     SPTYP_ENCHANTMENT | SPTYP_POISON,
+     SPTYP_CHARMS | SPTYP_POISON,
      SPFLAG_HELPFUL,
      4,
      200,
@@ -1316,7 +1335,7 @@
 
 {
     SPELL_PROJECTED_NOISE, "Projected Noise",
-     SPTYP_ENCHANTMENT,
+     SPTYP_HEXES,
      SPFLAG_NONE,
      2,
      0,
@@ -1327,6 +1346,7 @@
      false
 },
 
+#if TAG_MAJOR_VERSION == 32
 {
     SPELL_ALTER_SELF, "Alter Self",
      SPTYP_TRANSMUTATION,
@@ -1339,6 +1359,7 @@
      false,
      false
 },
+#endif
 
 {
     SPELL_DEBUGGING_RAY, "Debugging Ray",
@@ -1366,19 +1387,6 @@
      true
 },
 
-{
-    SPELL_PORTAL, "Portal",
-     SPTYP_TRANSLOCATION,
-     SPFLAG_ESCAPE,
-     7,
-     0,
-     -1, -1,
-     0,
-     NULL,
-     false,
-     true
-},
-
 {
     SPELL_AGONY, "Agony",
      SPTYP_NECROMANCY,
@@ -1388,14 +1396,14 @@
      LOS_RADIUS, LOS_RADIUS,
      0,
      NULL,
-     false,
+     true,
      false
 },
 
 {
     SPELL_SPIDER_FORM, "Spider Form",
      SPTYP_TRANSMUTATION | SPTYP_POISON,
-     SPFLAG_HELPFUL,
+     SPFLAG_HELPFUL | SPFLAG_CHAOTIC,
      3,
      200,
      -1, -1,
@@ -1421,7 +1429,7 @@
 {
     SPELL_BLADE_HANDS, "Blade Hands",
      SPTYP_TRANSMUTATION,
-     SPFLAG_HELPFUL | SPFLAG_BATTLE,
+     SPFLAG_HELPFUL | SPFLAG_BATTLE | SPFLAG_CHAOTIC,
      5,  // only removes weapon, so I raised this from 4 -- bwr
      200,
      -1, -1,
@@ -1434,7 +1442,7 @@
 {
     SPELL_STATUE_FORM, "Statue Form",
      SPTYP_TRANSMUTATION | SPTYP_EARTH,
-     SPFLAG_HELPFUL,
+     SPFLAG_HELPFUL | SPFLAG_CHAOTIC,
      6,
      200,
      -1, -1,
@@ -1447,7 +1455,7 @@
 {
     SPELL_ICE_FORM, "Ice Form",
      SPTYP_ICE | SPTYP_TRANSMUTATION,
-     SPFLAG_HELPFUL,
+     SPFLAG_HELPFUL | SPFLAG_CHAOTIC,
      4, // doesn't allow for equipment, so I lowered this from 5 -- bwr
      200,
      -1, -1,
@@ -1460,7 +1468,7 @@
 {
     SPELL_DRAGON_FORM, "Dragon Form",
      SPTYP_FIRE | SPTYP_TRANSMUTATION,
-     SPFLAG_HELPFUL,
+     SPFLAG_HELPFUL | SPFLAG_CHAOTIC,
      8,
      200,
      -1, -1,
@@ -1473,7 +1481,7 @@
 {
     SPELL_NECROMUTATION, "Necromutation",
      SPTYP_TRANSMUTATION | SPTYP_NECROMANCY,
-     SPFLAG_HELPFUL | SPFLAG_CORPSE_VIOLATING,
+     SPFLAG_HELPFUL | SPFLAG_CORPSE_VIOLATING | SPFLAG_CHAOTIC,
      8,
      200,
      -1, -1,
@@ -1487,7 +1495,7 @@
     SPELL_DEATH_CHANNEL, "Death Channel",
      SPTYP_NECROMANCY,
      SPFLAG_HELPFUL,
-     9,
+     7,
      200,
      -1, -1,
      0,
@@ -1496,10 +1504,11 @@
      true
 },
 
+// Monster-only, players can use Kiku's ability
 {
     SPELL_SYMBOL_OF_TORMENT, "Symbol of Torment",
      SPTYP_NECROMANCY,
-     SPFLAG_AREA,
+     SPFLAG_AREA | SPFLAG_MONSTER,
      6,
      0,
      -1, -1,
@@ -1511,7 +1520,7 @@
 
 {
     SPELL_DEFLECT_MISSILES, "Deflect Missiles",
-     SPTYP_ENCHANTMENT | SPTYP_AIR,
+     SPTYP_CHARMS | SPTYP_AIR,
      SPFLAG_HELPFUL,
      6,
      200,
@@ -1576,7 +1585,7 @@
 
 {
     SPELL_CONFUSING_TOUCH, "Confusing Touch",
-     SPTYP_ENCHANTMENT,
+     SPTYP_HEXES,
      SPFLAG_NONE,
      1,
      200,
@@ -1589,7 +1598,7 @@
 
 {
     SPELL_SURE_BLADE, "Sure Blade",
-     SPTYP_ENCHANTMENT,
+     SPTYP_HEXES | SPTYP_CHARMS,
      SPFLAG_HELPFUL | SPFLAG_BATTLE,
      2,
      200,
@@ -1609,14 +1618,14 @@
      1, 4,
      0,
      NULL,
-     false,
+     true,
      false
 },
 
 {
-     SPELL_PASSWALL, "Passwall",
+    SPELL_PASSWALL, "Passwall",
      SPTYP_TRANSMUTATION | SPTYP_EARTH,
-     SPFLAG_DIR | SPFLAG_ESCAPE,
+     SPFLAG_DIR | SPFLAG_ESCAPE | SPFLAG_NOT_SELF,
      3,
      200,
      1, 1,
@@ -1630,7 +1639,7 @@
     SPELL_IGNITE_POISON, "Ignite Poison",
      SPTYP_FIRE | SPTYP_TRANSMUTATION,
      SPFLAG_AREA | SPFLAG_BATTLE,
-     6,
+     5,
      200,
      -1, -1,
      0,
@@ -1641,7 +1650,7 @@
 
 {
     SPELL_STICKS_TO_SNAKES, "Sticks to Snakes",
-     SPTYP_TRANSMUTATION | SPTYP_SUMMONING,
+     SPTYP_TRANSMUTATION,
      SPFLAG_BATTLE,
      2,
      200,
@@ -1667,7 +1676,7 @@
 
 {
     SPELL_SUMMON_DRAGON, "Summon Dragon",
-     SPTYP_FIRE | SPTYP_SUMMONING,
+     SPTYP_SUMMONING,
      SPFLAG_NONE,
      9,
      200,
@@ -1678,27 +1687,14 @@
      false
 },
 
-{
-    SPELL_TAME_BEASTS, "Tame Beasts",
-     SPTYP_ENCHANTMENT,
-     SPFLAG_AREA,
-     5,
-     200,
-     -1, -1,
-     0,
-     NULL,
-     false,
-     false
-},
-
 {
     SPELL_HIBERNATION, "Ensorcelled Hibernation",
-     SPTYP_ENCHANTMENT | SPTYP_ICE,
+     SPTYP_HEXES | SPTYP_ICE,
      SPFLAG_DIR_OR_TARGET | SPFLAG_NOT_SELF,
      2,
      56,
      LOS_RADIUS, LOS_RADIUS,
-     0,
+     -2, //putting a monster to sleep should be silent
      NULL,
      true,
      false
@@ -1706,12 +1702,12 @@
 
 {
     SPELL_ENGLACIATION, "Metabolic Englaciation",
-     SPTYP_ENCHANTMENT | SPTYP_ICE,
+     SPTYP_HEXES | SPTYP_ICE,
      SPFLAG_AREA,
      7,
      200,
      -1, -1,
-     0,
+     -7,//putting a monster to sleep should be silent
      NULL,
      false,
      false
@@ -1732,7 +1728,7 @@
 
 {
     SPELL_SEE_INVISIBLE, "See Invisible",
-     SPTYP_ENCHANTMENT,
+     SPTYP_CHARMS,
      SPFLAG_HELPFUL,
      4,
      200,
@@ -1761,7 +1757,7 @@
      SPTYP_SUMMONING,
      SPFLAG_NONE,
      1,
-     200,
+     100,
      -1, -1,
      0,
      NULL,
@@ -1771,9 +1767,9 @@
 
 {
     SPELL_WARP_BRAND, "Warp Weapon",
-     SPTYP_ENCHANTMENT | SPTYP_TRANSLOCATION,
+     SPTYP_CHARMS | SPTYP_TRANSLOCATION,
      SPFLAG_HELPFUL | SPFLAG_BATTLE,
-     7,     // this is high for a reason - Warp brands are very powerful.
+     5,     // this is high for a reason - Warp brands are very powerful.
      0,
      -1, -1,
      0,
@@ -1784,7 +1780,7 @@
 
 {
     SPELL_SILENCE, "Silence",
-     SPTYP_ENCHANTMENT | SPTYP_AIR,
+     SPTYP_CHARMS | SPTYP_AIR,
      SPFLAG_AREA,
      5,
      200,
@@ -1797,7 +1793,7 @@
 
 {
     SPELL_SHATTER, "Shatter",
-     SPTYP_TRANSMUTATION | SPTYP_EARTH,
+     SPTYP_EARTH,
      SPFLAG_AREA,
      9,
      200,
@@ -1812,7 +1808,7 @@
     SPELL_DISPERSAL, "Dispersal",
      SPTYP_TRANSLOCATION,
      SPFLAG_AREA | SPFLAG_ESCAPE,
-     7,
+     6,
      200,
      -1, -1,
      0,
@@ -1836,7 +1832,7 @@
 
 {
     SPELL_CORONA, "Corona",
-     SPTYP_ENCHANTMENT,
+     SPTYP_HEXES,
      SPFLAG_DIR_OR_TARGET | SPFLAG_NOT_SELF,
      1,
      200,
@@ -1863,7 +1859,7 @@
 {
     SPELL_EVAPORATE, "Evaporate",
      SPTYP_FIRE | SPTYP_TRANSMUTATION,
-     SPFLAG_DIR_OR_TARGET | SPFLAG_AREA,
+     SPFLAG_DIR_OR_TARGET | SPFLAG_AREA | SPFLAG_ALLOW_SELF,
      2,   // XXX: level 2 or 3, what should it be now? -- bwr
      200,
      6, 6,
@@ -1875,7 +1871,7 @@
 
 {
     SPELL_FRAGMENTATION, "Lee's Rapid Deconstruction",
-     SPTYP_TRANSMUTATION | SPTYP_EARTH,
+     SPTYP_EARTH,
      SPFLAG_GRID,
      5,
      200,
@@ -1888,11 +1884,11 @@
 
 {
     SPELL_SANDBLAST, "Sandblast",
-     SPTYP_TRANSMUTATION | SPTYP_EARTH,
+     SPTYP_EARTH,
      SPFLAG_DIR_OR_TARGET | SPFLAG_NOT_SELF | SPFLAG_BATTLE,
      1,
      50,
-     1, 1,                      // Special-cased!
+     2, 2,                      // Special-cased!
      0,
      NULL,
      true,
@@ -1901,7 +1897,7 @@
 
 {
     SPELL_MAXWELLS_SILVER_HAMMER, "Maxwell's Silver Hammer",
-     SPTYP_TRANSMUTATION | SPTYP_EARTH,
+     SPTYP_CHARMS | SPTYP_EARTH,
      SPFLAG_HELPFUL | SPFLAG_BATTLE,
      2,
      200,
@@ -1979,7 +1975,7 @@
 
 {
     SPELL_EXCRUCIATING_WOUNDS, "Excruciating Wounds",
-     SPTYP_ENCHANTMENT | SPTYP_NECROMANCY,
+     SPTYP_CHARMS | SPTYP_NECROMANCY,
      SPFLAG_HELPFUL | SPFLAG_BATTLE,
      5,     // fairly high level - potentially one of the best brands
      200,
@@ -2007,7 +2003,7 @@
     SPELL_SUMMON_UGLY_THING, "Summon Ugly Thing",
      SPTYP_SUMMONING,
      SPFLAG_CHAOTIC,
-     5,
+     6,
      200,
      -1, -1,
      0,
@@ -2016,11 +2012,24 @@
      false
 },
 
+{
+    SPELL_GOLUBRIAS_PASSAGE, "Passage of Golubria",
+     SPTYP_TRANSLOCATION,
+     SPFLAG_GRID | SPFLAG_NEUTRAL | SPFLAG_NOT_SELF | SPFLAG_ESCAPE,
+     4,
+     0,
+     LOS_RADIUS, LOS_RADIUS,
+     0,
+     NULL,
+     false,
+     false
+},
+
 // From here on, all spells are monster-only spells.
 
 {
     SPELL_PARALYSE, "Paralyse",
-     SPTYP_ENCHANTMENT,
+     SPTYP_HEXES,
      SPFLAG_DIR_OR_TARGET | SPFLAG_MONSTER,
      4,
      200,
@@ -2033,7 +2042,7 @@
 
 {
     SPELL_MINOR_HEALING, "Minor Healing",
-     SPTYP_ENCHANTMENT, // was SPTYP_HOLY
+     SPTYP_NONE, // was SPTYP_HOLY
      SPFLAG_RECOVERY | SPFLAG_HELPFUL | SPFLAG_MONSTER,
      2,
      0,
@@ -2046,7 +2055,7 @@
 
 {
     SPELL_MAJOR_HEALING, "Major Healing",
-     SPTYP_ENCHANTMENT, // was SPTYP_HOLY
+     SPTYP_NONE, // was SPTYP_HOLY
      SPFLAG_RECOVERY | SPFLAG_HELPFUL | SPFLAG_MONSTER,
      6,
      0,
@@ -2110,6 +2119,19 @@
      false
 },
 
+{
+    SPELL_NOXIOUS_CLOUD, "Noxious Cloud",
+     SPTYP_CONJURATION | SPTYP_POISON | SPTYP_AIR,
+     SPFLAG_GRID | SPFLAG_AREA | SPFLAG_MONSTER,
+     5,
+     200,
+     6, 6,
+     0,
+     NULL,
+     true,
+     false
+},
+
 {
     SPELL_STEAM_BALL, "Steam Ball",
      SPTYP_CONJURATION | SPTYP_FIRE,
@@ -2267,7 +2289,7 @@
 },
 
 {
-     SPELL_BLINK_OTHER_CLOSE, "Blink Other Close",
+    SPELL_BLINK_OTHER_CLOSE, "Blink Other Close",
      SPTYP_TRANSLOCATION,
      SPFLAG_TARGET | SPFLAG_MONSTER,
      2,
@@ -2305,6 +2327,20 @@
      false
 },
 
+// Monster version of the spell (with full range)
+{
+    SPELL_STICKY_FLAME_RANGE, "Sticky Flame Range",
+     SPTYP_CONJURATION | SPTYP_FIRE,
+     SPFLAG_DIR_OR_TARGET | SPFLAG_MONSTER,
+     4,
+     100,
+     5, 5,
+     0,
+     NULL,
+     true,
+     false
+},
+
 {
     SPELL_STICKY_FLAME_SPLASH, "Sticky Flame Splash",
      SPTYP_CONJURATION | SPTYP_FIRE,
@@ -2372,8 +2408,8 @@
 
 {
     SPELL_PORKALATOR, "Porkalator",
-     SPTYP_ENCHANTMENT | SPTYP_TRANSMUTATION,
-     SPFLAG_DIR_OR_TARGET,
+     SPTYP_HEXES | SPTYP_TRANSMUTATION,
+     SPFLAG_DIR_OR_TARGET | SPFLAG_CHAOTIC,
      5,
      200,
      LOS_RADIUS, LOS_RADIUS,
@@ -2424,7 +2460,7 @@
 
 {
     SPELL_HASTE_OTHER, "Haste Other",
-     SPTYP_ENCHANTMENT,
+     SPTYP_HEXES,
      SPFLAG_DIR_OR_TARGET | SPFLAG_HELPFUL | SPFLAG_HASTY,
      6,
      200,
@@ -2474,9 +2510,22 @@
      false
 },
 
+{
+    SPELL_IRON_ELEMENTALS, "Summon Iron Elementals",
+     SPTYP_SUMMONING,
+     SPFLAG_MONSTER,
+     5,
+     0,
+     -1, -1,
+     0,
+     NULL,
+     false,
+     false
+},
+
 {
     SPELL_SLEEP, "Sleep",
-     SPTYP_ENCHANTMENT,
+     SPTYP_HEXES,
      SPFLAG_DIR_OR_TARGET | SPFLAG_NOT_SELF,
      5,
      200,
@@ -2515,7 +2564,7 @@
 
 {
     SPELL_MISLEAD, "Mislead",
-     SPTYP_ENCHANTMENT,
+     SPTYP_HEXES,
      SPFLAG_TARGET | SPFLAG_NOT_SELF,
      5,
      200,
@@ -2591,6 +2640,318 @@
      true
 },
 
+{
+    SPELL_MIGHT, "Might",
+     SPTYP_CHARMS,
+     SPFLAG_DIR_OR_TARGET | SPFLAG_HELPFUL,
+     3,
+     200,
+     LOS_RADIUS, LOS_RADIUS,
+     0,
+     NULL,
+     false,
+     true
+},
+
+{
+    SPELL_SUNRAY, "Sunray",
+     SPTYP_CONJURATION,
+     SPFLAG_DIR_OR_TARGET,
+     6,
+     200,
+     9, 9,
+     -9,
+     NULL,
+     true,
+     false
+},
+
+{
+    SPELL_AWAKEN_FOREST, "Awaken Forest",
+     SPTYP_HEXES,
+     SPFLAG_AREA,
+     6,
+     200,
+     LOS_RADIUS, LOS_RADIUS,
+     0,
+     NULL,
+     false,
+     false
+},
+
+{
+    SPELL_SUMMON_CANIFORMS, "Summon Caniforms",
+     SPTYP_SUMMONING,
+     SPFLAG_MONSTER,
+     6,
+     0,
+     -1, -1,
+     0,
+     NULL,
+     false,
+     false
+},
+
+{
+    SPELL_BROTHERS_IN_ARMS, "Brothers in Arms",
+     SPTYP_SUMMONING,
+     SPFLAG_MONSTER,
+     6,
+     0,
+     -1, -1,
+     0,
+     NULL,
+     false,
+     false
+},
+
+{
+    SPELL_TROGS_HAND, "Trog's Hand",
+     SPTYP_NONE,
+     SPFLAG_MONSTER,
+     3,
+     0,
+     -1, -1,
+     0,
+     NULL,
+     false,
+     false
+},
+
+{
+    SPELL_SUMMON_SPECTRAL_ORCS, "Summon Spectral Orcs",
+     SPTYP_NECROMANCY,
+     SPFLAG_MONSTER | SPFLAG_TARGET,
+     4,
+     0,
+     LOS_RADIUS, LOS_RADIUS,
+     1,
+     NULL,
+     false,
+     false,
+},
+
+{
+    SPELL_RESURRECT, "Resurrection",
+     SPTYP_HOLY,
+     SPFLAG_DIR_OR_TARGET | SPFLAG_NOT_SELF | SPFLAG_HELPFUL,
+     6,
+     200,
+     LOS_RADIUS, LOS_RADIUS,
+     0,
+     NULL,
+     true,
+     false
+},
+
+{
+    SPELL_LIGHT_BEAM, "Light Beam",
+     SPTYP_CONJURATION | SPTYP_HOLY,
+     SPFLAG_DIR_OR_TARGET,
+     6,
+     200,
+     6, 6,
+     0,
+     NULL,
+     true,
+     false
+},
+
+{
+    SPELL_SUMMON_HOLIES, "Summon Holies",
+     SPTYP_SUMMONING | SPTYP_HOLY,
+     SPFLAG_MONSTER,
+     5,
+     0,
+     -1, -1,
+     0,
+     NULL,
+     false,
+     false
+},
+
+{
+    SPELL_SUMMON_GREATER_HOLY, "Summon Greater Holy",
+     SPTYP_SUMMONING | SPTYP_HOLY,
+     SPFLAG_MONSTER,
+     9,
+     0,
+     -1, -1,
+     0,
+     NULL,
+     false,
+     false
+},
+
+{
+    SPELL_HOLY_WORD, "Holy Word",
+     SPTYP_HOLY,
+     SPFLAG_AREA,
+     6,
+     0,
+     -1, -1,
+     0,
+     NULL,
+     false,
+     false
+},
+
+{
+    SPELL_HEAL_OTHER, "Heal Other",
+     SPTYP_NONE,
+     SPFLAG_DIR_OR_TARGET | SPFLAG_HELPFUL,
+     6,
+     200,
+     LOS_RADIUS, LOS_RADIUS,
+     0,
+     NULL,
+     true,
+     false
+},
+
+{
+    SPELL_SACRIFICE, "Sacrifice",
+     SPTYP_HOLY,
+     SPFLAG_DIR_OR_TARGET | SPFLAG_NOT_SELF | SPFLAG_HELPFUL,
+     6,
+     200,
+     LOS_RADIUS, LOS_RADIUS,
+     0,
+     NULL,
+     true,
+     false
+},
+
+{
+    SPELL_HOLY_FLAMES, "Holy Flames",
+     SPTYP_HOLY,
+     SPFLAG_TARGET | SPFLAG_NOT_SELF | SPFLAG_BATTLE,
+     7,
+     200,
+     LOS_RADIUS, LOS_RADIUS,
+     0,
+     NULL,
+     false,
+     false
+},
+
+{
+    SPELL_HOLY_BREATH, "Holy Breath",
+     SPTYP_CONJURATION | SPTYP_HOLY,
+     SPFLAG_DIR_OR_TARGET | SPFLAG_AREA,
+     5,
+     200,
+     6, 6,
+     0,
+     NULL,
+     true,
+     false
+},
+
+{
+    SPELL_MIRROR_DAMAGE, "Mirror Damage",
+     SPTYP_NONE,
+     SPFLAG_DIR_OR_TARGET | SPFLAG_HELPFUL,
+     4,
+     200,
+     LOS_RADIUS, LOS_RADIUS,
+     0,
+     NULL,
+     false,
+     true
+},
+
+{
+    SPELL_DRAIN_LIFE, "Drain Life",
+     SPTYP_NECROMANCY,
+     SPFLAG_AREA,
+     6,
+     0,
+     -1, -1,
+     0,
+     NULL,
+     false,
+     false
+},
+
+{
+    SPELL_LEDAS_LIQUEFACTION, "Leda's Liquefaction",
+     SPTYP_EARTH | SPTYP_TRANSMUTATION,
+     SPFLAG_AREA,
+     4,
+     200,
+     -1, -1,
+     0,
+     NULL,
+     false,
+     false
+},
+
+{
+    SPELL_SUMMON_HYDRA, "Summon Hydra",
+     SPTYP_SUMMONING,
+     SPFLAG_NONE,
+     7,
+     200,
+     -1, -1,
+     0,
+     NULL,
+     false,
+     false
+},
+
+{
+    SPELL_DARKNESS, "Darkness",
+     SPTYP_HEXES,
+     SPFLAG_NONE,
+     7,
+     200,
+     -1, -1,
+     0,
+     NULL,
+     false,
+     false
+},
+
+{
+    SPELL_MESMERISE, "Mesmerise",
+     SPTYP_HEXES,
+     SPFLAG_AREA,
+     5,
+     200,
+     LOS_RADIUS, LOS_RADIUS,
+     0,
+     NULL,
+     false,
+     false
+},
+
+{
+    SPELL_MELEE, "melee",
+     0,
+     SPFLAG_TESTING,
+     1,
+     0,
+     -1, -1,
+     0,
+     NULL,
+     false,
+     false
+},
+
+{
+    SPELL_FIRE_SUMMON, "Fire Summon",
+     SPTYP_SUMMONING | SPTYP_FIRE,
+     SPFLAG_MONSTER,
+     8,
+     0,
+     -1, -1,
+     0,
+     NULL,
+     false,
+     false
+},
+
 {
     SPELL_NO_SPELL, "nonexistent spell",
      0,
