@@ -1,8 +1,7 @@
-/*
- *  File:       spl-util.cc                                          *
- *  Summary:    data handlers for player-available spell list        *
- *  Written by: don brodale <dbrodale@bigfootinteractive.com>        *
- */
+/**
+ * @file
+ * @brief data handlers for player-available spell list
+**/
 
 #include "AppHdr.h"
 
@@ -19,6 +18,7 @@
 #include "externs.h"
 
 #include "beam.h"
+#include "coord.h"
 #include "coordit.h"
 #include "directn.h"
 #include "debug.h"
@@ -26,17 +26,16 @@
 #include "stuff.h"
 #include "env.h"
 #include "libutil.h"
-#include "macro.h"
 #include "mon-behv.h"
 #include "mon-util.h"
 #include "notes.h"
 #include "player.h"
 #include "religion.h"
-#include "spells4.h"
 #include "spl-cast.h"
 #include "spl-book.h"
+#include "spl-damage.h"
+#include "spl-zap.h"
 #include "terrain.h"
-#include "itemprop.h"
 #include "item_use.h"
 #include "transform.h"
 
@@ -48,13 +47,17 @@ struct spell_desc
     unsigned int disciplines; // bitfield
     unsigned int flags;       // bitfield
     unsigned int level;
+
+    // Usually in the range 0..200 (0 means uncapped).
+    // Note that some spells are also capped through zap_type.
+    // See spell_power_cap below.
     int power_cap;
 
     // At power 0, you get min_range. At power power_cap, you get max_range.
     int min_range;
     int max_range;
 
-    // How much louder or quieter the spell is than the default.
+    // Modify spell level for spell noise purposes.
     int noise_mod;
 
     const char  *target_prompt;
@@ -66,8 +69,9 @@ struct spell_desc
     bool         ms_utility;
 };
 
-static struct spell_desc spelldata[] = {
-#include "spl-data.h"
+static struct spell_desc spelldata[] =
+{
+    #include "spl-data.h"
 };
 
 static int spell_list[NUM_SPELLS];
@@ -77,8 +81,7 @@ static int spell_list[NUM_SPELLS];
 static struct spell_desc *_seekspell(spell_type spellid);
 static bool _cloud_helper(cloud_func func, const coord_def& where,
                           int pow, int spread_rate,
-                          cloud_type ctype, kill_category whose,
-                          killer_type killer, int colour,
+                          cloud_type ctype, const actor *agent, int colour,
                           std::string name, std::string tile);
 
 //
@@ -99,7 +102,7 @@ void init_spell_descs(void)
         if (data.id < SPELL_NO_SPELL || data.id >= NUM_SPELLS)
             end(1, false, "spell #%d has invalid id %d", i, data.id);
 
-        if (data.title == NULL || strlen(data.title) == 0)
+        if (data.title == NULL || !*data.title)
             end(1, false, "spell #%d, id %d has no name", i, data.id);
 
         if (data.level < 1 || data.level > 9)
@@ -238,11 +241,11 @@ spschool_flag_type school_by_name(std::string name)
    return SPTYP_NONE;
 }
 
-int get_spell_slot_by_letter( char letter )
+int get_spell_slot_by_letter(char letter)
 {
-    ASSERT( isalpha( letter ) );
+    ASSERT(isaalpha(letter));
 
-    const int index = letter_to_index( letter );
+    const int index = letter_to_index(letter);
 
     if (you.spell_letter_table[ index ] == -1)
         return (-1);
@@ -250,21 +253,35 @@ int get_spell_slot_by_letter( char letter )
     return (you.spell_letter_table[index]);
 }
 
-spell_type get_spell_by_letter( char letter )
+int get_spell_slot(spell_type spell)
 {
-    ASSERT( isalpha( letter ) );
+    for (int i = 0; i < MAX_KNOWN_SPELLS; i++)
+        if (you.spells[i] == spell)
+            return i;
 
-    const int slot = get_spell_slot_by_letter( letter );
+    return -1;
+}
+
+int get_spell_letter(spell_type spell)
+{
+    return index_to_letter(get_spell_slot(spell));
+}
+
+spell_type get_spell_by_letter(char letter)
+{
+    ASSERT(isaalpha(letter));
+
+    const int slot = get_spell_slot_by_letter(letter);
 
     return ((slot == -1) ? SPELL_NO_SPELL : you.spells[slot]);
 }
 
-bool add_spell_to_memory( spell_type spell )
+bool add_spell_to_memory(spell_type spell)
 {
     int i, j;
 
     // first we find a slot in our head:
-    for (i = 0; i < 25; i++)
+    for (i = 0; i < MAX_KNOWN_SPELLS; i++)
     {
         if (you.spells[i] == SPELL_NO_SPELL)
             break;
@@ -285,13 +302,23 @@ bool add_spell_to_memory( spell_type spell )
 
     take_note(Note(NOTE_LEARN_SPELL, spell));
 
+#ifdef USE_TILE
+    tiles.layout_statcol();
+    redraw_screen();
+#endif
+
     return (true);
 }
 
-bool del_spell_from_memory_by_slot( int slot )
+bool del_spell_from_memory_by_slot(int slot)
 {
+    ASSERT(slot >= 0 && slot < MAX_KNOWN_SPELLS);
     int j;
 
+    if (you.last_cast_spell == you.spells[slot])
+        you.last_cast_spell = SPELL_NO_SPELL;
+
+    mprf("Your memory of %s unravels.", spell_title(you.spells[slot]));
     you.spells[ slot ] = SPELL_NO_SPELL;
 
     for (j = 0; j < 52; j++)
@@ -303,11 +330,24 @@ bool del_spell_from_memory_by_slot( int slot )
 
     you.spell_no--;
 
+#ifdef USE_TILE
+    tiles.layout_statcol();
+    redraw_screen();
+#endif
+
     return (true);
 }
 
+bool del_spell_from_memory(spell_type spell)
+{
+    int i = get_spell_slot(spell);
+    if (i == -1)
+        return (false);
+    else
+        return del_spell_from_memory_by_slot(i);
+}
 
-int spell_hunger(spell_type which_spell)
+int spell_hunger(spell_type which_spell, bool rod)
 {
     const int level = spell_difficulty(which_spell);
 
@@ -322,7 +362,13 @@ int spell_hunger(spell_type which_spell)
     else
         hunger = (basehunger[0] * level * level) / 4;
 
-    hunger -= you.intel * you.skills[SK_SPELLCASTING];
+    if (rod)
+    {
+        hunger -= 10 * you.skill(SK_EVOCATIONS);
+        hunger = std::max(hunger, level * 5);
+    }
+    else
+        hunger -= you.intel() * you.skill(SK_SPELLCASTING);
 
     if (hunger < 0)
         hunger = 0;
@@ -405,14 +451,14 @@ int spell_difficulty(spell_type which_spell)
     return (_seekspell(which_spell)->level);
 }
 
-int spell_levels_required( spell_type which_spell )
+int spell_levels_required(spell_type which_spell)
 {
-    int levels = spell_difficulty( which_spell );
+    int levels = spell_difficulty(which_spell);
 
     if (which_spell == SPELL_DELAYED_FIREBALL
         && you.has_spell(SPELL_FIREBALL))
     {
-        levels -= spell_difficulty( SPELL_FIREBALL );
+        levels -= spell_difficulty(SPELL_FIREBALL);
     }
     else if (which_spell == SPELL_FIREBALL
             && you.has_spell(SPELL_DELAYED_FIREBALL))
@@ -423,25 +469,29 @@ int spell_levels_required( spell_type which_spell )
     return (levels);
 }
 
-unsigned int get_spell_flags( spell_type which_spell )
+unsigned int get_spell_flags(spell_type which_spell)
 {
     return (_seekspell(which_spell)->flags);
 }
 
-const char *get_spell_target_prompt( spell_type which_spell )
+const char *get_spell_target_prompt(spell_type which_spell)
 {
     return (_seekspell(which_spell)->target_prompt);
 }
 
 bool spell_typematch(spell_type which_spell, unsigned int which_discipline)
 {
-    return (_seekspell(which_spell)->disciplines & which_discipline);
+    return (get_spell_disciplines(which_spell) & which_discipline);
 }
 
 //jmf: next two for simple bit handling
 unsigned int get_spell_disciplines(spell_type spell)
 {
-    return (_seekspell(spell)->disciplines);
+    unsigned int dis = _seekspell(spell)->disciplines;
+    if (spell == SPELL_DRAGON_FORM && player_genus(GENPC_DRACONIAN))
+        dis &= (~SPTYP_FIRE);
+
+    return dis;
 }
 
 int count_bits(unsigned int bits)
@@ -462,7 +512,7 @@ bool disciplines_conflict(unsigned int disc1, unsigned int disc2)
 {
     const unsigned int combined = disc1 | disc2;
 
-    return (   (combined & SPTYP_EARTH) && (combined & SPTYP_AIR)
+    return ((combined & SPTYP_EARTH) && (combined & SPTYP_AIR)
             || (combined & SPTYP_FIRE)  && (combined & SPTYP_ICE)
             || (combined & SPTYP_HOLY)  && (combined & SPTYP_NECROMANCY));
 }
@@ -482,14 +532,22 @@ const char *spell_title(spell_type spell)
 // Apply a function-pointer to all visible squares
 // Returns summation of return values from passed in function.
 int apply_area_visible(cell_func cf, int power,
-                       bool pass_through_trans, actor *agent)
+                       bool pass_through_trans, actor *agent,
+                       bool affect_scryed)
 {
     int rv = 0;
 
+    bool xray = you.xray_vision;
+
+    if (!affect_scryed)
+        you.xray_vision = false;
+
     for (radius_iterator ri(you.pos(), LOS_RADIUS); ri; ++ri)
         if (pass_through_trans || you.see_cell_no_trans(*ri))
             rv += cf(*ri, power, 0, agent);
 
+    you.xray_vision = xray;
+
     return (rv);
 }
 
@@ -520,6 +578,26 @@ int apply_area_around_square(cell_func cf, const coord_def& where, int power,
     return (rv);
 }
 
+// Like apply_area_around_square, but for monsters in those squares,
+// and takes care not to affect monsters twice that change position.
+int apply_monsters_around_square(monster_func mf, const coord_def& where,
+                                  int power)
+{
+    int rv = 0;
+    std::set<const monster*> affected;
+    for (adjacent_iterator ai(where, true); ai; ++ai)
+    {
+        monster* mon = monster_at(*ai);
+        if (mon && affected.find(mon) == affected.end())
+        {
+            rv += mf(mon, power);
+            affected.insert(mon);
+        }
+    }
+
+   return (rv);
+}
+
 // Affect up to max_targs monsters around a point, chosen randomly.
 // Return varies with the function called; return values will be added up.
 int apply_random_around_square(cell_func cf, const coord_def& where,
@@ -683,17 +761,20 @@ int apply_area_within_radius(cell_func cf, const coord_def& where,
 // We really need some sort of a queue structure, since ideally I'd like
 // to do a (shallow) breadth-first-search of the dungeon floor.
 // This ought to work okay for small clouds.
-void apply_area_cloud( cloud_func func, const coord_def& where,
+void apply_area_cloud(cloud_func func, const coord_def& where,
                        int pow, int number, cloud_type ctype,
-                       kill_category whose, killer_type killer,
+                       const actor *agent,
                        int spread_rate, int colour, std::string name,
                        std::string tile)
 {
+    if (!in_bounds(where))
+        return;
+
     int good_squares = 0;
     int neighbours[8] = { 0, 0, 0, 0, 0, 0, 0, 0 };
 
-    if (number && _cloud_helper(func, where, pow, spread_rate, ctype, whose,
-                                killer, colour, name, tile))
+    if (number && _cloud_helper(func, where, pow, spread_rate, ctype, agent,
+                                colour, name, tile))
         number--;
 
     if (number == 0)
@@ -705,15 +786,15 @@ void apply_area_cloud( cloud_func func, const coord_def& where,
 
     int* const arrs[2] = { compass_order_orth, compass_order_diag };
 
-    for ( int m = 0; m < 2; ++m )
+    for (int m = 0; m < 2; ++m)
     {
         // Randomise, but do orthogonals first and diagonals later.
-        std::random_shuffle( arrs[m], arrs[m] + 4 );
-        for ( int i = 0; i < 4 && number; ++i )
+        std::random_shuffle(arrs[m], arrs[m] + 4);
+        for (int i = 0; i < 4 && number; ++i)
         {
             const int aux = arrs[m][i];
-            if ( _cloud_helper(func, where + Compass[aux],
-                               pow, spread_rate, ctype, whose, killer, colour,
+            if (_cloud_helper(func, where + Compass[aux],
+                               pow, spread_rate, ctype, agent, colour,
                                name, tile))
             {
                 number--;
@@ -725,7 +806,7 @@ void apply_area_cloud( cloud_func func, const coord_def& where,
 
     // Get a random permutation.
     int perm[8];
-    for ( int i = 0; i < 8; ++i )
+    for (int i = 0; i < 8; ++i)
         perm[i] = i;
     std::random_shuffle(perm, perm+8);
     for (int i = 0; i < 8 && number; i++)
@@ -739,8 +820,8 @@ void apply_area_cloud( cloud_func func, const coord_def& where,
         int spread = number / good_squares;
         number -= spread;
         good_squares--;
-        apply_area_cloud(func, where + Compass[j], pow, spread, ctype, whose,
-                         killer, spread_rate, colour, name, tile);
+        apply_area_cloud(func, where + Compass[j], pow, spread, ctype, agent,
+                         spread_rate, colour, name, tile);
     }
 }
 
@@ -748,12 +829,13 @@ void apply_area_cloud( cloud_func func, const coord_def& where,
 // Return false if the user canceled, true otherwise.
 // FIXME: this should accept a direction_chooser_args directly rather
 // than move the arguments into one.
-bool spell_direction( dist &spelld, bolt &pbolt,
+bool spell_direction(dist &spelld, bolt &pbolt,
                       targeting_type restrict, targ_mode_type mode,
                       int range,
                       bool needs_path, bool may_target_monster,
                       bool may_target_self, const char *target_prefix,
-                      const char* top_prompt, bool cancel_at_self )
+                      const char* top_prompt, bool cancel_at_self,
+                      targetter *hitfunc)
 {
     if (range < 1)
         range = (pbolt.range < 1) ? LOS_RADIUS : pbolt.range;
@@ -771,6 +853,7 @@ bool spell_direction( dist &spelld, bolt &pbolt,
         args.top_prompt = top_prompt;
     args.behaviour = NULL;
     args.cancel_at_self = cancel_at_self;
+    args.hitfunc = hitfunc;
 
     direction(spelld, args);
 
@@ -787,14 +870,16 @@ bool spell_direction( dist &spelld, bolt &pbolt,
     return (true);
 }
 
-const char* spelltype_short_name( int which_spelltype )
+const char* spelltype_short_name(int which_spelltype)
 {
     switch (which_spelltype)
     {
     case SPTYP_CONJURATION:
         return ("Conj");
-    case SPTYP_ENCHANTMENT:
-        return ("Ench");
+    case SPTYP_HEXES:
+        return ("Hex");
+    case SPTYP_CHARMS:
+        return ("Chrm");
     case SPTYP_FIRE:
         return ("Fire");
     case SPTYP_ICE:
@@ -824,14 +909,16 @@ const char* spelltype_short_name( int which_spelltype )
     }
 }
 
-const char* spelltype_long_name( int which_spelltype )
+const char* spelltype_long_name(int which_spelltype)
 {
     switch (which_spelltype)
     {
     case SPTYP_CONJURATION:
         return ("Conjuration");
-    case SPTYP_ENCHANTMENT:
-        return ("Enchantment");
+    case SPTYP_HEXES:
+        return ("Hexes");
+    case SPTYP_CHARMS:
+        return ("Charms");
     case SPTYP_FIRE:
         return ("Fire");
     case SPTYP_ICE:
@@ -861,12 +948,13 @@ const char* spelltype_long_name( int which_spelltype )
     }
 }
 
-int spell_type2skill(unsigned int spelltype)
+skill_type spell_type2skill(unsigned int spelltype)
 {
     switch (spelltype)
     {
     case SPTYP_CONJURATION:    return (SK_CONJURATIONS);
-    case SPTYP_ENCHANTMENT:    return (SK_ENCHANTMENTS);
+    case SPTYP_HEXES:          return (SK_HEXES);
+    case SPTYP_CHARMS:         return (SK_CHARMS);
     case SPTYP_FIRE:           return (SK_FIRE_MAGIC);
     case SPTYP_ICE:            return (SK_ICE_MAGIC);
     case SPTYP_TRANSMUTATION:  return (SK_TRANSMUTATIONS);
@@ -882,9 +970,9 @@ int spell_type2skill(unsigned int spelltype)
     case SPTYP_DIVINATION:
 #ifdef DEBUG_DIAGNOSTICS
         mprf(MSGCH_DIAGNOSTICS, "spell_type2skill: called with spelltype %u",
-             spelltype );
+             spelltype);
 #endif
-        return (-1);
+        return (SK_NONE);
     }
 }                               // end spell_type2skill()
 
@@ -914,23 +1002,46 @@ bool is_valid_spell(spell_type spell)
 
 static bool _cloud_helper(cloud_func func, const coord_def& where,
                           int pow, int spread_rate,
-                          cloud_type ctype, kill_category whose,
-                          killer_type killer, int colour, std::string name,
-                          std::string tile)
+                          cloud_type ctype, const actor* agent, int colour,
+                          std::string name, std::string tile)
 {
-    if (!feat_is_solid(grd(where)) && env.cgrid(where) == EMPTY_CLOUD)
+    if (in_bounds(where)
+        && !feat_is_solid(grd(where))
+        && env.cgrid(where) == EMPTY_CLOUD)
     {
-        func(where, pow, spread_rate, ctype, whose, killer, colour, name,
-             tile);
+        func(where, pow, spread_rate, ctype, agent, colour, name, tile);
         return (true);
     }
 
     return (false);
 }
 
+static bool _spell_range_varies(spell_type spell)
+{
+    int minrange = _seekspell(spell)->min_range;
+    int maxrange = _seekspell(spell)->max_range;
+
+    return (minrange < maxrange);
+}
+
 int spell_power_cap(spell_type spell)
 {
-    return (_seekspell(spell)->power_cap);
+    const int scap = _seekspell(spell)->power_cap;
+    const int zcap = spell_zap_power_cap(spell);
+
+    if (scap == 0)
+        return (zcap);
+    else if (zcap == 0)
+        return (scap);
+    else
+    {
+        // Two separate power caps; pre-zapping spell power
+        // goes into range.
+        if (scap <= zcap || _spell_range_varies(spell))
+            return (scap);
+        else
+            return (zcap);
+    }
 }
 
 int spell_range(spell_type spell, int pow, bool real_cast, bool player_spell)
@@ -953,6 +1064,8 @@ int spell_range(spell_type spell, int pow, bool real_cast, bool player_spell)
     if (player_spell
         && vehumet_supports_spell(spell)
         && you.religion == GOD_VEHUMET
+        && spell != SPELL_STICKY_FLAME
+        && spell != SPELL_FREEZE
         && !player_under_penance()
         && you.piety >= piety_breakpoint(2))
     {
@@ -975,13 +1088,37 @@ int spell_range(spell_type spell, int pow, bool real_cast, bool player_spell)
     return ((pow * (maxrange - minrange) + powercap / 2) / powercap + minrange);
 }
 
+/**
+ * Spell casting noise.
+ *
+ * Returns the noise generated by the casting of a spell. The noise depends on
+ * the spell schools and level. A modifier (noise_mod) can be applied to the
+ * spell level.
+ * @see spl-data.h
+ *
+ * \param spell  The spell being casted.
+ * \return       The amount of noise generated.
+**/
 int spell_noise(spell_type spell)
 {
     const spell_desc *desc = _seekspell(spell);
 
-    return desc->noise_mod + spell_noise(desc->disciplines, desc->level);
+    return spell_noise(desc->disciplines, desc->level + desc->noise_mod);
 }
 
+/**
+ * Spell default noise.
+ *
+ * Default value for spell noise given a level and a set of schools.
+ * Formula (use first match):
+ * - Conjuration (noisy)    = \f$ level \f$
+ * - Air and poison (quiet) = \f$ \frac{level}{2} \f$
+ * - Other (normal)         = \f$ \frac{3 \times level}{4} \f$
+ *
+ * \param disciplines  An integer which contain the school flags.
+ * \param level        The level of the spell.
+ * \return             The amount of noise generated.
+**/
 int spell_noise(unsigned int disciplines, int level)
 {
     if (disciplines == SPTYP_NONE)
@@ -996,57 +1133,57 @@ int spell_noise(unsigned int disciplines, int level)
 
 spell_type zap_type_to_spell(zap_type zap)
 {
-    switch(zap)
+    switch (zap)
     {
     case ZAP_FLAME:
-        return(SPELL_THROW_FLAME);
+        return SPELL_THROW_FLAME;
     case ZAP_FROST:
-        return(SPELL_THROW_FROST);
+        return SPELL_THROW_FROST;
     case ZAP_SLOWING:
-        return(SPELL_SLOW);
+        return SPELL_SLOW;
     case ZAP_HASTING:
-        return(SPELL_HASTE);
+        return SPELL_HASTE;
     case ZAP_MAGIC_DARTS:
-        return(SPELL_MAGIC_DART);
+        return SPELL_MAGIC_DART;
     case ZAP_HEALING:
-        return(SPELL_MAJOR_HEALING);
+        return SPELL_MAJOR_HEALING;
     case ZAP_PARALYSIS:
-        return(SPELL_PARALYSE);
+        return SPELL_PARALYSE;
     case ZAP_FIRE:
-        return(SPELL_BOLT_OF_FIRE);
+        return SPELL_BOLT_OF_FIRE;
     case ZAP_COLD:
-        return(SPELL_BOLT_OF_COLD);
+        return SPELL_BOLT_OF_COLD;
     case ZAP_PRIMAL_WAVE:
-        return(SPELL_PRIMAL_WAVE);
+        return SPELL_PRIMAL_WAVE;
     case ZAP_CONFUSION:
-        return(SPELL_CONFUSE);
+        return SPELL_CONFUSE;
     case ZAP_INVISIBILITY:
-        return(SPELL_INVISIBILITY);
+        return SPELL_INVISIBILITY;
     case ZAP_DIGGING:
-        return(SPELL_DIG);
+        return SPELL_DIG;
     case ZAP_FIREBALL:
-        return(SPELL_FIREBALL);
+        return SPELL_FIREBALL;
     case ZAP_TELEPORTATION:
-        return(SPELL_TELEPORT_OTHER);
+        return SPELL_TELEPORT_OTHER;
     case ZAP_LIGHTNING:
-        return(SPELL_LIGHTNING_BOLT);
+        return SPELL_LIGHTNING_BOLT;
     case ZAP_POLYMORPH_OTHER:
-        return(SPELL_POLYMORPH_OTHER);
+        return SPELL_POLYMORPH_OTHER;
     case ZAP_NEGATIVE_ENERGY:
-        return(SPELL_BOLT_OF_DRAINING);
+        return SPELL_BOLT_OF_DRAINING;
     case ZAP_ENSLAVEMENT:
-        return(SPELL_ENSLAVEMENT);
+        return SPELL_ENSLAVEMENT;
     case ZAP_DISINTEGRATION:
-        return(SPELL_DISINTEGRATE);
+        return SPELL_DISINTEGRATE;
     default:
-        DEBUGSTR("zap_type_to_spell() only handles wand zaps for now");
+        die("zap_type_to_spell() only handles wand zaps for now");
     }
     return SPELL_NO_SPELL;
 }
 
 bool spell_is_empowered(spell_type spell)
 {
-    if( (you.religion == GOD_VEHUMET)
+    if ((you.religion == GOD_VEHUMET)
         && vehumet_supports_spell(spell)
         && piety_rank() > 2)
     {
@@ -1059,119 +1196,72 @@ bool spell_is_empowered(spell_type spell)
         // looking at player_movement_speed, this should be correct ~DMB
         if (player_movement_speed() > 6
             && you.duration[DUR_CONTROLLED_FLIGHT] > 0
-            && you.duration[DUR_SWIFTNESS]  < 1)
+            && you.duration[DUR_SWIFTNESS] < 1)
+        {
             return (true);
+        }
         break;
     case SPELL_STONESKIN:
         if (you.duration[DUR_TRANSFORMATION] > 0
-            && you.attribute[ATTR_TRANSFORMATION] == TRAN_STATUE
+            && you.form == TRAN_STATUE
             && you.duration[DUR_STONESKIN] < 1)
+        {
             return (true);
+        }
         break;
     case SPELL_OZOCUBUS_ARMOUR:
         if (you.duration[DUR_TRANSFORMATION] > 0
-            && you.attribute[ATTR_TRANSFORMATION] == TRAN_ICE_BEAST
+            && you.form == TRAN_ICE_BEAST
             && you.duration[DUR_ICY_ARMOUR] < 1)
-            return (true);
-        break;
-    default:
-        break;
-    }
-
-    return (false);
-}
-
-bool spell_is_useful(spell_type spell)
-{
-    if (you_cannot_memorise(spell) || god_hates_spell(spell, you.religion))
-        return (false);
-    if (you.duration[DUR_CONF] > 0)
-        return (false);
-    if (spell_is_empowered(spell))
-        return (true);
-
-    // due to the way this function is used, you should generally try to be
-    // fairly specific about the circumstances in which a spell is "useful".
-    switch (spell)
-    {
-    case SPELL_CONTROL_TELEPORT:
-        if (you.duration[DUR_TELEPORT] > 0
-            && you.duration[DUR_CONTROL_TELEPORT] < 1)
         {
             return (true);
         }
         break;
-    case SPELL_HASTE:
-        if(you.duration[DUR_SLOW] > 0
-           && you.duration[DUR_HASTE] <= 0)
-        {
-            return (true);
-        }
-    case SPELL_BLINK:
-    case SPELL_CONTROLLED_BLINK:
-    case SPELL_TELEPORT_SELF:
-        return (true);
-    default: // quash unhandled constants warnings
-        break;
-    }
-
-    return (false);
-}
-
-bool spell_is_risky(spell_type spell)
-{
-    if (god_hates_spell(spell, you.religion))
-        return (true);
-
-    switch(spell)
-    {
-    case SPELL_ALTER_SELF:
-        return (true);
     default:
         break;
-    };
+    }
 
     return (false);
 }
 
 // This function attempts to determine if 'spell' is useless to
 // the player. if 'transient' is true, then it will include checks
-// for volitile or temporary states (such as status effects, mana, etc.)
+// for volatile or temporary states (such as status effects, mana, etc.)
 //
 // its notably used by 'spell_highlight_by_utility'
 bool spell_is_useless(spell_type spell, bool transient)
 {
     if (you_cannot_memorise(spell))
         return (true);
-    if(transient)
+
+    if (transient)
     {
-        if(you.duration[DUR_CONF] > 0
-          || (spell_mana(spell) > you.magic_points)
-          || spell_no_hostile_in_range(spell, get_dist_to_nearest_monster()))
+        if (you.duration[DUR_CONF] > 0
+            || spell_mana(spell) > you.magic_points
+            || spell_no_hostile_in_range(spell, get_dist_to_nearest_monster()))
+        {
             return (true);
+        }
     }
 
-    switch(spell)
+    switch (spell)
     {
     case SPELL_BLINK:
     case SPELL_CONTROLLED_BLINK:
     case SPELL_TELEPORT_SELF:
-        if (item_blocks_teleport(false, false) )
+        if (item_blocks_teleport(false, false))
             return true;
         break;
     case SPELL_SWIFTNESS:
         // looking at player_movement_speed, this should be correct ~DMB
-        if(player_movement_speed() <= 6)
+        if (player_movement_speed() <= 6)
             return (true);
         break;
     case SPELL_LEVITATION:
     case SPELL_FLY:
-        if (you.mutation[MUT_BIG_WINGS] >= 1
-            || (you.species == SP_KENKU && you.experience_level >= 5))
-        {
+        if (you.species == SP_KENKU && you.experience_level >= 15)
             return (true);
-        }
-        if (transient && you.duration[DUR_LEVITATION] > 0)
+        if (transient && you.is_levitating())
             return (true);
         break;
     case SPELL_REGENERATION:
@@ -1179,15 +1269,36 @@ bool spell_is_useless(spell_type spell, bool transient)
             return (true);
         break;
     case SPELL_INVISIBILITY:
-        if (transient
-           && (you.duration[DUR_INVIS] > 0
-              || you.backlit() ) )
+        if (transient && (you.duration[DUR_INVIS] > 0 || you.backlit()))
             return (true);
         break;
     case SPELL_CONTROL_TELEPORT:
         if (transient && you.duration[DUR_CONTROL_TELEPORT] > 0)
             return (true);
         break;
+    case SPELL_SEE_INVISIBLE:
+        if (you.can_see_invisible(false, false))
+            return (true);
+        break;
+    // weapon branding is useless
+    case SPELL_FIRE_BRAND:
+    case SPELL_FREEZING_AURA:
+    case SPELL_LETHAL_INFUSION:
+    case SPELL_WARP_BRAND:
+    case SPELL_EXCRUCIATING_WOUNDS:
+    case SPELL_POISON_WEAPON:
+    // could be useful if it didn't require wielding
+    case SPELL_TUKIMAS_DANCE:
+        if (you.species == SP_CAT)
+            return (true);
+        break;
+    case SPELL_DARKNESS:
+        // mere corona is not enough, but divine light blocks it completely
+        if (transient && you.haloed())
+            return true;
+        if (you.religion == GOD_SHINING_ONE && player_under_penance())
+            return true;
+        break;
     default:
         break; // quash unhandled constants warnings
     }
@@ -1195,13 +1306,6 @@ bool spell_is_useless(spell_type spell, bool transient)
     return (false);
 }
 
-bool spell_is_known(spell_type spell)
-{
-    for (int i = 0; i < 25; i++)
-        if(you.spells[i] == spell) return (true);
-    return (false);
-}
-
 // This function takes a spell, and determines what color it should be
 // highlighted with. You shouldn't have to touch this unless you want
 // to add new highlighting options.
@@ -1210,18 +1314,10 @@ bool spell_is_known(spell_type spell)
 //       god_hates_spell(spell, god)
 //       god_likes_spell(spell, god)
 //       spell_is_empowered(spell)
-//       spell_is_useful(spell)
 //       spell_is_useless(spell, transient)
-//       spell_is_risky(spell)
-//       spell_is_known(spell)
 int spell_highlight_by_utility(spell_type spell, int default_color,
-                               bool transient, bool force_known, bool rod_spell)
+                               bool transient, bool rod_spell)
 {
-    // if Force_known is true, and the spell is
-    // known, thats all that matters.
-    if (force_known && !spell_is_known(spell))
-        return COL_UNMEMORIZED;
-
     // If your god hates the spell, that
     // overrides all other concerns
     if (god_hates_spell(spell, you.religion))
@@ -1250,6 +1346,7 @@ bool spell_no_hostile_in_range(spell_type spell, int minRange)
     case SPELL_CONJURE_FLAME:
     case SPELL_DIG:
     case SPELL_PASSWALL:
+    case SPELL_GOLUBRIAS_PASSAGE:
 
     // Airstrike has LOS_RANGE and can go through glass walls.
     case SPELL_AIRSTRIKE:
@@ -1266,6 +1363,7 @@ bool spell_no_hostile_in_range(spell_type spell, int minRange)
     case SPELL_MEPHITIC_CLOUD:
     case SPELL_FIREBALL:
     case SPELL_FREEZING_CLOUD:
+    case SPELL_NOXIOUS_CLOUD:
     case SPELL_POISONOUS_CLOUD:
         // Increase range by one due to cloud radius.
         bonus = 1;
@@ -1283,9 +1381,9 @@ bool spell_no_hostile_in_range(spell_type spell, int minRange)
     if (range < 0)
         return (false);
 
-    if (range + bonus < minRange)
+    const int rsq = (range + bonus) * (range + bonus) + 1;
+    if (rsq < minRange)
         return (true);
 
     return (false);
 }
-
