@@ -1,8 +1,7 @@
-/*
- *  File:       stash.cc
- *  Summary:    Classes tracking player stashes
- *  Written by: Darshan Shaligram
- */
+/**
+ * @file
+ * @brief Classes tracking player stashes
+**/
 
 #include "AppHdr.h"
 
@@ -12,12 +11,14 @@
 #include "clua.h"
 #include "command.h"
 #include "coord.h"
+#include "coordit.h"
 #include "describe.h"
 #include "directn.h"
 #include "food.h"
 #include "itemname.h"
 #include "itemprop.h"
 #include "files.h"
+#include "godpassive.h"
 #include "invent.h"
 #include "items.h"
 #include "kills.h"
@@ -35,17 +36,19 @@
 #include "spl-book.h"
 #include "stash.h"
 #include "stuff.h"
+#include "syscalls.h"
 #include "env.h"
 #include "tags.h"
 #include "terrain.h"
 #include "traps.h"
 #include "travel.h"
-#include "tutorial.h"
+#include "hints.h"
+#include "unicode.h"
 #include "viewgeom.h"
+#include "viewmap.h"
 
 #include <cctype>
 #include <cstdio>
-#include <fstream>
 #include <sstream>
 #include <algorithm>
 
@@ -55,8 +58,8 @@ StashTracker StashTrack;
 void stash_init_new_level()
 {
     // If there's an existing stash level for Pan, blow it away.
-    StashTrack.remove_level( level_id(LEVEL_PANDEMONIUM) );
-    StashTrack.remove_level( level_id(LEVEL_PORTAL_VAULT) );
+    StashTrack.remove_level(level_id(LEVEL_PANDEMONIUM));
+    StashTrack.remove_level(level_id(LEVEL_PORTAL_VAULT));
 }
 
 std::string userdef_annotate_item(const char *s, const item_def *item,
@@ -81,26 +84,31 @@ std::string stash_annotate_item(const char *s,
                                 bool exclusive = false)
 {
     std::string text = userdef_annotate_item(s, item, exclusive);
-    if (item->base_type == OBJ_BOOKS
-            && item_type_known(*item)
-            && item->sub_type != BOOK_MANUAL
-            && item->sub_type != BOOK_DESTRUCTION
-        || count_staff_spells(*item, true) > 1)
+
+    if (item->has_spells())
     {
         formatted_string fs;
         item_def dup = *item;
         spellbook_contents(dup, item->base_type == OBJ_BOOKS ? RBOOK_READ_SPELL
                                                              : RBOOK_USE_STAFF,
                            &fs);
-        text += EOL;
+        text += "\n";
         text += fs.tostring(2, -2);
     }
+
+    // Include singular form (royal jelly vs royal jellies).
+    if (item->quantity > 1)
+    {
+        text += "\n";
+        text += item->name(DESC_BASENAME);
+    }
+
     return text;
 }
 
 void maybe_update_stashes()
 {
-    if (Options.stash_tracking && !crawl_state.arena)
+    if (Options.stash_tracking && !crawl_state.game_is_arena())
     {
         StashTrack.update_visible_stashes(
             Options.stash_tracking == STM_ALL ? StashTracker::ST_AGGRESSIVE
@@ -165,12 +173,12 @@ std::vector<item_def> item_list_in_stash(const coord_def& pos)
 
 static void _fully_identify_item(item_def *item)
 {
-    if (!item || !item->is_valid())
+    if (!item || !item->defined())
         return;
 
-    set_ident_flags( *item, ISFLAG_IDENT_MASK );
+    set_ident_flags(*item, ISFLAG_IDENT_MASK);
     if (item->base_type != OBJ_WEAPONS)
-        set_ident_type( *item, ID_KNOWN_TYPE );
+        set_ident_type(*item, ID_KNOWN_TYPE);
 }
 
 // ----------------------------------------------------------------------
@@ -188,8 +196,8 @@ Stash::Stash(int xp, int yp) : enabled(true), items()
         xp = you.pos().x;
         yp = you.pos().y;
     }
-    x = (unsigned char) xp;
-    y = (unsigned char) yp;
+    x = xp;
+    y = yp;
     abspos = GXM * (int) y + x;
 
     update();
@@ -219,7 +227,7 @@ void Stash::filter(const std::string &str)
 {
     std::string base = str;
 
-    unsigned char subc = 255;
+    uint8_t       subc = 255;
     std::string   subs = "";
     std::string::size_type cpos = base.find(":", 0);
     if (cpos != std::string::npos)
@@ -248,7 +256,7 @@ void Stash::filter(const std::string &str)
     }
 }
 
-void Stash::filter(object_class_type base, unsigned char sub)
+void Stash::filter(object_class_type base, uint8_t sub)
 {
     item_def item;
     item.base_type = base;
@@ -290,9 +298,9 @@ bool Stash::pickup_eligible() const
     return (false);
 }
 
-bool Stash::is_boring_feature(dungeon_feature_type feat)
+bool Stash::is_boring_feature(dungeon_feature_type feature)
 {
-    switch (feat)
+    switch (feature)
     {
     // Discard spammy dungeon features.
     case DNGN_SHALLOW_WATER:
@@ -312,14 +320,21 @@ bool Stash::is_boring_feature(dungeon_feature_type feat)
     case DNGN_UNDISCOVERED_TRAP:
         return (true);
     default:
-        return (feat_is_solid(feat));
+        return (feat_is_solid(feature));
     }
 }
 
 static bool _grid_has_mimic_item(const coord_def& pos)
 {
-    const monsters *mon = monster_at(pos);
-    return (mon && mons_is_unknown_mimic(mon));
+    const monster* mon = monster_at(pos);
+    return (mon && mons_is_unknown_mimic(mon) && mons_is_item_mimic(mon->type));
+}
+
+static bool _grid_has_mimic_shop(const coord_def& pos)
+{
+    const monster* mon = monster_at(pos);
+    return (mon && mons_is_unknown_mimic(mon)
+            && mon->type == MONS_SHOP_MIMIC);
 }
 
 static bool _grid_has_perceived_item(const coord_def& pos)
@@ -346,6 +361,10 @@ void Stash::update()
     feat = grd(p);
     trap = NUM_TRAPS;
 
+    const monster* mon = monster_at(p);
+    if ((mon && mons_is_unknown_mimic(mon) && mons_is_feat_mimic(mon->type)))
+        feat = get_mimic_feat(mon);
+
     if (is_boring_feature(feat))
         feat = DNGN_FLOOR;
 
@@ -382,7 +401,7 @@ void Stash::update()
         else
         {
             pitem = &mitm[you.visible_igrd(p)];
-            tutorial_first_item(*pitem);
+            hints_first_item(*pitem);
         }
 
         const item_def& item = *pitem;
@@ -515,7 +534,7 @@ void StashMenu::draw_title()
     {
         cgotoxy(1, 1);
         textcolor(title->colour);
-        cprintf( "%s", title->text.c_str());
+        cprintf("%s", title->text.c_str());
         if (title->quantity)
         {
             cprintf(", %d item%s", title->quantity,
@@ -565,7 +584,7 @@ unsigned char StashMenu::getkey() const
 
 static MenuEntry *stash_menu_fixup(MenuEntry *me)
 {
-    const item_def *item = static_cast<const item_def *>( me->data );
+    const item_def *item = static_cast<const item_def *>(me->data);
     if (item->base_type == OBJ_GOLD)
     {
         me->quantity = 0;
@@ -584,7 +603,7 @@ bool Stash::show_menu(const level_pos &prefix, bool can_travel) const
     menu.can_travel   = can_travel;
     mtitle->quantity  = items.size();
     menu.set_title(mtitle);
-    menu.load_items( InvMenu::xlat_itemvect(items), stash_menu_fixup);
+    menu.load_items(InvMenu::xlat_itemvect(items), stash_menu_fixup);
 
     std::vector<MenuEntry*> sel;
     while (true)
@@ -596,7 +615,7 @@ bool Stash::show_menu(const level_pos &prefix, bool can_travel) const
         if (sel.size() != 1)
             break;
 
-        item_def *item = static_cast<item_def *>( sel[0]->data );
+        item_def *item = static_cast<item_def *>(sel[0]->data);
         describe_item(*item);
     }
     return (false);
@@ -615,8 +634,8 @@ std::string Stash::description() const
     {
         char additionals[50];
         snprintf(additionals, sizeof additionals,
-                " (...%ld)",
-                 (unsigned long) (sz - 1));
+                " (...%u)",
+                 (unsigned int) (sz - 1));
         desc += additionals;
     }
     return (desc);
@@ -643,19 +662,21 @@ bool Stash::matches_search(const std::string &prefix,
         if (Stash::is_filtered(item))
             continue;
 
-        std::string s   = stash_item_name(item);
-        std::string ann = stash_annotate_item(STASH_LUA_SEARCH_ANNOTATE, &item);
+        const std::string s   = stash_item_name(item);
+        const std::string ann =
+            stash_annotate_item(STASH_LUA_SEARCH_ANNOTATE, &item);
         if (search.matches(prefix + " " + ann + s))
         {
             if (!res.count++)
                 res.match = s;
             res.matches += item.quantity;
+            res.set_matching_item(item);
             continue;
         }
 
         if (is_dumpable_artefact(item, false))
         {
-            std::string desc =
+            const std::string desc =
                 munge_description(get_item_description(item, false, true));
 
             if (search.matches(desc))
@@ -663,6 +684,7 @@ bool Stash::matches_search(const std::string &prefix,
                 if (!res.count++)
                     res.match = s;
                 res.matches += item.quantity;
+                res.set_matching_item(item);
             }
         }
     }
@@ -688,7 +710,7 @@ bool Stash::matches_search(const std::string &prefix,
     return !!res.matches;
 }
 
-void Stash::_update_corpses(long rot_time)
+void Stash::_update_corpses(int rot_time)
 {
     for (int i = items.size() - 1; i >= 0; i--)
     {
@@ -697,7 +719,7 @@ void Stash::_update_corpses(long rot_time)
         if (!_is_rottable(item))
             continue;
 
-        long new_rot = static_cast<long>(item.plus2) - rot_time;
+        int new_rot = static_cast<int>(item.plus2) - rot_time;
 
         if (new_rot <= _min_rot(item))
         {
@@ -708,6 +730,12 @@ void Stash::_update_corpses(long rot_time)
     }
 }
 
+void Stash::_update_identification()
+{
+    for (int i = items.size() - 1; i >= 0; i--)
+        ash_id_item(items[i]);
+}
+
 void Stash::add_item(const item_def &item, bool add_to_front)
 {
     if (_is_rottable(item))
@@ -737,7 +765,7 @@ void Stash::add_item(const item_def &item, bool add_to_front)
     }
 }
 
-void Stash::write(std::ostream &os, int refx, int refy,
+void Stash::write(FILE *f, int refx, int refy,
                   std::string place, bool identify)
     const
 {
@@ -747,10 +775,8 @@ void Stash::write(std::ostream &os, int refx, int refy,
     bool note_status = notes_are_active();
     activate_notes(false);
 
-    os << "(" << ((int) x - refx) << ", " << ((int) y - refy)
-       << (place.length()? ", " + place : "")
-       << ")"
-       << std::endl;
+    fprintf(f, "(%d, %d%s%s)\n", x - refx, y - refy,
+            place.empty() ? "" : ", ", OUTS(place));
 
     char buf[ITEMNAME_SIZE];
     for (int i = 0; i < (int) items.size(); ++i)
@@ -771,10 +797,8 @@ void Stash::write(std::ostream &os, int refx, int refy,
             ann = " " + ann;
         }
 
-        os << "  " << buf
-           << (!ann.empty()? ann : std::string())
-           << (!verified && (items.size() > 1 || i) ? " (still there?)" : "")
-           << std::endl;
+        fprintf(f, "  %s%s%s\n", OUTS(buf), OUTS(ann),
+            (!verified && (items.size() > 1 || i) ? " (still there?)" : ""));
 
         if (is_dumpable_artefact(item, false))
         {
@@ -785,20 +809,20 @@ void Stash::write(std::ostream &os, int refx, int refy,
             desc.erase(desc.find_last_not_of(" \n\t") + 1);
             desc.erase(0, desc.find_first_not_of(" \n\t"));
             // If string is not-empty, pad out to a neat indent
-            if (desc.length())
+            if (!desc.empty())
             {
                 // Walk backwards and prepend indenting spaces to \n characters.
                 for (int j = desc.length() - 1; j >= 0; --j)
                     if (desc[j] == '\n')
                         desc.insert(j + 1, " ");
 
-                os << "    " << desc << std::endl;
+                fprintf(f, "    %s\n", OUTS(desc));
             }
         }
     }
 
     if (items.size() <= 1 && !verified)
-        os << "  (unseen)" << std::endl;
+        fprintf(f, "  (unseen)\n");
 
     activate_notes(note_status);
 }
@@ -815,8 +839,7 @@ void Stash::save(writer& outf) const
     marshallByte(outf, trap);
 
     // Note: Enabled save value is inverted logic, so that it defaults to true
-    marshallByte(outf,
-        (unsigned char) ((verified? 1 : 0) | (!enabled? 2 : 0)) );
+    marshallByte(outf, ((verified? 1 : 0) | (!enabled? 2 : 0)));
 
     // And dump the items individually. We don't bother saving fields we're
     // not interested in (and don't anticipate being interested in).
@@ -832,12 +855,10 @@ void Stash::load(reader& inf)
     x = unmarshallByte(inf);
     y = unmarshallByte(inf);
 
-    feat =  static_cast<dungeon_feature_type>(
-                static_cast<unsigned char>( unmarshallByte(inf) ));
-    trap =  static_cast<trap_type>(
-                static_cast<unsigned char>( unmarshallByte(inf) ));
+    feat =  static_cast<dungeon_feature_type>(unmarshallUByte(inf));
+    trap =  static_cast<trap_type>(unmarshallUByte(inf));
 
-    unsigned char flags = unmarshallByte(inf);
+    uint8_t flags = unmarshallUByte(inf);
     verified = (flags & 1) != 0;
 
     // Note: Enabled save value is inverted so it defaults to true.
@@ -857,12 +878,6 @@ void Stash::load(reader& inf)
     }
 }
 
-std::ostream &operator << (std::ostream &os, const Stash &s)
-{
-    s.write(os);
-    return os;
-}
-
 ShopInfo::ShopInfo(int xp, int yp) : x(xp), y(yp), name(), shoptype(-1),
                                      visited(false), items()
 {
@@ -883,22 +898,15 @@ void ShopInfo::add_item(const item_def &sitem, unsigned price)
 
 std::string ShopInfo::shop_item_name(const shop_item &si) const
 {
-    const unsigned long oldflags = si.item.flags;
-
-    if (shoptype_identifies_stock(static_cast<shop_type>(this->shoptype)))
-        const_cast<shop_item&>(si).item.flags |= ISFLAG_IDENT_MASK;
-
-    if (oldflags != si.item.flags)
-        const_cast<shop_item&>(si).item.flags = oldflags;
-
-    return make_stringf("%s (%u gold)", Stash::stash_item_name(si.item).c_str(), si.price);
+    return make_stringf("%s (%u gold)",
+                        Stash::stash_item_name(si.item).c_str(), si.price);
 }
 
 std::string ShopInfo::shop_item_desc(const shop_item &si) const
 {
     std::string desc;
 
-    const unsigned long oldflags = si.item.flags;
+    const iflags_t oldflags = si.item.flags;
 
     if (shoptype_identifies_stock(static_cast<shop_type>(this->shoptype)))
         const_cast<shop_item&>(si).item.flags |= ISFLAG_IDENT_MASK;
@@ -922,13 +930,14 @@ std::string ShopInfo::shop_item_desc(const shop_item &si) const
 
 void ShopInfo::describe_shop_item(const shop_item &si) const
 {
-    const unsigned long oldflags = si.item.flags;
+    const iflags_t oldflags = si.item.flags;
 
     if (shoptype_identifies_stock(static_cast<shop_type>(this->shoptype)))
-        const_cast<shop_item&>(si).item.flags |= ISFLAG_IDENT_MASK;
+        const_cast<shop_item&>(si).item.flags |= ISFLAG_IDENT_MASK
+            | ISFLAG_NOTED_ID | ISFLAG_NOTED_GET;
 
     item_def it = static_cast<item_def>(si.item);
-    describe_item( it );
+    describe_item(it);
 
     if (oldflags != si.item.flags)
         const_cast<shop_item&>(si).item.flags = oldflags;
@@ -948,12 +957,12 @@ public:
         on_list = _on_list;
     }
 
-    std::string get_text() const
+    std::string get_text(const bool = false) const
     {
         ASSERT(level == MEL_ITEM && hotkeys.size());
         char buf[300];
         snprintf(buf, sizeof buf, " %c %c %s",
-                 hotkeys[0], on_list ? '@' : '-', text.c_str());
+                 hotkeys[0], on_list ? '$' : '-', text.c_str());
         return std::string(buf);
     }
 };
@@ -1010,7 +1019,7 @@ bool ShopInfo::show_menu(const level_pos &place,
         if (sel.size() != 1)
             break;
 
-        const shop_item *item = static_cast<const shop_item *>( sel[0]->data );
+        const shop_item *item = static_cast<const shop_item *>(sel[0]->data);
         if (menu.menu_action == Menu::ACT_EXAMINE)
             describe_shop_item(*item);
         else
@@ -1052,9 +1061,9 @@ bool ShopInfo::matches_search(const std::string &prefix,
         if (Stash::is_filtered(items[i].item))
             continue;
 
-        std::string sname = shop_item_name(items[i]);
-        std::string ann   = stash_annotate_item( STASH_LUA_SEARCH_ANNOTATE,
-                                                  &items[i].item, true );
+        const std::string sname = shop_item_name(items[i]);
+        const std::string ann   = stash_annotate_item(STASH_LUA_SEARCH_ANNOTATE,
+                                                      &items[i].item, true);
 
         bool thismatch = false;
         if (search.matches(prefix + " " + ann + sname))
@@ -1071,6 +1080,7 @@ bool ShopInfo::matches_search(const std::string &prefix,
             if (!res.count++)
                 res.match = sname;
             res.matches++;
+            res.set_matching_item(items[i].item);
         }
     }
 
@@ -1097,11 +1107,11 @@ bool ShopInfo::matches_search(const std::string &prefix,
     return (match || res.matches);
 }
 
-void ShopInfo::write(std::ostream &os, bool identify) const
+void ShopInfo::write(FILE *f, bool identify) const
 {
     bool note_status = notes_are_active();
     activate_notes(false);
-    os << "[Shop] " << name << std::endl;
+    fprintf(f, "[Shop] %s\n", OUTS(name));
     if (items.size() > 0)
     {
         for (unsigned i = 0; i < items.size(); ++i)
@@ -1111,16 +1121,16 @@ void ShopInfo::write(std::ostream &os, bool identify) const
             if (identify)
                 _fully_identify_item(&item.item);
 
-            os << "  " << shop_item_name(item) << std::endl;
+            fprintf(f, "  %s\n", OUTS(shop_item_name(item)));
             std::string desc = shop_item_desc(item);
-            if (desc.length() > 0)
-                os << "    " << desc << std::endl;
+            if (!desc.empty())
+                fprintf(f, "    %s\n", OUTS(desc));
         }
     }
     else if (visited)
-        os << "  (Shop is empty)" << std::endl;
+        fprintf(f, "  (Shop is empty)\n");
     else
-        os << "  (Shop contents are unknown)" << std::endl;
+        fprintf(f, "  (Shop contents are unknown)\n");
 
     activate_notes(note_status);
 }
@@ -1142,7 +1152,7 @@ void ShopInfo::save(writer& outf) const
     for (unsigned i = 0; i < items.size(); ++i)
     {
         marshallItem(outf, items[i].item);
-        marshallShort(outf, (short) items[i].price );
+        marshallShort(outf, (short) items[i].price);
     }
 }
 
@@ -1168,12 +1178,6 @@ void ShopInfo::load(reader& inf)
     }
 }
 
-std::ostream &operator << (std::ostream &os, const ShopInfo &s)
-{
-    s.write(os);
-    return os;
-}
-
 LevelStashes::LevelStashes()
     : m_place(level_id::current()),
       m_stashes(),
@@ -1219,6 +1223,9 @@ const ShopInfo *LevelStashes::find_shop(const coord_def& c) const
 
 bool LevelStashes::shop_needs_visit(const coord_def& c) const
 {
+    if (_grid_has_mimic_shop(c))
+        return true;
+
     const ShopInfo *shop = find_shop(c);
     return (shop && !shop->is_visited());
 }
@@ -1232,6 +1239,12 @@ bool LevelStashes::needs_visit(const coord_def& c) const
     return (shop_needs_visit(c));
 }
 
+bool LevelStashes::unverified_stash(const coord_def &c) const
+{
+    const Stash *s = find_stash(c);
+    return (s && s->unverified());
+}
+
 ShopInfo &LevelStashes::get_shop(const coord_def& c)
 {
     for (unsigned i = 0; i < m_shops.size(); ++i)
@@ -1365,7 +1378,7 @@ void LevelStashes::get_matching_stashes(
     }
 }
 
-void LevelStashes::_update_corpses(long rot_time)
+void LevelStashes::_update_corpses(int rot_time)
 {
     for (stashes_t::iterator iter = m_stashes.begin();
             iter != m_stashes.end(); iter++)
@@ -1374,15 +1387,25 @@ void LevelStashes::_update_corpses(long rot_time)
     }
 }
 
-void LevelStashes::write(std::ostream &os, bool identify) const
+void LevelStashes::_update_identification()
+{
+    for (stashes_t::iterator iter = m_stashes.begin();
+            iter != m_stashes.end(); iter++)
+    {
+        iter->second._update_identification();
+    }
+}
+
+void LevelStashes::write(FILE *f, bool identify) const
 {
     if (visible_stash_count() == 0)
         return;
 
-    os << level_name() << std::endl;
+    // very unlikely level names will be localized, but hey
+    fprintf(f, "%s\n", OUTS(level_name()));
 
     for (unsigned i = 0; i < m_shops.size(); ++i)
-        m_shops[i].write(os, identify);
+        m_shops[i].write(f, identify);
 
     if (m_stashes.size())
     {
@@ -1392,10 +1415,10 @@ void LevelStashes::write(std::ostream &os, bool identify) const
         for (stashes_t::const_iterator iter = m_stashes.begin();
              iter != m_stashes.end(); iter++)
         {
-            iter->second.write(os, refx, refy, levname, identify);
+            iter->second.write(f, refx, refy, levname, identify);
         }
     }
-    os << std::endl;
+    fprintf(f, "\n");
 }
 
 void LevelStashes::save(writer& outf) const
@@ -1442,10 +1465,28 @@ void LevelStashes::load(reader& inf)
     }
 }
 
-std::ostream &operator << (std::ostream &os, const LevelStashes &ls)
+void LevelStashes::remove_shop(const coord_def& c)
 {
-    ls.write(os);
-    return os;
+    const monster* mon = monster_at(c);
+    std::string mimic_name;
+    bool mimic_shop = false;
+
+    // If there are both shop mimic and normal shop here, then we want to erase
+    // just mimic entry.
+    if (mon && mon->type == MONS_SHOP_MIMIC)
+    {
+        mimic_shop = true;
+        if (mon->props.exists("shop_name"))
+            mimic_name = mon->props["shop_name"].get_string();
+    }
+
+    for (unsigned i = 0; i < m_shops.size(); ++i)
+        if (m_shops[i].isAt(c)
+            && (!mimic_shop || m_shops[i].description() == mimic_name))
+        {
+            m_shops.erase(m_shops.begin() + i);
+            return;
+        }
 }
 
 LevelStashes &StashTracker::get_current_level()
@@ -1516,25 +1557,25 @@ void StashTracker::add_stash(int x, int y, bool verbose)
 
 void StashTracker::dump(const char *filename, bool identify) const
 {
-    std::ofstream outf(filename);
+    FILE *outf = fopen_u(filename, "w");
     if (outf)
     {
         write(outf, identify);
-        outf.close();
+        fclose(outf);
     }
 }
 
-void StashTracker::write(std::ostream &os, bool identify) const
+void StashTracker::write(FILE *f, bool identify) const
 {
-    os << you.your_name << std::endl << std::endl;
+    fprintf(f, "%s\n\n", OUTS(you.your_name));
     if (!levels.size())
-        os << "  You have no stashes." << std::endl;
+        fprintf(f, "  You have no stashes.\n");
     else
     {
         for (stash_levels_t::const_iterator iter = levels.begin();
              iter != levels.end(); iter++)
         {
-            iter->second.write(os, identify);
+            iter->second.write(f, identify);
         }
     }
 }
@@ -1542,29 +1583,21 @@ void StashTracker::write(std::ostream &os, bool identify) const
 void StashTracker::save(writer& outf) const
 {
     // Time of last corpse update.
-    marshallLong(outf, last_corpse_update);
+    marshallInt(outf, last_corpse_update);
 
     // How many levels have we?
     marshallShort(outf, (short) levels.size());
 
     // And ask each level to write itself to the tag
     stash_levels_t::const_iterator iter = levels.begin();
-    for ( ; iter != levels.end(); iter++)
+    for (; iter != levels.end(); iter++)
         iter->second.save(outf);
 }
 
 void StashTracker::load(reader& inf)
 {
-    if (inf.getMinorVersion() < TAG_MINOR_STASHVER)
-    {
-        // Stash version is now a part of the global save.
-        unmarshallByte(inf),
-        unmarshallByte(inf);
-        last_corpse_update = (long)unmarshallFloat(inf);
-    }
-    else
-        // Time of last corpse update.
-        last_corpse_update = unmarshallLong(inf);
+    // Time of last corpse update.
+    last_corpse_update = unmarshallInt(inf);
 
     int count = unmarshallShort(inf);
 
@@ -1586,27 +1619,28 @@ void StashTracker::update_visible_stashes(
 
     LevelStashes *lev = find_current_level();
     coord_def c;
-    for (c.y = crawl_view.glos1.y; c.y <= crawl_view.glos2.y; ++c.y)
-        for (c.x = crawl_view.glos1.x; c.x <= crawl_view.glos2.x; ++c.x)
+    for (radius_iterator ri(you.get_los()); ri; ++ri)
+    {
+        dungeon_feature_type feat = grd(*ri);
+
+        const monster* mon = monster_at(*ri);
+        if (mon && mons_is_unknown_mimic(mon) && mons_is_feat_mimic(mon->type))
+            feat = get_mimic_feat(mon);
+
+        if ((!lev || !lev->update_stash(*ri))
+            && mode == ST_AGGRESSIVE
+            && (_grid_has_perceived_item(*ri)
+                || !Stash::is_boring_feature(feat)))
         {
-            if (!in_bounds(c) || !you.see_cell(c))
-                continue;
-
-            const dungeon_feature_type grid = grd(c);
-            if ((!lev || !lev->update_stash(c))
-                && mode == ST_AGGRESSIVE
-                && (_grid_has_perceived_item(c)
-                    || !Stash::is_boring_feature(grid)))
-            {
-                if (!lev)
-                    lev = &get_current_level();
-                lev->add_stash(c.x, c.y);
-            }
-
-            if (grid == DNGN_ENTER_SHOP)
-                get_shop(c);
+            if (!lev)
+                lev = &get_current_level();
+            lev->add_stash(ri->x, ri->y);
         }
 
+        if (feat == DNGN_ENTER_SHOP)
+            get_shop(*ri);
+    }
+
     if (lev && !lev->stash_count())
         remove_level();
 }
@@ -1620,7 +1654,7 @@ std::string StashTracker::stash_search_prompt()
     std::vector<std::string> opts;
     if (!lastsearch.empty())
         opts.push_back(
-            make_stringf("Enter for \"%s\"", lastsearch.c_str()) );
+            make_stringf("Enter for \"%s\"", lastsearch.c_str()));
     if (level_type_is_stash_trackable(you.level_type)
         && lastsearch != ".")
     {
@@ -1636,6 +1670,13 @@ std::string StashTracker::stash_search_prompt()
     return (make_stringf("Search for what%s? ", prompt_qual.c_str()));
 }
 
+void StashTracker::remove_shop(const coord_def& c)
+{
+    LevelStashes *lev = find_current_level();
+    if (lev)
+        lev->remove_shop(c);
+}
+
 class stash_search_reader : public line_reader
 {
 public:
@@ -1657,8 +1698,9 @@ protected:
 };
 
 // helper for search_stashes
-struct compare_by_distance
+class compare_by_distance
 {
+public:
     bool operator()(const stash_search_result& lhs,
                     const stash_search_result& rhs)
     {
@@ -1672,14 +1714,20 @@ struct compare_by_distance
             // Then by decreasing number of matches
             return (lhs.matches > rhs.matches);
         }
+        else if (lhs.match != rhs.match)
+        {
+            // Then by name.
+            return (lhs.match < rhs.match);
+        }
         else
             return (false);
     }
 };
 
 // helper for search_stashes
-struct compare_by_name
+class compare_by_name
 {
+public:
     bool operator()(const stash_search_result& lhs,
                     const stash_search_result& rhs)
     {
@@ -1708,6 +1756,7 @@ void StashTracker::search_stashes()
     char buf[400];
 
     this->update_corpses();
+    this->update_identification();
 
     stash_search_reader reader(buf, sizeof buf);
 
@@ -1735,7 +1784,7 @@ void StashTracker::search_stashes()
     msgwin_reply(validline ? buf : "");
 
     mesclr();
-    if (!validline || (!*buf && !lastsearch.length()))
+    if (!validline || (!*buf && lastsearch.empty()))
         return;
 
     std::string csearch = *buf? buf : lastsearch;
@@ -1764,10 +1813,10 @@ void StashTracker::search_stashes()
 
     base_pattern *search = NULL;
 
-    text_pattern tpat( csearch, true );
+    text_pattern tpat(csearch, true);
     search = &tpat;
 
-    lua_text_pattern ltpat( csearch );
+    lua_text_pattern ltpat(csearch);
 
     if (lua_text_pattern::is_lua_pattern(csearch))
         search = &ltpat;
@@ -1821,7 +1870,7 @@ void StashTracker::get_matching_stashes(
     const
 {
     stash_levels_t::const_iterator iter = levels.begin();
-    for ( ; iter != levels.end(); iter++)
+    for (; iter != levels.end(); iter++)
     {
         iter->second.get_matching_stashes(search, results);
         if (results.size() > SEARCH_SPAM_THRESHOLD)
@@ -1895,11 +1944,9 @@ bool StashTracker::display_search_results(
     if (results.empty())
         return (false);
 
-    bool travelable = can_travel_interlevel();
-
     StashSearchMenu stashmenu(sort_style);
     stashmenu.set_tag("stash");
-    stashmenu.can_travel   = travelable;
+    stashmenu.can_travel   = can_travel_interlevel();
     stashmenu.action_cycle = Menu::CYCLE_TOGGLE;
     stashmenu.menu_action  = Menu::ACT_EXECUTE;
     std::string title = "match";
@@ -1933,10 +1980,20 @@ bool StashTracker::display_search_results(
         if (res.shop && !res.shop->is_visited())
             me->colour = CYAN;
 
+        if (res.matching_item.get())
+        {
+            const item_def &first(*res.matching_item);
+            const int itemcol = menu_colour(first.name(DESC_PLAIN).c_str(),
+                                            menu_colour_item_prefix(first),
+                                            "pickup");
+            if (itemcol != -1)
+                me->colour = itemcol;
+        }
+
         stashmenu.add_entry(me);
     }
 
-    stashmenu.set_flags( MF_SINGLESELECT );
+    stashmenu.set_flags(MF_SINGLESELECT);
 
     std::vector<MenuEntry*> sel;
     while (true)
@@ -1967,9 +2024,12 @@ bool StashTracker::display_search_results(
             if (dotravel && can_travel_to(res->pos.id))
             {
                 redraw_screen();
-                const travel_target lp = res->pos;
-                start_translevel_travel(lp);
-                return (false);
+                level_pos lp = res->pos;
+                if (show_map(lp, true, true, true))
+                {
+                    start_translevel_travel(lp);
+                    return (false);
+                }
             }
             continue;
         }
@@ -1981,8 +2041,11 @@ bool StashTracker::display_search_results(
     {
         const stash_search_result *res =
                 static_cast<stash_search_result *>(sel[0]->data);
-        const level_pos lp = res->pos;
-        start_translevel_travel(lp);
+        level_pos lp = res->pos;
+        if (show_map(lp, true, true, true))
+            start_translevel_travel(lp);
+        else
+            return (true);
     }
     return (false);
 }
@@ -1992,7 +2055,7 @@ void StashTracker::update_corpses()
     if (you.elapsed_time - last_corpse_update < 20)
         return;
 
-    const long rot_time = (you.elapsed_time - last_corpse_update) / 20;
+    const int rot_time = (you.elapsed_time - last_corpse_update) / 20;
 
     last_corpse_update = you.elapsed_time;
 
@@ -2003,6 +2066,18 @@ void StashTracker::update_corpses()
     }
 }
 
+void StashTracker::update_identification()
+{
+    if (you.religion != GOD_ASHENZARI)
+        return;
+
+    for (stash_levels_t::iterator iter = levels.begin();
+            iter != levels.end(); iter++)
+    {
+        iter->second._update_identification();
+    }
+}
+
 //////////////////////////////////////////////
 
 ST_ItemIterator::ST_ItemIterator()
@@ -2066,7 +2141,7 @@ const ST_ItemIterator& ST_ItemIterator::operator ++ ()
         {
             const ShopInfo::shop_item &item = *m_shop_item_it++;
             m_item  = &(item.item);
-            ASSERT(m_item->is_valid());
+            ASSERT(m_item->defined());
             m_price = item.price;
             return (*this);
         }
@@ -2082,7 +2157,7 @@ const ST_ItemIterator& ST_ItemIterator::operator ++ ()
         if (m_stash_item_it != m_stash_it->second.items.end())
         {
             m_item = &(*m_stash_item_it++);
-            ASSERT(m_item->is_valid());
+            ASSERT(m_item->defined());
             return (*this);
         }
 
@@ -2120,7 +2195,7 @@ void ST_ItemIterator::new_level()
         if (m_stash_item_it != m_stash_it->second.items.end())
         {
             m_item = &(*m_stash_item_it++);
-            ASSERT(m_item->is_valid());
+            ASSERT(m_item->defined());
         }
     }
 
@@ -2135,7 +2210,7 @@ void ST_ItemIterator::new_level()
         {
             const ShopInfo::shop_item &item = *m_shop_item_it++;
             m_item  = &(item.item);
-            ASSERT(m_item->is_valid());
+            ASSERT(m_item->defined());
             m_price = item.price;
             m_shop  = &si;
         }
