@@ -1,9 +1,8 @@
-/*
- *  File:       store.cc
- *  Summary:    Saveable hash-table and vector capable of storing
- *              multiple types of data.
- *  Written by: Matthew Cline
- */
+/**
+ * @file
+ * @brief Saveable hash-table and vector capable of storing
+ *             multiple types of data.
+**/
 
 #include "AppHdr.h"
 
@@ -45,7 +44,8 @@ CrawlStoreValue::CrawlStoreValue(const CrawlStoreValue &other)
     case SV_BOOL:
     case SV_BYTE:
     case SV_SHORT:
-    case SV_LONG:
+    case SV_INT:
+    case SV_INT64:
     case SV_FLOAT:
         val = other.val;
         break;
@@ -110,8 +110,8 @@ CrawlStoreValue::CrawlStoreValue(const CrawlStoreValue &other)
 
     case SV_MONST:
     {
-        monsters* mon;
-        mon = new monsters(*static_cast<monsters*>(other.val.ptr));
+        monster* mon;
+        mon = new monster(*static_cast<monster* >(other.val.ptr));
         val.ptr = static_cast<void*>(mon);
         break;
     }
@@ -125,7 +125,7 @@ CrawlStoreValue::CrawlStoreValue(const CrawlStoreValue &other)
     }
 
     case NUM_STORE_VAL_TYPES:
-        ASSERT(false);
+        die("unknown stored value type");
     }
 }
 
@@ -159,10 +159,16 @@ CrawlStoreValue::CrawlStoreValue(const short &_val)
     get_short() = _val;
 }
 
-CrawlStoreValue::CrawlStoreValue(const long &_val)
-    : type(SV_LONG), flags(SFLAG_UNSET)
+CrawlStoreValue::CrawlStoreValue(const int &_val)
+    : type(SV_INT), flags(SFLAG_UNSET)
 {
-    get_long() = _val;
+    get_int() = _val;
+}
+
+CrawlStoreValue::CrawlStoreValue(const int64_t &_val)
+    : type(SV_INT), flags(SFLAG_UNSET)
+{
+    get_int64() = _val;
 }
 
 CrawlStoreValue::CrawlStoreValue(const float &_val)
@@ -227,7 +233,7 @@ CrawlStoreValue::CrawlStoreValue(const level_pos &_val)
     get_level_pos() = _val;
 }
 
-CrawlStoreValue::CrawlStoreValue(const monsters &_val)
+CrawlStoreValue::CrawlStoreValue(const monster& _val)
     : type(SV_MONST), flags(SFLAG_UNSET)
 {
     val.ptr = NULL;
@@ -270,8 +276,12 @@ void CrawlStoreValue::unset(bool force)
         val._short = 0;
         break;
 
-    case SV_LONG:
-        val._long = 0;
+    case SV_INT:
+        val._int = 0;
+        break;
+
+    case SV_INT64:
+        val._int64 = 0;
         break;
 
     case SV_FLOAT:
@@ -336,7 +346,7 @@ void CrawlStoreValue::unset(bool force)
 
     case SV_MONST:
     {
-        monsters* mon = static_cast<monsters*>(val.ptr);
+        monster* mon = static_cast<monster* >(val.ptr);
         delete mon;
         val.ptr = NULL;
         break;
@@ -351,11 +361,11 @@ void CrawlStoreValue::unset(bool force)
     }
 
     case SV_NONE:
-        DEBUGSTR("CrawlStoreValue::unset: unsetting nothing");
+        die("CrawlStoreValue::unset: unsetting nothing");
         break;
 
     default:
-        DEBUGSTR("CrawlStoreValue::unset: unsetting invalid type");
+        die("CrawlStoreValue::unset: unsetting invalid type");
         break;
     }
 
@@ -393,7 +403,8 @@ CrawlStoreValue &CrawlStoreValue::operator = (const CrawlStoreValue &other)
     case SV_BOOL:
     case SV_BYTE:
     case SV_SHORT:
-    case SV_LONG:
+    case SV_INT:
+    case SV_INT64:
     case SV_FLOAT:
         val = other.val;
         break;
@@ -427,7 +438,7 @@ CrawlStoreValue &CrawlStoreValue::operator = (const CrawlStoreValue &other)
         break;
 
      default:
-        DEBUGSTR("CrawlStoreValue has invalid type");
+        die("CrawlStoreValue has invalid type");
         break;
     }
 
@@ -482,8 +493,12 @@ void CrawlStoreValue::write(writer &th) const
         marshallShort(th, val._short);
         break;
 
-    case SV_LONG:
-        marshallLong(th, val._long);
+    case SV_INT:
+        marshallInt(th, val._int);
+        break;
+
+    case SV_INT64:
+        marshallSigned(th, val._int64);
         break;
 
     case SV_FLOAT:
@@ -541,7 +556,7 @@ void CrawlStoreValue::write(writer &th) const
 
     case SV_MONST:
     {
-        monsters* mon = static_cast<monsters*>(val.ptr);
+        monster* mon = static_cast<monster* >(val.ptr);
         marshallMonster(th, *mon);
         break;
     }
@@ -557,7 +572,7 @@ void CrawlStoreValue::write(writer &th) const
         break;
 
     case NUM_STORE_VAL_TYPES:
-        ASSERT(false);
+        die("unknown stored value type");
     }
 }
 
@@ -583,8 +598,12 @@ void CrawlStoreValue::read(reader &th)
         val._short = unmarshallShort(th);
         break;
 
-    case SV_LONG:
-        val._long = unmarshallLong(th);
+    case SV_INT:
+        val._int = unmarshallInt(th);
+        break;
+
+    case SV_INT64:
+        val._int64 = unmarshallSigned(th);
         break;
 
     case SV_FLOAT:
@@ -600,8 +619,7 @@ void CrawlStoreValue::read(reader &th)
 
     case SV_COORD:
     {
-        coord_def coord;
-        unmarshallCoord(th, coord);
+        const coord_def coord = unmarshallCoord(th);
         val.ptr = (void*) new coord_def(coord);
 
         break;
@@ -654,9 +672,9 @@ void CrawlStoreValue::read(reader &th)
 
     case SV_MONST:
     {
-        monsters mon;
+        monster mon;
         unmarshallMonster(th, mon);
-        val.ptr = (void*) new monsters(mon);
+        val.ptr = (void*) new monster(mon);
 
         break;
     }
@@ -674,7 +692,7 @@ void CrawlStoreValue::read(reader &th)
         break;
 
     case NUM_STORE_VAL_TYPES:
-        ASSERT(false);
+        die("unknown stored value type");
     }
 }
 
@@ -744,14 +762,17 @@ CrawlVector &CrawlStoreValue::new_vector(store_val_type _type,
             case SV_SHORT: \
                 field = (_type) val._short; \
                 break; \
-            case SV_LONG: \
-                field = (_type) val._long; \
+            case SV_INT: \
+                field = (_type) val._int; \
+                break; \
+            case SV_INT64: \
+                field = (_type) val._int64; \
                 break; \
             case SV_FLOAT: \
                 field = (_type) val._float; \
                 break; \
             default: \
-                ASSERT(false); \
+                die("unknown stored value type"); \
             } \
             type = (x); \
         } \
@@ -795,9 +816,14 @@ short &CrawlStoreValue::get_short()
     GET_VAL(SV_SHORT, short, val._short, 0);
 }
 
-long &CrawlStoreValue::get_long()
+int &CrawlStoreValue::get_int()
 {
-    GET_VAL(SV_LONG, long, val._long, 0);
+    GET_VAL(SV_INT, int, val._int, 0);
+}
+
+int64_t &CrawlStoreValue::get_int64()
+{
+    GET_VAL(SV_INT64, int64_t, val._int64, 0);
 }
 
 float &CrawlStoreValue::get_float()
@@ -840,9 +866,9 @@ level_pos &CrawlStoreValue::get_level_pos()
     GET_VAL_PTR(SV_LEV_POS, level_pos*, new level_pos());
 }
 
-monsters &CrawlStoreValue::get_monster()
+monster &CrawlStoreValue::get_monster()
 {
-    GET_VAL_PTR(SV_MONST, monsters*, new monsters());
+    GET_VAL_PTR(SV_MONST, monster* , new monster());
 }
 
 dlua_chunk &CrawlStoreValue::get_lua()
@@ -884,10 +910,16 @@ short CrawlStoreValue::get_short() const
     return val._short;
 }
 
-long CrawlStoreValue::get_long() const
+int CrawlStoreValue::get_int() const
 {
-    GET_CONST_SETUP(SV_LONG);
-    return val._long;
+    GET_CONST_SETUP(SV_INT);
+    return val._int;
+}
+
+int64_t CrawlStoreValue::get_int64() const
+{
+    GET_CONST_SETUP(SV_INT64);
+    return val._int64;
 }
 
 float CrawlStoreValue::get_float() const
@@ -956,7 +988,8 @@ CrawlStoreValue::operator bool&()                  { return get_bool();       }
 CrawlStoreValue::operator char&()                  { return get_byte();       }
 CrawlStoreValue::operator short&()                 { return get_short();      }
 CrawlStoreValue::operator float&()                 { return get_float();      }
-CrawlStoreValue::operator long&()                  { return get_long();       }
+CrawlStoreValue::operator int&()                   { return get_int();        }
+CrawlStoreValue::operator int64_t&()               { return get_int64();      }
 CrawlStoreValue::operator std::string&()           { return get_string();     }
 CrawlStoreValue::operator coord_def&()             { return get_coord();      }
 CrawlStoreValue::operator CrawlHashTable&()        { return get_table();      }
@@ -964,7 +997,7 @@ CrawlStoreValue::operator CrawlVector&()           { return get_vector();     }
 CrawlStoreValue::operator item_def&()              { return get_item();       }
 CrawlStoreValue::operator level_id&()              { return get_level_id();   }
 CrawlStoreValue::operator level_pos&()             { return get_level_pos();  }
-CrawlStoreValue::operator monsters&()              { return get_monster();    }
+CrawlStoreValue::operator monster& ()              { return get_monster();    }
 CrawlStoreValue::operator dlua_chunk&()            { return get_lua(); }
 
 ///////////////////////////
@@ -981,11 +1014,10 @@ CrawlStoreValue::operator bool() const
         return get_byte(); \
     case SV_SHORT: \
         return get_short(); \
-    case SV_LONG: \
-        return get_long(); \
+    case SV_INT: \
+        return get_int(); \
     default: \
-        ASSERT(false); \
-        return 0; \
+        die("unknown stored value type"); \
     }
 
 CrawlStoreValue::operator char() const
@@ -998,11 +1030,29 @@ CrawlStoreValue::operator short() const
     CONST_INT_CAST();
 }
 
-CrawlStoreValue::operator long() const
+CrawlStoreValue::operator int() const
 {
     CONST_INT_CAST();
 }
 
+CrawlStoreValue::operator int64_t() const
+{
+    // Allow upgrading but not downgrading.
+    switch (type)
+    {
+    case SV_BYTE:
+        return get_byte();
+    case SV_SHORT:
+        return get_short();
+    case SV_INT:
+        return get_int();
+    case SV_INT64:
+        return get_int64();
+    default:
+        die("unknown stored value type");
+    }
+}
+
 CrawlStoreValue::operator float() const
 {
     return get_float();
@@ -1048,9 +1098,15 @@ CrawlStoreValue &CrawlStoreValue::operator = (const short &_val)
     return (*this);
 }
 
-CrawlStoreValue &CrawlStoreValue::operator = (const long &_val)
+CrawlStoreValue &CrawlStoreValue::operator = (const int &_val)
 {
-    get_long() = _val;
+    get_int() = _val;
+    return (*this);
+}
+
+CrawlStoreValue &CrawlStoreValue::operator = (const int64_t &_val)
+{
+    get_int64() = _val;
     return (*this);
 }
 
@@ -1108,7 +1164,7 @@ CrawlStoreValue &CrawlStoreValue::operator = (const level_pos &_val)
     return (*this);
 }
 
-CrawlStoreValue &CrawlStoreValue::operator = (const monsters &_val)
+CrawlStoreValue &CrawlStoreValue::operator = (const monster& _val)
 {
     get_monster() = _val;
     return (*this);
@@ -1138,36 +1194,42 @@ CrawlStoreValue &CrawlStoreValue::operator = (const dlua_chunk &_val)
         temp op; \
         return temp; \
     } \
-    case SV_LONG: \
+    case SV_INT: \
     { \
-        long &temp = get_long(); \
+        int &temp = get_int(); \
+        temp op; \
+        return temp; \
+    } \
+    case SV_INT64: \
+    { \
+        int64_t &temp = get_int64(); \
         temp op; \
         return temp; \
     } \
  \
     default: \
-        ASSERT(false); \
+        die("unknown stored value type"); \
         return 0; \
     }
 
 // Prefix
-long CrawlStoreValue::operator ++ ()
+int CrawlStoreValue::operator ++ ()
 {
     INT_OPERATOR_UNARY(++);
 }
 
-long CrawlStoreValue::operator -- ()
+int CrawlStoreValue::operator -- ()
 {
     INT_OPERATOR_UNARY(--);
 }
 
 // Postfix
-long CrawlStoreValue::operator ++ (int)
+int CrawlStoreValue::operator ++ (int)
 {
     INT_OPERATOR_UNARY(++);
 }
 
-long CrawlStoreValue::operator -- (int)
+int CrawlStoreValue::operator -- (int)
 {
     INT_OPERATOR_UNARY(--);
 }
@@ -1424,7 +1486,7 @@ void CrawlHashTable::erase(const std::string key)
 
     if (i != hash_map->end())
     {
-#ifdef DEBUG
+#ifdef ASSERTS
         CrawlStoreValue &val = i->second;
         ASSERT(!(val.flags & SFLAG_NO_ERASE));
 #endif
@@ -1601,7 +1663,7 @@ store_val_type CrawlVector::get_type() const
 
 void CrawlVector::assert_validity() const
 {
-#ifdef DEBUG
+#ifdef ASSERTS
     ASSERT(!(default_flags & SFLAG_UNSET));
     ASSERT(max_size > 0);
     ASSERT(max_size >= size());
@@ -1729,7 +1791,7 @@ CrawlStoreValue& CrawlVector::pop_back()
 
 void CrawlVector::push_back(CrawlStoreValue val)
 {
-#ifdef DEBUG
+#ifdef ASSERTS
     if (type != SV_NONE)
         ASSERT(type == val.type);
 
@@ -1862,50 +1924,3 @@ CrawlVector::const_iterator CrawlVector::end() const
     assert_validity();
     return vector.end();
 }
-
-/////////////////////////////////////////////////////////////////////////////
-/////////////////////////////////////////////////////////////////////////////
-
-template <typename T, store_val_type TYPE>
-CrawlTableWrapper<T, TYPE>::CrawlTableWrapper()
-{
-    table = NULL;
-}
-
-template <typename T, store_val_type TYPE>
-CrawlTableWrapper<T, TYPE>::CrawlTableWrapper(CrawlHashTable& _table)
-{
-    wrap(_table);
-}
-
-template <typename T, store_val_type TYPE>
-CrawlTableWrapper<T, TYPE>::CrawlTableWrapper(CrawlHashTable* _table)
-{
-    wrap(_table);
-}
-
-template <typename T, store_val_type TYPE>
-void CrawlTableWrapper<T, TYPE>::wrap(CrawlHashTable& _table)
-{
-    wrap(&_table);
-}
-
-template <typename T, store_val_type TYPE>
-void CrawlTableWrapper<T, TYPE>::wrap(CrawlHashTable* _table)
-{
-    ASSERT(_table != NULL);
-
-    table = _table;
-}
-
-template <typename T, store_val_type TYPE>
-T& CrawlTableWrapper<T, TYPE>::operator[] (const std::string &key)
-{
-    return (T&) (*table)[key];
-}
-
-template <typename T, store_val_type TYPE>
-const T CrawlTableWrapper<T, TYPE>::operator[] (const std::string &key) const
-{
-    return (T) (*table)[key];
-}
