@@ -1,8 +1,7 @@
-/*
- *  File:       stuff.cc
- *  Summary:    Misc stuff.
- *  Written by: Linley Henzell
- */
+/**
+ * @file
+ * @brief Misc stuff.
+**/
 
 #include "AppHdr.h"
 
@@ -11,10 +10,12 @@
 #include "areas.h"
 #include "beam.h"
 #include "cio.h"
+#include "colour.h"
 #include "coord.h"
 #include "coordit.h"
 #include "database.h"
 #include "directn.h"
+#include "dungeon.h"
 #include "env.h"
 #include "libutil.h"
 #include "los.h"
@@ -23,7 +24,6 @@
 #include "misc.h"
 #include "mon-place.h"
 #include "terrain.h"
-#include "mgen_data.h"
 #include "state.h"
 #include "travel.h"
 #include "view.h"
@@ -63,13 +63,13 @@
 #include "output.h"
 #include "player.h"
 #include "religion.h"
-#include "tutorial.h"
+#include "hints.h"
 #include "view.h"
 
 stack_iterator::stack_iterator(const coord_def& pos, bool accessible)
 {
     cur_link = accessible ? you.visible_igrd(pos) : igrd(pos);
-    if ( cur_link != NON_ITEM )
+    if (cur_link != NON_ITEM)
         next_link = mitm[cur_link].link;
     else
         next_link = NON_ITEM;
@@ -78,7 +78,7 @@ stack_iterator::stack_iterator(const coord_def& pos, bool accessible)
 stack_iterator::stack_iterator(int start_link)
 {
     cur_link = start_link;
-    if ( cur_link != NON_ITEM )
+    if (cur_link != NON_ITEM)
         next_link = mitm[cur_link].link;
     else
         next_link = NON_ITEM;
@@ -86,18 +86,18 @@ stack_iterator::stack_iterator(int start_link)
 
 stack_iterator::operator bool() const
 {
-    return ( cur_link != NON_ITEM );
+    return (cur_link != NON_ITEM);
 }
 
 item_def& stack_iterator::operator*() const
 {
-    ASSERT( cur_link != NON_ITEM );
+    ASSERT(cur_link != NON_ITEM);
     return mitm[cur_link];
 }
 
 item_def* stack_iterator::operator->() const
 {
-    ASSERT( cur_link != NON_ITEM );
+    ASSERT(cur_link != NON_ITEM);
     return &mitm[cur_link];
 }
 
@@ -109,7 +109,7 @@ int stack_iterator::link() const
 const stack_iterator& stack_iterator::operator ++ ()
 {
     cur_link = next_link;
-    if ( cur_link != NON_ITEM )
+    if (cur_link != NON_ITEM)
         next_link = mitm[cur_link].link;
     return *this;
 }
@@ -129,21 +129,13 @@ std::string make_time_string(time_t abs_time, bool terse)
     const int mins  = (abs_time % 3600) / 60;
     const int secs  = abs_time % 60;
 
-    std::ostringstream buff;
-    buff << std::setfill('0');
-
+    std::string buff;
     if (days > 0)
     {
-        if (terse)
-            buff << days << ", ";
-        else
-            buff << days << (days > 1 ? " days" : "day");
+        buff += make_stringf("%d%s ", days, terse ? ","
+                             : days > 1 ? "days" : "day");
     }
-
-    buff << std::setw(2) << hours << ':'
-         << std::setw(2) << mins << ':'
-         << std::setw(2) << secs;
-    return buff.str();
+    return buff + make_stringf("%02d:%02d:%02d", hours, mins, secs);
 }
 
 std::string make_file_time(time_t when)
@@ -161,12 +153,12 @@ std::string make_file_time(time_t when)
     return ("");
 }
 
-void set_redraw_status(unsigned long flags)
+void set_redraw_status(uint64_t flags)
 {
     you.redraw_status_flags |= flags;
 }
 
-static bool _is_religious_follower(const monsters* mon)
+static bool _is_religious_follower(const monster* mon)
 {
     return ((you.religion == GOD_YREDELEMNUL || you.religion == GOD_BEOGH)
             && is_follower(mon));
@@ -177,12 +169,11 @@ static bool _tag_follower_at(const coord_def &pos, bool &real_follower)
     if (!in_bounds(pos) || pos == you.pos())
         return (false);
 
-    monsters *fmenv = monster_at(pos);
+    monster* fmenv = monster_at(pos);
     if (fmenv == NULL)
         return (false);
 
-    if (fmenv->type == MONS_PLAYER_GHOST
-        || !fmenv->alive()
+    if (!fmenv->alive()
         || fmenv->speed_increment < 50
         || fmenv->incapacitated()
         || mons_is_stationary(fmenv))
@@ -193,10 +184,11 @@ static bool _tag_follower_at(const coord_def &pos, bool &real_follower)
     if (!monster_habitable_grid(fmenv, DNGN_FLOOR))
         return (false);
 
-    // Only friendly monsters, or those actively seeking the
-    // player, will follow up/down stairs.
+    // Only non-wandering friendly monsters or those actively
+    // seeking the player will follow up/down stairs.
     if (!fmenv->friendly()
-        && (!mons_is_seeking(fmenv) || fmenv->foe != MHITYOU))
+          && (!mons_is_seeking(fmenv) || fmenv->foe != MHITYOU)
+        || fmenv->foe == MHITNOT)
     {
         return (false);
     }
@@ -237,8 +229,10 @@ static bool _tag_follower_at(const coord_def &pos, bool &real_follower)
     fmenv->travel_path.clear();
     fmenv->travel_target = MTRAV_NONE;
 
+    fmenv->clear_clinging();
+
     dprf("%s is marked for following.",
-         fmenv->name(DESC_CAP_THE, true).c_str() );
+         fmenv->name(DESC_CAP_THE, true).c_str());
 
     return (true);
 }
@@ -249,7 +243,7 @@ static int follower_tag_radius2()
     // only adjacent friendlies may follow.
     for (adjacent_iterator ai(you.pos()); ai; ++ai)
     {
-        if (const monsters *mon = monster_at(*ai))
+        if (const monster* mon = monster_at(*ai))
             if (!mon->friendly())
                 return (2);
     }
@@ -305,10 +299,10 @@ void untag_followers()
 unsigned char get_ch()
 {
     mouse_control mc(MOUSE_MODE_MORE);
-    unsigned char gotched = getch();
+    unsigned char gotched = getchm();
 
     if (gotched == 0)
-        gotched = getch();
+        gotched = getchm();
 
     return gotched;
 }
@@ -336,13 +330,6 @@ void cio_init()
 #ifdef USE_TILE
     tiles.resize();
 #endif
-
-    if (Options.char_set == CSET_UNICODE && !crawl_state.unicode_ok)
-    {
-        crawl_state.add_startup_error(
-            "Unicode glyphs are not available, falling back to ASCII.");
-        Options.char_set = CSET_ASCII;
-    }
 }
 
 void cio_cleanup()
@@ -360,8 +347,6 @@ void cio_cleanup()
     deinit_libw32c();
 #endif
 
-    msg::deinitialise_mpr_streams();
-    clear_globals_on_exit();
     crawl_state.io_inited = false;
 }
 
@@ -370,6 +355,8 @@ void clear_globals_on_exit()
 {
     clear_rays_on_exit();
     clear_zap_info_on_exit();
+    clear_colours_on_exit();
+    dgn_clear_vault_placements(env.level_vaults);
 }
 
 // Used by do_crash_dump() to tell if the crash happened during exit() hooks.
@@ -377,14 +364,11 @@ void clear_globals_on_exit()
 // free'd by exit() hooks when exit() is called, and we don't want to reference
 // free'd memory.
 bool CrawlIsExiting = false;
+bool CrawlIsCrashing = false;
 
-void end(int exit_code, bool print_error, const char *format, ...)
+NORETURN void end(int exit_code, bool print_error, const char *format, ...)
 {
     std::string error = print_error? strerror(errno) : "";
-
-    cio_cleanup();
-    databaseSystemShutdown();
-
     if (format)
     {
         va_list arg;
@@ -397,13 +381,29 @@ void end(int exit_code, bool print_error, const char *format, ...)
             error = std::string(buffer);
         else
             error = std::string(buffer) + ": " + error;
+
+        if (!error.empty() && error[error.length() - 1] != '\n')
+            error += "\n";
     }
 
+#if (defined(TARGET_OS_WINDOWS) && !defined(USE_TILE)) \
+     || defined(TARGET_OS_DOS) \
+     || defined(DGL_PAUSE_AFTER_ERROR)
+    bool need_pause = true;
+    if (exit_code && !error.empty())
+    {
+        if (print_error_screen("%s", error.c_str()))
+            need_pause = false;
+    }
+#endif
+
+    cio_cleanup();
+    msg::deinitialise_mpr_streams();
+    clear_globals_on_exit();
+    databaseSystemShutdown();
+
     if (!error.empty())
     {
-        if (error[error.length() - 1] != '\n')
-            error += "\n";
-
         fprintf(stderr, "%s", error.c_str());
         error.clear();
     }
@@ -411,7 +411,7 @@ void end(int exit_code, bool print_error, const char *format, ...)
 #if (defined(TARGET_OS_WINDOWS) && !defined(USE_TILE)) \
      || defined(TARGET_OS_DOS) \
      || defined(DGL_PAUSE_AFTER_ERROR)
-    if (exit_code && !crawl_state.arena
+    if (need_pause && exit_code && !crawl_state.game_is_arena()
         && !crawl_state.seen_hups && !crawl_state.test)
     {
         fprintf(stderr, "Hit Enter to continue...\n");
@@ -420,13 +420,61 @@ void end(int exit_code, bool print_error, const char *format, ...)
 #endif
 
     CrawlIsExiting = true;
+    if (exit_code)
+        CrawlIsCrashing = true;
+
+#ifdef DEBUG_GLOBALS
+    delete real_env;         real_env = 0;
+    delete real_crawl_state; real_crawl_state = 0;
+    delete real_dlua;        real_dlua = 0;
+    delete real_clua;        real_clua = 0;
+    delete real_you;         real_you = 0;
+    delete real_Options;     real_Options = 0;
+#endif
+
     exit(exit_code);
 }
 
+NORETURN void game_ended()
+{
+    if (!crawl_state.seen_hups)
+        throw game_ended_condition();
+    else
+        end(0);
+}
+
+NORETURN void game_ended_with_error(const std::string &message)
+{
+    if (crawl_state.seen_hups)
+        end(1);
+
+    if (Options.restart_after_game)
+    {
+        if (crawl_state.io_inited)
+        {
+            mprf(MSGCH_ERROR, "%s", message.c_str());
+            more();
+        }
+        else
+        {
+            fprintf(stderr, "%s\nHit Enter to continue...\n", message.c_str());
+            getchar();
+        }
+        game_ended();
+    }
+    else
+    {
+        end(1, false, "%s", message.c_str());
+    }
+}
+
 // Print error message on the screen.
 // Ugly, but better than not showing anything at all. (jpeg)
-void print_error_screen(const char *message, ...)
+bool print_error_screen(const char *message, ...)
 {
+    if (!crawl_state.io_inited || crawl_state.seen_hups)
+        return false;
+
     // Get complete error message.
     std::string error_msg;
     {
@@ -439,14 +487,14 @@ void print_error_screen(const char *message, ...)
         error_msg = std::string(buffer);
     }
     if (error_msg.empty())
-        return;
+        return false;
 
     // Escape '<'.
     // NOTE: This assumes that the error message doesn't contain
     //       any formatting!
     error_msg = replace_all(error_msg, "<", "<<");
 
-    error_msg += EOL EOL EOL "Hit any key to exit..." EOL;
+    error_msg += "\n\n\nHit any key to exit...\n";
 
     // Break message into correctly sized lines.
     int width = 80;
@@ -460,7 +508,8 @@ void print_error_screen(const char *message, ...)
     // And finally output the message.
     clrscr();
     formatted_string::parse_string(error_msg, false).display();
-    getch();
+    getchm();
+    return true;
 }
 
 void redraw_screen(void)
@@ -476,9 +525,7 @@ void redraw_screen(void)
 
     you.redraw_hit_points   = true;
     you.redraw_magic_points = true;
-    you.redraw_strength     = true;
-    you.redraw_intelligence = true;
-    you.redraw_dexterity    = true;
+    you.redraw_stats.init(true);
     you.redraw_armour_class = true;
     you.redraw_evasion      = true;
     you.redraw_experience   = true;
@@ -486,7 +533,7 @@ void redraw_screen(void)
     you.redraw_quiver       = true;
 
     set_redraw_status(
-        REDRAW_LINE_1_MASK | REDRAW_LINE_2_MASK | REDRAW_LINE_3_MASK );
+        REDRAW_LINE_1_MASK | REDRAW_LINE_2_MASK | REDRAW_LINE_3_MASK);
 
     print_stats();
 
@@ -499,7 +546,7 @@ void redraw_screen(void)
     update_turn_count();
     activate_notes(note_status);
 
-    viewwindow(false);
+    viewwindow();
 
     // Display the message window at the end because it places
     // the cursor behind possible prompts.
@@ -566,66 +613,26 @@ int stepdown_value(int base_value, int stepping, int first_step,
 
 }
 
-int skill_bump( int skill )
+int skill_bump(skill_type skill)
 {
-    return ((you.skills[skill] < 3) ? you.skills[skill] * 2
-                                    : you.skills[skill] + 3);
+    int sk = you.skill(skill);
+    return sk < 3 ? sk * 2 : sk + 3;
 }
 
 // This gives (default div = 20, shift = 3):
 // - shift/div% @ stat_level = 0; (default 3/20 = 15%, or 20% at stat 1)
 // - even (100%) @ stat_level = div - shift; (default 17)
 // - 1/div% per stat_level (default 1/20 = 5%)
-int stat_mult( int stat_level, int value, int div, int shift )
+int stat_mult(int stat_level, int value, int div, int shift)
 {
     return (((stat_level + shift) * value) / ((div > 1) ? div : 1));
 }
 
-// As above but inverted (ie 5x penalty at stat 1)
-int stat_div( int stat_level, int value, int mult, int shift )
-{
-    int div = stat_level + shift;
-
-    if (div < 1)
-        div = 1;
-
-    return ((mult * value) / div);
-}
-
 int div_round_up(int num, int den)
 {
     return (num / den + (num % den != 0));
 }
 
-// Simple little function to quickly modify all three stats
-// at once - does check for '0' modifiers to prevent needless
-// adding .. could use checking for sums less than zero, I guess.
-// Used in conjunction with newgame::species_stat_init() and
-// newgame::job_stat_init() routines 24jan2000. {dlb}
-void modify_all_stats(int STmod, int IQmod, int DXmod)
-{
-    if (STmod)
-    {
-        you.strength     += STmod;
-        you.max_strength += STmod;
-        you.redraw_strength = true;
-    }
-
-    if (IQmod)
-    {
-        you.intel     += IQmod;
-        you.max_intel += IQmod;
-        you.redraw_intelligence = true;
-    }
-
-    if (DXmod)
-    {
-        you.dex     += DXmod;
-        you.max_dex += DXmod;
-        you.redraw_dexterity = true;
-    }
-}
-
 void canned_msg(canned_message_type which_message)
 {
     switch (which_message)
@@ -638,9 +645,12 @@ void canned_msg(canned_message_type which_message)
     case MSG_NOTHING_HAPPENS:
         mpr("Nothing appears to happen.");
         break;
+    case MSG_YOU_UNAFFECTED:
+        mpr("You are unaffected.");
+        break;
     case MSG_YOU_RESIST:
         mpr("You resist.");
-        learned_something_new(TUT_YOU_RESIST);
+        learned_something_new(HINT_YOU_RESIST);
         break;
     case MSG_YOU_PARTIALLY_RESIST:
         mpr("You partially resist.");
@@ -669,6 +679,14 @@ void canned_msg(canned_message_type which_message)
         mpr("Why would you want to do that?");
         crawl_state.cancel_cmd_repeat();
         break;
+    case MSG_NOTHING_THERE:
+        mpr("There's nothing there!");
+        crawl_state.cancel_cmd_repeat();
+        break;
+    case MSG_NOTHING_CLOSE_ENOUGH:
+        mpr("There's nothing close enough!");
+        crawl_state.cancel_cmd_repeat();
+        break;
     case MSG_SPELL_FIZZLES:
         mpr("The spell fizzles.");
         break;
@@ -677,24 +695,36 @@ void canned_msg(canned_message_type which_message)
         crawl_state.cancel_cmd_repeat();
         break;
     case MSG_EMPTY_HANDED:
-        mpr("You are now empty-handed.");
+        if (you.species == SP_CAT)
+            mpr("Your mouth is now empty.");
+        else
+            mpr("You are now empty-handed.");
         break;
     case MSG_YOU_BLINK:
         mpr("You blink.");
         break;
-    case MSG_WEIRD_STASIS:
-        mpr("You feel a weird sense of stasis.");
+    case MSG_STRANGE_STASIS:
+        mpr("You feel a strange sense of stasis.");
         break;
     case MSG_NO_SPELLS:
         mpr("You don't know any spells.");
         break;
+    case MSG_MANA_INCREASE:
+        mpr("You feel your mana capacity increase.");
+        break;
+    case MSG_MANA_DECREASE:
+        mpr("You feel your mana capacity decrease.");
+        break;
+    case MSG_TOO_HUNGRY:
+        mpr("You're too hungry.");
+        break;
     }
 }
 
 // Like yesno, but requires a full typed answer.
 // Unlike yesno, prompt should have no trailing space.
 // Returns true if the user typed "yes", false if something else or cancel.
-bool yes_or_no( const char* fmt, ... )
+bool yes_or_no(const char* fmt, ...)
 {
     char buf[200];
     va_list args;
@@ -721,7 +751,7 @@ bool yesno(const char *str, bool safe, int safeanswer, bool clear_after,
 {
     bool message = (region == GOTO_MSG);
     if (interrupt_delays && !crawl_state.is_repeating_cmd())
-        interrupt_activity( AI_FORCE_INTERRUPT );
+        interrupt_activity(AI_FORCE_INTERRUPT);
 
     std::string prompt = make_stringf("%s ", str ? str : "Buggy prompt?");
 
@@ -733,7 +763,7 @@ bool yesno(const char *str, bool safe, int safeanswer, bool clear_after,
             if (message)
                 mpr(prompt.c_str(), MSGCH_PROMPT);
             else
-                cprintf(prompt.c_str());
+                cprintf("%s", prompt.c_str());
         }
 
         int tmp = getchm(KMC_CONFIRM);
@@ -751,7 +781,7 @@ bool yesno(const char *str, bool safe, int safeanswer, bool clear_after,
             tmp = map->find(tmp)->second;
 
         if (safeanswer
-            && (tmp == ' ' || tmp == ESCAPE || tmp == CONTROL('G')
+            && (tmp == ' ' || key_is_escape(tmp)
                 || tmp == '\r' || tmp == '\n' || crawl_state.seen_hups))
         {
             tmp = safeanswer;
@@ -761,7 +791,7 @@ bool yesno(const char *str, bool safe, int safeanswer, bool clear_after,
             || tmp == safeanswer
             || Options.easy_confirm == CONFIRM_SAFE_EASY && safe)
         {
-            tmp = toupper( tmp );
+            tmp = toupper(tmp);
         }
 
         if (clear_after && message)
@@ -773,11 +803,14 @@ bool yesno(const char *str, bool safe, int safeanswer, bool clear_after,
             return (true);
         else if (!noprompt)
         {
-            const std::string pr = "[Y]es or [N]o only, please.";
+            bool upper = (!safe && crawl_state.game_is_hints_tutorial());
+            const std::string pr
+                = make_stringf("%s[Y]es or [N]o only, please.",
+                               upper ? "Uppercase " : "");
             if (message)
                 mpr(pr);
             else
-                cprintf((EOL + pr + EOL).c_str());
+                cprintf(("\n" + pr + "\n").c_str());
         }
     }
 }
@@ -825,11 +858,11 @@ static std::string _list_allowed_keys(char yes1, char yes2,
                                       bool allow_all = false)
 {
     std::string result = " [";
-                result += (lowered ? "y" : "Y");
+                result += (lowered ? "(y)es" : "(Y)es");
                 result += _list_alternative_yes(yes1, yes2, lowered);
                 if (allow_all)
-                    result += (lowered? "/a" : "/A");
-                result += (lowered ? "/n/q" : "/N/Q");
+                    result += (lowered? "/(a)ll" : "/(A)ll");
+                result += (lowered ? "/(n)o/(q)uit" : "/(N)o/(Q)uit");
                 result += "]";
 
     return (result);
@@ -838,11 +871,11 @@ static std::string _list_allowed_keys(char yes1, char yes2,
 // Like yesno(), but returns 0 for no, 1 for yes, and -1 for quit.
 // alt_yes and alt_yes2 allow up to two synonyms for 'Y'.
 // FIXME: This function is shaping up to be a monster. Help!
-int yesnoquit( const char* str, bool safe, int safeanswer, bool allow_all,
-               bool clear_after, char alt_yes, char alt_yes2 )
+int yesnoquit(const char* str, bool safe, int safeanswer, bool allow_all,
+               bool clear_after, char alt_yes, char alt_yes2)
 {
     if (!crawl_state.is_repeating_cmd())
-        interrupt_activity( AI_FORCE_INTERRUPT );
+        interrupt_activity(AI_FORCE_INTERRUPT);
 
     mouse_control mc(MOUSE_MODE_MORE);
 
@@ -856,7 +889,7 @@ int yesnoquit( const char* str, bool safe, int safeanswer, bool allow_all,
 
         int tmp = getchm(KMC_CONFIRM);
 
-        if (tmp == CK_ESCAPE || tmp == CONTROL('G') || tmp == 'q' || tmp == 'Q'
+        if (key_is_escape(tmp) || tmp == 'q' || tmp == 'Q'
             || crawl_state.seen_hups)
         {
             return -1;
@@ -869,7 +902,7 @@ int yesnoquit( const char* str, bool safe, int safeanswer, bool allow_all,
             || tmp == safeanswer
             || safe && Options.easy_confirm == CONFIRM_SAFE_EASY)
         {
-            tmp = toupper( tmp );
+            tmp = toupper(tmp);
         }
 
         if (clear_after)
@@ -884,12 +917,18 @@ int yesnoquit( const char* str, bool safe, int safeanswer, bool allow_all,
             if (tmp == 'A')
                 return 2;
             else
-                mprf("Choose [Y]es%s, [N]o, [Q]uit, or [A]ll!",
+            {
+                bool upper = (!safe && crawl_state.game_is_hints_tutorial());
+                mprf("Choose %s[Y]es%s, [N]o, [Q]uit, or [A]ll!",
+                     upper ? "uppercase " : "",
                      _list_alternative_yes(alt_yes, alt_yes2, false, true).c_str());
+            }
         }
         else
         {
-            mprf("[Y]es%s, [N]o or [Q]uit only, please.",
+            bool upper = (!safe && crawl_state.game_is_hints_tutorial());
+            mprf("%s[Y]es%s, [N]o or [Q]uit only, please.",
+                 upper ? "Uppercase " : "",
                  _list_alternative_yes(alt_yes, alt_yes2, false, true).c_str());
         }
     }
@@ -957,11 +996,6 @@ int near_stairs(const coord_def &p, int max_dist,
     return (false);
 }
 
-bool is_trap_square(dungeon_feature_type grid)
-{
-    return (grid >= DNGN_TRAP_MECHANICAL && grid <= DNGN_UNDISCOVERED_TRAP);
-}
-
 // Does the equivalent of KILL_RESET on all monsters in LOS. Should only be
 // applied to new games.
 void zap_los_monsters(bool items_also)
@@ -977,18 +1011,18 @@ void zap_los_monsters(bool items_also)
         {
             int item = igrd(*ri);
 
-            if (item != NON_ITEM && mitm[item].is_valid() )
+            if (item != NON_ITEM && mitm[item].defined())
                 destroy_item(item);
         }
 
         // If we ever allow starting with a friendly monster,
         // we'll have to check here.
-        monsters *mon = monster_at(*ri);
+        monster* mon = monster_at(*ri);
         if (mon == NULL || mons_class_flag(mon->type, M_NO_EXP_GAIN))
             continue;
 
         dprf("Dismissing %s",
-             mon->name(DESC_PLAIN, true).c_str() );
+             mon->name(DESC_PLAIN, true).c_str());
 
         // Do a hard reset so the monster's items will be discarded.
         mon->flags |= MF_HARD_RESET;
@@ -1028,3 +1062,64 @@ bool tobool(maybe_bool mb, bool def)
     }
 }
 
+coord_def get_random_stair()
+{
+    std::vector<coord_def> st;
+    for (rectangle_iterator ri(1); ri; ++ri)
+    {
+        const dungeon_feature_type feat = grd(*ri);
+        if (feat_is_travelable_stair(feat) && !feat_is_escape_hatch(feat))
+        {
+            st.push_back(*ri);
+        }
+    }
+    if (!st.size())
+        return coord_def();        // sanity check: shouldn't happen
+    return st[random2(st.size())];
+}
+
+
+//---------------------------------------------------------------
+//
+// prompt_for_quantity
+//
+// Returns -1 if ; or enter is pressed (pickup all).
+// Else, returns quantity.
+//---------------------------------------------------------------
+int prompt_for_quantity(const char *prompt)
+{
+    msgwin_prompt(prompt);
+
+    int ch = getch_ck();
+    if (ch == CK_ENTER || ch == ';')
+        return -1;
+
+    macro_buf_add(ch);
+    return prompt_for_int("", false);
+}
+
+//---------------------------------------------------------------
+//
+// prompt_for_int
+//
+// If nonneg, then it returns a non-negative number or -1 on fail
+// If !nonneg, then it returns an integer, and 0 on fail
+//
+//---------------------------------------------------------------
+int prompt_for_int(const char *prompt, bool nonneg)
+{
+    char specs[80];
+
+    msgwin_get_line(prompt, specs, sizeof(specs));
+
+    if (specs[0] == '\0')
+        return (nonneg ? -1 : 0);
+
+    char *end;
+    int   ret = strtol(specs, &end, 10);
+
+    if (ret < 0 && nonneg || ret == 0 && end == specs)
+        ret = (nonneg ? -1 : 0);
+
+    return (ret);
+}
