@@ -1,8 +1,7 @@
-/*
- *  File:       terrain.cc
- *  Summary:    Terrain related functions.
- *  Written by: Linley Henzell
- */
+/**
+ * @file
+ * @brief Terrain related functions.
+**/
 
 #include "AppHdr.h"
 
@@ -12,12 +11,14 @@
 #include <algorithm>
 #include <sstream>
 
+#include "areas.h"
 #include "cloud.h"
 #include "coordit.h"
 #include "dgn-overview.h"
 #include "dgnevent.h"
 #include "directn.h"
 #include "map_knowledge.h"
+#include "feature.h"
 #include "fprop.h"
 #include "godabil.h"
 #include "itemprop.h"
@@ -33,13 +34,18 @@
 #include "player.h"
 #include "random.h"
 #include "religion.h"
-#include "spells3.h"
+#include "species.h"
+#include "spl-transloc.h"
 #include "stuff.h"
 #include "env.h"
+#ifdef USE_TILE
+ #include "tileview.h"
+#endif
 #include "travel.h"
 #include "transform.h"
 #include "traps.h"
 #include "view.h"
+#include "viewchar.h"
 
 actor* actor_at(const coord_def& c)
 {
@@ -50,6 +56,32 @@ actor* actor_at(const coord_def& c)
     return (monster_at(c));
 }
 
+int count_neighbours_with_func (const coord_def& c, bool (*checker)(dungeon_feature_type))
+{
+    int count = 0;
+    for (adjacent_iterator ai(c); ai; ++ai)
+    {
+        if (checker(grd(*ai)))
+            count++;
+    }
+    return count;
+}
+
+bool feat_is_test (dungeon_feature_type feat, bool (*checker)(dungeon_feature_type))
+{
+    return (checker(feat));
+}
+
+bool feat_is_test (const coord_def& c, bool (*checker)(dungeon_feature_type))
+{
+    return (checker(grd(c)));
+}
+
+bool feat_is_malign_gateway_suitable (dungeon_feature_type feat)
+{
+    return (feat == DNGN_FLOOR || feat == DNGN_SHALLOW_WATER);
+}
+
 bool feat_is_wall(dungeon_feature_type feat)
 {
     return (feat >= DNGN_MINWALL && feat <= DNGN_MAXWALL);
@@ -120,7 +152,8 @@ bool feat_sealable_portal(dungeon_feature_type feat)
 
 bool feat_is_portal(dungeon_feature_type feat)
 {
-    return (feat == DNGN_ENTER_PORTAL_VAULT || feat == DNGN_EXIT_PORTAL_VAULT);
+    return (feat == DNGN_ENTER_PORTAL_VAULT || feat == DNGN_EXIT_PORTAL_VAULT
+            || feat == DNGN_TEMP_PORTAL);
 }
 
 // Returns true if the given dungeon feature is a stair, i.e., a level
@@ -150,6 +183,7 @@ bool feat_is_travelable_stair(dungeon_feature_type feat)
     case DNGN_ENTER_GEHENNA:
     case DNGN_ENTER_COCYTUS:
     case DNGN_ENTER_TARTARUS:
+    case DNGN_ENTER_DWARVEN_HALL:
     case DNGN_ENTER_ORCISH_MINES:
     case DNGN_ENTER_HIVE:
     case DNGN_ENTER_LAIR:
@@ -164,6 +198,7 @@ bool feat_is_travelable_stair(dungeon_feature_type feat)
     case DNGN_ENTER_TOMB:
     case DNGN_ENTER_SWAMP:
     case DNGN_ENTER_SHOALS:
+    case DNGN_RETURN_FROM_DWARVEN_HALL:
     case DNGN_RETURN_FROM_ORCISH_MINES:
     case DNGN_RETURN_FROM_HIVE:
     case DNGN_RETURN_FROM_LAIR:
@@ -246,6 +281,7 @@ command_type feat_stair_direction(dungeon_feature_type feat)
     case DNGN_STONE_STAIRS_UP_II:
     case DNGN_STONE_STAIRS_UP_III:
     case DNGN_ESCAPE_HATCH_UP:
+    case DNGN_RETURN_FROM_DWARVEN_HALL:
     case DNGN_RETURN_FROM_ORCISH_MINES:
     case DNGN_RETURN_FROM_HIVE:
     case DNGN_RETURN_FROM_LAIR:
@@ -281,6 +317,7 @@ command_type feat_stair_direction(dungeon_feature_type feat)
     case DNGN_ENTER_PANDEMONIUM:
     case DNGN_EXIT_PANDEMONIUM:
     case DNGN_TRANSIT_PANDEMONIUM:
+    case DNGN_ENTER_DWARVEN_HALL:
     case DNGN_ENTER_ORCISH_MINES:
     case DNGN_ENTER_HIVE:
     case DNGN_ENTER_LAIR:
@@ -351,7 +388,7 @@ bool feat_is_secret_door(dungeon_feature_type feat)
 
 bool feat_is_statue_or_idol(dungeon_feature_type feat)
 {
-    return (feat >= DNGN_ORCISH_IDOL && feat <= DNGN_STATUE_RESERVED);
+    return (feat >= DNGN_ORCISH_IDOL && feat <= DNGN_GRANITE_STATUE);
 }
 
 bool feat_is_rock(dungeon_feature_type feat)
@@ -380,6 +417,7 @@ bool feat_is_water(dungeon_feature_type feat)
     return (feat == DNGN_SHALLOW_WATER
             || feat == DNGN_DEEP_WATER
             || feat == DNGN_OPEN_SEA
+            || feat == DNGN_SWAMP_TREE
             || feat == DNGN_WATER_RESERVED);
 }
 
@@ -427,6 +465,20 @@ bool feat_is_branch_stairs(dungeon_feature_type feat)
             || (feat >= DNGN_ENTER_DIS && feat <= DNGN_ENTER_TARTARUS));
 }
 
+bool feat_is_tree(dungeon_feature_type feat)
+{
+    return feat == DNGN_TREE || feat == DNGN_SWAMP_TREE;
+}
+
+bool feat_is_bidirectional_portal(dungeon_feature_type feat)
+{
+    return (get_feature_dchar(feat) == DCHAR_ARCH
+            && feat_stair_direction(feat) != CMD_NO_CMD
+            && feat != DNGN_ENTER_ZOT
+            && feat != DNGN_RETURN_FROM_ZOT
+            && feat != DNGN_EXIT_HELL);
+}
+
 // Find all connected cells containing ft, starting at d.
 void find_connected_identical(const coord_def &d, dungeon_feature_type ft,
                               std::set<coord_def>& out)
@@ -455,9 +507,10 @@ void find_connected_identical(const coord_def &d, dungeon_feature_type ft,
 }
 
 // Find all connected cells containing ft_min to ft_max, starting at d.
-void find_connected_range(const coord_def& d, dungeon_feature_type ft_min,
-                          dungeon_feature_type ft_max,
-                          std::set<coord_def>& out)
+static void _find_connected_range(const coord_def& d,
+                                  dungeon_feature_type ft_min,
+                                  dungeon_feature_type ft_max,
+                                  std::set<coord_def>& out)
 {
     if (grd(d) < ft_min || grd(d) > ft_max)
         return;
@@ -475,17 +528,17 @@ void find_connected_range(const coord_def& d, dungeon_feature_type ft_min,
 
     if (out.insert(d).second)
     {
-        find_connected_range(coord_def(d.x+1, d.y), ft_min, ft_max, out);
-        find_connected_range(coord_def(d.x-1, d.y), ft_min, ft_max, out);
-        find_connected_range(coord_def(d.x, d.y+1), ft_min, ft_max, out);
-        find_connected_range(coord_def(d.x, d.y-1), ft_min, ft_max, out);
+        _find_connected_range(coord_def(d.x+1, d.y), ft_min, ft_max, out);
+        _find_connected_range(coord_def(d.x-1, d.y), ft_min, ft_max, out);
+        _find_connected_range(coord_def(d.x, d.y+1), ft_min, ft_max, out);
+        _find_connected_range(coord_def(d.x, d.y-1), ft_min, ft_max, out);
     }
 }
 
 std::set<coord_def> connected_doors(const coord_def& d)
 {
     std::set<coord_def> doors;
-    find_connected_range(d, DNGN_CLOSED_DOOR, DNGN_SECRET_DOOR, doors);
+    _find_connected_range(d, DNGN_CLOSED_DOOR, DNGN_SECRET_DOOR, doors);
     return (doors);
 }
 
@@ -524,12 +577,9 @@ bool find_secret_door_info(const coord_def &where,
                            dungeon_feature_type *appearance,
                            coord_def *gc)
 {
-    std::set<coord_def>           doors;
+    std::set<coord_def> doors = connected_doors(where);
     std::set<coord_def>::iterator it;
 
-    find_connected_range(where, DNGN_CLOSED_DOOR, DNGN_SECRET_DOOR,
-                         doors);
-
     dungeon_feature_type feat = DNGN_FLOOR;
     coord_def loc = where;
 
@@ -580,6 +630,52 @@ dungeon_feature_type grid_secret_door_appearance(const coord_def &where)
     return (feat);
 }
 
+
+typedef FixedArray<bool, GXM, GYM> map_mask_boolean;
+static std::auto_ptr<map_mask_boolean> _slime_wall_precomputed_neighbour_mask;
+
+static void _precompute_slime_wall_neighbours()
+{
+    map_mask_boolean &mask(*_slime_wall_precomputed_neighbour_mask.get());
+    for (rectangle_iterator ri(1); ri; ++ri)
+    {
+        if (grd(*ri) == DNGN_SLIMY_WALL)
+        {
+            for (adjacent_iterator ai(*ri); ai; ++ai)
+                mask(*ai) = true;
+        }
+    }
+}
+
+unwind_slime_wall_precomputer::unwind_slime_wall_precomputer(bool docompute)
+    : did_compute_mask(false)
+{
+    if (docompute && !_slime_wall_precomputed_neighbour_mask.get())
+    {
+        did_compute_mask = true;
+        _slime_wall_precomputed_neighbour_mask.reset(
+            new map_mask_boolean(false));
+        _precompute_slime_wall_neighbours();
+    }
+}
+
+unwind_slime_wall_precomputer::~unwind_slime_wall_precomputer()
+{
+    if (did_compute_mask)
+        _slime_wall_precomputed_neighbour_mask.reset(NULL);
+}
+
+bool slime_wall_neighbour(const coord_def& c)
+{
+    if (_slime_wall_precomputed_neighbour_mask.get())
+        return (*_slime_wall_precomputed_neighbour_mask)(c);
+
+    for (adjacent_iterator ai(c); ai; ++ai)
+        if (env.grid(*ai) == DNGN_SLIMY_WALL)
+            return (true);
+    return (false);
+}
+
 bool feat_destroys_item(dungeon_feature_type feat, const item_def &item,
                         bool noisy)
 {
@@ -587,9 +683,6 @@ bool feat_destroys_item(dungeon_feature_type feat, const item_def &item,
     {
     case DNGN_SHALLOW_WATER:
     case DNGN_DEEP_WATER:
-        if (is_rune(item) && item.plus == RUNE_ABYSSAL)
-            return (true);
-
         if (noisy)
             mprf(MSGCH_SOUND, "You hear a splash.");
         return (false);
@@ -597,7 +690,34 @@ bool feat_destroys_item(dungeon_feature_type feat, const item_def &item,
     case DNGN_LAVA:
         if (noisy)
             mprf(MSGCH_SOUND, "You hear a sizzling splash.");
-        return (item.base_type == OBJ_SCROLLS);
+        return (true);
+
+    default:
+        return (false);
+    }
+}
+
+// For checking whether items would be inaccessible when they wouldn't technically be
+// destroyed - ignores Merfolk/Fedhas ability to access items in deep water.
+bool feat_virtually_destroys_item(dungeon_feature_type feat, const item_def &item,
+                                  bool noisy)
+{
+    switch (feat)
+    {
+    case DNGN_SHALLOW_WATER:
+        if (noisy)
+            mprf(MSGCH_SOUND, "You hear a splash.");
+        return (false);
+
+    case DNGN_DEEP_WATER:
+        if (noisy)
+        mprf(MSGCH_SOUND, "You hear a splash.");
+        return (true);
+
+    case DNGN_LAVA:
+        if (noisy)
+            mprf(MSGCH_SOUND, "You hear a sizzling splash.");
+        return (true);
 
     default:
         return (false);
@@ -675,7 +795,8 @@ static bool _dgn_shift_item(const coord_def &pos, item_def &item)
 bool is_critical_feature(dungeon_feature_type feat)
 {
     return (feat_stair_direction(feat) != CMD_NO_CMD
-            || feat_altar_god(feat) != GOD_NO_GOD);
+            || feat_altar_god(feat) != GOD_NO_GOD
+            || feat == DNGN_TEMP_PORTAL);
 }
 
 static bool _is_feature_shift_target(const coord_def &pos, void*)
@@ -683,6 +804,112 @@ static bool _is_feature_shift_target(const coord_def &pos, void*)
     return (grd(pos) == DNGN_FLOOR && !dungeon_events.has_listeners_at(pos));
 }
 
+// Moves everything at src to dst. This is not a swap operation: src
+// will be left with the same feature it started with, and should be
+// overwritten with something new.
+//
+// Things that are moved:
+// 1. Dungeon terrain (set to DNGN_UNSEEN)
+// 2. Actors (including the player)
+// 3. Items
+// 4. Clouds
+// 5. Terrain properties
+// 6. Terrain colours
+// 7. Vault (map) mask
+// 8. Vault id mask
+// 9. Map markers, dungeon listeners, shopping list
+void dgn_move_entities_at(coord_def src, coord_def dst,
+                          bool move_player,
+                          bool move_monster,
+                          bool move_items)
+{
+    if (!in_bounds(dst) || !in_bounds(src) || src == dst)
+        return;
+
+    // Move terrain.
+    if (you.level_type == LEVEL_DUNGEON)
+        move_notable_thing(src, dst);
+
+    dungeon_feature_type dfeat = grd(src);
+    if (dfeat == DNGN_ENTER_SHOP)
+    {
+        if (shop_struct *s = get_shop(src))
+        {
+            env.tgrid(dst)    = env.tgrid(s->pos);
+            env.tgrid(s->pos) = NON_ENTITY;
+            // Can't leave the source square as a shop now that all
+            // the bookkeeping data has moved.
+            grd(src)          = DNGN_FLOOR;
+            s->pos = dst;
+        }
+        else // Destroy invalid shops.
+            dfeat = DNGN_FLOOR;
+    }
+    else if (feat_is_trap(dfeat, true))
+    {
+        if (trap_def *trap = find_trap(src))
+        {
+            env.tgrid(dst) = env.tgrid(trap->pos);
+            env.tgrid(trap->pos) = NON_ENTITY;
+            // Can't leave the source square as a trap now that all
+            // the bookkeeping data has moved.
+            grd(src)          = DNGN_FLOOR;
+            trap->pos = dst;
+        }
+        else // Destroy invalid traps.
+            dfeat = DNGN_FLOOR;
+    }
+
+    grd(dst) = dfeat;
+
+    if (move_monster)
+    {
+        if (monster* mon = monster_at(src))
+        {
+            mon->moveto(dst);
+            if (mon->type == MONS_ELDRITCH_TENTACLE)
+            {
+                if (mon->props.exists("base_position"))
+                {
+                    coord_def delta = dst - src;
+                    coord_def base_pos = mon->props["base_position"].get_coord();
+                    base_pos += delta;
+                    mon->props["base_position"].get_coord() = base_pos;
+                }
+
+            }
+            mgrd(dst) = mgrd(src);
+            mgrd(src) = NON_MONSTER;
+        }
+    }
+
+    if (move_player && you.pos() == src)
+        you.shiftto(dst);
+
+    if (move_items)
+        move_item_stack_to_grid(src, dst);
+
+    move_cloud_to(src, dst);
+
+    // Move terrain colours and properties.
+    env.pgrid(dst) = env.pgrid(src);
+    env.grid_colours(dst) = env.grid_colours(src);
+#ifdef USE_TILE
+    env.tile_bk_fg(dst) = env.tile_bk_fg(src);
+    env.tile_bk_bg(dst) = env.tile_bk_bg(src);
+    env.tile_flv(dst) = env.tile_flv(src);
+#endif
+
+    // Move vault masks.
+    env.level_map_mask(dst) = env.level_map_mask(src);
+    env.level_map_ids(dst) = env.level_map_ids(src);
+
+    // Move markers, dungeon listeners and shopping list.
+    env.markers.move(src, dst);
+    dungeon_events.move_listeners(src, dst);
+    shopping_list.move_things(src, dst);
+}
+
 static bool _dgn_shift_feature(const coord_def &pos)
 {
     const dungeon_feature_type dfeat = grd(pos);
@@ -692,18 +919,7 @@ static bool _dgn_shift_feature(const coord_def &pos)
     const coord_def dest =
         _dgn_find_nearest_square(pos, NULL, _is_feature_shift_target);
 
-    if (in_bounds(dest) && dest != pos)
-    {
-        grd(dest) = dfeat;
-
-        if (dfeat == DNGN_ENTER_SHOP)
-            if (shop_struct *s = get_shop(pos))
-                s->pos = dest;
-
-        env.markers.move(pos, dest);
-        dungeon_events.move_listeners(pos, dest);
-        shopping_list.move_things(pos, dest);
-    }
+    dgn_move_entities_at(pos, dest, false, false, false);
     return (true);
 }
 
@@ -712,7 +928,6 @@ static void _dgn_check_terrain_items(const coord_def &pos, bool preserve_items)
     const dungeon_feature_type feat = grd(pos);
 
     int item = igrd(pos);
-    bool did_destroy = false;
     while (item != NON_ITEM)
     {
         const int curr = item;
@@ -729,14 +944,13 @@ static void _dgn_check_terrain_items(const coord_def &pos, bool preserve_items)
             feat_destroys_item(feat, mitm[curr], true);
             item_was_destroyed(mitm[curr]);
             destroy_item(curr);
-            did_destroy = true;
         }
     }
 }
 
 static void _dgn_check_terrain_monsters(const coord_def &pos)
 {
-    if (monsters* m = monster_at(pos))
+    if (monster* m = monster_at(pos))
         m->apply_location_effects(pos);
 }
 
@@ -748,7 +962,7 @@ static void _dgn_check_terrain_covering(const coord_def &pos,
                                      dungeon_feature_type new_feat)
 {
     if (!testbits(env.pgrid(pos), FPROP_BLOODY)
-        && !testbits(env.pgrid(pos), FPROP_MOLD))
+        && !is_moldy(pos))
     {
         return;
     }
@@ -760,7 +974,7 @@ static void _dgn_check_terrain_covering(const coord_def &pos,
         if (old_feat != DNGN_FLOOR && !feat_is_solid(old_feat))
         {
             env.pgrid(pos) &= ~(FPROP_BLOODY);
-            env.pgrid(pos) &= ~(FPROP_MOLD);
+            remove_mold(pos);
         }
     }
     else
@@ -770,12 +984,12 @@ static void _dgn_check_terrain_covering(const coord_def &pos,
             || is_critical_feature(new_feat))
         {
             env.pgrid(pos) &= ~(FPROP_BLOODY);
-            env.pgrid(pos) &= ~(FPROP_MOLD);
+            remove_mold(pos);
         }
     }
 }
 
-void _dgn_check_terrain_player(const coord_def pos)
+static void _dgn_check_terrain_player(const coord_def pos)
 {
     if (pos != you.pos())
         return;
@@ -785,10 +999,10 @@ void _dgn_check_terrain_player(const coord_def pos)
         // If the monster can't stay submerged in the new terrain and
         // there aren't any adjacent squares where it can stay
         // submerged then move it.
-        monsters* mon = monster_at(pos);
+        monster* mon = monster_at(pos);
         if (mon && !mon->submerged())
             monster_teleport(mon, true, false);
-        move_player_to_grid(pos, false, true, true);
+        move_player_to_grid(pos, false, true);
     }
     else
         you_teleport_now(true, false);
@@ -811,6 +1025,9 @@ void dungeon_terrain_changed(const coord_def &pos,
             _dgn_shift_feature(pos);
 
         unnotice_feature(level_pos(level_id::current(), pos));
+        if (grd(pos) == DNGN_ENTER_SHOP)
+            StashTrack.remove_shop(pos);
+
         grd(pos) = nfeat;
         env.grid_colours(pos) = BLACK;
         if (is_notable_terrain(nfeat) && you.see_cell(pos))
@@ -849,7 +1066,7 @@ static void _announce_swap_real(coord_def orig_pos, coord_def dest_pos)
     std::string orig_actor, dest_actor;
     if (orig_pos == you.pos())
         orig_actor = "you";
-    else if (const monsters *m = monster_at(orig_pos))
+    else if (const monster* m = monster_at(orig_pos))
     {
         if (you.can_see(m))
             orig_actor = m->name(DESC_NOCAP_THE);
@@ -857,7 +1074,7 @@ static void _announce_swap_real(coord_def orig_pos, coord_def dest_pos)
 
     if (dest_pos == you.pos())
         dest_actor = "you";
-    else if (const monsters *m = monster_at(dest_pos))
+    else if (const monster* m = monster_at(dest_pos))
     {
         if (you.can_see(m))
             dest_actor = m->name(DESC_NOCAP_THE);
@@ -932,13 +1149,13 @@ bool swap_features(const coord_def &pos1, const coord_def &pos2,
     const dungeon_feature_type feat2 = grd(pos2);
 
     if (is_notable_terrain(feat1) && !you.see_cell(pos1)
-        && is_terrain_known(pos1))
+        && env.map_knowledge(pos1).known())
     {
         return (false);
     }
 
     if (is_notable_terrain(feat2) && !you.see_cell(pos2)
-        && is_terrain_known(pos2))
+        && env.map_knowledge(pos2).known())
     {
         return (false);
     }
@@ -1067,12 +1284,12 @@ bool swap_features(const coord_def &pos1, const coord_def &pos2,
     if (pos1 == you.pos())
     {
         you.set_position(pos2);
-        viewwindow(false);
+        viewwindow();
     }
     else if (pos2 == you.pos())
     {
         you.set_position(pos1);
-        viewwindow(false);
+        viewwindow();
     }
 
     set_terrain_changed(pos1);
@@ -1140,16 +1357,18 @@ bool slide_feature_over(const coord_def &src, coord_def prefered_dest,
 }
 
 // Returns true if we manage to scramble free.
-bool fall_into_a_pool( const coord_def& entry, bool allow_shift,
-                       unsigned char terrain )
+bool fall_into_a_pool(const coord_def& entry, bool allow_shift,
+                       dungeon_feature_type terrain)
 {
     bool escape = false;
+    bool clinging = false;
     coord_def empty;
 
-    if (you.species == SP_MERFOLK && terrain == DNGN_DEEP_WATER)
+    if (species_likes_water(you.species) && terrain == DNGN_DEEP_WATER
+        && !form_likes_water() && !you.transform_uncancellable)
     {
         // These can happen when we enter deep water directly -- bwr
-        merfolk_start_swimming();
+        emergency_untransform();
         return (false);
     }
 
@@ -1171,17 +1390,17 @@ bool fall_into_a_pool( const coord_def& entry, bool allow_shift,
 
         if (resist <= 0)
         {
-            mpr( "The lava burns you to a cinder!" );
+            mpr("The lava burns you to a cinder!");
             ouch(INSTANT_DEATH, NON_MONSTER, KILLED_BY_LAVA);
         }
         else
         {
             // should boost # of bangs per damage in the future {dlb}
-            mpr( "The lava burns you!" );
+            mpr("The lava burns you!");
             ouch((10 + roll_dice(2, 50)) / resist, NON_MONSTER, KILLED_BY_LAVA);
         }
 
-        expose_player_to_element( BEAM_LAVA, 14 );
+        expose_player_to_element(BEAM_LAVA, 14);
     }
 
     // a distinction between stepping and falling from you.duration[DUR_LEVITATION]
@@ -1190,7 +1409,9 @@ bool fall_into_a_pool( const coord_def& entry, bool allow_shift,
     {
         if (allow_shift)
         {
-            escape = empty_surrounds(you.pos(), DNGN_FLOOR, 1, false, empty);
+            escape = empty_surrounds(you.pos(), DNGN_FLOOR, 1, false, empty)
+                     || you.check_clinging(false);
+            clinging = you.is_wall_clinging();
         }
         else
         {
@@ -1209,7 +1430,7 @@ bool fall_into_a_pool( const coord_def& entry, bool allow_shift,
     }
     else
     {
-        if (you.attribute[ATTR_TRANSFORMATION] == TRAN_STATUE)
+        if (you.form == TRAN_STATUE)
             mpr("You sink like a stone!");
         else
             mpr("You try to escape, but your burden drags you down!");
@@ -1217,13 +1438,14 @@ bool fall_into_a_pool( const coord_def& entry, bool allow_shift,
 
     if (escape)
     {
-        if (in_bounds(empty) && !is_feat_dangerous(grd(empty)))
+        if (in_bounds(empty) && !is_feat_dangerous(grd(empty)) || clinging)
         {
             mpr("You manage to scramble free!");
-            move_player_to_grid( empty, false, false, true );
+            if (!clinging)
+                move_player_to_grid(empty, false, false);
 
             if (terrain == DNGN_LAVA)
-                expose_player_to_element( BEAM_LAVA, 14 );
+                expose_player_to_element(BEAM_LAVA, 14);
 
             return (true);
         }
@@ -1353,16 +1575,15 @@ std::string stair_climb_verb(dungeon_feature_type feat)
 const char *dngn_feature_names[] =
 {
 "unseen", "closed_door", "detected_secret_door", "secret_door",
-"wax_wall", "metal_wall", "green_crystal_wall", "rock_wall", "stone_wall",
-"permarock_wall",
-"clear_rock_wall", "clear_stone_wall", "clear_permarock_wall", "open_sea",
-"trees", "orcish_idol", "", "", "", "", "",
-"granite_statue", "statue_reserved_1", "statue_reserved_2",
-"", "", "", "", "", "", "", "",
+"wax_wall", "metal_wall", "green_crystal_wall", "rock_wall",
+"slimy_wall", "stone_wall", "permarock_wall",
+"clear_rock_wall", "clear_stone_wall", "clear_permarock_wall", "iron_grate",
+"open_sea", "tree", "orcish_idol", "swamp_tree", "", "",
+"granite_statue", "", "", "", "", "", "", "", "", "", "",
 "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "",
 "", "", "", "", "", "", "", "", "", "", "", "", "", "lava",
 "deep_water", "", "", "shallow_water", "water_stuck", "floor",
-"floor_special", "floor_reserved", "exit_hell", "enter_hell",
+"", "floor_reserved", "exit_hell", "enter_hell",
 "open_door", "", "", "trap_mechanical", "trap_magical", "trap_natural",
 "undiscovered_trap", "", "enter_shop", "enter_labyrinth",
 "stone_stairs_down_i", "stone_stairs_down_ii",
@@ -1371,31 +1592,33 @@ const char *dngn_feature_names[] =
 "", "enter_dis", "enter_gehenna", "enter_cocytus",
 "enter_tartarus", "enter_abyss", "exit_abyss", "stone_arch",
 "enter_pandemonium", "exit_pandemonium", "transit_pandemonium",
-"", "", "", "builder_special_wall", "builder_special_floor", "",
-"", "", "enter_orcish_mines", "enter_hive", "enter_lair",
+"", "", "", "", "", "",
+"", "", "enter_dwarven_hall", "enter_orcish_mines", "enter_hive", "enter_lair",
 "enter_slime_pits", "enter_vaults", "enter_crypt",
 "enter_hall_of_blades", "enter_zot", "enter_temple",
 "enter_snake_pit", "enter_elven_halls", "enter_tomb",
 "enter_swamp", "enter_shoals", "enter_reserved_2",
-"enter_reserved_3", "enter_reserved_4", "", "", "",
-"return_from_orcish_mines", "return_from_hive",
+"enter_reserved_3", "enter_reserved_4", "", "",
+"return_from_dwarven_hall", "return_from_orcish_mines", "return_from_hive",
 "return_from_lair", "return_from_slime_pits",
 "return_from_vaults", "return_from_crypt",
 "return_from_hall_of_blades", "return_from_zot",
 "return_from_temple", "return_from_snake_pit",
 "return_from_elven_halls", "return_from_tomb",
 "return_from_swamp", "return_from_shoals", "return_reserved_2",
-"return_reserved_3", "return_reserved_4", "", "", "", "", "", "",
+"return_reserved_3", "return_reserved_4", "", "", "", "", "",
 "", "", "", "", "", "", "", "enter_portal_vault", "exit_portal_vault",
 "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "",
 "", "", "altar_zin", "altar_shining_one", "altar_kikubaaqudgha",
 "altar_yredelemnul", "altar_xom", "altar_vehumet",
 "altar_okawaru", "altar_makhleb", "altar_sif_muna", "altar_trog",
 "altar_nemelex_xobeh", "altar_elyvilon", "altar_lugonu",
-"altar_beogh", "altar_jiyva", "altar_fedhas", "altar_cheibriados", "", "", "",
+"altar_beogh", "altar_jiyva", "altar_fedhas", "altar_cheibriados",
+"altar_ashenzari", "", "",
 "fountain_blue", "fountain_sparkling", "fountain_blood",
 "dry_fountain_blue", "dry_fountain_sparkling", "dry_fountain_blood",
-"permadry_fountain", "abandoned_shop"
+"permadry_fountain", "abandoned_shop", "", "",
+"explore_horizon",
 };
 
 dungeon_feature_type dungeon_feature_by_name(const std::string &name)
@@ -1442,3 +1665,76 @@ const char *dungeon_feature_name(dungeon_feature_type rfeat)
 
     return dngn_feature_names[feat];
 }
+
+void nuke_wall(const coord_def& p)
+{
+    if (!in_bounds(p))
+        return;
+
+    // Blood does not transfer onto floor.
+    if (is_bloodcovered(p))
+        env.pgrid(p) &= ~(FPROP_BLOODY);
+
+    remove_mold(p);
+
+    grd(p) = (grd(p) == DNGN_SWAMP_TREE) ? DNGN_SHALLOW_WATER : DNGN_FLOOR;
+    set_terrain_changed(p);
+}
+
+/*
+ * Check if an actor can cling to a cell.
+ *
+ * Wall clinging is done only on orthogonal walls.
+ *
+ * @param pos The coordinates of the cell.
+ *
+ * @return Whether the cell is clingable.
+ */
+bool cell_is_clingable(const coord_def pos)
+{
+    for (orth_adjacent_iterator ai(pos); ai; ++ai)
+        if (feat_is_wall(env.grid(*ai)) || feat_is_closed_door(env.grid(*ai)))
+            return true;
+
+    return false;
+}
+
+/*
+ * Check if an actor can cling from a cell to another.
+ *
+ * "clinging" to a wall means being orthogonally (left, right, up, down) next
+ * to it. A spider can cling to several squares. A move is allowed if the
+ * spider clings to an adjacent wall square or the same wall square before and
+ * after moving. Being over floor or shallow water and next to a wall counts as
+ * clinging to that wall (no further action needed).
+ *
+ * Example:
+ * ~ = deep water
+ * * = deep water the spider can reach
+ *
+ *  #####
+ *  ~~#~~
+ *  ~~~*~
+ *  **s#*
+ *  #####
+ *
+ * Look at Mantis #2704 for more examples.
+ *
+ * @param from The coordinates of the starting position.
+ * @param to The coordinates of the destination.
+ *
+ * @return Whether it is possible to cling from one cell to another.
+ */
+bool cell_can_cling_to(const coord_def& from, const coord_def to)
+{
+    if (!in_bounds(to))
+        return false;
+
+    for (orth_adjacent_iterator ai(from); ai; ++ai)
+        if (feat_is_wall(env.grid(*ai)))
+            for (orth_adjacent_iterator ai2(to, false); ai2; ++ai2)
+                if (feat_is_wall(env.grid(*ai2)) && distance(*ai, *ai2) <= 1)
+                    return true;
+
+        return false;
+}
