@@ -1,14 +1,15 @@
 #include "AppHdr.h"
 
 #ifdef USE_TILE
+#include "tilemcache.h"
 
 #include "env.h"
-#include "tilemcache.h"
-#include "tags.h"
 #include "ghost.h"
 #include "mon-util.h"
-
+#include "tags.h"
 #include "tiledef-player.h"
+#include "tilepick.h"
+#include "tilepick-p.h"
 
 mcache_manager mcache;
 
@@ -21,34 +22,43 @@ enum mcache_type
     MCACHE_DEMON,
     MCACHE_MAX,
 
-    MCACHE_NULL
+    MCACHE_NULL,
+};
+
+struct demon_data
+{
+    demon_data() { head = body = wings = 0; }
+
+    tileidx_t head;
+    tileidx_t body;
+    tileidx_t wings;
 };
 
 // Custom marshall/unmarshall functions.
 static void unmarshallDoll(reader &th, dolls_data &doll)
 {
     for (unsigned int i = 0; i < TILEP_PART_MAX; i++)
-        doll.parts[i] = unmarshallLong(th);
+        doll.parts[i] = unmarshallInt(th);
 }
 
 static void marshallDoll(writer &th, const dolls_data &doll)
 {
     for (unsigned int i = 0; i < TILEP_PART_MAX; i++)
-        marshallLong(th, doll.parts[i]);
+        marshallInt(th, doll.parts[i]);
 }
 
 static void unmarshallDemon(reader &th, demon_data &demon)
 {
-    demon.head = unmarshallLong(th);
-    demon.body = unmarshallLong(th);
-    demon.wings = unmarshallLong(th);
+    demon.head = unmarshallInt(th);
+    demon.body = unmarshallInt(th);
+    demon.wings = unmarshallInt(th);
 }
 
 static void marshallDemon(writer &th, const demon_data &demon)
 {
-    marshallLong(th, demon.head);
-    marshallLong(th, demon.body);
-    marshallLong(th, demon.wings);
+    marshallInt(th, demon.head);
+    marshallInt(th, demon.body);
+    marshallInt(th, demon.wings);
 }
 
 // Internal mcache classes.  The mcache_manager creates these internally.
@@ -58,49 +68,49 @@ static void marshallDemon(writer &th, const demon_data &demon)
 class mcache_monster : public mcache_entry
 {
 public:
-    mcache_monster(const monsters *mon);
+    mcache_monster(const monster* mon);
     mcache_monster(reader &th);
 
-    virtual unsigned int info(tile_draw_info *dinfo) const;
+    virtual int info(tile_draw_info *dinfo) const;
 
-    static bool valid(const monsters *mon);
+    static bool valid(const monster* mon);
 
-    static bool get_weapon_offset(int mon_tile, int &ofs_x, int &ofs_y);
+    static bool get_weapon_offset(tileidx_t mon_tile, int *ofs_x, int *ofs_y);
 
     virtual void construct(writer &th);
 
 protected:
-    int m_mon_tile;
-    int m_equ_tile;
+    tileidx_t m_mon_tile;
+    tileidx_t m_equ_tile;
 };
 
 class mcache_draco : public mcache_entry
 {
 public:
-    mcache_draco(const monsters *mon);
+    mcache_draco(const monster* mon);
     mcache_draco(reader &th);
 
-    virtual unsigned int info(tile_draw_info *dinfo) const;
+    virtual int info(tile_draw_info *dinfo) const;
 
-    static bool valid(const monsters *mon);
+    static bool valid(const monster* mon);
 
     virtual void construct(writer &th);
 
 protected:
-    int m_mon_tile;
-    int m_job_tile;
-    int m_equ_tile;
+    tileidx_t m_mon_tile;
+    tileidx_t m_job_tile;
+    tileidx_t m_equ_tile;
 };
 
 class mcache_ghost : public mcache_entry
 {
 public:
-    mcache_ghost(const monsters *mon);
+    mcache_ghost(const monster* mon);
     mcache_ghost(reader &th);
 
     virtual const dolls_data *doll() const;
 
-    static bool valid(const monsters *mon);
+    static bool valid(const monster* mon);
 
     virtual void construct(writer &th);
 
@@ -113,12 +123,12 @@ protected:
 class mcache_demon : public mcache_entry
 {
 public:
-    mcache_demon(const monsters *mon);
+    mcache_demon(const monster* mon);
     mcache_demon(reader &th);
 
-    virtual unsigned int info(tile_draw_info *dinfo) const;
+    virtual int info(tile_draw_info *dinfo) const;
 
-    static bool valid(const monsters *mon);
+    static bool valid(const monster* mon);
 
     virtual void construct(writer &th);
 
@@ -129,7 +139,7 @@ protected:
 /////////////////////////////////////////////////////////////////////////////
 // tile_fg_store
 
-unsigned int tile_fg_store::operator=(unsigned int tile)
+tileidx_t tile_fg_store::operator=(tileidx_t tile)
 {
     if ((tile & TILE_FLAG_MASK) == (m_tile & TILE_FLAG_MASK))
     {
@@ -160,9 +170,9 @@ mcache_manager::~mcache_manager()
     clear_all();
 }
 
-unsigned int mcache_manager::register_monster(const monsters *mon)
+unsigned int mcache_manager::register_monster(const monster* mon)
 {
-    assert(mon);
+    ASSERT(mon);
     if (!mon)
         return 0;
 
@@ -182,7 +192,7 @@ unsigned int mcache_manager::register_monster(const monsters *mon)
     else
         return 0;
 
-    unsigned int idx = ~0;
+    tileidx_t idx = ~0;
 
     for (unsigned int i = 0; i < m_entries.size(); i++)
     {
@@ -223,14 +233,14 @@ void mcache_manager::clear_all()
     m_entries.resize(0);
 }
 
-mcache_entry *mcache_manager::get(unsigned int tile)
+mcache_entry *mcache_manager::get(tileidx_t tile)
 {
-    unsigned int idx = tile & TILE_FLAG_MASK;
+    tileidx_t idx = tile & TILE_FLAG_MASK;
     if (idx < TILEP_MCACHE_START)
-        return NULL;
+        return (NULL);
 
     if (idx >= TILEP_MCACHE_START + m_entries.size())
-        return NULL;
+        return (NULL);
 
     mcache_entry *entry = m_entries[idx - TILEP_MCACHE_START];
     return (entry);
@@ -238,7 +248,7 @@ mcache_entry *mcache_manager::get(unsigned int tile)
 
 void mcache_manager::read(reader &th)
 {
-    unsigned int size = unmarshallLong(th);
+    unsigned int size = unmarshallInt(th);
     m_entries.reserve(size);
     m_entries.clear();
 
@@ -262,7 +272,7 @@ void mcache_manager::read(reader &th)
             entry = new mcache_demon(th);
             break;
         default:
-            ASSERT(!"Invalid streamed mcache type.");
+            die("Invalid streamed mcache type.");
         case MCACHE_NULL:
             entry = NULL;
             break;
@@ -274,7 +284,7 @@ void mcache_manager::read(reader &th)
 
 void mcache_manager::construct(writer &th)
 {
-    marshallLong(th, m_entries.size());
+    marshallInt(th, m_entries.size());
     for (unsigned int i = 0; i < m_entries.size(); i++)
     {
         if (m_entries[i] == NULL)
@@ -306,22 +316,22 @@ void mcache_manager::construct(writer &th)
 
 mcache_entry::mcache_entry(reader &th)
 {
-    m_ref_count = unmarshallLong(th);
+    m_ref_count = unmarshallInt(th);
 }
 
 void mcache_entry::construct(writer &th)
 {
-    marshallLong(th, m_ref_count);
+    marshallInt(th, m_ref_count);
 }
 
 /////////////////////////////////////////////////////////////////////////////
 // mcache_monster
 
-mcache_monster::mcache_monster(const monsters *mon)
+mcache_monster::mcache_monster(const monster* mon)
 {
-    assert(mcache_monster::valid(mon));
+    ASSERT(mcache_monster::valid(mon));
 
-    m_mon_tile = tileidx_monster(mon, false) & TILE_FLAG_MASK;
+    m_mon_tile = tileidx_monster(mon) & TILE_FLAG_MASK;
 
     const int mon_wep = mon->inv[MSLOT_WEAPON];
     m_equ_tile = tilep_equ_weapon(mitm[mon_wep]);
@@ -330,19 +340,20 @@ mcache_monster::mcache_monster(const monsters *mon)
 // Returns the amount of pixels necessary to shift a wielded weapon
 // from its default placement. Tiles showing monsters already wielding
 // a weapon should not be listed here.
-bool mcache_monster::get_weapon_offset(int mon_tile, int &ofs_x, int &ofs_y)
+bool mcache_monster::get_weapon_offset(tileidx_t mon_tile,
+                                       int *ofs_x, int *ofs_y)
 {
     switch (mon_tile)
     {
     // No shift necessary.
     case TILEP_MONS_VAULT_GUARD:
-    case TILEP_MONS_HOBGOBLIN:
     case TILEP_MONS_DEEP_ELF_MASTER_ARCHER:
     case TILEP_MONS_DEEP_ELF_BLADEMASTER:
     case TILEP_MONS_IMP:
+    case TILEP_MONS_IRON_IMP:
+    case TILEP_MONS_SHADOW_IMP:
     case TILEP_MONS_NORRIS:
     case TILEP_MONS_MAUD:
-    case TILEP_MONS_DUANE:
     case TILEP_MONS_EDMUND:
     case TILEP_MONS_FRANCES:
     case TILEP_MONS_HAROLD:
@@ -350,13 +361,15 @@ bool mcache_monster::get_weapon_offset(int mon_tile, int &ofs_x, int &ofs_y)
     case TILEP_MONS_JOZEF:
     case TILEP_MONS_RUPERT:
     case TILEP_MONS_TERENCE:
-    case TILEP_MONS_WAYNE:
+    case TILEP_MONS_WIGLAF:
     case TILEP_MONS_FREDERICK:
     case TILEP_MONS_RAKSHASA:
     case TILEP_MONS_RAKSHASA_FAKE:
     case TILEP_MONS_VAMPIRE_KNIGHT:
     case TILEP_MONS_SKELETAL_WARRIOR:
     case TILEP_MONS_ANGEL:
+    case TILEP_MONS_CHERUB:
+    case TILEP_MONS_MENNAS:
     case TILEP_MONS_MERFOLK:
     case TILEP_MONS_MERFOLK_WATER:
     case TILEP_MONS_MERFOLK_JAVELINEER:
@@ -371,44 +384,66 @@ bool mcache_monster::get_weapon_offset(int mon_tile, int &ofs_x, int &ofs_y)
     case TILEP_MONS_SIREN_WATER:
     case TILEP_MONS_ILSUIW:
     case TILEP_MONS_ILSUIW_WATER:
-        ofs_x = 0;
-        ofs_y = 0;
+    case TILEP_MONS_SPRIGGAN:
+    case TILEP_MONS_KENKU:
+    case TILEP_MONS_DEEP_DWARF_ARTIFICER:
+    case TILEP_MONS_DEEP_DWARF_DEATH_KNIGHT:
+    case TILEP_MONS_KOBOLD:
+        *ofs_x = 0;
+        *ofs_y = 0;
         break;
     // Shift to the left.
     case TILEP_MONS_GNOLL:
     case TILEP_MONS_GRUM:
     case TILEP_MONS_CRAZY_YIUF:
     case TILEP_MONS_DEEP_ELF_DEATH_MAGE:
-        ofs_x = -1;
-        ofs_y = 0;
+    case TILEP_MONS_SPRIGGAN_DEFENDER:
+    case TILEP_MONS_SPRIGGAN_BERSERKER:
+    case TILEP_MONS_BIG_KOBOLD:
+        *ofs_x = -1;
+        *ofs_y = 0;
         break;
+    case TILEP_MONS_HOBGOBLIN:
     case TILEP_MONS_TIAMAT:
-        ofs_x = -2;
-        ofs_y = 0;
+    case TILEP_MONS_TIAMAT+1:
+    case TILEP_MONS_TIAMAT+2:
+    case TILEP_MONS_TIAMAT+3:
+    case TILEP_MONS_TIAMAT+4:
+    case TILEP_MONS_TIAMAT+5:
+    case TILEP_MONS_TIAMAT+6:
+    case TILEP_MONS_TIAMAT+7:
+    case TILEP_MONS_TIAMAT+8:
+        *ofs_x = -2;
+        *ofs_y = 0;
         break;
     // Shift to the right.
+    case TILEP_MONS_DEMONSPAWN:
+    case TILEP_MONS_DONALD:
+        *ofs_x = 1;
+        *ofs_y = 0;
+        break;
     case TILEP_MONS_YAKTAUR_MELEE:
-        ofs_x = 2;
-        ofs_y = 0;
+        *ofs_x = 2;
+        *ofs_y = 0;
         break;
     case TILEP_MONS_YAKTAUR_CAPTAIN_MELEE:
-        ofs_x = 4;
-        ofs_y = 0;
+        *ofs_x = 4;
+        *ofs_y = 0;
+        break;
+    case TILEP_MONS_FIRE_GIANT:
+        *ofs_x = 5;
+        *ofs_y = 0;
         break;
     // Shift upwards.
     case TILEP_MONS_CENTAUR_WARRIOR_MELEE:
     case TILEP_MONS_DEEP_ELF_SORCERER:
     case TILEP_MONS_DEEP_ELF_HIGH_PRIEST:
-        ofs_x = 0;
-        ofs_y = -1;
+        *ofs_x = 0;
+        *ofs_y = -1;
         break;
     case TILEP_MONS_MIDGE:
-        ofs_x = 0;
-        ofs_y = -2;
-        break;
-    case TILEP_MONS_KOBOLD_DEMONOLOGIST:
-        ofs_x = 0;
-        ofs_y = -10;
+        *ofs_x = 0;
+        *ofs_y = -2;
         break;
     // Shift downwards.
     case TILEP_MONS_DEEP_ELF_KNIGHT:
@@ -417,8 +452,12 @@ bool mcache_monster::get_weapon_offset(int mon_tile, int &ofs_x, int &ofs_y)
     case TILEP_MONS_NAGA_WARRIOR:
     case TILEP_MONS_GUARDIAN_SERPENT:
     case TILEP_MONS_NAGA_MAGE:
-        ofs_x = 0;
-        ofs_y = 1;
+    case TILEP_MONS_THE_ENCHANTRESS:
+    case TILEP_MONS_DEEP_DWARF:
+    case TILEP_MONS_DEEP_DWARF_SCION:
+    case TILEP_MONS_DEEP_DWARF_BERSERKER:
+        *ofs_x = 0;
+        *ofs_y = 1;
         break;
     case TILEP_MONS_ORC:
     case TILEP_MONS_URUG:
@@ -430,13 +469,10 @@ bool mcache_monster::get_weapon_offset(int mon_tile, int &ofs_x, int &ofs_y)
     case TILEP_MONS_BOGGART:
     case TILEP_MONS_DEEP_ELF_FIGHTER:
     case TILEP_MONS_DEEP_ELF_SOLDIER:
-        ofs_x = 0;
-        ofs_y = 2;
-        break;
-    case TILEP_MONS_GOBLIN:
-    case TILEP_MONS_IJYB:
-        ofs_x = 0;
-        ofs_y = 4;
+    case TILEP_MONS_DEEP_DWARF_NECROMANCER:
+    case TILEP_MONS_UNBORN_DEEP_DWARF:
+        *ofs_x = 0;
+        *ofs_y = 2;
         break;
     // Shift upwards and to the left.
     case TILEP_MONS_DEEP_ELF_MAGE:
@@ -445,64 +481,77 @@ bool mcache_monster::get_weapon_offset(int mon_tile, int &ofs_x, int &ofs_y)
     case TILEP_MONS_DEEP_ELF_PRIEST:
     case TILEP_MONS_DEEP_ELF_DEMONOLOGIST:
     case TILEP_MONS_DEEP_ELF_ANNIHILATOR:
-        ofs_x = -1;
-        ofs_y = -2;
+        *ofs_x = -1;
+        *ofs_y = -2;
         break;
     case TILEP_MONS_CENTAUR_MELEE:
-        ofs_x = -1;
-        ofs_y = -3;
-        break;
-    case TILEP_MONS_MAURICE:
-        ofs_x = -2;
-        ofs_y = -2;
+        *ofs_x = -1;
+        *ofs_y = -3;
         break;
     case TILEP_MONS_SONJA:
-        ofs_x = -2;
-        ofs_y = -7;
+        *ofs_x = -2;
+        *ofs_y = -2;
         break;
-    case TILEP_MONS_OGRE_MAGE:
-        ofs_x = -4;
-        ofs_y = -2;
+    case TILEP_MONS_SPRIGGAN_AIR_MAGE:
+        *ofs_x = -3;
+        *ofs_y = -5;
         break;
     // Shift upwards and to the right.
     case TILEP_MONS_AGNES:
-        ofs_x = 1;
-        ofs_y = -3;
+        *ofs_x = 1;
+        *ofs_y = -3;
         break;
     case TILEP_MONS_WIZARD:
-        ofs_x = 2;
-        ofs_y = -2;
+        *ofs_x = 2;
+        *ofs_y = -2;
         break;
     case TILEP_MONS_RED_DEVIL:
-        ofs_x = 2;
-        ofs_y = -3;
+        *ofs_x = 2;
+        *ofs_y = -3;
         break;
     // Shift downwards and to the left.
     case TILEP_MONS_HELL_KNIGHT:
-        ofs_x = -1;
-        ofs_y = 3;
+        *ofs_x = -1;
+        *ofs_y = 3;
+        break;
+    case TILEP_MONS_GOBLIN:
+    case TILEP_MONS_IJYB:
+        *ofs_x = -2;
+        *ofs_y = 4;
         break;
     // Shift downwards and to the right.
-    case TILEP_MONS_BIG_KOBOLD:
-        ofs_x = 2;
-        ofs_y = 3;
+    case TILEP_MONS_ETTIN:
+        *ofs_x = 2;
+        *ofs_y = 1;
         break;
-    case TILEP_MONS_KOBOLD:
-        ofs_x = 3;
-        ofs_y = 4;
+    case TILEP_MONS_FROST_GIANT:
+        *ofs_x = 2;
+        *ofs_y = 3;
+        break;
+    case TILEP_MONS_TRAINING_DUMMY:
+        *ofs_x = 3;
+        *ofs_y = 2;
         break;
     case TILEP_MONS_ELF:
     case TILEP_MONS_ZOMBIE_LARGE:
-        ofs_x = 4;
-        ofs_y = 1;
+        *ofs_x = 4;
+        *ofs_y = 1;
+        break;
+    case TILEP_MONS_HALFLING:
+        *ofs_x = 4;
+        *ofs_y = 2;
         break;
     case TILEP_MONS_ZOMBIE_SMALL:
-        ofs_x = 4;
-        ofs_y = 3;
+        *ofs_x = 4;
+        *ofs_y = 3;
         break;
     case TILEP_MONS_HUMAN:
-        ofs_x = 5;
-        ofs_y = 2;
+        *ofs_x = 5;
+        *ofs_y = 2;
+        break;
+    case TILEP_MONS_HILL_GIANT:
+        *ofs_x = 6;
+        *ofs_y = 2;
         break;
     default:
         // This monster cannot be displayed with a weapon.
@@ -512,10 +561,10 @@ bool mcache_monster::get_weapon_offset(int mon_tile, int &ofs_x, int &ofs_y)
     return (true);
 }
 
-unsigned int mcache_monster::info(tile_draw_info *dinfo) const
+int mcache_monster::info(tile_draw_info *dinfo) const
 {
     int ofs_x, ofs_y;
-    get_weapon_offset(m_mon_tile, ofs_x, ofs_y);
+    get_weapon_offset(m_mon_tile, &ofs_x, &ofs_y);
 
     int count = 0;
     dinfo[count++].set(m_mon_tile);
@@ -525,7 +574,7 @@ unsigned int mcache_monster::info(tile_draw_info *dinfo) const
     // In some cases, overlay a second weapon tile...
     if (m_mon_tile == TILEP_MONS_DEEP_ELF_BLADEMASTER)
     {
-        int eq2;
+        tileidx_t eq2;
         switch (m_equ_tile)
         {
             case TILEP_HAND1_DAGGER:
@@ -547,7 +596,7 @@ unsigned int mcache_monster::info(tile_draw_info *dinfo) const
     return (count);
 }
 
-bool mcache_monster::valid(const monsters *mon)
+bool mcache_monster::valid(const monster* mon)
 {
     if (!mon)
         return (false);
@@ -555,85 +604,42 @@ bool mcache_monster::valid(const monsters *mon)
     if (mon_wep == NON_ITEM)
         return (false);
 
-    int mon_tile = tileidx_monster(mon, false) & TILE_FLAG_MASK;
+    tileidx_t mon_tile = tileidx_monster(mon) & TILE_FLAG_MASK;
 
     int ox, oy;
-    return get_weapon_offset(mon_tile, ox, oy);
+    return get_weapon_offset(mon_tile, &ox, &oy);
 }
 
 mcache_monster::mcache_monster(reader &th) : mcache_entry(th)
 {
-    m_mon_tile = unmarshallLong(th);
-    m_equ_tile = unmarshallLong(th);
+    m_mon_tile = unmarshallInt(th);
+    m_equ_tile = unmarshallInt(th);
 }
 
 void mcache_monster::construct(writer &th)
 {
     mcache_entry::construct(th);
 
-    marshallLong(th, m_mon_tile);
-    marshallLong(th, m_equ_tile);
+    marshallInt(th, m_mon_tile);
+    marshallInt(th, m_equ_tile);
 }
 
 /////////////////////////////////////////////////////////////////////////////
 // mcache_draco
 
-mcache_draco::mcache_draco(const monsters *mon)
+mcache_draco::mcache_draco(const monster* mon)
 {
-    assert(mcache_draco::valid(mon));
+    ASSERT(mcache_draco::valid(mon));
 
-    int draco = draco_subspecies(mon);
-    int colour;
-    switch (draco)
-    {
-    default:
-    case MONS_DRACONIAN:        colour = 0; break;
-    case MONS_BLACK_DRACONIAN:  colour = 1; break;
-    case MONS_YELLOW_DRACONIAN: colour = 2; break;
-    case MONS_GREEN_DRACONIAN:  colour = 3; break;
-    case MONS_MOTTLED_DRACONIAN:colour = 4; break;
-    case MONS_PALE_DRACONIAN:   colour = 5; break;
-    case MONS_PURPLE_DRACONIAN: colour = 6; break;
-    case MONS_RED_DRACONIAN:    colour = 7; break;
-    case MONS_WHITE_DRACONIAN:  colour = 8; break;
-    }
-
-    m_mon_tile = TILEP_DRACO_BASE + colour;
+    m_mon_tile = tileidx_draco_base(mon);
     int mon_wep = mon->inv[MSLOT_WEAPON];
     m_equ_tile = (mon_wep != NON_ITEM) ? tilep_equ_weapon(mitm[mon_wep]) : 0;
-
-    switch (mon->type)
-    {
-        case MONS_DRACONIAN_CALLER:
-            m_job_tile = TILEP_DRACO_CALLER;
-            break;
-        case MONS_DRACONIAN_MONK:
-            m_job_tile = TILEP_DRACO_MONK;
-            break;
-        case MONS_DRACONIAN_ZEALOT:
-            m_job_tile = TILEP_DRACO_ZEALOT;
-            break;
-        case MONS_DRACONIAN_SHIFTER:
-            m_job_tile = TILEP_DRACO_SHIFTER;
-            break;
-        case MONS_DRACONIAN_ANNIHILATOR:
-            m_job_tile = TILEP_DRACO_ANNIHILATOR;
-            break;
-        case MONS_DRACONIAN_KNIGHT:
-            m_job_tile = TILEP_DRACO_KNIGHT;
-            break;
-        case MONS_DRACONIAN_SCORCHER:
-            m_job_tile = TILEP_DRACO_SCORCHER;
-            break;
-        default:
-            m_job_tile = 0;
-            break;
-    }
+    m_job_tile = tileidx_draco_job(mon);
 }
 
-unsigned int mcache_draco::info(tile_draw_info *dinfo) const
+int mcache_draco::info(tile_draw_info *dinfo) const
 {
-    unsigned int i = 0;
+    int i = 0;
 
     dinfo[i++].set(m_mon_tile);
     if (m_job_tile)
@@ -644,43 +650,40 @@ unsigned int mcache_draco::info(tile_draw_info *dinfo) const
     return i;
 }
 
-bool mcache_draco::valid(const monsters *mon)
+bool mcache_draco::valid(const monster* mon)
 {
-    return (mon && mon->type >= MONS_FIRST_DRACONIAN
-            && mon->type <= MONS_LAST_DRACONIAN);
+    return (mon && mons_is_draconian(mon->type));
 }
 
 mcache_draco::mcache_draco(reader &th) : mcache_entry(th)
 {
-    m_mon_tile = unmarshallLong(th);
-    m_job_tile = unmarshallLong(th);
-    m_equ_tile = unmarshallLong(th);
+    m_mon_tile = unmarshallInt(th);
+    m_job_tile = unmarshallInt(th);
+    m_equ_tile = unmarshallInt(th);
 }
 
 void mcache_draco::construct(writer &th)
 {
     mcache_entry::construct(th);
 
-    marshallLong(th, m_mon_tile);
-    marshallLong(th, m_job_tile);
-    marshallLong(th, m_equ_tile);
+    marshallInt(th, m_mon_tile);
+    marshallInt(th, m_job_tile);
+    marshallInt(th, m_equ_tile);
 }
 
 /////////////////////////////////////////////////////////////////////////////
 // mcache_ghost
 
-mcache_ghost::mcache_ghost(const monsters *mon)
+mcache_ghost::mcache_ghost(const monster* mon)
 {
-    assert(mcache_ghost::valid(mon));
+    ASSERT(mcache_ghost::valid(mon));
 
-    const struct ghost_demon &ghost = *mon->ghost;
+    const class ghost_demon &ghost = *mon->ghost;
 
     unsigned int pseudo_rand = ghost.max_hp * 54321 * 54321;
-    int gender = (pseudo_rand >> 8) & 1;
 
-    tilep_race_default(ghost.species, gender,
-                       ghost.xl, m_doll.parts);
-    tilep_job_default(ghost.job, gender, m_doll.parts);
+    tilep_race_default(ghost.species, ghost.xl, &m_doll);
+    tilep_job_default(ghost.job, &m_doll);
 
     for (int p = TILEP_PART_CLOAK; p < TILEP_PART_MAX; p++)
     {
@@ -703,7 +706,7 @@ mcache_ghost::mcache_ghost(const monsters *mon)
         m_doll.parts[TILEP_PART_BODY]= TILEP_BODY_SCALEMAIL;
     else if (ac > 10)
         m_doll.parts[TILEP_PART_BODY]= TILEP_BODY_CHAINMAIL;
-    else if (ac > 5 )
+    else if (ac > 5)
         m_doll.parts[TILEP_PART_BODY]= TILEP_BODY_LEATHER_HEAVY;
     else
         m_doll.parts[TILEP_PART_BODY]= TILEP_BODY_ROBE_BLUE;
@@ -806,7 +809,7 @@ const dolls_data *mcache_ghost::doll() const
     return &m_doll;
 }
 
-bool mcache_ghost::valid(const monsters *mon)
+bool mcache_ghost::valid(const monster* mon)
 {
     return (mon && mons_is_pghost(mon->type));
 }
@@ -831,11 +834,11 @@ bool mcache_ghost::transparent() const
 /////////////////////////////////////////////////////////////////////////////
 // mcache_demon
 
-mcache_demon::mcache_demon(const monsters *mon)
+mcache_demon::mcache_demon(const monster* mon)
 {
-    assert(mcache_demon::valid(mon));
+    ASSERT(mcache_demon::valid(mon));
 
-    const struct ghost_demon &ghost = *mon->ghost;
+    const class ghost_demon &ghost = *mon->ghost;
 
     unsigned int pseudo_rand1 = ghost.max_hp * 54321 * 54321;
     unsigned int pseudo_rand2 = ghost.ac * 54321 * 54321;
@@ -856,7 +859,7 @@ mcache_demon::mcache_demon(const monsters *mon)
         m_demon.wings = 0;
 }
 
-unsigned int mcache_demon::info(tile_draw_info *dinfo) const
+int mcache_demon::info(tile_draw_info *dinfo) const
 {
     if (m_demon.wings)
     {
@@ -873,7 +876,7 @@ unsigned int mcache_demon::info(tile_draw_info *dinfo) const
     }
 }
 
-bool mcache_demon::valid(const monsters *mon)
+bool mcache_demon::valid(const monster* mon)
 {
     return (mon && mon->type == MONS_PANDEMONIUM_DEMON);
 }
