@@ -1,8 +1,7 @@
-/*
- *  File:       traps.cc
- *  Summary:    Traps related functions.
- *  Written by: Linley Henzell
- */
+/**
+ * @file
+ * @brief Traps related functions.
+**/
 
 #include "AppHdr.h"
 
@@ -14,11 +13,14 @@
 #include "artefact.h"
 #include "beam.h"
 #include "branch.h"
+#include "cloud.h"
 #include "clua.h"
 #include "coord.h"
+#include "coordit.h"
 #include "delay.h"
 #include "describe.h"
 #include "directn.h"
+#include "exercise.h"
 #include "map_knowledge.h"
 #include "itemname.h"
 #include "itemprop.h"
@@ -33,7 +35,7 @@
 #include "ouch.h"
 #include "player.h"
 #include "skills.h"
-#include "spl-mis.h"
+#include "spl-miscast.h"
 #include "spl-util.h"
 #include "state.h"
 #include "stuff.h"
@@ -42,7 +44,7 @@
 #include "areas.h"
 #include "terrain.h"
 #include "transform.h"
-#include "tutorial.h"
+#include "hints.h"
 #include "view.h"
 #include "shout.h"
 #include "xom.h"
@@ -85,7 +87,7 @@ void trap_def::disarm()
 void trap_def::destroy()
 {
     if (!in_bounds(this->pos))
-        ASSERT("trap position out of bounds!");
+        die("Trap position out of bounds!");
 
     grd(this->pos) = DNGN_FLOOR;
     this->ammo_qty = 0;
@@ -114,11 +116,21 @@ void trap_def::prepare_ammo()
         break;
     case TRAP_ALARM:
         this->ammo_qty = 1 + random2(3);
+        // Zotdef: alarm traps have practically unlimited ammo
+        if (crawl_state.game_is_zotdef())
+            this->ammo_qty = 100000;
+        break;
+    case TRAP_GOLUBRIA:
+        // really, turns until it vanishes
+        this->ammo_qty = 30 + random2(20);
         break;
     default:
         this->ammo_qty = 0;
         break;
     }
+    // Zot def: traps have 10x as much ammo
+    if (crawl_state.game_is_zotdef())
+        this->ammo_qty *= 10;
 }
 
 void trap_def::reveal()
@@ -156,8 +168,8 @@ bool trap_def::is_known(const actor* act) const
         return (player_knows);
     else if (act->atype() == ACT_MONSTER)
     {
-        const monsters* monster = dynamic_cast<const monsters*>(act);
-        const int intel = mons_intel(monster);
+        const monster* mons = act->as_monster();
+        const int intel = mons_intel(mons);
 
         // Smarter trap handling for intelligent monsters
         // * monsters native to a branch can be assumed to know the trap
@@ -165,7 +177,7 @@ bool trap_def::is_known(const actor* act) const
         // * friendlies and good neutrals can be assumed to have been warned
         //   by the player about all traps s/he knows about
         // * very intelligent monsters can be assumed to have a high T&D
-        //   skill (or have memorised part of the dungeon layout ;) )
+        //   skill (or have memorised part of the dungeon layout ;))
 
         if (this->category() == DNGN_TRAP_NATURAL)
         {
@@ -174,20 +186,18 @@ bool trap_def::is_known(const actor* act) const
             // * Allied zombies won't fall through shafts. (No herding!)
             // * Highly intelligent monsters never fall through shafts.
             return (intel >= I_HIGH
-                    || intel > I_PLANT && mons_is_native_in_branch(monster)
-                    || player_knows && monster->wont_attack());
+                    || intel > I_PLANT && mons_is_native_in_branch(mons)
+                    || player_knows && mons->wont_attack());
         }
         else
         {
             return (intel >= I_NORMAL
-                    && (mons_is_native_in_branch(monster)
-                        || player_knows && monster->wont_attack()
+                    && (mons_is_native_in_branch(mons)
+                        || player_knows && mons->wont_attack()
                         || intel >= I_HIGH && one_chance_in(3)));
         }
     }
-    ASSERT(false);
-
-    return (false);
+    die("invalid actor type");
 }
 
 
@@ -210,7 +220,7 @@ int get_trapping_net(const coord_def& where, bool trapped)
 
 // If there are more than one net on this square
 // split off one of them for checking/setting values.
-static void maybe_split_nets(item_def &item, const coord_def& where)
+static void _maybe_split_nets(item_def &item, const coord_def& where)
 {
     if (item.quantity == 1)
     {
@@ -232,21 +242,21 @@ static void maybe_split_nets(item_def &item, const coord_def& where)
     item.quantity = 1;
     set_item_stationary(item);
 
-    copy_item_to_grid( it, where );
+    copy_item_to_grid(it, where);
 }
 
-void mark_net_trapping(const coord_def& where)
+static void _mark_net_trapping(const coord_def& where)
 {
     int net = get_trapping_net(where);
     if (net == NON_ITEM)
     {
         net = get_trapping_net(where, false);
         if (net != NON_ITEM)
-            maybe_split_nets(mitm[net], where);
+            _maybe_split_nets(mitm[net], where);
     }
 }
 
-void monster_caught_in_net(monsters *mon, bolt &pbolt)
+void monster_caught_in_net(monster* mon, bolt &pbolt)
 {
     if (mon->body_size(PSIZE_BODY) >= SIZE_GIANT)
         return;
@@ -315,12 +325,13 @@ bool player_caught_in_net()
         }
 
         stop_delay(true); // even stair delays
+        redraw_screen(); // Account for changes in display.
         return (true);
     }
     return (false);
 }
 
-void check_net_will_hold_monster(monsters *mons)
+void check_net_will_hold_monster(monster* mons)
 {
     if (mons->body_size(PSIZE_BODY) >= SIZE_GIANT)
     {
@@ -362,6 +373,61 @@ void check_net_will_hold_monster(monsters *mons)
         mons->add_ench(ENCH_HELD);
 }
 
+std::vector<coord_def> find_golubria_on_level()
+{
+    std::vector<coord_def> ret;
+    for (rectangle_iterator ri(coord_def(0, 0), coord_def(GXM-1, GYM-1)); ri; ++ri)
+    {
+        trap_def *trap = find_trap(*ri);
+        if (trap && trap->type == TRAP_GOLUBRIA)
+            ret.push_back(*ri);
+    }
+    ASSERT(ret.size() <= 2);
+    return ret;
+}
+
+static bool _find_other_passage_side(coord_def& to)
+{
+    std::vector<coord_def> passages = find_golubria_on_level();
+    if (passages.size() < 2)
+        return false;
+
+    if (to == passages[0])
+    {
+        to = passages[1];
+        return true;
+    }
+    else if (to == passages[1])
+    {
+        to = passages[0];
+        return true;
+    }
+    else
+        die("Golubria's passage not found");
+}
+
+// Returns a direction string from you.pos to the
+// specified position. If fuzz is true, may be wrong.
+// Returns an empty string if no direction could be
+// determined (if fuzz if false, this is only if
+// you.pos==pos).
+std::string direction_string(coord_def pos, bool fuzz)
+{
+    int dx = you.pos().x - pos.x;
+    if (fuzz)
+        dx += random2avg(41,2) - 20;
+    int dy = you.pos().y - pos.y;
+    if (fuzz)
+        dy += random2avg(41,2) - 20;
+    const char *ew=((dx > 0) ? "west" : ((dx < 0) ? "east" : ""));
+    const char *ns=((dy < 0) ? "south" : ((dy > 0) ? "north" : ""));
+    if (abs(dy) > 2 * abs(dx))
+        ew="";
+    if (abs(dx) > 2 * abs(dy))
+        ns="";
+    return (std::string(ns) + ew);
+}
+
 void trap_def::trigger(actor& triggerer, bool flat_footed)
 {
     const bool you_know = this->is_known();
@@ -370,13 +436,18 @@ void trap_def::trigger(actor& triggerer, bool flat_footed)
     const bool you_trigger = (triggerer.atype() == ACT_PLAYER);
     const bool in_sight = you.see_cell(this->pos);
 
+    // Zot def - player never sets off known traps
+    if (crawl_state.game_is_zotdef() && you_trigger && you_know)
+    {
+        mpr("You step safely past the trap.");
+        return;
+    }
+
     // If set, the trap will be removed at the end of the
     // triggering process.
-    bool trap_destroyed = false;
+    bool trap_destroyed = false, know_trap_destroyed = false;;
 
-    monsters* m = NULL;
-    if (triggerer.atype() == ACT_MONSTER)
-        m = dynamic_cast<monsters*>(&triggerer);
+    monster* m = triggerer.as_monster();
 
     // Smarter monsters and those native to the level will simply
     // side-step known shafts. Unless they are already looking for
@@ -387,17 +458,22 @@ void trap_def::trigger(actor& triggerer, bool flat_footed)
             || trig_knows && !mons_is_fleeing(m) && !m->pacified())
         {
             // No message for flying monsters to avoid message spam.
-            if (you_know && !triggerer.airborne())
+            if (you_know && triggerer.ground_level())
                 simple_monster_message(m, " carefully avoids the shaft.");
             return;
         }
     }
 
-
-    // Only magical traps affect flying critters.
-    if (triggerer.airborne() && this->category() != DNGN_TRAP_MAGICAL)
+    // Zot def - friendly monsters never set off known traps
+    if (crawl_state.game_is_zotdef() && m && m->friendly() && trig_knows)
     {
-        if (you_know && m)
+        simple_monster_message(m," carefully avoids a trap.");
+        return;
+    }
+    // Only magical traps affect flying critters.
+    if (!triggerer.ground_level() && this->category() != DNGN_TRAP_MAGICAL)
+    {
+        if (you_know && m && triggerer.airborne())
             simple_monster_message(m, " flies safely over a trap.");
         return;
     }
@@ -418,9 +494,35 @@ void trap_def::trigger(actor& triggerer, bool flat_footed)
         this->shoot_ammo(triggerer, trig_knows);
     else switch (this->type)
     {
+    case TRAP_GOLUBRIA: {
+        coord_def to = p;
+        if (_find_other_passage_side(to))
+        {
+            if (you_trigger)
+                mpr("You enter the passage of Golubria.");
+            else
+                simple_monster_message(m, " enters the passage of Golubria.");
+
+            if (triggerer.move_to_pos(to))
+            {
+                if (you_trigger)
+                    place_cloud(CLOUD_TLOC_ENERGY, p, 1 + random2(3), &you);
+                else
+                    place_cloud(CLOUD_TLOC_ENERGY, p, 1 + random2(3), m);
+                trap_destroyed = true;
+                know_trap_destroyed = you_trigger;
+            }
+            else
+            {
+                mpr("But it is blocked!");
+            }
+        }
+        break;
+    }
     case TRAP_TELEPORT:
         // Never revealed by monsters.
-        if (!you_trigger && !you_know)
+        // except when it's in sight, it's pretty obvious what happened. -doy
+        if (!you_trigger && !you_know && !in_sight)
             this->hide();
         triggerer.teleport(true);
         break;
@@ -447,17 +549,27 @@ void trap_def::trigger(actor& triggerer, bool flat_footed)
         {
             // Alarm traps aren't set off by hostile monsters, because
             // that would be way too nasty for the player.
-            const char* message_here = "An alarm trap emits a blaring wail!";
-            const char* message_near = "You hear a blaring wail!";
-            const char* message_far  = "You hear a distant blaring wail!";
-            const char* msg = (you_trigger ? message_here :
-                                 (in_sight ? message_near : message_far));
+            std::string msg;
+            if (you_trigger)
+                msg = "An alarm trap emits a blaring wail!";
+            else
+            {
+                std::string dir=direction_string(this->pos, !in_sight);
+                msg = std::string("You hear a ") +
+                    ((in_sight) ? "" : "distant ")
+                    + "blaring wail "
+                    + (!dir.empty()? ("to the " + dir + ".") : "behind you!");
+            }
             // Monsters of normal or greater intelligence will realize that
             // they were the one to set off the trap.
             int source = !m ? you.mindex() :
                          mons_intel(m) >= I_NORMAL ? m->mindex() : -1;
 
-            noisy(12, this->pos, msg, source);
+            // Zotdef - Made alarm traps noisier and more noticeable
+            int noiselevel = crawl_state.game_is_zotdef() ? 30 : 12;
+            noisy(noiselevel, this->pos, msg.c_str(), source, false);
+            if (crawl_state.game_is_zotdef())
+                more();
         }
         break;
 
@@ -467,7 +579,7 @@ void trap_def::trigger(actor& triggerer, bool flat_footed)
             if (trig_knows && one_chance_in(3))
                 mpr("You avoid triggering a blade trap.");
             else if (random2limit(player_evasion(), 40)
-                     + (random2(you.dex) / 3) + (trig_knows ? 3 : 0) > 8)
+                     + (random2(you.dex()) / 3) + (trig_knows ? 3 : 0) > 8)
             {
                 mpr("A huge blade swings just past you!");
             }
@@ -476,7 +588,8 @@ void trap_def::trigger(actor& triggerer, bool flat_footed)
                 mpr("A huge blade swings out and slices into you!");
                 const int damage = (you.absdepth0 * 2) + random2avg(29, 2)
                     - random2(1 + you.armour_class());
-                ouch(damage, NON_MONSTER, KILLED_BY_TRAP, "blade");
+                std::string n = name(DESC_NOCAP_A) + " trap";
+                ouch(damage, NON_MONSTER, KILLED_BY_TRAP, n.c_str());
                 bleed_onto_floor(you.pos(), MONS_PLAYER, damage, true);
             }
         }
@@ -527,6 +640,14 @@ void trap_def::trigger(actor& triggerer, bool flat_footed)
                 m->hurt(NULL, damage_taken);
                 if (in_sight && m->alive())
                     print_wounds(m);
+
+                // zotdef: blade traps break eventually
+                if (crawl_state.game_is_zotdef() && one_chance_in(200))
+                {
+                    if (in_sight)
+                        mpr("The blade breaks!");
+                    disarm();
+                }
             }
         }
         break;
@@ -539,7 +660,7 @@ void trap_def::trigger(actor& triggerer, bool flat_footed)
             else
             {
                 if (random2limit(player_evasion(), 40)
-                    + (random2(you.dex) / 3) + (trig_knows ? 3 : 0) > 12)
+                    + (random2(you.dex()) / 3) + (trig_knows ? 3 : 0) > 12)
                 {
                     mpr("A net drops to the ground!");
                 }
@@ -554,7 +675,7 @@ void trap_def::trigger(actor& triggerer, bool flat_footed)
                 copy_item_to_grid(item, triggerer.pos());
 
                 if (you.attribute[ATTR_HELD])
-                    mark_net_trapping(you.pos());
+                    _mark_net_trapping(you.pos());
 
                 trap_destroyed = true;
             }
@@ -612,7 +733,7 @@ void trap_def::trigger(actor& triggerer, bool flat_footed)
                 copy_item_to_grid(item, triggerer.pos());
 
                 if (m->caught())
-                    mark_net_trapping(m->pos());
+                    _mark_net_trapping(m->pos());
 
                 trap_destroyed = true;
             }
@@ -627,8 +748,8 @@ void trap_def::trigger(actor& triggerer, bool flat_footed)
             if (!trig_knows)
                 xom_is_stimulated(32);
 
-            MiscastEffect( &you, ZOT_TRAP_MISCAST, SPTYP_RANDOM,
-                           3, "a Zot trap" );
+            MiscastEffect(&you, ZOT_TRAP_MISCAST, SPTYP_RANDOM,
+                           3, "a Zot trap");
         }
         else if (m)
         {
@@ -639,22 +760,30 @@ void trap_def::trigger(actor& triggerer, bool flat_footed)
             // in 99% of cases - a player can just watch who stepped where
             // and mark the trap on an external paper map.  Not good.
 
-            if (m->wont_attack() || crawl_state.arena)
-            {
-                MiscastEffect( m, ZOT_TRAP_MISCAST, SPTYP_RANDOM,
-                               3, "the power of Zot" );
-            }
+            actor* targ = NULL;
+            if (m->wont_attack() || crawl_state.game_is_arena())
+                targ = m;
             else if (in_sight && one_chance_in(5))
-            {
-                mpr("The power of Zot is invoked against you!");
-                MiscastEffect( &you, ZOT_TRAP_MISCAST, SPTYP_RANDOM,
-                               3, "the power of Zot" );
-            }
-            else if (player_can_hear(this->pos))
+                targ = &you;
+
+            // Give the player a chance to figure out what happened
+            // to their friend.
+            if (player_can_hear(this->pos) && (!targ || !in_sight))
             {
                 mprf(MSGCH_SOUND, "You hear a %s \"Zot\"!",
                      in_sight ? "loud" : "distant");
             }
+
+            if (targ)
+            {
+                if (in_sight)
+                {
+                    mprf("The power of Zot is invoked against %s!",
+                         targ->name(DESC_NOCAP_THE).c_str());
+                }
+                MiscastEffect(targ, ZOT_TRAP_MISCAST, SPTYP_RANDOM,
+                              3, "the power of Zot");
+            }
         }
         break;
 
@@ -682,6 +811,11 @@ void trap_def::trigger(actor& triggerer, bool flat_footed)
         if (trig_knows)
             break;
 
+        // Depending on total (body + equipment) weight, give monsters
+        // and player a chance to escape a shaft.
+        if (x_chance_in_y(200, triggerer.total_weight()))
+            break;
+
         // Fire away!
         triggerer.do_shaft();
 
@@ -695,22 +829,32 @@ void trap_def::trigger(actor& triggerer, bool flat_footed)
         }
         break;
 
+    case TRAP_PLATE:
+        dungeon_events.fire_position_event(DET_PRESSURE_PLATE, pos);
+        break;
+
     default:
         break;
     }
 
     if (you_trigger)
     {
-        learned_something_new(TUT_SEEN_TRAP, p);
+        learned_something_new(HINT_SEEN_TRAP, p);
 
         // Exercise T&D if the trap revealed itself, but not if it ran
         // out of ammo.
         if (!you_know && this->type != TRAP_UNASSIGNED && this->is_known())
-            exercise(SK_TRAPS_DOORS, ((coinflip()) ? 2 : 1));
+            practise(EX_TRAP_TRIGGER);
     }
 
     if (trap_destroyed)
+    {
+        if (know_trap_destroyed)
+        {
+            env.map_knowledge(this->pos).set_feature(DNGN_FLOOR);
+        }
         this->destroy();
+    }
 }
 
 int trap_def::max_damage(const actor& act)
@@ -759,11 +903,11 @@ int reveal_traps(const int range)
         if (!trap.active())
             continue;
 
-        if (grid_distance(you.pos(), trap.pos) < range && !trap.is_known())
+        if (distance(you.pos(), trap.pos) < dist_range(range) && !trap.is_known())
         {
             traps_found++;
             trap.reveal();
-            set_map_knowledge_obj(trap.pos, show_type(grd(trap.pos)));
+            env.map_knowledge(trap.pos).set_feature(grd(trap.pos));
             set_terrain_mapped(trap.pos);
         }
     }
@@ -771,7 +915,7 @@ int reveal_traps(const int range)
     return (traps_found);
 }
 
-void destroy_trap( const coord_def& pos )
+void destroy_trap(const coord_def& pos)
 {
     if (trap_def* ptrap = find_trap(pos))
         ptrap->destroy();
@@ -779,19 +923,42 @@ void destroy_trap( const coord_def& pos )
 
 trap_def* find_trap(const coord_def& pos)
 {
-    for (int i = 0; i < MAX_TRAPS; ++i)
-        if (env.trap[i].pos == pos && env.trap[i].type != TRAP_UNASSIGNED)
-            return (&env.trap[i]);
-    return (NULL);
+    if (!feat_is_trap(grd(pos), true))
+        return (NULL);
+
+    unsigned short t = env.tgrid(pos);
+    ASSERT(t != NON_ENTITY && t < MAX_TRAPS);
+    ASSERT(env.trap[t].pos == pos && env.trap[t].type != TRAP_UNASSIGNED);
+
+    return (&env.trap[t]);
 }
 
 trap_type get_trap_type(const coord_def& pos)
 {
     if (trap_def* ptrap = find_trap(pos))
         return (ptrap->type);
+
+    if (feature_mimic_at(pos))
+    {
+        monster *mimic = monster_at(pos);
+        if (mimic->props.exists("trap_type"))
+            return static_cast<trap_type>(mimic->props["trap_type"].get_short());
+    }
+
     return (TRAP_UNASSIGNED);
 }
 
+// Returns the unqualified name ("blade", "dart") of the trap at the
+// given position. Does not check if the trap has been discovered, and
+// will faithfully report the names of unknown traps.
+//
+// If there is no trap at the given position, returns an empty string.
+const char *trap_name_at(const coord_def& c)
+{
+    const trap_type trap = get_trap_type(c);
+    return trap != TRAP_UNASSIGNED? trap_name(trap) : "";
+}
+
 static bool _disarm_is_deadly(trap_def& trap)
 {
     int dam = trap.max_damage(you);
@@ -815,8 +982,13 @@ void disarm_trap(const coord_def& where)
     switch (trap.category())
     {
     case DNGN_TRAP_MAGICAL:
-        mpr("You can't disarm that trap.");
-        return;
+        // Zotdef - allow alarm traps to be disarmed
+        if (!crawl_state.game_is_zotdef() || trap.type != TRAP_ALARM)
+        {
+            mpr("You can't disarm that trap.");
+            return;
+        }
+        break;
     case DNGN_TRAP_NATURAL:
         // Only shafts for now.
         mpr("You can't disarm a shaft.");
@@ -832,7 +1004,7 @@ void disarm_trap(const coord_def& where)
                                "Really try disarming that %s?",
                                feature_description(trap.category(),
                                                    get_trap_type(where),
-                                                   false, DESC_BASENAME,
+                                                   "", DESC_BASENAME,
                                                    false).c_str());
 
         if (!yesno(prompt.c_str(), true, 'n'))
@@ -844,11 +1016,11 @@ void disarm_trap(const coord_def& where)
 
     // Make the actual attempt
     you.turn_is_over = true;
-    if (random2(you.skills[SK_TRAPS_DOORS] + 2) <= random2(you.absdepth0 + 5))
+    if (random2(you.skill(SK_TRAPS_DOORS) + 2) <= random2(you.absdepth0 + 5))
     {
         mpr("You failed to disarm the trap.");
-        if (random2(you.dex) > 5 + random2(5 + you.absdepth0))
-            exercise(SK_TRAPS_DOORS, 1 + random2(you.absdepth0 / 5));
+        if (random2(you.dex()) > 5 + random2(5 + you.absdepth0))
+            practise(EX_TRAP_DISARM_FAIL, you.absdepth0);
         else
         {
             if (trap.type == TRAP_NET && trap.pos != you.pos())
@@ -856,21 +1028,20 @@ void disarm_trap(const coord_def& where)
                 if (coinflip())
                 {
                     mpr("You stumble into the trap!");
-                    move_player_to_grid(trap.pos, true, false, true);
+                    move_player_to_grid(trap.pos, true, false);
                 }
             }
             else
                 trap.trigger(you, true);
 
-            if (coinflip())
-                exercise(SK_TRAPS_DOORS, 1);
+            practise(EX_TRAP_DISARM_TRIGGER);
         }
     }
     else
     {
         mpr("You have disarmed the trap.");
         trap.disarm();
-        exercise(SK_TRAPS_DOORS, 1 + random2(5) + (you.absdepth0/5));
+        practise(EX_TRAP_DISARM, you.absdepth0);
     }
 }
 
@@ -878,7 +1049,7 @@ void disarm_trap(const coord_def& where)
 // This doesn't actually have any effect (yet).
 // Do not expect gratitude for this!
 // ----------------------------------
-void remove_net_from(monsters *mon)
+void remove_net_from(monster* mon)
 {
     you.turn_is_over = true;
 
@@ -900,16 +1071,17 @@ void remove_net_from(monsters *mon)
         invis = 3 + random2(5);
 
     bool net_destroyed = false;
-    if ( random2(you.skills[SK_TRAPS_DOORS] + 2) + paralys
-           <= random2( 2*mon->body_size(PSIZE_BODY) + 3 ) + invis)
+    if (random2(you.skill(SK_TRAPS_DOORS) + 2) + paralys
+           <= random2(2*mon->body_size(PSIZE_BODY) + 3) + invis)
     {
-        if (one_chance_in(you.skills[SK_TRAPS_DOORS] + you.dex/2))
+        if (one_chance_in(you.skill(SK_TRAPS_DOORS) + you.dex()/2))
         {
             mitm[net].plus--;
             mpr("You tear at the net.");
             if (mitm[net].plus < -7)
             {
-                mpr("Whoops! The net comes apart in your hands!");
+                mprf("Whoops! The net comes apart in your %s!",
+                     your_hand(true).c_str());
                 mon->del_ench(ENCH_HELD, true);
                 destroy_item(net);
                 net_destroyed = true;
@@ -927,8 +1099,7 @@ void remove_net_from(monsters *mon)
                 mpr("You fail to remove the net.");
         }
 
-        if (random2(you.dex) > 5 + random2( 2*mon->body_size(PSIZE_BODY) ))
-            exercise(SK_TRAPS_DOORS, 1 + random2(mon->body_size(PSIZE_BODY)/2));
+        practise(EX_REMOVE_NET);
         return;
     }
 
@@ -967,7 +1138,7 @@ static int damage_or_escape_net(int hold)
         if (brand == SPWPN_FLAMING || brand == SPWPN_VORPAL)
             damage++;
     }
-    else if (you.attribute[ATTR_TRANSFORMATION] == TRAN_BLADE_HANDS)
+    else if (you.form == TRAN_BLADE_HANDS)
         damage += 2;
     else if (you.has_usable_claws())
     {
@@ -983,9 +1154,9 @@ static int damage_or_escape_net(int hold)
         damage += 2;
 
     // Check stats.
-    if (x_chance_in_y(you.strength, 18))
+    if (x_chance_in_y(you.strength(), 18))
         damage++;
-    if (x_chance_in_y(you.dex, 12))
+    if (x_chance_in_y(you.dex(), 12))
         escape++;
     if (x_chance_in_y(player_evasion(), 20))
         escape++;
@@ -1034,6 +1205,7 @@ void free_self_from_net()
     if (net == NON_ITEM) // really shouldn't happen!
     {
         you.attribute[ATTR_HELD] = 0;
+        you.redraw_quiver = true;
         return;
     }
 
@@ -1047,7 +1219,7 @@ void free_self_from_net()
         // For previously undamaged nets this takes at least 2 and at most
         // 8 turns.
         bool can_slice =
-            (you.attribute[ATTR_TRANSFORMATION] == TRAN_BLADE_HANDS)
+            (you.form == TRAN_BLADE_HANDS)
             || (you.weapon() && can_cut_meat(*you.weapon()));
 
         int damage = -do_what;
@@ -1078,6 +1250,7 @@ void free_self_from_net()
             destroy_item(net);
 
             you.attribute[ATTR_HELD] = 0;
+            you.redraw_quiver = true;
             return;
         }
 
@@ -1102,9 +1275,9 @@ void free_self_from_net()
    else
    {
         // You try to escape (takes at least 3 turns, and at most 10).
-        unsigned int escape = do_what;
+        int escape = do_what;
 
-        if (you.duration[DUR_HASTE]) // extra bonus, also Berserk
+        if (you.duration[DUR_HASTE] || you.duration[DUR_BERSERK]) // extra bonus
             escape++;
 
         // Medium sized characters are at a disadvantage and sometimes
@@ -1123,6 +1296,7 @@ void free_self_from_net()
                 mpr("You break free from the net!");
 
             you.attribute[ATTR_HELD] = 0;
+            you.redraw_quiver = true;
             remove_item_stationary(mitm[net]);
             return;
         }
@@ -1149,6 +1323,7 @@ void clear_trapping_net()
         remove_item_stationary(mitm[net]);
 
     you.attribute[ATTR_HELD] = 0;
+    you.redraw_quiver = true;
 }
 
 item_def trap_def::generate_trap_item()
@@ -1183,13 +1358,6 @@ item_def trap_def::generate_trap_item()
         set_item_ego_type(item, base, SPWPN_NORMAL);
     }
 
-    // give appropriate racial flag for Orcish Mines and Elven Halls
-    // should we ever allow properties of dungeon features, we could use that
-    if (you.where_are_you == BRANCH_ORCISH_MINES)
-        set_equip_race( item, ISFLAG_ORCISH );
-    else if (you.where_are_you == BRANCH_ELVEN_HALLS)
-        set_equip_race( item, ISFLAG_ELVEN );
-
     item_colour(item);
     return item;
 }
@@ -1213,9 +1381,17 @@ void trap_def::shoot_ammo(actor& act, bool was_known)
         const coord_def apos = act.pos();
 
         item_def shot = this->generate_trap_item();
+
+        bool force_poison = (env.markers.property_at(pos, MAT_ANY,
+                                "poisoned_needle_trap") == "true");
+
+        bool force_hit = (env.markers.property_at(pos, MAT_ANY,
+                                "force_hit") == "true");
+
         bool poison = (this->type == TRAP_NEEDLE
                        && !act.res_poison()
-                       && x_chance_in_y(50 - (3*act.armour_class()) / 2, 100));
+                       && (x_chance_in_y(50 - (3*act.armour_class()) / 2, 100)
+                            || force_poison));
 
         int damage_taken =
             std::max(this->shot_damage(act) - random2(act.armour_class()+1),0);
@@ -1224,10 +1400,10 @@ void trap_def::shoot_ammo(actor& act, bool was_known)
 
         if (act.atype() == ACT_PLAYER)
         {
-            if (one_chance_in(5) || (was_known && !one_chance_in(4)))
+            if (!force_hit && (one_chance_in(5) || was_known && !one_chance_in(4)))
             {
-                mprf( "You avoid triggering %s trap.",
-                      this->name(DESC_NOCAP_A).c_str() );
+                mprf("You avoid triggering %s trap.",
+                      this->name(DESC_NOCAP_A).c_str());
 
                 return;         // no ammo generated either
             }
@@ -1237,12 +1413,12 @@ void trap_def::shoot_ammo(actor& act, bool was_known)
 
             // Check for shield blocking.
             // Exercise only if the trap was unknown (to prevent scumming.)
-            if (!was_known && player_shield_class() && coinflip())
-                exercise(SK_SHIELDS, 1);
+            if (!was_known && player_shield_class())
+                practise(EX_SHIELD_TRAP);
 
             const int con_block = random2(20 + you.shield_block_penalty());
             const int pro_block = you.shield_bonus();
-            if (pro_block >= con_block)
+            if (pro_block >= con_block && !force_hit)
             {
                 // Note that we don't call shield_block_succeeded()
                 // because that can exercise Shields skill.
@@ -1257,24 +1433,26 @@ void trap_def::shoot_ammo(actor& act, bool was_known)
                 // Note that this uses full (not random2limit(foo,40))
                 // player_evasion.
                 int your_dodge = you.melee_evasion(NULL) - 2
-                    + (random2(you.dex) / 3)
+                    + (random2(you.dex()) / 3)
                     + (repel_turns * 10);
 
                 // Check if it got past dodging. Deflect Missiles provides
                 // immunity to such traps.
                 if (trap_hit >= your_dodge
-                    && you.duration[DUR_DEFLECT_MISSILES] == 0)
+                    && you.duration[DUR_DEFLECT_MISSILES] == 0
+                    || force_hit)
                 {
                     // OK, we've been hit.
                     msg += "hits you!";
                     mpr(msg.c_str());
 
+                    std::string n = name(DESC_NOCAP_A) + " trap";
+
                     // Needle traps can poison.
                     if (poison)
-                        you.poison(NULL, 1 + random2(3));
+                        poison_player(1 + random2(3), "", n);
 
-                    ouch(damage_taken, NON_MONSTER, KILLED_BY_TRAP,
-                         shot.name(DESC_PLAIN).c_str());
+                    ouch(damage_taken, NON_MONSTER, KILLED_BY_TRAP, n.c_str());
                 }
                 else            // trap dodged
                 {
@@ -1283,8 +1461,8 @@ void trap_def::shoot_ammo(actor& act, bool was_known)
                 }
 
                 // Exercise only if the trap was unknown (to prevent scumming.)
-                if (!was_known && coinflip())
-                    you.check_train_dodging();
+                if (!was_known)
+                    practise(EX_DODGE_TRAP);
             }
         }
         else if (act.atype() == ACT_MONSTER)
@@ -1335,6 +1513,7 @@ dungeon_feature_type trap_category(trap_type type)
     case TRAP_TELEPORT:
     case TRAP_ALARM:
     case TRAP_ZOT:
+    case TRAP_GOLUBRIA:
         return (DNGN_TRAP_MAGICAL);
 
     case TRAP_DART:
@@ -1345,18 +1524,41 @@ dungeon_feature_type trap_category(trap_type type)
     case TRAP_BOLT:
     case TRAP_NEEDLE:
     case TRAP_NET:
+    case TRAP_PLATE:
     default:                    // what *would* be the default? {dlb}
         return (DNGN_TRAP_MECHANICAL);
     }
 }
 
+trap_type random_trap()
+{
+    return (static_cast<trap_type>(random2(TRAP_MAX_REGULAR+1)));
+}
+
+trap_type random_trap(dungeon_feature_type feat)
+{
+    ASSERT(feat_is_trap(feat, false));
+    trap_type trap = NUM_TRAPS;
+    do
+        trap = random_trap();
+    while (trap_category(trap) != feat);
+    return trap;
+}
+
 bool is_valid_shaft_level(const level_id &place)
 {
+    if (crawl_state.test
+        || crawl_state.game_is_sprint()
+        || crawl_state.game_is_zotdef())
+    {
+        return (false);
+    }
+
     if (place.level_type != LEVEL_DUNGEON)
         return (false);
 
-    // Shafts are now allowed on the first two levels,
-    // as they have a good chance of being detected.
+    // Shafts are now allowed on the first two levels, as they have a
+    // good chance of being detected. You'll also fall less deep.
     /* if (place == BRANCH_MAIN_DUNGEON && you.absdepth0 < 2)
         return (false); */
 
@@ -1396,7 +1598,6 @@ bool shaft_known(int depth, bool randomly_placed)
 level_id generic_shaft_dest(level_pos lpos, bool known = false)
 {
     level_id  lid   = lpos.id;
-    coord_def pos   = lpos.pos;
 
     if (lid.level_type != LEVEL_DUNGEON)
         return lid;
@@ -1468,7 +1669,7 @@ void handle_items_on_shaft(const coord_def& pos, bool open_shaft)
 
     igrd(pos) = NON_ITEM;
 
-    if (is_terrain_seen(pos) && open_shaft)
+    if (env.map_knowledge(pos).seen() && open_shaft)
     {
         mpr("A shaft opens up in the floor!");
         grd(pos) = DNGN_TRAP_NATURAL;
@@ -1478,9 +1679,9 @@ void handle_items_on_shaft(const coord_def& pos, bool open_shaft)
     {
         int next = mitm[o].link;
 
-        if (mitm[o].is_valid())
+        if (mitm[o].defined())
         {
-            if (is_terrain_seen(pos))
+            if (env.map_knowledge(pos).seen())
             {
                 mprf("%s falls through the shaft.",
                      mitm[o].name(DESC_INVENTORY).c_str());
@@ -1496,7 +1697,7 @@ void handle_items_on_shaft(const coord_def& pos, bool open_shaft)
     }
 }
 
-static int num_traps_default(int level_number, const level_id &place)
+static int _num_traps_default(int level_number, const level_id &place)
 {
     return random2avg(9, 2);
 }
@@ -1512,15 +1713,14 @@ int num_traps_for_place(int level_number, const level_id &place)
         if (branches[place.branch].num_traps_function != NULL)
             return branches[place.branch].num_traps_function(level_number);
         else
-            return num_traps_default(level_number, place);
-    case LEVEL_ABYSS:
-        return traps_abyss_number(level_number);
+            return _num_traps_default(level_number, place);
     case LEVEL_PANDEMONIUM:
-        return traps_pan_number(level_number);
+        return _num_traps_default(level_number, place);
     case LEVEL_LABYRINTH:
     case LEVEL_PORTAL_VAULT:
-        ASSERT(false);
+        die("invalid place for traps");
         break;
+    case LEVEL_ABYSS:
     default:
         return 0;
     }
@@ -1539,7 +1739,7 @@ trap_type random_trap_slime(int level_number)
 
     if (one_chance_in(5) && is_valid_shaft_level(level_id::current()))
         type = TRAP_SHAFT;
-    if (one_chance_in(5))
+    if (one_chance_in(5) && !crawl_state.game_is_sprint())
         type = TRAP_TELEPORT;
     if (one_chance_in(10))
         type = TRAP_ALARM;
@@ -1547,7 +1747,7 @@ trap_type random_trap_slime(int level_number)
     return (type);
 }
 
-static trap_type random_trap_default(int level_number, const level_id &place)
+static trap_type _random_trap_default(int level_number, const level_id &place)
 {
     trap_type type = TRAP_DART;
 
@@ -1582,7 +1782,7 @@ static trap_type random_trap_default(int level_number, const level_id &place)
 
     if (one_chance_in(20) && is_valid_shaft_level(place))
         type = TRAP_SHAFT;
-    if (one_chance_in(20))
+    if (one_chance_in(20) && !crawl_state.game_is_sprint())
         type = TRAP_TELEPORT;
     if (one_chance_in(40))
         type = TRAP_ALARM;
@@ -1595,21 +1795,11 @@ trap_type random_trap_for_place(int level_number, const level_id &place)
     if (level_number == -1)
         level_number = place.absdepth();
 
-    switch (place.level_type)
-    {
-    case LEVEL_DUNGEON:
+    if (place.level_type == LEVEL_DUNGEON)
         if (branches[place.branch].rand_trap_function != NULL)
             return branches[place.branch].rand_trap_function(level_number);
-        else
-            return random_trap_default(level_number, place);
-    case LEVEL_ABYSS:
-        return traps_abyss_type(level_number);
-    case LEVEL_PANDEMONIUM:
-        return traps_pan_type(level_number);
-    default:
-        return random_trap_default(level_number, place);
-    }
-    return NUM_TRAPS;
+
+    return _random_trap_default(level_number, place);
 }
 
 int traps_zero_number(int level_number)
@@ -1617,32 +1807,11 @@ int traps_zero_number(int level_number)
     return 0;
 }
 
-int traps_pan_number(int level_number)
+int count_traps(trap_type ttyp)
 {
-    return num_traps_default(level_number, level_id(LEVEL_PANDEMONIUM));
-}
-
-trap_type traps_pan_type(int level_number)
-{
-    return random_trap_default(level_number, level_id(LEVEL_PANDEMONIUM));
-}
-
-int traps_abyss_number(int level_number)
-{
-    return num_traps_default(level_number, level_id(LEVEL_ABYSS));
-}
-
-trap_type traps_abyss_type(int level_number)
-{
-    return random_trap_default(level_number, level_id(LEVEL_ABYSS));
-}
-
-int traps_lab_number(int level_number)
-{
-    return num_traps_default(level_number, level_id(LEVEL_LABYRINTH));
-}
-
-trap_type traps_lab_type(int level_number)
-{
-    return random_trap_default(level_number, level_id(LEVEL_LABYRINTH));
+    int num = 0;
+    for (int tcount = 0; tcount < MAX_TRAPS; tcount++)
+        if (env.trap[tcount].type == ttyp)
+            num++;
+    return num;
 }
