@@ -1,14 +1,13 @@
-/*
- *  File:       travel.h
- *  Summary:    Travel stuff
- *  Written by: Darshan Shaligram
- */
+/**
+ * @file
+ * @brief Travel stuff
+**/
 #ifndef TRAVEL_H
 #define TRAVEL_H
 
 #include "externs.h"
-
 #include "exclude.h"
+#include "map_knowledge.h"
 
 // For travel_distance_col and travel_distance_grid_t
 #include "travel_defs.h"
@@ -25,7 +24,7 @@ enum run_check_type
 {
     RCHECK_LEFT,
     RCHECK_FRONT,
-    RCHECK_RIGHT
+    RCHECK_RIGHT,
 };
 
 enum run_dir_type
@@ -38,7 +37,7 @@ enum run_dir_type
     RDIR_DOWN_LEFT,
     RDIR_LEFT,
     RDIR_UP_LEFT,
-    RDIR_REST
+    RDIR_REST,
 };
 
 enum run_mode_type
@@ -50,7 +49,7 @@ enum run_mode_type
     RMODE_NOT_RUNNING    = 0,  // must remain equal to 0
     RMODE_CONTINUE,
     RMODE_START,
-    RMODE_REST_DURATION = 100
+    RMODE_REST_DURATION = 100,
 };
 
 /* ***********************************************************************
@@ -59,22 +58,23 @@ enum run_mode_type
 void initialise_travel();
 void init_travel_terrain_check(bool check_race_equip = true);
 void stop_running(void);
+void travel_init_load_level();
 void travel_init_new_level();
 
-unsigned char is_waypoint(const coord_def &p);
-command_type direction_to_command(char x, char y);
+uint8_t is_waypoint(const coord_def &p);
+command_type direction_to_command(int x, int y);
 bool is_resting(void);
-#ifdef CLUA_BINDINGS
-const char *trap_name(const coord_def &p);
-#endif
 void explore_pickup_event(int did_pickup, int tried_pickup);
 bool feat_is_traversable(dungeon_feature_type feat);
 bool is_unknown_stair(const coord_def &p);
 
-void find_travel_pos(const coord_def& youpos, char *move_x, char *move_y,
+void find_travel_pos(const coord_def& youpos, int *move_x, int *move_y,
                      std::vector<coord_def>* coords = NULL);
 
-bool is_travelsafe_square(const coord_def& c, bool ignore_hostile = false);
+bool is_stair_exclusion(const coord_def &p);
+
+bool is_travelsafe_square(const coord_def& c, bool ignore_hostile = false,
+                          bool ignore_danger = false);
 
 /* ***********************************************************************
  * Initiates explore - the character runs around the level to map it. Note
@@ -101,7 +101,7 @@ void start_travel(const coord_def& p);
 
 command_type travel();
 
-int travel_direction(unsigned char branch, int subdungeondepth);
+int travel_direction(uint8_t branch, int subdungeondepth);
 
 void prevent_travel_to(const std::string &dungeon_feature_name);
 
@@ -114,7 +114,7 @@ bool can_travel_to(const level_id &lid);
 bool can_travel_interlevel();
 bool prompt_stop_explore(int es_why);
 
-bool travel_kill_monster(const monsters * monster);
+bool travel_kill_monster(monster_type mons);
 
 enum translevel_prompt_flags
 {
@@ -126,7 +126,7 @@ enum translevel_prompt_flags
     TPF_SHOW_ALL_BRANCHES = 0x8,
 
     TPF_DEFAULT_OPTIONS   = TPF_ALLOW_WAYPOINTS | TPF_ALLOW_UPDOWN
-                                                | TPF_REMEMBER_TARGET
+                                                | TPF_REMEMBER_TARGET,
 };
 
 travel_target prompt_translevel_target(int prompt_flags,
@@ -143,6 +143,9 @@ const int PD_EXCLUDED = -20099;
 // This square is within LOS radius of an excluded square
 const int PD_EXCLUDED_RADIUS = -20100;
 
+// This square has a damaging cloud
+const int PD_CLOUD = -20101;
+
 /* ***********************************************************************
  * Array of points on the map, each value being the distance the character
  * would have to travel to get there. Negative distances imply that the point
@@ -155,21 +158,48 @@ extern travel_distance_grid_t travel_point_distance;
 
 enum explore_stop_type
 {
-    ES_NONE                 = 0x000,
-    ES_ITEM                 = 0x001,
-    ES_GREEDY_PICKUP        = 0x002,
-    ES_GREEDY_PICKUP_SMART  = 0x004,
-    ES_GREEDY_PICKUP_THROWN = 0x008,
-    ES_GREEDY_PICKUP_MASK   = (ES_GREEDY_PICKUP | ES_GREEDY_PICKUP_SMART
-                               | ES_GREEDY_PICKUP_THROWN),
-    ES_GREEDY_ITEM          = 0x010,
-    ES_STAIR                = 0x020,
-    ES_SHOP                 = 0x040,
-    ES_ALTAR                = 0x080,
-    ES_PORTAL               = 0x100,
-    ES_GLOWING_ITEM         = 0x200,
-    ES_ARTEFACT             = 0x400,
-    ES_RUNE                 = 0x800
+    ES_NONE                      = 0x0000,
+
+    // Explored into view of an item that is NOT eligible for autopickup.
+    ES_ITEM                      = 0x0001,
+
+    // Picked up an item during greedy explore; will stop for anything
+    // that's not explicitly ignored and that is not gold.
+    ES_GREEDY_PICKUP             = 0x0002,
+
+    // Stop when picking up gold with greedy explore.
+    ES_GREEDY_PICKUP_GOLD        = 0x0004,
+
+    // Picked up an item during greedy explore, ignoring items that were
+    // thrown by the PC, and items that the player already has one of in
+    // inventory, or a bunch of other conditions (see
+    // _interesting_explore_pickup in items.cc)
+    ES_GREEDY_PICKUP_SMART       = 0x0008,
+
+    // Greedy-picked up an item previously thrown by the PC.
+    ES_GREEDY_PICKUP_THROWN      = 0x0010,
+    ES_GREEDY_PICKUP_MASK        = (ES_GREEDY_PICKUP
+                                    | ES_GREEDY_PICKUP_GOLD
+                                    | ES_GREEDY_PICKUP_SMART
+                                    | ES_GREEDY_PICKUP_THROWN),
+
+    // Explored into view of an item eligible for autopickup.
+    ES_GREEDY_ITEM               = 0x0020,
+
+    // Stepped onto a stack of items that was previously unknown to
+    // the player (for instance, when stepping onto the heap of items
+    // of a freshly killed monster).
+    ES_GREEDY_VISITED_ITEM_STACK = 0x0040,
+
+    // Explored into view of a stair, shop, altar, portal, glowing
+    // item, or artefact.
+    ES_STAIR                     = 0x0080,
+    ES_SHOP                      = 0x0100,
+    ES_ALTAR                     = 0x0200,
+    ES_PORTAL                    = 0x0400,
+    ES_GLOWING_ITEM              = 0x0800,
+    ES_ARTEFACT                  = 0x1000,
+    ES_RUNE                      = 0x2000,
 };
 
 ////////////////////////////////////////////////////////////////////////////
@@ -230,6 +260,7 @@ private:
         }
     };
 
+    bool can_autopickup;
     int es_flags;
     const LevelStashes *current_level;
     std::vector< named_thing<item_def> > items;
@@ -261,7 +292,7 @@ public:
     enum stair_type
     {
         PHYSICAL,
-        PLACEHOLDER
+        PLACEHOLDER,
     };
 
 public:
@@ -295,10 +326,11 @@ struct LevelInfo
 {
     LevelInfo() : stairs(), excludes(), stair_distances(), id()
     {
+        da_counters.init(0);
     }
 
     void save(writer&) const;
-    void load(reader&, char minorVersion);
+    void load(reader&, int minorVersion);
 
     std::vector<stair_info> &get_stairs()
     {
@@ -323,6 +355,7 @@ struct LevelInfo
     // or does not exist in our list of stairs, returns 0.
     int distance_between(const stair_info *s1, const stair_info *s2) const;
 
+    void update_excludes();
     void update();              // Update LevelInfo to be correct for the
                                 // current level.
 
@@ -335,7 +368,9 @@ struct LevelInfo
 
     // Returns true if the given branch is known to be accessible from the
     // current level.
-    bool is_known_branch(unsigned char branch) const;
+    bool is_known_branch(uint8_t branch) const;
+
+    FixedVector<int, NUM_DA_COUNTERS> da_counters;
 
 private:
     // Gets a list of coordinates of all player-known stairs on the current
@@ -346,6 +381,7 @@ private:
     void update_stair_distances();
     void sync_all_branch_stairs();
     void sync_branch_stairs(const stair_info *si);
+    void set_distance_between_stairs(int a, int b, int dist);
     void fixup();
 
 private:
@@ -384,11 +420,17 @@ public:
         return (i != levels.end()? &i->second : NULL);
     }
 
+    void erase_level_info(const level_id& lev)
+    {
+        levels.erase(lev);
+    }
+
     bool know_stair(const coord_def &c) const;
     bool know_level(const level_id &lev) const
     {
         return levels.find(lev) != levels.end();
     }
+    std::vector<level_id> known_levels() const;
 
     const level_pos &get_waypoint(int number) const
     {
@@ -401,17 +443,22 @@ public:
 
     void add_waypoint(int x = -1, int y = -1);
     void delete_waypoint();
-    unsigned char is_waypoint(const level_pos &lp) const;
+    uint8_t is_waypoint(const level_pos &lp) const;
     void list_waypoints() const;
     void update_waypoints() const;
 
-
+    void update_excludes();
     void update();
 
     void save(writer&) const;
-    void load(reader&, char minorVersion);
+    void load(reader&, int minorVersion);
 
-    bool is_known_branch(unsigned char branch) const;
+    bool is_known_branch(uint8_t branch) const;
+
+    void update_da_counters(); // of the current level
+
+    unsigned int query_da_counter(daction_type c);
+    void clear_da_counter(daction_type c);
 
 private:
     void fixup_levels();
@@ -472,6 +519,11 @@ public:
     // RMODE_EXPLORE_GREEDY.
     const coord_def unexplored_square() const;
 
+    inline void set_ignore_danger()
+    {
+        ignore_danger = true;
+    }
+
 protected:
     bool is_greed_inducing_square(const coord_def &c) const;
     bool path_examine_point(const coord_def &c);
@@ -504,6 +556,10 @@ protected:
     // hostile squares.
     bool ignore_hostile;
 
+    // Set to true for Tiles mode clicking, so you can move one step
+    // at a time through excluded areas and around stationary monsters.
+    bool ignore_danger;
+
     // If true, use magic numbers in point distance array which can be
     // used to colour the level-map.
     bool annotate_map;
@@ -554,4 +610,12 @@ extern TravelCache travel_cache;
 
 void do_interlevel_travel();
 
+// Travel from a mouse click.  Take one step if not safe.  Attack if adjacent.
+// If force is true, then the player will attack empty squares/open doors.
+#ifdef USE_TILE
+int click_travel(const coord_def &gc, bool force);
+#endif
+
+bool check_for_interesting_features();
+
 #endif // TRAVEL_H
