@@ -1,15 +1,13 @@
-/*
- *  File:       view.cc
- *  Summary:    Misc function used to render the dungeon.
- *  Written by: Linley Henzell
- */
+/**
+ * @file
+ * @brief Misc function used to render the dungeon.
+**/
 
 #include "AppHdr.h"
 
 #include "view.h"
 #include "shout.h"
 
-#include <stdint.h>
 #include <string.h>
 #include <cmath>
 #include <sstream>
@@ -21,7 +19,6 @@
 #include "map_knowledge.h"
 #include "viewchar.h"
 #include "viewgeom.h"
-#include "viewmap.h"
 #include "showsymb.h"
 
 #include "attitude-change.h"
@@ -36,10 +33,14 @@
 #include "delay.h"
 #include "dgn-overview.h"
 #include "directn.h"
+#include "env.h"
 #include "exclude.h"
 #include "feature.h"
 #include "files.h"
-#include "godabil.h"
+#include "fprop.h"
+#include "godconduct.h"
+#include "godpassive.h"
+#include "hints.h"
 #include "libutil.h"
 #include "macro.h"
 #include "message.h"
@@ -48,63 +49,57 @@
 #include "mon-iter.h"
 #include "mon-stuff.h"
 #include "mon-util.h"
-#include "newgame.h"
 #include "options.h"
 #include "notes.h"
 #include "output.h"
 #include "player.h"
 #include "random.h"
-#include "stuff.h"
-#include "env.h"
-#include "spells3.h"
 #include "stash.h"
-#include "tiles.h"
-#include "travel.h"
 #include "state.h"
+#include "stuff.h"
 #include "terrain.h"
 #include "tilemcache.h"
 #include "tilesdl.h"
 #include "travel.h"
-#include "tutorial.h"
 #include "xom.h"
 
 #ifdef USE_TILE
-#include "tiledef-dngn.h"
+ #include "tilepick.h"
+ #include "tilepick-p.h"
+ #include "tileview.h"
 #endif
 
-#define DEBUG_PANE_BOUNDS 0
+//#define DEBUG_PANE_BOUNDS
 
 crawl_view_geometry crawl_view;
 
-bool is_notable_terrain(dungeon_feature_type ftype)
+bool handle_seen_interrupt(monster* mons, std::vector<std::string>* msgs_buf)
 {
-    return (get_feature_def(ftype).is_notable());
-}
+    if (mons_is_unknown_mimic(mons))
+        return false;
 
-void handle_seen_interrupt(monsters* monster)
-{
-    if (mons_is_unknown_mimic(monster))
-        return;
-
-    activity_interrupt_data aid(monster);
-    if (!monster->seen_context.empty())
-        aid.context = monster->seen_context;
+    activity_interrupt_data aid(mons);
+    if (!mons->seen_context.empty())
+        aid.context = mons->seen_context;
     // XXX: Hack to make the 'seen' monster spec flag work.
-    else if (testbits(monster->flags, MF_WAS_IN_VIEW)
-        || testbits(monster->flags, MF_SEEN))
+    else if (testbits(mons->flags, MF_WAS_IN_VIEW)
+        || testbits(mons->flags, MF_SEEN))
     {
         aid.context = "already seen";
     }
     else
         aid.context = "newly seen";
 
-    if (!mons_is_safe(monster)
-        && !mons_class_flag(monster->type, M_NO_EXP_GAIN)
-            || monster->type == MONS_BALLISTOMYCETE && monster->number > 0)
+    seen_monster(mons);
+
+    if (!mons_is_safe(mons)
+        && !mons_class_flag(mons->type, M_NO_EXP_GAIN)
+            || mons->type == MONS_BALLISTOMYCETE && mons->number > 0)
     {
-        interrupt_activity(AI_SEE_MONSTER, aid);
+        return interrupt_activity(AI_SEE_MONSTER, aid, msgs_buf);
     }
-    seen_monster( monster );
+
+    return false;
 }
 
 void flush_comes_into_view()
@@ -115,7 +110,7 @@ void flush_comes_into_view()
         return;
     }
 
-    monsters* mon = crawl_state.which_mon_acting();
+    monster* mon = crawl_state.which_mon_acting();
 
     if (!mon || !mon->alive() || (mon->flags & MF_WAS_IN_VIEW)
         || !you.can_see(mon))
@@ -126,9 +121,12 @@ void flush_comes_into_view()
     handle_seen_interrupt(mon);
 }
 
-void monster_grid_updates()
+void seen_monsters_react()
 {
-    for (monster_iterator mi(&you.get_los()); mi; ++mi)
+    if (you.duration[DUR_TIME_STEP] || crawl_state.game_is_arena())
+        return;
+
+    for (monster_iterator mi(you.get_los()); mi; ++mi)
     {
         if ((mi->asleep() || mons_is_wandering(*mi))
             && check_awaken(*mi))
@@ -143,12 +141,11 @@ void monster_grid_updates()
         good_god_follower_attitude_change(*mi);
         beogh_follower_convert(*mi);
         slime_convert(*mi);
-        // XXX: Probably quite hackish. Allows for monsters going berserk when
-        //      they see the player. Currently only used for Duvessa, see the
-        //      function _elven_twin_dies in mon-stuff.cc.
-        if (mi->flags & MF_GOING_BERSERK)
+
+        // XXX: Hack for triggering Duvessa's going berserk.
+        if (mi->props.exists("duvessa_berserk"))
         {
-            mi->flags &= ~MF_GOING_BERSERK;
+            mi->props.erase("duvessa_berserk");
             mi->go_berserk(true);
         }
 
@@ -162,9 +159,95 @@ void monster_grid_updates()
     }
 }
 
+static std::string _desc_mons_type_map(std::map<monster_type, int> types)
+{
+    std::string message;
+    unsigned int count = 1;
+    for (std::map<monster_type, int>::iterator it = types.begin();
+         it != types.end(); it++)
+    {
+        std::string name;
+        description_level_type desc;
+        if (it->second == 1)
+        {
+            desc = (it == types.begin() ? DESC_CAP_A
+                                                : DESC_NOCAP_A);
+        }
+        else
+            desc = DESC_PLAIN;
+
+        name = mons_type_name(it->first, desc);
+        if (it->second > 1)
+        {
+            name = make_stringf("%d %s", it->second,
+                                pluralise(name).c_str());
+        }
+
+        message += name;
+        if (count == types.size() - 1)
+            message += " and ";
+        else if (count < types.size())
+            message += ", ";
+        ++count;
+    }
+    return make_stringf("%s come into view.", message.c_str());
+}
+
+/*
+ * Monster list simplification
+ *
+ * When too many monsters come into view at once, we group the ones with the
+ * same genus, starting with the most represented genus.
+ *
+ * @param types monster types and the number of monster for each type.
+ * @param genera monster genera and the number of monster for each genus.
+ */
+static void _genus_factoring(std::map<monster_type, int> &types,
+                             std::map<monster_type, int> &genera)
+{
+    monster_type genus = MONS_NO_MONSTER;
+    int num = 0;
+    std::map<monster_type, int>::iterator it;
+    // Find the most represented genus.
+    for (it = genera.begin(); it != genera.end(); it++)
+        if (it->second > num)
+        {
+            genus = it->first;
+            num = it->second;
+        }
+
+    // The most represented genus has a single member.
+    // No more factoring is possible, we're done.
+    if (num == 1)
+    {
+        genera.clear();
+        return;
+    }
+
+    genera.erase(genus);
+    it = types.begin();
+    do
+    {
+        if (mons_genus(it->first) != genus)
+            continue;
+
+        // This genus has a single monster type. Can't factor.
+        if (it->second == num)
+            return;
+
+        types.erase(it->first);
+
+    } while (++it != types.end());
+
+    types[genus] = num;
+}
+
 void update_monsters_in_view()
 {
-    unsigned int num_hostile = 0;
+    const unsigned int max_msgs = 4;
+    int num_hostile = 0;
+    std::vector<std::string> msgs;
+    std::vector<monster*> monsters;
 
     for (monster_iterator mi; mi; ++mi)
     {
@@ -182,7 +265,8 @@ void update_monsters_in_view()
             }
             else if (mi->visible_to(&you))
             {
-                handle_seen_interrupt(*mi);
+                if (handle_seen_interrupt(*mi, &msgs))
+                    monsters.push_back(*mi);
                 seen_monster(*mi);
             }
             else
@@ -193,7 +277,65 @@ void update_monsters_in_view()
 
         // If the monster hasn't been seen by the time that the player
         // gets control back then seen_context is out of date.
-        mi->seen_context.clear();
+        if (!you.turn_is_over)
+            mi->seen_context.clear();
+    }
+
+    if (!msgs.empty())
+    {
+        unsigned int size = monsters.size();
+        std::map<monster_type, int> types;
+        std::map<monster_type, int> genera; // This is the plural for genus!
+        for (unsigned int i = 0; i < size; ++i)
+        {
+            monster_type type;
+            if (monsters[i]->props.exists("mislead_as") && you.misled())
+                type = monsters[i]->get_mislead_type();
+            else
+                type = monsters[i]->type;
+
+            types[type]++;
+            genera[mons_genus(type)]++;
+        }
+
+        if (size == 1)
+            mpr(msgs[0], MSGCH_WARN);
+        else
+        {
+            while (types.size() > max_msgs && !genera.empty())
+                _genus_factoring(types, genera);
+            mpr(_desc_mons_type_map(types), MSGCH_WARN);
+        }
+
+        bool warning = false;
+        std::string warning_msg = "Ashenzari warns you: ";
+        for (unsigned int i = 0; i < size; ++i)
+        {
+            const monster* mon = monsters[i];
+            if (!mon->props.exists("ash_id"))
+                continue;
+
+            if (warning)
+                warning_msg += " ";
+            else
+                warning = true;
+
+            if (size == 1)
+                warning_msg += mon->pronoun(PRONOUN_CAP);
+            else if (mon->type == MONS_DANCING_WEAPON)
+                warning_msg += "There";
+            else if (types[mon->type] == 1)
+                warning_msg += mon->full_name(DESC_CAP_THE);
+            else
+                warning_msg += mon->full_name(DESC_CAP_A);
+
+            warning_msg += " is";
+            warning_msg += get_monster_equipment_desc(mon, DESC_IDENTIFIED,
+                                                      DESC_NONE);
+            warning_msg += ".";
+        }
+        if (warning)
+            mpr(warning_msg, MSGCH_GOD);
     }
 
     // Xom thinks it's hilarious the way the player picks up an ever
@@ -219,12 +361,12 @@ void update_monsters_in_view()
 // Random difficulties are used in the few cases where we want repeated maps
 // to give different results; scrolls and cards, since they are a finite
 // resource.
-static const FixedArray<char, GXM, GYM>& _tile_difficulties(bool random)
+static const FixedArray<uint8_t, GXM, GYM>& _tile_difficulties(bool random)
 {
     // We will often be called with the same level parameter and cutoff, so
     // cache this (DS with passive mapping autoexploring could be 5000 calls
     // in a second or so).
-    static FixedArray<char, GXM, GYM> cache;
+    static FixedArray<uint8_t, GXM, GYM> cache;
     static int cache_seed = -1;
 
     int seed = random ? -1 :
@@ -327,14 +469,14 @@ bool magic_mapping(int map_radius, int proportion, bool suppress_msg,
     }
 
     // now gradually weaker with distance:
-    const int pfar     = (map_radius * 7) / 10;
-    const int very_far = (map_radius * 9) / 10;
+    const int pfar     = dist_range((map_radius * 7) / 10);
+    const int very_far = dist_range((map_radius * 9) / 10);
 
     bool did_map = false;
     int  num_altars        = 0;
     int  num_shops_portals = 0;
 
-    const FixedArray<char, GXM, GYM>& difficulty =
+    const FixedArray<uint8_t, GXM, GYM>& difficulty =
         _tile_difficulties(!deterministic);
 
     std::auto_ptr<FixedArray<bool, GXM, GYM> > detectable;
@@ -349,7 +491,7 @@ bool magic_mapping(int map_radius, int proportion, bool suppress_msg,
         {
             int threshold = proportion;
 
-            const int dist = grid_distance( you.pos(), *ri );
+            const int dist = distance(you.pos(), *ri);
 
             if (dist > very_far)
                 threshold = threshold / 3;
@@ -360,10 +502,10 @@ bool magic_mapping(int map_radius, int proportion, bool suppress_msg,
                 continue;
         }
 
-        if (is_terrain_changed(*ri))
-            clear_map_knowledge_grid(*ri);
+        if (env.map_knowledge(*ri).changed())
+            env.map_knowledge(*ri).clear();
 
-        if (!wizard_map && (is_terrain_seen(*ri) || is_terrain_mapped(*ri)))
+        if (!wizard_map && (env.map_knowledge(*ri).seen() || env.map_knowledge(*ri).mapped()))
             continue;
 
         if (!wizard_map && !deterministic && !((*detectable)(*ri)))
@@ -390,9 +532,9 @@ bool magic_mapping(int map_radius, int proportion, bool suppress_msg,
         if (open)
         {
             if (wizard_map)
-                set_map_knowledge_obj(*ri, grd(*ri));
-            else if (!get_map_knowledge_obj(*ri))
-                set_map_knowledge_obj(*ri, magic_map_base_feat(grd(*ri)));
+                env.map_knowledge(*ri).set_feature(grd(*ri));
+            else if (!env.map_knowledge(*ri).feat())
+                env.map_knowledge(*ri).set_feature(magic_map_base_feat(grd(*ri)));
 
             if (wizard_map)
             {
@@ -401,10 +543,7 @@ bool magic_mapping(int map_radius, int proportion, bool suppress_msg,
 
                 set_terrain_seen(*ri);
 #ifdef USE_TILE
-                // Can't use set_map_knowledge_obj because that would
-                // overwrite the gmap.
-                env.tile_bk_bg(*ri) = tile_idx_unseen_terrain(ri->x, ri->y,
-                                                              grd(*ri));
+                tile_wizmap_terrain(*ri);
 #endif
             }
             else
@@ -446,123 +585,106 @@ bool magic_mapping(int map_radius, int proportion, bool suppress_msg,
     return (did_map);
 }
 
-// Is the given monster near (in LOS of) the player?
-bool mons_near(const monsters *monster)
+void fully_map_level()
 {
-    if (crawl_state.arena || crawl_state.arena_suspended)
-        return (true);
-    return (you.see_cell(monster->pos()));
+    for (rectangle_iterator ri(1); ri; ++ri)
+    {
+        bool ok = false;
+        for (adjacent_iterator ai(*ri, false); ai; ++ai)
+            if (grd(*ai) >= DNGN_MINSEE)
+                ok = true;
+        if (!ok)
+            continue;
+        set_terrain_visible(*ri);
+        env.map_knowledge(*ri).set_feature(grd(*ri));
+        if (igrd(*ri) != NON_ITEM)
+            env.map_knowledge(*ri).set_detected_item();
+        env.pgrid(*ri) |= FPROP_SEEN_OR_NOEXP;
+    }
 }
 
-bool mon_enemies_around(const monsters *monster)
+// Is the given monster near (in LOS of) the player?
+bool mons_near(const monster* mons)
+{
+    if (crawl_state.game_is_arena() || crawl_state.arena_suspended)
+        return (true);
+    return (you.see_cell(mons->pos()));
+}
+
+bool mon_enemies_around(const monster* mons)
 {
     // If the monster has a foe, return true.
-    if (monster->foe != MHITNOT && monster->foe != MHITYOU)
+    if (mons->foe != MHITNOT && mons->foe != MHITYOU)
         return (true);
 
-    if (crawl_state.arena)
+    if (crawl_state.game_is_arena())
     {
         // If the arena-mode code in _handle_behaviour() hasn't set a foe then
         // we don't have one.
         return (false);
     }
-    else if (monster->wont_attack())
+    else if (mons->wont_attack())
     {
         // Additionally, if an ally is nearby and *you* have a foe,
         // consider it as the ally's enemy too.
-        return (mons_near(monster) && there_are_monsters_nearby(true));
+        return (mons_near(mons) && there_are_monsters_nearby(true));
     }
     else
     {
-        // For hostile monsters *you* are the main enemy.
-        return (mons_near(monster));
+        // For hostile monster* you* are the main enemy.
+        return (mons_near(mons));
     }
 }
 
-// Returns a string containing an ASCII representation of the map. If fullscreen
-// is set to false, only the viewable area is returned. Leading and trailing
-// spaces are trimmed from each line. Leading and trailing empty lines are also
-// snipped.
-std::string screenshot(bool fullscreen)
+// Returns a string containing a representation of the map.  Leading and
+// trailing spaces are trimmed from each line.  Leading and trailing empty
+// lines are also snipped.
+std::string screenshot()
 {
-    UNUSED(fullscreen);
-
-    // [ds] Screenshots need to be straight ASCII. We will now proceed to force
-    // the char and feature tables back to ASCII.
-    FixedVector<unsigned, NUM_DCHAR_TYPES> char_table_bk;
-    char_table_bk = Options.char_table;
-
-    init_char_table(CSET_ASCII);
-    init_show_table();
-
-    int firstnonspace = -1;
-    int firstpopline  = -1;
-    int lastpopline   = -1;
-
     std::vector<std::string> lines(crawl_view.viewsz.y);
-    for (int count_y = 1; count_y <= crawl_view.viewsz.y; count_y++)
+    unsigned int lsp = GXM;
+    for (int y = 0; y < crawl_view.viewsz.y; y++)
     {
-        int lastnonspace = -1;
-
-        for (int count_x = 1; count_x <= crawl_view.viewsz.x; count_x++)
+        std::string line;
+        for (int x = 0; x < crawl_view.viewsz.x; x++)
         {
             // in grid coords
-            const coord_def gc = view2grid(coord_def(count_x, count_y));
-
-            int ch =
-                  (!map_bounds(gc))             ? 0 :
-                  (!crawl_view.in_grid_los(gc)) ? get_map_knowledge_char(gc) :
-                  (gc == you.pos())             ? you.symbol
-                                                : get_screen_glyph(gc);
-
-            if (ch && !isprint(ch))
-            {
-                // [ds] Evil hack time again. Peek at grid, use that character.
-                ch = get_feat_symbol(grid_appearance(gc));
-            }
-
-            // More mangling to accommodate C strings.
-            if (!ch)
-                ch = ' ';
-
-            if (ch != ' ')
-            {
-                lastnonspace = count_x;
-                lastpopline = count_y;
-
-                if (firstnonspace == -1 || firstnonspace > count_x)
-                    firstnonspace = count_x;
-
-                if (firstpopline == -1)
-                    firstpopline = count_y;
-            }
-
-            lines[count_y - 1] += ch;
+            const coord_def gc = view2grid(crawl_view.viewp +
+                                     coord_def(x, y));
+            ucs_t ch =
+                  (!map_bounds(gc))             ? ' ' :
+                  (gc == you.pos())             ? mons_char(you.symbol)
+                                                : get_cell_glyph(gc).ch;
+            line += stringize_glyph(ch);
         }
-
-        if (lastnonspace < (int) lines[count_y - 1].length())
-            lines[count_y - 1].erase(lastnonspace + 1);
+        // right-trim the line
+        for (int x = line.length() - 1; x >= 0; x--)
+            if (line[x] == ' ')
+                line.erase(x);
+            else
+                break;
+        // see how much it can be left-trimmed
+        for (unsigned int x = 0; x < line.length(); x++)
+            if (line[x] != ' ')
+            {
+                if (lsp > x)
+                    lsp = x;
+                break;
+            }
+        lines[y] = line;
     }
 
-    // Restore char and feature tables.
-    Options.char_table = char_table_bk;
-    init_show_table();
+    for (unsigned int y = 0; y < lines.size(); y++)
+        lines[y].erase(0, lsp); // actually trim from the left
+    while (!lines.empty() && lines.back().empty())
+        lines.pop_back();       // then from the bottom
 
     std::ostringstream ss;
-    if (firstpopline != -1 && lastpopline != -1)
-    {
-        if (firstnonspace == -1)
-            firstnonspace = 0;
-
-        for (int i = firstpopline; i <= lastpopline; ++i)
-        {
-            const std::string &ref = lines[i - 1];
-            if (firstnonspace < (int) ref.length())
-                ss << ref.substr(firstnonspace);
-            ss << EOL;
-        }
-    }
-
+    unsigned int y = 0;
+    for (y = 0; y < lines.size() && lines[y].empty(); y++)
+        ;                       // ... and from the top
+    for (; y < lines.size(); y++)
+        ss << lines[y] << "\n";
     return (ss.str());
 }
 
@@ -583,22 +705,27 @@ void view_update_at(const coord_def &pos)
     if (pos == you.pos())
         return;
 
-    const coord_def vp = grid2view(pos);
-    const coord_def ep = view2show(vp);
-    env.show.update_at(pos, ep);
+    show_update_at(pos);
 
 #ifndef USE_TILE
-    show_type object = env.show(ep);
-    if (!object)
+    if (!env.map_knowledge(pos).visible())
         return;
-    glyph g = get_show_glyph(object);
+    glyph g = get_cell_glyph(pos);
 
-    int flash_colour = you.flash_colour;
-    if (flash_colour == BLACK)
-        flash_colour = _viewmap_flash_colour();
+    int flash_colour = you.flash_colour == BLACK
+        ? _viewmap_flash_colour()
+        : you.flash_colour;
+    int mons = env.map_knowledge(pos).monster();
+    int cell_colour =
+        flash_colour &&
+        (mons == MONS_NO_MONSTER || mons_class_is_firewood(mons) ||
+         !you.berserk())
+            ? real_colour(flash_colour)
+            : g.col;
 
-    cgotoxy(vp.x, vp.y);
-    put_colour_ch(flash_colour? real_colour(flash_colour) : g.col, g.ch);
+    const coord_def vp = grid2view(pos);
+    cgotoxy(vp.x, vp.y, GOTO_DNGN);
+    put_colour_ch(cell_colour, g.ch);
 
     // Force colour back to normal, else clrscr() will flood screen
     // with this colour on DOS.
@@ -607,12 +734,12 @@ void view_update_at(const coord_def &pos)
 }
 
 #ifndef USE_TILE
-void flash_monster_colour(const monsters *mon, unsigned char fmc_colour,
+void flash_monster_colour(const monster* mon, uint8_t fmc_colour,
                           int fmc_delay)
 {
     if (you.can_see(mon))
     {
-        unsigned char old_flash_colour = you.flash_colour;
+        uint8_t old_flash_colour = you.flash_colour;
         coord_def c(mon->pos());
 
         you.flash_colour = fmc_colour;
@@ -632,23 +759,23 @@ bool view_update()
 {
     if (you.num_turns > you.last_view_update)
     {
-        viewwindow(false);
+        viewwindow();
         return (true);
     }
     return (false);
 }
 
-void flash_view(int colour)
+void flash_view(uint8_t colour)
 {
     you.flash_colour = colour;
-    viewwindow(false, false);
+    viewwindow(false);
 }
 
-void flash_view_delay(int colour, long flash_delay)
+void flash_view_delay(uint8_t colour, int flash_delay)
 {
     flash_view(colour);
     // Scale delay to match change in arena_delay.
-    if (crawl_state.arena)
+    if (crawl_state.game_is_arena())
     {
         flash_delay *= Options.arena_delay;
         flash_delay /= 600;
@@ -659,7 +786,7 @@ void flash_view_delay(int colour, long flash_delay)
 
 static void _debug_pane_bounds()
 {
-#if DEBUG_PANE_BOUNDS
+#ifdef DEBUG_PANE_BOUNDS
     // Doesn't work for HUD because print_stats() overwrites it.
 
     if (crawl_view.mlistsz.y > 0)
@@ -693,29 +820,28 @@ static void _debug_pane_bounds()
 enum update_flags
 {
     UF_AFFECT_EXCLUDES = (1 << 0),
-    UF_ADDED_EXCLUDE   = (1 << 1)
+    UF_ADDED_EXCLUDE   = (1 << 1),
 };
 
 // Do various updates when the player sees a cell. Returns whether
 // exclusion LOS might have been affected.
 static int player_view_update_at(const coord_def &gc)
 {
-    const coord_def ep = grid2show(gc);
     maybe_remove_autoexclusion(gc);
     int ret = 0;
 
     // Set excludes in a radius of 1 around harmful clouds genereated
     // by neither monsters nor the player.
     const int cloudidx = env.cgrid(gc);
-    if (cloudidx != EMPTY_CLOUD && !crawl_state.arena)
+    if (cloudidx != EMPTY_CLOUD && !crawl_state.game_is_arena())
     {
         cloud_struct &cl   = env.cloud[cloudidx];
         cloud_type   ctype = cl.type;
 
         bool did_exclude = false;
-        if (!is_harmless_cloud(ctype)
-            && cl.whose  == KC_OTHER
-            && cl.killer == KILL_MISC)
+        if (cl.whose  == KC_OTHER
+            && cl.killer == KILL_MISC
+            && is_damaging_cloud(cl.type, false))
         {
             // Steam clouds are less dangerous than the other ones,
             // so don't exclude the neighbour cells.
@@ -727,19 +853,28 @@ static int player_view_update_at(const coord_def &gc)
         }
     }
 
-    // Print tutorial messages for features in LOS.
-    if (Tutorial.tutorial_left)
-        tutorial_observe_cell(gc);
+    // Print hints mode messages for features in LOS.
+    if (crawl_state.game_is_hints())
+        hints_observe_cell(gc);
 
-    if (is_terrain_changed(gc) || !is_terrain_seen(gc))
+    if (env.map_knowledge(gc).changed() || !env.map_knowledge(gc).seen())
         ret |= UF_AFFECT_EXCLUDES;
 
-    set_terrain_seen(gc);
-    set_map_knowledge_obj(gc, to_knowledge(env.show(ep)));
-    set_map_knowledge_detected_mons(gc, false);
-    set_map_knowledge_detected_item(gc, false);
+    set_terrain_visible(gc);
+
+    if (!(env.pgrid(gc) & FPROP_SEEN_OR_NOEXP))
+    {
+        env.pgrid(gc) |= FPROP_SEEN_OR_NOEXP;
+        if (!crawl_state.game_is_arena())
+        {
+            const int density = env.density ? env.density : 2000;
+            did_god_conduct(DID_EXPLORATION, density);
+            you.exploration += div_rand_round(1<<16, density);
+        }
+    }
 
 #ifdef USE_TILE
+    const coord_def ep = grid2show(gc);
     if (!player_in_mappable_area())
         return (ret); // XXX: is this necessary?
 
@@ -759,7 +894,7 @@ static void player_view_update()
 {
     std::vector<coord_def> update_excludes;
     bool need_update = false;
-    for (radius_iterator ri(&you.get_los()); ri; ++ri)
+    for (radius_iterator ri(you.get_los()); ri; ++ri)
     {
         int flags = player_view_update_at(*ri);
         if (flags & UF_AFFECT_EXCLUDES)
@@ -774,151 +909,109 @@ static void player_view_update()
         deferred_exclude_update();
 }
 
+static void _draw_out_of_bounds(screen_cell_t *cell)
+{
+    cell->glyph  = ' ';
+    cell->colour = DARKGREY;
 #ifdef USE_TILE
-void tile_draw_floor()
-{
-    for (int cy = 0; cy < env.tile_fg.height(); cy++)
-        for (int cx = 0; cx < env.tile_fg.width(); cx++)
-        {
-            const coord_def ep(cx, cy);
-            const coord_def gc = show2grid(ep);
-
-            int bg = TILE_DNGN_UNSEEN | tile_unseen_flag(gc);
-
-            if (you.see_cell(gc))
-            {
-                dungeon_feature_type feat = grid_appearance(gc);
-                bg = tileidx_feature(grd(gc), gc.x, gc.y);
-
-                if (feat == DNGN_DETECTED_SECRET_DOOR)
-                     bg |= TILE_FLAG_WAS_SECRET;
-                else if (is_unknown_stair(gc))
-                     bg |= TILE_FLAG_NEW_STAIR;
-            }
-
-
-            // init tiles
-            env.tile_bg[ep.x][ep.y] = bg;
-            env.tile_fg[ep.x][ep.y] = 0;
-        }
-}
-#endif
-
-static void draw_unseen(screen_buffer_t* buffy, const coord_def &gc)
-{
-#ifndef USE_TILE
-    buffy[0] = ' ';
-    buffy[1] = DARKGREY;
-#else
-    tileidx_unseen(buffy[0], buffy[1], ' ', gc);
+    cell->tile.fg = 0;
+    cell->tile.bg = tileidx_out_of_bounds(you.where_are_you);
 #endif
 }
 
-static void draw_outside_los(screen_buffer_t* buffy, const coord_def &gc)
+static void _draw_outside_los(screen_cell_t *cell, const coord_def &gc)
 {
-#ifndef USE_TILE
     // Outside the env.show area.
-    buffy[0] = get_map_knowledge_char(gc);
-    buffy[1] = DARKGREY;
-    if (Options.colour_map)
-        buffy[1] = real_colour(get_map_col(gc, false));
-#else
-    unsigned int bg = env.tile_bk_bg(gc);
-    unsigned int fg = env.tile_bk_fg(gc);
-    if (bg == 0 && fg == 0)
-        tileidx_unseen(fg, bg, get_map_knowledge_char(gc), gc);
+    glyph g = get_cell_glyph(gc, Options.clean_map);
+    cell->glyph  = g.ch;
+    cell->colour = g.col;
 
-    buffy[0] = fg;
-    buffy[1] = bg | tile_unseen_flag(gc);
+#ifdef USE_TILE
+    tileidx_out_of_los(&cell->tile.fg, &cell->tile.bg, gc);
 #endif
 }
 
-static void draw_player(screen_buffer_t* buffy,
-                        const coord_def& gc, const coord_def& ep)
+static void _draw_player(screen_cell_t *cell,
+                         const coord_def &gc, const coord_def &ep,
+                         bool anim_updates)
 {
-#ifndef USE_TILE
     // Player overrides everything in cell.
-    buffy[0] = you.symbol;
-    buffy[1] = you.colour;
+    cell->glyph  = mons_char(you.symbol);
+    cell->colour = mons_class_colour(you.symbol);
     if (you.swimming())
     {
         if (grd(gc) == DNGN_DEEP_WATER)
-            buffy[1] = BLUE;
+            cell->colour = BLUE;
         else
-            buffy[1] = CYAN;
+            cell->colour = CYAN;
     }
     if (Options.use_fake_player_cursor)
-        buffy[1] |= COLFLAG_REVERSE;
+        cell->colour |= COLFLAG_REVERSE;
+
+#ifdef USE_TILE
+    cell->tile.fg = env.tile_fg(ep) = tileidx_player();
+    cell->tile.bg = env.tile_bg(ep);
+    if (anim_updates)
+        tile_apply_animations(cell->tile.bg, &env.tile_flv(gc));
 #else
-    buffy[0] = env.tile_fg(ep) = tileidx_player(you.char_class);
-    buffy[1] = env.tile_bg(ep);
+    UNUSED(anim_updates);
 #endif
 }
 
-static void draw_los(screen_buffer_t* buffy,
-                     const coord_def& gc, const coord_def& ep)
+static void _draw_los(screen_cell_t *cell,
+                      const coord_def &gc, const coord_def &ep,
+                      bool anim_updates)
 {
-#ifndef USE_TILE
-    glyph g = get_show_glyph(env.show(ep));
-    buffy[0] = g.ch;
-    buffy[1] = g.col;
+    glyph g = get_cell_glyph(gc);
+    cell->glyph  = g.ch;
+    cell->colour = g.col;
+
+#ifdef USE_TILE
+    cell->tile.fg = env.tile_fg(ep);
+    cell->tile.bg = env.tile_bg(ep);
+    if (anim_updates)
+        tile_apply_animations(cell->tile.bg, &env.tile_flv(gc));
 #else
-    buffy[0] = env.tile_fg(ep);
-    buffy[1] = env.tile_bg(ep);
+    UNUSED(anim_updates);
 #endif
 }
 
-static void draw_los_backup(screen_buffer_t* buffy,
-                            const coord_def& gc, const coord_def& ep)
-{
-#ifndef USE_TILE
-    buffy[0] = get_map_knowledge_char(gc);
-    buffy[1] = DARKGREY;
-
-    if (Options.colour_map)
-        buffy[1] = real_colour(get_map_col(gc, false));
-#else
-    if (env.tile_bk_fg(gc) != 0
-        || env.tile_bk_bg(gc) != 0)
-    {
-        buffy[0] = env.tile_bk_fg(gc);
-        buffy[1] = env.tile_bk_bg(gc) | tile_unseen_flag(gc);
-    }
-    else
-        tileidx_unseen(buffy[0], buffy[1], get_map_knowledge_char(gc), gc);
-#endif
-}
+static bool _show_terrain = false;
 
 //---------------------------------------------------------------
 //
 // Draws the main window using the character set returned
 // by get_show_glyph().
 //
-// If monster_updates is set, stealth and conversion checks
-// take place. Should be set once per turn.
-//
 // If show_updates is set, env.show and dependent structures
 // are updated. Should be set if anything in view has changed.
 //---------------------------------------------------------------
-void viewwindow(bool monster_updates, bool show_updates)
+void viewwindow(bool show_updates)
 {
     if (you.duration[DUR_TIME_STEP])
         return;
 
-    screen_buffer_t *buffy(crawl_view.vbuf);
+    screen_cell_t *cell(crawl_view.vbuf);
+
+    // Update the animation of cells only once per turn.
+    const bool anim_updates = (you.last_view_update != you.num_turns);
+    // Except for elemental colours, which should be updated every refresh.
+    you.frame_no++;
 
 #ifdef USE_TILE
     tiles.clear_text_tags(TAG_NAMED_MONSTER);
+
     if (show_updates)
         mcache.clear_nonref();
 #endif
 
-    if (show_updates)
+    if (show_updates || _show_terrain)
     {
         if (!player_in_mappable_area())
+        {
             env.map_knowledge.init(map_cell());
-
-        you.update_los();
+            ash_detect_portals(false);
+        }
 
 #ifdef USE_TILE
         tile_draw_floor();
@@ -926,12 +1019,9 @@ void viewwindow(bool monster_updates, bool show_updates)
         tiles.clear_overlays();
 #endif
 
-        env.show.init();
+        show_init(_show_terrain);
     }
 
-    if (monster_updates && !crawl_state.arena)
-        monster_grid_updates();
-
     if (show_updates)
         player_view_update();
 
@@ -939,7 +1029,12 @@ void viewwindow(bool monster_updates, bool show_updates)
                 && (!you.running.is_explore() || Options.explore_delay < 0);
 
     if (run_dont_draw || you.asleep())
+    {
+        // Reset env.show if we munged it.
+        if (_show_terrain)
+            show_init();
         return;
+    }
 
     cursor_control cs(false);
 
@@ -947,77 +1042,120 @@ void viewwindow(bool monster_updates, bool show_updates)
     if (flash_colour == BLACK)
         flash_colour = _viewmap_flash_colour();
 
-    const coord_def &tl = crawl_view.viewp;
-    const coord_def br  = tl + crawl_view.viewsz - coord_def(1,1);
-    int bufcount = 0;
-    for (rectangle_iterator ri(tl, br); ri; ++ri, bufcount += 2)
+    const coord_def tl = coord_def(1, 1);
+    const coord_def br = crawl_view.viewsz;
+    for (rectangle_iterator ri(tl, br); ri; ++ri)
     {
+#ifdef USE_TILE
+        cell->tile.clear();
+#endif
+
         // in grid coords
         const coord_def gc = view2grid(*ri);
-        const coord_def ep = view2show(grid2view(gc));
+        const coord_def ep = grid2show(gc);
 
         if (!map_bounds(gc))
-            draw_unseen(&buffy[bufcount], gc);
-        else if (!crawl_view.in_grid_los(gc))
-            draw_outside_los(&buffy[bufcount], gc);
-        else if (gc == you.pos() && you.on_current_level
-                 && !crawl_state.arena && !crawl_state.arena_suspended)
+            _draw_out_of_bounds(cell);
+        else if (!crawl_view.in_los_bounds_g(gc))
+            _draw_outside_los(cell, gc);
+        else if (gc == you.pos() && you.on_current_level && !_show_terrain
+                 && !crawl_state.game_is_arena()
+                 && !crawl_state.arena_suspended)
         {
-            draw_player(&buffy[bufcount], gc, ep);
+            _draw_player(cell, gc, ep, anim_updates);
         }
         else if (you.see_cell(gc) && you.on_current_level)
-            draw_los(&buffy[bufcount], gc, ep);
+            _draw_los(cell, gc, ep, anim_updates);
         else
-            draw_los_backup(&buffy[bufcount], gc, ep);
+            _draw_outside_los(cell, gc);
+
+        cell->flash_colour = BLACK;
 
         // Alter colour if flashing the characters vision.
         if (flash_colour)
         {
-#ifndef USE_TILE
-            buffy[bufcount + 1] = you.see_cell(gc) ? real_colour(flash_colour)
-                                                   : DARKGREY;
+            if (you.see_cell(gc))
+            {
+#ifdef USE_TILE
+                cell->colour = real_colour(flash_colour);
+#else
+                monster_type mons = env.map_knowledge(gc).monster();
+                if (mons == MONS_NO_MONSTER || mons_class_is_firewood(mons) ||
+                    !you.berserk())
+                {
+                    cell->colour = real_colour(flash_colour);
+                }
 #endif
+            }
+            else
+            {
+                cell->colour = DARKGREY;
+            }
+            cell->flash_colour = cell->colour;
         }
         else if (crawl_state.darken_range >= 0)
         {
-            bool out_of_range = grid_distance(you.pos(), gc) >
-                                    crawl_state.darken_range
+            const int rsq = (crawl_state.darken_range
+                             * crawl_state.darken_range) + 1;
+            bool out_of_range = distance(you.pos(), gc) > rsq
                                 || !you.see_cell(gc);
-#ifndef USE_TILE
             if (out_of_range)
-                buffy[bufcount + 1] = DARKGREY;
-#else
-            if (out_of_range && you.see_cell(gc))
-                buffy[bufcount + 1] |= TILE_FLAG_OOR;
+            {
+                cell->colour = DARKGREY;
+#ifdef USE_TILE
+                if (you.see_cell(gc))
+                    cell->tile.bg |= TILE_FLAG_OOR;
 #endif
+            }
         }
 #ifdef USE_TILE
         // Grey out grids that cannot be reached due to beholders.
         else if (you.get_beholder(gc))
-            buffy[bufcount + 1] |= TILE_FLAG_OOR;
+            cell->tile.bg |= TILE_FLAG_OOR;
+
+        else if (you.get_fearmonger(gc))
+            cell->tile.bg |= TILE_FLAG_OOR;
+
+        tile_apply_properties(gc, cell->tile);
 #endif
+
+        cell++;
     }
 
     // Leaving it this way because short flashes can occur in long ones,
     // and this simply works without requiring a stack.
     you.flash_colour = BLACK;
-    you.mold_colour = LIGHTGREEN;
-#ifndef USE_TILE
     you.last_view_update = you.num_turns;
-    puttext(crawl_view.viewp.x, crawl_view.viewp.y,
-            crawl_view.viewp.x + crawl_view.viewsz.x - 1,
-            crawl_view.viewp.y + crawl_view.viewsz.y - 1,
-            buffy);
+#ifndef USE_TILE
+    puttext(crawl_view.viewp.x, crawl_view.viewp.y, crawl_view.vbuf);
     update_monster_pane();
 #else
     tiles.set_need_redraw(you.running ? Options.tile_runrest_rate : 0);
-    tiles.load_dungeon(&buffy[0], crawl_view.vgrdc);
-    tiles.update_inventory();
+    tiles.load_dungeon(crawl_view.vbuf, crawl_view.vgrdc);
+    tiles.update_tabs();
 #endif
 
+    // Reset env.show if we munged it.
+    if (_show_terrain)
+        show_init();
+
     _debug_pane_bounds();
 }
 
+void toggle_show_terrain()
+{
+    _show_terrain = !_show_terrain;
+    if (_show_terrain)
+        mpr("Showing terrain only.");
+    else
+        mpr("Returning to normal view.");
+}
+
+void reset_show_terrain()
+{
+    _show_terrain = false;
+}
+
 ////////////////////////////////////////////////////////////////////////////
 // Term resize handling (generic).
 
