@@ -1,18 +1,21 @@
-/*
- *  File:       dbg-mon.cc
- *  Summary:    Monster related debugging functions.
- *  Written by: Linley Henzell and Jesse Jones
- */
+/**
+ * @file
+ * @brief Monster related debugging functions.
+**/
 
 #include "AppHdr.h"
 
 #include "wiz-mon.h"
 
+#include "abyss.h"
+#include "areas.h"
 #include "cio.h"
 #include "colour.h"
+#include "coord.h"
 #include "dbg-util.h"
 #include "delay.h"
 #include "dungeon.h"
+#include "env.h"
 #include "files.h"
 #include "ghost.h"
 #include "goditem.h"
@@ -21,32 +24,31 @@
 #include "jobs.h"
 #include "macro.h"
 #include "map_knowledge.h"
-#include "message.h"
-#include "mon-place.h"
-#include "terrain.h"
-#include "mgen_data.h"
-#include "coord.h"
 #include "mapdef.h"
+#include "message.h"
+#include "mgen_data.h"
+#include "mon-iter.h"
 #include "mon-pathfind.h"
+#include "mon-place.h"
 #include "mon-speak.h"
 #include "mon-stuff.h"
-#include "mon-iter.h"
 #include "mon-util.h"
 #include "output.h"
 #include "religion.h"
-#include "spells2.h"
-#include "spl-mis.h"
+#include "shout.h"
+#include "showsymb.h"
+#include "spl-miscast.h"
 #include "spl-util.h"
 #include "stuff.h"
-#include "env.h"
-#include "areas.h"
+#include "terrain.h"
 #include "view.h"
+#include "viewmap.h"
 
 #ifdef WIZARD
 // Creates a specific monster by mon type number.
 void wizard_create_spec_monster(void)
 {
-    int mon = debug_prompt_for_int( "Create which monster by number? ", true );
+    int mon = prompt_for_int("Which monster by number? ", true);
 
     if (mon == -1 || (mon >= NUM_MONSTERS
                       && mon != RANDOM_MONSTER
@@ -55,7 +57,7 @@ void wizard_create_spec_monster(void)
                       && mon != RANDOM_NONBASE_DRACONIAN
                       && mon != WANDERING_MONSTER))
     {
-        canned_msg( MSG_OK );
+        canned_msg(MSG_OK);
     }
     else
     {
@@ -83,10 +85,10 @@ void wizard_create_spec_monster_name()
     if (!err.empty())
     {
         std::string newerr;
-        // Try for a partial match, but not if the user accidently entered
+        // Try for a partial match, but not if the user accidentally entered
         // only a few letters.
         monster_type partial = get_monster_by_name(specs);
-        if (strlen(specs) >= 3 && partial != NON_MONSTER)
+        if (strlen(specs) >= 3 && partial != MONS_NO_MONSTER)
         {
             mlist.clear();
             newerr = mlist.add_mons(mons_type_name(partial, DESC_PLAIN));
@@ -158,7 +160,7 @@ void wizard_create_spec_monster_name()
     }
 
     // FIXME: This is a bit useless, seeing how you cannot set the
-    // ghost's stats, brand or level.
+    // ghost's stats, brand or level, among other things.
     if (mspec.mid == MONS_PLAYER_GHOST)
     {
         unsigned short mid = mgrd(place);
@@ -182,14 +184,14 @@ void wizard_create_spec_monster_name()
             return;
         }
 
-        monsters    &mon = menv[mid];
+        monster    &mon = menv[mid];
         ghost_demon ghost;
 
         ghost.name = "John Doe";
 
         char input_str[80];
-        mpr("Make player ghost which species? (case-sensitive) ", MSGCH_PROMPT);
-        get_input_line( input_str, sizeof( input_str ) );
+        msgwin_get_line("Make player ghost which species? (case-sensitive) ",
+                        input_str, sizeof(input_str));
 
         species_type sp_id = get_species_by_abbrev(input_str);
         if (sp_id == SP_UNKNOWN)
@@ -201,8 +203,8 @@ void wizard_create_spec_monster_name()
         }
         ghost.species = static_cast<species_type>(sp_id);
 
-        mpr("Give player ghost which background? ", MSGCH_PROMPT);
-        get_input_line( input_str, sizeof( input_str ) );
+        msgwin_get_line("Give player ghost which background? ",
+                        input_str, sizeof(input_str));
 
         int job_id = get_job_by_abbrev(input_str);
 
@@ -225,8 +227,8 @@ void wizard_create_spec_monster_name()
 
 static bool _sort_monster_list(int a, int b)
 {
-    const monsters* m1 = &menv[a];
-    const monsters* m2 = &menv[b];
+    const monster* m1 = &menv[a];
+    const monster* m2 = &menv[b];
 
     if (m1->alive() != m2->alive())
         return m1->alive();
@@ -238,7 +240,7 @@ static bool _sort_monster_list(int a, int b)
         if (!m1->alive() || !m2->alive())
             return (false);
 
-        return ( m1->name(DESC_PLAIN, true) < m2->name(DESC_PLAIN, true) );
+        return (m1->name(DESC_PLAIN, true) < m2->name(DESC_PLAIN, true));
     }
 
     const unsigned glyph1 = mons_char(m1->type);
@@ -263,7 +265,7 @@ void debug_list_monsters()
 
     std::sort(mon_nums, mon_nums + MAX_MONSTERS, _sort_monster_list);
 
-    long total_exp = 0, total_adj_exp = 0, total_nonuniq_exp = 0;
+    int total_exp = 0, total_adj_exp = 0, total_nonuniq_exp = 0;
 
     std::string prev_name = "";
     int         count     = 0;
@@ -274,7 +276,7 @@ void debug_list_monsters()
         if (invalid_monster_index(idx))
             continue;
 
-        const monsters *mi(&menv[idx]);
+        const monster* mi(&menv[idx]);
         if (!mi->alive())
             continue;
 
@@ -284,9 +286,10 @@ void debug_list_monsters()
         {
             char buf[80];
             if (count > 1)
-                sprintf(buf, "%d %s", count, pluralise(prev_name).c_str());
+                snprintf(buf, sizeof(buf), "%d %s", count,
+                         pluralise(prev_name).c_str());
             else
-                sprintf(buf, "%s", prev_name.c_str());
+                snprintf(buf, sizeof(buf), "%s", prev_name.c_str());
             mons.push_back(buf);
 
             count = 0;
@@ -313,21 +316,21 @@ void debug_list_monsters()
 
     char buf[80];
     if (count > 1)
-        sprintf(buf, "%d %s", count, pluralise(prev_name).c_str());
+        snprintf(buf, sizeof(buf), "%d %s", count, pluralise(prev_name).c_str());
     else
-        sprintf(buf, "%s", prev_name.c_str());
+        snprintf(buf, sizeof(buf), "%s", prev_name.c_str());
     mons.push_back(buf);
 
     mpr_comma_separated_list("Monsters: ", mons);
 
     if (total_adj_exp == total_exp)
     {
-        mprf("%d monsters, %ld total exp value (%ld non-uniq)",
+        mprf("%d monsters, %d total exp value (%d non-uniq)",
              nfound, total_exp, total_nonuniq_exp);
     }
     else
     {
-        mprf("%d monsters, %ld total exp value (%ld non-uniq, %ld adjusted)",
+        mprf("%d monsters, %d total exp value (%d non-uniq, %d adjusted)",
              nfound, total_exp, total_nonuniq_exp, total_adj_exp);
     }
 }
@@ -335,7 +338,7 @@ void debug_list_monsters()
 void wizard_spawn_control()
 {
     mpr("(c)hange spawn rate or (s)pawn monsters? ", MSGCH_PROMPT);
-    const int c = tolower(getch());
+    const int c = tolower(getchm());
 
     char specs[256];
     bool done = false;
@@ -361,6 +364,7 @@ void wizard_spawn_control()
         int max_spawn = MAX_MONSTERS - 50;
         for (monster_iterator mi; mi; ++mi)
             if (mi->alive())
+                max_spawn--;
 
         if (max_spawn <= 0)
         {
@@ -423,19 +427,19 @@ void debug_stethoscope(int mon)
         {
             mprf(MSGCH_DIAGNOSTICS, "cloud type: %d delay: %d",
                  env.cloud[ env.cgrid(stethpos) ].type,
-                 env.cloud[ env.cgrid(stethpos) ].decay );
+                 env.cloud[ env.cgrid(stethpos) ].decay);
         }
 
         if (!monster_at(stethpos))
         {
-            mprf(MSGCH_DIAGNOSTICS, "item grid = %d", igrd(stethpos) );
+            mprf(MSGCH_DIAGNOSTICS, "item grid = %d", igrd(stethpos));
             return;
         }
 
         i = mgrd(stethpos);
     }
 
-    monsters& mons(menv[i]);
+    monster& mons(menv[i]);
 
     // Print type of monster.
     mprf(MSGCH_DIAGNOSTICS, "%s (id #%d; type=%d loc=(%d,%d) align=%s)",
@@ -446,28 +450,30 @@ void debug_stethoscope(int mon)
           (mons.attitude == ATT_NEUTRAL)        ? "neutral" :
           (mons.attitude == ATT_GOOD_NEUTRAL)   ? "good neutral":
           (mons.attitude == ATT_STRICT_NEUTRAL) ? "strictly neutral"
-                                                : "unknown alignment") );
+                                                : "unknown alignment"));
 
     // Print stats and other info.
     mprf(MSGCH_DIAGNOSTICS,
-         "HD=%d (%lu) HP=%d/%d AC=%d EV=%d MR=%d SP=%d "
-         "energy=%d%s%s num=%d flags=%04lx",
+         "HD=%d (%u) HP=%d/%d AC=%d(%d) EV=%d MR=%d SP=%d "
+         "energy=%d%s%s mid=%u num=%d stealth=%d flags=%04"PRIx64,
          mons.hit_dice,
          mons.experience,
          mons.hit_points, mons.max_hit_points,
-         mons.ac, mons.ev,
+         mons.ac, mons.armour_class(),
+         mons.ev,
          mons.res_magic(),
          mons.speed, mons.speed_increment,
          mons.base_monster != MONS_NO_MONSTER ? " base=" : "",
          mons.base_monster != MONS_NO_MONSTER ?
          get_monster_data(mons.base_monster)->name : "",
-         mons.number, mons.flags );
+         mons.mid, mons.number, mons.stealth(), mons.flags);
 
     // Print habitat and behaviour information.
     const habitat_type hab = mons_habitat(&mons);
 
     mprf(MSGCH_DIAGNOSTICS,
-         "hab=%s beh=%s(%d) foe=%s(%d) mem=%d target=(%d,%d) god=%s",
+         "hab=%s beh=%s(%d) foe=%s(%d) mem=%d target=(%d,%d) "
+         "firing_pos=(%d,%d) god=%s",
          ((hab == HT_LAND)                       ? "land" :
           (hab == HT_AMPHIBIOUS)                 ? "amphibious" :
           (hab == HT_WATER)                      ? "water" :
@@ -490,7 +496,8 @@ void debug_stethoscope(int mon)
          mons.foe,
          mons.foe_memory,
          mons.target.x, mons.target.y,
-         god_name(mons.god).c_str() );
+         mons.firing_pos.x, mons.firing_pos.y,
+         god_name(mons.god).c_str());
 
     // Print resistances.
     mprf(MSGCH_DIAGNOSTICS, "resist: fire=%d cold=%d elec=%d pois=%d neg=%d "
@@ -545,15 +552,11 @@ void debug_stethoscope(int mon)
 // Detects all monsters on the level, using their exact positions.
 void wizard_detect_creatures()
 {
-    const int prev_detected = count_detected_mons();
-    const int num_creatures = detect_creatures(60, true);
-
-    if (!num_creatures)
-        mpr("You detect nothing.");
-    else if (num_creatures == prev_detected)
-        mpr("You detect no further creatures.");
-    else
-        mpr("You detect creatures!");
+    for (monster_iterator mi; mi; ++mi)
+    {
+        env.map_knowledge(mi->pos()).set_monster(monster_info(*mi));
+        env.map_knowledge(mi->pos()).set_detected_monster(mi->type);
+    }
 }
 
 // Dismisses all monsters on the level or all monsters that match a user
@@ -568,7 +571,7 @@ void wizard_dismiss_all_monsters(bool force_all)
 
         if (!validline)
         {
-            canned_msg( MSG_OK );
+            canned_msg(MSG_OK);
             return;
         }
     }
@@ -579,9 +582,7 @@ void wizard_dismiss_all_monsters(bool force_all)
         autotoggle_autopickup(false);
 }
 
-extern void force_monster_shout(monsters* monster);
-
-void debug_make_monster_shout(monsters* mon)
+void debug_make_monster_shout(monster* mon)
 {
     mpr("Make the monster (S)hout or (T)alk? ", MSGCH_PROMPT);
 
@@ -590,15 +591,15 @@ void debug_make_monster_shout(monsters* mon)
 
     if (type != 's' && type != 't')
     {
-        canned_msg( MSG_OK );
+        canned_msg(MSG_OK);
         return;
     }
 
-    int num_times = debug_prompt_for_int("How many times? ", false);
+    int num_times = prompt_for_int("How many times? ", false);
 
     if (num_times <= 0)
     {
-        canned_msg( MSG_OK );
+        canned_msg(MSG_OK);
         return;
     }
 
@@ -637,12 +638,12 @@ void debug_make_monster_shout(monsters* mon)
     mpr("== Done ==");
 }
 
-static bool _force_suitable(const monsters *mon)
+static bool _force_suitable(const monster* mon)
 {
     return (mon->alive());
 }
 
-void wizard_apply_monster_blessing(monsters* mon)
+void wizard_apply_monster_blessing(monster* mon)
 {
     mpr("Apply blessing of (B)eogh, The (S)hining One, or (R)andomly? ",
         MSGCH_PROMPT);
@@ -652,7 +653,7 @@ void wizard_apply_monster_blessing(monsters* mon)
 
     if (type != 'b' && type != 's' && type != 'r')
     {
-        canned_msg( MSG_OK );
+        canned_msg(MSG_OK);
         return;
     }
     god_type god = GOD_NO_GOD;
@@ -665,7 +666,7 @@ void wizard_apply_monster_blessing(monsters* mon)
         mprf("%s won't bless this monster for you!", god_name(god).c_str());
 }
 
-void wizard_give_monster_item(monsters *mon)
+void wizard_give_monster_item(monster* mon)
 {
     mon_itemuse_type item_use = mons_itemuse(mon);
     if (item_use < MONUSE_STARTING_EQUIPMENT)
@@ -674,8 +675,8 @@ void wizard_give_monster_item(monsters *mon)
         return;
     }
 
-    int player_slot = prompt_invent_item( "Give which item to monster?",
-                                          MT_DROP, -1 );
+    int player_slot = prompt_invent_item("Give which item to monster?",
+                                          MT_DROP, -1);
 
     if (player_slot == PROMPT_ABORT)
         return;
@@ -693,6 +694,7 @@ void wizard_give_monster_item(monsters *mon)
     switch (item.base_type)
     {
     case OBJ_WEAPONS:
+    case OBJ_STAVES:
         // Let wizard specify which slot to put weapon into via
         // inscriptions.
         if (item.inscription.find("first") != std::string::npos
@@ -798,7 +800,6 @@ void wizard_give_monster_item(monsters *mon)
         return;
     }
 
-    // Shouldn't we be using MONUSE_MAGIC_ITEMS?
     if (item_use == MONUSE_STARTING_EQUIPMENT
         && !mons_is_unique(mon->type))
     {
@@ -890,7 +891,10 @@ static void _move_player(const coord_def& where)
 {
     if (!you.can_pass_through_feat(grd(where)))
         grd(where) = DNGN_FLOOR;
-    move_player_to_grid(where, false, true, true);
+    move_player_to_grid(where, false, true);
+    // If necessary, update the Abyss.
+    if (you.level_type == LEVEL_ABYSS)
+        maybe_shift_abyss_around_player();
 }
 
 static void _move_monster(const coord_def& where, int mid1)
@@ -904,10 +908,10 @@ static void _move_monster(const coord_def& where, int mid1)
     if (!moves.isValid || !in_bounds(moves.target))
         return;
 
-    monsters* mon1 = &menv[mid1];
+    monster* mon1 = &menv[mid1];
 
     const int mid2 = mgrd(moves.target);
-    monsters* mon2 = monster_at(moves.target);
+    monster* mon2 = monster_at(moves.target);
 
     mon1->moveto(moves.target);
     mgrd(moves.target) = mid1;
@@ -955,7 +959,7 @@ void wizard_move_player_or_monster(const coord_def& where)
     already_moving = false;
 }
 
-void wizard_make_monster_summoned(monsters* mon)
+void wizard_make_monster_summoned(monster* mon)
 {
     int summon_type = 0;
     if (mon->is_summoned(NULL, &summon_type) || summon_type != 0)
@@ -964,11 +968,11 @@ void wizard_make_monster_summoned(monsters* mon)
         return;
     }
 
-    int dur = debug_prompt_for_int("What summon longevity (1 to 6)? ", true);
+    int dur = prompt_for_int("What summon longevity (1 to 6)? ", true);
 
     if (dur < 1 || dur > 6)
     {
-        canned_msg( MSG_OK );
+        canned_msg(MSG_OK);
         return;
     }
 
@@ -978,11 +982,11 @@ void wizard_make_monster_summoned(monsters* mon)
 
     mpr("Which summon type? ", MSGCH_PROMPT);
 
-    char choice = tolower(getch());
+    char choice = tolower(getchm());
 
     if (!(choice >= 'a' && choice <= 'g') && choice != 'm' && choice != 's')
     {
-        canned_msg( MSG_OK );
+        canned_msg(MSG_OK);
         return;
     }
 
@@ -1003,12 +1007,12 @@ void wizard_make_monster_summoned(monsters* mon)
         {
             char specs[80];
 
-            mpr("Cast which spell by name? ", MSGCH_PROMPT);
-            get_input_line( specs, sizeof( specs ) );
+            msgwin_get_line("Cast which spell by name? ",
+                            specs, sizeof(specs));
 
             if (specs[0] == '\0')
             {
-                canned_msg( MSG_OK );
+                canned_msg(MSG_OK);
                 return;
             }
 
@@ -1023,7 +1027,7 @@ void wizard_make_monster_summoned(monsters* mon)
         }
 
         default:
-            DEBUGSTR("Invalid summon type choice.");
+            die("Invalid summon type choice.");
             break;
     }
 
@@ -1031,14 +1035,14 @@ void wizard_make_monster_summoned(monsters* mon)
     mpr("Monster is now summoned.");
 }
 
-void wizard_polymorph_monster(monsters* mon)
+void wizard_polymorph_monster(monster* mon)
 {
     monster_type old_type =  mon->type;
     monster_type type     = debug_prompt_for_monster();
 
     if (type == NUM_MONSTERS)
     {
-        canned_msg( MSG_OK );
+        canned_msg(MSG_OK);
         return;
     }
 
@@ -1087,16 +1091,16 @@ void debug_pathfind(int mid)
 #endif
     coord_def dest;
     level_pos ldest;
-    show_map(ldest, false);
+    bool chose = show_map(ldest, false, true, false);
     dest = ldest.pos;
     redraw_screen();
-    if (!dest.x)
+    if (!chose)
     {
         canned_msg(MSG_OK);
         return;
     }
 
-    monsters &mon = menv[mid];
+    monster& mon = menv[mid];
     mprf("Attempting to calculate a path from (%d, %d) to (%d, %d)...",
          mon.pos().x, mon.pos().y, dest.x, dest.y);
     monster_pathfind mp;
@@ -1114,10 +1118,10 @@ void debug_pathfind(int mid)
         mpr(path_str.c_str());
         mprf("-> path length: %d", path.size());
 
-        mpr(EOL);
+        mpr("");
         path = mp.calc_waypoints();
         path_str = "";
-        mpr(EOL);
+        mpr("");
         mpr("And here are the needed waypoints: ");
         for (unsigned int i = 0; i < path.size(); ++i)
         {
@@ -1131,7 +1135,7 @@ void debug_pathfind(int mid)
 
 static void _miscast_screen_update()
 {
-    viewwindow(false);
+    viewwindow();
 
     you.redraw_status_flags =
         REDRAW_LINE_1_MASK | REDRAW_LINE_2_MASK | REDRAW_LINE_3_MASK;
@@ -1142,7 +1146,7 @@ static void _miscast_screen_update()
 #endif
 }
 
-void debug_miscast( int target_index )
+void debug_miscast(int target_index)
 {
     crawl_state.cancel_cmd_repeat();
 
@@ -1214,7 +1218,7 @@ void debug_miscast( int target_index )
         mprf("Miscasting school %s.", spelltype_long_name(school));
 
     if (spell != SPELL_NO_SPELL)
-        mpr("Enter spell_power,spell_failure: ", MSGCH_PROMPT );
+        mpr("Enter spell_power,spell_failure: ", MSGCH_PROMPT);
     else
     {
         mpr("Enter miscast_level or spell_power,spell_failure: ",
@@ -1264,7 +1268,7 @@ void debug_miscast( int target_index )
 
     // Handle repeats ourselves since miscasts are likely to interrupt
     // command repetions, especially if the player is the target.
-    int repeats = debug_prompt_for_int("Number of repetitions? ", true);
+    int repeats = prompt_for_int("Number of repetitions? ", true);
     if (repeats < 1)
     {
         canned_msg(MSG_OK);
@@ -1323,10 +1327,11 @@ void debug_miscast( int target_index )
     delete miscast;
 }
 
+#ifdef DEBUG_BONES
 void debug_ghosts()
 {
     mpr("(C)reate or (L)oad bones file?", MSGCH_PROMPT);
-    const char c = tolower(getch());
+    const char c = tolower(getchm());
 
     if (c == 'c')
         save_ghost(true);
@@ -1335,5 +1340,6 @@ void debug_ghosts()
     else
         canned_msg(MSG_OK);
 }
+#endif
 
 #endif
