@@ -1,12 +1,13 @@
-/*
- *  File:       wiz-mon.h
- *  Summary:    Monster related wizard functions.
- *  Written by: Linley Henzell and Jesse Jones
- */
+/**
+ * @file
+ * @brief Monster related wizard functions.
+**/
 
 #ifndef WIZMON_H
 #define WIZMON_H
 
+#ifdef WIZARD
+
 void wizard_create_spec_monster(void);
 void wizard_create_spec_monster_name(void);
 void wizard_spawn_control();
@@ -17,16 +18,18 @@ void debug_stethoscope(int mon);
 void debug_miscast(int target);
 void debug_ghosts();
 
-class monsters;
+class monster;
 struct coord_def;
 
-void wizard_apply_monster_blessing(monsters* mon);
-void wizard_give_monster_item(monsters* mon);
+void wizard_apply_monster_blessing(monster* mon);
+void wizard_give_monster_item(monster* mon);
 void wizard_move_player_or_monster(const coord_def& where);
-void wizard_make_monster_summoned(monsters* mon);
-void wizard_polymorph_monster(monsters* mon);
-void debug_make_monster_shout(monsters* mon);
+void wizard_make_monster_summoned(monster* mon);
+void wizard_polymorph_monster(monster* mon);
+void debug_make_monster_shout(monster* mon);
 
 void debug_pathfind(int mid);
 
 #endif
+
+#endif
