@@ -1,21 +1,23 @@
-/*
- *  File:       wiz-you.cc
- *  Summary:    Player related debugging functions.
- *  Written by: Linley Henzell and Jesse Jones
- */
+/**
+ * @file
+ * @brief Player related debugging functions.
+**/
 
 #include "AppHdr.h"
 
 #include "wiz-you.h"
 
 #include "cio.h"
+#include "debug.h"
 #include "dbg-util.h"
 #include "food.h"
 #include "godprayer.h"
+#include "godwrath.h"
 #include "libutil.h"
 #include "message.h"
 #include "mutation.h"
 #include "newgame.h"
+#include "ng-setup.h"
 #include "output.h"
 #include "player.h"
 #include "random.h"
@@ -26,16 +28,17 @@
 #include "spl-util.h"
 #include "stuff.h"
 #include "terrain.h"
+#include "view.h"
 #include "xom.h"
 
 #ifdef WIZARD
-void wizard_change_species( void )
+void wizard_change_species(void)
 {
     char specs[80];
     int i;
 
-    mpr("What species would you like to be now? " , MSGCH_PROMPT);
-    get_input_line( specs, sizeof( specs ) );
+    msgwin_get_line("What species would you like to be now? " ,
+                    specs, sizeof(specs));
 
     if (specs[0] == '\0')
         return;
@@ -46,8 +49,7 @@ void wizard_change_species( void )
     for (i = 0; i < NUM_SPECIES; ++i)
     {
         const species_type si = static_cast<species_type>(i);
-        const std::string sp_name =
-            lowercase_string(species_name(si, you.experience_level));
+        const std::string sp_name = lowercase_string(species_name(si));
 
         std::string::size_type pos = sp_name.find(specs);
         if (pos != std::string::npos)
@@ -70,98 +72,114 @@ void wizard_change_species( void )
     }
 
     // Re-scale skill-points.
-    for (i = 0; i < NUM_SKILLS; ++i)
+    for (i = SK_FIRST_SKILL; i < NUM_SKILLS; ++i)
     {
-        you.skill_points[i] *= species_skills( i, sp );
-        you.skill_points[i] /= species_skills( i, you.species );
+        skill_type sk = static_cast<skill_type>(i);
+        you.skill_points[i] *= species_apt_factor(sk, sp)
+                               / species_apt_factor(sk);
     }
 
     you.species = sp;
     you.is_undead = get_undead_state(sp);
 
     // Change permanent mutations, but preserve non-permanent ones.
-    unsigned char prev_muts[NUM_MUTATIONS];
+    uint8_t prev_muts[NUM_MUTATIONS];
     for (i = 0; i < NUM_MUTATIONS; ++i)
     {
-        if (you.demon_pow[i] > 0)
+        if (you.innate_mutations[i] > 0)
         {
-            if (you.demon_pow[i] > you.mutation[i])
+            if (you.innate_mutations[i] > you.mutation[i])
                 you.mutation[i] = 0;
             else
-                you.mutation[i] -= you.demon_pow[i];
+                you.mutation[i] -= you.innate_mutations[i];
 
-            you.demon_pow[i] = 0;
+            you.innate_mutations[i] = 0;
         }
         prev_muts[i] = you.mutation[i];
     }
     give_basic_mutations(sp);
     for (i = 0; i < NUM_MUTATIONS; ++i)
     {
-        if (prev_muts[i] > you.demon_pow[i])
-            you.demon_pow[i] = 0;
+        if (prev_muts[i] > you.innate_mutations[i])
+            you.innate_mutations[i] = 0;
         else
-            you.demon_pow[i] -= prev_muts[i];
+            you.innate_mutations[i] -= prev_muts[i];
     }
 
     switch (sp)
     {
-    case SP_GREEN_DRACONIAN:
-        if (you.experience_level >= 7)
-            perma_mutate(MUT_POISON_RESISTANCE, 1);
-        break;
-
     case SP_RED_DRACONIAN:
-        if (you.experience_level >= 14)
+        if (you.experience_level >= 7)
             perma_mutate(MUT_HEAT_RESISTANCE, 1);
         break;
 
     case SP_WHITE_DRACONIAN:
-        if (you.experience_level >= 14)
+        if (you.experience_level >= 7)
             perma_mutate(MUT_COLD_RESISTANCE, 1);
         break;
 
+    case SP_GREEN_DRACONIAN:
+        if (you.experience_level >= 7)
+            perma_mutate(MUT_POISON_RESISTANCE, 1);
+        if (you.experience_level >= 14)
+            perma_mutate(MUT_STINGER, 1);
+        break;
+
+    case SP_YELLOW_DRACONIAN:
+        if (you.experience_level >= 14)
+            perma_mutate(MUT_ACIDIC_BITE, 1);
+        break;
+
+    case SP_GREY_DRACONIAN:
+        if (you.experience_level >= 7)
+            perma_mutate(MUT_UNBREATHING, 1);
+        break;
 
     case SP_BLACK_DRACONIAN:
-        if (you.experience_level >= 18)
+        if (you.experience_level >= 7)
             perma_mutate(MUT_SHOCK_RESISTANCE, 1);
+        if (you.experience_level >= 14)
+            perma_mutate(MUT_BIG_WINGS, 1);
         break;
 
     case SP_DEMONSPAWN:
     {
-        int powers = 0;
-
-        if (you.experience_level < 4)
-            powers = 0;
-        else if (you.experience_level < 9)
-            powers = 1;
-        else if (you.experience_level < 14)
-            powers = 2;
-        else if (you.experience_level < 19)
-            powers = 3;
-        else if (you.experience_level < 24)
-            powers = 4;
-        else if (you.experience_level == 27)
-            powers = 5;
-
-        int levels[] = {4, 9, 14, 19, 27};
-        int real_level = you.experience_level;
-
-        for (i = 0; i < powers; ++i)
+        roll_demonspawn_mutations();
+        for (i = 0; i < int(you.demonic_traits.size()); ++i)
         {
-            // The types of demonspawn mutations you get depends on your
-            // experience level at the time of gaining it.
-            you.experience_level = levels[i];
-            demonspawn();
-        }
-        you.experience_level = real_level;
+            mutation_type m = you.demonic_traits[i].mutation;
 
+            if (you.demonic_traits[i].level_gained > you.experience_level)
+                continue;
+
+            ++you.mutation[m];
+            ++you.innate_mutations[m];
+        }
         break;
     }
 
+    case SP_DEEP_DWARF:
+        if (you.experience_level >= 9)
+            perma_mutate(MUT_PASSIVE_MAPPING, 1);
+        if (you.experience_level >= 14)
+            perma_mutate(MUT_NEGATIVE_ENERGY_RESISTANCE, 1);
+        if (you.experience_level >= 18)
+            perma_mutate(MUT_PASSIVE_MAPPING, 1);
+        break;
+
+    case SP_CAT:
+        if (you.experience_level >= 6)
+            perma_mutate(MUT_SHAGGY_FUR, 1);
+        if (you.experience_level >= 12)
+            perma_mutate(MUT_SHAGGY_FUR, 1);
+        break;
+
     default:
         break;
     }
 
+    burden_change();
+    update_player_symbol();
 #ifdef USE_TILE
     init_player_doll();
 #endif
@@ -177,10 +195,10 @@ void wizard_cast_spec_spell(void)
     int spell;
 
     mpr("Cast which spell? ", MSGCH_PROMPT);
-    if (cancelable_get_line_autohist( specs, sizeof( specs ) )
+    if (cancelable_get_line_autohist(specs, sizeof(specs))
         || specs[0] == '\0')
     {
-        canned_msg( MSG_OK );
+        canned_msg(MSG_OK);
         crawl_state.cancel_cmd_repeat();
         return;
     }
@@ -197,7 +215,7 @@ void wizard_cast_spec_spell(void)
         }
     }
 
-    if (your_spells( static_cast<spell_type>(spell), 0, false )
+    if (your_spells(static_cast<spell_type>(spell), 0, false)
                 == SPRET_ABORT)
     {
         crawl_state.cancel_cmd_repeat();
@@ -213,16 +231,7 @@ void wizard_heal(bool super_heal)
         you.magic_contamination = 0;
         you.duration[DUR_LIQUID_FLAMES] = 0;
         you.clear_beholders();
-
-        // If we're repeating then do the HP increase all at once.
-        int amount = 10;
-        if (crawl_state.cmd_repeat_goal > 0)
-        {
-            amount *= crawl_state.cmd_repeat_goal;
-            crawl_state.cancel_cmd_repeat();
-        }
-
-        inc_hp(amount, true);
+        inc_hp(10, true);
     }
 
     // Clear most status ailments.
@@ -249,7 +258,7 @@ void wizard_set_hunger_state()
 
     mprf(MSGCH_PROMPT, "%s", hunger_prompt.c_str());
 
-    const int c = tolower(getch());
+    const int c = tolower(getchk());
 
     // Values taken from food.cc.
     switch (c)
@@ -262,7 +271,9 @@ void wizard_set_hunger_state()
     case 'e': you.hunger = 12000; break;
     default:  canned_msg(MSG_OK); break;
     }
+
     food_change();
+
     if (you.species == SP_GHOUL && you.hunger_state >= HS_SATIATED)
         mpr("Ghouls can never be full or above!");
 }
@@ -332,7 +343,7 @@ void wizard_set_piety()
     mprf("Setting piety to %d.", newpiety);
     int diff = newpiety - you.piety;
     if (diff > 0)
-        gain_piety(diff);
+        gain_piety(diff, 1, true, false);
     else
         lose_piety(-diff);
 
@@ -349,9 +360,9 @@ void wizard_set_piety()
 #ifdef WIZARD
 void wizard_exercise_skill(void)
 {
-    int skill = debug_prompt_for_skill( "Which skill (by name)? " );
+    skill_type skill = debug_prompt_for_skill("Which skill (by name)? ");
 
-    if (skill == -1)
+    if (skill == SK_NONE)
         mpr("That skill doesn't seem to exist.");
     else
     {
@@ -362,26 +373,27 @@ void wizard_exercise_skill(void)
 #endif
 
 #ifdef WIZARD
-void wizard_set_skill_level(void)
+void wizard_set_skill_level(skill_type skill)
 {
-    int skill = debug_prompt_for_skill( "Which skill (by name)? " );
+    if (skill == SK_NONE)
+        skill = debug_prompt_for_skill("Which skill (by name)? ");
 
-    if (skill == -1)
+    if (skill == SK_NONE)
         mpr("That skill doesn't seem to exist.");
     else
     {
         mpr(skill_name(skill));
-        int amount = debug_prompt_for_int( "To what level? ", true );
+        int amount = prompt_for_int("To what level? ", true);
 
         if (amount < 0)
-            canned_msg( MSG_OK );
+            canned_msg(MSG_OK);
         else
         {
             const int old_amount = you.skills[skill];
-            const int points = (skill_exp_needed( amount )
-                                * species_skills( skill, you.species )) / 100;
+            const int points = skill_exp_needed(amount, skill);
 
             you.skill_points[skill] = points + 1;
+            you.ct_skill_points[skill] = 0;
             you.skills[skill] = amount;
 
             calc_total_skill_points();
@@ -434,26 +446,26 @@ void wizard_set_skill_level(void)
 void wizard_set_all_skills(void)
 {
     int i;
-    int amount =
-            debug_prompt_for_int( "Set all skills to what level? ", true );
+    int amount = prompt_for_int("Set all skills to what level? ", true);
 
     if (amount < 0)             // cancel returns -1 -- bwr
-        canned_msg( MSG_OK );
+        canned_msg(MSG_OK);
     else
     {
         if (amount > 27)
             amount = 27;
 
-        for (i = SK_FIGHTING; i < NUM_SKILLS; ++i)
+        for (i = SK_FIRST_SKILL; i < NUM_SKILLS; ++i)
         {
-            if (is_invalid_skill(i))
+            skill_type sk = static_cast<skill_type>(i);
+            if (is_invalid_skill(sk))
                 continue;
 
-            const int points = (skill_exp_needed( amount )
-                                * species_skills( i, you.species )) / 100;
+            const int points = skill_exp_needed(amount, sk);
 
-            you.skill_points[i] = points + 1;
-            you.skills[i] = amount;
+            you.skill_points[sk] = points + 1;
+            you.ct_skill_points[sk] = 0;
+            you.skills[sk] = amount;
         }
 
         redraw_skill(you.your_name, player_title());
@@ -470,8 +482,6 @@ void wizard_set_all_skills(void)
 #endif
 
 #ifdef WIZARD
-extern mutation_def mutation_defs[];
-
 bool wizard_add_mutation()
 {
     bool success = false;
@@ -483,9 +493,9 @@ bool wizard_add_mutation()
         const char* msg;
 
         if (you.mutation[MUT_MUTATION_RESISTANCE] == 3)
-            msg = "You are immune to mutations, remove immunity?";
+            msg = "You are immune to mutations; remove immunity?";
         else
-            msg = "You are resistant to mutations, remove resistance?";
+            msg = "You are resistant to mutations; remove resistance?";
 
         if (yesno(msg, true, 'n'))
         {
@@ -517,8 +527,9 @@ bool wizard_add_mutation()
     }
     const bool god_gift = (answer == 1);
 
-    mpr("Which mutation (name, 'good', 'bad', 'any', 'xom')? ", MSGCH_PROMPT);
-    get_input_line( specs, sizeof( specs ) );
+    msgwin_get_line("Which mutation (name, 'good', 'bad', 'any', "
+                    "'xom', 'slime')? ",
+                    specs, sizeof(specs));
 
     if (specs[0] == '\0')
         return (false);
@@ -527,7 +538,6 @@ bool wizard_add_mutation()
 
     mutation_type mutat = NUM_MUTATIONS;
 
-
     if (strcmp(specs, "good") == 0)
         mutat = RANDOM_GOOD_MUTATION;
     else if (strcmp(specs, "bad") == 0)
@@ -536,6 +546,8 @@ bool wizard_add_mutation()
         mutat = RANDOM_MUTATION;
     else if (strcmp(specs, "xom") == 0)
         mutat = RANDOM_XOM_MUTATION;
+    else if (strcmp(specs, "slime") == 0)
+        mutat = RANDOM_SLIME_MUTATION;
 
     if (mutat != NUM_MUTATIONS)
     {
@@ -554,21 +566,21 @@ bool wizard_add_mutation()
 
     std::vector<mutation_type> partial_matches;
 
-    for (unsigned i = 0; true; ++i)
+    for (int i = 0; i < NUM_MUTATIONS; ++i)
     {
-        if (strcmp(specs, mutation_defs[i].wizname) == 0)
+        mutation_type mut = static_cast<mutation_type>(i);
+        if (!is_valid_mutation(mut))
+            continue;
+
+        const mutation_def& mdef = get_mutation_def(mut);
+        if (strcmp(specs, mdef.wizname) == 0)
         {
-            mutat = mutation_defs[i].mutation;
+            mutat = mut;
             break;
         }
 
-        if (strstr(mutation_defs[i].wizname, specs))
-            partial_matches.push_back(mutation_defs[i].mutation);
-
-        // FIXME: hack, but I don't want to export the size
-        // of the array...this is even worse.
-        if (mutation_defs[i].mutation + 1 == NUM_MUTATIONS)
-            break;
+        if (strstr(mdef.wizname, specs))
+            partial_matches.push_back(mut);
     }
 
     // If only one matching mutation, use that.
@@ -606,7 +618,7 @@ bool wizard_add_mutation()
              mutation_name(mutat, 1, false).c_str());
 
         const int levels =
-            debug_prompt_for_int("How many levels to increase or decrease? ",
+            prompt_for_int("How many levels to increase or decrease? ",
                                   false);
 
         if (levels == 0)
@@ -637,8 +649,7 @@ void wizard_get_religion(void)
 {
     char specs[80];
 
-    mpr("Which god (by name)? ", MSGCH_PROMPT);
-    get_input_line( specs, sizeof( specs ) );
+    msgwin_get_line("Which god (by name)? ", specs, sizeof(specs));
 
     if (specs[0] == '\0')
         return;
@@ -662,7 +673,7 @@ void wizard_get_religion(void)
     else
     {
         dungeon_feature_type feat =
-            static_cast<dungeon_feature_type>( DNGN_ALTAR_FIRST_GOD + god - 1 );
+            static_cast<dungeon_feature_type>(DNGN_ALTAR_FIRST_GOD + god - 1);
         dungeon_terrain_changed(you.pos(), feat, false);
 
         pray();
@@ -677,23 +688,21 @@ void wizard_set_stats()
     if (cancelable_get_line_autohist(buf, sizeof buf))
         return;
 
-    int sstr = you.strength,
-        sdex = you.dex,
-        sint = you.intel;
+    int sstr = you.strength(),
+        sdex = you.dex(),
+        sint = you.intel();
 
     sscanf(buf, "%d %d %d", &sstr, &sint, &sdex);
 
-    you.max_strength = you.strength = debug_cap_stat(sstr);
-    you.max_dex      = you.dex      = debug_cap_stat(sdex);
-    you.max_intel    = you.intel    = debug_cap_stat(sint);
-
-    you.redraw_strength     = true;
-    you.redraw_dexterity    = true;
-    you.redraw_intelligence = true;
-    you.redraw_evasion      = true;
+    you.base_stats[STAT_STR] = debug_cap_stat(sstr);
+    you.base_stats[STAT_INT] = debug_cap_stat(sint);
+    you.base_stats[STAT_DEX] = debug_cap_stat(sdex);
+    you.stat_loss.init(0);
+    you.redraw_stats.init(true);
+    you.redraw_evasion = true;
 }
 
-static const char* dur_names[NUM_DURATIONS] =
+static const char* dur_names[] =
 {
     "invis",
     "conf",
@@ -702,12 +711,14 @@ static const char* dur_names[NUM_DURATIONS] =
     "mesmerised",
     "haste",
     "might",
+    "brilliance",
+    "agility",
     "levitation",
     "berserker",
     "poisoning",
     "confusing touch",
     "sure blade",
-    "backlight",
+    "corona",
     "deaths door",
     "fire shield",
     "building rage",
@@ -722,7 +733,9 @@ static const char* dur_names[NUM_DURATIONS] =
     "divine shield",
     "regeneration",
     "swiftness",
+#if TAG_MAJOR_VERSION == 32
     "stonemail",
+#endif
     "controlled flight",
     "teleport",
     "control teleport",
@@ -733,6 +746,8 @@ static const char* dur_names[NUM_DURATIONS] =
     "phase shift",
     "see invisible",
     "weapon brand",
+    "demonic guardian",
+    "pbd",
     "silence",
     "condensation shield",
     "stoneskin",
@@ -751,11 +766,28 @@ static const char* dur_names[NUM_DURATIONS] =
     "petrified",
     "lowered mr",
     "repel stairs move",
-    "repel stairs climb"
+    "repel stairs climb",
+    "coloured smoke trail",
+    "slimify",
+    "time step",
+    "icemail depleted",
+    "misled",
+    "quad damage",
+    "afraid",
+    "mirror damage",
+    "scrying",
+    "tornado",
+    "liquefying",
+    "heroism",
+    "finesse",
+    "lifesaving",
+    "paralysis immunity",
+    "darkness",
 };
 
-void wizard_edit_durations( void )
+void wizard_edit_durations(void)
 {
+    COMPILE_CHECK(ARRAYSZ(dur_names) == NUM_DURATIONS, dur_names_size);
     std::vector<int> durs;
     size_t max_len = 0;
 
@@ -784,17 +816,15 @@ void wizard_edit_durations( void )
 
     char buf[80];
 
-    if (cancelable_get_line_autohist(buf, sizeof buf) || strlen(buf) == 0)
+    if (cancelable_get_line_autohist(buf, sizeof buf) || !*buf)
     {
-        canned_msg( MSG_OK );
+        canned_msg(MSG_OK);
         return;
     }
 
-    strcpy(buf, lowercase_string(trimmed_string(buf)).c_str());
-
-    if (strlen(buf) == 0)
+    if (!strlcpy(buf, lowercase_string(trimmed_string(buf)).c_str(), sizeof(buf)))
     {
-        canned_msg( MSG_OK );
+        canned_msg(MSG_OK);
         return;
     }
 
@@ -856,8 +886,8 @@ void wizard_edit_durations( void )
         }
     }
 
-    sprintf(buf, "Set '%s' to: ", dur_names[choice]);
-    int num = debug_prompt_for_int(buf, false);
+    snprintf(buf, sizeof(buf), "Set '%s' to: ", dur_names[choice]);
+    int num = prompt_for_int(buf, false);
 
     if (num == 0)
     {
@@ -870,11 +900,8 @@ void wizard_edit_durations( void )
 
 static void debug_uptick_xl(int newxl)
 {
-    while (newxl > you.experience_level)
-    {
-        you.experience = 1 + exp_needed( 2 + you.experience_level );
-        level_change(true);
-    }
+    you.experience = exp_needed(newxl);
+    level_change(true);
 }
 
 static void debug_downtick_xl(int newxl)
@@ -887,7 +914,8 @@ static void debug_downtick_xl(int newxl)
         you.hp     = std::max(5, you.hp);
         you.hp_max = std::max(5, you.hp_max);
 
-        lose_level();
+        you.experience = exp_needed(you.experience_level) - 1;
+        level_change();
     }
 
     you.hp       = std::max(1, you.hp);
@@ -929,6 +957,25 @@ void wizard_get_god_gift (void)
         return;
     }
 
-    if (!do_god_gift(false, true))
+    if (!do_god_gift(true))
         mpr("Nothing happens.");
 }
+
+void wizard_toggle_xray_vision()
+{
+    you.xray_vision = !you.xray_vision;
+    viewwindow(true);
+}
+
+void wizard_god_wrath()
+{
+    if (you.religion == GOD_NO_GOD)
+    {
+        mpr("You suffer the terrible wrath of No God.");
+        return;
+    }
+
+    if (!divine_retribution(you.religion, true, true))
+        // Currently only dead Jiyva.
+        mpr("You're not eligible for wrath.");
+}
